// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/input_reader.proto

#include "second/protos/input_reader.pb.h"

#include <algorithm>

#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/extension_set.h>
#include <google/protobuf/wire_format_lite.h>
#include <google/protobuf/descriptor.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/reflection_ops.h>
#include <google/protobuf/wire_format.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>

PROTOBUF_PRAGMA_INIT_SEG

namespace _pb = ::PROTOBUF_NAMESPACE_ID;
namespace _pbi = _pb::internal;

namespace second {
namespace protos {
PROTOBUF_CONSTEXPR InputReader::InputReader(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.class_names_)*/{}
  , /*decltype(_impl_.groundtruth_localization_noise_std_)*/{}
  , /*decltype(_impl_.groundtruth_rotation_uniform_noise_)*/{}
  , /*decltype(_impl_.global_rotation_uniform_noise_)*/{}
  , /*decltype(_impl_.global_scaling_uniform_noise_)*/{}
  , /*decltype(_impl_.global_random_rotation_range_per_object_)*/{}
  , /*decltype(_impl_.database_prep_steps_)*/{}
  , /*decltype(_impl_.record_file_path_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.kitti_info_path_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.kitti_root_path_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.target_assigner_)*/nullptr
  , /*decltype(_impl_.database_sampler_)*/nullptr
  , /*decltype(_impl_.unlabeled_database_sampler_)*/nullptr
  , /*decltype(_impl_.batch_size_)*/0u
  , /*decltype(_impl_.max_num_epochs_)*/0u
  , /*decltype(_impl_.prefetch_size_)*/0u
  , /*decltype(_impl_.max_number_of_voxels_)*/0u
  , /*decltype(_impl_.num_workers_)*/0u
  , /*decltype(_impl_.shuffle_points_)*/false
  , /*decltype(_impl_.remove_unknown_examples_)*/false
  , /*decltype(_impl_.remove_points_after_sample_)*/false
  , /*decltype(_impl_.remove_environment_)*/false
  , /*decltype(_impl_.anchor_area_threshold_)*/0
  , /*decltype(_impl_.groundtruth_points_drop_percentage_)*/0
  , /*decltype(_impl_.groundtruth_drop_max_keep_points_)*/0u
  , /*decltype(_impl_.unlabeled_training_)*/false
  , /*decltype(_impl_.use_group_id_)*/false
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct InputReaderDefaultTypeInternal {
  PROTOBUF_CONSTEXPR InputReaderDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~InputReaderDefaultTypeInternal() {}
  union {
    InputReader _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 InputReaderDefaultTypeInternal _InputReader_default_instance_;
}  // namespace protos
}  // namespace second
static ::_pb::Metadata file_level_metadata_second_2fprotos_2finput_5freader_2eproto[1];
static constexpr ::_pb::EnumDescriptor const** file_level_enum_descriptors_second_2fprotos_2finput_5freader_2eproto = nullptr;
static constexpr ::_pb::ServiceDescriptor const** file_level_service_descriptors_second_2fprotos_2finput_5freader_2eproto = nullptr;

const uint32_t TableStruct_second_2fprotos_2finput_5freader_2eproto::offsets[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.record_file_path_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.class_names_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.batch_size_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.max_num_epochs_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.prefetch_size_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.max_number_of_voxels_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.target_assigner_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.kitti_info_path_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.kitti_root_path_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.shuffle_points_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.groundtruth_localization_noise_std_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.groundtruth_rotation_uniform_noise_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.global_rotation_uniform_noise_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.global_scaling_uniform_noise_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.remove_unknown_examples_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.num_workers_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.anchor_area_threshold_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.remove_points_after_sample_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.groundtruth_points_drop_percentage_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.groundtruth_drop_max_keep_points_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.remove_environment_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.unlabeled_training_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.global_random_rotation_range_per_object_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.database_prep_steps_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.database_sampler_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.use_group_id_),
  PROTOBUF_FIELD_OFFSET(::second::protos::InputReader, _impl_.unlabeled_database_sampler_),
};
static const ::_pbi::MigrationSchema schemas[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  { 0, -1, -1, sizeof(::second::protos::InputReader)},
};

static const ::_pb::Message* const file_default_instances[] = {
  &::second::protos::_InputReader_default_instance_._instance,
};

const char descriptor_table_protodef_second_2fprotos_2finput_5freader_2eproto[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) =
  "\n second/protos/input_reader.proto\022\rseco"
  "nd.protos\032\032second/protos/target.proto\032\036s"
  "econd/protos/preprocess.proto\032\033second/pr"
  "otos/sampler.proto\"\307\007\n\013InputReader\022\030\n\020re"
  "cord_file_path\030\001 \001(\t\022\023\n\013class_names\030\002 \003("
  "\t\022\022\n\nbatch_size\030\003 \001(\r\022\026\n\016max_num_epochs\030"
  "\004 \001(\r\022\025\n\rprefetch_size\030\005 \001(\r\022\034\n\024max_numb"
  "er_of_voxels\030\006 \001(\r\0226\n\017target_assigner\030\007 "
  "\001(\0132\035.second.protos.TargetAssigner\022\027\n\017ki"
  "tti_info_path\030\010 \001(\t\022\027\n\017kitti_root_path\030\t"
  " \001(\t\022\026\n\016shuffle_points\030\n \001(\010\022*\n\"groundtr"
  "uth_localization_noise_std\030\013 \003(\002\022*\n\"grou"
  "ndtruth_rotation_uniform_noise\030\014 \003(\002\022%\n\035"
  "global_rotation_uniform_noise\030\r \003(\002\022$\n\034g"
  "lobal_scaling_uniform_noise\030\016 \003(\002\022\037\n\027rem"
  "ove_unknown_examples\030\017 \001(\010\022\023\n\013num_worker"
  "s\030\020 \001(\r\022\035\n\025anchor_area_threshold\030\021 \001(\002\022\""
  "\n\032remove_points_after_sample\030\022 \001(\010\022*\n\"gr"
  "oundtruth_points_drop_percentage\030\023 \001(\002\022("
  "\n groundtruth_drop_max_keep_points\030\024 \001(\r"
  "\022\032\n\022remove_environment\030\025 \001(\010\022\032\n\022unlabele"
  "d_training\030\026 \001(\010\022/\n\'global_random_rotati"
  "on_range_per_object\030\027 \003(\002\022E\n\023database_pr"
  "ep_steps\030\030 \003(\0132(.second.protos.DatabaseP"
  "reprocessingStep\0220\n\020database_sampler\030\031 \001"
  "(\0132\026.second.protos.Sampler\022\024\n\014use_group_"
  "id\030\032 \001(\010\022:\n\032unlabeled_database_sampler\030\033"
  " \001(\0132\026.second.protos.Samplerb\006proto3"
  ;
static const ::_pbi::DescriptorTable* const descriptor_table_second_2fprotos_2finput_5freader_2eproto_deps[3] = {
  &::descriptor_table_second_2fprotos_2fpreprocess_2eproto,
  &::descriptor_table_second_2fprotos_2fsampler_2eproto,
  &::descriptor_table_second_2fprotos_2ftarget_2eproto,
};
static ::_pbi::once_flag descriptor_table_second_2fprotos_2finput_5freader_2eproto_once;
const ::_pbi::DescriptorTable descriptor_table_second_2fprotos_2finput_5freader_2eproto = {
    false, false, 1116, descriptor_table_protodef_second_2fprotos_2finput_5freader_2eproto,
    "second/protos/input_reader.proto",
    &descriptor_table_second_2fprotos_2finput_5freader_2eproto_once, descriptor_table_second_2fprotos_2finput_5freader_2eproto_deps, 3, 1,
    schemas, file_default_instances, TableStruct_second_2fprotos_2finput_5freader_2eproto::offsets,
    file_level_metadata_second_2fprotos_2finput_5freader_2eproto, file_level_enum_descriptors_second_2fprotos_2finput_5freader_2eproto,
    file_level_service_descriptors_second_2fprotos_2finput_5freader_2eproto,
};
PROTOBUF_ATTRIBUTE_WEAK const ::_pbi::DescriptorTable* descriptor_table_second_2fprotos_2finput_5freader_2eproto_getter() {
  return &descriptor_table_second_2fprotos_2finput_5freader_2eproto;
}

// Force running AddDescriptors() at dynamic initialization time.
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::_pbi::AddDescriptorsRunner dynamic_init_dummy_second_2fprotos_2finput_5freader_2eproto(&descriptor_table_second_2fprotos_2finput_5freader_2eproto);
namespace second {
namespace protos {

// ===================================================================

class InputReader::_Internal {
 public:
  static const ::second::protos::TargetAssigner& target_assigner(const InputReader* msg);
  static const ::second::protos::Sampler& database_sampler(const InputReader* msg);
  static const ::second::protos::Sampler& unlabeled_database_sampler(const InputReader* msg);
};

const ::second::protos::TargetAssigner&
InputReader::_Internal::target_assigner(const InputReader* msg) {
  return *msg->_impl_.target_assigner_;
}
const ::second::protos::Sampler&
InputReader::_Internal::database_sampler(const InputReader* msg) {
  return *msg->_impl_.database_sampler_;
}
const ::second::protos::Sampler&
InputReader::_Internal::unlabeled_database_sampler(const InputReader* msg) {
  return *msg->_impl_.unlabeled_database_sampler_;
}
void InputReader::clear_target_assigner() {
  if (GetArenaForAllocation() == nullptr && _impl_.target_assigner_ != nullptr) {
    delete _impl_.target_assigner_;
  }
  _impl_.target_assigner_ = nullptr;
}
void InputReader::clear_database_prep_steps() {
  _impl_.database_prep_steps_.Clear();
}
void InputReader::clear_database_sampler() {
  if (GetArenaForAllocation() == nullptr && _impl_.database_sampler_ != nullptr) {
    delete _impl_.database_sampler_;
  }
  _impl_.database_sampler_ = nullptr;
}
void InputReader::clear_unlabeled_database_sampler() {
  if (GetArenaForAllocation() == nullptr && _impl_.unlabeled_database_sampler_ != nullptr) {
    delete _impl_.unlabeled_database_sampler_;
  }
  _impl_.unlabeled_database_sampler_ = nullptr;
}
InputReader::InputReader(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.InputReader)
}
InputReader::InputReader(const InputReader& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  InputReader* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.class_names_){from._impl_.class_names_}
    , decltype(_impl_.groundtruth_localization_noise_std_){from._impl_.groundtruth_localization_noise_std_}
    , decltype(_impl_.groundtruth_rotation_uniform_noise_){from._impl_.groundtruth_rotation_uniform_noise_}
    , decltype(_impl_.global_rotation_uniform_noise_){from._impl_.global_rotation_uniform_noise_}
    , decltype(_impl_.global_scaling_uniform_noise_){from._impl_.global_scaling_uniform_noise_}
    , decltype(_impl_.global_random_rotation_range_per_object_){from._impl_.global_random_rotation_range_per_object_}
    , decltype(_impl_.database_prep_steps_){from._impl_.database_prep_steps_}
    , decltype(_impl_.record_file_path_){}
    , decltype(_impl_.kitti_info_path_){}
    , decltype(_impl_.kitti_root_path_){}
    , decltype(_impl_.target_assigner_){nullptr}
    , decltype(_impl_.database_sampler_){nullptr}
    , decltype(_impl_.unlabeled_database_sampler_){nullptr}
    , decltype(_impl_.batch_size_){}
    , decltype(_impl_.max_num_epochs_){}
    , decltype(_impl_.prefetch_size_){}
    , decltype(_impl_.max_number_of_voxels_){}
    , decltype(_impl_.num_workers_){}
    , decltype(_impl_.shuffle_points_){}
    , decltype(_impl_.remove_unknown_examples_){}
    , decltype(_impl_.remove_points_after_sample_){}
    , decltype(_impl_.remove_environment_){}
    , decltype(_impl_.anchor_area_threshold_){}
    , decltype(_impl_.groundtruth_points_drop_percentage_){}
    , decltype(_impl_.groundtruth_drop_max_keep_points_){}
    , decltype(_impl_.unlabeled_training_){}
    , decltype(_impl_.use_group_id_){}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _impl_.record_file_path_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.record_file_path_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (!from._internal_record_file_path().empty()) {
    _this->_impl_.record_file_path_.Set(from._internal_record_file_path(), 
      _this->GetArenaForAllocation());
  }
  _impl_.kitti_info_path_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.kitti_info_path_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (!from._internal_kitti_info_path().empty()) {
    _this->_impl_.kitti_info_path_.Set(from._internal_kitti_info_path(), 
      _this->GetArenaForAllocation());
  }
  _impl_.kitti_root_path_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.kitti_root_path_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (!from._internal_kitti_root_path().empty()) {
    _this->_impl_.kitti_root_path_.Set(from._internal_kitti_root_path(), 
      _this->GetArenaForAllocation());
  }
  if (from._internal_has_target_assigner()) {
    _this->_impl_.target_assigner_ = new ::second::protos::TargetAssigner(*from._impl_.target_assigner_);
  }
  if (from._internal_has_database_sampler()) {
    _this->_impl_.database_sampler_ = new ::second::protos::Sampler(*from._impl_.database_sampler_);
  }
  if (from._internal_has_unlabeled_database_sampler()) {
    _this->_impl_.unlabeled_database_sampler_ = new ::second::protos::Sampler(*from._impl_.unlabeled_database_sampler_);
  }
  ::memcpy(&_impl_.batch_size_, &from._impl_.batch_size_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.use_group_id_) -
    reinterpret_cast<char*>(&_impl_.batch_size_)) + sizeof(_impl_.use_group_id_));
  // @@protoc_insertion_point(copy_constructor:second.protos.InputReader)
}

inline void InputReader::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.class_names_){arena}
    , decltype(_impl_.groundtruth_localization_noise_std_){arena}
    , decltype(_impl_.groundtruth_rotation_uniform_noise_){arena}
    , decltype(_impl_.global_rotation_uniform_noise_){arena}
    , decltype(_impl_.global_scaling_uniform_noise_){arena}
    , decltype(_impl_.global_random_rotation_range_per_object_){arena}
    , decltype(_impl_.database_prep_steps_){arena}
    , decltype(_impl_.record_file_path_){}
    , decltype(_impl_.kitti_info_path_){}
    , decltype(_impl_.kitti_root_path_){}
    , decltype(_impl_.target_assigner_){nullptr}
    , decltype(_impl_.database_sampler_){nullptr}
    , decltype(_impl_.unlabeled_database_sampler_){nullptr}
    , decltype(_impl_.batch_size_){0u}
    , decltype(_impl_.max_num_epochs_){0u}
    , decltype(_impl_.prefetch_size_){0u}
    , decltype(_impl_.max_number_of_voxels_){0u}
    , decltype(_impl_.num_workers_){0u}
    , decltype(_impl_.shuffle_points_){false}
    , decltype(_impl_.remove_unknown_examples_){false}
    , decltype(_impl_.remove_points_after_sample_){false}
    , decltype(_impl_.remove_environment_){false}
    , decltype(_impl_.anchor_area_threshold_){0}
    , decltype(_impl_.groundtruth_points_drop_percentage_){0}
    , decltype(_impl_.groundtruth_drop_max_keep_points_){0u}
    , decltype(_impl_.unlabeled_training_){false}
    , decltype(_impl_.use_group_id_){false}
    , /*decltype(_impl_._cached_size_)*/{}
  };
  _impl_.record_file_path_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.record_file_path_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  _impl_.kitti_info_path_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.kitti_info_path_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  _impl_.kitti_root_path_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.kitti_root_path_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
}

InputReader::~InputReader() {
  // @@protoc_insertion_point(destructor:second.protos.InputReader)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void InputReader::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.class_names_.~RepeatedPtrField();
  _impl_.groundtruth_localization_noise_std_.~RepeatedField();
  _impl_.groundtruth_rotation_uniform_noise_.~RepeatedField();
  _impl_.global_rotation_uniform_noise_.~RepeatedField();
  _impl_.global_scaling_uniform_noise_.~RepeatedField();
  _impl_.global_random_rotation_range_per_object_.~RepeatedField();
  _impl_.database_prep_steps_.~RepeatedPtrField();
  _impl_.record_file_path_.Destroy();
  _impl_.kitti_info_path_.Destroy();
  _impl_.kitti_root_path_.Destroy();
  if (this != internal_default_instance()) delete _impl_.target_assigner_;
  if (this != internal_default_instance()) delete _impl_.database_sampler_;
  if (this != internal_default_instance()) delete _impl_.unlabeled_database_sampler_;
}

void InputReader::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void InputReader::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.InputReader)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.class_names_.Clear();
  _impl_.groundtruth_localization_noise_std_.Clear();
  _impl_.groundtruth_rotation_uniform_noise_.Clear();
  _impl_.global_rotation_uniform_noise_.Clear();
  _impl_.global_scaling_uniform_noise_.Clear();
  _impl_.global_random_rotation_range_per_object_.Clear();
  _impl_.database_prep_steps_.Clear();
  _impl_.record_file_path_.ClearToEmpty();
  _impl_.kitti_info_path_.ClearToEmpty();
  _impl_.kitti_root_path_.ClearToEmpty();
  if (GetArenaForAllocation() == nullptr && _impl_.target_assigner_ != nullptr) {
    delete _impl_.target_assigner_;
  }
  _impl_.target_assigner_ = nullptr;
  if (GetArenaForAllocation() == nullptr && _impl_.database_sampler_ != nullptr) {
    delete _impl_.database_sampler_;
  }
  _impl_.database_sampler_ = nullptr;
  if (GetArenaForAllocation() == nullptr && _impl_.unlabeled_database_sampler_ != nullptr) {
    delete _impl_.unlabeled_database_sampler_;
  }
  _impl_.unlabeled_database_sampler_ = nullptr;
  ::memset(&_impl_.batch_size_, 0, static_cast<size_t>(
      reinterpret_cast<char*>(&_impl_.use_group_id_) -
      reinterpret_cast<char*>(&_impl_.batch_size_)) + sizeof(_impl_.use_group_id_));
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* InputReader::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // string record_file_path = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          auto str = _internal_mutable_record_file_path();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          CHK_(::_pbi::VerifyUTF8(str, "second.protos.InputReader.record_file_path"));
        } else
          goto handle_unusual;
        continue;
      // repeated string class_names = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 18)) {
          ptr -= 1;
          do {
            ptr += 1;
            auto str = _internal_add_class_names();
            ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
            CHK_(ptr);
            CHK_(::_pbi::VerifyUTF8(str, "second.protos.InputReader.class_names"));
            if (!ctx->DataAvailable(ptr)) break;
          } while (::PROTOBUF_NAMESPACE_ID::internal::ExpectTag<18>(ptr));
        } else
          goto handle_unusual;
        continue;
      // uint32 batch_size = 3;
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 24)) {
          _impl_.batch_size_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // uint32 max_num_epochs = 4;
      case 4:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 32)) {
          _impl_.max_num_epochs_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // uint32 prefetch_size = 5;
      case 5:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 40)) {
          _impl_.prefetch_size_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // uint32 max_number_of_voxels = 6;
      case 6:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 48)) {
          _impl_.max_number_of_voxels_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.TargetAssigner target_assigner = 7;
      case 7:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 58)) {
          ptr = ctx->ParseMessage(_internal_mutable_target_assigner(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // string kitti_info_path = 8;
      case 8:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 66)) {
          auto str = _internal_mutable_kitti_info_path();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          CHK_(::_pbi::VerifyUTF8(str, "second.protos.InputReader.kitti_info_path"));
        } else
          goto handle_unusual;
        continue;
      // string kitti_root_path = 9;
      case 9:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 74)) {
          auto str = _internal_mutable_kitti_root_path();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          CHK_(::_pbi::VerifyUTF8(str, "second.protos.InputReader.kitti_root_path"));
        } else
          goto handle_unusual;
        continue;
      // bool shuffle_points = 10;
      case 10:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 80)) {
          _impl_.shuffle_points_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // repeated float groundtruth_localization_noise_std = 11;
      case 11:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 90)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedFloatParser(_internal_mutable_groundtruth_localization_noise_std(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 93) {
          _internal_add_groundtruth_localization_noise_std(::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr));
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // repeated float groundtruth_rotation_uniform_noise = 12;
      case 12:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 98)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedFloatParser(_internal_mutable_groundtruth_rotation_uniform_noise(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 101) {
          _internal_add_groundtruth_rotation_uniform_noise(::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr));
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // repeated float global_rotation_uniform_noise = 13;
      case 13:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 106)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedFloatParser(_internal_mutable_global_rotation_uniform_noise(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 109) {
          _internal_add_global_rotation_uniform_noise(::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr));
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // repeated float global_scaling_uniform_noise = 14;
      case 14:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 114)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedFloatParser(_internal_mutable_global_scaling_uniform_noise(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 117) {
          _internal_add_global_scaling_uniform_noise(::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr));
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // bool remove_unknown_examples = 15;
      case 15:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 120)) {
          _impl_.remove_unknown_examples_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // uint32 num_workers = 16;
      case 16:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 128)) {
          _impl_.num_workers_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // float anchor_area_threshold = 17;
      case 17:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 141)) {
          _impl_.anchor_area_threshold_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // bool remove_points_after_sample = 18;
      case 18:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 144)) {
          _impl_.remove_points_after_sample_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // float groundtruth_points_drop_percentage = 19;
      case 19:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 157)) {
          _impl_.groundtruth_points_drop_percentage_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // uint32 groundtruth_drop_max_keep_points = 20;
      case 20:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 160)) {
          _impl_.groundtruth_drop_max_keep_points_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool remove_environment = 21;
      case 21:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 168)) {
          _impl_.remove_environment_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool unlabeled_training = 22;
      case 22:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 176)) {
          _impl_.unlabeled_training_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // repeated float global_random_rotation_range_per_object = 23;
      case 23:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 186)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedFloatParser(_internal_mutable_global_random_rotation_range_per_object(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 189) {
          _internal_add_global_random_rotation_range_per_object(::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr));
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // repeated .second.protos.DatabasePreprocessingStep database_prep_steps = 24;
      case 24:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 194)) {
          ptr -= 2;
          do {
            ptr += 2;
            ptr = ctx->ParseMessage(_internal_add_database_prep_steps(), ptr);
            CHK_(ptr);
            if (!ctx->DataAvailable(ptr)) break;
          } while (::PROTOBUF_NAMESPACE_ID::internal::ExpectTag<194>(ptr));
        } else
          goto handle_unusual;
        continue;
      // .second.protos.Sampler database_sampler = 25;
      case 25:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 202)) {
          ptr = ctx->ParseMessage(_internal_mutable_database_sampler(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool use_group_id = 26;
      case 26:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 208)) {
          _impl_.use_group_id_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.Sampler unlabeled_database_sampler = 27;
      case 27:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 218)) {
          ptr = ctx->ParseMessage(_internal_mutable_unlabeled_database_sampler(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* InputReader::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.InputReader)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // string record_file_path = 1;
  if (!this->_internal_record_file_path().empty()) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(
      this->_internal_record_file_path().data(), static_cast<int>(this->_internal_record_file_path().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::SERIALIZE,
      "second.protos.InputReader.record_file_path");
    target = stream->WriteStringMaybeAliased(
        1, this->_internal_record_file_path(), target);
  }

  // repeated string class_names = 2;
  for (int i = 0, n = this->_internal_class_names_size(); i < n; i++) {
    const auto& s = this->_internal_class_names(i);
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(
      s.data(), static_cast<int>(s.length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::SERIALIZE,
      "second.protos.InputReader.class_names");
    target = stream->WriteString(2, s, target);
  }

  // uint32 batch_size = 3;
  if (this->_internal_batch_size() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(3, this->_internal_batch_size(), target);
  }

  // uint32 max_num_epochs = 4;
  if (this->_internal_max_num_epochs() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(4, this->_internal_max_num_epochs(), target);
  }

  // uint32 prefetch_size = 5;
  if (this->_internal_prefetch_size() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(5, this->_internal_prefetch_size(), target);
  }

  // uint32 max_number_of_voxels = 6;
  if (this->_internal_max_number_of_voxels() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(6, this->_internal_max_number_of_voxels(), target);
  }

  // .second.protos.TargetAssigner target_assigner = 7;
  if (this->_internal_has_target_assigner()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(7, _Internal::target_assigner(this),
        _Internal::target_assigner(this).GetCachedSize(), target, stream);
  }

  // string kitti_info_path = 8;
  if (!this->_internal_kitti_info_path().empty()) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(
      this->_internal_kitti_info_path().data(), static_cast<int>(this->_internal_kitti_info_path().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::SERIALIZE,
      "second.protos.InputReader.kitti_info_path");
    target = stream->WriteStringMaybeAliased(
        8, this->_internal_kitti_info_path(), target);
  }

  // string kitti_root_path = 9;
  if (!this->_internal_kitti_root_path().empty()) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(
      this->_internal_kitti_root_path().data(), static_cast<int>(this->_internal_kitti_root_path().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::SERIALIZE,
      "second.protos.InputReader.kitti_root_path");
    target = stream->WriteStringMaybeAliased(
        9, this->_internal_kitti_root_path(), target);
  }

  // bool shuffle_points = 10;
  if (this->_internal_shuffle_points() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(10, this->_internal_shuffle_points(), target);
  }

  // repeated float groundtruth_localization_noise_std = 11;
  if (this->_internal_groundtruth_localization_noise_std_size() > 0) {
    target = stream->WriteFixedPacked(11, _internal_groundtruth_localization_noise_std(), target);
  }

  // repeated float groundtruth_rotation_uniform_noise = 12;
  if (this->_internal_groundtruth_rotation_uniform_noise_size() > 0) {
    target = stream->WriteFixedPacked(12, _internal_groundtruth_rotation_uniform_noise(), target);
  }

  // repeated float global_rotation_uniform_noise = 13;
  if (this->_internal_global_rotation_uniform_noise_size() > 0) {
    target = stream->WriteFixedPacked(13, _internal_global_rotation_uniform_noise(), target);
  }

  // repeated float global_scaling_uniform_noise = 14;
  if (this->_internal_global_scaling_uniform_noise_size() > 0) {
    target = stream->WriteFixedPacked(14, _internal_global_scaling_uniform_noise(), target);
  }

  // bool remove_unknown_examples = 15;
  if (this->_internal_remove_unknown_examples() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(15, this->_internal_remove_unknown_examples(), target);
  }

  // uint32 num_workers = 16;
  if (this->_internal_num_workers() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(16, this->_internal_num_workers(), target);
  }

  // float anchor_area_threshold = 17;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_anchor_area_threshold = this->_internal_anchor_area_threshold();
  uint32_t raw_anchor_area_threshold;
  memcpy(&raw_anchor_area_threshold, &tmp_anchor_area_threshold, sizeof(tmp_anchor_area_threshold));
  if (raw_anchor_area_threshold != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(17, this->_internal_anchor_area_threshold(), target);
  }

  // bool remove_points_after_sample = 18;
  if (this->_internal_remove_points_after_sample() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(18, this->_internal_remove_points_after_sample(), target);
  }

  // float groundtruth_points_drop_percentage = 19;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_groundtruth_points_drop_percentage = this->_internal_groundtruth_points_drop_percentage();
  uint32_t raw_groundtruth_points_drop_percentage;
  memcpy(&raw_groundtruth_points_drop_percentage, &tmp_groundtruth_points_drop_percentage, sizeof(tmp_groundtruth_points_drop_percentage));
  if (raw_groundtruth_points_drop_percentage != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(19, this->_internal_groundtruth_points_drop_percentage(), target);
  }

  // uint32 groundtruth_drop_max_keep_points = 20;
  if (this->_internal_groundtruth_drop_max_keep_points() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(20, this->_internal_groundtruth_drop_max_keep_points(), target);
  }

  // bool remove_environment = 21;
  if (this->_internal_remove_environment() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(21, this->_internal_remove_environment(), target);
  }

  // bool unlabeled_training = 22;
  if (this->_internal_unlabeled_training() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(22, this->_internal_unlabeled_training(), target);
  }

  // repeated float global_random_rotation_range_per_object = 23;
  if (this->_internal_global_random_rotation_range_per_object_size() > 0) {
    target = stream->WriteFixedPacked(23, _internal_global_random_rotation_range_per_object(), target);
  }

  // repeated .second.protos.DatabasePreprocessingStep database_prep_steps = 24;
  for (unsigned i = 0,
      n = static_cast<unsigned>(this->_internal_database_prep_steps_size()); i < n; i++) {
    const auto& repfield = this->_internal_database_prep_steps(i);
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
        InternalWriteMessage(24, repfield, repfield.GetCachedSize(), target, stream);
  }

  // .second.protos.Sampler database_sampler = 25;
  if (this->_internal_has_database_sampler()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(25, _Internal::database_sampler(this),
        _Internal::database_sampler(this).GetCachedSize(), target, stream);
  }

  // bool use_group_id = 26;
  if (this->_internal_use_group_id() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(26, this->_internal_use_group_id(), target);
  }

  // .second.protos.Sampler unlabeled_database_sampler = 27;
  if (this->_internal_has_unlabeled_database_sampler()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(27, _Internal::unlabeled_database_sampler(this),
        _Internal::unlabeled_database_sampler(this).GetCachedSize(), target, stream);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.InputReader)
  return target;
}

size_t InputReader::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.InputReader)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // repeated string class_names = 2;
  total_size += 1 *
      ::PROTOBUF_NAMESPACE_ID::internal::FromIntSize(_impl_.class_names_.size());
  for (int i = 0, n = _impl_.class_names_.size(); i < n; i++) {
    total_size += ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
      _impl_.class_names_.Get(i));
  }

  // repeated float groundtruth_localization_noise_std = 11;
  {
    unsigned int count = static_cast<unsigned int>(this->_internal_groundtruth_localization_noise_std_size());
    size_t data_size = 4UL * count;
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    total_size += data_size;
  }

  // repeated float groundtruth_rotation_uniform_noise = 12;
  {
    unsigned int count = static_cast<unsigned int>(this->_internal_groundtruth_rotation_uniform_noise_size());
    size_t data_size = 4UL * count;
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    total_size += data_size;
  }

  // repeated float global_rotation_uniform_noise = 13;
  {
    unsigned int count = static_cast<unsigned int>(this->_internal_global_rotation_uniform_noise_size());
    size_t data_size = 4UL * count;
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    total_size += data_size;
  }

  // repeated float global_scaling_uniform_noise = 14;
  {
    unsigned int count = static_cast<unsigned int>(this->_internal_global_scaling_uniform_noise_size());
    size_t data_size = 4UL * count;
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    total_size += data_size;
  }

  // repeated float global_random_rotation_range_per_object = 23;
  {
    unsigned int count = static_cast<unsigned int>(this->_internal_global_random_rotation_range_per_object_size());
    size_t data_size = 4UL * count;
    if (data_size > 0) {
      total_size += 2 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    total_size += data_size;
  }

  // repeated .second.protos.DatabasePreprocessingStep database_prep_steps = 24;
  total_size += 2UL * this->_internal_database_prep_steps_size();
  for (const auto& msg : this->_impl_.database_prep_steps_) {
    total_size +=
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(msg);
  }

  // string record_file_path = 1;
  if (!this->_internal_record_file_path().empty()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
        this->_internal_record_file_path());
  }

  // string kitti_info_path = 8;
  if (!this->_internal_kitti_info_path().empty()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
        this->_internal_kitti_info_path());
  }

  // string kitti_root_path = 9;
  if (!this->_internal_kitti_root_path().empty()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
        this->_internal_kitti_root_path());
  }

  // .second.protos.TargetAssigner target_assigner = 7;
  if (this->_internal_has_target_assigner()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.target_assigner_);
  }

  // .second.protos.Sampler database_sampler = 25;
  if (this->_internal_has_database_sampler()) {
    total_size += 2 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.database_sampler_);
  }

  // .second.protos.Sampler unlabeled_database_sampler = 27;
  if (this->_internal_has_unlabeled_database_sampler()) {
    total_size += 2 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.unlabeled_database_sampler_);
  }

  // uint32 batch_size = 3;
  if (this->_internal_batch_size() != 0) {
    total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_batch_size());
  }

  // uint32 max_num_epochs = 4;
  if (this->_internal_max_num_epochs() != 0) {
    total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_max_num_epochs());
  }

  // uint32 prefetch_size = 5;
  if (this->_internal_prefetch_size() != 0) {
    total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_prefetch_size());
  }

  // uint32 max_number_of_voxels = 6;
  if (this->_internal_max_number_of_voxels() != 0) {
    total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_max_number_of_voxels());
  }

  // uint32 num_workers = 16;
  if (this->_internal_num_workers() != 0) {
    total_size += 2 +
      ::_pbi::WireFormatLite::UInt32Size(
        this->_internal_num_workers());
  }

  // bool shuffle_points = 10;
  if (this->_internal_shuffle_points() != 0) {
    total_size += 1 + 1;
  }

  // bool remove_unknown_examples = 15;
  if (this->_internal_remove_unknown_examples() != 0) {
    total_size += 1 + 1;
  }

  // bool remove_points_after_sample = 18;
  if (this->_internal_remove_points_after_sample() != 0) {
    total_size += 2 + 1;
  }

  // bool remove_environment = 21;
  if (this->_internal_remove_environment() != 0) {
    total_size += 2 + 1;
  }

  // float anchor_area_threshold = 17;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_anchor_area_threshold = this->_internal_anchor_area_threshold();
  uint32_t raw_anchor_area_threshold;
  memcpy(&raw_anchor_area_threshold, &tmp_anchor_area_threshold, sizeof(tmp_anchor_area_threshold));
  if (raw_anchor_area_threshold != 0) {
    total_size += 2 + 4;
  }

  // float groundtruth_points_drop_percentage = 19;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_groundtruth_points_drop_percentage = this->_internal_groundtruth_points_drop_percentage();
  uint32_t raw_groundtruth_points_drop_percentage;
  memcpy(&raw_groundtruth_points_drop_percentage, &tmp_groundtruth_points_drop_percentage, sizeof(tmp_groundtruth_points_drop_percentage));
  if (raw_groundtruth_points_drop_percentage != 0) {
    total_size += 2 + 4;
  }

  // uint32 groundtruth_drop_max_keep_points = 20;
  if (this->_internal_groundtruth_drop_max_keep_points() != 0) {
    total_size += 2 +
      ::_pbi::WireFormatLite::UInt32Size(
        this->_internal_groundtruth_drop_max_keep_points());
  }

  // bool unlabeled_training = 22;
  if (this->_internal_unlabeled_training() != 0) {
    total_size += 2 + 1;
  }

  // bool use_group_id = 26;
  if (this->_internal_use_group_id() != 0) {
    total_size += 2 + 1;
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData InputReader::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    InputReader::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*InputReader::GetClassData() const { return &_class_data_; }


void InputReader::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<InputReader*>(&to_msg);
  auto& from = static_cast<const InputReader&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.InputReader)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.class_names_.MergeFrom(from._impl_.class_names_);
  _this->_impl_.groundtruth_localization_noise_std_.MergeFrom(from._impl_.groundtruth_localization_noise_std_);
  _this->_impl_.groundtruth_rotation_uniform_noise_.MergeFrom(from._impl_.groundtruth_rotation_uniform_noise_);
  _this->_impl_.global_rotation_uniform_noise_.MergeFrom(from._impl_.global_rotation_uniform_noise_);
  _this->_impl_.global_scaling_uniform_noise_.MergeFrom(from._impl_.global_scaling_uniform_noise_);
  _this->_impl_.global_random_rotation_range_per_object_.MergeFrom(from._impl_.global_random_rotation_range_per_object_);
  _this->_impl_.database_prep_steps_.MergeFrom(from._impl_.database_prep_steps_);
  if (!from._internal_record_file_path().empty()) {
    _this->_internal_set_record_file_path(from._internal_record_file_path());
  }
  if (!from._internal_kitti_info_path().empty()) {
    _this->_internal_set_kitti_info_path(from._internal_kitti_info_path());
  }
  if (!from._internal_kitti_root_path().empty()) {
    _this->_internal_set_kitti_root_path(from._internal_kitti_root_path());
  }
  if (from._internal_has_target_assigner()) {
    _this->_internal_mutable_target_assigner()->::second::protos::TargetAssigner::MergeFrom(
        from._internal_target_assigner());
  }
  if (from._internal_has_database_sampler()) {
    _this->_internal_mutable_database_sampler()->::second::protos::Sampler::MergeFrom(
        from._internal_database_sampler());
  }
  if (from._internal_has_unlabeled_database_sampler()) {
    _this->_internal_mutable_unlabeled_database_sampler()->::second::protos::Sampler::MergeFrom(
        from._internal_unlabeled_database_sampler());
  }
  if (from._internal_batch_size() != 0) {
    _this->_internal_set_batch_size(from._internal_batch_size());
  }
  if (from._internal_max_num_epochs() != 0) {
    _this->_internal_set_max_num_epochs(from._internal_max_num_epochs());
  }
  if (from._internal_prefetch_size() != 0) {
    _this->_internal_set_prefetch_size(from._internal_prefetch_size());
  }
  if (from._internal_max_number_of_voxels() != 0) {
    _this->_internal_set_max_number_of_voxels(from._internal_max_number_of_voxels());
  }
  if (from._internal_num_workers() != 0) {
    _this->_internal_set_num_workers(from._internal_num_workers());
  }
  if (from._internal_shuffle_points() != 0) {
    _this->_internal_set_shuffle_points(from._internal_shuffle_points());
  }
  if (from._internal_remove_unknown_examples() != 0) {
    _this->_internal_set_remove_unknown_examples(from._internal_remove_unknown_examples());
  }
  if (from._internal_remove_points_after_sample() != 0) {
    _this->_internal_set_remove_points_after_sample(from._internal_remove_points_after_sample());
  }
  if (from._internal_remove_environment() != 0) {
    _this->_internal_set_remove_environment(from._internal_remove_environment());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_anchor_area_threshold = from._internal_anchor_area_threshold();
  uint32_t raw_anchor_area_threshold;
  memcpy(&raw_anchor_area_threshold, &tmp_anchor_area_threshold, sizeof(tmp_anchor_area_threshold));
  if (raw_anchor_area_threshold != 0) {
    _this->_internal_set_anchor_area_threshold(from._internal_anchor_area_threshold());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_groundtruth_points_drop_percentage = from._internal_groundtruth_points_drop_percentage();
  uint32_t raw_groundtruth_points_drop_percentage;
  memcpy(&raw_groundtruth_points_drop_percentage, &tmp_groundtruth_points_drop_percentage, sizeof(tmp_groundtruth_points_drop_percentage));
  if (raw_groundtruth_points_drop_percentage != 0) {
    _this->_internal_set_groundtruth_points_drop_percentage(from._internal_groundtruth_points_drop_percentage());
  }
  if (from._internal_groundtruth_drop_max_keep_points() != 0) {
    _this->_internal_set_groundtruth_drop_max_keep_points(from._internal_groundtruth_drop_max_keep_points());
  }
  if (from._internal_unlabeled_training() != 0) {
    _this->_internal_set_unlabeled_training(from._internal_unlabeled_training());
  }
  if (from._internal_use_group_id() != 0) {
    _this->_internal_set_use_group_id(from._internal_use_group_id());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void InputReader::CopyFrom(const InputReader& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.InputReader)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool InputReader::IsInitialized() const {
  return true;
}

void InputReader::InternalSwap(InputReader* other) {
  using std::swap;
  auto* lhs_arena = GetArenaForAllocation();
  auto* rhs_arena = other->GetArenaForAllocation();
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  _impl_.class_names_.InternalSwap(&other->_impl_.class_names_);
  _impl_.groundtruth_localization_noise_std_.InternalSwap(&other->_impl_.groundtruth_localization_noise_std_);
  _impl_.groundtruth_rotation_uniform_noise_.InternalSwap(&other->_impl_.groundtruth_rotation_uniform_noise_);
  _impl_.global_rotation_uniform_noise_.InternalSwap(&other->_impl_.global_rotation_uniform_noise_);
  _impl_.global_scaling_uniform_noise_.InternalSwap(&other->_impl_.global_scaling_uniform_noise_);
  _impl_.global_random_rotation_range_per_object_.InternalSwap(&other->_impl_.global_random_rotation_range_per_object_);
  _impl_.database_prep_steps_.InternalSwap(&other->_impl_.database_prep_steps_);
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.record_file_path_, lhs_arena,
      &other->_impl_.record_file_path_, rhs_arena
  );
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.kitti_info_path_, lhs_arena,
      &other->_impl_.kitti_info_path_, rhs_arena
  );
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.kitti_root_path_, lhs_arena,
      &other->_impl_.kitti_root_path_, rhs_arena
  );
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(InputReader, _impl_.use_group_id_)
      + sizeof(InputReader::_impl_.use_group_id_)
      - PROTOBUF_FIELD_OFFSET(InputReader, _impl_.target_assigner_)>(
          reinterpret_cast<char*>(&_impl_.target_assigner_),
          reinterpret_cast<char*>(&other->_impl_.target_assigner_));
}

::PROTOBUF_NAMESPACE_ID::Metadata InputReader::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2finput_5freader_2eproto_getter, &descriptor_table_second_2fprotos_2finput_5freader_2eproto_once,
      file_level_metadata_second_2fprotos_2finput_5freader_2eproto[0]);
}

// @@protoc_insertion_point(namespace_scope)
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> PROTOBUF_NOINLINE ::second::protos::InputReader*
Arena::CreateMaybeMessage< ::second::protos::InputReader >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::InputReader >(arena);
}
PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)
#include <google/protobuf/port_undef.inc>
