// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/preprocess.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fpreprocess_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fpreprocess_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/map.h>  // IWYU pragma: export
#include <google/protobuf/map_entry.h>
#include <google/protobuf/map_field_inl.h>
#include <google/protobuf/unknown_field_set.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_second_2fprotos_2fpreprocess_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_second_2fprotos_2fpreprocess_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_second_2fprotos_2fpreprocess_2eproto;
namespace second {
namespace protos {
class DBFilterByDifficulty;
struct DBFilterByDifficultyDefaultTypeInternal;
extern DBFilterByDifficultyDefaultTypeInternal _DBFilterByDifficulty_default_instance_;
class DBFilterByMinNumPointInGroundTruth;
struct DBFilterByMinNumPointInGroundTruthDefaultTypeInternal;
extern DBFilterByMinNumPointInGroundTruthDefaultTypeInternal _DBFilterByMinNumPointInGroundTruth_default_instance_;
class DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse;
struct DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUseDefaultTypeInternal;
extern DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUseDefaultTypeInternal _DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse_default_instance_;
class DatabasePreprocessingStep;
struct DatabasePreprocessingStepDefaultTypeInternal;
extern DatabasePreprocessingStepDefaultTypeInternal _DatabasePreprocessingStep_default_instance_;
class Preprocess;
struct PreprocessDefaultTypeInternal;
extern PreprocessDefaultTypeInternal _Preprocess_default_instance_;
class PreprocessingStep;
struct PreprocessingStepDefaultTypeInternal;
extern PreprocessingStepDefaultTypeInternal _PreprocessingStep_default_instance_;
class RandomGlobalRotation;
struct RandomGlobalRotationDefaultTypeInternal;
extern RandomGlobalRotationDefaultTypeInternal _RandomGlobalRotation_default_instance_;
class RandomGlobalScaling;
struct RandomGlobalScalingDefaultTypeInternal;
extern RandomGlobalScalingDefaultTypeInternal _RandomGlobalScaling_default_instance_;
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> ::second::protos::DBFilterByDifficulty* Arena::CreateMaybeMessage<::second::protos::DBFilterByDifficulty>(Arena*);
template<> ::second::protos::DBFilterByMinNumPointInGroundTruth* Arena::CreateMaybeMessage<::second::protos::DBFilterByMinNumPointInGroundTruth>(Arena*);
template<> ::second::protos::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse* Arena::CreateMaybeMessage<::second::protos::DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse>(Arena*);
template<> ::second::protos::DatabasePreprocessingStep* Arena::CreateMaybeMessage<::second::protos::DatabasePreprocessingStep>(Arena*);
template<> ::second::protos::Preprocess* Arena::CreateMaybeMessage<::second::protos::Preprocess>(Arena*);
template<> ::second::protos::PreprocessingStep* Arena::CreateMaybeMessage<::second::protos::PreprocessingStep>(Arena*);
template<> ::second::protos::RandomGlobalRotation* Arena::CreateMaybeMessage<::second::protos::RandomGlobalRotation>(Arena*);
template<> ::second::protos::RandomGlobalScaling* Arena::CreateMaybeMessage<::second::protos::RandomGlobalScaling>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace second {
namespace protos {

// ===================================================================

class Preprocess final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.Preprocess) */ {
 public:
  inline Preprocess() : Preprocess(nullptr) {}
  ~Preprocess() override;
  explicit PROTOBUF_CONSTEXPR Preprocess(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Preprocess(const Preprocess& from);
  Preprocess(Preprocess&& from) noexcept
    : Preprocess() {
    *this = ::std::move(from);
  }

  inline Preprocess& operator=(const Preprocess& from) {
    CopyFrom(from);
    return *this;
  }
  inline Preprocess& operator=(Preprocess&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Preprocess& default_instance() {
    return *internal_default_instance();
  }
  static inline const Preprocess* internal_default_instance() {
    return reinterpret_cast<const Preprocess*>(
               &_Preprocess_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(Preprocess& a, Preprocess& b) {
    a.Swap(&b);
  }
  inline void Swap(Preprocess* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Preprocess* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Preprocess* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<Preprocess>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const Preprocess& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const Preprocess& from) {
    Preprocess::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Preprocess* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.Preprocess";
  }
  protected:
  explicit Preprocess(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kRandomGlobalRotationMinRadFieldNumber = 1,
    kRandomGlobalRotationMaxRadFieldNumber = 2,
    kRandomGlobalScalingMinFieldNumber = 3,
    kRandomGlobalScalingMaxFieldNumber = 4,
    kRandomNoisePerGroundtruthMinRadFieldNumber = 5,
    kRandomNoisePerGroundtruthMaxRadFieldNumber = 6,
    kRandomNoisePerGroundtruthPositionStdFieldNumber = 7,
  };
  // float random_global_rotation_min_rad = 1;
  void clear_random_global_rotation_min_rad();
  float random_global_rotation_min_rad() const;
  void set_random_global_rotation_min_rad(float value);
  private:
  float _internal_random_global_rotation_min_rad() const;
  void _internal_set_random_global_rotation_min_rad(float value);
  public:

  // float random_global_rotation_max_rad = 2;
  void clear_random_global_rotation_max_rad();
  float random_global_rotation_max_rad() const;
  void set_random_global_rotation_max_rad(float value);
  private:
  float _internal_random_global_rotation_max_rad() const;
  void _internal_set_random_global_rotation_max_rad(float value);
  public:

  // float random_global_scaling_min = 3;
  void clear_random_global_scaling_min();
  float random_global_scaling_min() const;
  void set_random_global_scaling_min(float value);
  private:
  float _internal_random_global_scaling_min() const;
  void _internal_set_random_global_scaling_min(float value);
  public:

  // float random_global_scaling_max = 4;
  void clear_random_global_scaling_max();
  float random_global_scaling_max() const;
  void set_random_global_scaling_max(float value);
  private:
  float _internal_random_global_scaling_max() const;
  void _internal_set_random_global_scaling_max(float value);
  public:

  // float random_noise_per_groundtruth_min_rad = 5;
  void clear_random_noise_per_groundtruth_min_rad();
  float random_noise_per_groundtruth_min_rad() const;
  void set_random_noise_per_groundtruth_min_rad(float value);
  private:
  float _internal_random_noise_per_groundtruth_min_rad() const;
  void _internal_set_random_noise_per_groundtruth_min_rad(float value);
  public:

  // float random_noise_per_groundtruth_max_rad = 6;
  void clear_random_noise_per_groundtruth_max_rad();
  float random_noise_per_groundtruth_max_rad() const;
  void set_random_noise_per_groundtruth_max_rad(float value);
  private:
  float _internal_random_noise_per_groundtruth_max_rad() const;
  void _internal_set_random_noise_per_groundtruth_max_rad(float value);
  public:

  // float random_noise_per_groundtruth_position_std = 7;
  void clear_random_noise_per_groundtruth_position_std();
  float random_noise_per_groundtruth_position_std() const;
  void set_random_noise_per_groundtruth_position_std(float value);
  private:
  float _internal_random_noise_per_groundtruth_position_std() const;
  void _internal_set_random_noise_per_groundtruth_position_std(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.Preprocess)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    float random_global_rotation_min_rad_;
    float random_global_rotation_max_rad_;
    float random_global_scaling_min_;
    float random_global_scaling_max_;
    float random_noise_per_groundtruth_min_rad_;
    float random_noise_per_groundtruth_max_rad_;
    float random_noise_per_groundtruth_position_std_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fpreprocess_2eproto;
};
// -------------------------------------------------------------------

class DatabasePreprocessingStep final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.DatabasePreprocessingStep) */ {
 public:
  inline DatabasePreprocessingStep() : DatabasePreprocessingStep(nullptr) {}
  ~DatabasePreprocessingStep() override;
  explicit PROTOBUF_CONSTEXPR DatabasePreprocessingStep(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  DatabasePreprocessingStep(const DatabasePreprocessingStep& from);
  DatabasePreprocessingStep(DatabasePreprocessingStep&& from) noexcept
    : DatabasePreprocessingStep() {
    *this = ::std::move(from);
  }

  inline DatabasePreprocessingStep& operator=(const DatabasePreprocessingStep& from) {
    CopyFrom(from);
    return *this;
  }
  inline DatabasePreprocessingStep& operator=(DatabasePreprocessingStep&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const DatabasePreprocessingStep& default_instance() {
    return *internal_default_instance();
  }
  enum DatabasePreprocessingStepCase {
    kFilterByDifficulty = 1,
    kFilterByMinNumPoints = 2,
    DATABASE_PREPROCESSING_STEP_NOT_SET = 0,
  };

  static inline const DatabasePreprocessingStep* internal_default_instance() {
    return reinterpret_cast<const DatabasePreprocessingStep*>(
               &_DatabasePreprocessingStep_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(DatabasePreprocessingStep& a, DatabasePreprocessingStep& b) {
    a.Swap(&b);
  }
  inline void Swap(DatabasePreprocessingStep* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DatabasePreprocessingStep* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  DatabasePreprocessingStep* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<DatabasePreprocessingStep>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const DatabasePreprocessingStep& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const DatabasePreprocessingStep& from) {
    DatabasePreprocessingStep::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(DatabasePreprocessingStep* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.DatabasePreprocessingStep";
  }
  protected:
  explicit DatabasePreprocessingStep(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kFilterByDifficultyFieldNumber = 1,
    kFilterByMinNumPointsFieldNumber = 2,
  };
  // .second.protos.DBFilterByDifficulty filter_by_difficulty = 1;
  bool has_filter_by_difficulty() const;
  private:
  bool _internal_has_filter_by_difficulty() const;
  public:
  void clear_filter_by_difficulty();
  const ::second::protos::DBFilterByDifficulty& filter_by_difficulty() const;
  PROTOBUF_NODISCARD ::second::protos::DBFilterByDifficulty* release_filter_by_difficulty();
  ::second::protos::DBFilterByDifficulty* mutable_filter_by_difficulty();
  void set_allocated_filter_by_difficulty(::second::protos::DBFilterByDifficulty* filter_by_difficulty);
  private:
  const ::second::protos::DBFilterByDifficulty& _internal_filter_by_difficulty() const;
  ::second::protos::DBFilterByDifficulty* _internal_mutable_filter_by_difficulty();
  public:
  void unsafe_arena_set_allocated_filter_by_difficulty(
      ::second::protos::DBFilterByDifficulty* filter_by_difficulty);
  ::second::protos::DBFilterByDifficulty* unsafe_arena_release_filter_by_difficulty();

  // .second.protos.DBFilterByMinNumPointInGroundTruth filter_by_min_num_points = 2;
  bool has_filter_by_min_num_points() const;
  private:
  bool _internal_has_filter_by_min_num_points() const;
  public:
  void clear_filter_by_min_num_points();
  const ::second::protos::DBFilterByMinNumPointInGroundTruth& filter_by_min_num_points() const;
  PROTOBUF_NODISCARD ::second::protos::DBFilterByMinNumPointInGroundTruth* release_filter_by_min_num_points();
  ::second::protos::DBFilterByMinNumPointInGroundTruth* mutable_filter_by_min_num_points();
  void set_allocated_filter_by_min_num_points(::second::protos::DBFilterByMinNumPointInGroundTruth* filter_by_min_num_points);
  private:
  const ::second::protos::DBFilterByMinNumPointInGroundTruth& _internal_filter_by_min_num_points() const;
  ::second::protos::DBFilterByMinNumPointInGroundTruth* _internal_mutable_filter_by_min_num_points();
  public:
  void unsafe_arena_set_allocated_filter_by_min_num_points(
      ::second::protos::DBFilterByMinNumPointInGroundTruth* filter_by_min_num_points);
  ::second::protos::DBFilterByMinNumPointInGroundTruth* unsafe_arena_release_filter_by_min_num_points();

  void clear_database_preprocessing_step();
  DatabasePreprocessingStepCase database_preprocessing_step_case() const;
  // @@protoc_insertion_point(class_scope:second.protos.DatabasePreprocessingStep)
 private:
  class _Internal;
  void set_has_filter_by_difficulty();
  void set_has_filter_by_min_num_points();

  inline bool has_database_preprocessing_step() const;
  inline void clear_has_database_preprocessing_step();

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    union DatabasePreprocessingStepUnion {
      constexpr DatabasePreprocessingStepUnion() : _constinit_{} {}
        ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized _constinit_;
      ::second::protos::DBFilterByDifficulty* filter_by_difficulty_;
      ::second::protos::DBFilterByMinNumPointInGroundTruth* filter_by_min_num_points_;
    } database_preprocessing_step_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    uint32_t _oneof_case_[1];

  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fpreprocess_2eproto;
};
// -------------------------------------------------------------------

class DBFilterByDifficulty final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.DBFilterByDifficulty) */ {
 public:
  inline DBFilterByDifficulty() : DBFilterByDifficulty(nullptr) {}
  ~DBFilterByDifficulty() override;
  explicit PROTOBUF_CONSTEXPR DBFilterByDifficulty(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  DBFilterByDifficulty(const DBFilterByDifficulty& from);
  DBFilterByDifficulty(DBFilterByDifficulty&& from) noexcept
    : DBFilterByDifficulty() {
    *this = ::std::move(from);
  }

  inline DBFilterByDifficulty& operator=(const DBFilterByDifficulty& from) {
    CopyFrom(from);
    return *this;
  }
  inline DBFilterByDifficulty& operator=(DBFilterByDifficulty&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const DBFilterByDifficulty& default_instance() {
    return *internal_default_instance();
  }
  static inline const DBFilterByDifficulty* internal_default_instance() {
    return reinterpret_cast<const DBFilterByDifficulty*>(
               &_DBFilterByDifficulty_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    2;

  friend void swap(DBFilterByDifficulty& a, DBFilterByDifficulty& b) {
    a.Swap(&b);
  }
  inline void Swap(DBFilterByDifficulty* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DBFilterByDifficulty* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  DBFilterByDifficulty* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<DBFilterByDifficulty>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const DBFilterByDifficulty& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const DBFilterByDifficulty& from) {
    DBFilterByDifficulty::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(DBFilterByDifficulty* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.DBFilterByDifficulty";
  }
  protected:
  explicit DBFilterByDifficulty(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kRemovedDifficultiesFieldNumber = 1,
  };
  // repeated int32 removed_difficulties = 1;
  int removed_difficulties_size() const;
  private:
  int _internal_removed_difficulties_size() const;
  public:
  void clear_removed_difficulties();
  private:
  int32_t _internal_removed_difficulties(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< int32_t >&
      _internal_removed_difficulties() const;
  void _internal_add_removed_difficulties(int32_t value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< int32_t >*
      _internal_mutable_removed_difficulties();
  public:
  int32_t removed_difficulties(int index) const;
  void set_removed_difficulties(int index, int32_t value);
  void add_removed_difficulties(int32_t value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< int32_t >&
      removed_difficulties() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< int32_t >*
      mutable_removed_difficulties();

  // @@protoc_insertion_point(class_scope:second.protos.DBFilterByDifficulty)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< int32_t > removed_difficulties_;
    mutable std::atomic<int> _removed_difficulties_cached_byte_size_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fpreprocess_2eproto;
};
// -------------------------------------------------------------------

class DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse : public ::PROTOBUF_NAMESPACE_ID::internal::MapEntry<DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse, 
    std::string, uint32_t,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_STRING,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_UINT32> {
public:
  typedef ::PROTOBUF_NAMESPACE_ID::internal::MapEntry<DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse, 
    std::string, uint32_t,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_STRING,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_UINT32> SuperType;
  DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse();
  explicit PROTOBUF_CONSTEXPR DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse(
      ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);
  explicit DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  void MergeFrom(const DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse& other);
  static const DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse* internal_default_instance() { return reinterpret_cast<const DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse*>(&_DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse_default_instance_); }
  static bool ValidateKey(std::string* s) {
    return ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(s->data(), static_cast<int>(s->size()), ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::PARSE, "second.protos.DBFilterByMinNumPointInGroundTruth.MinNumPointPairsEntry.key");
 }
  static bool ValidateValue(void*) { return true; }
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;
  friend struct ::TableStruct_second_2fprotos_2fpreprocess_2eproto;
};

// -------------------------------------------------------------------

class DBFilterByMinNumPointInGroundTruth final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.DBFilterByMinNumPointInGroundTruth) */ {
 public:
  inline DBFilterByMinNumPointInGroundTruth() : DBFilterByMinNumPointInGroundTruth(nullptr) {}
  ~DBFilterByMinNumPointInGroundTruth() override;
  explicit PROTOBUF_CONSTEXPR DBFilterByMinNumPointInGroundTruth(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  DBFilterByMinNumPointInGroundTruth(const DBFilterByMinNumPointInGroundTruth& from);
  DBFilterByMinNumPointInGroundTruth(DBFilterByMinNumPointInGroundTruth&& from) noexcept
    : DBFilterByMinNumPointInGroundTruth() {
    *this = ::std::move(from);
  }

  inline DBFilterByMinNumPointInGroundTruth& operator=(const DBFilterByMinNumPointInGroundTruth& from) {
    CopyFrom(from);
    return *this;
  }
  inline DBFilterByMinNumPointInGroundTruth& operator=(DBFilterByMinNumPointInGroundTruth&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const DBFilterByMinNumPointInGroundTruth& default_instance() {
    return *internal_default_instance();
  }
  static inline const DBFilterByMinNumPointInGroundTruth* internal_default_instance() {
    return reinterpret_cast<const DBFilterByMinNumPointInGroundTruth*>(
               &_DBFilterByMinNumPointInGroundTruth_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    4;

  friend void swap(DBFilterByMinNumPointInGroundTruth& a, DBFilterByMinNumPointInGroundTruth& b) {
    a.Swap(&b);
  }
  inline void Swap(DBFilterByMinNumPointInGroundTruth* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DBFilterByMinNumPointInGroundTruth* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  DBFilterByMinNumPointInGroundTruth* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<DBFilterByMinNumPointInGroundTruth>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const DBFilterByMinNumPointInGroundTruth& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const DBFilterByMinNumPointInGroundTruth& from) {
    DBFilterByMinNumPointInGroundTruth::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(DBFilterByMinNumPointInGroundTruth* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.DBFilterByMinNumPointInGroundTruth";
  }
  protected:
  explicit DBFilterByMinNumPointInGroundTruth(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  private:
  static void ArenaDtor(void* object);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------


  // accessors -------------------------------------------------------

  enum : int {
    kMinNumPointPairsFieldNumber = 1,
  };
  // map<string, uint32> min_num_point_pairs = 1;
  int min_num_point_pairs_size() const;
  private:
  int _internal_min_num_point_pairs_size() const;
  public:
  void clear_min_num_point_pairs();
  private:
  const ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >&
      _internal_min_num_point_pairs() const;
  ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >*
      _internal_mutable_min_num_point_pairs();
  public:
  const ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >&
      min_num_point_pairs() const;
  ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >*
      mutable_min_num_point_pairs();

  // @@protoc_insertion_point(class_scope:second.protos.DBFilterByMinNumPointInGroundTruth)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::MapField<
        DBFilterByMinNumPointInGroundTruth_MinNumPointPairsEntry_DoNotUse,
        std::string, uint32_t,
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_STRING,
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_UINT32> min_num_point_pairs_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fpreprocess_2eproto;
};
// -------------------------------------------------------------------

class PreprocessingStep final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.PreprocessingStep) */ {
 public:
  inline PreprocessingStep() : PreprocessingStep(nullptr) {}
  ~PreprocessingStep() override;
  explicit PROTOBUF_CONSTEXPR PreprocessingStep(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  PreprocessingStep(const PreprocessingStep& from);
  PreprocessingStep(PreprocessingStep&& from) noexcept
    : PreprocessingStep() {
    *this = ::std::move(from);
  }

  inline PreprocessingStep& operator=(const PreprocessingStep& from) {
    CopyFrom(from);
    return *this;
  }
  inline PreprocessingStep& operator=(PreprocessingStep&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PreprocessingStep& default_instance() {
    return *internal_default_instance();
  }
  enum PreprocessingStepCase {
    kRandomGlobalScaling = 1,
    kRandomGlobalRotation = 2,
    PREPROCESSING_STEP_NOT_SET = 0,
  };

  static inline const PreprocessingStep* internal_default_instance() {
    return reinterpret_cast<const PreprocessingStep*>(
               &_PreprocessingStep_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    5;

  friend void swap(PreprocessingStep& a, PreprocessingStep& b) {
    a.Swap(&b);
  }
  inline void Swap(PreprocessingStep* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PreprocessingStep* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PreprocessingStep* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<PreprocessingStep>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const PreprocessingStep& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const PreprocessingStep& from) {
    PreprocessingStep::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(PreprocessingStep* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.PreprocessingStep";
  }
  protected:
  explicit PreprocessingStep(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kRandomGlobalScalingFieldNumber = 1,
    kRandomGlobalRotationFieldNumber = 2,
  };
  // .second.protos.RandomGlobalScaling random_global_scaling = 1;
  bool has_random_global_scaling() const;
  private:
  bool _internal_has_random_global_scaling() const;
  public:
  void clear_random_global_scaling();
  const ::second::protos::RandomGlobalScaling& random_global_scaling() const;
  PROTOBUF_NODISCARD ::second::protos::RandomGlobalScaling* release_random_global_scaling();
  ::second::protos::RandomGlobalScaling* mutable_random_global_scaling();
  void set_allocated_random_global_scaling(::second::protos::RandomGlobalScaling* random_global_scaling);
  private:
  const ::second::protos::RandomGlobalScaling& _internal_random_global_scaling() const;
  ::second::protos::RandomGlobalScaling* _internal_mutable_random_global_scaling();
  public:
  void unsafe_arena_set_allocated_random_global_scaling(
      ::second::protos::RandomGlobalScaling* random_global_scaling);
  ::second::protos::RandomGlobalScaling* unsafe_arena_release_random_global_scaling();

  // .second.protos.RandomGlobalRotation random_global_rotation = 2;
  bool has_random_global_rotation() const;
  private:
  bool _internal_has_random_global_rotation() const;
  public:
  void clear_random_global_rotation();
  const ::second::protos::RandomGlobalRotation& random_global_rotation() const;
  PROTOBUF_NODISCARD ::second::protos::RandomGlobalRotation* release_random_global_rotation();
  ::second::protos::RandomGlobalRotation* mutable_random_global_rotation();
  void set_allocated_random_global_rotation(::second::protos::RandomGlobalRotation* random_global_rotation);
  private:
  const ::second::protos::RandomGlobalRotation& _internal_random_global_rotation() const;
  ::second::protos::RandomGlobalRotation* _internal_mutable_random_global_rotation();
  public:
  void unsafe_arena_set_allocated_random_global_rotation(
      ::second::protos::RandomGlobalRotation* random_global_rotation);
  ::second::protos::RandomGlobalRotation* unsafe_arena_release_random_global_rotation();

  void clear_preprocessing_step();
  PreprocessingStepCase preprocessing_step_case() const;
  // @@protoc_insertion_point(class_scope:second.protos.PreprocessingStep)
 private:
  class _Internal;
  void set_has_random_global_scaling();
  void set_has_random_global_rotation();

  inline bool has_preprocessing_step() const;
  inline void clear_has_preprocessing_step();

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    union PreprocessingStepUnion {
      constexpr PreprocessingStepUnion() : _constinit_{} {}
        ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized _constinit_;
      ::second::protos::RandomGlobalScaling* random_global_scaling_;
      ::second::protos::RandomGlobalRotation* random_global_rotation_;
    } preprocessing_step_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    uint32_t _oneof_case_[1];

  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fpreprocess_2eproto;
};
// -------------------------------------------------------------------

class RandomGlobalScaling final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.RandomGlobalScaling) */ {
 public:
  inline RandomGlobalScaling() : RandomGlobalScaling(nullptr) {}
  ~RandomGlobalScaling() override;
  explicit PROTOBUF_CONSTEXPR RandomGlobalScaling(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  RandomGlobalScaling(const RandomGlobalScaling& from);
  RandomGlobalScaling(RandomGlobalScaling&& from) noexcept
    : RandomGlobalScaling() {
    *this = ::std::move(from);
  }

  inline RandomGlobalScaling& operator=(const RandomGlobalScaling& from) {
    CopyFrom(from);
    return *this;
  }
  inline RandomGlobalScaling& operator=(RandomGlobalScaling&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RandomGlobalScaling& default_instance() {
    return *internal_default_instance();
  }
  static inline const RandomGlobalScaling* internal_default_instance() {
    return reinterpret_cast<const RandomGlobalScaling*>(
               &_RandomGlobalScaling_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    6;

  friend void swap(RandomGlobalScaling& a, RandomGlobalScaling& b) {
    a.Swap(&b);
  }
  inline void Swap(RandomGlobalScaling* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RandomGlobalScaling* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RandomGlobalScaling* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<RandomGlobalScaling>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const RandomGlobalScaling& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const RandomGlobalScaling& from) {
    RandomGlobalScaling::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(RandomGlobalScaling* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.RandomGlobalScaling";
  }
  protected:
  explicit RandomGlobalScaling(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kMinScaleFieldNumber = 1,
    kMaxScaleFieldNumber = 2,
  };
  // float min_scale = 1;
  void clear_min_scale();
  float min_scale() const;
  void set_min_scale(float value);
  private:
  float _internal_min_scale() const;
  void _internal_set_min_scale(float value);
  public:

  // float max_scale = 2;
  void clear_max_scale();
  float max_scale() const;
  void set_max_scale(float value);
  private:
  float _internal_max_scale() const;
  void _internal_set_max_scale(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.RandomGlobalScaling)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    float min_scale_;
    float max_scale_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fpreprocess_2eproto;
};
// -------------------------------------------------------------------

class RandomGlobalRotation final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.RandomGlobalRotation) */ {
 public:
  inline RandomGlobalRotation() : RandomGlobalRotation(nullptr) {}
  ~RandomGlobalRotation() override;
  explicit PROTOBUF_CONSTEXPR RandomGlobalRotation(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  RandomGlobalRotation(const RandomGlobalRotation& from);
  RandomGlobalRotation(RandomGlobalRotation&& from) noexcept
    : RandomGlobalRotation() {
    *this = ::std::move(from);
  }

  inline RandomGlobalRotation& operator=(const RandomGlobalRotation& from) {
    CopyFrom(from);
    return *this;
  }
  inline RandomGlobalRotation& operator=(RandomGlobalRotation&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RandomGlobalRotation& default_instance() {
    return *internal_default_instance();
  }
  static inline const RandomGlobalRotation* internal_default_instance() {
    return reinterpret_cast<const RandomGlobalRotation*>(
               &_RandomGlobalRotation_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    7;

  friend void swap(RandomGlobalRotation& a, RandomGlobalRotation& b) {
    a.Swap(&b);
  }
  inline void Swap(RandomGlobalRotation* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RandomGlobalRotation* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RandomGlobalRotation* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<RandomGlobalRotation>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const RandomGlobalRotation& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const RandomGlobalRotation& from) {
    RandomGlobalRotation::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(RandomGlobalRotation* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.RandomGlobalRotation";
  }
  protected:
  explicit RandomGlobalRotation(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kMinRadFieldNumber = 1,
    kMaxRadFieldNumber = 2,
  };
  // float min_rad = 1;
  void clear_min_rad();
  float min_rad() const;
  void set_min_rad(float value);
  private:
  float _internal_min_rad() const;
  void _internal_set_min_rad(float value);
  public:

  // float max_rad = 2;
  void clear_max_rad();
  float max_rad() const;
  void set_max_rad(float value);
  private:
  float _internal_max_rad() const;
  void _internal_set_max_rad(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.RandomGlobalRotation)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    float min_rad_;
    float max_rad_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fpreprocess_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// Preprocess

// float random_global_rotation_min_rad = 1;
inline void Preprocess::clear_random_global_rotation_min_rad() {
  _impl_.random_global_rotation_min_rad_ = 0;
}
inline float Preprocess::_internal_random_global_rotation_min_rad() const {
  return _impl_.random_global_rotation_min_rad_;
}
inline float Preprocess::random_global_rotation_min_rad() const {
  // @@protoc_insertion_point(field_get:second.protos.Preprocess.random_global_rotation_min_rad)
  return _internal_random_global_rotation_min_rad();
}
inline void Preprocess::_internal_set_random_global_rotation_min_rad(float value) {
  
  _impl_.random_global_rotation_min_rad_ = value;
}
inline void Preprocess::set_random_global_rotation_min_rad(float value) {
  _internal_set_random_global_rotation_min_rad(value);
  // @@protoc_insertion_point(field_set:second.protos.Preprocess.random_global_rotation_min_rad)
}

// float random_global_rotation_max_rad = 2;
inline void Preprocess::clear_random_global_rotation_max_rad() {
  _impl_.random_global_rotation_max_rad_ = 0;
}
inline float Preprocess::_internal_random_global_rotation_max_rad() const {
  return _impl_.random_global_rotation_max_rad_;
}
inline float Preprocess::random_global_rotation_max_rad() const {
  // @@protoc_insertion_point(field_get:second.protos.Preprocess.random_global_rotation_max_rad)
  return _internal_random_global_rotation_max_rad();
}
inline void Preprocess::_internal_set_random_global_rotation_max_rad(float value) {
  
  _impl_.random_global_rotation_max_rad_ = value;
}
inline void Preprocess::set_random_global_rotation_max_rad(float value) {
  _internal_set_random_global_rotation_max_rad(value);
  // @@protoc_insertion_point(field_set:second.protos.Preprocess.random_global_rotation_max_rad)
}

// float random_global_scaling_min = 3;
inline void Preprocess::clear_random_global_scaling_min() {
  _impl_.random_global_scaling_min_ = 0;
}
inline float Preprocess::_internal_random_global_scaling_min() const {
  return _impl_.random_global_scaling_min_;
}
inline float Preprocess::random_global_scaling_min() const {
  // @@protoc_insertion_point(field_get:second.protos.Preprocess.random_global_scaling_min)
  return _internal_random_global_scaling_min();
}
inline void Preprocess::_internal_set_random_global_scaling_min(float value) {
  
  _impl_.random_global_scaling_min_ = value;
}
inline void Preprocess::set_random_global_scaling_min(float value) {
  _internal_set_random_global_scaling_min(value);
  // @@protoc_insertion_point(field_set:second.protos.Preprocess.random_global_scaling_min)
}

// float random_global_scaling_max = 4;
inline void Preprocess::clear_random_global_scaling_max() {
  _impl_.random_global_scaling_max_ = 0;
}
inline float Preprocess::_internal_random_global_scaling_max() const {
  return _impl_.random_global_scaling_max_;
}
inline float Preprocess::random_global_scaling_max() const {
  // @@protoc_insertion_point(field_get:second.protos.Preprocess.random_global_scaling_max)
  return _internal_random_global_scaling_max();
}
inline void Preprocess::_internal_set_random_global_scaling_max(float value) {
  
  _impl_.random_global_scaling_max_ = value;
}
inline void Preprocess::set_random_global_scaling_max(float value) {
  _internal_set_random_global_scaling_max(value);
  // @@protoc_insertion_point(field_set:second.protos.Preprocess.random_global_scaling_max)
}

// float random_noise_per_groundtruth_min_rad = 5;
inline void Preprocess::clear_random_noise_per_groundtruth_min_rad() {
  _impl_.random_noise_per_groundtruth_min_rad_ = 0;
}
inline float Preprocess::_internal_random_noise_per_groundtruth_min_rad() const {
  return _impl_.random_noise_per_groundtruth_min_rad_;
}
inline float Preprocess::random_noise_per_groundtruth_min_rad() const {
  // @@protoc_insertion_point(field_get:second.protos.Preprocess.random_noise_per_groundtruth_min_rad)
  return _internal_random_noise_per_groundtruth_min_rad();
}
inline void Preprocess::_internal_set_random_noise_per_groundtruth_min_rad(float value) {
  
  _impl_.random_noise_per_groundtruth_min_rad_ = value;
}
inline void Preprocess::set_random_noise_per_groundtruth_min_rad(float value) {
  _internal_set_random_noise_per_groundtruth_min_rad(value);
  // @@protoc_insertion_point(field_set:second.protos.Preprocess.random_noise_per_groundtruth_min_rad)
}

// float random_noise_per_groundtruth_max_rad = 6;
inline void Preprocess::clear_random_noise_per_groundtruth_max_rad() {
  _impl_.random_noise_per_groundtruth_max_rad_ = 0;
}
inline float Preprocess::_internal_random_noise_per_groundtruth_max_rad() const {
  return _impl_.random_noise_per_groundtruth_max_rad_;
}
inline float Preprocess::random_noise_per_groundtruth_max_rad() const {
  // @@protoc_insertion_point(field_get:second.protos.Preprocess.random_noise_per_groundtruth_max_rad)
  return _internal_random_noise_per_groundtruth_max_rad();
}
inline void Preprocess::_internal_set_random_noise_per_groundtruth_max_rad(float value) {
  
  _impl_.random_noise_per_groundtruth_max_rad_ = value;
}
inline void Preprocess::set_random_noise_per_groundtruth_max_rad(float value) {
  _internal_set_random_noise_per_groundtruth_max_rad(value);
  // @@protoc_insertion_point(field_set:second.protos.Preprocess.random_noise_per_groundtruth_max_rad)
}

// float random_noise_per_groundtruth_position_std = 7;
inline void Preprocess::clear_random_noise_per_groundtruth_position_std() {
  _impl_.random_noise_per_groundtruth_position_std_ = 0;
}
inline float Preprocess::_internal_random_noise_per_groundtruth_position_std() const {
  return _impl_.random_noise_per_groundtruth_position_std_;
}
inline float Preprocess::random_noise_per_groundtruth_position_std() const {
  // @@protoc_insertion_point(field_get:second.protos.Preprocess.random_noise_per_groundtruth_position_std)
  return _internal_random_noise_per_groundtruth_position_std();
}
inline void Preprocess::_internal_set_random_noise_per_groundtruth_position_std(float value) {
  
  _impl_.random_noise_per_groundtruth_position_std_ = value;
}
inline void Preprocess::set_random_noise_per_groundtruth_position_std(float value) {
  _internal_set_random_noise_per_groundtruth_position_std(value);
  // @@protoc_insertion_point(field_set:second.protos.Preprocess.random_noise_per_groundtruth_position_std)
}

// -------------------------------------------------------------------

// DatabasePreprocessingStep

// .second.protos.DBFilterByDifficulty filter_by_difficulty = 1;
inline bool DatabasePreprocessingStep::_internal_has_filter_by_difficulty() const {
  return database_preprocessing_step_case() == kFilterByDifficulty;
}
inline bool DatabasePreprocessingStep::has_filter_by_difficulty() const {
  return _internal_has_filter_by_difficulty();
}
inline void DatabasePreprocessingStep::set_has_filter_by_difficulty() {
  _impl_._oneof_case_[0] = kFilterByDifficulty;
}
inline void DatabasePreprocessingStep::clear_filter_by_difficulty() {
  if (_internal_has_filter_by_difficulty()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.database_preprocessing_step_.filter_by_difficulty_;
    }
    clear_has_database_preprocessing_step();
  }
}
inline ::second::protos::DBFilterByDifficulty* DatabasePreprocessingStep::release_filter_by_difficulty() {
  // @@protoc_insertion_point(field_release:second.protos.DatabasePreprocessingStep.filter_by_difficulty)
  if (_internal_has_filter_by_difficulty()) {
    clear_has_database_preprocessing_step();
    ::second::protos::DBFilterByDifficulty* temp = _impl_.database_preprocessing_step_.filter_by_difficulty_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.database_preprocessing_step_.filter_by_difficulty_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::second::protos::DBFilterByDifficulty& DatabasePreprocessingStep::_internal_filter_by_difficulty() const {
  return _internal_has_filter_by_difficulty()
      ? *_impl_.database_preprocessing_step_.filter_by_difficulty_
      : reinterpret_cast< ::second::protos::DBFilterByDifficulty&>(::second::protos::_DBFilterByDifficulty_default_instance_);
}
inline const ::second::protos::DBFilterByDifficulty& DatabasePreprocessingStep::filter_by_difficulty() const {
  // @@protoc_insertion_point(field_get:second.protos.DatabasePreprocessingStep.filter_by_difficulty)
  return _internal_filter_by_difficulty();
}
inline ::second::protos::DBFilterByDifficulty* DatabasePreprocessingStep::unsafe_arena_release_filter_by_difficulty() {
  // @@protoc_insertion_point(field_unsafe_arena_release:second.protos.DatabasePreprocessingStep.filter_by_difficulty)
  if (_internal_has_filter_by_difficulty()) {
    clear_has_database_preprocessing_step();
    ::second::protos::DBFilterByDifficulty* temp = _impl_.database_preprocessing_step_.filter_by_difficulty_;
    _impl_.database_preprocessing_step_.filter_by_difficulty_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void DatabasePreprocessingStep::unsafe_arena_set_allocated_filter_by_difficulty(::second::protos::DBFilterByDifficulty* filter_by_difficulty) {
  clear_database_preprocessing_step();
  if (filter_by_difficulty) {
    set_has_filter_by_difficulty();
    _impl_.database_preprocessing_step_.filter_by_difficulty_ = filter_by_difficulty;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.DatabasePreprocessingStep.filter_by_difficulty)
}
inline ::second::protos::DBFilterByDifficulty* DatabasePreprocessingStep::_internal_mutable_filter_by_difficulty() {
  if (!_internal_has_filter_by_difficulty()) {
    clear_database_preprocessing_step();
    set_has_filter_by_difficulty();
    _impl_.database_preprocessing_step_.filter_by_difficulty_ = CreateMaybeMessage< ::second::protos::DBFilterByDifficulty >(GetArenaForAllocation());
  }
  return _impl_.database_preprocessing_step_.filter_by_difficulty_;
}
inline ::second::protos::DBFilterByDifficulty* DatabasePreprocessingStep::mutable_filter_by_difficulty() {
  ::second::protos::DBFilterByDifficulty* _msg = _internal_mutable_filter_by_difficulty();
  // @@protoc_insertion_point(field_mutable:second.protos.DatabasePreprocessingStep.filter_by_difficulty)
  return _msg;
}

// .second.protos.DBFilterByMinNumPointInGroundTruth filter_by_min_num_points = 2;
inline bool DatabasePreprocessingStep::_internal_has_filter_by_min_num_points() const {
  return database_preprocessing_step_case() == kFilterByMinNumPoints;
}
inline bool DatabasePreprocessingStep::has_filter_by_min_num_points() const {
  return _internal_has_filter_by_min_num_points();
}
inline void DatabasePreprocessingStep::set_has_filter_by_min_num_points() {
  _impl_._oneof_case_[0] = kFilterByMinNumPoints;
}
inline void DatabasePreprocessingStep::clear_filter_by_min_num_points() {
  if (_internal_has_filter_by_min_num_points()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.database_preprocessing_step_.filter_by_min_num_points_;
    }
    clear_has_database_preprocessing_step();
  }
}
inline ::second::protos::DBFilterByMinNumPointInGroundTruth* DatabasePreprocessingStep::release_filter_by_min_num_points() {
  // @@protoc_insertion_point(field_release:second.protos.DatabasePreprocessingStep.filter_by_min_num_points)
  if (_internal_has_filter_by_min_num_points()) {
    clear_has_database_preprocessing_step();
    ::second::protos::DBFilterByMinNumPointInGroundTruth* temp = _impl_.database_preprocessing_step_.filter_by_min_num_points_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.database_preprocessing_step_.filter_by_min_num_points_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::second::protos::DBFilterByMinNumPointInGroundTruth& DatabasePreprocessingStep::_internal_filter_by_min_num_points() const {
  return _internal_has_filter_by_min_num_points()
      ? *_impl_.database_preprocessing_step_.filter_by_min_num_points_
      : reinterpret_cast< ::second::protos::DBFilterByMinNumPointInGroundTruth&>(::second::protos::_DBFilterByMinNumPointInGroundTruth_default_instance_);
}
inline const ::second::protos::DBFilterByMinNumPointInGroundTruth& DatabasePreprocessingStep::filter_by_min_num_points() const {
  // @@protoc_insertion_point(field_get:second.protos.DatabasePreprocessingStep.filter_by_min_num_points)
  return _internal_filter_by_min_num_points();
}
inline ::second::protos::DBFilterByMinNumPointInGroundTruth* DatabasePreprocessingStep::unsafe_arena_release_filter_by_min_num_points() {
  // @@protoc_insertion_point(field_unsafe_arena_release:second.protos.DatabasePreprocessingStep.filter_by_min_num_points)
  if (_internal_has_filter_by_min_num_points()) {
    clear_has_database_preprocessing_step();
    ::second::protos::DBFilterByMinNumPointInGroundTruth* temp = _impl_.database_preprocessing_step_.filter_by_min_num_points_;
    _impl_.database_preprocessing_step_.filter_by_min_num_points_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void DatabasePreprocessingStep::unsafe_arena_set_allocated_filter_by_min_num_points(::second::protos::DBFilterByMinNumPointInGroundTruth* filter_by_min_num_points) {
  clear_database_preprocessing_step();
  if (filter_by_min_num_points) {
    set_has_filter_by_min_num_points();
    _impl_.database_preprocessing_step_.filter_by_min_num_points_ = filter_by_min_num_points;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.DatabasePreprocessingStep.filter_by_min_num_points)
}
inline ::second::protos::DBFilterByMinNumPointInGroundTruth* DatabasePreprocessingStep::_internal_mutable_filter_by_min_num_points() {
  if (!_internal_has_filter_by_min_num_points()) {
    clear_database_preprocessing_step();
    set_has_filter_by_min_num_points();
    _impl_.database_preprocessing_step_.filter_by_min_num_points_ = CreateMaybeMessage< ::second::protos::DBFilterByMinNumPointInGroundTruth >(GetArenaForAllocation());
  }
  return _impl_.database_preprocessing_step_.filter_by_min_num_points_;
}
inline ::second::protos::DBFilterByMinNumPointInGroundTruth* DatabasePreprocessingStep::mutable_filter_by_min_num_points() {
  ::second::protos::DBFilterByMinNumPointInGroundTruth* _msg = _internal_mutable_filter_by_min_num_points();
  // @@protoc_insertion_point(field_mutable:second.protos.DatabasePreprocessingStep.filter_by_min_num_points)
  return _msg;
}

inline bool DatabasePreprocessingStep::has_database_preprocessing_step() const {
  return database_preprocessing_step_case() != DATABASE_PREPROCESSING_STEP_NOT_SET;
}
inline void DatabasePreprocessingStep::clear_has_database_preprocessing_step() {
  _impl_._oneof_case_[0] = DATABASE_PREPROCESSING_STEP_NOT_SET;
}
inline DatabasePreprocessingStep::DatabasePreprocessingStepCase DatabasePreprocessingStep::database_preprocessing_step_case() const {
  return DatabasePreprocessingStep::DatabasePreprocessingStepCase(_impl_._oneof_case_[0]);
}
// -------------------------------------------------------------------

// DBFilterByDifficulty

// repeated int32 removed_difficulties = 1;
inline int DBFilterByDifficulty::_internal_removed_difficulties_size() const {
  return _impl_.removed_difficulties_.size();
}
inline int DBFilterByDifficulty::removed_difficulties_size() const {
  return _internal_removed_difficulties_size();
}
inline void DBFilterByDifficulty::clear_removed_difficulties() {
  _impl_.removed_difficulties_.Clear();
}
inline int32_t DBFilterByDifficulty::_internal_removed_difficulties(int index) const {
  return _impl_.removed_difficulties_.Get(index);
}
inline int32_t DBFilterByDifficulty::removed_difficulties(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.DBFilterByDifficulty.removed_difficulties)
  return _internal_removed_difficulties(index);
}
inline void DBFilterByDifficulty::set_removed_difficulties(int index, int32_t value) {
  _impl_.removed_difficulties_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.DBFilterByDifficulty.removed_difficulties)
}
inline void DBFilterByDifficulty::_internal_add_removed_difficulties(int32_t value) {
  _impl_.removed_difficulties_.Add(value);
}
inline void DBFilterByDifficulty::add_removed_difficulties(int32_t value) {
  _internal_add_removed_difficulties(value);
  // @@protoc_insertion_point(field_add:second.protos.DBFilterByDifficulty.removed_difficulties)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< int32_t >&
DBFilterByDifficulty::_internal_removed_difficulties() const {
  return _impl_.removed_difficulties_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< int32_t >&
DBFilterByDifficulty::removed_difficulties() const {
  // @@protoc_insertion_point(field_list:second.protos.DBFilterByDifficulty.removed_difficulties)
  return _internal_removed_difficulties();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< int32_t >*
DBFilterByDifficulty::_internal_mutable_removed_difficulties() {
  return &_impl_.removed_difficulties_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< int32_t >*
DBFilterByDifficulty::mutable_removed_difficulties() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.DBFilterByDifficulty.removed_difficulties)
  return _internal_mutable_removed_difficulties();
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// DBFilterByMinNumPointInGroundTruth

// map<string, uint32> min_num_point_pairs = 1;
inline int DBFilterByMinNumPointInGroundTruth::_internal_min_num_point_pairs_size() const {
  return _impl_.min_num_point_pairs_.size();
}
inline int DBFilterByMinNumPointInGroundTruth::min_num_point_pairs_size() const {
  return _internal_min_num_point_pairs_size();
}
inline void DBFilterByMinNumPointInGroundTruth::clear_min_num_point_pairs() {
  _impl_.min_num_point_pairs_.Clear();
}
inline const ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >&
DBFilterByMinNumPointInGroundTruth::_internal_min_num_point_pairs() const {
  return _impl_.min_num_point_pairs_.GetMap();
}
inline const ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >&
DBFilterByMinNumPointInGroundTruth::min_num_point_pairs() const {
  // @@protoc_insertion_point(field_map:second.protos.DBFilterByMinNumPointInGroundTruth.min_num_point_pairs)
  return _internal_min_num_point_pairs();
}
inline ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >*
DBFilterByMinNumPointInGroundTruth::_internal_mutable_min_num_point_pairs() {
  return _impl_.min_num_point_pairs_.MutableMap();
}
inline ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >*
DBFilterByMinNumPointInGroundTruth::mutable_min_num_point_pairs() {
  // @@protoc_insertion_point(field_mutable_map:second.protos.DBFilterByMinNumPointInGroundTruth.min_num_point_pairs)
  return _internal_mutable_min_num_point_pairs();
}

// -------------------------------------------------------------------

// PreprocessingStep

// .second.protos.RandomGlobalScaling random_global_scaling = 1;
inline bool PreprocessingStep::_internal_has_random_global_scaling() const {
  return preprocessing_step_case() == kRandomGlobalScaling;
}
inline bool PreprocessingStep::has_random_global_scaling() const {
  return _internal_has_random_global_scaling();
}
inline void PreprocessingStep::set_has_random_global_scaling() {
  _impl_._oneof_case_[0] = kRandomGlobalScaling;
}
inline void PreprocessingStep::clear_random_global_scaling() {
  if (_internal_has_random_global_scaling()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.preprocessing_step_.random_global_scaling_;
    }
    clear_has_preprocessing_step();
  }
}
inline ::second::protos::RandomGlobalScaling* PreprocessingStep::release_random_global_scaling() {
  // @@protoc_insertion_point(field_release:second.protos.PreprocessingStep.random_global_scaling)
  if (_internal_has_random_global_scaling()) {
    clear_has_preprocessing_step();
    ::second::protos::RandomGlobalScaling* temp = _impl_.preprocessing_step_.random_global_scaling_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.preprocessing_step_.random_global_scaling_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::second::protos::RandomGlobalScaling& PreprocessingStep::_internal_random_global_scaling() const {
  return _internal_has_random_global_scaling()
      ? *_impl_.preprocessing_step_.random_global_scaling_
      : reinterpret_cast< ::second::protos::RandomGlobalScaling&>(::second::protos::_RandomGlobalScaling_default_instance_);
}
inline const ::second::protos::RandomGlobalScaling& PreprocessingStep::random_global_scaling() const {
  // @@protoc_insertion_point(field_get:second.protos.PreprocessingStep.random_global_scaling)
  return _internal_random_global_scaling();
}
inline ::second::protos::RandomGlobalScaling* PreprocessingStep::unsafe_arena_release_random_global_scaling() {
  // @@protoc_insertion_point(field_unsafe_arena_release:second.protos.PreprocessingStep.random_global_scaling)
  if (_internal_has_random_global_scaling()) {
    clear_has_preprocessing_step();
    ::second::protos::RandomGlobalScaling* temp = _impl_.preprocessing_step_.random_global_scaling_;
    _impl_.preprocessing_step_.random_global_scaling_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void PreprocessingStep::unsafe_arena_set_allocated_random_global_scaling(::second::protos::RandomGlobalScaling* random_global_scaling) {
  clear_preprocessing_step();
  if (random_global_scaling) {
    set_has_random_global_scaling();
    _impl_.preprocessing_step_.random_global_scaling_ = random_global_scaling;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.PreprocessingStep.random_global_scaling)
}
inline ::second::protos::RandomGlobalScaling* PreprocessingStep::_internal_mutable_random_global_scaling() {
  if (!_internal_has_random_global_scaling()) {
    clear_preprocessing_step();
    set_has_random_global_scaling();
    _impl_.preprocessing_step_.random_global_scaling_ = CreateMaybeMessage< ::second::protos::RandomGlobalScaling >(GetArenaForAllocation());
  }
  return _impl_.preprocessing_step_.random_global_scaling_;
}
inline ::second::protos::RandomGlobalScaling* PreprocessingStep::mutable_random_global_scaling() {
  ::second::protos::RandomGlobalScaling* _msg = _internal_mutable_random_global_scaling();
  // @@protoc_insertion_point(field_mutable:second.protos.PreprocessingStep.random_global_scaling)
  return _msg;
}

// .second.protos.RandomGlobalRotation random_global_rotation = 2;
inline bool PreprocessingStep::_internal_has_random_global_rotation() const {
  return preprocessing_step_case() == kRandomGlobalRotation;
}
inline bool PreprocessingStep::has_random_global_rotation() const {
  return _internal_has_random_global_rotation();
}
inline void PreprocessingStep::set_has_random_global_rotation() {
  _impl_._oneof_case_[0] = kRandomGlobalRotation;
}
inline void PreprocessingStep::clear_random_global_rotation() {
  if (_internal_has_random_global_rotation()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.preprocessing_step_.random_global_rotation_;
    }
    clear_has_preprocessing_step();
  }
}
inline ::second::protos::RandomGlobalRotation* PreprocessingStep::release_random_global_rotation() {
  // @@protoc_insertion_point(field_release:second.protos.PreprocessingStep.random_global_rotation)
  if (_internal_has_random_global_rotation()) {
    clear_has_preprocessing_step();
    ::second::protos::RandomGlobalRotation* temp = _impl_.preprocessing_step_.random_global_rotation_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.preprocessing_step_.random_global_rotation_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::second::protos::RandomGlobalRotation& PreprocessingStep::_internal_random_global_rotation() const {
  return _internal_has_random_global_rotation()
      ? *_impl_.preprocessing_step_.random_global_rotation_
      : reinterpret_cast< ::second::protos::RandomGlobalRotation&>(::second::protos::_RandomGlobalRotation_default_instance_);
}
inline const ::second::protos::RandomGlobalRotation& PreprocessingStep::random_global_rotation() const {
  // @@protoc_insertion_point(field_get:second.protos.PreprocessingStep.random_global_rotation)
  return _internal_random_global_rotation();
}
inline ::second::protos::RandomGlobalRotation* PreprocessingStep::unsafe_arena_release_random_global_rotation() {
  // @@protoc_insertion_point(field_unsafe_arena_release:second.protos.PreprocessingStep.random_global_rotation)
  if (_internal_has_random_global_rotation()) {
    clear_has_preprocessing_step();
    ::second::protos::RandomGlobalRotation* temp = _impl_.preprocessing_step_.random_global_rotation_;
    _impl_.preprocessing_step_.random_global_rotation_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void PreprocessingStep::unsafe_arena_set_allocated_random_global_rotation(::second::protos::RandomGlobalRotation* random_global_rotation) {
  clear_preprocessing_step();
  if (random_global_rotation) {
    set_has_random_global_rotation();
    _impl_.preprocessing_step_.random_global_rotation_ = random_global_rotation;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.PreprocessingStep.random_global_rotation)
}
inline ::second::protos::RandomGlobalRotation* PreprocessingStep::_internal_mutable_random_global_rotation() {
  if (!_internal_has_random_global_rotation()) {
    clear_preprocessing_step();
    set_has_random_global_rotation();
    _impl_.preprocessing_step_.random_global_rotation_ = CreateMaybeMessage< ::second::protos::RandomGlobalRotation >(GetArenaForAllocation());
  }
  return _impl_.preprocessing_step_.random_global_rotation_;
}
inline ::second::protos::RandomGlobalRotation* PreprocessingStep::mutable_random_global_rotation() {
  ::second::protos::RandomGlobalRotation* _msg = _internal_mutable_random_global_rotation();
  // @@protoc_insertion_point(field_mutable:second.protos.PreprocessingStep.random_global_rotation)
  return _msg;
}

inline bool PreprocessingStep::has_preprocessing_step() const {
  return preprocessing_step_case() != PREPROCESSING_STEP_NOT_SET;
}
inline void PreprocessingStep::clear_has_preprocessing_step() {
  _impl_._oneof_case_[0] = PREPROCESSING_STEP_NOT_SET;
}
inline PreprocessingStep::PreprocessingStepCase PreprocessingStep::preprocessing_step_case() const {
  return PreprocessingStep::PreprocessingStepCase(_impl_._oneof_case_[0]);
}
// -------------------------------------------------------------------

// RandomGlobalScaling

// float min_scale = 1;
inline void RandomGlobalScaling::clear_min_scale() {
  _impl_.min_scale_ = 0;
}
inline float RandomGlobalScaling::_internal_min_scale() const {
  return _impl_.min_scale_;
}
inline float RandomGlobalScaling::min_scale() const {
  // @@protoc_insertion_point(field_get:second.protos.RandomGlobalScaling.min_scale)
  return _internal_min_scale();
}
inline void RandomGlobalScaling::_internal_set_min_scale(float value) {
  
  _impl_.min_scale_ = value;
}
inline void RandomGlobalScaling::set_min_scale(float value) {
  _internal_set_min_scale(value);
  // @@protoc_insertion_point(field_set:second.protos.RandomGlobalScaling.min_scale)
}

// float max_scale = 2;
inline void RandomGlobalScaling::clear_max_scale() {
  _impl_.max_scale_ = 0;
}
inline float RandomGlobalScaling::_internal_max_scale() const {
  return _impl_.max_scale_;
}
inline float RandomGlobalScaling::max_scale() const {
  // @@protoc_insertion_point(field_get:second.protos.RandomGlobalScaling.max_scale)
  return _internal_max_scale();
}
inline void RandomGlobalScaling::_internal_set_max_scale(float value) {
  
  _impl_.max_scale_ = value;
}
inline void RandomGlobalScaling::set_max_scale(float value) {
  _internal_set_max_scale(value);
  // @@protoc_insertion_point(field_set:second.protos.RandomGlobalScaling.max_scale)
}

// -------------------------------------------------------------------

// RandomGlobalRotation

// float min_rad = 1;
inline void RandomGlobalRotation::clear_min_rad() {
  _impl_.min_rad_ = 0;
}
inline float RandomGlobalRotation::_internal_min_rad() const {
  return _impl_.min_rad_;
}
inline float RandomGlobalRotation::min_rad() const {
  // @@protoc_insertion_point(field_get:second.protos.RandomGlobalRotation.min_rad)
  return _internal_min_rad();
}
inline void RandomGlobalRotation::_internal_set_min_rad(float value) {
  
  _impl_.min_rad_ = value;
}
inline void RandomGlobalRotation::set_min_rad(float value) {
  _internal_set_min_rad(value);
  // @@protoc_insertion_point(field_set:second.protos.RandomGlobalRotation.min_rad)
}

// float max_rad = 2;
inline void RandomGlobalRotation::clear_max_rad() {
  _impl_.max_rad_ = 0;
}
inline float RandomGlobalRotation::_internal_max_rad() const {
  return _impl_.max_rad_;
}
inline float RandomGlobalRotation::max_rad() const {
  // @@protoc_insertion_point(field_get:second.protos.RandomGlobalRotation.max_rad)
  return _internal_max_rad();
}
inline void RandomGlobalRotation::_internal_set_max_rad(float value) {
  
  _impl_.max_rad_ = value;
}
inline void RandomGlobalRotation::set_max_rad(float value) {
  _internal_set_max_rad(value);
  // @@protoc_insertion_point(field_set:second.protos.RandomGlobalRotation.max_rad)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------


// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace second

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fpreprocess_2eproto
