// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/pipeline.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fpipeline_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fpipeline_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "second/protos/input_reader.pb.h"
#include "second/protos/model.pb.h"
#include "second/protos/train.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_second_2fprotos_2fpipeline_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_second_2fprotos_2fpipeline_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_second_2fprotos_2fpipeline_2eproto;
namespace second {
namespace protos {
class TrainEvalPipelineConfig;
struct TrainEvalPipelineConfigDefaultTypeInternal;
extern TrainEvalPipelineConfigDefaultTypeInternal _TrainEvalPipelineConfig_default_instance_;
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> ::second::protos::TrainEvalPipelineConfig* Arena::CreateMaybeMessage<::second::protos::TrainEvalPipelineConfig>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace second {
namespace protos {

// ===================================================================

class TrainEvalPipelineConfig final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.TrainEvalPipelineConfig) */ {
 public:
  inline TrainEvalPipelineConfig() : TrainEvalPipelineConfig(nullptr) {}
  ~TrainEvalPipelineConfig() override;
  explicit PROTOBUF_CONSTEXPR TrainEvalPipelineConfig(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  TrainEvalPipelineConfig(const TrainEvalPipelineConfig& from);
  TrainEvalPipelineConfig(TrainEvalPipelineConfig&& from) noexcept
    : TrainEvalPipelineConfig() {
    *this = ::std::move(from);
  }

  inline TrainEvalPipelineConfig& operator=(const TrainEvalPipelineConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline TrainEvalPipelineConfig& operator=(TrainEvalPipelineConfig&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const TrainEvalPipelineConfig& default_instance() {
    return *internal_default_instance();
  }
  static inline const TrainEvalPipelineConfig* internal_default_instance() {
    return reinterpret_cast<const TrainEvalPipelineConfig*>(
               &_TrainEvalPipelineConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(TrainEvalPipelineConfig& a, TrainEvalPipelineConfig& b) {
    a.Swap(&b);
  }
  inline void Swap(TrainEvalPipelineConfig* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TrainEvalPipelineConfig* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  TrainEvalPipelineConfig* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<TrainEvalPipelineConfig>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const TrainEvalPipelineConfig& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const TrainEvalPipelineConfig& from) {
    TrainEvalPipelineConfig::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(TrainEvalPipelineConfig* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.TrainEvalPipelineConfig";
  }
  protected:
  explicit TrainEvalPipelineConfig(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kModelFieldNumber = 1,
    kTrainInputReaderFieldNumber = 2,
    kTrainConfigFieldNumber = 3,
    kEvalInputReaderFieldNumber = 4,
  };
  // .second.protos.DetectionModel model = 1;
  bool has_model() const;
  private:
  bool _internal_has_model() const;
  public:
  void clear_model();
  const ::second::protos::DetectionModel& model() const;
  PROTOBUF_NODISCARD ::second::protos::DetectionModel* release_model();
  ::second::protos::DetectionModel* mutable_model();
  void set_allocated_model(::second::protos::DetectionModel* model);
  private:
  const ::second::protos::DetectionModel& _internal_model() const;
  ::second::protos::DetectionModel* _internal_mutable_model();
  public:
  void unsafe_arena_set_allocated_model(
      ::second::protos::DetectionModel* model);
  ::second::protos::DetectionModel* unsafe_arena_release_model();

  // .second.protos.InputReader train_input_reader = 2;
  bool has_train_input_reader() const;
  private:
  bool _internal_has_train_input_reader() const;
  public:
  void clear_train_input_reader();
  const ::second::protos::InputReader& train_input_reader() const;
  PROTOBUF_NODISCARD ::second::protos::InputReader* release_train_input_reader();
  ::second::protos::InputReader* mutable_train_input_reader();
  void set_allocated_train_input_reader(::second::protos::InputReader* train_input_reader);
  private:
  const ::second::protos::InputReader& _internal_train_input_reader() const;
  ::second::protos::InputReader* _internal_mutable_train_input_reader();
  public:
  void unsafe_arena_set_allocated_train_input_reader(
      ::second::protos::InputReader* train_input_reader);
  ::second::protos::InputReader* unsafe_arena_release_train_input_reader();

  // .second.protos.TrainConfig train_config = 3;
  bool has_train_config() const;
  private:
  bool _internal_has_train_config() const;
  public:
  void clear_train_config();
  const ::second::protos::TrainConfig& train_config() const;
  PROTOBUF_NODISCARD ::second::protos::TrainConfig* release_train_config();
  ::second::protos::TrainConfig* mutable_train_config();
  void set_allocated_train_config(::second::protos::TrainConfig* train_config);
  private:
  const ::second::protos::TrainConfig& _internal_train_config() const;
  ::second::protos::TrainConfig* _internal_mutable_train_config();
  public:
  void unsafe_arena_set_allocated_train_config(
      ::second::protos::TrainConfig* train_config);
  ::second::protos::TrainConfig* unsafe_arena_release_train_config();

  // .second.protos.InputReader eval_input_reader = 4;
  bool has_eval_input_reader() const;
  private:
  bool _internal_has_eval_input_reader() const;
  public:
  void clear_eval_input_reader();
  const ::second::protos::InputReader& eval_input_reader() const;
  PROTOBUF_NODISCARD ::second::protos::InputReader* release_eval_input_reader();
  ::second::protos::InputReader* mutable_eval_input_reader();
  void set_allocated_eval_input_reader(::second::protos::InputReader* eval_input_reader);
  private:
  const ::second::protos::InputReader& _internal_eval_input_reader() const;
  ::second::protos::InputReader* _internal_mutable_eval_input_reader();
  public:
  void unsafe_arena_set_allocated_eval_input_reader(
      ::second::protos::InputReader* eval_input_reader);
  ::second::protos::InputReader* unsafe_arena_release_eval_input_reader();

  // @@protoc_insertion_point(class_scope:second.protos.TrainEvalPipelineConfig)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::second::protos::DetectionModel* model_;
    ::second::protos::InputReader* train_input_reader_;
    ::second::protos::TrainConfig* train_config_;
    ::second::protos::InputReader* eval_input_reader_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fpipeline_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// TrainEvalPipelineConfig

// .second.protos.DetectionModel model = 1;
inline bool TrainEvalPipelineConfig::_internal_has_model() const {
  return this != internal_default_instance() && _impl_.model_ != nullptr;
}
inline bool TrainEvalPipelineConfig::has_model() const {
  return _internal_has_model();
}
inline const ::second::protos::DetectionModel& TrainEvalPipelineConfig::_internal_model() const {
  const ::second::protos::DetectionModel* p = _impl_.model_;
  return p != nullptr ? *p : reinterpret_cast<const ::second::protos::DetectionModel&>(
      ::second::protos::_DetectionModel_default_instance_);
}
inline const ::second::protos::DetectionModel& TrainEvalPipelineConfig::model() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainEvalPipelineConfig.model)
  return _internal_model();
}
inline void TrainEvalPipelineConfig::unsafe_arena_set_allocated_model(
    ::second::protos::DetectionModel* model) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.model_);
  }
  _impl_.model_ = model;
  if (model) {
    
  } else {
    
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.TrainEvalPipelineConfig.model)
}
inline ::second::protos::DetectionModel* TrainEvalPipelineConfig::release_model() {
  
  ::second::protos::DetectionModel* temp = _impl_.model_;
  _impl_.model_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::second::protos::DetectionModel* TrainEvalPipelineConfig::unsafe_arena_release_model() {
  // @@protoc_insertion_point(field_release:second.protos.TrainEvalPipelineConfig.model)
  
  ::second::protos::DetectionModel* temp = _impl_.model_;
  _impl_.model_ = nullptr;
  return temp;
}
inline ::second::protos::DetectionModel* TrainEvalPipelineConfig::_internal_mutable_model() {
  
  if (_impl_.model_ == nullptr) {
    auto* p = CreateMaybeMessage<::second::protos::DetectionModel>(GetArenaForAllocation());
    _impl_.model_ = p;
  }
  return _impl_.model_;
}
inline ::second::protos::DetectionModel* TrainEvalPipelineConfig::mutable_model() {
  ::second::protos::DetectionModel* _msg = _internal_mutable_model();
  // @@protoc_insertion_point(field_mutable:second.protos.TrainEvalPipelineConfig.model)
  return _msg;
}
inline void TrainEvalPipelineConfig::set_allocated_model(::second::protos::DetectionModel* model) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.model_);
  }
  if (model) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(model));
    if (message_arena != submessage_arena) {
      model = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, model, submessage_arena);
    }
    
  } else {
    
  }
  _impl_.model_ = model;
  // @@protoc_insertion_point(field_set_allocated:second.protos.TrainEvalPipelineConfig.model)
}

// .second.protos.InputReader train_input_reader = 2;
inline bool TrainEvalPipelineConfig::_internal_has_train_input_reader() const {
  return this != internal_default_instance() && _impl_.train_input_reader_ != nullptr;
}
inline bool TrainEvalPipelineConfig::has_train_input_reader() const {
  return _internal_has_train_input_reader();
}
inline const ::second::protos::InputReader& TrainEvalPipelineConfig::_internal_train_input_reader() const {
  const ::second::protos::InputReader* p = _impl_.train_input_reader_;
  return p != nullptr ? *p : reinterpret_cast<const ::second::protos::InputReader&>(
      ::second::protos::_InputReader_default_instance_);
}
inline const ::second::protos::InputReader& TrainEvalPipelineConfig::train_input_reader() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainEvalPipelineConfig.train_input_reader)
  return _internal_train_input_reader();
}
inline void TrainEvalPipelineConfig::unsafe_arena_set_allocated_train_input_reader(
    ::second::protos::InputReader* train_input_reader) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.train_input_reader_);
  }
  _impl_.train_input_reader_ = train_input_reader;
  if (train_input_reader) {
    
  } else {
    
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.TrainEvalPipelineConfig.train_input_reader)
}
inline ::second::protos::InputReader* TrainEvalPipelineConfig::release_train_input_reader() {
  
  ::second::protos::InputReader* temp = _impl_.train_input_reader_;
  _impl_.train_input_reader_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::second::protos::InputReader* TrainEvalPipelineConfig::unsafe_arena_release_train_input_reader() {
  // @@protoc_insertion_point(field_release:second.protos.TrainEvalPipelineConfig.train_input_reader)
  
  ::second::protos::InputReader* temp = _impl_.train_input_reader_;
  _impl_.train_input_reader_ = nullptr;
  return temp;
}
inline ::second::protos::InputReader* TrainEvalPipelineConfig::_internal_mutable_train_input_reader() {
  
  if (_impl_.train_input_reader_ == nullptr) {
    auto* p = CreateMaybeMessage<::second::protos::InputReader>(GetArenaForAllocation());
    _impl_.train_input_reader_ = p;
  }
  return _impl_.train_input_reader_;
}
inline ::second::protos::InputReader* TrainEvalPipelineConfig::mutable_train_input_reader() {
  ::second::protos::InputReader* _msg = _internal_mutable_train_input_reader();
  // @@protoc_insertion_point(field_mutable:second.protos.TrainEvalPipelineConfig.train_input_reader)
  return _msg;
}
inline void TrainEvalPipelineConfig::set_allocated_train_input_reader(::second::protos::InputReader* train_input_reader) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.train_input_reader_);
  }
  if (train_input_reader) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(train_input_reader));
    if (message_arena != submessage_arena) {
      train_input_reader = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, train_input_reader, submessage_arena);
    }
    
  } else {
    
  }
  _impl_.train_input_reader_ = train_input_reader;
  // @@protoc_insertion_point(field_set_allocated:second.protos.TrainEvalPipelineConfig.train_input_reader)
}

// .second.protos.TrainConfig train_config = 3;
inline bool TrainEvalPipelineConfig::_internal_has_train_config() const {
  return this != internal_default_instance() && _impl_.train_config_ != nullptr;
}
inline bool TrainEvalPipelineConfig::has_train_config() const {
  return _internal_has_train_config();
}
inline const ::second::protos::TrainConfig& TrainEvalPipelineConfig::_internal_train_config() const {
  const ::second::protos::TrainConfig* p = _impl_.train_config_;
  return p != nullptr ? *p : reinterpret_cast<const ::second::protos::TrainConfig&>(
      ::second::protos::_TrainConfig_default_instance_);
}
inline const ::second::protos::TrainConfig& TrainEvalPipelineConfig::train_config() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainEvalPipelineConfig.train_config)
  return _internal_train_config();
}
inline void TrainEvalPipelineConfig::unsafe_arena_set_allocated_train_config(
    ::second::protos::TrainConfig* train_config) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.train_config_);
  }
  _impl_.train_config_ = train_config;
  if (train_config) {
    
  } else {
    
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.TrainEvalPipelineConfig.train_config)
}
inline ::second::protos::TrainConfig* TrainEvalPipelineConfig::release_train_config() {
  
  ::second::protos::TrainConfig* temp = _impl_.train_config_;
  _impl_.train_config_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::second::protos::TrainConfig* TrainEvalPipelineConfig::unsafe_arena_release_train_config() {
  // @@protoc_insertion_point(field_release:second.protos.TrainEvalPipelineConfig.train_config)
  
  ::second::protos::TrainConfig* temp = _impl_.train_config_;
  _impl_.train_config_ = nullptr;
  return temp;
}
inline ::second::protos::TrainConfig* TrainEvalPipelineConfig::_internal_mutable_train_config() {
  
  if (_impl_.train_config_ == nullptr) {
    auto* p = CreateMaybeMessage<::second::protos::TrainConfig>(GetArenaForAllocation());
    _impl_.train_config_ = p;
  }
  return _impl_.train_config_;
}
inline ::second::protos::TrainConfig* TrainEvalPipelineConfig::mutable_train_config() {
  ::second::protos::TrainConfig* _msg = _internal_mutable_train_config();
  // @@protoc_insertion_point(field_mutable:second.protos.TrainEvalPipelineConfig.train_config)
  return _msg;
}
inline void TrainEvalPipelineConfig::set_allocated_train_config(::second::protos::TrainConfig* train_config) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.train_config_);
  }
  if (train_config) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(train_config));
    if (message_arena != submessage_arena) {
      train_config = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, train_config, submessage_arena);
    }
    
  } else {
    
  }
  _impl_.train_config_ = train_config;
  // @@protoc_insertion_point(field_set_allocated:second.protos.TrainEvalPipelineConfig.train_config)
}

// .second.protos.InputReader eval_input_reader = 4;
inline bool TrainEvalPipelineConfig::_internal_has_eval_input_reader() const {
  return this != internal_default_instance() && _impl_.eval_input_reader_ != nullptr;
}
inline bool TrainEvalPipelineConfig::has_eval_input_reader() const {
  return _internal_has_eval_input_reader();
}
inline const ::second::protos::InputReader& TrainEvalPipelineConfig::_internal_eval_input_reader() const {
  const ::second::protos::InputReader* p = _impl_.eval_input_reader_;
  return p != nullptr ? *p : reinterpret_cast<const ::second::protos::InputReader&>(
      ::second::protos::_InputReader_default_instance_);
}
inline const ::second::protos::InputReader& TrainEvalPipelineConfig::eval_input_reader() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainEvalPipelineConfig.eval_input_reader)
  return _internal_eval_input_reader();
}
inline void TrainEvalPipelineConfig::unsafe_arena_set_allocated_eval_input_reader(
    ::second::protos::InputReader* eval_input_reader) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.eval_input_reader_);
  }
  _impl_.eval_input_reader_ = eval_input_reader;
  if (eval_input_reader) {
    
  } else {
    
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.TrainEvalPipelineConfig.eval_input_reader)
}
inline ::second::protos::InputReader* TrainEvalPipelineConfig::release_eval_input_reader() {
  
  ::second::protos::InputReader* temp = _impl_.eval_input_reader_;
  _impl_.eval_input_reader_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::second::protos::InputReader* TrainEvalPipelineConfig::unsafe_arena_release_eval_input_reader() {
  // @@protoc_insertion_point(field_release:second.protos.TrainEvalPipelineConfig.eval_input_reader)
  
  ::second::protos::InputReader* temp = _impl_.eval_input_reader_;
  _impl_.eval_input_reader_ = nullptr;
  return temp;
}
inline ::second::protos::InputReader* TrainEvalPipelineConfig::_internal_mutable_eval_input_reader() {
  
  if (_impl_.eval_input_reader_ == nullptr) {
    auto* p = CreateMaybeMessage<::second::protos::InputReader>(GetArenaForAllocation());
    _impl_.eval_input_reader_ = p;
  }
  return _impl_.eval_input_reader_;
}
inline ::second::protos::InputReader* TrainEvalPipelineConfig::mutable_eval_input_reader() {
  ::second::protos::InputReader* _msg = _internal_mutable_eval_input_reader();
  // @@protoc_insertion_point(field_mutable:second.protos.TrainEvalPipelineConfig.eval_input_reader)
  return _msg;
}
inline void TrainEvalPipelineConfig::set_allocated_eval_input_reader(::second::protos::InputReader* eval_input_reader) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.eval_input_reader_);
  }
  if (eval_input_reader) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(eval_input_reader));
    if (message_arena != submessage_arena) {
      eval_input_reader = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, eval_input_reader, submessage_arena);
    }
    
  } else {
    
  }
  _impl_.eval_input_reader_ = eval_input_reader;
  // @@protoc_insertion_point(field_set_allocated:second.protos.TrainEvalPipelineConfig.eval_input_reader)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace second

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fpipeline_2eproto
