/*
 * Copyright 2019 xilinx Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "vitis/ai/c2d2_lite.hpp"

#include <string>

#include "./c2d2_lite_imp.hpp"

namespace vitis {
namespace ai {
C2D2_lite::C2D2_lite() {}
C2D2_lite::~C2D2_lite() {}

std::unique_ptr<C2D2_lite> C2D2_lite::create(const std::string& model_name0,
                                             const std::string& model_name1,
                                             bool need_preprocess) {
  return std::unique_ptr<C2D2_lite>(
      new C2D2_liteImp(model_name0, model_name1, need_preprocess));
}
}  // namespace ai
}  // namespace vitis
