// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/voxel_generator.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fvoxel_5fgenerator_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fvoxel_5fgenerator_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_second_2fprotos_2fvoxel_5fgenerator_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_second_2fprotos_2fvoxel_5fgenerator_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_second_2fprotos_2fvoxel_5fgenerator_2eproto;
namespace second {
namespace protos {
class VoxelGenerator;
struct VoxelGeneratorDefaultTypeInternal;
extern VoxelGeneratorDefaultTypeInternal _VoxelGenerator_default_instance_;
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> ::second::protos::VoxelGenerator* Arena::CreateMaybeMessage<::second::protos::VoxelGenerator>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace second {
namespace protos {

// ===================================================================

class VoxelGenerator final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.VoxelGenerator) */ {
 public:
  inline VoxelGenerator() : VoxelGenerator(nullptr) {}
  ~VoxelGenerator() override;
  explicit PROTOBUF_CONSTEXPR VoxelGenerator(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  VoxelGenerator(const VoxelGenerator& from);
  VoxelGenerator(VoxelGenerator&& from) noexcept
    : VoxelGenerator() {
    *this = ::std::move(from);
  }

  inline VoxelGenerator& operator=(const VoxelGenerator& from) {
    CopyFrom(from);
    return *this;
  }
  inline VoxelGenerator& operator=(VoxelGenerator&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const VoxelGenerator& default_instance() {
    return *internal_default_instance();
  }
  static inline const VoxelGenerator* internal_default_instance() {
    return reinterpret_cast<const VoxelGenerator*>(
               &_VoxelGenerator_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(VoxelGenerator& a, VoxelGenerator& b) {
    a.Swap(&b);
  }
  inline void Swap(VoxelGenerator* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(VoxelGenerator* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  VoxelGenerator* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<VoxelGenerator>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const VoxelGenerator& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const VoxelGenerator& from) {
    VoxelGenerator::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(VoxelGenerator* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.VoxelGenerator";
  }
  protected:
  explicit VoxelGenerator(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kVoxelSizeFieldNumber = 1,
    kPointCloudRangeFieldNumber = 2,
    kSubmanifoldSizeFieldNumber = 5,
    kMaxNumberOfPointsPerVoxelFieldNumber = 3,
    kSubmanifoldGroupFieldNumber = 4,
    kSubmanifoldMaxPointsFieldNumber = 6,
  };
  // repeated float voxel_size = 1;
  int voxel_size_size() const;
  private:
  int _internal_voxel_size_size() const;
  public:
  void clear_voxel_size();
  private:
  float _internal_voxel_size(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_voxel_size() const;
  void _internal_add_voxel_size(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_voxel_size();
  public:
  float voxel_size(int index) const;
  void set_voxel_size(int index, float value);
  void add_voxel_size(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      voxel_size() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_voxel_size();

  // repeated float point_cloud_range = 2;
  int point_cloud_range_size() const;
  private:
  int _internal_point_cloud_range_size() const;
  public:
  void clear_point_cloud_range();
  private:
  float _internal_point_cloud_range(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_point_cloud_range() const;
  void _internal_add_point_cloud_range(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_point_cloud_range();
  public:
  float point_cloud_range(int index) const;
  void set_point_cloud_range(int index, float value);
  void add_point_cloud_range(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      point_cloud_range() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_point_cloud_range();

  // repeated uint32 submanifold_size = 5;
  int submanifold_size_size() const;
  private:
  int _internal_submanifold_size_size() const;
  public:
  void clear_submanifold_size();
  private:
  uint32_t _internal_submanifold_size(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< uint32_t >&
      _internal_submanifold_size() const;
  void _internal_add_submanifold_size(uint32_t value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< uint32_t >*
      _internal_mutable_submanifold_size();
  public:
  uint32_t submanifold_size(int index) const;
  void set_submanifold_size(int index, uint32_t value);
  void add_submanifold_size(uint32_t value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< uint32_t >&
      submanifold_size() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< uint32_t >*
      mutable_submanifold_size();

  // uint32 max_number_of_points_per_voxel = 3;
  void clear_max_number_of_points_per_voxel();
  uint32_t max_number_of_points_per_voxel() const;
  void set_max_number_of_points_per_voxel(uint32_t value);
  private:
  uint32_t _internal_max_number_of_points_per_voxel() const;
  void _internal_set_max_number_of_points_per_voxel(uint32_t value);
  public:

  // bool submanifold_group = 4;
  void clear_submanifold_group();
  bool submanifold_group() const;
  void set_submanifold_group(bool value);
  private:
  bool _internal_submanifold_group() const;
  void _internal_set_submanifold_group(bool value);
  public:

  // uint32 submanifold_max_points = 6;
  void clear_submanifold_max_points();
  uint32_t submanifold_max_points() const;
  void set_submanifold_max_points(uint32_t value);
  private:
  uint32_t _internal_submanifold_max_points() const;
  void _internal_set_submanifold_max_points(uint32_t value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.VoxelGenerator)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > voxel_size_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > point_cloud_range_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< uint32_t > submanifold_size_;
    mutable std::atomic<int> _submanifold_size_cached_byte_size_;
    uint32_t max_number_of_points_per_voxel_;
    bool submanifold_group_;
    uint32_t submanifold_max_points_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fvoxel_5fgenerator_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// VoxelGenerator

// repeated float voxel_size = 1;
inline int VoxelGenerator::_internal_voxel_size_size() const {
  return _impl_.voxel_size_.size();
}
inline int VoxelGenerator::voxel_size_size() const {
  return _internal_voxel_size_size();
}
inline void VoxelGenerator::clear_voxel_size() {
  _impl_.voxel_size_.Clear();
}
inline float VoxelGenerator::_internal_voxel_size(int index) const {
  return _impl_.voxel_size_.Get(index);
}
inline float VoxelGenerator::voxel_size(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.VoxelGenerator.voxel_size)
  return _internal_voxel_size(index);
}
inline void VoxelGenerator::set_voxel_size(int index, float value) {
  _impl_.voxel_size_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.VoxelGenerator.voxel_size)
}
inline void VoxelGenerator::_internal_add_voxel_size(float value) {
  _impl_.voxel_size_.Add(value);
}
inline void VoxelGenerator::add_voxel_size(float value) {
  _internal_add_voxel_size(value);
  // @@protoc_insertion_point(field_add:second.protos.VoxelGenerator.voxel_size)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
VoxelGenerator::_internal_voxel_size() const {
  return _impl_.voxel_size_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
VoxelGenerator::voxel_size() const {
  // @@protoc_insertion_point(field_list:second.protos.VoxelGenerator.voxel_size)
  return _internal_voxel_size();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
VoxelGenerator::_internal_mutable_voxel_size() {
  return &_impl_.voxel_size_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
VoxelGenerator::mutable_voxel_size() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.VoxelGenerator.voxel_size)
  return _internal_mutable_voxel_size();
}

// repeated float point_cloud_range = 2;
inline int VoxelGenerator::_internal_point_cloud_range_size() const {
  return _impl_.point_cloud_range_.size();
}
inline int VoxelGenerator::point_cloud_range_size() const {
  return _internal_point_cloud_range_size();
}
inline void VoxelGenerator::clear_point_cloud_range() {
  _impl_.point_cloud_range_.Clear();
}
inline float VoxelGenerator::_internal_point_cloud_range(int index) const {
  return _impl_.point_cloud_range_.Get(index);
}
inline float VoxelGenerator::point_cloud_range(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.VoxelGenerator.point_cloud_range)
  return _internal_point_cloud_range(index);
}
inline void VoxelGenerator::set_point_cloud_range(int index, float value) {
  _impl_.point_cloud_range_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.VoxelGenerator.point_cloud_range)
}
inline void VoxelGenerator::_internal_add_point_cloud_range(float value) {
  _impl_.point_cloud_range_.Add(value);
}
inline void VoxelGenerator::add_point_cloud_range(float value) {
  _internal_add_point_cloud_range(value);
  // @@protoc_insertion_point(field_add:second.protos.VoxelGenerator.point_cloud_range)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
VoxelGenerator::_internal_point_cloud_range() const {
  return _impl_.point_cloud_range_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
VoxelGenerator::point_cloud_range() const {
  // @@protoc_insertion_point(field_list:second.protos.VoxelGenerator.point_cloud_range)
  return _internal_point_cloud_range();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
VoxelGenerator::_internal_mutable_point_cloud_range() {
  return &_impl_.point_cloud_range_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
VoxelGenerator::mutable_point_cloud_range() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.VoxelGenerator.point_cloud_range)
  return _internal_mutable_point_cloud_range();
}

// uint32 max_number_of_points_per_voxel = 3;
inline void VoxelGenerator::clear_max_number_of_points_per_voxel() {
  _impl_.max_number_of_points_per_voxel_ = 0u;
}
inline uint32_t VoxelGenerator::_internal_max_number_of_points_per_voxel() const {
  return _impl_.max_number_of_points_per_voxel_;
}
inline uint32_t VoxelGenerator::max_number_of_points_per_voxel() const {
  // @@protoc_insertion_point(field_get:second.protos.VoxelGenerator.max_number_of_points_per_voxel)
  return _internal_max_number_of_points_per_voxel();
}
inline void VoxelGenerator::_internal_set_max_number_of_points_per_voxel(uint32_t value) {
  
  _impl_.max_number_of_points_per_voxel_ = value;
}
inline void VoxelGenerator::set_max_number_of_points_per_voxel(uint32_t value) {
  _internal_set_max_number_of_points_per_voxel(value);
  // @@protoc_insertion_point(field_set:second.protos.VoxelGenerator.max_number_of_points_per_voxel)
}

// bool submanifold_group = 4;
inline void VoxelGenerator::clear_submanifold_group() {
  _impl_.submanifold_group_ = false;
}
inline bool VoxelGenerator::_internal_submanifold_group() const {
  return _impl_.submanifold_group_;
}
inline bool VoxelGenerator::submanifold_group() const {
  // @@protoc_insertion_point(field_get:second.protos.VoxelGenerator.submanifold_group)
  return _internal_submanifold_group();
}
inline void VoxelGenerator::_internal_set_submanifold_group(bool value) {
  
  _impl_.submanifold_group_ = value;
}
inline void VoxelGenerator::set_submanifold_group(bool value) {
  _internal_set_submanifold_group(value);
  // @@protoc_insertion_point(field_set:second.protos.VoxelGenerator.submanifold_group)
}

// repeated uint32 submanifold_size = 5;
inline int VoxelGenerator::_internal_submanifold_size_size() const {
  return _impl_.submanifold_size_.size();
}
inline int VoxelGenerator::submanifold_size_size() const {
  return _internal_submanifold_size_size();
}
inline void VoxelGenerator::clear_submanifold_size() {
  _impl_.submanifold_size_.Clear();
}
inline uint32_t VoxelGenerator::_internal_submanifold_size(int index) const {
  return _impl_.submanifold_size_.Get(index);
}
inline uint32_t VoxelGenerator::submanifold_size(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.VoxelGenerator.submanifold_size)
  return _internal_submanifold_size(index);
}
inline void VoxelGenerator::set_submanifold_size(int index, uint32_t value) {
  _impl_.submanifold_size_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.VoxelGenerator.submanifold_size)
}
inline void VoxelGenerator::_internal_add_submanifold_size(uint32_t value) {
  _impl_.submanifold_size_.Add(value);
}
inline void VoxelGenerator::add_submanifold_size(uint32_t value) {
  _internal_add_submanifold_size(value);
  // @@protoc_insertion_point(field_add:second.protos.VoxelGenerator.submanifold_size)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< uint32_t >&
VoxelGenerator::_internal_submanifold_size() const {
  return _impl_.submanifold_size_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< uint32_t >&
VoxelGenerator::submanifold_size() const {
  // @@protoc_insertion_point(field_list:second.protos.VoxelGenerator.submanifold_size)
  return _internal_submanifold_size();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< uint32_t >*
VoxelGenerator::_internal_mutable_submanifold_size() {
  return &_impl_.submanifold_size_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< uint32_t >*
VoxelGenerator::mutable_submanifold_size() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.VoxelGenerator.submanifold_size)
  return _internal_mutable_submanifold_size();
}

// uint32 submanifold_max_points = 6;
inline void VoxelGenerator::clear_submanifold_max_points() {
  _impl_.submanifold_max_points_ = 0u;
}
inline uint32_t VoxelGenerator::_internal_submanifold_max_points() const {
  return _impl_.submanifold_max_points_;
}
inline uint32_t VoxelGenerator::submanifold_max_points() const {
  // @@protoc_insertion_point(field_get:second.protos.VoxelGenerator.submanifold_max_points)
  return _internal_submanifold_max_points();
}
inline void VoxelGenerator::_internal_set_submanifold_max_points(uint32_t value) {
  
  _impl_.submanifold_max_points_ = value;
}
inline void VoxelGenerator::set_submanifold_max_points(uint32_t value) {
  _internal_set_submanifold_max_points(value);
  // @@protoc_insertion_point(field_set:second.protos.VoxelGenerator.submanifold_max_points)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace second

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fvoxel_5fgenerator_2eproto
