// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/layers.proto

#include "second/protos/layers.pb.h"

#include <algorithm>

#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/extension_set.h>
#include <google/protobuf/wire_format_lite.h>
#include <google/protobuf/descriptor.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/reflection_ops.h>
#include <google/protobuf/wire_format.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>

PROTOBUF_PRAGMA_INIT_SEG

namespace _pb = ::PROTOBUF_NAMESPACE_ID;
namespace _pbi = _pb::internal;

namespace second {
}  // namespace second
static constexpr ::_pb::EnumDescriptor const** file_level_enum_descriptors_second_2fprotos_2flayers_2eproto = nullptr;
static constexpr ::_pb::ServiceDescriptor const** file_level_service_descriptors_second_2fprotos_2flayers_2eproto = nullptr;
const uint32_t TableStruct_second_2fprotos_2flayers_2eproto::offsets[1] = {};
static constexpr ::_pbi::MigrationSchema* schemas = nullptr;
static constexpr ::_pb::Message* const* file_default_instances = nullptr;

const char descriptor_table_protodef_second_2fprotos_2flayers_2eproto[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) =
  "\n\032second/protos/layers.proto\022\006secondb\006pr"
  "oto3"
  ;
static ::_pbi::once_flag descriptor_table_second_2fprotos_2flayers_2eproto_once;
const ::_pbi::DescriptorTable descriptor_table_second_2fprotos_2flayers_2eproto = {
    false, false, 44, descriptor_table_protodef_second_2fprotos_2flayers_2eproto,
    "second/protos/layers.proto",
    &descriptor_table_second_2fprotos_2flayers_2eproto_once, nullptr, 0, 0,
    schemas, file_default_instances, TableStruct_second_2fprotos_2flayers_2eproto::offsets,
    nullptr, file_level_enum_descriptors_second_2fprotos_2flayers_2eproto,
    file_level_service_descriptors_second_2fprotos_2flayers_2eproto,
};
PROTOBUF_ATTRIBUTE_WEAK const ::_pbi::DescriptorTable* descriptor_table_second_2fprotos_2flayers_2eproto_getter() {
  return &descriptor_table_second_2fprotos_2flayers_2eproto;
}

// Force running AddDescriptors() at dynamic initialization time.
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::_pbi::AddDescriptorsRunner dynamic_init_dummy_second_2fprotos_2flayers_2eproto(&descriptor_table_second_2fprotos_2flayers_2eproto);
namespace second {

// @@protoc_insertion_point(namespace_scope)
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)
#include <google/protobuf/port_undef.inc>
