syntax = "proto2";

package object_detection.protos;

// Configuration proto for bipartite matcher. See
// matchers/bipartite_matcher.py for details.
message BipartiteMatcher {
  // Force constructed match objects to use matrix multiplication based gather
  // instead of standard tf.gather
  optional bool use_matmul_gather = 6 [default = false];
}
