# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vitis/ai/proto/dpu_model_param.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$vitis/ai/proto/dpu_model_param.proto\x12\x0evitis.ai.proto\"\x9c\x02\n\x0bYoloV3Param\x12\x13\n\x0bnum_classes\x18\x01 \x01(\x05\x12\x11\n\tanchorCnt\x18\x02 \x01(\x05\x12\x16\n\x0e\x63onf_threshold\x18\x03 \x01(\x02\x12\x15\n\rnms_threshold\x18\x04 \x01(\x02\x12\x0e\n\x06\x62iases\x18\x05 \x03(\x02\x12\x10\n\x08test_mAP\x18\x06 \x01(\x08\x12\x12\n\nlayer_name\x18\x07 \x03(\t\x12.\n\x04type\x18\x08 \x01(\x0e\x32 .vitis.ai.proto.YoloV3Param.Type\x12\x10\n\x08iou_type\x18\t \x01(\x05\">\n\x04Type\x12\n\n\x06NORMAL\x10\x00\x12\x0e\n\nYOLOV4_CSP\x10\x01\x12\n\n\x06YOLOV5\x10\x02\x12\x0e\n\nYOLOX_NANO\x10\x03\"\x9f\x01\n\x14MedicalPriorBoxParam\x12\x13\n\x0binput_shape\x18\x01 \x03(\x05\x12\x16\n\x0e\x66\x65\x61ture_shapes\x18\x02 \x03(\x05\x12\x11\n\tmin_sizes\x18\x03 \x03(\x05\x12\x11\n\tmax_sizes\x18\x04 \x03(\x05\x12\x15\n\raspect_ratios\x18\x05 \x03(\x02\x12\r\n\x05steps\x18\x06 \x03(\x05\x12\x0e\n\x06offset\x18\x07 \x01(\x02\"\xdf\x01\n\rPriorBoxParam\x12\x13\n\x0blayer_width\x18\x01 \x01(\x05\x12\x14\n\x0clayer_height\x18\x02 \x01(\x05\x12\x11\n\tvariances\x18\x04 \x03(\x02\x12\x11\n\tmin_sizes\x18\x05 \x03(\x02\x12\x11\n\tmax_sizes\x18\x06 \x03(\x02\x12\x15\n\raspect_ratios\x18\x07 \x03(\x02\x12\x0e\n\x06offset\x18\x08 \x01(\x02\x12\x12\n\nstep_width\x18\t \x01(\x02\x12\x13\n\x0bstep_height\x18\n \x01(\x02\x12\x0c\n\x04\x66lip\x18\x0b \x01(\x08\x12\x0c\n\x04\x63lip\x18\x0c \x01(\x08\"\xad\x03\n\x08SSDParam\x12\x36\n\x0fprior_box_param\x18\x01 \x03(\x0b\x32\x1d.vitis.ai.proto.PriorBoxParam\x12\x13\n\x0bnum_classes\x18\x02 \x01(\x05\x12\x15\n\rnms_threshold\x18\x03 \x01(\x02\x12\x16\n\x0e\x63onf_threshold\x18\x04 \x03(\x02\x12\x12\n\nkeep_top_k\x18\x05 \x01(\x05\x12\r\n\x05top_k\x18\x06 \x01(\x05\x12\x11\n\tis_mlperf\x18\x07 \x01(\x08\x12\x18\n\x10\x62\x62ox_layer_index\x18\x08 \x03(\x05\x12\x38\n\x0boutput_info\x18\t \x03(\x0b\x32#.vitis.ai.proto.SSDParam.OutputInfo\x1a\\\n\nOutputInfo\x12\x31\n\x04type\x18\x01 \x01(\x0e\x32#.vitis.ai.proto.SSDParam.OutputType\x12\r\n\x05order\x18\x02 \x01(\x05\x12\x0c\n\x04name\x18\x03 \x01(\t\"=\n\nOutputType\x12\x0b\n\x07USELESS\x10\x00\x12\x08\n\x04\x43ONF\x10\x01\x12\x08\n\x04\x42\x42OX\x10\x02\x12\x0e\n\nFUNCTIONAL\x10\x03\"e\n\x0eRetinaNetParam\x12\x15\n\rbox_threshold\x18\x01 \x01(\x02\x12\x15\n\rnms_threshold\x18\x02 \x01(\x02\x12\x16\n\x0e\x63onf_threshold\x18\x03 \x01(\x02\x12\r\n\x05top_k\x18\x04 \x01(\x05\"t\n\x0c\x46\x61irMotParam\x12\x15\n\rnms_threshold\x18\x01 \x01(\x02\x12\x16\n\x0e\x63onf_threshold\x18\x02 \x01(\x02\x12\x12\n\nkeep_top_k\x18\x03 \x01(\x05\x12\r\n\x05top_k\x18\x04 \x01(\x05\x12\x12\n\nlayer_name\x18\x05 \x03(\t\"Q\n\x0cMovenetParam\x12\x16\n\x0e\x63onf_threshold\x18\x01 \x01(\x02\x12\x12\n\nlayer_name\x18\x02 \x03(\t\x12\x15\n\rcenter_weight\x18\x03 \x03(\x02\"\xc3\x01\n\x0eRefineDetParam\x12\x36\n\x0fprior_box_param\x18\x01 \x03(\x0b\x32\x1d.vitis.ai.proto.PriorBoxParam\x12\x13\n\x0bnum_classes\x18\x02 \x01(\x05\x12\x15\n\rnms_threshold\x18\x03 \x01(\x02\x12\x16\n\x0e\x63onf_threshold\x18\x04 \x03(\x02\x12\x12\n\nkeep_top_k\x18\x05 \x01(\x05\x12\r\n\x05top_k\x18\x06 \x01(\x05\x12\x12\n\nlayer_name\x18\x07 \x03(\t\"\xf0\x01\n\x10TFRefineDetParam\x12=\n\x0fprior_box_param\x18\x01 \x01(\x0b\x32$.vitis.ai.proto.MedicalPriorBoxParam\x12\x13\n\x0bnum_classes\x18\x02 \x01(\x05\x12\x15\n\rnms_threshold\x18\x03 \x01(\x02\x12\x16\n\x0e\x63onf_threshold\x18\x04 \x01(\x02\x12\x12\n\nkeep_top_k\x18\x05 \x01(\x05\x12\r\n\x05top_k\x18\x06 \x01(\x05\x12\x12\n\nlayer_name\x18\x07 \x03(\t\x12\x10\n\x08scale_xy\x18\x08 \x01(\x02\x12\x10\n\x08scale_wh\x18\t \x01(\x02\"\xee\x03\n\x15MedicalRefineDetParam\x12\x13\n\x0bnum_classes\x18\x01 \x01(\x05\x12\x10\n\x08scale_xy\x18\x02 \x01(\x02\x12\x10\n\x08scale_wh\x18\x03 \x01(\x02\x12\x15\n\rnms_threshold\x18\x04 \x01(\x02\x12\x16\n\x0e\x63onf_threshold\x18\x05 \x01(\x02\x12\x12\n\nkeep_top_k\x18\x06 \x01(\x05\x12\r\n\x05top_k\x18\x07 \x01(\x05\x12\x45\n\x17medical_prior_box_param\x18\x08 \x01(\x0b\x32$.vitis.ai.proto.MedicalPriorBoxParam\x12\x45\n\x0boutput_info\x18\t \x03(\x0b\x32\x30.vitis.ai.proto.MedicalRefineDetParam.OutputInfo\x1ai\n\nOutputInfo\x12>\n\x04type\x18\x01 \x01(\x0e\x32\x30.vitis.ai.proto.MedicalRefineDetParam.OutputType\x12\r\n\x05order\x18\x02 \x01(\x05\x12\x0c\n\x04name\x18\x03 \x01(\t\"Q\n\nOutputType\x12\x0b\n\x07USELESS\x10\x00\x12\x0c\n\x08\x41RM_CONF\x10\x01\x12\x0c\n\x08\x41RM_BBOX\x10\x02\x12\x0c\n\x08ODM_CONF\x10\x03\x12\x0c\n\x08ODM_BBOX\x10\x04\"B\n\tReidParam\x12\x10\n\x08\x62n_means\x18\x01 \x03(\x02\x12\x12\n\nbn_weights\x18\x02 \x03(\x02\x12\x0f\n\x07\x62n_vars\x18\x03 \x03(\x02\"#\n\rOpenPoseParam\x12\x12\n\nlayer_name\x18\x01 \x03(\t\"\x90\x04\n\x13\x43lassificationParam\x12\r\n\x05top_k\x18\x01 \x01(\x05\x12\x15\n\rtest_accuracy\x18\x02 \x01(\x08\x12K\n\x0fpreprocess_type\x18\x03 \x01(\x0e\x32\x32.vitis.ai.proto.ClassificationParam.PreprocessType\x12\x12\n\nlayer_name\x18\x04 \x01(\t\x12\x12\n\nlabel_type\x18\x05 \x01(\t\x12H\n\x0e\x61vg_pool_param\x18\x06 \x01(\x0b\x32\x30.vitis.ai.proto.ClassificationParam.AvgPoolParam\x1a\x1d\n\x0c\x41vgPoolParam\x12\r\n\x05scale\x18\x01 \x01(\x05\"\xf4\x01\n\x0ePreprocessType\x12\x10\n\x0cUNKNOWN_TYPE\x10\x00\x12\t\n\x05\x43\x41\x46\x46\x45\x10\x01\x12\x12\n\x0eVGG_PREPROCESS\x10\x02\x12\x18\n\x14INCEPTION_PREPROCESS\x10\x03\x12\x10\n\x0cINCEPTION_PT\x10\x04\x12\x10\n\x0cRESNET50_TF2\x10\x05\x12\x1b\n\x17\x45\x46\x46ICIENTNET_PREPROCESS\x10\x06\x12\x13\n\x0fOFA_RESNET50_PT\x10\x07\x12\x17\n\x13OFA_DEPTHWISE_RES50\x10\x08\x12\x12\n\x0eRGB_PREPROCESS\x10\t\x12\x14\n\x10OFA_RESNET50_PT2\x10\n\"U\n\rDenseBoxParam\x12\x16\n\x0enum_of_classes\x18\x01 \x01(\x05\x12\x15\n\rnms_threshold\x18\x02 \x01(\x02\x12\x15\n\rdet_threshold\x18\x03 \x01(\x02\"\xe8\x02\n\x0eMultiTaskParam\x12\x0f\n\x07th_conf\x18\x01 \x03(\x02\x12 \n\x18num_of_detection_classes\x18\x02 \x01(\x05\x12#\n\x1bnum_of_segmentation_classes\x18\x03 \x01(\x05\x12\x12\n\nkeep_top_k\x18\x04 \x01(\x05\x12\r\n\x05top_k\x18\x05 \x01(\x05\x12\x15\n\rnms_threshold\x18\x06 \x01(\x02\x12\x36\n\x0fprior_box_param\x18\x07 \x03(\x0b\x32\x1d.vitis.ai.proto.PriorBoxParam\x12\x10\n\x08loc_name\x18\x08 \x03(\t\x12\x11\n\tconf_name\x18\t \x03(\t\x12\x17\n\x0f\x63\x65nterness_name\x18\n \x03(\t\x12\x10\n\x08seg_name\x18\x0b \x01(\t\x12\x15\n\rdrivable_name\x18\x0c \x01(\t\x12\x12\n\ndepth_name\x18\r \x01(\t\x12\x11\n\tlane_name\x18\x0e \x01(\t\"\xed\x02\n\rRoadlineParam\x12\r\n\x05ratio\x18\x01 \x01(\x05\x12\x11\n\tipm_width\x18\x02 \x01(\x02\x12\x12\n\nipm_height\x18\x03 \x01(\x02\x12\x10\n\x08ipm_left\x18\x04 \x01(\x02\x12\x11\n\tipm_right\x18\x05 \x01(\x02\x12\x0f\n\x07ipm_top\x18\x06 \x01(\x02\x12\x12\n\nipm_bottom\x18\x07 \x01(\x02\x12\x19\n\x11ipm_interpolation\x18\x08 \x01(\x02\x12\x16\n\x0eipm_vp_portion\x18\t \x01(\x02\x12\x16\n\x0e\x66ocal_length_x\x18\n \x01(\x02\x12\x16\n\x0e\x66ocal_length_y\x18\x0b \x01(\x02\x12\x18\n\x10optical_center_x\x18\x0c \x01(\x02\x12\x18\n\x10optical_center_y\x18\r \x01(\x02\x12\x15\n\rcamera_height\x18\x0e \x01(\x02\x12\r\n\x05pitch\x18\x0f \x01(\x02\x12\x0b\n\x03yaw\x18\x10 \x01(\x02\x12\x12\n\nlayer_name\x18\x11 \x03(\t\"\x1a\n\tRcanParam\x12\r\n\x05scale\x18\x01 \x01(\x02\"\x9f\x01\n\x13RoadlineDeephiParam\x12\x10\n\x08resize_w\x18\x01 \x01(\x05\x12\x10\n\x08resize_h\x18\x02 \x01(\x05\x12\x0e\n\x06\x63rop_x\x18\x03 \x01(\x05\x12\x0e\n\x06\x63rop_y\x18\x04 \x01(\x05\x12\x0e\n\x06\x63rop_w\x18\x05 \x01(\x05\x12\x0e\n\x06\x63rop_h\x18\x06 \x01(\x05\x12\x16\n\x0e\x61rea_threshold\x18\x07 \x01(\x05\x12\x0c\n\x04\x65pow\x18\x08 \x01(\x05\"}\n\x11SegmentationParam\x12\x0e\n\x06\x63olor1\x18\x01 \x01(\t\x12\x0e\n\x06\x63olor2\x18\x02 \x01(\t\x12\x0e\n\x06\x63olor3\x18\x03 \x01(\t\x12\x1c\n\x14specify_output_layer\x18\x04 \x01(\x08\x12\x1a\n\x12output_tensor_name\x18\x05 \x01(\t\"J\n\rPlatenumParam\x12\r\n\x05\x66irst\x18\x01 \x03(\x05\x12\x0e\n\x06second\x18\x02 \x03(\x05\x12\x1a\n\x12output_tensor_name\x18\x03 \x03(\t\"\xc0\x01\n\tSoloParam\x12\r\n\x05\x66irst\x18\x01 \x03(\x05\x12\x0e\n\x06second\x18\x02 \x03(\x05\x12\x19\n\x11input_tensor_name\x18\x03 \x03(\t\x12\x1a\n\x12output_tensor_name\x18\x04 \x03(\t\x12\x11\n\tscore_thr\x18\x05 \x01(\x02\x12\x10\n\x08mask_thr\x18\x06 \x01(\x02\x12\x0f\n\x07nms_pre\x18\x07 \x01(\r\x12\x12\n\nupdate_thr\x18\x08 \x01(\x02\x12\x13\n\x0bmax_per_img\x18\t \x01(\r\"@\n\tLaneParam\x12\r\n\x05roi_x\x18\x01 \x01(\x05\x12\r\n\x05roi_y\x18\x02 \x01(\x05\x12\x15\n\rexpect_height\x18\x03 \x01(\x05\"\xad\x01\n\x0c\x42linkerParam\x12\x36\n\x0fprior_box_param\x18\x01 \x03(\x0b\x32\x1d.vitis.ai.proto.PriorBoxParam\x12\x13\n\x0bnum_classes\x18\x02 \x01(\x05\x12\x15\n\rnms_threshold\x18\x03 \x01(\x02\x12\x16\n\x0e\x63onf_threshold\x18\x04 \x03(\x02\x12\x12\n\nkeep_top_k\x18\x05 \x01(\x05\x12\r\n\x05top_k\x18\x06 \x01(\x05\"\xac\x01\n\x0bSegdetParam\x12\x36\n\x0fprior_box_param\x18\x01 \x03(\x0b\x32\x1d.vitis.ai.proto.PriorBoxParam\x12\x13\n\x0bnum_classes\x18\x02 \x01(\x05\x12\x15\n\rnms_threshold\x18\x03 \x01(\x02\x12\x16\n\x0e\x63onf_threshold\x18\x04 \x03(\x02\x12\x12\n\nkeep_top_k\x18\x05 \x01(\x05\x12\r\n\x05top_k\x18\x06 \x01(\x05\"\x81\x01\n\x13\x46\x61\x63\x65Quality5ptParam\x12\x1a\n\x12use_day_night_mode\x18\x01 \x01(\x08\x12\x18\n\x10original_quality\x18\x02 \x01(\x08\x12\x1a\n\x12quality_layer_name\x18\x03 \x01(\t\x12\x18\n\x10point_layer_name\x18\x04 \x01(\t\"\x82\x05\n\x0fRetinaFaceParam\x12?\n\x0boutput_info\x18\x01 \x03(\x0b\x32*.vitis.ai.proto.RetinaFaceParam.OutputInfo\x12\x15\n\rnms_threshold\x18\x02 \x01(\x02\x12\x15\n\rdet_threshold\x18\x03 \x01(\x02\x12\x41\n\x0c\x61nchor_param\x18\x04 \x03(\x0b\x32+.vitis.ai.proto.RetinaFaceParam.AnchorParam\x1a[\n\x10OutputAnchorInfo\x12\x38\n\x04type\x18\x01 \x01(\x0e\x32*.vitis.ai.proto.RetinaFaceParam.AnchorType\x12\r\n\x05index\x18\x02 \x01(\x05\x1a\xb2\x01\n\nOutputInfo\x12\x38\n\x04type\x18\x01 \x01(\x0e\x32*.vitis.ai.proto.RetinaFaceParam.OutputType\x12\x0e\n\x06stride\x18\x02 \x01(\x05\x12\x0c\n\x04name\x18\x03 \x01(\t\x12L\n\x12output_anchor_info\x18\x04 \x01(\x0b\x32\x30.vitis.ai.proto.RetinaFaceParam.OutputAnchorInfo\x1aP\n\x0b\x41nchorParam\x12\x0e\n\x06stride\x18\x01 \x01(\x05\x12\x11\n\tbase_size\x18\x02 \x01(\x05\x12\x0e\n\x06ratios\x18\x03 \x03(\x02\x12\x0e\n\x06scales\x18\x04 \x03(\x05\";\n\nOutputType\x12\x0b\n\x07USELESS\x10\x00\x12\x08\n\x04\x43ONF\x10\x01\x12\x08\n\x04\x42\x42OX\x10\x02\x12\x0c\n\x08LANDMARK\x10\x03\"\x1c\n\nAnchorType\x12\x06\n\x02\x42G\x10\x00\x12\x06\n\x02\x46G\x10\x01\";\n\x0e\x44puKernelParam\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04mean\x18\x02 \x03(\x02\x12\r\n\x05scale\x18\x03 \x03(\x02\"\xf1\x01\n\nTfssdParam\x12:\n\x0boutput_info\x18\x01 \x03(\x0b\x32%.vitis.ai.proto.TfssdParam.OutputInfo\x12\x18\n\x10\x66\x65\x61ture_map_list\x18\x02 \x03(\x05\x1a^\n\nOutputInfo\x12\x33\n\x04type\x18\x01 \x01(\x0e\x32%.vitis.ai.proto.TfssdParam.OutputType\x12\r\n\x05order\x18\x02 \x01(\x05\x12\x0c\n\x04name\x18\x03 \x01(\t\"-\n\nOutputType\x12\x0b\n\x07USELESS\x10\x00\x12\x08\n\x04\x43ONF\x10\x01\x12\x08\n\x04\x42\x42OX\x10\x02\"\xa7\x05\n\x14PointPillarsNusParam\x12\x19\n\x11point_cloud_range\x18\x01 \x03(\x02\x12\x13\n\x0bin_channels\x18\x02 \x01(\x05\x12\x16\n\x0emax_points_num\x18\x03 \x01(\x05\x12\x16\n\x0emax_voxels_num\x18\x04 \x01(\x05\x12\x14\n\x0c\x66\x65\x61tmap_size\x18\x05 \x03(\x05\x12\x44\n\x0b\x61nchor_info\x18\x06 \x01(\x0b\x32/.vitis.ai.proto.PointPillarsNusParam.AnchorInfo\x12\x13\n\x0bnum_classes\x18\x07 \x01(\x05\x12\x0f\n\x07nms_pre\x18\x08 \x01(\x02\x12\x12\n\nnms_thresh\x18\t \x01(\x02\x12\x0f\n\x07max_num\x18\n \x01(\x02\x12\x14\n\x0cscore_thresh\x18\x0b \x01(\x02\x12\x18\n\x10score_layer_name\x18\x0c \x01(\t\x12\x17\n\x0f\x62\x62ox_layer_name\x18\r \x01(\t\x12\x16\n\x0e\x64ir_layer_name\x18\x0e \x01(\t\x1a\xa6\x02\n\nAnchorInfo\x12\x46\n\x06ranges\x18\x01 \x03(\x0b\x32\x36.vitis.ai.proto.PointPillarsNusParam.AnchorInfo.Ranges\x12\x44\n\x05sizes\x18\x02 \x03(\x0b\x32\x35.vitis.ai.proto.PointPillarsNusParam.AnchorInfo.Sizes\x12\x11\n\trotations\x18\x03 \x03(\x02\x12\x14\n\x0c\x63ustom_value\x18\x04 \x03(\x02\x12\x14\n\x0c\x61lign_corner\x18\x05 \x01(\x08\x12\r\n\x05scale\x18\x06 \x01(\x02\x1a\x1e\n\x06Ranges\x12\x14\n\x0csingle_range\x18\x01 \x03(\x02\x1a\x1c\n\x05Sizes\x12\x13\n\x0bsingle_size\x18\x01 \x03(\x02\")\n\x13Segmentation3DParam\x12\x12\n\nenable_knn\x18\x01 \x01(\x08\"z\n\x0bYoloV5Param\x12/\n\nyolo_param\x18\x01 \x01(\x0b\x32\x1b.vitis.ai.proto.YoloV3Param\x12\x0e\n\x06stride\x18\x02 \x03(\x02\x12\x15\n\rmax_boxes_num\x18\x03 \x01(\x02\x12\x13\n\x0bmax_nms_num\x18\x04 \x01(\x02\"\x88\x02\n\x0bYoloV6Param\x12\x13\n\x0bnum_classes\x18\x01 \x01(\x05\x12\x11\n\tanchorCnt\x18\x02 \x01(\x05\x12\x16\n\x0e\x63onf_threshold\x18\x03 \x01(\x02\x12\x15\n\rnms_threshold\x18\x04 \x01(\x02\x12\x0e\n\x06\x62iases\x18\x05 \x03(\x02\x12\x10\n\x08test_mAP\x18\x06 \x01(\x08\x12\x17\n\x0f\x62\x62ox_layer_name\x18\x07 \x03(\t\x12\x16\n\x0e\x63ls_layer_name\x18\x08 \x03(\t\x12\x0e\n\x06stride\x18\t \x03(\x02\x12\x15\n\rmax_boxes_num\x18\n \x01(\x02\x12\x13\n\x0bmax_nms_num\x18\x0b \x01(\x02\x12\x13\n\x0bwithout_dfl\x18\x0c \x01(\x08\"y\n\x0bYoloV7Param\x12\x13\n\x0b\x63onf_thresh\x18\x01 \x01(\x02\x12\x15\n\rnms_threshold\x18\x02 \x01(\x02\x12\x13\n\x0bnum_classes\x18\x03 \x01(\x05\x12\x19\n\x11\x64\x65tect_layer_name\x18\x04 \x03(\t\x12\x0e\n\x06\x62iases\x18\x05 \x03(\x02\"\xa5\x01\n\x0bYoloV8Param\x12\x13\n\x0b\x63onf_thresh\x18\x01 \x01(\x02\x12\x15\n\rmax_boxes_num\x18\x02 \x01(\x02\x12\x15\n\rnms_threshold\x18\x03 \x01(\x02\x12\x13\n\x0bmax_nms_num\x18\x04 \x01(\x02\x12\x13\n\x0bnum_classes\x18\x05 \x01(\x05\x12\x19\n\x11\x64\x65tect_layer_name\x18\x06 \x03(\t\x12\x0e\n\x06stride\x18\x07 \x03(\x02\"\xe0\x01\n\tFcosParam\x12\x16\n\x0epre_nms_thresh\x18\x01 \x01(\x02\x12\x15\n\rpre_nms_top_n\x18\x02 \x01(\x05\x12\x15\n\rnms_threshold\x18\x03 \x01(\x02\x12\x1a\n\x12\x66pn_post_nms_top_n\x18\x04 \x01(\x02\x12\x13\n\x0bnum_classes\x18\x05 \x01(\x05\x12\x19\n\x11\x63\x65nter_layer_name\x18\x06 \x03(\t\x12\x16\n\x0e\x63ls_layer_name\x18\x07 \x03(\t\x12\x19\n\x11offset_layer_name\x18\x08 \x03(\t\x12\x0e\n\x06stride\x18\t \x03(\x02\"}\n\x08\x42TSParam\x12\x12\n\nmul_factor\x18\x01 \x01(\x02\x12\x1b\n\x13\x64\x65pth_scaled_factor\x18\x02 \x01(\x02\x12!\n\x19hard_sigmoid_output_scale\x18\x03 \x01(\x02\x12\x1d\n\x15multiple_output_scale\x18\x04 \x01(\x02\"\x9f\x01\n\x15PointPillarsBaseParam\x12\x19\n\x11point_cloud_range\x18\x01 \x03(\x02\x12\x12\n\nvoxel_size\x18\x02 \x03(\x02\x12\x12\n\npoints_dim\x18\x03 \x01(\x05\x12\x16\n\x0emax_points_num\x18\x04 \x01(\x05\x12\x16\n\x0emax_voxels_num\x18\x05 \x01(\x05\x12\x13\n\x0bin_channels\x18\x06 \x01(\x05\"M\n\x16PointPillarsKittiParam\x12\x33\n\x04\x62\x61se\x18\x01 \x01(\x0b\x32%.vitis.ai.proto.PointPillarsBaseParam\"\xea\x04\n\x13X_Autonomous3DParam\x12\x19\n\x11point_cloud_range\x18\x01 \x03(\x02\x12\x12\n\nvoxel_size\x18\x02 \x03(\x02\x12\x12\n\npoints_dim\x18\x03 \x01(\x05\x12\x16\n\x0emax_points_num\x18\x04 \x01(\x05\x12\x16\n\x0emax_voxels_num\x18\x05 \x01(\x05\x12\x13\n\x0bin_channels\x18\x06 \x01(\x05\x12P\n\x0coutput_names\x18\x07 \x01(\x0b\x32:.vitis.ai.proto.X_Autonomous3DParam.CenterPointOutputLayer\x12\x13\n\x0bnum_classes\x18\x08 \x01(\x05\x12\x17\n\x0fout_size_facter\x18\t \x01(\x02\x12\x19\n\x11iou_quality_alpha\x18\n \x03(\x02\x12\x1e\n\x16post_point_cloud_range\x18\x0b \x03(\x02\x12\x15\n\rbbox_pre_size\x18\x0c \x01(\x05\x12\x15\n\rbbox_max_size\x18\r \x01(\x05\x12\x12\n\nnms_thresh\x18\x0e \x03(\x02\x12\x14\n\x0cscore_thresh\x18\x0f \x01(\x02\x1a\xb7\x01\n\x16\x43\x65nterPointOutputLayer\x12\x16\n\x0ereg_layer_name\x18\x01 \x01(\t\x12\x19\n\x11height_layer_name\x18\x02 \x01(\t\x12\x16\n\x0e\x64im_layer_name\x18\x03 \x01(\t\x12\x16\n\x0erot_layer_name\x18\x04 \x01(\t\x12\x1a\n\x12heatmap_layer_name\x18\x05 \x01(\t\x12\x1e\n\x16iou_quality_layer_name\x18\x06 \x01(\t\"\xc9\x04\n\x13\x45\x66\x66icientDetD2Param\x12\x43\n\x0b\x61nchor_info\x18\x01 \x01(\x0b\x32..vitis.ai.proto.EfficientDetD2Param.AnchorInfo\x12\x13\n\x0bnum_classes\x18\x02 \x01(\x05\x12\x13\n\x0bpre_nms_num\x18\x03 \x01(\x05\x12\x15\n\rnms_threshold\x18\x04 \x01(\x02\x12\x16\n\x0e\x63onf_threshold\x18\x05 \x01(\x02\x12\x16\n\x0enms_output_num\x18\x06 \x01(\x05\x12\x43\n\x0boutput_info\x18\x07 \x03(\x0b\x32..vitis.ai.proto.EfficientDetD2Param.OutputInfo\x1ag\n\nOutputInfo\x12<\n\x04type\x18\x01 \x01(\x0e\x32..vitis.ai.proto.EfficientDetD2Param.OutputType\x12\r\n\x05level\x18\x02 \x01(\x05\x12\x0c\n\x04name\x18\x03 \x01(\t\x1a\x9e\x01\n\nAnchorInfo\x12\x11\n\tmin_level\x18\x01 \x01(\x05\x12\x11\n\tmax_level\x18\x02 \x01(\x05\x12\x12\n\nnum_scales\x18\x03 \x01(\x05\x12\x15\n\ranchor_scales\x18\x04 \x03(\x02\x12\x14\n\x0c\x61spect_ratio\x18\x05 \x03(\x02\x12\x13\n\x0bimage_width\x18\x06 \x01(\x05\x12\x14\n\x0cimage_height\x18\x07 \x01(\x05\"-\n\nOutputType\x12\x0b\n\x07USELESS\x10\x00\x12\x08\n\x04\x43ONF\x10\x01\x12\x08\n\x04\x42\x42OX\x10\x02\"\xc6\x01\n\rSubgraphParam\x12\x0e\n\x06\x63ycles\x18\x01 \x01(\x05\x12\x14\n\x0csubgraph_idx\x18\x02 \x01(\t\x12\x37\n\x05input\x18\x03 \x03(\x0b\x32(.vitis.ai.proto.SubgraphParam.InputParam\x1aV\n\nInputParam\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1a\n\x12previous_model_idx\x18\x02 \x01(\x05\x12\x1e\n\x16previous_output_tensor\x18\x03 \x01(\t\"\xb2\x19\n\rDpuModelParam\x12\x0c\n\x04name\x18\x01 \x01(\t\x12.\n\x06kernel\x18\x02 \x03(\x0b\x32\x1e.vitis.ai.proto.DpuKernelParam\x12\x18\n\x10use_graph_runner\x18\x1c \x01(\x08\x12;\n\nmodel_type\x18\x03 \x01(\x0e\x32\'.vitis.ai.proto.DpuModelParam.ModelType\x12\x38\n\x10refine_det_param\x18\x04 \x01(\x0b\x32\x1e.vitis.ai.proto.RefineDetParam\x12\x32\n\ryolo_v3_param\x18\x05 \x01(\x0b\x32\x1b.vitis.ai.proto.YoloV3Param\x12+\n\tssd_param\x18\x06 \x01(\x0b\x32\x18.vitis.ai.proto.SSDParam\x12\x41\n\x14\x63lassification_param\x18\x07 \x01(\x0b\x32#.vitis.ai.proto.ClassificationParam\x12\x36\n\x0f\x64\x65nse_box_param\x18\x08 \x01(\x0b\x32\x1d.vitis.ai.proto.DenseBoxParam\x12\x38\n\x10multi_task_param\x18\t \x01(\x0b\x32\x1e.vitis.ai.proto.MultiTaskParam\x12\x35\n\x0eroadline_param\x18\n \x01(\x0b\x32\x1d.vitis.ai.proto.RoadlineParam\x12=\n\x12segmentation_param\x18\x0b \x01(\x0b\x32!.vitis.ai.proto.SegmentationParam\x12-\n\nlane_param\x18\x0c \x01(\x0b\x32\x19.vitis.ai.proto.LaneParam\x12\x33\n\rblinker_param\x18\r \x01(\x0b\x32\x1c.vitis.ai.proto.BlinkerParam\x12\x31\n\x0csegdet_param\x18\x0e \x01(\x0b\x32\x1b.vitis.ai.proto.SegdetParam\x12>\n\x11roadline_dp_param\x18\x0f \x01(\x0b\x32#.vitis.ai.proto.RoadlineDeephiParam\x12\r\n\x05is_tf\x18\x10 \x01(\x08\x12\x42\n\x15\x66\x61\x63\x65_quality5pt_param\x18\x11 \x01(\x0b\x32#.vitis.ai.proto.FaceQuality5ptParam\x12/\n\x0btfssd_param\x18\x12 \x01(\x0b\x32\x1a.vitis.ai.proto.TfssdParam\x12\x36\n\x0fopen_pose_param\x18\x13 \x01(\x0b\x32\x1d.vitis.ai.proto.OpenPoseParam\x12-\n\nreid_param\x18\x14 \x01(\x0b\x32\x19.vitis.ai.proto.ReidParam\x12G\n\x18medical_refine_det_param\x18\x15 \x01(\x0b\x32%.vitis.ai.proto.MedicalRefineDetParam\x12\x35\n\x0eplatenum_param\x18\x16 \x01(\x0b\x32\x1d.vitis.ai.proto.PlatenumParam\x12;\n\norder_type\x18\x17 \x01(\x0e\x32\'.vitis.ai.proto.DpuModelParam.OrderType\x12\x39\n\x10retinaface_param\x18\x18 \x01(\x0b\x32\x1f.vitis.ai.proto.RetinaFaceParam\x12;\n\x11tfrefinedet_param\x18\x19 \x01(\x0b\x32 .vitis.ai.proto.TFRefineDetParam\x12\x44\n\x16pointpillars_nus_param\x18\x1a \x01(\x0b\x32$.vitis.ai.proto.PointPillarsNusParam\x12\x42\n\x15segmentation_3d_param\x18\x1b \x01(\x0b\x32#.vitis.ai.proto.Segmentation3DParam\x12\x32\n\ryolo_v5_param\x18\x1d \x01(\x0b\x32\x1b.vitis.ai.proto.YoloV5Param\x12+\n\tbts_param\x18\x1e \x01(\x0b\x32\x18.vitis.ai.proto.BTSParam\x12\x34\n\x0e\x66\x61ir_mot_param\x18\x1f \x01(\x0b\x32\x1c.vitis.ai.proto.FairMotParam\x12\x41\n\x14x_autonomous3d_param\x18  \x01(\x0b\x32#.vitis.ai.proto.X_Autonomous3DParam\x12H\n\x18pointpillars_kitti_param\x18! \x01(\x0b\x32&.vitis.ai.proto.PointPillarsKittiParam\x12-\n\nsolo_param\x18\" \x01(\x0b\x32\x19.vitis.ai.proto.SoloParam\x12/\n\x08subgraph\x18# \x03(\x0b\x32\x1d.vitis.ai.proto.SubgraphParam\x12H\n\x1bvehicleclassification_param\x18$ \x01(\x0b\x32#.vitis.ai.proto.ClassificationParam\x12\x42\n\x15\x65\x66\x66icientdet_d2_param\x18% \x01(\x0b\x32#.vitis.ai.proto.EfficientDetD2Param\x12\x33\n\rmovenet_param\x18& \x01(\x0b\x32\x1c.vitis.ai.proto.MovenetParam\x12-\n\nrcan_param\x18\' \x01(\x0b\x32\x19.vitis.ai.proto.RcanParam\x12\x32\n\ryolo_v6_param\x18( \x01(\x0b\x32\x1b.vitis.ai.proto.YoloV6Param\x12-\n\nfcos_param\x18) \x01(\x0b\x32\x19.vitis.ai.proto.FcosParam\x12\x32\n\ryolo_v8_param\x18* \x01(\x0b\x32\x1b.vitis.ai.proto.YoloV8Param\x12\x37\n\x0fretinanet_param\x18+ \x01(\x0b\x32\x1e.vitis.ai.proto.RetinaNetParam\x12\x32\n\ryolo_v7_param\x18, \x01(\x0b\x32\x1b.vitis.ai.proto.YoloV7Param\"\x8f\x06\n\tModelType\x12\x10\n\x0cUNKNOWN_TYPE\x10\x00\x12\r\n\tREFINEDET\x10\x01\x12\x07\n\x03SSD\x10\x02\x12\n\n\x06YOLOv3\x10\x03\x12\x12\n\x0e\x43LASSIFICATION\x10\x04\x12\r\n\tDENSE_BOX\x10\x05\x12\x0e\n\nMULTI_TASK\x10\x06\x12\x0c\n\x08OPENPOSE\x10\x07\x12\x0c\n\x08ROADLINE\x10\x08\x12\x10\n\x0cSEGMENTATION\x10\t\x12\x0e\n\nPOSEDETECT\x10\n\x12\x08\n\x04LANE\x10\x0b\x12\x0b\n\x07\x42LINKER\x10\x0c\x12\n\n\x06SEGDET\x10\r\x12\x13\n\x0fROADLINE_DEEPHI\x10\x0e\x12\x12\n\x0e\x46\x41\x43\x45QUALITY5PT\x10\x0f\x12\x08\n\x04REID\x10\x10\x12\n\n\x06YOLOv2\x10\x11\x12\t\n\x05TFSSD\x10\x12\x12\x10\n\x0c\x46\x41\x43\x45LANDMARK\x10\x13\x12\x0f\n\x0bPLATEDETECT\x10\x14\x12\x0c\n\x08PLATENUM\x10\x15\x12\x0e\n\nPLATERECOG\x10\x16\x12\x0f\n\x0b\x46\x41\x43\x45\x46\x45\x41TURE\x10\x17\x12\x14\n\x10SEGMENTATION8UC1\x10\x18\x12\x14\n\x10SEGMENTATION8UC3\x10\x19\x12\x17\n\x13MEDICALSEGMENTATION\x10\x1a\x12\x11\n\rMULTITASK8UC1\x10\x1b\x12\x11\n\rMULTITASK8UC3\x10\x1c\x12\x14\n\x10MEDICALREFINEDET\x10\x1d\x12\x13\n\x0f\x46\x41\x43\x45\x44\x45TECTRECOG\x10\x1e\x12\r\n\tHOURGLASS\x10\x1f\x12\x0e\n\nRETINAFACE\x10 \x12\x0f\n\x0bTFREFINEDET\x10!\x12\x14\n\x10POINTPILLARS_NUS\x10\"\x12\n\n\x06YOLOv5\x10#\x12\x0c\n\x08\x42TSParam\x10$\x12\x0b\n\x07\x46\x41IRMOT\x10%\x12\x12\n\x0eX_AUTONOMOUS3D\x10&\x12\x10\n\x0cPOINTPILLARS\x10\'\x12\x08\n\x04SOLO\x10(\x12\x19\n\x15VEHICLECLASSIFICATION\x10)\x12\x13\n\x0f\x45\x46\x46ICIENTDET_D2\x10*\x12\x0b\n\x07MOVENET\x10+\x12\n\n\x06YOLOv6\x10,\x12\x08\n\x04\x46\x43OS\x10-\x12\n\n\x06YOLOv8\x10.\x12\n\n\x06YOLOv7\x10/\"9\n\tOrderType\x12\x10\n\x0cUNKNOWNORDER\x10\x00\x12\x07\n\x03\x42GR\x10\x01\x12\x07\n\x03RGB\x10\x02\x12\x08\n\x04GREY\x10\x03\"A\n\x11\x44puModelParamList\x12,\n\x05model\x18\x01 \x03(\x0b\x32\x1d.vitis.ai.proto.DpuModelParam\"8\n\tAttribute\x12\r\n\x05label\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\r\x12\r\n\x05score\x18\x03 \x01(\x02\"\x1d\n\x05Point\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\"%\n\x04Size\x12\r\n\x05width\x18\x01 \x01(\x02\x12\x0e\n\x06height\x18\x02 \x01(\x02\"\x8e\x01\n\x0b\x42oundingBox\x12\x32\n\x05label\x18\x01 \x01(\x0b\x32#.vitis.ai.proto.ClassificationLabel\x12\'\n\x08top_left\x18\x03 \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12\"\n\x04size\x18\x04 \x01(\x0b\x32\x14.vitis.ai.proto.Size\"?\n\x0f\x41ttributeResult\x12,\n\tattribute\x18\x01 \x03(\x0b\x32\x19.vitis.ai.proto.Attribute\"b\n\x0eLandmarkResult\x12\r\n\x05score\x18\x01 \x01(\x02\x12$\n\x05point\x18\x02 \x03(\x0b\x32\x15.vitis.ai.proto.Point\x12\x0e\n\x06gender\x18\x03 \x01(\r\x12\x0b\n\x03\x61ge\x18\x04 \x01(\r\"A\n\x0c\x44\x65tectResult\x12\x31\n\x0c\x62ounding_box\x18\x01 \x03(\x0b\x32\x1b.vitis.ai.proto.BoundingBox\"C\n\rLineAttribute\x12\x0c\n\x04type\x18\x01 \x01(\x05\x12$\n\x05point\x18\x02 \x03(\x0b\x32\x15.vitis.ai.proto.Point\"G\n\x0eRoadlineResult\x12\x35\n\x0eline_attribute\x18\x01 \x03(\x0b\x32\x1d.vitis.ai.proto.LineAttribute\"%\n\x05Score\x12\r\n\x05index\x18\x01 \x01(\x05\x12\r\n\x05score\x18\x02 \x01(\x02\"A\n\x13\x43lassificationLabel\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05index\x18\x02 \x01(\r\x12\r\n\x05score\x18\x03 \x01(\x02\"I\n\x14\x43lassificationResult\x12\x31\n\x04topk\x18\x01 \x03(\x0b\x32#.vitis.ai.proto.ClassificationLabel\":\n\x12\x46\x61\x63\x65LandmarkResult\x12$\n\x05point\x18\x01 \x03(\x0b\x32\x15.vitis.ai.proto.Point\"F\n\x11PlateDetectResult\x12\x31\n\x0c\x62ounding_box\x18\x01 \x01(\x0b\x32\x1b.vitis.ai.proto.BoundingBox\">\n\x11PlateNumberResult\x12\x14\n\x0cplate_number\x18\x01 \x01(\t\x12\x13\n\x0bplate_color\x18\x02 \x01(\t\"p\n\x10PlateRecogResult\x12\x31\n\x0c\x62ounding_box\x18\x01 \x01(\x0b\x32\x1b.vitis.ai.proto.BoundingBox\x12\x14\n\x0cplate_number\x18\x02 \x01(\t\x12\x13\n\x0bplate_color\x18\x03 \x01(\t\"\xe6\x04\n\x10PoseDetectResult\x12-\n\x0eright_shoulder\x18\x01 \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12*\n\x0bright_elbow\x18\x02 \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12*\n\x0bright_wrist\x18\x03 \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12,\n\rleft_shoulder\x18\x04 \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12)\n\nleft_elbow\x18\x05 \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12)\n\nleft_wrist\x18\x06 \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12(\n\tright_hip\x18\x07 \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12)\n\nright_knee\x18\x08 \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12*\n\x0bright_ankle\x18\t \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12\'\n\x08left_hip\x18\n \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12(\n\tleft_knee\x18\x0b \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12)\n\nleft_ankle\x18\x0c \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12#\n\x04head\x18\r \x01(\x0b\x32\x15.vitis.ai.proto.Point\x12#\n\x04neck\x18\x0e \x01(\x0b\x32\x15.vitis.ai.proto.Point\"F\n\x11\x46\x61\x63\x65\x46\x65\x61tureResult\x12\x11\n\tfloat_vec\x18\x01 \x03(\x02\x12\x0f\n\x07\x66ix_vec\x18\x02 \x01(\x0c\x12\r\n\x05scale\x18\x03 \x01(\x02\"~\n\x15\x46\x61\x63\x65\x44\x65tectRecogResult\x12\x31\n\x0c\x62ounding_box\x18\x01 \x03(\x0b\x32\x1b.vitis.ai.proto.BoundingBox\x12\x32\n\x07\x66\x65\x61ture\x18\x02 \x03(\x0b\x32!.vitis.ai.proto.FaceFeatureResult\"!\n\x11SegmetationResult\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\r\"S\n\x18MedicalSegmetationResult\x12\x37\n\x0csegmentation\x18\x01 \x03(\x0b\x32!.vitis.ai.proto.SegmetationResult\"q\n\rVehicleResult\x12\r\n\x05label\x18\x01 \x01(\x05\x12\r\n\x05score\x18\x02 \x01(\x02\x12\t\n\x01x\x18\x03 \x01(\x02\x12\t\n\x01y\x18\x04 \x01(\x02\x12\r\n\x05width\x18\x05 \x01(\x02\x12\x0e\n\x06height\x18\x06 \x01(\x02\x12\r\n\x05\x61ngle\x18\x07 \x01(\x02\"\x81\x01\n\x0fMultiTaskResult\x12\x37\n\x0csegmentation\x18\x01 \x01(\x0b\x32!.vitis.ai.proto.SegmetationResult\x12\x35\n\x0evehicle_reuslt\x18\x02 \x03(\x0b\x32\x1d.vitis.ai.proto.VehicleResult\"\x1a\n\nReidResult\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\r\"\xb5\t\n\x0e\x44puModelResult\x12\x39\n\x0flandmark_result\x18\x01 \x01(\x0b\x32\x1e.vitis.ai.proto.LandmarkResultH\x00\x12\x35\n\rdetect_result\x18\x02 \x01(\x0b\x32\x1c.vitis.ai.proto.DetectResultH\x00\x12;\n\x10\x61ttribute_result\x18\x03 \x01(\x0b\x32\x1f.vitis.ai.proto.AttributeResultH\x00\x12\x39\n\x11refine_det_result\x18\x04 \x01(\x0b\x32\x1c.vitis.ai.proto.DetectResultH\x00\x12\x45\n\x15\x63lassification_result\x18\x05 \x01(\x0b\x32$.vitis.ai.proto.ClassificationResultH\x00\x12\x39\n\x0froadline_result\x18\x06 \x01(\x0b\x32\x1e.vitis.ai.proto.RoadlineResultH\x00\x12\x41\n\x13\x66\x61\x63\x65landmark_result\x18\x07 \x01(\x0b\x32\".vitis.ai.proto.FaceLandmarkResultH\x00\x12@\n\x13plate_detect_result\x18\x08 \x01(\x0b\x32!.vitis.ai.proto.PlateDetectResultH\x00\x12@\n\x13plate_number_result\x18\t \x01(\x0b\x32!.vitis.ai.proto.PlateNumberResultH\x00\x12>\n\x12plate_recog_result\x18\n \x01(\x0b\x32 .vitis.ai.proto.PlateRecogResultH\x00\x12>\n\x12pose_detect_result\x18\x0b \x01(\x0b\x32 .vitis.ai.proto.PoseDetectResultH\x00\x12@\n\x13\x66\x61\x63\x65_feature_result\x18\x0c \x01(\x0b\x32!.vitis.ai.proto.FaceFeatureResultH\x00\x12@\n\x13segmentation_result\x18\r \x01(\x0b\x32!.vitis.ai.proto.SegmetationResultH\x00\x12O\n\x1bmedical_segmentation_result\x18\x0e \x01(\x0b\x32(.vitis.ai.proto.MedicalSegmetationResultH\x00\x12;\n\x10multitask_result\x18\x0f \x01(\x0b\x32\x1f.vitis.ai.proto.MultiTaskResultH\x00\x12\x31\n\x0breid_result\x18\x10 \x01(\x0b\x32\x1a.vitis.ai.proto.ReidResultH\x00\x12I\n\x18\x66\x61\x63\x65_detect_recog_result\x18\x11 \x01(\x0b\x32%.vitis.ai.proto.FaceDetectRecogResultH\x00\x12L\n\x1cvehicleclassification_result\x18\x12 \x01(\x0b\x32$.vitis.ai.proto.ClassificationResultH\x00\x42\x12\n\x10\x64pu_model_resultb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'vitis.ai.proto.dpu_model_param_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _YOLOV3PARAM._serialized_start=57
  _YOLOV3PARAM._serialized_end=341
  _YOLOV3PARAM_TYPE._serialized_start=279
  _YOLOV3PARAM_TYPE._serialized_end=341
  _MEDICALPRIORBOXPARAM._serialized_start=344
  _MEDICALPRIORBOXPARAM._serialized_end=503
  _PRIORBOXPARAM._serialized_start=506
  _PRIORBOXPARAM._serialized_end=729
  _SSDPARAM._serialized_start=732
  _SSDPARAM._serialized_end=1161
  _SSDPARAM_OUTPUTINFO._serialized_start=1006
  _SSDPARAM_OUTPUTINFO._serialized_end=1098
  _SSDPARAM_OUTPUTTYPE._serialized_start=1100
  _SSDPARAM_OUTPUTTYPE._serialized_end=1161
  _RETINANETPARAM._serialized_start=1163
  _RETINANETPARAM._serialized_end=1264
  _FAIRMOTPARAM._serialized_start=1266
  _FAIRMOTPARAM._serialized_end=1382
  _MOVENETPARAM._serialized_start=1384
  _MOVENETPARAM._serialized_end=1465
  _REFINEDETPARAM._serialized_start=1468
  _REFINEDETPARAM._serialized_end=1663
  _TFREFINEDETPARAM._serialized_start=1666
  _TFREFINEDETPARAM._serialized_end=1906
  _MEDICALREFINEDETPARAM._serialized_start=1909
  _MEDICALREFINEDETPARAM._serialized_end=2403
  _MEDICALREFINEDETPARAM_OUTPUTINFO._serialized_start=2215
  _MEDICALREFINEDETPARAM_OUTPUTINFO._serialized_end=2320
  _MEDICALREFINEDETPARAM_OUTPUTTYPE._serialized_start=2322
  _MEDICALREFINEDETPARAM_OUTPUTTYPE._serialized_end=2403
  _REIDPARAM._serialized_start=2405
  _REIDPARAM._serialized_end=2471
  _OPENPOSEPARAM._serialized_start=2473
  _OPENPOSEPARAM._serialized_end=2508
  _CLASSIFICATIONPARAM._serialized_start=2511
  _CLASSIFICATIONPARAM._serialized_end=3039
  _CLASSIFICATIONPARAM_AVGPOOLPARAM._serialized_start=2763
  _CLASSIFICATIONPARAM_AVGPOOLPARAM._serialized_end=2792
  _CLASSIFICATIONPARAM_PREPROCESSTYPE._serialized_start=2795
  _CLASSIFICATIONPARAM_PREPROCESSTYPE._serialized_end=3039
  _DENSEBOXPARAM._serialized_start=3041
  _DENSEBOXPARAM._serialized_end=3126
  _MULTITASKPARAM._serialized_start=3129
  _MULTITASKPARAM._serialized_end=3489
  _ROADLINEPARAM._serialized_start=3492
  _ROADLINEPARAM._serialized_end=3857
  _RCANPARAM._serialized_start=3859
  _RCANPARAM._serialized_end=3885
  _ROADLINEDEEPHIPARAM._serialized_start=3888
  _ROADLINEDEEPHIPARAM._serialized_end=4047
  _SEGMENTATIONPARAM._serialized_start=4049
  _SEGMENTATIONPARAM._serialized_end=4174
  _PLATENUMPARAM._serialized_start=4176
  _PLATENUMPARAM._serialized_end=4250
  _SOLOPARAM._serialized_start=4253
  _SOLOPARAM._serialized_end=4445
  _LANEPARAM._serialized_start=4447
  _LANEPARAM._serialized_end=4511
  _BLINKERPARAM._serialized_start=4514
  _BLINKERPARAM._serialized_end=4687
  _SEGDETPARAM._serialized_start=4690
  _SEGDETPARAM._serialized_end=4862
  _FACEQUALITY5PTPARAM._serialized_start=4865
  _FACEQUALITY5PTPARAM._serialized_end=4994
  _RETINAFACEPARAM._serialized_start=4997
  _RETINAFACEPARAM._serialized_end=5639
  _RETINAFACEPARAM_OUTPUTANCHORINFO._serialized_start=5194
  _RETINAFACEPARAM_OUTPUTANCHORINFO._serialized_end=5285
  _RETINAFACEPARAM_OUTPUTINFO._serialized_start=5288
  _RETINAFACEPARAM_OUTPUTINFO._serialized_end=5466
  _RETINAFACEPARAM_ANCHORPARAM._serialized_start=5468
  _RETINAFACEPARAM_ANCHORPARAM._serialized_end=5548
  _RETINAFACEPARAM_OUTPUTTYPE._serialized_start=5550
  _RETINAFACEPARAM_OUTPUTTYPE._serialized_end=5609
  _RETINAFACEPARAM_ANCHORTYPE._serialized_start=5611
  _RETINAFACEPARAM_ANCHORTYPE._serialized_end=5639
  _DPUKERNELPARAM._serialized_start=5641
  _DPUKERNELPARAM._serialized_end=5700
  _TFSSDPARAM._serialized_start=5703
  _TFSSDPARAM._serialized_end=5944
  _TFSSDPARAM_OUTPUTINFO._serialized_start=5803
  _TFSSDPARAM_OUTPUTINFO._serialized_end=5897
  _TFSSDPARAM_OUTPUTTYPE._serialized_start=1100
  _TFSSDPARAM_OUTPUTTYPE._serialized_end=1145
  _POINTPILLARSNUSPARAM._serialized_start=5947
  _POINTPILLARSNUSPARAM._serialized_end=6626
  _POINTPILLARSNUSPARAM_ANCHORINFO._serialized_start=6332
  _POINTPILLARSNUSPARAM_ANCHORINFO._serialized_end=6626
  _POINTPILLARSNUSPARAM_ANCHORINFO_RANGES._serialized_start=6566
  _POINTPILLARSNUSPARAM_ANCHORINFO_RANGES._serialized_end=6596
  _POINTPILLARSNUSPARAM_ANCHORINFO_SIZES._serialized_start=6598
  _POINTPILLARSNUSPARAM_ANCHORINFO_SIZES._serialized_end=6626
  _SEGMENTATION3DPARAM._serialized_start=6628
  _SEGMENTATION3DPARAM._serialized_end=6669
  _YOLOV5PARAM._serialized_start=6671
  _YOLOV5PARAM._serialized_end=6793
  _YOLOV6PARAM._serialized_start=6796
  _YOLOV6PARAM._serialized_end=7060
  _YOLOV7PARAM._serialized_start=7062
  _YOLOV7PARAM._serialized_end=7183
  _YOLOV8PARAM._serialized_start=7186
  _YOLOV8PARAM._serialized_end=7351
  _FCOSPARAM._serialized_start=7354
  _FCOSPARAM._serialized_end=7578
  _BTSPARAM._serialized_start=7580
  _BTSPARAM._serialized_end=7705
  _POINTPILLARSBASEPARAM._serialized_start=7708
  _POINTPILLARSBASEPARAM._serialized_end=7867
  _POINTPILLARSKITTIPARAM._serialized_start=7869
  _POINTPILLARSKITTIPARAM._serialized_end=7946
  _X_AUTONOMOUS3DPARAM._serialized_start=7949
  _X_AUTONOMOUS3DPARAM._serialized_end=8567
  _X_AUTONOMOUS3DPARAM_CENTERPOINTOUTPUTLAYER._serialized_start=8384
  _X_AUTONOMOUS3DPARAM_CENTERPOINTOUTPUTLAYER._serialized_end=8567
  _EFFICIENTDETD2PARAM._serialized_start=8570
  _EFFICIENTDETD2PARAM._serialized_end=9155
  _EFFICIENTDETD2PARAM_OUTPUTINFO._serialized_start=8844
  _EFFICIENTDETD2PARAM_OUTPUTINFO._serialized_end=8947
  _EFFICIENTDETD2PARAM_ANCHORINFO._serialized_start=8950
  _EFFICIENTDETD2PARAM_ANCHORINFO._serialized_end=9108
  _EFFICIENTDETD2PARAM_OUTPUTTYPE._serialized_start=1100
  _EFFICIENTDETD2PARAM_OUTPUTTYPE._serialized_end=1145
  _SUBGRAPHPARAM._serialized_start=9158
  _SUBGRAPHPARAM._serialized_end=9356
  _SUBGRAPHPARAM_INPUTPARAM._serialized_start=9270
  _SUBGRAPHPARAM_INPUTPARAM._serialized_end=9356
  _DPUMODELPARAM._serialized_start=9359
  _DPUMODELPARAM._serialized_end=12609
  _DPUMODELPARAM_MODELTYPE._serialized_start=11767
  _DPUMODELPARAM_MODELTYPE._serialized_end=12550
  _DPUMODELPARAM_ORDERTYPE._serialized_start=12552
  _DPUMODELPARAM_ORDERTYPE._serialized_end=12609
  _DPUMODELPARAMLIST._serialized_start=12611
  _DPUMODELPARAMLIST._serialized_end=12676
  _ATTRIBUTE._serialized_start=12678
  _ATTRIBUTE._serialized_end=12734
  _POINT._serialized_start=12736
  _POINT._serialized_end=12765
  _SIZE._serialized_start=12767
  _SIZE._serialized_end=12804
  _BOUNDINGBOX._serialized_start=12807
  _BOUNDINGBOX._serialized_end=12949
  _ATTRIBUTERESULT._serialized_start=12951
  _ATTRIBUTERESULT._serialized_end=13014
  _LANDMARKRESULT._serialized_start=13016
  _LANDMARKRESULT._serialized_end=13114
  _DETECTRESULT._serialized_start=13116
  _DETECTRESULT._serialized_end=13181
  _LINEATTRIBUTE._serialized_start=13183
  _LINEATTRIBUTE._serialized_end=13250
  _ROADLINERESULT._serialized_start=13252
  _ROADLINERESULT._serialized_end=13323
  _SCORE._serialized_start=13325
  _SCORE._serialized_end=13362
  _CLASSIFICATIONLABEL._serialized_start=13364
  _CLASSIFICATIONLABEL._serialized_end=13429
  _CLASSIFICATIONRESULT._serialized_start=13431
  _CLASSIFICATIONRESULT._serialized_end=13504
  _FACELANDMARKRESULT._serialized_start=13506
  _FACELANDMARKRESULT._serialized_end=13564
  _PLATEDETECTRESULT._serialized_start=13566
  _PLATEDETECTRESULT._serialized_end=13636
  _PLATENUMBERRESULT._serialized_start=13638
  _PLATENUMBERRESULT._serialized_end=13700
  _PLATERECOGRESULT._serialized_start=13702
  _PLATERECOGRESULT._serialized_end=13814
  _POSEDETECTRESULT._serialized_start=13817
  _POSEDETECTRESULT._serialized_end=14431
  _FACEFEATURERESULT._serialized_start=14433
  _FACEFEATURERESULT._serialized_end=14503
  _FACEDETECTRECOGRESULT._serialized_start=14505
  _FACEDETECTRECOGRESULT._serialized_end=14631
  _SEGMETATIONRESULT._serialized_start=14633
  _SEGMETATIONRESULT._serialized_end=14666
  _MEDICALSEGMETATIONRESULT._serialized_start=14668
  _MEDICALSEGMETATIONRESULT._serialized_end=14751
  _VEHICLERESULT._serialized_start=14753
  _VEHICLERESULT._serialized_end=14866
  _MULTITASKRESULT._serialized_start=14869
  _MULTITASKRESULT._serialized_end=14998
  _REIDRESULT._serialized_start=15000
  _REIDRESULT._serialized_end=15026
  _DPUMODELRESULT._serialized_start=15029
  _DPUMODELRESULT._serialized_end=16234
# @@protoc_insertion_point(module_scope)
