// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: object_detection/protos/region_similarity_calculator.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_bases.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto;
namespace object_detection {
namespace protos {
class IoaSimilarity;
struct IoaSimilarityDefaultTypeInternal;
extern IoaSimilarityDefaultTypeInternal _IoaSimilarity_default_instance_;
class IouSimilarity;
struct IouSimilarityDefaultTypeInternal;
extern IouSimilarityDefaultTypeInternal _IouSimilarity_default_instance_;
class NegSqDistSimilarity;
struct NegSqDistSimilarityDefaultTypeInternal;
extern NegSqDistSimilarityDefaultTypeInternal _NegSqDistSimilarity_default_instance_;
class RegionSimilarityCalculator;
struct RegionSimilarityCalculatorDefaultTypeInternal;
extern RegionSimilarityCalculatorDefaultTypeInternal _RegionSimilarityCalculator_default_instance_;
class ThresholdedIouSimilarity;
struct ThresholdedIouSimilarityDefaultTypeInternal;
extern ThresholdedIouSimilarityDefaultTypeInternal _ThresholdedIouSimilarity_default_instance_;
}  // namespace protos
}  // namespace object_detection
PROTOBUF_NAMESPACE_OPEN
template<> ::object_detection::protos::IoaSimilarity* Arena::CreateMaybeMessage<::object_detection::protos::IoaSimilarity>(Arena*);
template<> ::object_detection::protos::IouSimilarity* Arena::CreateMaybeMessage<::object_detection::protos::IouSimilarity>(Arena*);
template<> ::object_detection::protos::NegSqDistSimilarity* Arena::CreateMaybeMessage<::object_detection::protos::NegSqDistSimilarity>(Arena*);
template<> ::object_detection::protos::RegionSimilarityCalculator* Arena::CreateMaybeMessage<::object_detection::protos::RegionSimilarityCalculator>(Arena*);
template<> ::object_detection::protos::ThresholdedIouSimilarity* Arena::CreateMaybeMessage<::object_detection::protos::ThresholdedIouSimilarity>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace object_detection {
namespace protos {

// ===================================================================

class RegionSimilarityCalculator final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.RegionSimilarityCalculator) */ {
 public:
  inline RegionSimilarityCalculator() : RegionSimilarityCalculator(nullptr) {}
  ~RegionSimilarityCalculator() override;
  explicit PROTOBUF_CONSTEXPR RegionSimilarityCalculator(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  RegionSimilarityCalculator(const RegionSimilarityCalculator& from);
  RegionSimilarityCalculator(RegionSimilarityCalculator&& from) noexcept
    : RegionSimilarityCalculator() {
    *this = ::std::move(from);
  }

  inline RegionSimilarityCalculator& operator=(const RegionSimilarityCalculator& from) {
    CopyFrom(from);
    return *this;
  }
  inline RegionSimilarityCalculator& operator=(RegionSimilarityCalculator&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RegionSimilarityCalculator& default_instance() {
    return *internal_default_instance();
  }
  enum RegionSimilarityCase {
    kNegSqDistSimilarity = 1,
    kIouSimilarity = 2,
    kIoaSimilarity = 3,
    kThresholdedIouSimilarity = 4,
    REGION_SIMILARITY_NOT_SET = 0,
  };

  static inline const RegionSimilarityCalculator* internal_default_instance() {
    return reinterpret_cast<const RegionSimilarityCalculator*>(
               &_RegionSimilarityCalculator_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(RegionSimilarityCalculator& a, RegionSimilarityCalculator& b) {
    a.Swap(&b);
  }
  inline void Swap(RegionSimilarityCalculator* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RegionSimilarityCalculator* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RegionSimilarityCalculator* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<RegionSimilarityCalculator>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const RegionSimilarityCalculator& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const RegionSimilarityCalculator& from) {
    RegionSimilarityCalculator::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(RegionSimilarityCalculator* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.RegionSimilarityCalculator";
  }
  protected:
  explicit RegionSimilarityCalculator(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kNegSqDistSimilarityFieldNumber = 1,
    kIouSimilarityFieldNumber = 2,
    kIoaSimilarityFieldNumber = 3,
    kThresholdedIouSimilarityFieldNumber = 4,
  };
  // .object_detection.protos.NegSqDistSimilarity neg_sq_dist_similarity = 1;
  bool has_neg_sq_dist_similarity() const;
  private:
  bool _internal_has_neg_sq_dist_similarity() const;
  public:
  void clear_neg_sq_dist_similarity();
  const ::object_detection::protos::NegSqDistSimilarity& neg_sq_dist_similarity() const;
  PROTOBUF_NODISCARD ::object_detection::protos::NegSqDistSimilarity* release_neg_sq_dist_similarity();
  ::object_detection::protos::NegSqDistSimilarity* mutable_neg_sq_dist_similarity();
  void set_allocated_neg_sq_dist_similarity(::object_detection::protos::NegSqDistSimilarity* neg_sq_dist_similarity);
  private:
  const ::object_detection::protos::NegSqDistSimilarity& _internal_neg_sq_dist_similarity() const;
  ::object_detection::protos::NegSqDistSimilarity* _internal_mutable_neg_sq_dist_similarity();
  public:
  void unsafe_arena_set_allocated_neg_sq_dist_similarity(
      ::object_detection::protos::NegSqDistSimilarity* neg_sq_dist_similarity);
  ::object_detection::protos::NegSqDistSimilarity* unsafe_arena_release_neg_sq_dist_similarity();

  // .object_detection.protos.IouSimilarity iou_similarity = 2;
  bool has_iou_similarity() const;
  private:
  bool _internal_has_iou_similarity() const;
  public:
  void clear_iou_similarity();
  const ::object_detection::protos::IouSimilarity& iou_similarity() const;
  PROTOBUF_NODISCARD ::object_detection::protos::IouSimilarity* release_iou_similarity();
  ::object_detection::protos::IouSimilarity* mutable_iou_similarity();
  void set_allocated_iou_similarity(::object_detection::protos::IouSimilarity* iou_similarity);
  private:
  const ::object_detection::protos::IouSimilarity& _internal_iou_similarity() const;
  ::object_detection::protos::IouSimilarity* _internal_mutable_iou_similarity();
  public:
  void unsafe_arena_set_allocated_iou_similarity(
      ::object_detection::protos::IouSimilarity* iou_similarity);
  ::object_detection::protos::IouSimilarity* unsafe_arena_release_iou_similarity();

  // .object_detection.protos.IoaSimilarity ioa_similarity = 3;
  bool has_ioa_similarity() const;
  private:
  bool _internal_has_ioa_similarity() const;
  public:
  void clear_ioa_similarity();
  const ::object_detection::protos::IoaSimilarity& ioa_similarity() const;
  PROTOBUF_NODISCARD ::object_detection::protos::IoaSimilarity* release_ioa_similarity();
  ::object_detection::protos::IoaSimilarity* mutable_ioa_similarity();
  void set_allocated_ioa_similarity(::object_detection::protos::IoaSimilarity* ioa_similarity);
  private:
  const ::object_detection::protos::IoaSimilarity& _internal_ioa_similarity() const;
  ::object_detection::protos::IoaSimilarity* _internal_mutable_ioa_similarity();
  public:
  void unsafe_arena_set_allocated_ioa_similarity(
      ::object_detection::protos::IoaSimilarity* ioa_similarity);
  ::object_detection::protos::IoaSimilarity* unsafe_arena_release_ioa_similarity();

  // .object_detection.protos.ThresholdedIouSimilarity thresholded_iou_similarity = 4;
  bool has_thresholded_iou_similarity() const;
  private:
  bool _internal_has_thresholded_iou_similarity() const;
  public:
  void clear_thresholded_iou_similarity();
  const ::object_detection::protos::ThresholdedIouSimilarity& thresholded_iou_similarity() const;
  PROTOBUF_NODISCARD ::object_detection::protos::ThresholdedIouSimilarity* release_thresholded_iou_similarity();
  ::object_detection::protos::ThresholdedIouSimilarity* mutable_thresholded_iou_similarity();
  void set_allocated_thresholded_iou_similarity(::object_detection::protos::ThresholdedIouSimilarity* thresholded_iou_similarity);
  private:
  const ::object_detection::protos::ThresholdedIouSimilarity& _internal_thresholded_iou_similarity() const;
  ::object_detection::protos::ThresholdedIouSimilarity* _internal_mutable_thresholded_iou_similarity();
  public:
  void unsafe_arena_set_allocated_thresholded_iou_similarity(
      ::object_detection::protos::ThresholdedIouSimilarity* thresholded_iou_similarity);
  ::object_detection::protos::ThresholdedIouSimilarity* unsafe_arena_release_thresholded_iou_similarity();

  void clear_region_similarity();
  RegionSimilarityCase region_similarity_case() const;
  // @@protoc_insertion_point(class_scope:object_detection.protos.RegionSimilarityCalculator)
 private:
  class _Internal;
  void set_has_neg_sq_dist_similarity();
  void set_has_iou_similarity();
  void set_has_ioa_similarity();
  void set_has_thresholded_iou_similarity();

  inline bool has_region_similarity() const;
  inline void clear_has_region_similarity();

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    union RegionSimilarityUnion {
      constexpr RegionSimilarityUnion() : _constinit_{} {}
        ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized _constinit_;
      ::object_detection::protos::NegSqDistSimilarity* neg_sq_dist_similarity_;
      ::object_detection::protos::IouSimilarity* iou_similarity_;
      ::object_detection::protos::IoaSimilarity* ioa_similarity_;
      ::object_detection::protos::ThresholdedIouSimilarity* thresholded_iou_similarity_;
    } region_similarity_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    uint32_t _oneof_case_[1];

  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto;
};
// -------------------------------------------------------------------

class NegSqDistSimilarity final :
    public ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase /* @@protoc_insertion_point(class_definition:object_detection.protos.NegSqDistSimilarity) */ {
 public:
  inline NegSqDistSimilarity() : NegSqDistSimilarity(nullptr) {}
  explicit PROTOBUF_CONSTEXPR NegSqDistSimilarity(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  NegSqDistSimilarity(const NegSqDistSimilarity& from);
  NegSqDistSimilarity(NegSqDistSimilarity&& from) noexcept
    : NegSqDistSimilarity() {
    *this = ::std::move(from);
  }

  inline NegSqDistSimilarity& operator=(const NegSqDistSimilarity& from) {
    CopyFrom(from);
    return *this;
  }
  inline NegSqDistSimilarity& operator=(NegSqDistSimilarity&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const NegSqDistSimilarity& default_instance() {
    return *internal_default_instance();
  }
  static inline const NegSqDistSimilarity* internal_default_instance() {
    return reinterpret_cast<const NegSqDistSimilarity*>(
               &_NegSqDistSimilarity_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(NegSqDistSimilarity& a, NegSqDistSimilarity& b) {
    a.Swap(&b);
  }
  inline void Swap(NegSqDistSimilarity* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(NegSqDistSimilarity* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  NegSqDistSimilarity* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<NegSqDistSimilarity>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const NegSqDistSimilarity& from) {
    ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const NegSqDistSimilarity& from) {
    ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }
  public:

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.NegSqDistSimilarity";
  }
  protected:
  explicit NegSqDistSimilarity(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  // @@protoc_insertion_point(class_scope:object_detection.protos.NegSqDistSimilarity)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
  };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto;
};
// -------------------------------------------------------------------

class IouSimilarity final :
    public ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase /* @@protoc_insertion_point(class_definition:object_detection.protos.IouSimilarity) */ {
 public:
  inline IouSimilarity() : IouSimilarity(nullptr) {}
  explicit PROTOBUF_CONSTEXPR IouSimilarity(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  IouSimilarity(const IouSimilarity& from);
  IouSimilarity(IouSimilarity&& from) noexcept
    : IouSimilarity() {
    *this = ::std::move(from);
  }

  inline IouSimilarity& operator=(const IouSimilarity& from) {
    CopyFrom(from);
    return *this;
  }
  inline IouSimilarity& operator=(IouSimilarity&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const IouSimilarity& default_instance() {
    return *internal_default_instance();
  }
  static inline const IouSimilarity* internal_default_instance() {
    return reinterpret_cast<const IouSimilarity*>(
               &_IouSimilarity_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    2;

  friend void swap(IouSimilarity& a, IouSimilarity& b) {
    a.Swap(&b);
  }
  inline void Swap(IouSimilarity* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(IouSimilarity* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  IouSimilarity* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<IouSimilarity>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const IouSimilarity& from) {
    ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const IouSimilarity& from) {
    ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }
  public:

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.IouSimilarity";
  }
  protected:
  explicit IouSimilarity(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  // @@protoc_insertion_point(class_scope:object_detection.protos.IouSimilarity)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
  };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto;
};
// -------------------------------------------------------------------

class IoaSimilarity final :
    public ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase /* @@protoc_insertion_point(class_definition:object_detection.protos.IoaSimilarity) */ {
 public:
  inline IoaSimilarity() : IoaSimilarity(nullptr) {}
  explicit PROTOBUF_CONSTEXPR IoaSimilarity(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  IoaSimilarity(const IoaSimilarity& from);
  IoaSimilarity(IoaSimilarity&& from) noexcept
    : IoaSimilarity() {
    *this = ::std::move(from);
  }

  inline IoaSimilarity& operator=(const IoaSimilarity& from) {
    CopyFrom(from);
    return *this;
  }
  inline IoaSimilarity& operator=(IoaSimilarity&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const IoaSimilarity& default_instance() {
    return *internal_default_instance();
  }
  static inline const IoaSimilarity* internal_default_instance() {
    return reinterpret_cast<const IoaSimilarity*>(
               &_IoaSimilarity_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    3;

  friend void swap(IoaSimilarity& a, IoaSimilarity& b) {
    a.Swap(&b);
  }
  inline void Swap(IoaSimilarity* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(IoaSimilarity* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  IoaSimilarity* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<IoaSimilarity>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const IoaSimilarity& from) {
    ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const IoaSimilarity& from) {
    ::PROTOBUF_NAMESPACE_ID::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }
  public:

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.IoaSimilarity";
  }
  protected:
  explicit IoaSimilarity(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  // @@protoc_insertion_point(class_scope:object_detection.protos.IoaSimilarity)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
  };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto;
};
// -------------------------------------------------------------------

class ThresholdedIouSimilarity final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.ThresholdedIouSimilarity) */ {
 public:
  inline ThresholdedIouSimilarity() : ThresholdedIouSimilarity(nullptr) {}
  ~ThresholdedIouSimilarity() override;
  explicit PROTOBUF_CONSTEXPR ThresholdedIouSimilarity(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  ThresholdedIouSimilarity(const ThresholdedIouSimilarity& from);
  ThresholdedIouSimilarity(ThresholdedIouSimilarity&& from) noexcept
    : ThresholdedIouSimilarity() {
    *this = ::std::move(from);
  }

  inline ThresholdedIouSimilarity& operator=(const ThresholdedIouSimilarity& from) {
    CopyFrom(from);
    return *this;
  }
  inline ThresholdedIouSimilarity& operator=(ThresholdedIouSimilarity&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ThresholdedIouSimilarity& default_instance() {
    return *internal_default_instance();
  }
  static inline const ThresholdedIouSimilarity* internal_default_instance() {
    return reinterpret_cast<const ThresholdedIouSimilarity*>(
               &_ThresholdedIouSimilarity_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    4;

  friend void swap(ThresholdedIouSimilarity& a, ThresholdedIouSimilarity& b) {
    a.Swap(&b);
  }
  inline void Swap(ThresholdedIouSimilarity* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ThresholdedIouSimilarity* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ThresholdedIouSimilarity* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<ThresholdedIouSimilarity>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const ThresholdedIouSimilarity& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const ThresholdedIouSimilarity& from) {
    ThresholdedIouSimilarity::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(ThresholdedIouSimilarity* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.ThresholdedIouSimilarity";
  }
  protected:
  explicit ThresholdedIouSimilarity(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kIouThresholdFieldNumber = 1,
  };
  // optional float iou_threshold = 1 [default = 0.5];
  bool has_iou_threshold() const;
  private:
  bool _internal_has_iou_threshold() const;
  public:
  void clear_iou_threshold();
  float iou_threshold() const;
  void set_iou_threshold(float value);
  private:
  float _internal_iou_threshold() const;
  void _internal_set_iou_threshold(float value);
  public:

  // @@protoc_insertion_point(class_scope:object_detection.protos.ThresholdedIouSimilarity)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    float iou_threshold_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// RegionSimilarityCalculator

// .object_detection.protos.NegSqDistSimilarity neg_sq_dist_similarity = 1;
inline bool RegionSimilarityCalculator::_internal_has_neg_sq_dist_similarity() const {
  return region_similarity_case() == kNegSqDistSimilarity;
}
inline bool RegionSimilarityCalculator::has_neg_sq_dist_similarity() const {
  return _internal_has_neg_sq_dist_similarity();
}
inline void RegionSimilarityCalculator::set_has_neg_sq_dist_similarity() {
  _impl_._oneof_case_[0] = kNegSqDistSimilarity;
}
inline void RegionSimilarityCalculator::clear_neg_sq_dist_similarity() {
  if (_internal_has_neg_sq_dist_similarity()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.region_similarity_.neg_sq_dist_similarity_;
    }
    clear_has_region_similarity();
  }
}
inline ::object_detection::protos::NegSqDistSimilarity* RegionSimilarityCalculator::release_neg_sq_dist_similarity() {
  // @@protoc_insertion_point(field_release:object_detection.protos.RegionSimilarityCalculator.neg_sq_dist_similarity)
  if (_internal_has_neg_sq_dist_similarity()) {
    clear_has_region_similarity();
    ::object_detection::protos::NegSqDistSimilarity* temp = _impl_.region_similarity_.neg_sq_dist_similarity_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.region_similarity_.neg_sq_dist_similarity_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::object_detection::protos::NegSqDistSimilarity& RegionSimilarityCalculator::_internal_neg_sq_dist_similarity() const {
  return _internal_has_neg_sq_dist_similarity()
      ? *_impl_.region_similarity_.neg_sq_dist_similarity_
      : reinterpret_cast< ::object_detection::protos::NegSqDistSimilarity&>(::object_detection::protos::_NegSqDistSimilarity_default_instance_);
}
inline const ::object_detection::protos::NegSqDistSimilarity& RegionSimilarityCalculator::neg_sq_dist_similarity() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.RegionSimilarityCalculator.neg_sq_dist_similarity)
  return _internal_neg_sq_dist_similarity();
}
inline ::object_detection::protos::NegSqDistSimilarity* RegionSimilarityCalculator::unsafe_arena_release_neg_sq_dist_similarity() {
  // @@protoc_insertion_point(field_unsafe_arena_release:object_detection.protos.RegionSimilarityCalculator.neg_sq_dist_similarity)
  if (_internal_has_neg_sq_dist_similarity()) {
    clear_has_region_similarity();
    ::object_detection::protos::NegSqDistSimilarity* temp = _impl_.region_similarity_.neg_sq_dist_similarity_;
    _impl_.region_similarity_.neg_sq_dist_similarity_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void RegionSimilarityCalculator::unsafe_arena_set_allocated_neg_sq_dist_similarity(::object_detection::protos::NegSqDistSimilarity* neg_sq_dist_similarity) {
  clear_region_similarity();
  if (neg_sq_dist_similarity) {
    set_has_neg_sq_dist_similarity();
    _impl_.region_similarity_.neg_sq_dist_similarity_ = neg_sq_dist_similarity;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.RegionSimilarityCalculator.neg_sq_dist_similarity)
}
inline ::object_detection::protos::NegSqDistSimilarity* RegionSimilarityCalculator::_internal_mutable_neg_sq_dist_similarity() {
  if (!_internal_has_neg_sq_dist_similarity()) {
    clear_region_similarity();
    set_has_neg_sq_dist_similarity();
    _impl_.region_similarity_.neg_sq_dist_similarity_ = CreateMaybeMessage< ::object_detection::protos::NegSqDistSimilarity >(GetArenaForAllocation());
  }
  return _impl_.region_similarity_.neg_sq_dist_similarity_;
}
inline ::object_detection::protos::NegSqDistSimilarity* RegionSimilarityCalculator::mutable_neg_sq_dist_similarity() {
  ::object_detection::protos::NegSqDistSimilarity* _msg = _internal_mutable_neg_sq_dist_similarity();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.RegionSimilarityCalculator.neg_sq_dist_similarity)
  return _msg;
}

// .object_detection.protos.IouSimilarity iou_similarity = 2;
inline bool RegionSimilarityCalculator::_internal_has_iou_similarity() const {
  return region_similarity_case() == kIouSimilarity;
}
inline bool RegionSimilarityCalculator::has_iou_similarity() const {
  return _internal_has_iou_similarity();
}
inline void RegionSimilarityCalculator::set_has_iou_similarity() {
  _impl_._oneof_case_[0] = kIouSimilarity;
}
inline void RegionSimilarityCalculator::clear_iou_similarity() {
  if (_internal_has_iou_similarity()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.region_similarity_.iou_similarity_;
    }
    clear_has_region_similarity();
  }
}
inline ::object_detection::protos::IouSimilarity* RegionSimilarityCalculator::release_iou_similarity() {
  // @@protoc_insertion_point(field_release:object_detection.protos.RegionSimilarityCalculator.iou_similarity)
  if (_internal_has_iou_similarity()) {
    clear_has_region_similarity();
    ::object_detection::protos::IouSimilarity* temp = _impl_.region_similarity_.iou_similarity_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.region_similarity_.iou_similarity_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::object_detection::protos::IouSimilarity& RegionSimilarityCalculator::_internal_iou_similarity() const {
  return _internal_has_iou_similarity()
      ? *_impl_.region_similarity_.iou_similarity_
      : reinterpret_cast< ::object_detection::protos::IouSimilarity&>(::object_detection::protos::_IouSimilarity_default_instance_);
}
inline const ::object_detection::protos::IouSimilarity& RegionSimilarityCalculator::iou_similarity() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.RegionSimilarityCalculator.iou_similarity)
  return _internal_iou_similarity();
}
inline ::object_detection::protos::IouSimilarity* RegionSimilarityCalculator::unsafe_arena_release_iou_similarity() {
  // @@protoc_insertion_point(field_unsafe_arena_release:object_detection.protos.RegionSimilarityCalculator.iou_similarity)
  if (_internal_has_iou_similarity()) {
    clear_has_region_similarity();
    ::object_detection::protos::IouSimilarity* temp = _impl_.region_similarity_.iou_similarity_;
    _impl_.region_similarity_.iou_similarity_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void RegionSimilarityCalculator::unsafe_arena_set_allocated_iou_similarity(::object_detection::protos::IouSimilarity* iou_similarity) {
  clear_region_similarity();
  if (iou_similarity) {
    set_has_iou_similarity();
    _impl_.region_similarity_.iou_similarity_ = iou_similarity;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.RegionSimilarityCalculator.iou_similarity)
}
inline ::object_detection::protos::IouSimilarity* RegionSimilarityCalculator::_internal_mutable_iou_similarity() {
  if (!_internal_has_iou_similarity()) {
    clear_region_similarity();
    set_has_iou_similarity();
    _impl_.region_similarity_.iou_similarity_ = CreateMaybeMessage< ::object_detection::protos::IouSimilarity >(GetArenaForAllocation());
  }
  return _impl_.region_similarity_.iou_similarity_;
}
inline ::object_detection::protos::IouSimilarity* RegionSimilarityCalculator::mutable_iou_similarity() {
  ::object_detection::protos::IouSimilarity* _msg = _internal_mutable_iou_similarity();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.RegionSimilarityCalculator.iou_similarity)
  return _msg;
}

// .object_detection.protos.IoaSimilarity ioa_similarity = 3;
inline bool RegionSimilarityCalculator::_internal_has_ioa_similarity() const {
  return region_similarity_case() == kIoaSimilarity;
}
inline bool RegionSimilarityCalculator::has_ioa_similarity() const {
  return _internal_has_ioa_similarity();
}
inline void RegionSimilarityCalculator::set_has_ioa_similarity() {
  _impl_._oneof_case_[0] = kIoaSimilarity;
}
inline void RegionSimilarityCalculator::clear_ioa_similarity() {
  if (_internal_has_ioa_similarity()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.region_similarity_.ioa_similarity_;
    }
    clear_has_region_similarity();
  }
}
inline ::object_detection::protos::IoaSimilarity* RegionSimilarityCalculator::release_ioa_similarity() {
  // @@protoc_insertion_point(field_release:object_detection.protos.RegionSimilarityCalculator.ioa_similarity)
  if (_internal_has_ioa_similarity()) {
    clear_has_region_similarity();
    ::object_detection::protos::IoaSimilarity* temp = _impl_.region_similarity_.ioa_similarity_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.region_similarity_.ioa_similarity_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::object_detection::protos::IoaSimilarity& RegionSimilarityCalculator::_internal_ioa_similarity() const {
  return _internal_has_ioa_similarity()
      ? *_impl_.region_similarity_.ioa_similarity_
      : reinterpret_cast< ::object_detection::protos::IoaSimilarity&>(::object_detection::protos::_IoaSimilarity_default_instance_);
}
inline const ::object_detection::protos::IoaSimilarity& RegionSimilarityCalculator::ioa_similarity() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.RegionSimilarityCalculator.ioa_similarity)
  return _internal_ioa_similarity();
}
inline ::object_detection::protos::IoaSimilarity* RegionSimilarityCalculator::unsafe_arena_release_ioa_similarity() {
  // @@protoc_insertion_point(field_unsafe_arena_release:object_detection.protos.RegionSimilarityCalculator.ioa_similarity)
  if (_internal_has_ioa_similarity()) {
    clear_has_region_similarity();
    ::object_detection::protos::IoaSimilarity* temp = _impl_.region_similarity_.ioa_similarity_;
    _impl_.region_similarity_.ioa_similarity_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void RegionSimilarityCalculator::unsafe_arena_set_allocated_ioa_similarity(::object_detection::protos::IoaSimilarity* ioa_similarity) {
  clear_region_similarity();
  if (ioa_similarity) {
    set_has_ioa_similarity();
    _impl_.region_similarity_.ioa_similarity_ = ioa_similarity;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.RegionSimilarityCalculator.ioa_similarity)
}
inline ::object_detection::protos::IoaSimilarity* RegionSimilarityCalculator::_internal_mutable_ioa_similarity() {
  if (!_internal_has_ioa_similarity()) {
    clear_region_similarity();
    set_has_ioa_similarity();
    _impl_.region_similarity_.ioa_similarity_ = CreateMaybeMessage< ::object_detection::protos::IoaSimilarity >(GetArenaForAllocation());
  }
  return _impl_.region_similarity_.ioa_similarity_;
}
inline ::object_detection::protos::IoaSimilarity* RegionSimilarityCalculator::mutable_ioa_similarity() {
  ::object_detection::protos::IoaSimilarity* _msg = _internal_mutable_ioa_similarity();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.RegionSimilarityCalculator.ioa_similarity)
  return _msg;
}

// .object_detection.protos.ThresholdedIouSimilarity thresholded_iou_similarity = 4;
inline bool RegionSimilarityCalculator::_internal_has_thresholded_iou_similarity() const {
  return region_similarity_case() == kThresholdedIouSimilarity;
}
inline bool RegionSimilarityCalculator::has_thresholded_iou_similarity() const {
  return _internal_has_thresholded_iou_similarity();
}
inline void RegionSimilarityCalculator::set_has_thresholded_iou_similarity() {
  _impl_._oneof_case_[0] = kThresholdedIouSimilarity;
}
inline void RegionSimilarityCalculator::clear_thresholded_iou_similarity() {
  if (_internal_has_thresholded_iou_similarity()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.region_similarity_.thresholded_iou_similarity_;
    }
    clear_has_region_similarity();
  }
}
inline ::object_detection::protos::ThresholdedIouSimilarity* RegionSimilarityCalculator::release_thresholded_iou_similarity() {
  // @@protoc_insertion_point(field_release:object_detection.protos.RegionSimilarityCalculator.thresholded_iou_similarity)
  if (_internal_has_thresholded_iou_similarity()) {
    clear_has_region_similarity();
    ::object_detection::protos::ThresholdedIouSimilarity* temp = _impl_.region_similarity_.thresholded_iou_similarity_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.region_similarity_.thresholded_iou_similarity_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::object_detection::protos::ThresholdedIouSimilarity& RegionSimilarityCalculator::_internal_thresholded_iou_similarity() const {
  return _internal_has_thresholded_iou_similarity()
      ? *_impl_.region_similarity_.thresholded_iou_similarity_
      : reinterpret_cast< ::object_detection::protos::ThresholdedIouSimilarity&>(::object_detection::protos::_ThresholdedIouSimilarity_default_instance_);
}
inline const ::object_detection::protos::ThresholdedIouSimilarity& RegionSimilarityCalculator::thresholded_iou_similarity() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.RegionSimilarityCalculator.thresholded_iou_similarity)
  return _internal_thresholded_iou_similarity();
}
inline ::object_detection::protos::ThresholdedIouSimilarity* RegionSimilarityCalculator::unsafe_arena_release_thresholded_iou_similarity() {
  // @@protoc_insertion_point(field_unsafe_arena_release:object_detection.protos.RegionSimilarityCalculator.thresholded_iou_similarity)
  if (_internal_has_thresholded_iou_similarity()) {
    clear_has_region_similarity();
    ::object_detection::protos::ThresholdedIouSimilarity* temp = _impl_.region_similarity_.thresholded_iou_similarity_;
    _impl_.region_similarity_.thresholded_iou_similarity_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void RegionSimilarityCalculator::unsafe_arena_set_allocated_thresholded_iou_similarity(::object_detection::protos::ThresholdedIouSimilarity* thresholded_iou_similarity) {
  clear_region_similarity();
  if (thresholded_iou_similarity) {
    set_has_thresholded_iou_similarity();
    _impl_.region_similarity_.thresholded_iou_similarity_ = thresholded_iou_similarity;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.RegionSimilarityCalculator.thresholded_iou_similarity)
}
inline ::object_detection::protos::ThresholdedIouSimilarity* RegionSimilarityCalculator::_internal_mutable_thresholded_iou_similarity() {
  if (!_internal_has_thresholded_iou_similarity()) {
    clear_region_similarity();
    set_has_thresholded_iou_similarity();
    _impl_.region_similarity_.thresholded_iou_similarity_ = CreateMaybeMessage< ::object_detection::protos::ThresholdedIouSimilarity >(GetArenaForAllocation());
  }
  return _impl_.region_similarity_.thresholded_iou_similarity_;
}
inline ::object_detection::protos::ThresholdedIouSimilarity* RegionSimilarityCalculator::mutable_thresholded_iou_similarity() {
  ::object_detection::protos::ThresholdedIouSimilarity* _msg = _internal_mutable_thresholded_iou_similarity();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.RegionSimilarityCalculator.thresholded_iou_similarity)
  return _msg;
}

inline bool RegionSimilarityCalculator::has_region_similarity() const {
  return region_similarity_case() != REGION_SIMILARITY_NOT_SET;
}
inline void RegionSimilarityCalculator::clear_has_region_similarity() {
  _impl_._oneof_case_[0] = REGION_SIMILARITY_NOT_SET;
}
inline RegionSimilarityCalculator::RegionSimilarityCase RegionSimilarityCalculator::region_similarity_case() const {
  return RegionSimilarityCalculator::RegionSimilarityCase(_impl_._oneof_case_[0]);
}
// -------------------------------------------------------------------

// NegSqDistSimilarity

// -------------------------------------------------------------------

// IouSimilarity

// -------------------------------------------------------------------

// IoaSimilarity

// -------------------------------------------------------------------

// ThresholdedIouSimilarity

// optional float iou_threshold = 1 [default = 0.5];
inline bool ThresholdedIouSimilarity::_internal_has_iou_threshold() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool ThresholdedIouSimilarity::has_iou_threshold() const {
  return _internal_has_iou_threshold();
}
inline void ThresholdedIouSimilarity::clear_iou_threshold() {
  _impl_.iou_threshold_ = 0.5f;
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline float ThresholdedIouSimilarity::_internal_iou_threshold() const {
  return _impl_.iou_threshold_;
}
inline float ThresholdedIouSimilarity::iou_threshold() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.ThresholdedIouSimilarity.iou_threshold)
  return _internal_iou_threshold();
}
inline void ThresholdedIouSimilarity::_internal_set_iou_threshold(float value) {
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.iou_threshold_ = value;
}
inline void ThresholdedIouSimilarity::set_iou_threshold(float value) {
  _internal_set_iou_threshold(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.ThresholdedIouSimilarity.iou_threshold)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------


// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace object_detection

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fregion_5fsimilarity_5fcalculator_2eproto
