// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/anchors.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fanchors_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fanchors_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_second_2fprotos_2fanchors_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_second_2fprotos_2fanchors_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_second_2fprotos_2fanchors_2eproto;
namespace second {
namespace protos {
class AnchorGeneratorCollection;
struct AnchorGeneratorCollectionDefaultTypeInternal;
extern AnchorGeneratorCollectionDefaultTypeInternal _AnchorGeneratorCollection_default_instance_;
class AnchorGeneratorOld;
struct AnchorGeneratorOldDefaultTypeInternal;
extern AnchorGeneratorOldDefaultTypeInternal _AnchorGeneratorOld_default_instance_;
class AnchorGeneratorRange;
struct AnchorGeneratorRangeDefaultTypeInternal;
extern AnchorGeneratorRangeDefaultTypeInternal _AnchorGeneratorRange_default_instance_;
class AnchorGeneratorStride;
struct AnchorGeneratorStrideDefaultTypeInternal;
extern AnchorGeneratorStrideDefaultTypeInternal _AnchorGeneratorStride_default_instance_;
class AnchorGeneratorV1;
struct AnchorGeneratorV1DefaultTypeInternal;
extern AnchorGeneratorV1DefaultTypeInternal _AnchorGeneratorV1_default_instance_;
class AnchorGenerator_depara;
struct AnchorGenerator_deparaDefaultTypeInternal;
extern AnchorGenerator_deparaDefaultTypeInternal _AnchorGenerator_depara_default_instance_;
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> ::second::protos::AnchorGeneratorCollection* Arena::CreateMaybeMessage<::second::protos::AnchorGeneratorCollection>(Arena*);
template<> ::second::protos::AnchorGeneratorOld* Arena::CreateMaybeMessage<::second::protos::AnchorGeneratorOld>(Arena*);
template<> ::second::protos::AnchorGeneratorRange* Arena::CreateMaybeMessage<::second::protos::AnchorGeneratorRange>(Arena*);
template<> ::second::protos::AnchorGeneratorStride* Arena::CreateMaybeMessage<::second::protos::AnchorGeneratorStride>(Arena*);
template<> ::second::protos::AnchorGeneratorV1* Arena::CreateMaybeMessage<::second::protos::AnchorGeneratorV1>(Arena*);
template<> ::second::protos::AnchorGenerator_depara* Arena::CreateMaybeMessage<::second::protos::AnchorGenerator_depara>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace second {
namespace protos {

// ===================================================================

class AnchorGeneratorOld final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.AnchorGeneratorOld) */ {
 public:
  inline AnchorGeneratorOld() : AnchorGeneratorOld(nullptr) {}
  ~AnchorGeneratorOld() override;
  explicit PROTOBUF_CONSTEXPR AnchorGeneratorOld(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  AnchorGeneratorOld(const AnchorGeneratorOld& from);
  AnchorGeneratorOld(AnchorGeneratorOld&& from) noexcept
    : AnchorGeneratorOld() {
    *this = ::std::move(from);
  }

  inline AnchorGeneratorOld& operator=(const AnchorGeneratorOld& from) {
    CopyFrom(from);
    return *this;
  }
  inline AnchorGeneratorOld& operator=(AnchorGeneratorOld&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const AnchorGeneratorOld& default_instance() {
    return *internal_default_instance();
  }
  static inline const AnchorGeneratorOld* internal_default_instance() {
    return reinterpret_cast<const AnchorGeneratorOld*>(
               &_AnchorGeneratorOld_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(AnchorGeneratorOld& a, AnchorGeneratorOld& b) {
    a.Swap(&b);
  }
  inline void Swap(AnchorGeneratorOld* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AnchorGeneratorOld* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  AnchorGeneratorOld* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<AnchorGeneratorOld>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const AnchorGeneratorOld& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const AnchorGeneratorOld& from) {
    AnchorGeneratorOld::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(AnchorGeneratorOld* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.AnchorGeneratorOld";
  }
  protected:
  explicit AnchorGeneratorOld(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kBevRangeFieldNumber = 2,
    kAnchorSizesFieldNumber = 4,
    kClassNameFieldNumber = 1,
    kAnchorCenterZFieldNumber = 3,
    kMatchedThresholdFieldNumber = 5,
    kUnmatchedThresholdFieldNumber = 6,
  };
  // repeated float bev_range = 2;
  int bev_range_size() const;
  private:
  int _internal_bev_range_size() const;
  public:
  void clear_bev_range();
  private:
  float _internal_bev_range(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_bev_range() const;
  void _internal_add_bev_range(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_bev_range();
  public:
  float bev_range(int index) const;
  void set_bev_range(int index, float value);
  void add_bev_range(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      bev_range() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_bev_range();

  // repeated float anchor_sizes = 4;
  int anchor_sizes_size() const;
  private:
  int _internal_anchor_sizes_size() const;
  public:
  void clear_anchor_sizes();
  private:
  float _internal_anchor_sizes(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_anchor_sizes() const;
  void _internal_add_anchor_sizes(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_anchor_sizes();
  public:
  float anchor_sizes(int index) const;
  void set_anchor_sizes(int index, float value);
  void add_anchor_sizes(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      anchor_sizes() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_anchor_sizes();

  // string class_name = 1;
  void clear_class_name();
  const std::string& class_name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_class_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_class_name();
  PROTOBUF_NODISCARD std::string* release_class_name();
  void set_allocated_class_name(std::string* class_name);
  private:
  const std::string& _internal_class_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_class_name(const std::string& value);
  std::string* _internal_mutable_class_name();
  public:

  // float anchor_center_z = 3;
  void clear_anchor_center_z();
  float anchor_center_z() const;
  void set_anchor_center_z(float value);
  private:
  float _internal_anchor_center_z() const;
  void _internal_set_anchor_center_z(float value);
  public:

  // float matched_threshold = 5;
  void clear_matched_threshold();
  float matched_threshold() const;
  void set_matched_threshold(float value);
  private:
  float _internal_matched_threshold() const;
  void _internal_set_matched_threshold(float value);
  public:

  // float unmatched_threshold = 6;
  void clear_unmatched_threshold();
  float unmatched_threshold() const;
  void set_unmatched_threshold(float value);
  private:
  float _internal_unmatched_threshold() const;
  void _internal_set_unmatched_threshold(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.AnchorGeneratorOld)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > bev_range_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > anchor_sizes_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr class_name_;
    float anchor_center_z_;
    float matched_threshold_;
    float unmatched_threshold_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fanchors_2eproto;
};
// -------------------------------------------------------------------

class AnchorGeneratorStride final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.AnchorGeneratorStride) */ {
 public:
  inline AnchorGeneratorStride() : AnchorGeneratorStride(nullptr) {}
  ~AnchorGeneratorStride() override;
  explicit PROTOBUF_CONSTEXPR AnchorGeneratorStride(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  AnchorGeneratorStride(const AnchorGeneratorStride& from);
  AnchorGeneratorStride(AnchorGeneratorStride&& from) noexcept
    : AnchorGeneratorStride() {
    *this = ::std::move(from);
  }

  inline AnchorGeneratorStride& operator=(const AnchorGeneratorStride& from) {
    CopyFrom(from);
    return *this;
  }
  inline AnchorGeneratorStride& operator=(AnchorGeneratorStride&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const AnchorGeneratorStride& default_instance() {
    return *internal_default_instance();
  }
  static inline const AnchorGeneratorStride* internal_default_instance() {
    return reinterpret_cast<const AnchorGeneratorStride*>(
               &_AnchorGeneratorStride_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(AnchorGeneratorStride& a, AnchorGeneratorStride& b) {
    a.Swap(&b);
  }
  inline void Swap(AnchorGeneratorStride* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AnchorGeneratorStride* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  AnchorGeneratorStride* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<AnchorGeneratorStride>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const AnchorGeneratorStride& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const AnchorGeneratorStride& from) {
    AnchorGeneratorStride::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(AnchorGeneratorStride* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.AnchorGeneratorStride";
  }
  protected:
  explicit AnchorGeneratorStride(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kSizesFieldNumber = 2,
    kStridesFieldNumber = 3,
    kOffsetsFieldNumber = 4,
    kRotationsFieldNumber = 5,
    kClassNameFieldNumber = 1,
    kMatchedThresholdFieldNumber = 6,
    kUnmatchedThresholdFieldNumber = 7,
  };
  // repeated float sizes = 2;
  int sizes_size() const;
  private:
  int _internal_sizes_size() const;
  public:
  void clear_sizes();
  private:
  float _internal_sizes(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_sizes() const;
  void _internal_add_sizes(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_sizes();
  public:
  float sizes(int index) const;
  void set_sizes(int index, float value);
  void add_sizes(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      sizes() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_sizes();

  // repeated float strides = 3;
  int strides_size() const;
  private:
  int _internal_strides_size() const;
  public:
  void clear_strides();
  private:
  float _internal_strides(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_strides() const;
  void _internal_add_strides(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_strides();
  public:
  float strides(int index) const;
  void set_strides(int index, float value);
  void add_strides(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      strides() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_strides();

  // repeated float offsets = 4;
  int offsets_size() const;
  private:
  int _internal_offsets_size() const;
  public:
  void clear_offsets();
  private:
  float _internal_offsets(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_offsets() const;
  void _internal_add_offsets(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_offsets();
  public:
  float offsets(int index) const;
  void set_offsets(int index, float value);
  void add_offsets(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      offsets() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_offsets();

  // repeated float rotations = 5;
  int rotations_size() const;
  private:
  int _internal_rotations_size() const;
  public:
  void clear_rotations();
  private:
  float _internal_rotations(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_rotations() const;
  void _internal_add_rotations(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_rotations();
  public:
  float rotations(int index) const;
  void set_rotations(int index, float value);
  void add_rotations(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      rotations() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_rotations();

  // string class_name = 1;
  void clear_class_name();
  const std::string& class_name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_class_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_class_name();
  PROTOBUF_NODISCARD std::string* release_class_name();
  void set_allocated_class_name(std::string* class_name);
  private:
  const std::string& _internal_class_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_class_name(const std::string& value);
  std::string* _internal_mutable_class_name();
  public:

  // float matched_threshold = 6;
  void clear_matched_threshold();
  float matched_threshold() const;
  void set_matched_threshold(float value);
  private:
  float _internal_matched_threshold() const;
  void _internal_set_matched_threshold(float value);
  public:

  // float unmatched_threshold = 7;
  void clear_unmatched_threshold();
  float unmatched_threshold() const;
  void set_unmatched_threshold(float value);
  private:
  float _internal_unmatched_threshold() const;
  void _internal_set_unmatched_threshold(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.AnchorGeneratorStride)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > sizes_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > strides_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > offsets_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > rotations_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr class_name_;
    float matched_threshold_;
    float unmatched_threshold_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fanchors_2eproto;
};
// -------------------------------------------------------------------

class AnchorGeneratorRange final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.AnchorGeneratorRange) */ {
 public:
  inline AnchorGeneratorRange() : AnchorGeneratorRange(nullptr) {}
  ~AnchorGeneratorRange() override;
  explicit PROTOBUF_CONSTEXPR AnchorGeneratorRange(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  AnchorGeneratorRange(const AnchorGeneratorRange& from);
  AnchorGeneratorRange(AnchorGeneratorRange&& from) noexcept
    : AnchorGeneratorRange() {
    *this = ::std::move(from);
  }

  inline AnchorGeneratorRange& operator=(const AnchorGeneratorRange& from) {
    CopyFrom(from);
    return *this;
  }
  inline AnchorGeneratorRange& operator=(AnchorGeneratorRange&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const AnchorGeneratorRange& default_instance() {
    return *internal_default_instance();
  }
  static inline const AnchorGeneratorRange* internal_default_instance() {
    return reinterpret_cast<const AnchorGeneratorRange*>(
               &_AnchorGeneratorRange_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    2;

  friend void swap(AnchorGeneratorRange& a, AnchorGeneratorRange& b) {
    a.Swap(&b);
  }
  inline void Swap(AnchorGeneratorRange* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AnchorGeneratorRange* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  AnchorGeneratorRange* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<AnchorGeneratorRange>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const AnchorGeneratorRange& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const AnchorGeneratorRange& from) {
    AnchorGeneratorRange::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(AnchorGeneratorRange* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.AnchorGeneratorRange";
  }
  protected:
  explicit AnchorGeneratorRange(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kSizesFieldNumber = 2,
    kAnchorRangesFieldNumber = 3,
    kRotationsFieldNumber = 4,
    kClassNameFieldNumber = 1,
    kMatchedThresholdFieldNumber = 5,
    kUnmatchedThresholdFieldNumber = 6,
  };
  // repeated float sizes = 2;
  int sizes_size() const;
  private:
  int _internal_sizes_size() const;
  public:
  void clear_sizes();
  private:
  float _internal_sizes(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_sizes() const;
  void _internal_add_sizes(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_sizes();
  public:
  float sizes(int index) const;
  void set_sizes(int index, float value);
  void add_sizes(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      sizes() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_sizes();

  // repeated float anchor_ranges = 3;
  int anchor_ranges_size() const;
  private:
  int _internal_anchor_ranges_size() const;
  public:
  void clear_anchor_ranges();
  private:
  float _internal_anchor_ranges(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_anchor_ranges() const;
  void _internal_add_anchor_ranges(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_anchor_ranges();
  public:
  float anchor_ranges(int index) const;
  void set_anchor_ranges(int index, float value);
  void add_anchor_ranges(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      anchor_ranges() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_anchor_ranges();

  // repeated float rotations = 4;
  int rotations_size() const;
  private:
  int _internal_rotations_size() const;
  public:
  void clear_rotations();
  private:
  float _internal_rotations(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_rotations() const;
  void _internal_add_rotations(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_rotations();
  public:
  float rotations(int index) const;
  void set_rotations(int index, float value);
  void add_rotations(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      rotations() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_rotations();

  // string class_name = 1;
  void clear_class_name();
  const std::string& class_name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_class_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_class_name();
  PROTOBUF_NODISCARD std::string* release_class_name();
  void set_allocated_class_name(std::string* class_name);
  private:
  const std::string& _internal_class_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_class_name(const std::string& value);
  std::string* _internal_mutable_class_name();
  public:

  // float matched_threshold = 5;
  void clear_matched_threshold();
  float matched_threshold() const;
  void set_matched_threshold(float value);
  private:
  float _internal_matched_threshold() const;
  void _internal_set_matched_threshold(float value);
  public:

  // float unmatched_threshold = 6;
  void clear_unmatched_threshold();
  float unmatched_threshold() const;
  void set_unmatched_threshold(float value);
  private:
  float _internal_unmatched_threshold() const;
  void _internal_set_unmatched_threshold(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.AnchorGeneratorRange)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > sizes_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > anchor_ranges_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > rotations_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr class_name_;
    float matched_threshold_;
    float unmatched_threshold_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fanchors_2eproto;
};
// -------------------------------------------------------------------

class AnchorGeneratorCollection final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.AnchorGeneratorCollection) */ {
 public:
  inline AnchorGeneratorCollection() : AnchorGeneratorCollection(nullptr) {}
  ~AnchorGeneratorCollection() override;
  explicit PROTOBUF_CONSTEXPR AnchorGeneratorCollection(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  AnchorGeneratorCollection(const AnchorGeneratorCollection& from);
  AnchorGeneratorCollection(AnchorGeneratorCollection&& from) noexcept
    : AnchorGeneratorCollection() {
    *this = ::std::move(from);
  }

  inline AnchorGeneratorCollection& operator=(const AnchorGeneratorCollection& from) {
    CopyFrom(from);
    return *this;
  }
  inline AnchorGeneratorCollection& operator=(AnchorGeneratorCollection&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const AnchorGeneratorCollection& default_instance() {
    return *internal_default_instance();
  }
  enum AnchorGeneratorCase {
    kAnchorGeneratorStride = 1,
    kAnchorGeneratorRange = 2,
    kAnchorGeneratorOld = 3,
    ANCHOR_GENERATOR_NOT_SET = 0,
  };

  static inline const AnchorGeneratorCollection* internal_default_instance() {
    return reinterpret_cast<const AnchorGeneratorCollection*>(
               &_AnchorGeneratorCollection_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    3;

  friend void swap(AnchorGeneratorCollection& a, AnchorGeneratorCollection& b) {
    a.Swap(&b);
  }
  inline void Swap(AnchorGeneratorCollection* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AnchorGeneratorCollection* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  AnchorGeneratorCollection* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<AnchorGeneratorCollection>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const AnchorGeneratorCollection& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const AnchorGeneratorCollection& from) {
    AnchorGeneratorCollection::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(AnchorGeneratorCollection* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.AnchorGeneratorCollection";
  }
  protected:
  explicit AnchorGeneratorCollection(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kAnchorGeneratorStrideFieldNumber = 1,
    kAnchorGeneratorRangeFieldNumber = 2,
    kAnchorGeneratorOldFieldNumber = 3,
  };
  // .second.protos.AnchorGeneratorStride anchor_generator_stride = 1;
  bool has_anchor_generator_stride() const;
  private:
  bool _internal_has_anchor_generator_stride() const;
  public:
  void clear_anchor_generator_stride();
  const ::second::protos::AnchorGeneratorStride& anchor_generator_stride() const;
  PROTOBUF_NODISCARD ::second::protos::AnchorGeneratorStride* release_anchor_generator_stride();
  ::second::protos::AnchorGeneratorStride* mutable_anchor_generator_stride();
  void set_allocated_anchor_generator_stride(::second::protos::AnchorGeneratorStride* anchor_generator_stride);
  private:
  const ::second::protos::AnchorGeneratorStride& _internal_anchor_generator_stride() const;
  ::second::protos::AnchorGeneratorStride* _internal_mutable_anchor_generator_stride();
  public:
  void unsafe_arena_set_allocated_anchor_generator_stride(
      ::second::protos::AnchorGeneratorStride* anchor_generator_stride);
  ::second::protos::AnchorGeneratorStride* unsafe_arena_release_anchor_generator_stride();

  // .second.protos.AnchorGeneratorRange anchor_generator_range = 2;
  bool has_anchor_generator_range() const;
  private:
  bool _internal_has_anchor_generator_range() const;
  public:
  void clear_anchor_generator_range();
  const ::second::protos::AnchorGeneratorRange& anchor_generator_range() const;
  PROTOBUF_NODISCARD ::second::protos::AnchorGeneratorRange* release_anchor_generator_range();
  ::second::protos::AnchorGeneratorRange* mutable_anchor_generator_range();
  void set_allocated_anchor_generator_range(::second::protos::AnchorGeneratorRange* anchor_generator_range);
  private:
  const ::second::protos::AnchorGeneratorRange& _internal_anchor_generator_range() const;
  ::second::protos::AnchorGeneratorRange* _internal_mutable_anchor_generator_range();
  public:
  void unsafe_arena_set_allocated_anchor_generator_range(
      ::second::protos::AnchorGeneratorRange* anchor_generator_range);
  ::second::protos::AnchorGeneratorRange* unsafe_arena_release_anchor_generator_range();

  // .second.protos.AnchorGeneratorOld anchor_generator_old = 3;
  bool has_anchor_generator_old() const;
  private:
  bool _internal_has_anchor_generator_old() const;
  public:
  void clear_anchor_generator_old();
  const ::second::protos::AnchorGeneratorOld& anchor_generator_old() const;
  PROTOBUF_NODISCARD ::second::protos::AnchorGeneratorOld* release_anchor_generator_old();
  ::second::protos::AnchorGeneratorOld* mutable_anchor_generator_old();
  void set_allocated_anchor_generator_old(::second::protos::AnchorGeneratorOld* anchor_generator_old);
  private:
  const ::second::protos::AnchorGeneratorOld& _internal_anchor_generator_old() const;
  ::second::protos::AnchorGeneratorOld* _internal_mutable_anchor_generator_old();
  public:
  void unsafe_arena_set_allocated_anchor_generator_old(
      ::second::protos::AnchorGeneratorOld* anchor_generator_old);
  ::second::protos::AnchorGeneratorOld* unsafe_arena_release_anchor_generator_old();

  void clear_anchor_generator();
  AnchorGeneratorCase anchor_generator_case() const;
  // @@protoc_insertion_point(class_scope:second.protos.AnchorGeneratorCollection)
 private:
  class _Internal;
  void set_has_anchor_generator_stride();
  void set_has_anchor_generator_range();
  void set_has_anchor_generator_old();

  inline bool has_anchor_generator() const;
  inline void clear_has_anchor_generator();

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    union AnchorGeneratorUnion {
      constexpr AnchorGeneratorUnion() : _constinit_{} {}
        ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized _constinit_;
      ::second::protos::AnchorGeneratorStride* anchor_generator_stride_;
      ::second::protos::AnchorGeneratorRange* anchor_generator_range_;
      ::second::protos::AnchorGeneratorOld* anchor_generator_old_;
    } anchor_generator_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    uint32_t _oneof_case_[1];

  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fanchors_2eproto;
};
// -------------------------------------------------------------------

class AnchorGenerator_depara final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.AnchorGenerator_depara) */ {
 public:
  inline AnchorGenerator_depara() : AnchorGenerator_depara(nullptr) {}
  ~AnchorGenerator_depara() override;
  explicit PROTOBUF_CONSTEXPR AnchorGenerator_depara(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  AnchorGenerator_depara(const AnchorGenerator_depara& from);
  AnchorGenerator_depara(AnchorGenerator_depara&& from) noexcept
    : AnchorGenerator_depara() {
    *this = ::std::move(from);
  }

  inline AnchorGenerator_depara& operator=(const AnchorGenerator_depara& from) {
    CopyFrom(from);
    return *this;
  }
  inline AnchorGenerator_depara& operator=(AnchorGenerator_depara&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const AnchorGenerator_depara& default_instance() {
    return *internal_default_instance();
  }
  static inline const AnchorGenerator_depara* internal_default_instance() {
    return reinterpret_cast<const AnchorGenerator_depara*>(
               &_AnchorGenerator_depara_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    4;

  friend void swap(AnchorGenerator_depara& a, AnchorGenerator_depara& b) {
    a.Swap(&b);
  }
  inline void Swap(AnchorGenerator_depara* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AnchorGenerator_depara* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  AnchorGenerator_depara* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<AnchorGenerator_depara>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const AnchorGenerator_depara& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const AnchorGenerator_depara& from) {
    AnchorGenerator_depara::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(AnchorGenerator_depara* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.AnchorGenerator_depara";
  }
  protected:
  explicit AnchorGenerator_depara(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kSizesFieldNumber = 2,
    kStridesFieldNumber = 3,
    kOffsetsFieldNumber = 4,
    kRotationsFieldNumber = 5,
    kClassNameFieldNumber = 1,
    kMatchedThresholdFieldNumber = 6,
    kUnmatchedThresholdFieldNumber = 7,
  };
  // repeated float sizes = 2;
  int sizes_size() const;
  private:
  int _internal_sizes_size() const;
  public:
  void clear_sizes();
  private:
  float _internal_sizes(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_sizes() const;
  void _internal_add_sizes(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_sizes();
  public:
  float sizes(int index) const;
  void set_sizes(int index, float value);
  void add_sizes(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      sizes() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_sizes();

  // repeated float strides = 3;
  int strides_size() const;
  private:
  int _internal_strides_size() const;
  public:
  void clear_strides();
  private:
  float _internal_strides(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_strides() const;
  void _internal_add_strides(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_strides();
  public:
  float strides(int index) const;
  void set_strides(int index, float value);
  void add_strides(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      strides() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_strides();

  // repeated float offsets = 4;
  int offsets_size() const;
  private:
  int _internal_offsets_size() const;
  public:
  void clear_offsets();
  private:
  float _internal_offsets(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_offsets() const;
  void _internal_add_offsets(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_offsets();
  public:
  float offsets(int index) const;
  void set_offsets(int index, float value);
  void add_offsets(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      offsets() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_offsets();

  // repeated float rotations = 5;
  int rotations_size() const;
  private:
  int _internal_rotations_size() const;
  public:
  void clear_rotations();
  private:
  float _internal_rotations(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_rotations() const;
  void _internal_add_rotations(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_rotations();
  public:
  float rotations(int index) const;
  void set_rotations(int index, float value);
  void add_rotations(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      rotations() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_rotations();

  // string class_name = 1;
  void clear_class_name();
  const std::string& class_name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_class_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_class_name();
  PROTOBUF_NODISCARD std::string* release_class_name();
  void set_allocated_class_name(std::string* class_name);
  private:
  const std::string& _internal_class_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_class_name(const std::string& value);
  std::string* _internal_mutable_class_name();
  public:

  // float matched_threshold = 6;
  void clear_matched_threshold();
  float matched_threshold() const;
  void set_matched_threshold(float value);
  private:
  float _internal_matched_threshold() const;
  void _internal_set_matched_threshold(float value);
  public:

  // float unmatched_threshold = 7;
  void clear_unmatched_threshold();
  float unmatched_threshold() const;
  void set_unmatched_threshold(float value);
  private:
  float _internal_unmatched_threshold() const;
  void _internal_set_unmatched_threshold(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.AnchorGenerator_depara)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > sizes_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > strides_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > offsets_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > rotations_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr class_name_;
    float matched_threshold_;
    float unmatched_threshold_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fanchors_2eproto;
};
// -------------------------------------------------------------------

class AnchorGeneratorV1 final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.AnchorGeneratorV1) */ {
 public:
  inline AnchorGeneratorV1() : AnchorGeneratorV1(nullptr) {}
  ~AnchorGeneratorV1() override;
  explicit PROTOBUF_CONSTEXPR AnchorGeneratorV1(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  AnchorGeneratorV1(const AnchorGeneratorV1& from);
  AnchorGeneratorV1(AnchorGeneratorV1&& from) noexcept
    : AnchorGeneratorV1() {
    *this = ::std::move(from);
  }

  inline AnchorGeneratorV1& operator=(const AnchorGeneratorV1& from) {
    CopyFrom(from);
    return *this;
  }
  inline AnchorGeneratorV1& operator=(AnchorGeneratorV1&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const AnchorGeneratorV1& default_instance() {
    return *internal_default_instance();
  }
  static inline const AnchorGeneratorV1* internal_default_instance() {
    return reinterpret_cast<const AnchorGeneratorV1*>(
               &_AnchorGeneratorV1_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    5;

  friend void swap(AnchorGeneratorV1& a, AnchorGeneratorV1& b) {
    a.Swap(&b);
  }
  inline void Swap(AnchorGeneratorV1* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AnchorGeneratorV1* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  AnchorGeneratorV1* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<AnchorGeneratorV1>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const AnchorGeneratorV1& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const AnchorGeneratorV1& from) {
    AnchorGeneratorV1::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(AnchorGeneratorV1* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.AnchorGeneratorV1";
  }
  protected:
  explicit AnchorGeneratorV1(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kAnchorSizesFieldNumber = 2,
    kAnchorCenterZFieldNumber = 1,
    kMatchedThresholdFieldNumber = 3,
    kUnmatchedThresholdFieldNumber = 4,
  };
  // repeated float anchor_sizes = 2;
  int anchor_sizes_size() const;
  private:
  int _internal_anchor_sizes_size() const;
  public:
  void clear_anchor_sizes();
  private:
  float _internal_anchor_sizes(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_anchor_sizes() const;
  void _internal_add_anchor_sizes(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_anchor_sizes();
  public:
  float anchor_sizes(int index) const;
  void set_anchor_sizes(int index, float value);
  void add_anchor_sizes(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      anchor_sizes() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_anchor_sizes();

  // float anchor_center_z = 1;
  void clear_anchor_center_z();
  float anchor_center_z() const;
  void set_anchor_center_z(float value);
  private:
  float _internal_anchor_center_z() const;
  void _internal_set_anchor_center_z(float value);
  public:

  // float matched_threshold = 3;
  void clear_matched_threshold();
  float matched_threshold() const;
  void set_matched_threshold(float value);
  private:
  float _internal_matched_threshold() const;
  void _internal_set_matched_threshold(float value);
  public:

  // float unmatched_threshold = 4;
  void clear_unmatched_threshold();
  float unmatched_threshold() const;
  void set_unmatched_threshold(float value);
  private:
  float _internal_unmatched_threshold() const;
  void _internal_set_unmatched_threshold(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.AnchorGeneratorV1)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > anchor_sizes_;
    float anchor_center_z_;
    float matched_threshold_;
    float unmatched_threshold_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fanchors_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// AnchorGeneratorOld

// string class_name = 1;
inline void AnchorGeneratorOld::clear_class_name() {
  _impl_.class_name_.ClearToEmpty();
}
inline const std::string& AnchorGeneratorOld::class_name() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorOld.class_name)
  return _internal_class_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void AnchorGeneratorOld::set_class_name(ArgT0&& arg0, ArgT... args) {
 
 _impl_.class_name_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorOld.class_name)
}
inline std::string* AnchorGeneratorOld::mutable_class_name() {
  std::string* _s = _internal_mutable_class_name();
  // @@protoc_insertion_point(field_mutable:second.protos.AnchorGeneratorOld.class_name)
  return _s;
}
inline const std::string& AnchorGeneratorOld::_internal_class_name() const {
  return _impl_.class_name_.Get();
}
inline void AnchorGeneratorOld::_internal_set_class_name(const std::string& value) {
  
  _impl_.class_name_.Set(value, GetArenaForAllocation());
}
inline std::string* AnchorGeneratorOld::_internal_mutable_class_name() {
  
  return _impl_.class_name_.Mutable(GetArenaForAllocation());
}
inline std::string* AnchorGeneratorOld::release_class_name() {
  // @@protoc_insertion_point(field_release:second.protos.AnchorGeneratorOld.class_name)
  return _impl_.class_name_.Release();
}
inline void AnchorGeneratorOld::set_allocated_class_name(std::string* class_name) {
  if (class_name != nullptr) {
    
  } else {
    
  }
  _impl_.class_name_.SetAllocated(class_name, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.class_name_.IsDefault()) {
    _impl_.class_name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:second.protos.AnchorGeneratorOld.class_name)
}

// repeated float bev_range = 2;
inline int AnchorGeneratorOld::_internal_bev_range_size() const {
  return _impl_.bev_range_.size();
}
inline int AnchorGeneratorOld::bev_range_size() const {
  return _internal_bev_range_size();
}
inline void AnchorGeneratorOld::clear_bev_range() {
  _impl_.bev_range_.Clear();
}
inline float AnchorGeneratorOld::_internal_bev_range(int index) const {
  return _impl_.bev_range_.Get(index);
}
inline float AnchorGeneratorOld::bev_range(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorOld.bev_range)
  return _internal_bev_range(index);
}
inline void AnchorGeneratorOld::set_bev_range(int index, float value) {
  _impl_.bev_range_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorOld.bev_range)
}
inline void AnchorGeneratorOld::_internal_add_bev_range(float value) {
  _impl_.bev_range_.Add(value);
}
inline void AnchorGeneratorOld::add_bev_range(float value) {
  _internal_add_bev_range(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGeneratorOld.bev_range)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorOld::_internal_bev_range() const {
  return _impl_.bev_range_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorOld::bev_range() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGeneratorOld.bev_range)
  return _internal_bev_range();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorOld::_internal_mutable_bev_range() {
  return &_impl_.bev_range_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorOld::mutable_bev_range() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGeneratorOld.bev_range)
  return _internal_mutable_bev_range();
}

// float anchor_center_z = 3;
inline void AnchorGeneratorOld::clear_anchor_center_z() {
  _impl_.anchor_center_z_ = 0;
}
inline float AnchorGeneratorOld::_internal_anchor_center_z() const {
  return _impl_.anchor_center_z_;
}
inline float AnchorGeneratorOld::anchor_center_z() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorOld.anchor_center_z)
  return _internal_anchor_center_z();
}
inline void AnchorGeneratorOld::_internal_set_anchor_center_z(float value) {
  
  _impl_.anchor_center_z_ = value;
}
inline void AnchorGeneratorOld::set_anchor_center_z(float value) {
  _internal_set_anchor_center_z(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorOld.anchor_center_z)
}

// repeated float anchor_sizes = 4;
inline int AnchorGeneratorOld::_internal_anchor_sizes_size() const {
  return _impl_.anchor_sizes_.size();
}
inline int AnchorGeneratorOld::anchor_sizes_size() const {
  return _internal_anchor_sizes_size();
}
inline void AnchorGeneratorOld::clear_anchor_sizes() {
  _impl_.anchor_sizes_.Clear();
}
inline float AnchorGeneratorOld::_internal_anchor_sizes(int index) const {
  return _impl_.anchor_sizes_.Get(index);
}
inline float AnchorGeneratorOld::anchor_sizes(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorOld.anchor_sizes)
  return _internal_anchor_sizes(index);
}
inline void AnchorGeneratorOld::set_anchor_sizes(int index, float value) {
  _impl_.anchor_sizes_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorOld.anchor_sizes)
}
inline void AnchorGeneratorOld::_internal_add_anchor_sizes(float value) {
  _impl_.anchor_sizes_.Add(value);
}
inline void AnchorGeneratorOld::add_anchor_sizes(float value) {
  _internal_add_anchor_sizes(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGeneratorOld.anchor_sizes)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorOld::_internal_anchor_sizes() const {
  return _impl_.anchor_sizes_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorOld::anchor_sizes() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGeneratorOld.anchor_sizes)
  return _internal_anchor_sizes();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorOld::_internal_mutable_anchor_sizes() {
  return &_impl_.anchor_sizes_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorOld::mutable_anchor_sizes() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGeneratorOld.anchor_sizes)
  return _internal_mutable_anchor_sizes();
}

// float matched_threshold = 5;
inline void AnchorGeneratorOld::clear_matched_threshold() {
  _impl_.matched_threshold_ = 0;
}
inline float AnchorGeneratorOld::_internal_matched_threshold() const {
  return _impl_.matched_threshold_;
}
inline float AnchorGeneratorOld::matched_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorOld.matched_threshold)
  return _internal_matched_threshold();
}
inline void AnchorGeneratorOld::_internal_set_matched_threshold(float value) {
  
  _impl_.matched_threshold_ = value;
}
inline void AnchorGeneratorOld::set_matched_threshold(float value) {
  _internal_set_matched_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorOld.matched_threshold)
}

// float unmatched_threshold = 6;
inline void AnchorGeneratorOld::clear_unmatched_threshold() {
  _impl_.unmatched_threshold_ = 0;
}
inline float AnchorGeneratorOld::_internal_unmatched_threshold() const {
  return _impl_.unmatched_threshold_;
}
inline float AnchorGeneratorOld::unmatched_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorOld.unmatched_threshold)
  return _internal_unmatched_threshold();
}
inline void AnchorGeneratorOld::_internal_set_unmatched_threshold(float value) {
  
  _impl_.unmatched_threshold_ = value;
}
inline void AnchorGeneratorOld::set_unmatched_threshold(float value) {
  _internal_set_unmatched_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorOld.unmatched_threshold)
}

// -------------------------------------------------------------------

// AnchorGeneratorStride

// string class_name = 1;
inline void AnchorGeneratorStride::clear_class_name() {
  _impl_.class_name_.ClearToEmpty();
}
inline const std::string& AnchorGeneratorStride::class_name() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorStride.class_name)
  return _internal_class_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void AnchorGeneratorStride::set_class_name(ArgT0&& arg0, ArgT... args) {
 
 _impl_.class_name_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorStride.class_name)
}
inline std::string* AnchorGeneratorStride::mutable_class_name() {
  std::string* _s = _internal_mutable_class_name();
  // @@protoc_insertion_point(field_mutable:second.protos.AnchorGeneratorStride.class_name)
  return _s;
}
inline const std::string& AnchorGeneratorStride::_internal_class_name() const {
  return _impl_.class_name_.Get();
}
inline void AnchorGeneratorStride::_internal_set_class_name(const std::string& value) {
  
  _impl_.class_name_.Set(value, GetArenaForAllocation());
}
inline std::string* AnchorGeneratorStride::_internal_mutable_class_name() {
  
  return _impl_.class_name_.Mutable(GetArenaForAllocation());
}
inline std::string* AnchorGeneratorStride::release_class_name() {
  // @@protoc_insertion_point(field_release:second.protos.AnchorGeneratorStride.class_name)
  return _impl_.class_name_.Release();
}
inline void AnchorGeneratorStride::set_allocated_class_name(std::string* class_name) {
  if (class_name != nullptr) {
    
  } else {
    
  }
  _impl_.class_name_.SetAllocated(class_name, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.class_name_.IsDefault()) {
    _impl_.class_name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:second.protos.AnchorGeneratorStride.class_name)
}

// repeated float sizes = 2;
inline int AnchorGeneratorStride::_internal_sizes_size() const {
  return _impl_.sizes_.size();
}
inline int AnchorGeneratorStride::sizes_size() const {
  return _internal_sizes_size();
}
inline void AnchorGeneratorStride::clear_sizes() {
  _impl_.sizes_.Clear();
}
inline float AnchorGeneratorStride::_internal_sizes(int index) const {
  return _impl_.sizes_.Get(index);
}
inline float AnchorGeneratorStride::sizes(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorStride.sizes)
  return _internal_sizes(index);
}
inline void AnchorGeneratorStride::set_sizes(int index, float value) {
  _impl_.sizes_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorStride.sizes)
}
inline void AnchorGeneratorStride::_internal_add_sizes(float value) {
  _impl_.sizes_.Add(value);
}
inline void AnchorGeneratorStride::add_sizes(float value) {
  _internal_add_sizes(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGeneratorStride.sizes)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorStride::_internal_sizes() const {
  return _impl_.sizes_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorStride::sizes() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGeneratorStride.sizes)
  return _internal_sizes();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorStride::_internal_mutable_sizes() {
  return &_impl_.sizes_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorStride::mutable_sizes() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGeneratorStride.sizes)
  return _internal_mutable_sizes();
}

// repeated float strides = 3;
inline int AnchorGeneratorStride::_internal_strides_size() const {
  return _impl_.strides_.size();
}
inline int AnchorGeneratorStride::strides_size() const {
  return _internal_strides_size();
}
inline void AnchorGeneratorStride::clear_strides() {
  _impl_.strides_.Clear();
}
inline float AnchorGeneratorStride::_internal_strides(int index) const {
  return _impl_.strides_.Get(index);
}
inline float AnchorGeneratorStride::strides(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorStride.strides)
  return _internal_strides(index);
}
inline void AnchorGeneratorStride::set_strides(int index, float value) {
  _impl_.strides_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorStride.strides)
}
inline void AnchorGeneratorStride::_internal_add_strides(float value) {
  _impl_.strides_.Add(value);
}
inline void AnchorGeneratorStride::add_strides(float value) {
  _internal_add_strides(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGeneratorStride.strides)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorStride::_internal_strides() const {
  return _impl_.strides_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorStride::strides() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGeneratorStride.strides)
  return _internal_strides();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorStride::_internal_mutable_strides() {
  return &_impl_.strides_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorStride::mutable_strides() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGeneratorStride.strides)
  return _internal_mutable_strides();
}

// repeated float offsets = 4;
inline int AnchorGeneratorStride::_internal_offsets_size() const {
  return _impl_.offsets_.size();
}
inline int AnchorGeneratorStride::offsets_size() const {
  return _internal_offsets_size();
}
inline void AnchorGeneratorStride::clear_offsets() {
  _impl_.offsets_.Clear();
}
inline float AnchorGeneratorStride::_internal_offsets(int index) const {
  return _impl_.offsets_.Get(index);
}
inline float AnchorGeneratorStride::offsets(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorStride.offsets)
  return _internal_offsets(index);
}
inline void AnchorGeneratorStride::set_offsets(int index, float value) {
  _impl_.offsets_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorStride.offsets)
}
inline void AnchorGeneratorStride::_internal_add_offsets(float value) {
  _impl_.offsets_.Add(value);
}
inline void AnchorGeneratorStride::add_offsets(float value) {
  _internal_add_offsets(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGeneratorStride.offsets)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorStride::_internal_offsets() const {
  return _impl_.offsets_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorStride::offsets() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGeneratorStride.offsets)
  return _internal_offsets();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorStride::_internal_mutable_offsets() {
  return &_impl_.offsets_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorStride::mutable_offsets() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGeneratorStride.offsets)
  return _internal_mutable_offsets();
}

// repeated float rotations = 5;
inline int AnchorGeneratorStride::_internal_rotations_size() const {
  return _impl_.rotations_.size();
}
inline int AnchorGeneratorStride::rotations_size() const {
  return _internal_rotations_size();
}
inline void AnchorGeneratorStride::clear_rotations() {
  _impl_.rotations_.Clear();
}
inline float AnchorGeneratorStride::_internal_rotations(int index) const {
  return _impl_.rotations_.Get(index);
}
inline float AnchorGeneratorStride::rotations(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorStride.rotations)
  return _internal_rotations(index);
}
inline void AnchorGeneratorStride::set_rotations(int index, float value) {
  _impl_.rotations_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorStride.rotations)
}
inline void AnchorGeneratorStride::_internal_add_rotations(float value) {
  _impl_.rotations_.Add(value);
}
inline void AnchorGeneratorStride::add_rotations(float value) {
  _internal_add_rotations(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGeneratorStride.rotations)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorStride::_internal_rotations() const {
  return _impl_.rotations_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorStride::rotations() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGeneratorStride.rotations)
  return _internal_rotations();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorStride::_internal_mutable_rotations() {
  return &_impl_.rotations_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorStride::mutable_rotations() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGeneratorStride.rotations)
  return _internal_mutable_rotations();
}

// float matched_threshold = 6;
inline void AnchorGeneratorStride::clear_matched_threshold() {
  _impl_.matched_threshold_ = 0;
}
inline float AnchorGeneratorStride::_internal_matched_threshold() const {
  return _impl_.matched_threshold_;
}
inline float AnchorGeneratorStride::matched_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorStride.matched_threshold)
  return _internal_matched_threshold();
}
inline void AnchorGeneratorStride::_internal_set_matched_threshold(float value) {
  
  _impl_.matched_threshold_ = value;
}
inline void AnchorGeneratorStride::set_matched_threshold(float value) {
  _internal_set_matched_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorStride.matched_threshold)
}

// float unmatched_threshold = 7;
inline void AnchorGeneratorStride::clear_unmatched_threshold() {
  _impl_.unmatched_threshold_ = 0;
}
inline float AnchorGeneratorStride::_internal_unmatched_threshold() const {
  return _impl_.unmatched_threshold_;
}
inline float AnchorGeneratorStride::unmatched_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorStride.unmatched_threshold)
  return _internal_unmatched_threshold();
}
inline void AnchorGeneratorStride::_internal_set_unmatched_threshold(float value) {
  
  _impl_.unmatched_threshold_ = value;
}
inline void AnchorGeneratorStride::set_unmatched_threshold(float value) {
  _internal_set_unmatched_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorStride.unmatched_threshold)
}

// -------------------------------------------------------------------

// AnchorGeneratorRange

// string class_name = 1;
inline void AnchorGeneratorRange::clear_class_name() {
  _impl_.class_name_.ClearToEmpty();
}
inline const std::string& AnchorGeneratorRange::class_name() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorRange.class_name)
  return _internal_class_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void AnchorGeneratorRange::set_class_name(ArgT0&& arg0, ArgT... args) {
 
 _impl_.class_name_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorRange.class_name)
}
inline std::string* AnchorGeneratorRange::mutable_class_name() {
  std::string* _s = _internal_mutable_class_name();
  // @@protoc_insertion_point(field_mutable:second.protos.AnchorGeneratorRange.class_name)
  return _s;
}
inline const std::string& AnchorGeneratorRange::_internal_class_name() const {
  return _impl_.class_name_.Get();
}
inline void AnchorGeneratorRange::_internal_set_class_name(const std::string& value) {
  
  _impl_.class_name_.Set(value, GetArenaForAllocation());
}
inline std::string* AnchorGeneratorRange::_internal_mutable_class_name() {
  
  return _impl_.class_name_.Mutable(GetArenaForAllocation());
}
inline std::string* AnchorGeneratorRange::release_class_name() {
  // @@protoc_insertion_point(field_release:second.protos.AnchorGeneratorRange.class_name)
  return _impl_.class_name_.Release();
}
inline void AnchorGeneratorRange::set_allocated_class_name(std::string* class_name) {
  if (class_name != nullptr) {
    
  } else {
    
  }
  _impl_.class_name_.SetAllocated(class_name, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.class_name_.IsDefault()) {
    _impl_.class_name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:second.protos.AnchorGeneratorRange.class_name)
}

// repeated float sizes = 2;
inline int AnchorGeneratorRange::_internal_sizes_size() const {
  return _impl_.sizes_.size();
}
inline int AnchorGeneratorRange::sizes_size() const {
  return _internal_sizes_size();
}
inline void AnchorGeneratorRange::clear_sizes() {
  _impl_.sizes_.Clear();
}
inline float AnchorGeneratorRange::_internal_sizes(int index) const {
  return _impl_.sizes_.Get(index);
}
inline float AnchorGeneratorRange::sizes(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorRange.sizes)
  return _internal_sizes(index);
}
inline void AnchorGeneratorRange::set_sizes(int index, float value) {
  _impl_.sizes_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorRange.sizes)
}
inline void AnchorGeneratorRange::_internal_add_sizes(float value) {
  _impl_.sizes_.Add(value);
}
inline void AnchorGeneratorRange::add_sizes(float value) {
  _internal_add_sizes(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGeneratorRange.sizes)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorRange::_internal_sizes() const {
  return _impl_.sizes_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorRange::sizes() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGeneratorRange.sizes)
  return _internal_sizes();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorRange::_internal_mutable_sizes() {
  return &_impl_.sizes_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorRange::mutable_sizes() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGeneratorRange.sizes)
  return _internal_mutable_sizes();
}

// repeated float anchor_ranges = 3;
inline int AnchorGeneratorRange::_internal_anchor_ranges_size() const {
  return _impl_.anchor_ranges_.size();
}
inline int AnchorGeneratorRange::anchor_ranges_size() const {
  return _internal_anchor_ranges_size();
}
inline void AnchorGeneratorRange::clear_anchor_ranges() {
  _impl_.anchor_ranges_.Clear();
}
inline float AnchorGeneratorRange::_internal_anchor_ranges(int index) const {
  return _impl_.anchor_ranges_.Get(index);
}
inline float AnchorGeneratorRange::anchor_ranges(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorRange.anchor_ranges)
  return _internal_anchor_ranges(index);
}
inline void AnchorGeneratorRange::set_anchor_ranges(int index, float value) {
  _impl_.anchor_ranges_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorRange.anchor_ranges)
}
inline void AnchorGeneratorRange::_internal_add_anchor_ranges(float value) {
  _impl_.anchor_ranges_.Add(value);
}
inline void AnchorGeneratorRange::add_anchor_ranges(float value) {
  _internal_add_anchor_ranges(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGeneratorRange.anchor_ranges)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorRange::_internal_anchor_ranges() const {
  return _impl_.anchor_ranges_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorRange::anchor_ranges() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGeneratorRange.anchor_ranges)
  return _internal_anchor_ranges();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorRange::_internal_mutable_anchor_ranges() {
  return &_impl_.anchor_ranges_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorRange::mutable_anchor_ranges() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGeneratorRange.anchor_ranges)
  return _internal_mutable_anchor_ranges();
}

// repeated float rotations = 4;
inline int AnchorGeneratorRange::_internal_rotations_size() const {
  return _impl_.rotations_.size();
}
inline int AnchorGeneratorRange::rotations_size() const {
  return _internal_rotations_size();
}
inline void AnchorGeneratorRange::clear_rotations() {
  _impl_.rotations_.Clear();
}
inline float AnchorGeneratorRange::_internal_rotations(int index) const {
  return _impl_.rotations_.Get(index);
}
inline float AnchorGeneratorRange::rotations(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorRange.rotations)
  return _internal_rotations(index);
}
inline void AnchorGeneratorRange::set_rotations(int index, float value) {
  _impl_.rotations_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorRange.rotations)
}
inline void AnchorGeneratorRange::_internal_add_rotations(float value) {
  _impl_.rotations_.Add(value);
}
inline void AnchorGeneratorRange::add_rotations(float value) {
  _internal_add_rotations(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGeneratorRange.rotations)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorRange::_internal_rotations() const {
  return _impl_.rotations_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorRange::rotations() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGeneratorRange.rotations)
  return _internal_rotations();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorRange::_internal_mutable_rotations() {
  return &_impl_.rotations_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorRange::mutable_rotations() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGeneratorRange.rotations)
  return _internal_mutable_rotations();
}

// float matched_threshold = 5;
inline void AnchorGeneratorRange::clear_matched_threshold() {
  _impl_.matched_threshold_ = 0;
}
inline float AnchorGeneratorRange::_internal_matched_threshold() const {
  return _impl_.matched_threshold_;
}
inline float AnchorGeneratorRange::matched_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorRange.matched_threshold)
  return _internal_matched_threshold();
}
inline void AnchorGeneratorRange::_internal_set_matched_threshold(float value) {
  
  _impl_.matched_threshold_ = value;
}
inline void AnchorGeneratorRange::set_matched_threshold(float value) {
  _internal_set_matched_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorRange.matched_threshold)
}

// float unmatched_threshold = 6;
inline void AnchorGeneratorRange::clear_unmatched_threshold() {
  _impl_.unmatched_threshold_ = 0;
}
inline float AnchorGeneratorRange::_internal_unmatched_threshold() const {
  return _impl_.unmatched_threshold_;
}
inline float AnchorGeneratorRange::unmatched_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorRange.unmatched_threshold)
  return _internal_unmatched_threshold();
}
inline void AnchorGeneratorRange::_internal_set_unmatched_threshold(float value) {
  
  _impl_.unmatched_threshold_ = value;
}
inline void AnchorGeneratorRange::set_unmatched_threshold(float value) {
  _internal_set_unmatched_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorRange.unmatched_threshold)
}

// -------------------------------------------------------------------

// AnchorGeneratorCollection

// .second.protos.AnchorGeneratorStride anchor_generator_stride = 1;
inline bool AnchorGeneratorCollection::_internal_has_anchor_generator_stride() const {
  return anchor_generator_case() == kAnchorGeneratorStride;
}
inline bool AnchorGeneratorCollection::has_anchor_generator_stride() const {
  return _internal_has_anchor_generator_stride();
}
inline void AnchorGeneratorCollection::set_has_anchor_generator_stride() {
  _impl_._oneof_case_[0] = kAnchorGeneratorStride;
}
inline void AnchorGeneratorCollection::clear_anchor_generator_stride() {
  if (_internal_has_anchor_generator_stride()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.anchor_generator_.anchor_generator_stride_;
    }
    clear_has_anchor_generator();
  }
}
inline ::second::protos::AnchorGeneratorStride* AnchorGeneratorCollection::release_anchor_generator_stride() {
  // @@protoc_insertion_point(field_release:second.protos.AnchorGeneratorCollection.anchor_generator_stride)
  if (_internal_has_anchor_generator_stride()) {
    clear_has_anchor_generator();
    ::second::protos::AnchorGeneratorStride* temp = _impl_.anchor_generator_.anchor_generator_stride_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.anchor_generator_.anchor_generator_stride_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::second::protos::AnchorGeneratorStride& AnchorGeneratorCollection::_internal_anchor_generator_stride() const {
  return _internal_has_anchor_generator_stride()
      ? *_impl_.anchor_generator_.anchor_generator_stride_
      : reinterpret_cast< ::second::protos::AnchorGeneratorStride&>(::second::protos::_AnchorGeneratorStride_default_instance_);
}
inline const ::second::protos::AnchorGeneratorStride& AnchorGeneratorCollection::anchor_generator_stride() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorCollection.anchor_generator_stride)
  return _internal_anchor_generator_stride();
}
inline ::second::protos::AnchorGeneratorStride* AnchorGeneratorCollection::unsafe_arena_release_anchor_generator_stride() {
  // @@protoc_insertion_point(field_unsafe_arena_release:second.protos.AnchorGeneratorCollection.anchor_generator_stride)
  if (_internal_has_anchor_generator_stride()) {
    clear_has_anchor_generator();
    ::second::protos::AnchorGeneratorStride* temp = _impl_.anchor_generator_.anchor_generator_stride_;
    _impl_.anchor_generator_.anchor_generator_stride_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void AnchorGeneratorCollection::unsafe_arena_set_allocated_anchor_generator_stride(::second::protos::AnchorGeneratorStride* anchor_generator_stride) {
  clear_anchor_generator();
  if (anchor_generator_stride) {
    set_has_anchor_generator_stride();
    _impl_.anchor_generator_.anchor_generator_stride_ = anchor_generator_stride;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.AnchorGeneratorCollection.anchor_generator_stride)
}
inline ::second::protos::AnchorGeneratorStride* AnchorGeneratorCollection::_internal_mutable_anchor_generator_stride() {
  if (!_internal_has_anchor_generator_stride()) {
    clear_anchor_generator();
    set_has_anchor_generator_stride();
    _impl_.anchor_generator_.anchor_generator_stride_ = CreateMaybeMessage< ::second::protos::AnchorGeneratorStride >(GetArenaForAllocation());
  }
  return _impl_.anchor_generator_.anchor_generator_stride_;
}
inline ::second::protos::AnchorGeneratorStride* AnchorGeneratorCollection::mutable_anchor_generator_stride() {
  ::second::protos::AnchorGeneratorStride* _msg = _internal_mutable_anchor_generator_stride();
  // @@protoc_insertion_point(field_mutable:second.protos.AnchorGeneratorCollection.anchor_generator_stride)
  return _msg;
}

// .second.protos.AnchorGeneratorRange anchor_generator_range = 2;
inline bool AnchorGeneratorCollection::_internal_has_anchor_generator_range() const {
  return anchor_generator_case() == kAnchorGeneratorRange;
}
inline bool AnchorGeneratorCollection::has_anchor_generator_range() const {
  return _internal_has_anchor_generator_range();
}
inline void AnchorGeneratorCollection::set_has_anchor_generator_range() {
  _impl_._oneof_case_[0] = kAnchorGeneratorRange;
}
inline void AnchorGeneratorCollection::clear_anchor_generator_range() {
  if (_internal_has_anchor_generator_range()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.anchor_generator_.anchor_generator_range_;
    }
    clear_has_anchor_generator();
  }
}
inline ::second::protos::AnchorGeneratorRange* AnchorGeneratorCollection::release_anchor_generator_range() {
  // @@protoc_insertion_point(field_release:second.protos.AnchorGeneratorCollection.anchor_generator_range)
  if (_internal_has_anchor_generator_range()) {
    clear_has_anchor_generator();
    ::second::protos::AnchorGeneratorRange* temp = _impl_.anchor_generator_.anchor_generator_range_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.anchor_generator_.anchor_generator_range_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::second::protos::AnchorGeneratorRange& AnchorGeneratorCollection::_internal_anchor_generator_range() const {
  return _internal_has_anchor_generator_range()
      ? *_impl_.anchor_generator_.anchor_generator_range_
      : reinterpret_cast< ::second::protos::AnchorGeneratorRange&>(::second::protos::_AnchorGeneratorRange_default_instance_);
}
inline const ::second::protos::AnchorGeneratorRange& AnchorGeneratorCollection::anchor_generator_range() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorCollection.anchor_generator_range)
  return _internal_anchor_generator_range();
}
inline ::second::protos::AnchorGeneratorRange* AnchorGeneratorCollection::unsafe_arena_release_anchor_generator_range() {
  // @@protoc_insertion_point(field_unsafe_arena_release:second.protos.AnchorGeneratorCollection.anchor_generator_range)
  if (_internal_has_anchor_generator_range()) {
    clear_has_anchor_generator();
    ::second::protos::AnchorGeneratorRange* temp = _impl_.anchor_generator_.anchor_generator_range_;
    _impl_.anchor_generator_.anchor_generator_range_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void AnchorGeneratorCollection::unsafe_arena_set_allocated_anchor_generator_range(::second::protos::AnchorGeneratorRange* anchor_generator_range) {
  clear_anchor_generator();
  if (anchor_generator_range) {
    set_has_anchor_generator_range();
    _impl_.anchor_generator_.anchor_generator_range_ = anchor_generator_range;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.AnchorGeneratorCollection.anchor_generator_range)
}
inline ::second::protos::AnchorGeneratorRange* AnchorGeneratorCollection::_internal_mutable_anchor_generator_range() {
  if (!_internal_has_anchor_generator_range()) {
    clear_anchor_generator();
    set_has_anchor_generator_range();
    _impl_.anchor_generator_.anchor_generator_range_ = CreateMaybeMessage< ::second::protos::AnchorGeneratorRange >(GetArenaForAllocation());
  }
  return _impl_.anchor_generator_.anchor_generator_range_;
}
inline ::second::protos::AnchorGeneratorRange* AnchorGeneratorCollection::mutable_anchor_generator_range() {
  ::second::protos::AnchorGeneratorRange* _msg = _internal_mutable_anchor_generator_range();
  // @@protoc_insertion_point(field_mutable:second.protos.AnchorGeneratorCollection.anchor_generator_range)
  return _msg;
}

// .second.protos.AnchorGeneratorOld anchor_generator_old = 3;
inline bool AnchorGeneratorCollection::_internal_has_anchor_generator_old() const {
  return anchor_generator_case() == kAnchorGeneratorOld;
}
inline bool AnchorGeneratorCollection::has_anchor_generator_old() const {
  return _internal_has_anchor_generator_old();
}
inline void AnchorGeneratorCollection::set_has_anchor_generator_old() {
  _impl_._oneof_case_[0] = kAnchorGeneratorOld;
}
inline void AnchorGeneratorCollection::clear_anchor_generator_old() {
  if (_internal_has_anchor_generator_old()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.anchor_generator_.anchor_generator_old_;
    }
    clear_has_anchor_generator();
  }
}
inline ::second::protos::AnchorGeneratorOld* AnchorGeneratorCollection::release_anchor_generator_old() {
  // @@protoc_insertion_point(field_release:second.protos.AnchorGeneratorCollection.anchor_generator_old)
  if (_internal_has_anchor_generator_old()) {
    clear_has_anchor_generator();
    ::second::protos::AnchorGeneratorOld* temp = _impl_.anchor_generator_.anchor_generator_old_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.anchor_generator_.anchor_generator_old_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::second::protos::AnchorGeneratorOld& AnchorGeneratorCollection::_internal_anchor_generator_old() const {
  return _internal_has_anchor_generator_old()
      ? *_impl_.anchor_generator_.anchor_generator_old_
      : reinterpret_cast< ::second::protos::AnchorGeneratorOld&>(::second::protos::_AnchorGeneratorOld_default_instance_);
}
inline const ::second::protos::AnchorGeneratorOld& AnchorGeneratorCollection::anchor_generator_old() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorCollection.anchor_generator_old)
  return _internal_anchor_generator_old();
}
inline ::second::protos::AnchorGeneratorOld* AnchorGeneratorCollection::unsafe_arena_release_anchor_generator_old() {
  // @@protoc_insertion_point(field_unsafe_arena_release:second.protos.AnchorGeneratorCollection.anchor_generator_old)
  if (_internal_has_anchor_generator_old()) {
    clear_has_anchor_generator();
    ::second::protos::AnchorGeneratorOld* temp = _impl_.anchor_generator_.anchor_generator_old_;
    _impl_.anchor_generator_.anchor_generator_old_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void AnchorGeneratorCollection::unsafe_arena_set_allocated_anchor_generator_old(::second::protos::AnchorGeneratorOld* anchor_generator_old) {
  clear_anchor_generator();
  if (anchor_generator_old) {
    set_has_anchor_generator_old();
    _impl_.anchor_generator_.anchor_generator_old_ = anchor_generator_old;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.AnchorGeneratorCollection.anchor_generator_old)
}
inline ::second::protos::AnchorGeneratorOld* AnchorGeneratorCollection::_internal_mutable_anchor_generator_old() {
  if (!_internal_has_anchor_generator_old()) {
    clear_anchor_generator();
    set_has_anchor_generator_old();
    _impl_.anchor_generator_.anchor_generator_old_ = CreateMaybeMessage< ::second::protos::AnchorGeneratorOld >(GetArenaForAllocation());
  }
  return _impl_.anchor_generator_.anchor_generator_old_;
}
inline ::second::protos::AnchorGeneratorOld* AnchorGeneratorCollection::mutable_anchor_generator_old() {
  ::second::protos::AnchorGeneratorOld* _msg = _internal_mutable_anchor_generator_old();
  // @@protoc_insertion_point(field_mutable:second.protos.AnchorGeneratorCollection.anchor_generator_old)
  return _msg;
}

inline bool AnchorGeneratorCollection::has_anchor_generator() const {
  return anchor_generator_case() != ANCHOR_GENERATOR_NOT_SET;
}
inline void AnchorGeneratorCollection::clear_has_anchor_generator() {
  _impl_._oneof_case_[0] = ANCHOR_GENERATOR_NOT_SET;
}
inline AnchorGeneratorCollection::AnchorGeneratorCase AnchorGeneratorCollection::anchor_generator_case() const {
  return AnchorGeneratorCollection::AnchorGeneratorCase(_impl_._oneof_case_[0]);
}
// -------------------------------------------------------------------

// AnchorGenerator_depara

// string class_name = 1;
inline void AnchorGenerator_depara::clear_class_name() {
  _impl_.class_name_.ClearToEmpty();
}
inline const std::string& AnchorGenerator_depara::class_name() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGenerator_depara.class_name)
  return _internal_class_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void AnchorGenerator_depara::set_class_name(ArgT0&& arg0, ArgT... args) {
 
 _impl_.class_name_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:second.protos.AnchorGenerator_depara.class_name)
}
inline std::string* AnchorGenerator_depara::mutable_class_name() {
  std::string* _s = _internal_mutable_class_name();
  // @@protoc_insertion_point(field_mutable:second.protos.AnchorGenerator_depara.class_name)
  return _s;
}
inline const std::string& AnchorGenerator_depara::_internal_class_name() const {
  return _impl_.class_name_.Get();
}
inline void AnchorGenerator_depara::_internal_set_class_name(const std::string& value) {
  
  _impl_.class_name_.Set(value, GetArenaForAllocation());
}
inline std::string* AnchorGenerator_depara::_internal_mutable_class_name() {
  
  return _impl_.class_name_.Mutable(GetArenaForAllocation());
}
inline std::string* AnchorGenerator_depara::release_class_name() {
  // @@protoc_insertion_point(field_release:second.protos.AnchorGenerator_depara.class_name)
  return _impl_.class_name_.Release();
}
inline void AnchorGenerator_depara::set_allocated_class_name(std::string* class_name) {
  if (class_name != nullptr) {
    
  } else {
    
  }
  _impl_.class_name_.SetAllocated(class_name, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.class_name_.IsDefault()) {
    _impl_.class_name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:second.protos.AnchorGenerator_depara.class_name)
}

// repeated float sizes = 2;
inline int AnchorGenerator_depara::_internal_sizes_size() const {
  return _impl_.sizes_.size();
}
inline int AnchorGenerator_depara::sizes_size() const {
  return _internal_sizes_size();
}
inline void AnchorGenerator_depara::clear_sizes() {
  _impl_.sizes_.Clear();
}
inline float AnchorGenerator_depara::_internal_sizes(int index) const {
  return _impl_.sizes_.Get(index);
}
inline float AnchorGenerator_depara::sizes(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGenerator_depara.sizes)
  return _internal_sizes(index);
}
inline void AnchorGenerator_depara::set_sizes(int index, float value) {
  _impl_.sizes_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGenerator_depara.sizes)
}
inline void AnchorGenerator_depara::_internal_add_sizes(float value) {
  _impl_.sizes_.Add(value);
}
inline void AnchorGenerator_depara::add_sizes(float value) {
  _internal_add_sizes(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGenerator_depara.sizes)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGenerator_depara::_internal_sizes() const {
  return _impl_.sizes_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGenerator_depara::sizes() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGenerator_depara.sizes)
  return _internal_sizes();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGenerator_depara::_internal_mutable_sizes() {
  return &_impl_.sizes_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGenerator_depara::mutable_sizes() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGenerator_depara.sizes)
  return _internal_mutable_sizes();
}

// repeated float strides = 3;
inline int AnchorGenerator_depara::_internal_strides_size() const {
  return _impl_.strides_.size();
}
inline int AnchorGenerator_depara::strides_size() const {
  return _internal_strides_size();
}
inline void AnchorGenerator_depara::clear_strides() {
  _impl_.strides_.Clear();
}
inline float AnchorGenerator_depara::_internal_strides(int index) const {
  return _impl_.strides_.Get(index);
}
inline float AnchorGenerator_depara::strides(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGenerator_depara.strides)
  return _internal_strides(index);
}
inline void AnchorGenerator_depara::set_strides(int index, float value) {
  _impl_.strides_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGenerator_depara.strides)
}
inline void AnchorGenerator_depara::_internal_add_strides(float value) {
  _impl_.strides_.Add(value);
}
inline void AnchorGenerator_depara::add_strides(float value) {
  _internal_add_strides(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGenerator_depara.strides)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGenerator_depara::_internal_strides() const {
  return _impl_.strides_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGenerator_depara::strides() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGenerator_depara.strides)
  return _internal_strides();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGenerator_depara::_internal_mutable_strides() {
  return &_impl_.strides_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGenerator_depara::mutable_strides() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGenerator_depara.strides)
  return _internal_mutable_strides();
}

// repeated float offsets = 4;
inline int AnchorGenerator_depara::_internal_offsets_size() const {
  return _impl_.offsets_.size();
}
inline int AnchorGenerator_depara::offsets_size() const {
  return _internal_offsets_size();
}
inline void AnchorGenerator_depara::clear_offsets() {
  _impl_.offsets_.Clear();
}
inline float AnchorGenerator_depara::_internal_offsets(int index) const {
  return _impl_.offsets_.Get(index);
}
inline float AnchorGenerator_depara::offsets(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGenerator_depara.offsets)
  return _internal_offsets(index);
}
inline void AnchorGenerator_depara::set_offsets(int index, float value) {
  _impl_.offsets_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGenerator_depara.offsets)
}
inline void AnchorGenerator_depara::_internal_add_offsets(float value) {
  _impl_.offsets_.Add(value);
}
inline void AnchorGenerator_depara::add_offsets(float value) {
  _internal_add_offsets(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGenerator_depara.offsets)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGenerator_depara::_internal_offsets() const {
  return _impl_.offsets_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGenerator_depara::offsets() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGenerator_depara.offsets)
  return _internal_offsets();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGenerator_depara::_internal_mutable_offsets() {
  return &_impl_.offsets_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGenerator_depara::mutable_offsets() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGenerator_depara.offsets)
  return _internal_mutable_offsets();
}

// repeated float rotations = 5;
inline int AnchorGenerator_depara::_internal_rotations_size() const {
  return _impl_.rotations_.size();
}
inline int AnchorGenerator_depara::rotations_size() const {
  return _internal_rotations_size();
}
inline void AnchorGenerator_depara::clear_rotations() {
  _impl_.rotations_.Clear();
}
inline float AnchorGenerator_depara::_internal_rotations(int index) const {
  return _impl_.rotations_.Get(index);
}
inline float AnchorGenerator_depara::rotations(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGenerator_depara.rotations)
  return _internal_rotations(index);
}
inline void AnchorGenerator_depara::set_rotations(int index, float value) {
  _impl_.rotations_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGenerator_depara.rotations)
}
inline void AnchorGenerator_depara::_internal_add_rotations(float value) {
  _impl_.rotations_.Add(value);
}
inline void AnchorGenerator_depara::add_rotations(float value) {
  _internal_add_rotations(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGenerator_depara.rotations)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGenerator_depara::_internal_rotations() const {
  return _impl_.rotations_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGenerator_depara::rotations() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGenerator_depara.rotations)
  return _internal_rotations();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGenerator_depara::_internal_mutable_rotations() {
  return &_impl_.rotations_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGenerator_depara::mutable_rotations() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGenerator_depara.rotations)
  return _internal_mutable_rotations();
}

// float matched_threshold = 6;
inline void AnchorGenerator_depara::clear_matched_threshold() {
  _impl_.matched_threshold_ = 0;
}
inline float AnchorGenerator_depara::_internal_matched_threshold() const {
  return _impl_.matched_threshold_;
}
inline float AnchorGenerator_depara::matched_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGenerator_depara.matched_threshold)
  return _internal_matched_threshold();
}
inline void AnchorGenerator_depara::_internal_set_matched_threshold(float value) {
  
  _impl_.matched_threshold_ = value;
}
inline void AnchorGenerator_depara::set_matched_threshold(float value) {
  _internal_set_matched_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGenerator_depara.matched_threshold)
}

// float unmatched_threshold = 7;
inline void AnchorGenerator_depara::clear_unmatched_threshold() {
  _impl_.unmatched_threshold_ = 0;
}
inline float AnchorGenerator_depara::_internal_unmatched_threshold() const {
  return _impl_.unmatched_threshold_;
}
inline float AnchorGenerator_depara::unmatched_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGenerator_depara.unmatched_threshold)
  return _internal_unmatched_threshold();
}
inline void AnchorGenerator_depara::_internal_set_unmatched_threshold(float value) {
  
  _impl_.unmatched_threshold_ = value;
}
inline void AnchorGenerator_depara::set_unmatched_threshold(float value) {
  _internal_set_unmatched_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGenerator_depara.unmatched_threshold)
}

// -------------------------------------------------------------------

// AnchorGeneratorV1

// float anchor_center_z = 1;
inline void AnchorGeneratorV1::clear_anchor_center_z() {
  _impl_.anchor_center_z_ = 0;
}
inline float AnchorGeneratorV1::_internal_anchor_center_z() const {
  return _impl_.anchor_center_z_;
}
inline float AnchorGeneratorV1::anchor_center_z() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorV1.anchor_center_z)
  return _internal_anchor_center_z();
}
inline void AnchorGeneratorV1::_internal_set_anchor_center_z(float value) {
  
  _impl_.anchor_center_z_ = value;
}
inline void AnchorGeneratorV1::set_anchor_center_z(float value) {
  _internal_set_anchor_center_z(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorV1.anchor_center_z)
}

// repeated float anchor_sizes = 2;
inline int AnchorGeneratorV1::_internal_anchor_sizes_size() const {
  return _impl_.anchor_sizes_.size();
}
inline int AnchorGeneratorV1::anchor_sizes_size() const {
  return _internal_anchor_sizes_size();
}
inline void AnchorGeneratorV1::clear_anchor_sizes() {
  _impl_.anchor_sizes_.Clear();
}
inline float AnchorGeneratorV1::_internal_anchor_sizes(int index) const {
  return _impl_.anchor_sizes_.Get(index);
}
inline float AnchorGeneratorV1::anchor_sizes(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorV1.anchor_sizes)
  return _internal_anchor_sizes(index);
}
inline void AnchorGeneratorV1::set_anchor_sizes(int index, float value) {
  _impl_.anchor_sizes_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorV1.anchor_sizes)
}
inline void AnchorGeneratorV1::_internal_add_anchor_sizes(float value) {
  _impl_.anchor_sizes_.Add(value);
}
inline void AnchorGeneratorV1::add_anchor_sizes(float value) {
  _internal_add_anchor_sizes(value);
  // @@protoc_insertion_point(field_add:second.protos.AnchorGeneratorV1.anchor_sizes)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorV1::_internal_anchor_sizes() const {
  return _impl_.anchor_sizes_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
AnchorGeneratorV1::anchor_sizes() const {
  // @@protoc_insertion_point(field_list:second.protos.AnchorGeneratorV1.anchor_sizes)
  return _internal_anchor_sizes();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorV1::_internal_mutable_anchor_sizes() {
  return &_impl_.anchor_sizes_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
AnchorGeneratorV1::mutable_anchor_sizes() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.AnchorGeneratorV1.anchor_sizes)
  return _internal_mutable_anchor_sizes();
}

// float matched_threshold = 3;
inline void AnchorGeneratorV1::clear_matched_threshold() {
  _impl_.matched_threshold_ = 0;
}
inline float AnchorGeneratorV1::_internal_matched_threshold() const {
  return _impl_.matched_threshold_;
}
inline float AnchorGeneratorV1::matched_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorV1.matched_threshold)
  return _internal_matched_threshold();
}
inline void AnchorGeneratorV1::_internal_set_matched_threshold(float value) {
  
  _impl_.matched_threshold_ = value;
}
inline void AnchorGeneratorV1::set_matched_threshold(float value) {
  _internal_set_matched_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorV1.matched_threshold)
}

// float unmatched_threshold = 4;
inline void AnchorGeneratorV1::clear_unmatched_threshold() {
  _impl_.unmatched_threshold_ = 0;
}
inline float AnchorGeneratorV1::_internal_unmatched_threshold() const {
  return _impl_.unmatched_threshold_;
}
inline float AnchorGeneratorV1::unmatched_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.AnchorGeneratorV1.unmatched_threshold)
  return _internal_unmatched_threshold();
}
inline void AnchorGeneratorV1::_internal_set_unmatched_threshold(float value) {
  
  _impl_.unmatched_threshold_ = value;
}
inline void AnchorGeneratorV1::set_unmatched_threshold(float value) {
  _internal_set_unmatched_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.AnchorGeneratorV1.unmatched_threshold)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------


// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace second

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fanchors_2eproto
