// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: object_detection/protos/matcher.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fmatcher_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fmatcher_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "object_detection/protos/argmax_matcher.pb.h"
#include "object_detection/protos/bipartite_matcher.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_object_5fdetection_2fprotos_2fmatcher_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_object_5fdetection_2fprotos_2fmatcher_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_object_5fdetection_2fprotos_2fmatcher_2eproto;
namespace object_detection {
namespace protos {
class Matcher;
struct MatcherDefaultTypeInternal;
extern MatcherDefaultTypeInternal _Matcher_default_instance_;
}  // namespace protos
}  // namespace object_detection
PROTOBUF_NAMESPACE_OPEN
template<> ::object_detection::protos::Matcher* Arena::CreateMaybeMessage<::object_detection::protos::Matcher>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace object_detection {
namespace protos {

// ===================================================================

class Matcher final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.Matcher) */ {
 public:
  inline Matcher() : Matcher(nullptr) {}
  ~Matcher() override;
  explicit PROTOBUF_CONSTEXPR Matcher(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Matcher(const Matcher& from);
  Matcher(Matcher&& from) noexcept
    : Matcher() {
    *this = ::std::move(from);
  }

  inline Matcher& operator=(const Matcher& from) {
    CopyFrom(from);
    return *this;
  }
  inline Matcher& operator=(Matcher&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Matcher& default_instance() {
    return *internal_default_instance();
  }
  enum MatcherOneofCase {
    kArgmaxMatcher = 1,
    kBipartiteMatcher = 2,
    MATCHER_ONEOF_NOT_SET = 0,
  };

  static inline const Matcher* internal_default_instance() {
    return reinterpret_cast<const Matcher*>(
               &_Matcher_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(Matcher& a, Matcher& b) {
    a.Swap(&b);
  }
  inline void Swap(Matcher* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Matcher* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Matcher* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<Matcher>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const Matcher& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const Matcher& from) {
    Matcher::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Matcher* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.Matcher";
  }
  protected:
  explicit Matcher(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kArgmaxMatcherFieldNumber = 1,
    kBipartiteMatcherFieldNumber = 2,
  };
  // .object_detection.protos.ArgMaxMatcher argmax_matcher = 1;
  bool has_argmax_matcher() const;
  private:
  bool _internal_has_argmax_matcher() const;
  public:
  void clear_argmax_matcher();
  const ::object_detection::protos::ArgMaxMatcher& argmax_matcher() const;
  PROTOBUF_NODISCARD ::object_detection::protos::ArgMaxMatcher* release_argmax_matcher();
  ::object_detection::protos::ArgMaxMatcher* mutable_argmax_matcher();
  void set_allocated_argmax_matcher(::object_detection::protos::ArgMaxMatcher* argmax_matcher);
  private:
  const ::object_detection::protos::ArgMaxMatcher& _internal_argmax_matcher() const;
  ::object_detection::protos::ArgMaxMatcher* _internal_mutable_argmax_matcher();
  public:
  void unsafe_arena_set_allocated_argmax_matcher(
      ::object_detection::protos::ArgMaxMatcher* argmax_matcher);
  ::object_detection::protos::ArgMaxMatcher* unsafe_arena_release_argmax_matcher();

  // .object_detection.protos.BipartiteMatcher bipartite_matcher = 2;
  bool has_bipartite_matcher() const;
  private:
  bool _internal_has_bipartite_matcher() const;
  public:
  void clear_bipartite_matcher();
  const ::object_detection::protos::BipartiteMatcher& bipartite_matcher() const;
  PROTOBUF_NODISCARD ::object_detection::protos::BipartiteMatcher* release_bipartite_matcher();
  ::object_detection::protos::BipartiteMatcher* mutable_bipartite_matcher();
  void set_allocated_bipartite_matcher(::object_detection::protos::BipartiteMatcher* bipartite_matcher);
  private:
  const ::object_detection::protos::BipartiteMatcher& _internal_bipartite_matcher() const;
  ::object_detection::protos::BipartiteMatcher* _internal_mutable_bipartite_matcher();
  public:
  void unsafe_arena_set_allocated_bipartite_matcher(
      ::object_detection::protos::BipartiteMatcher* bipartite_matcher);
  ::object_detection::protos::BipartiteMatcher* unsafe_arena_release_bipartite_matcher();

  void clear_matcher_oneof();
  MatcherOneofCase matcher_oneof_case() const;
  // @@protoc_insertion_point(class_scope:object_detection.protos.Matcher)
 private:
  class _Internal;
  void set_has_argmax_matcher();
  void set_has_bipartite_matcher();

  inline bool has_matcher_oneof() const;
  inline void clear_has_matcher_oneof();

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    union MatcherOneofUnion {
      constexpr MatcherOneofUnion() : _constinit_{} {}
        ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized _constinit_;
      ::object_detection::protos::ArgMaxMatcher* argmax_matcher_;
      ::object_detection::protos::BipartiteMatcher* bipartite_matcher_;
    } matcher_oneof_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    uint32_t _oneof_case_[1];

  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fmatcher_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// Matcher

// .object_detection.protos.ArgMaxMatcher argmax_matcher = 1;
inline bool Matcher::_internal_has_argmax_matcher() const {
  return matcher_oneof_case() == kArgmaxMatcher;
}
inline bool Matcher::has_argmax_matcher() const {
  return _internal_has_argmax_matcher();
}
inline void Matcher::set_has_argmax_matcher() {
  _impl_._oneof_case_[0] = kArgmaxMatcher;
}
inline ::object_detection::protos::ArgMaxMatcher* Matcher::release_argmax_matcher() {
  // @@protoc_insertion_point(field_release:object_detection.protos.Matcher.argmax_matcher)
  if (_internal_has_argmax_matcher()) {
    clear_has_matcher_oneof();
    ::object_detection::protos::ArgMaxMatcher* temp = _impl_.matcher_oneof_.argmax_matcher_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.matcher_oneof_.argmax_matcher_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::object_detection::protos::ArgMaxMatcher& Matcher::_internal_argmax_matcher() const {
  return _internal_has_argmax_matcher()
      ? *_impl_.matcher_oneof_.argmax_matcher_
      : reinterpret_cast< ::object_detection::protos::ArgMaxMatcher&>(::object_detection::protos::_ArgMaxMatcher_default_instance_);
}
inline const ::object_detection::protos::ArgMaxMatcher& Matcher::argmax_matcher() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.Matcher.argmax_matcher)
  return _internal_argmax_matcher();
}
inline ::object_detection::protos::ArgMaxMatcher* Matcher::unsafe_arena_release_argmax_matcher() {
  // @@protoc_insertion_point(field_unsafe_arena_release:object_detection.protos.Matcher.argmax_matcher)
  if (_internal_has_argmax_matcher()) {
    clear_has_matcher_oneof();
    ::object_detection::protos::ArgMaxMatcher* temp = _impl_.matcher_oneof_.argmax_matcher_;
    _impl_.matcher_oneof_.argmax_matcher_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void Matcher::unsafe_arena_set_allocated_argmax_matcher(::object_detection::protos::ArgMaxMatcher* argmax_matcher) {
  clear_matcher_oneof();
  if (argmax_matcher) {
    set_has_argmax_matcher();
    _impl_.matcher_oneof_.argmax_matcher_ = argmax_matcher;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.Matcher.argmax_matcher)
}
inline ::object_detection::protos::ArgMaxMatcher* Matcher::_internal_mutable_argmax_matcher() {
  if (!_internal_has_argmax_matcher()) {
    clear_matcher_oneof();
    set_has_argmax_matcher();
    _impl_.matcher_oneof_.argmax_matcher_ = CreateMaybeMessage< ::object_detection::protos::ArgMaxMatcher >(GetArenaForAllocation());
  }
  return _impl_.matcher_oneof_.argmax_matcher_;
}
inline ::object_detection::protos::ArgMaxMatcher* Matcher::mutable_argmax_matcher() {
  ::object_detection::protos::ArgMaxMatcher* _msg = _internal_mutable_argmax_matcher();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.Matcher.argmax_matcher)
  return _msg;
}

// .object_detection.protos.BipartiteMatcher bipartite_matcher = 2;
inline bool Matcher::_internal_has_bipartite_matcher() const {
  return matcher_oneof_case() == kBipartiteMatcher;
}
inline bool Matcher::has_bipartite_matcher() const {
  return _internal_has_bipartite_matcher();
}
inline void Matcher::set_has_bipartite_matcher() {
  _impl_._oneof_case_[0] = kBipartiteMatcher;
}
inline ::object_detection::protos::BipartiteMatcher* Matcher::release_bipartite_matcher() {
  // @@protoc_insertion_point(field_release:object_detection.protos.Matcher.bipartite_matcher)
  if (_internal_has_bipartite_matcher()) {
    clear_has_matcher_oneof();
    ::object_detection::protos::BipartiteMatcher* temp = _impl_.matcher_oneof_.bipartite_matcher_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.matcher_oneof_.bipartite_matcher_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::object_detection::protos::BipartiteMatcher& Matcher::_internal_bipartite_matcher() const {
  return _internal_has_bipartite_matcher()
      ? *_impl_.matcher_oneof_.bipartite_matcher_
      : reinterpret_cast< ::object_detection::protos::BipartiteMatcher&>(::object_detection::protos::_BipartiteMatcher_default_instance_);
}
inline const ::object_detection::protos::BipartiteMatcher& Matcher::bipartite_matcher() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.Matcher.bipartite_matcher)
  return _internal_bipartite_matcher();
}
inline ::object_detection::protos::BipartiteMatcher* Matcher::unsafe_arena_release_bipartite_matcher() {
  // @@protoc_insertion_point(field_unsafe_arena_release:object_detection.protos.Matcher.bipartite_matcher)
  if (_internal_has_bipartite_matcher()) {
    clear_has_matcher_oneof();
    ::object_detection::protos::BipartiteMatcher* temp = _impl_.matcher_oneof_.bipartite_matcher_;
    _impl_.matcher_oneof_.bipartite_matcher_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void Matcher::unsafe_arena_set_allocated_bipartite_matcher(::object_detection::protos::BipartiteMatcher* bipartite_matcher) {
  clear_matcher_oneof();
  if (bipartite_matcher) {
    set_has_bipartite_matcher();
    _impl_.matcher_oneof_.bipartite_matcher_ = bipartite_matcher;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.Matcher.bipartite_matcher)
}
inline ::object_detection::protos::BipartiteMatcher* Matcher::_internal_mutable_bipartite_matcher() {
  if (!_internal_has_bipartite_matcher()) {
    clear_matcher_oneof();
    set_has_bipartite_matcher();
    _impl_.matcher_oneof_.bipartite_matcher_ = CreateMaybeMessage< ::object_detection::protos::BipartiteMatcher >(GetArenaForAllocation());
  }
  return _impl_.matcher_oneof_.bipartite_matcher_;
}
inline ::object_detection::protos::BipartiteMatcher* Matcher::mutable_bipartite_matcher() {
  ::object_detection::protos::BipartiteMatcher* _msg = _internal_mutable_bipartite_matcher();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.Matcher.bipartite_matcher)
  return _msg;
}

inline bool Matcher::has_matcher_oneof() const {
  return matcher_oneof_case() != MATCHER_ONEOF_NOT_SET;
}
inline void Matcher::clear_has_matcher_oneof() {
  _impl_._oneof_case_[0] = MATCHER_ONEOF_NOT_SET;
}
inline Matcher::MatcherOneofCase Matcher::matcher_oneof_case() const {
  return Matcher::MatcherOneofCase(_impl_._oneof_case_[0]);
}
#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace object_detection

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fmatcher_2eproto
