// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/input_reader.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2finput_5freader_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2finput_5freader_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "second/protos/target.pb.h"
#include "second/protos/preprocess.pb.h"
#include "second/protos/sampler.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_second_2fprotos_2finput_5freader_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_second_2fprotos_2finput_5freader_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_second_2fprotos_2finput_5freader_2eproto;
namespace second {
namespace protos {
class InputReader;
struct InputReaderDefaultTypeInternal;
extern InputReaderDefaultTypeInternal _InputReader_default_instance_;
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> ::second::protos::InputReader* Arena::CreateMaybeMessage<::second::protos::InputReader>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace second {
namespace protos {

// ===================================================================

class InputReader final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.InputReader) */ {
 public:
  inline InputReader() : InputReader(nullptr) {}
  ~InputReader() override;
  explicit PROTOBUF_CONSTEXPR InputReader(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  InputReader(const InputReader& from);
  InputReader(InputReader&& from) noexcept
    : InputReader() {
    *this = ::std::move(from);
  }

  inline InputReader& operator=(const InputReader& from) {
    CopyFrom(from);
    return *this;
  }
  inline InputReader& operator=(InputReader&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const InputReader& default_instance() {
    return *internal_default_instance();
  }
  static inline const InputReader* internal_default_instance() {
    return reinterpret_cast<const InputReader*>(
               &_InputReader_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(InputReader& a, InputReader& b) {
    a.Swap(&b);
  }
  inline void Swap(InputReader* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(InputReader* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  InputReader* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<InputReader>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const InputReader& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const InputReader& from) {
    InputReader::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(InputReader* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.InputReader";
  }
  protected:
  explicit InputReader(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kClassNamesFieldNumber = 2,
    kGroundtruthLocalizationNoiseStdFieldNumber = 11,
    kGroundtruthRotationUniformNoiseFieldNumber = 12,
    kGlobalRotationUniformNoiseFieldNumber = 13,
    kGlobalScalingUniformNoiseFieldNumber = 14,
    kGlobalRandomRotationRangePerObjectFieldNumber = 23,
    kDatabasePrepStepsFieldNumber = 24,
    kRecordFilePathFieldNumber = 1,
    kKittiInfoPathFieldNumber = 8,
    kKittiRootPathFieldNumber = 9,
    kTargetAssignerFieldNumber = 7,
    kDatabaseSamplerFieldNumber = 25,
    kUnlabeledDatabaseSamplerFieldNumber = 27,
    kBatchSizeFieldNumber = 3,
    kMaxNumEpochsFieldNumber = 4,
    kPrefetchSizeFieldNumber = 5,
    kMaxNumberOfVoxelsFieldNumber = 6,
    kNumWorkersFieldNumber = 16,
    kShufflePointsFieldNumber = 10,
    kRemoveUnknownExamplesFieldNumber = 15,
    kRemovePointsAfterSampleFieldNumber = 18,
    kRemoveEnvironmentFieldNumber = 21,
    kAnchorAreaThresholdFieldNumber = 17,
    kGroundtruthPointsDropPercentageFieldNumber = 19,
    kGroundtruthDropMaxKeepPointsFieldNumber = 20,
    kUnlabeledTrainingFieldNumber = 22,
    kUseGroupIdFieldNumber = 26,
  };
  // repeated string class_names = 2;
  int class_names_size() const;
  private:
  int _internal_class_names_size() const;
  public:
  void clear_class_names();
  const std::string& class_names(int index) const;
  std::string* mutable_class_names(int index);
  void set_class_names(int index, const std::string& value);
  void set_class_names(int index, std::string&& value);
  void set_class_names(int index, const char* value);
  void set_class_names(int index, const char* value, size_t size);
  std::string* add_class_names();
  void add_class_names(const std::string& value);
  void add_class_names(std::string&& value);
  void add_class_names(const char* value);
  void add_class_names(const char* value, size_t size);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>& class_names() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>* mutable_class_names();
  private:
  const std::string& _internal_class_names(int index) const;
  std::string* _internal_add_class_names();
  public:

  // repeated float groundtruth_localization_noise_std = 11;
  int groundtruth_localization_noise_std_size() const;
  private:
  int _internal_groundtruth_localization_noise_std_size() const;
  public:
  void clear_groundtruth_localization_noise_std();
  private:
  float _internal_groundtruth_localization_noise_std(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_groundtruth_localization_noise_std() const;
  void _internal_add_groundtruth_localization_noise_std(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_groundtruth_localization_noise_std();
  public:
  float groundtruth_localization_noise_std(int index) const;
  void set_groundtruth_localization_noise_std(int index, float value);
  void add_groundtruth_localization_noise_std(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      groundtruth_localization_noise_std() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_groundtruth_localization_noise_std();

  // repeated float groundtruth_rotation_uniform_noise = 12;
  int groundtruth_rotation_uniform_noise_size() const;
  private:
  int _internal_groundtruth_rotation_uniform_noise_size() const;
  public:
  void clear_groundtruth_rotation_uniform_noise();
  private:
  float _internal_groundtruth_rotation_uniform_noise(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_groundtruth_rotation_uniform_noise() const;
  void _internal_add_groundtruth_rotation_uniform_noise(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_groundtruth_rotation_uniform_noise();
  public:
  float groundtruth_rotation_uniform_noise(int index) const;
  void set_groundtruth_rotation_uniform_noise(int index, float value);
  void add_groundtruth_rotation_uniform_noise(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      groundtruth_rotation_uniform_noise() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_groundtruth_rotation_uniform_noise();

  // repeated float global_rotation_uniform_noise = 13;
  int global_rotation_uniform_noise_size() const;
  private:
  int _internal_global_rotation_uniform_noise_size() const;
  public:
  void clear_global_rotation_uniform_noise();
  private:
  float _internal_global_rotation_uniform_noise(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_global_rotation_uniform_noise() const;
  void _internal_add_global_rotation_uniform_noise(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_global_rotation_uniform_noise();
  public:
  float global_rotation_uniform_noise(int index) const;
  void set_global_rotation_uniform_noise(int index, float value);
  void add_global_rotation_uniform_noise(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      global_rotation_uniform_noise() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_global_rotation_uniform_noise();

  // repeated float global_scaling_uniform_noise = 14;
  int global_scaling_uniform_noise_size() const;
  private:
  int _internal_global_scaling_uniform_noise_size() const;
  public:
  void clear_global_scaling_uniform_noise();
  private:
  float _internal_global_scaling_uniform_noise(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_global_scaling_uniform_noise() const;
  void _internal_add_global_scaling_uniform_noise(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_global_scaling_uniform_noise();
  public:
  float global_scaling_uniform_noise(int index) const;
  void set_global_scaling_uniform_noise(int index, float value);
  void add_global_scaling_uniform_noise(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      global_scaling_uniform_noise() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_global_scaling_uniform_noise();

  // repeated float global_random_rotation_range_per_object = 23;
  int global_random_rotation_range_per_object_size() const;
  private:
  int _internal_global_random_rotation_range_per_object_size() const;
  public:
  void clear_global_random_rotation_range_per_object();
  private:
  float _internal_global_random_rotation_range_per_object(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_global_random_rotation_range_per_object() const;
  void _internal_add_global_random_rotation_range_per_object(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_global_random_rotation_range_per_object();
  public:
  float global_random_rotation_range_per_object(int index) const;
  void set_global_random_rotation_range_per_object(int index, float value);
  void add_global_random_rotation_range_per_object(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      global_random_rotation_range_per_object() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_global_random_rotation_range_per_object();

  // repeated .second.protos.DatabasePreprocessingStep database_prep_steps = 24;
  int database_prep_steps_size() const;
  private:
  int _internal_database_prep_steps_size() const;
  public:
  void clear_database_prep_steps();
  ::second::protos::DatabasePreprocessingStep* mutable_database_prep_steps(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::DatabasePreprocessingStep >*
      mutable_database_prep_steps();
  private:
  const ::second::protos::DatabasePreprocessingStep& _internal_database_prep_steps(int index) const;
  ::second::protos::DatabasePreprocessingStep* _internal_add_database_prep_steps();
  public:
  const ::second::protos::DatabasePreprocessingStep& database_prep_steps(int index) const;
  ::second::protos::DatabasePreprocessingStep* add_database_prep_steps();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::DatabasePreprocessingStep >&
      database_prep_steps() const;

  // string record_file_path = 1;
  void clear_record_file_path();
  const std::string& record_file_path() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_record_file_path(ArgT0&& arg0, ArgT... args);
  std::string* mutable_record_file_path();
  PROTOBUF_NODISCARD std::string* release_record_file_path();
  void set_allocated_record_file_path(std::string* record_file_path);
  private:
  const std::string& _internal_record_file_path() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_record_file_path(const std::string& value);
  std::string* _internal_mutable_record_file_path();
  public:

  // string kitti_info_path = 8;
  void clear_kitti_info_path();
  const std::string& kitti_info_path() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_kitti_info_path(ArgT0&& arg0, ArgT... args);
  std::string* mutable_kitti_info_path();
  PROTOBUF_NODISCARD std::string* release_kitti_info_path();
  void set_allocated_kitti_info_path(std::string* kitti_info_path);
  private:
  const std::string& _internal_kitti_info_path() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_kitti_info_path(const std::string& value);
  std::string* _internal_mutable_kitti_info_path();
  public:

  // string kitti_root_path = 9;
  void clear_kitti_root_path();
  const std::string& kitti_root_path() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_kitti_root_path(ArgT0&& arg0, ArgT... args);
  std::string* mutable_kitti_root_path();
  PROTOBUF_NODISCARD std::string* release_kitti_root_path();
  void set_allocated_kitti_root_path(std::string* kitti_root_path);
  private:
  const std::string& _internal_kitti_root_path() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_kitti_root_path(const std::string& value);
  std::string* _internal_mutable_kitti_root_path();
  public:

  // .second.protos.TargetAssigner target_assigner = 7;
  bool has_target_assigner() const;
  private:
  bool _internal_has_target_assigner() const;
  public:
  void clear_target_assigner();
  const ::second::protos::TargetAssigner& target_assigner() const;
  PROTOBUF_NODISCARD ::second::protos::TargetAssigner* release_target_assigner();
  ::second::protos::TargetAssigner* mutable_target_assigner();
  void set_allocated_target_assigner(::second::protos::TargetAssigner* target_assigner);
  private:
  const ::second::protos::TargetAssigner& _internal_target_assigner() const;
  ::second::protos::TargetAssigner* _internal_mutable_target_assigner();
  public:
  void unsafe_arena_set_allocated_target_assigner(
      ::second::protos::TargetAssigner* target_assigner);
  ::second::protos::TargetAssigner* unsafe_arena_release_target_assigner();

  // .second.protos.Sampler database_sampler = 25;
  bool has_database_sampler() const;
  private:
  bool _internal_has_database_sampler() const;
  public:
  void clear_database_sampler();
  const ::second::protos::Sampler& database_sampler() const;
  PROTOBUF_NODISCARD ::second::protos::Sampler* release_database_sampler();
  ::second::protos::Sampler* mutable_database_sampler();
  void set_allocated_database_sampler(::second::protos::Sampler* database_sampler);
  private:
  const ::second::protos::Sampler& _internal_database_sampler() const;
  ::second::protos::Sampler* _internal_mutable_database_sampler();
  public:
  void unsafe_arena_set_allocated_database_sampler(
      ::second::protos::Sampler* database_sampler);
  ::second::protos::Sampler* unsafe_arena_release_database_sampler();

  // .second.protos.Sampler unlabeled_database_sampler = 27;
  bool has_unlabeled_database_sampler() const;
  private:
  bool _internal_has_unlabeled_database_sampler() const;
  public:
  void clear_unlabeled_database_sampler();
  const ::second::protos::Sampler& unlabeled_database_sampler() const;
  PROTOBUF_NODISCARD ::second::protos::Sampler* release_unlabeled_database_sampler();
  ::second::protos::Sampler* mutable_unlabeled_database_sampler();
  void set_allocated_unlabeled_database_sampler(::second::protos::Sampler* unlabeled_database_sampler);
  private:
  const ::second::protos::Sampler& _internal_unlabeled_database_sampler() const;
  ::second::protos::Sampler* _internal_mutable_unlabeled_database_sampler();
  public:
  void unsafe_arena_set_allocated_unlabeled_database_sampler(
      ::second::protos::Sampler* unlabeled_database_sampler);
  ::second::protos::Sampler* unsafe_arena_release_unlabeled_database_sampler();

  // uint32 batch_size = 3;
  void clear_batch_size();
  uint32_t batch_size() const;
  void set_batch_size(uint32_t value);
  private:
  uint32_t _internal_batch_size() const;
  void _internal_set_batch_size(uint32_t value);
  public:

  // uint32 max_num_epochs = 4;
  void clear_max_num_epochs();
  uint32_t max_num_epochs() const;
  void set_max_num_epochs(uint32_t value);
  private:
  uint32_t _internal_max_num_epochs() const;
  void _internal_set_max_num_epochs(uint32_t value);
  public:

  // uint32 prefetch_size = 5;
  void clear_prefetch_size();
  uint32_t prefetch_size() const;
  void set_prefetch_size(uint32_t value);
  private:
  uint32_t _internal_prefetch_size() const;
  void _internal_set_prefetch_size(uint32_t value);
  public:

  // uint32 max_number_of_voxels = 6;
  void clear_max_number_of_voxels();
  uint32_t max_number_of_voxels() const;
  void set_max_number_of_voxels(uint32_t value);
  private:
  uint32_t _internal_max_number_of_voxels() const;
  void _internal_set_max_number_of_voxels(uint32_t value);
  public:

  // uint32 num_workers = 16;
  void clear_num_workers();
  uint32_t num_workers() const;
  void set_num_workers(uint32_t value);
  private:
  uint32_t _internal_num_workers() const;
  void _internal_set_num_workers(uint32_t value);
  public:

  // bool shuffle_points = 10;
  void clear_shuffle_points();
  bool shuffle_points() const;
  void set_shuffle_points(bool value);
  private:
  bool _internal_shuffle_points() const;
  void _internal_set_shuffle_points(bool value);
  public:

  // bool remove_unknown_examples = 15;
  void clear_remove_unknown_examples();
  bool remove_unknown_examples() const;
  void set_remove_unknown_examples(bool value);
  private:
  bool _internal_remove_unknown_examples() const;
  void _internal_set_remove_unknown_examples(bool value);
  public:

  // bool remove_points_after_sample = 18;
  void clear_remove_points_after_sample();
  bool remove_points_after_sample() const;
  void set_remove_points_after_sample(bool value);
  private:
  bool _internal_remove_points_after_sample() const;
  void _internal_set_remove_points_after_sample(bool value);
  public:

  // bool remove_environment = 21;
  void clear_remove_environment();
  bool remove_environment() const;
  void set_remove_environment(bool value);
  private:
  bool _internal_remove_environment() const;
  void _internal_set_remove_environment(bool value);
  public:

  // float anchor_area_threshold = 17;
  void clear_anchor_area_threshold();
  float anchor_area_threshold() const;
  void set_anchor_area_threshold(float value);
  private:
  float _internal_anchor_area_threshold() const;
  void _internal_set_anchor_area_threshold(float value);
  public:

  // float groundtruth_points_drop_percentage = 19;
  void clear_groundtruth_points_drop_percentage();
  float groundtruth_points_drop_percentage() const;
  void set_groundtruth_points_drop_percentage(float value);
  private:
  float _internal_groundtruth_points_drop_percentage() const;
  void _internal_set_groundtruth_points_drop_percentage(float value);
  public:

  // uint32 groundtruth_drop_max_keep_points = 20;
  void clear_groundtruth_drop_max_keep_points();
  uint32_t groundtruth_drop_max_keep_points() const;
  void set_groundtruth_drop_max_keep_points(uint32_t value);
  private:
  uint32_t _internal_groundtruth_drop_max_keep_points() const;
  void _internal_set_groundtruth_drop_max_keep_points(uint32_t value);
  public:

  // bool unlabeled_training = 22;
  void clear_unlabeled_training();
  bool unlabeled_training() const;
  void set_unlabeled_training(bool value);
  private:
  bool _internal_unlabeled_training() const;
  void _internal_set_unlabeled_training(bool value);
  public:

  // bool use_group_id = 26;
  void clear_use_group_id();
  bool use_group_id() const;
  void set_use_group_id(bool value);
  private:
  bool _internal_use_group_id() const;
  void _internal_set_use_group_id(bool value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.InputReader)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string> class_names_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > groundtruth_localization_noise_std_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > groundtruth_rotation_uniform_noise_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > global_rotation_uniform_noise_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > global_scaling_uniform_noise_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > global_random_rotation_range_per_object_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::DatabasePreprocessingStep > database_prep_steps_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr record_file_path_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr kitti_info_path_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr kitti_root_path_;
    ::second::protos::TargetAssigner* target_assigner_;
    ::second::protos::Sampler* database_sampler_;
    ::second::protos::Sampler* unlabeled_database_sampler_;
    uint32_t batch_size_;
    uint32_t max_num_epochs_;
    uint32_t prefetch_size_;
    uint32_t max_number_of_voxels_;
    uint32_t num_workers_;
    bool shuffle_points_;
    bool remove_unknown_examples_;
    bool remove_points_after_sample_;
    bool remove_environment_;
    float anchor_area_threshold_;
    float groundtruth_points_drop_percentage_;
    uint32_t groundtruth_drop_max_keep_points_;
    bool unlabeled_training_;
    bool use_group_id_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2finput_5freader_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// InputReader

// string record_file_path = 1;
inline void InputReader::clear_record_file_path() {
  _impl_.record_file_path_.ClearToEmpty();
}
inline const std::string& InputReader::record_file_path() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.record_file_path)
  return _internal_record_file_path();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void InputReader::set_record_file_path(ArgT0&& arg0, ArgT... args) {
 
 _impl_.record_file_path_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:second.protos.InputReader.record_file_path)
}
inline std::string* InputReader::mutable_record_file_path() {
  std::string* _s = _internal_mutable_record_file_path();
  // @@protoc_insertion_point(field_mutable:second.protos.InputReader.record_file_path)
  return _s;
}
inline const std::string& InputReader::_internal_record_file_path() const {
  return _impl_.record_file_path_.Get();
}
inline void InputReader::_internal_set_record_file_path(const std::string& value) {
  
  _impl_.record_file_path_.Set(value, GetArenaForAllocation());
}
inline std::string* InputReader::_internal_mutable_record_file_path() {
  
  return _impl_.record_file_path_.Mutable(GetArenaForAllocation());
}
inline std::string* InputReader::release_record_file_path() {
  // @@protoc_insertion_point(field_release:second.protos.InputReader.record_file_path)
  return _impl_.record_file_path_.Release();
}
inline void InputReader::set_allocated_record_file_path(std::string* record_file_path) {
  if (record_file_path != nullptr) {
    
  } else {
    
  }
  _impl_.record_file_path_.SetAllocated(record_file_path, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.record_file_path_.IsDefault()) {
    _impl_.record_file_path_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:second.protos.InputReader.record_file_path)
}

// repeated string class_names = 2;
inline int InputReader::_internal_class_names_size() const {
  return _impl_.class_names_.size();
}
inline int InputReader::class_names_size() const {
  return _internal_class_names_size();
}
inline void InputReader::clear_class_names() {
  _impl_.class_names_.Clear();
}
inline std::string* InputReader::add_class_names() {
  std::string* _s = _internal_add_class_names();
  // @@protoc_insertion_point(field_add_mutable:second.protos.InputReader.class_names)
  return _s;
}
inline const std::string& InputReader::_internal_class_names(int index) const {
  return _impl_.class_names_.Get(index);
}
inline const std::string& InputReader::class_names(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.class_names)
  return _internal_class_names(index);
}
inline std::string* InputReader::mutable_class_names(int index) {
  // @@protoc_insertion_point(field_mutable:second.protos.InputReader.class_names)
  return _impl_.class_names_.Mutable(index);
}
inline void InputReader::set_class_names(int index, const std::string& value) {
  _impl_.class_names_.Mutable(index)->assign(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.class_names)
}
inline void InputReader::set_class_names(int index, std::string&& value) {
  _impl_.class_names_.Mutable(index)->assign(std::move(value));
  // @@protoc_insertion_point(field_set:second.protos.InputReader.class_names)
}
inline void InputReader::set_class_names(int index, const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  _impl_.class_names_.Mutable(index)->assign(value);
  // @@protoc_insertion_point(field_set_char:second.protos.InputReader.class_names)
}
inline void InputReader::set_class_names(int index, const char* value, size_t size) {
  _impl_.class_names_.Mutable(index)->assign(
    reinterpret_cast<const char*>(value), size);
  // @@protoc_insertion_point(field_set_pointer:second.protos.InputReader.class_names)
}
inline std::string* InputReader::_internal_add_class_names() {
  return _impl_.class_names_.Add();
}
inline void InputReader::add_class_names(const std::string& value) {
  _impl_.class_names_.Add()->assign(value);
  // @@protoc_insertion_point(field_add:second.protos.InputReader.class_names)
}
inline void InputReader::add_class_names(std::string&& value) {
  _impl_.class_names_.Add(std::move(value));
  // @@protoc_insertion_point(field_add:second.protos.InputReader.class_names)
}
inline void InputReader::add_class_names(const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  _impl_.class_names_.Add()->assign(value);
  // @@protoc_insertion_point(field_add_char:second.protos.InputReader.class_names)
}
inline void InputReader::add_class_names(const char* value, size_t size) {
  _impl_.class_names_.Add()->assign(reinterpret_cast<const char*>(value), size);
  // @@protoc_insertion_point(field_add_pointer:second.protos.InputReader.class_names)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>&
InputReader::class_names() const {
  // @@protoc_insertion_point(field_list:second.protos.InputReader.class_names)
  return _impl_.class_names_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>*
InputReader::mutable_class_names() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.InputReader.class_names)
  return &_impl_.class_names_;
}

// uint32 batch_size = 3;
inline void InputReader::clear_batch_size() {
  _impl_.batch_size_ = 0u;
}
inline uint32_t InputReader::_internal_batch_size() const {
  return _impl_.batch_size_;
}
inline uint32_t InputReader::batch_size() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.batch_size)
  return _internal_batch_size();
}
inline void InputReader::_internal_set_batch_size(uint32_t value) {
  
  _impl_.batch_size_ = value;
}
inline void InputReader::set_batch_size(uint32_t value) {
  _internal_set_batch_size(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.batch_size)
}

// uint32 max_num_epochs = 4;
inline void InputReader::clear_max_num_epochs() {
  _impl_.max_num_epochs_ = 0u;
}
inline uint32_t InputReader::_internal_max_num_epochs() const {
  return _impl_.max_num_epochs_;
}
inline uint32_t InputReader::max_num_epochs() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.max_num_epochs)
  return _internal_max_num_epochs();
}
inline void InputReader::_internal_set_max_num_epochs(uint32_t value) {
  
  _impl_.max_num_epochs_ = value;
}
inline void InputReader::set_max_num_epochs(uint32_t value) {
  _internal_set_max_num_epochs(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.max_num_epochs)
}

// uint32 prefetch_size = 5;
inline void InputReader::clear_prefetch_size() {
  _impl_.prefetch_size_ = 0u;
}
inline uint32_t InputReader::_internal_prefetch_size() const {
  return _impl_.prefetch_size_;
}
inline uint32_t InputReader::prefetch_size() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.prefetch_size)
  return _internal_prefetch_size();
}
inline void InputReader::_internal_set_prefetch_size(uint32_t value) {
  
  _impl_.prefetch_size_ = value;
}
inline void InputReader::set_prefetch_size(uint32_t value) {
  _internal_set_prefetch_size(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.prefetch_size)
}

// uint32 max_number_of_voxels = 6;
inline void InputReader::clear_max_number_of_voxels() {
  _impl_.max_number_of_voxels_ = 0u;
}
inline uint32_t InputReader::_internal_max_number_of_voxels() const {
  return _impl_.max_number_of_voxels_;
}
inline uint32_t InputReader::max_number_of_voxels() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.max_number_of_voxels)
  return _internal_max_number_of_voxels();
}
inline void InputReader::_internal_set_max_number_of_voxels(uint32_t value) {
  
  _impl_.max_number_of_voxels_ = value;
}
inline void InputReader::set_max_number_of_voxels(uint32_t value) {
  _internal_set_max_number_of_voxels(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.max_number_of_voxels)
}

// .second.protos.TargetAssigner target_assigner = 7;
inline bool InputReader::_internal_has_target_assigner() const {
  return this != internal_default_instance() && _impl_.target_assigner_ != nullptr;
}
inline bool InputReader::has_target_assigner() const {
  return _internal_has_target_assigner();
}
inline const ::second::protos::TargetAssigner& InputReader::_internal_target_assigner() const {
  const ::second::protos::TargetAssigner* p = _impl_.target_assigner_;
  return p != nullptr ? *p : reinterpret_cast<const ::second::protos::TargetAssigner&>(
      ::second::protos::_TargetAssigner_default_instance_);
}
inline const ::second::protos::TargetAssigner& InputReader::target_assigner() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.target_assigner)
  return _internal_target_assigner();
}
inline void InputReader::unsafe_arena_set_allocated_target_assigner(
    ::second::protos::TargetAssigner* target_assigner) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.target_assigner_);
  }
  _impl_.target_assigner_ = target_assigner;
  if (target_assigner) {
    
  } else {
    
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.InputReader.target_assigner)
}
inline ::second::protos::TargetAssigner* InputReader::release_target_assigner() {
  
  ::second::protos::TargetAssigner* temp = _impl_.target_assigner_;
  _impl_.target_assigner_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::second::protos::TargetAssigner* InputReader::unsafe_arena_release_target_assigner() {
  // @@protoc_insertion_point(field_release:second.protos.InputReader.target_assigner)
  
  ::second::protos::TargetAssigner* temp = _impl_.target_assigner_;
  _impl_.target_assigner_ = nullptr;
  return temp;
}
inline ::second::protos::TargetAssigner* InputReader::_internal_mutable_target_assigner() {
  
  if (_impl_.target_assigner_ == nullptr) {
    auto* p = CreateMaybeMessage<::second::protos::TargetAssigner>(GetArenaForAllocation());
    _impl_.target_assigner_ = p;
  }
  return _impl_.target_assigner_;
}
inline ::second::protos::TargetAssigner* InputReader::mutable_target_assigner() {
  ::second::protos::TargetAssigner* _msg = _internal_mutable_target_assigner();
  // @@protoc_insertion_point(field_mutable:second.protos.InputReader.target_assigner)
  return _msg;
}
inline void InputReader::set_allocated_target_assigner(::second::protos::TargetAssigner* target_assigner) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.target_assigner_);
  }
  if (target_assigner) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(target_assigner));
    if (message_arena != submessage_arena) {
      target_assigner = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, target_assigner, submessage_arena);
    }
    
  } else {
    
  }
  _impl_.target_assigner_ = target_assigner;
  // @@protoc_insertion_point(field_set_allocated:second.protos.InputReader.target_assigner)
}

// string kitti_info_path = 8;
inline void InputReader::clear_kitti_info_path() {
  _impl_.kitti_info_path_.ClearToEmpty();
}
inline const std::string& InputReader::kitti_info_path() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.kitti_info_path)
  return _internal_kitti_info_path();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void InputReader::set_kitti_info_path(ArgT0&& arg0, ArgT... args) {
 
 _impl_.kitti_info_path_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:second.protos.InputReader.kitti_info_path)
}
inline std::string* InputReader::mutable_kitti_info_path() {
  std::string* _s = _internal_mutable_kitti_info_path();
  // @@protoc_insertion_point(field_mutable:second.protos.InputReader.kitti_info_path)
  return _s;
}
inline const std::string& InputReader::_internal_kitti_info_path() const {
  return _impl_.kitti_info_path_.Get();
}
inline void InputReader::_internal_set_kitti_info_path(const std::string& value) {
  
  _impl_.kitti_info_path_.Set(value, GetArenaForAllocation());
}
inline std::string* InputReader::_internal_mutable_kitti_info_path() {
  
  return _impl_.kitti_info_path_.Mutable(GetArenaForAllocation());
}
inline std::string* InputReader::release_kitti_info_path() {
  // @@protoc_insertion_point(field_release:second.protos.InputReader.kitti_info_path)
  return _impl_.kitti_info_path_.Release();
}
inline void InputReader::set_allocated_kitti_info_path(std::string* kitti_info_path) {
  if (kitti_info_path != nullptr) {
    
  } else {
    
  }
  _impl_.kitti_info_path_.SetAllocated(kitti_info_path, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.kitti_info_path_.IsDefault()) {
    _impl_.kitti_info_path_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:second.protos.InputReader.kitti_info_path)
}

// string kitti_root_path = 9;
inline void InputReader::clear_kitti_root_path() {
  _impl_.kitti_root_path_.ClearToEmpty();
}
inline const std::string& InputReader::kitti_root_path() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.kitti_root_path)
  return _internal_kitti_root_path();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void InputReader::set_kitti_root_path(ArgT0&& arg0, ArgT... args) {
 
 _impl_.kitti_root_path_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:second.protos.InputReader.kitti_root_path)
}
inline std::string* InputReader::mutable_kitti_root_path() {
  std::string* _s = _internal_mutable_kitti_root_path();
  // @@protoc_insertion_point(field_mutable:second.protos.InputReader.kitti_root_path)
  return _s;
}
inline const std::string& InputReader::_internal_kitti_root_path() const {
  return _impl_.kitti_root_path_.Get();
}
inline void InputReader::_internal_set_kitti_root_path(const std::string& value) {
  
  _impl_.kitti_root_path_.Set(value, GetArenaForAllocation());
}
inline std::string* InputReader::_internal_mutable_kitti_root_path() {
  
  return _impl_.kitti_root_path_.Mutable(GetArenaForAllocation());
}
inline std::string* InputReader::release_kitti_root_path() {
  // @@protoc_insertion_point(field_release:second.protos.InputReader.kitti_root_path)
  return _impl_.kitti_root_path_.Release();
}
inline void InputReader::set_allocated_kitti_root_path(std::string* kitti_root_path) {
  if (kitti_root_path != nullptr) {
    
  } else {
    
  }
  _impl_.kitti_root_path_.SetAllocated(kitti_root_path, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.kitti_root_path_.IsDefault()) {
    _impl_.kitti_root_path_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:second.protos.InputReader.kitti_root_path)
}

// bool shuffle_points = 10;
inline void InputReader::clear_shuffle_points() {
  _impl_.shuffle_points_ = false;
}
inline bool InputReader::_internal_shuffle_points() const {
  return _impl_.shuffle_points_;
}
inline bool InputReader::shuffle_points() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.shuffle_points)
  return _internal_shuffle_points();
}
inline void InputReader::_internal_set_shuffle_points(bool value) {
  
  _impl_.shuffle_points_ = value;
}
inline void InputReader::set_shuffle_points(bool value) {
  _internal_set_shuffle_points(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.shuffle_points)
}

// repeated float groundtruth_localization_noise_std = 11;
inline int InputReader::_internal_groundtruth_localization_noise_std_size() const {
  return _impl_.groundtruth_localization_noise_std_.size();
}
inline int InputReader::groundtruth_localization_noise_std_size() const {
  return _internal_groundtruth_localization_noise_std_size();
}
inline void InputReader::clear_groundtruth_localization_noise_std() {
  _impl_.groundtruth_localization_noise_std_.Clear();
}
inline float InputReader::_internal_groundtruth_localization_noise_std(int index) const {
  return _impl_.groundtruth_localization_noise_std_.Get(index);
}
inline float InputReader::groundtruth_localization_noise_std(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.groundtruth_localization_noise_std)
  return _internal_groundtruth_localization_noise_std(index);
}
inline void InputReader::set_groundtruth_localization_noise_std(int index, float value) {
  _impl_.groundtruth_localization_noise_std_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.groundtruth_localization_noise_std)
}
inline void InputReader::_internal_add_groundtruth_localization_noise_std(float value) {
  _impl_.groundtruth_localization_noise_std_.Add(value);
}
inline void InputReader::add_groundtruth_localization_noise_std(float value) {
  _internal_add_groundtruth_localization_noise_std(value);
  // @@protoc_insertion_point(field_add:second.protos.InputReader.groundtruth_localization_noise_std)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
InputReader::_internal_groundtruth_localization_noise_std() const {
  return _impl_.groundtruth_localization_noise_std_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
InputReader::groundtruth_localization_noise_std() const {
  // @@protoc_insertion_point(field_list:second.protos.InputReader.groundtruth_localization_noise_std)
  return _internal_groundtruth_localization_noise_std();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
InputReader::_internal_mutable_groundtruth_localization_noise_std() {
  return &_impl_.groundtruth_localization_noise_std_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
InputReader::mutable_groundtruth_localization_noise_std() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.InputReader.groundtruth_localization_noise_std)
  return _internal_mutable_groundtruth_localization_noise_std();
}

// repeated float groundtruth_rotation_uniform_noise = 12;
inline int InputReader::_internal_groundtruth_rotation_uniform_noise_size() const {
  return _impl_.groundtruth_rotation_uniform_noise_.size();
}
inline int InputReader::groundtruth_rotation_uniform_noise_size() const {
  return _internal_groundtruth_rotation_uniform_noise_size();
}
inline void InputReader::clear_groundtruth_rotation_uniform_noise() {
  _impl_.groundtruth_rotation_uniform_noise_.Clear();
}
inline float InputReader::_internal_groundtruth_rotation_uniform_noise(int index) const {
  return _impl_.groundtruth_rotation_uniform_noise_.Get(index);
}
inline float InputReader::groundtruth_rotation_uniform_noise(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.groundtruth_rotation_uniform_noise)
  return _internal_groundtruth_rotation_uniform_noise(index);
}
inline void InputReader::set_groundtruth_rotation_uniform_noise(int index, float value) {
  _impl_.groundtruth_rotation_uniform_noise_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.groundtruth_rotation_uniform_noise)
}
inline void InputReader::_internal_add_groundtruth_rotation_uniform_noise(float value) {
  _impl_.groundtruth_rotation_uniform_noise_.Add(value);
}
inline void InputReader::add_groundtruth_rotation_uniform_noise(float value) {
  _internal_add_groundtruth_rotation_uniform_noise(value);
  // @@protoc_insertion_point(field_add:second.protos.InputReader.groundtruth_rotation_uniform_noise)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
InputReader::_internal_groundtruth_rotation_uniform_noise() const {
  return _impl_.groundtruth_rotation_uniform_noise_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
InputReader::groundtruth_rotation_uniform_noise() const {
  // @@protoc_insertion_point(field_list:second.protos.InputReader.groundtruth_rotation_uniform_noise)
  return _internal_groundtruth_rotation_uniform_noise();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
InputReader::_internal_mutable_groundtruth_rotation_uniform_noise() {
  return &_impl_.groundtruth_rotation_uniform_noise_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
InputReader::mutable_groundtruth_rotation_uniform_noise() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.InputReader.groundtruth_rotation_uniform_noise)
  return _internal_mutable_groundtruth_rotation_uniform_noise();
}

// repeated float global_rotation_uniform_noise = 13;
inline int InputReader::_internal_global_rotation_uniform_noise_size() const {
  return _impl_.global_rotation_uniform_noise_.size();
}
inline int InputReader::global_rotation_uniform_noise_size() const {
  return _internal_global_rotation_uniform_noise_size();
}
inline void InputReader::clear_global_rotation_uniform_noise() {
  _impl_.global_rotation_uniform_noise_.Clear();
}
inline float InputReader::_internal_global_rotation_uniform_noise(int index) const {
  return _impl_.global_rotation_uniform_noise_.Get(index);
}
inline float InputReader::global_rotation_uniform_noise(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.global_rotation_uniform_noise)
  return _internal_global_rotation_uniform_noise(index);
}
inline void InputReader::set_global_rotation_uniform_noise(int index, float value) {
  _impl_.global_rotation_uniform_noise_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.global_rotation_uniform_noise)
}
inline void InputReader::_internal_add_global_rotation_uniform_noise(float value) {
  _impl_.global_rotation_uniform_noise_.Add(value);
}
inline void InputReader::add_global_rotation_uniform_noise(float value) {
  _internal_add_global_rotation_uniform_noise(value);
  // @@protoc_insertion_point(field_add:second.protos.InputReader.global_rotation_uniform_noise)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
InputReader::_internal_global_rotation_uniform_noise() const {
  return _impl_.global_rotation_uniform_noise_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
InputReader::global_rotation_uniform_noise() const {
  // @@protoc_insertion_point(field_list:second.protos.InputReader.global_rotation_uniform_noise)
  return _internal_global_rotation_uniform_noise();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
InputReader::_internal_mutable_global_rotation_uniform_noise() {
  return &_impl_.global_rotation_uniform_noise_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
InputReader::mutable_global_rotation_uniform_noise() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.InputReader.global_rotation_uniform_noise)
  return _internal_mutable_global_rotation_uniform_noise();
}

// repeated float global_scaling_uniform_noise = 14;
inline int InputReader::_internal_global_scaling_uniform_noise_size() const {
  return _impl_.global_scaling_uniform_noise_.size();
}
inline int InputReader::global_scaling_uniform_noise_size() const {
  return _internal_global_scaling_uniform_noise_size();
}
inline void InputReader::clear_global_scaling_uniform_noise() {
  _impl_.global_scaling_uniform_noise_.Clear();
}
inline float InputReader::_internal_global_scaling_uniform_noise(int index) const {
  return _impl_.global_scaling_uniform_noise_.Get(index);
}
inline float InputReader::global_scaling_uniform_noise(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.global_scaling_uniform_noise)
  return _internal_global_scaling_uniform_noise(index);
}
inline void InputReader::set_global_scaling_uniform_noise(int index, float value) {
  _impl_.global_scaling_uniform_noise_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.global_scaling_uniform_noise)
}
inline void InputReader::_internal_add_global_scaling_uniform_noise(float value) {
  _impl_.global_scaling_uniform_noise_.Add(value);
}
inline void InputReader::add_global_scaling_uniform_noise(float value) {
  _internal_add_global_scaling_uniform_noise(value);
  // @@protoc_insertion_point(field_add:second.protos.InputReader.global_scaling_uniform_noise)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
InputReader::_internal_global_scaling_uniform_noise() const {
  return _impl_.global_scaling_uniform_noise_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
InputReader::global_scaling_uniform_noise() const {
  // @@protoc_insertion_point(field_list:second.protos.InputReader.global_scaling_uniform_noise)
  return _internal_global_scaling_uniform_noise();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
InputReader::_internal_mutable_global_scaling_uniform_noise() {
  return &_impl_.global_scaling_uniform_noise_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
InputReader::mutable_global_scaling_uniform_noise() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.InputReader.global_scaling_uniform_noise)
  return _internal_mutable_global_scaling_uniform_noise();
}

// bool remove_unknown_examples = 15;
inline void InputReader::clear_remove_unknown_examples() {
  _impl_.remove_unknown_examples_ = false;
}
inline bool InputReader::_internal_remove_unknown_examples() const {
  return _impl_.remove_unknown_examples_;
}
inline bool InputReader::remove_unknown_examples() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.remove_unknown_examples)
  return _internal_remove_unknown_examples();
}
inline void InputReader::_internal_set_remove_unknown_examples(bool value) {
  
  _impl_.remove_unknown_examples_ = value;
}
inline void InputReader::set_remove_unknown_examples(bool value) {
  _internal_set_remove_unknown_examples(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.remove_unknown_examples)
}

// uint32 num_workers = 16;
inline void InputReader::clear_num_workers() {
  _impl_.num_workers_ = 0u;
}
inline uint32_t InputReader::_internal_num_workers() const {
  return _impl_.num_workers_;
}
inline uint32_t InputReader::num_workers() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.num_workers)
  return _internal_num_workers();
}
inline void InputReader::_internal_set_num_workers(uint32_t value) {
  
  _impl_.num_workers_ = value;
}
inline void InputReader::set_num_workers(uint32_t value) {
  _internal_set_num_workers(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.num_workers)
}

// float anchor_area_threshold = 17;
inline void InputReader::clear_anchor_area_threshold() {
  _impl_.anchor_area_threshold_ = 0;
}
inline float InputReader::_internal_anchor_area_threshold() const {
  return _impl_.anchor_area_threshold_;
}
inline float InputReader::anchor_area_threshold() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.anchor_area_threshold)
  return _internal_anchor_area_threshold();
}
inline void InputReader::_internal_set_anchor_area_threshold(float value) {
  
  _impl_.anchor_area_threshold_ = value;
}
inline void InputReader::set_anchor_area_threshold(float value) {
  _internal_set_anchor_area_threshold(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.anchor_area_threshold)
}

// bool remove_points_after_sample = 18;
inline void InputReader::clear_remove_points_after_sample() {
  _impl_.remove_points_after_sample_ = false;
}
inline bool InputReader::_internal_remove_points_after_sample() const {
  return _impl_.remove_points_after_sample_;
}
inline bool InputReader::remove_points_after_sample() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.remove_points_after_sample)
  return _internal_remove_points_after_sample();
}
inline void InputReader::_internal_set_remove_points_after_sample(bool value) {
  
  _impl_.remove_points_after_sample_ = value;
}
inline void InputReader::set_remove_points_after_sample(bool value) {
  _internal_set_remove_points_after_sample(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.remove_points_after_sample)
}

// float groundtruth_points_drop_percentage = 19;
inline void InputReader::clear_groundtruth_points_drop_percentage() {
  _impl_.groundtruth_points_drop_percentage_ = 0;
}
inline float InputReader::_internal_groundtruth_points_drop_percentage() const {
  return _impl_.groundtruth_points_drop_percentage_;
}
inline float InputReader::groundtruth_points_drop_percentage() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.groundtruth_points_drop_percentage)
  return _internal_groundtruth_points_drop_percentage();
}
inline void InputReader::_internal_set_groundtruth_points_drop_percentage(float value) {
  
  _impl_.groundtruth_points_drop_percentage_ = value;
}
inline void InputReader::set_groundtruth_points_drop_percentage(float value) {
  _internal_set_groundtruth_points_drop_percentage(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.groundtruth_points_drop_percentage)
}

// uint32 groundtruth_drop_max_keep_points = 20;
inline void InputReader::clear_groundtruth_drop_max_keep_points() {
  _impl_.groundtruth_drop_max_keep_points_ = 0u;
}
inline uint32_t InputReader::_internal_groundtruth_drop_max_keep_points() const {
  return _impl_.groundtruth_drop_max_keep_points_;
}
inline uint32_t InputReader::groundtruth_drop_max_keep_points() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.groundtruth_drop_max_keep_points)
  return _internal_groundtruth_drop_max_keep_points();
}
inline void InputReader::_internal_set_groundtruth_drop_max_keep_points(uint32_t value) {
  
  _impl_.groundtruth_drop_max_keep_points_ = value;
}
inline void InputReader::set_groundtruth_drop_max_keep_points(uint32_t value) {
  _internal_set_groundtruth_drop_max_keep_points(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.groundtruth_drop_max_keep_points)
}

// bool remove_environment = 21;
inline void InputReader::clear_remove_environment() {
  _impl_.remove_environment_ = false;
}
inline bool InputReader::_internal_remove_environment() const {
  return _impl_.remove_environment_;
}
inline bool InputReader::remove_environment() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.remove_environment)
  return _internal_remove_environment();
}
inline void InputReader::_internal_set_remove_environment(bool value) {
  
  _impl_.remove_environment_ = value;
}
inline void InputReader::set_remove_environment(bool value) {
  _internal_set_remove_environment(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.remove_environment)
}

// bool unlabeled_training = 22;
inline void InputReader::clear_unlabeled_training() {
  _impl_.unlabeled_training_ = false;
}
inline bool InputReader::_internal_unlabeled_training() const {
  return _impl_.unlabeled_training_;
}
inline bool InputReader::unlabeled_training() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.unlabeled_training)
  return _internal_unlabeled_training();
}
inline void InputReader::_internal_set_unlabeled_training(bool value) {
  
  _impl_.unlabeled_training_ = value;
}
inline void InputReader::set_unlabeled_training(bool value) {
  _internal_set_unlabeled_training(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.unlabeled_training)
}

// repeated float global_random_rotation_range_per_object = 23;
inline int InputReader::_internal_global_random_rotation_range_per_object_size() const {
  return _impl_.global_random_rotation_range_per_object_.size();
}
inline int InputReader::global_random_rotation_range_per_object_size() const {
  return _internal_global_random_rotation_range_per_object_size();
}
inline void InputReader::clear_global_random_rotation_range_per_object() {
  _impl_.global_random_rotation_range_per_object_.Clear();
}
inline float InputReader::_internal_global_random_rotation_range_per_object(int index) const {
  return _impl_.global_random_rotation_range_per_object_.Get(index);
}
inline float InputReader::global_random_rotation_range_per_object(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.global_random_rotation_range_per_object)
  return _internal_global_random_rotation_range_per_object(index);
}
inline void InputReader::set_global_random_rotation_range_per_object(int index, float value) {
  _impl_.global_random_rotation_range_per_object_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.global_random_rotation_range_per_object)
}
inline void InputReader::_internal_add_global_random_rotation_range_per_object(float value) {
  _impl_.global_random_rotation_range_per_object_.Add(value);
}
inline void InputReader::add_global_random_rotation_range_per_object(float value) {
  _internal_add_global_random_rotation_range_per_object(value);
  // @@protoc_insertion_point(field_add:second.protos.InputReader.global_random_rotation_range_per_object)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
InputReader::_internal_global_random_rotation_range_per_object() const {
  return _impl_.global_random_rotation_range_per_object_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
InputReader::global_random_rotation_range_per_object() const {
  // @@protoc_insertion_point(field_list:second.protos.InputReader.global_random_rotation_range_per_object)
  return _internal_global_random_rotation_range_per_object();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
InputReader::_internal_mutable_global_random_rotation_range_per_object() {
  return &_impl_.global_random_rotation_range_per_object_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
InputReader::mutable_global_random_rotation_range_per_object() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.InputReader.global_random_rotation_range_per_object)
  return _internal_mutable_global_random_rotation_range_per_object();
}

// repeated .second.protos.DatabasePreprocessingStep database_prep_steps = 24;
inline int InputReader::_internal_database_prep_steps_size() const {
  return _impl_.database_prep_steps_.size();
}
inline int InputReader::database_prep_steps_size() const {
  return _internal_database_prep_steps_size();
}
inline ::second::protos::DatabasePreprocessingStep* InputReader::mutable_database_prep_steps(int index) {
  // @@protoc_insertion_point(field_mutable:second.protos.InputReader.database_prep_steps)
  return _impl_.database_prep_steps_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::DatabasePreprocessingStep >*
InputReader::mutable_database_prep_steps() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.InputReader.database_prep_steps)
  return &_impl_.database_prep_steps_;
}
inline const ::second::protos::DatabasePreprocessingStep& InputReader::_internal_database_prep_steps(int index) const {
  return _impl_.database_prep_steps_.Get(index);
}
inline const ::second::protos::DatabasePreprocessingStep& InputReader::database_prep_steps(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.database_prep_steps)
  return _internal_database_prep_steps(index);
}
inline ::second::protos::DatabasePreprocessingStep* InputReader::_internal_add_database_prep_steps() {
  return _impl_.database_prep_steps_.Add();
}
inline ::second::protos::DatabasePreprocessingStep* InputReader::add_database_prep_steps() {
  ::second::protos::DatabasePreprocessingStep* _add = _internal_add_database_prep_steps();
  // @@protoc_insertion_point(field_add:second.protos.InputReader.database_prep_steps)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::DatabasePreprocessingStep >&
InputReader::database_prep_steps() const {
  // @@protoc_insertion_point(field_list:second.protos.InputReader.database_prep_steps)
  return _impl_.database_prep_steps_;
}

// .second.protos.Sampler database_sampler = 25;
inline bool InputReader::_internal_has_database_sampler() const {
  return this != internal_default_instance() && _impl_.database_sampler_ != nullptr;
}
inline bool InputReader::has_database_sampler() const {
  return _internal_has_database_sampler();
}
inline const ::second::protos::Sampler& InputReader::_internal_database_sampler() const {
  const ::second::protos::Sampler* p = _impl_.database_sampler_;
  return p != nullptr ? *p : reinterpret_cast<const ::second::protos::Sampler&>(
      ::second::protos::_Sampler_default_instance_);
}
inline const ::second::protos::Sampler& InputReader::database_sampler() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.database_sampler)
  return _internal_database_sampler();
}
inline void InputReader::unsafe_arena_set_allocated_database_sampler(
    ::second::protos::Sampler* database_sampler) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.database_sampler_);
  }
  _impl_.database_sampler_ = database_sampler;
  if (database_sampler) {
    
  } else {
    
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.InputReader.database_sampler)
}
inline ::second::protos::Sampler* InputReader::release_database_sampler() {
  
  ::second::protos::Sampler* temp = _impl_.database_sampler_;
  _impl_.database_sampler_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::second::protos::Sampler* InputReader::unsafe_arena_release_database_sampler() {
  // @@protoc_insertion_point(field_release:second.protos.InputReader.database_sampler)
  
  ::second::protos::Sampler* temp = _impl_.database_sampler_;
  _impl_.database_sampler_ = nullptr;
  return temp;
}
inline ::second::protos::Sampler* InputReader::_internal_mutable_database_sampler() {
  
  if (_impl_.database_sampler_ == nullptr) {
    auto* p = CreateMaybeMessage<::second::protos::Sampler>(GetArenaForAllocation());
    _impl_.database_sampler_ = p;
  }
  return _impl_.database_sampler_;
}
inline ::second::protos::Sampler* InputReader::mutable_database_sampler() {
  ::second::protos::Sampler* _msg = _internal_mutable_database_sampler();
  // @@protoc_insertion_point(field_mutable:second.protos.InputReader.database_sampler)
  return _msg;
}
inline void InputReader::set_allocated_database_sampler(::second::protos::Sampler* database_sampler) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.database_sampler_);
  }
  if (database_sampler) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(database_sampler));
    if (message_arena != submessage_arena) {
      database_sampler = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, database_sampler, submessage_arena);
    }
    
  } else {
    
  }
  _impl_.database_sampler_ = database_sampler;
  // @@protoc_insertion_point(field_set_allocated:second.protos.InputReader.database_sampler)
}

// bool use_group_id = 26;
inline void InputReader::clear_use_group_id() {
  _impl_.use_group_id_ = false;
}
inline bool InputReader::_internal_use_group_id() const {
  return _impl_.use_group_id_;
}
inline bool InputReader::use_group_id() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.use_group_id)
  return _internal_use_group_id();
}
inline void InputReader::_internal_set_use_group_id(bool value) {
  
  _impl_.use_group_id_ = value;
}
inline void InputReader::set_use_group_id(bool value) {
  _internal_set_use_group_id(value);
  // @@protoc_insertion_point(field_set:second.protos.InputReader.use_group_id)
}

// .second.protos.Sampler unlabeled_database_sampler = 27;
inline bool InputReader::_internal_has_unlabeled_database_sampler() const {
  return this != internal_default_instance() && _impl_.unlabeled_database_sampler_ != nullptr;
}
inline bool InputReader::has_unlabeled_database_sampler() const {
  return _internal_has_unlabeled_database_sampler();
}
inline const ::second::protos::Sampler& InputReader::_internal_unlabeled_database_sampler() const {
  const ::second::protos::Sampler* p = _impl_.unlabeled_database_sampler_;
  return p != nullptr ? *p : reinterpret_cast<const ::second::protos::Sampler&>(
      ::second::protos::_Sampler_default_instance_);
}
inline const ::second::protos::Sampler& InputReader::unlabeled_database_sampler() const {
  // @@protoc_insertion_point(field_get:second.protos.InputReader.unlabeled_database_sampler)
  return _internal_unlabeled_database_sampler();
}
inline void InputReader::unsafe_arena_set_allocated_unlabeled_database_sampler(
    ::second::protos::Sampler* unlabeled_database_sampler) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.unlabeled_database_sampler_);
  }
  _impl_.unlabeled_database_sampler_ = unlabeled_database_sampler;
  if (unlabeled_database_sampler) {
    
  } else {
    
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.InputReader.unlabeled_database_sampler)
}
inline ::second::protos::Sampler* InputReader::release_unlabeled_database_sampler() {
  
  ::second::protos::Sampler* temp = _impl_.unlabeled_database_sampler_;
  _impl_.unlabeled_database_sampler_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::second::protos::Sampler* InputReader::unsafe_arena_release_unlabeled_database_sampler() {
  // @@protoc_insertion_point(field_release:second.protos.InputReader.unlabeled_database_sampler)
  
  ::second::protos::Sampler* temp = _impl_.unlabeled_database_sampler_;
  _impl_.unlabeled_database_sampler_ = nullptr;
  return temp;
}
inline ::second::protos::Sampler* InputReader::_internal_mutable_unlabeled_database_sampler() {
  
  if (_impl_.unlabeled_database_sampler_ == nullptr) {
    auto* p = CreateMaybeMessage<::second::protos::Sampler>(GetArenaForAllocation());
    _impl_.unlabeled_database_sampler_ = p;
  }
  return _impl_.unlabeled_database_sampler_;
}
inline ::second::protos::Sampler* InputReader::mutable_unlabeled_database_sampler() {
  ::second::protos::Sampler* _msg = _internal_mutable_unlabeled_database_sampler();
  // @@protoc_insertion_point(field_mutable:second.protos.InputReader.unlabeled_database_sampler)
  return _msg;
}
inline void InputReader::set_allocated_unlabeled_database_sampler(::second::protos::Sampler* unlabeled_database_sampler) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.unlabeled_database_sampler_);
  }
  if (unlabeled_database_sampler) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(unlabeled_database_sampler));
    if (message_arena != submessage_arena) {
      unlabeled_database_sampler = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, unlabeled_database_sampler, submessage_arena);
    }
    
  } else {
    
  }
  _impl_.unlabeled_database_sampler_ = unlabeled_database_sampler;
  // @@protoc_insertion_point(field_set_allocated:second.protos.InputReader.unlabeled_database_sampler)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace second

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2finput_5freader_2eproto
