// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: object_detection/protos/calibration.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fcalibration_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fcalibration_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/map.h>  // IWYU pragma: export
#include <google/protobuf/map_entry.h>
#include <google/protobuf/map_field_inl.h>
#include <google/protobuf/generated_enum_reflection.h>
#include <google/protobuf/unknown_field_set.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_object_5fdetection_2fprotos_2fcalibration_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto;
namespace object_detection {
namespace protos {
class CalibrationConfig;
struct CalibrationConfigDefaultTypeInternal;
extern CalibrationConfigDefaultTypeInternal _CalibrationConfig_default_instance_;
class ClassIdFunctionApproximations;
struct ClassIdFunctionApproximationsDefaultTypeInternal;
extern ClassIdFunctionApproximationsDefaultTypeInternal _ClassIdFunctionApproximations_default_instance_;
class ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse;
struct ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUseDefaultTypeInternal;
extern ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUseDefaultTypeInternal _ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse_default_instance_;
class ClassIdSigmoidCalibrations;
struct ClassIdSigmoidCalibrationsDefaultTypeInternal;
extern ClassIdSigmoidCalibrationsDefaultTypeInternal _ClassIdSigmoidCalibrations_default_instance_;
class ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse;
struct ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUseDefaultTypeInternal;
extern ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUseDefaultTypeInternal _ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse_default_instance_;
class FunctionApproximation;
struct FunctionApproximationDefaultTypeInternal;
extern FunctionApproximationDefaultTypeInternal _FunctionApproximation_default_instance_;
class SigmoidCalibration;
struct SigmoidCalibrationDefaultTypeInternal;
extern SigmoidCalibrationDefaultTypeInternal _SigmoidCalibration_default_instance_;
class SigmoidParameters;
struct SigmoidParametersDefaultTypeInternal;
extern SigmoidParametersDefaultTypeInternal _SigmoidParameters_default_instance_;
class XYPairs;
struct XYPairsDefaultTypeInternal;
extern XYPairsDefaultTypeInternal _XYPairs_default_instance_;
class XYPairs_XYPair;
struct XYPairs_XYPairDefaultTypeInternal;
extern XYPairs_XYPairDefaultTypeInternal _XYPairs_XYPair_default_instance_;
}  // namespace protos
}  // namespace object_detection
PROTOBUF_NAMESPACE_OPEN
template<> ::object_detection::protos::CalibrationConfig* Arena::CreateMaybeMessage<::object_detection::protos::CalibrationConfig>(Arena*);
template<> ::object_detection::protos::ClassIdFunctionApproximations* Arena::CreateMaybeMessage<::object_detection::protos::ClassIdFunctionApproximations>(Arena*);
template<> ::object_detection::protos::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse* Arena::CreateMaybeMessage<::object_detection::protos::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse>(Arena*);
template<> ::object_detection::protos::ClassIdSigmoidCalibrations* Arena::CreateMaybeMessage<::object_detection::protos::ClassIdSigmoidCalibrations>(Arena*);
template<> ::object_detection::protos::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse* Arena::CreateMaybeMessage<::object_detection::protos::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse>(Arena*);
template<> ::object_detection::protos::FunctionApproximation* Arena::CreateMaybeMessage<::object_detection::protos::FunctionApproximation>(Arena*);
template<> ::object_detection::protos::SigmoidCalibration* Arena::CreateMaybeMessage<::object_detection::protos::SigmoidCalibration>(Arena*);
template<> ::object_detection::protos::SigmoidParameters* Arena::CreateMaybeMessage<::object_detection::protos::SigmoidParameters>(Arena*);
template<> ::object_detection::protos::XYPairs* Arena::CreateMaybeMessage<::object_detection::protos::XYPairs>(Arena*);
template<> ::object_detection::protos::XYPairs_XYPair* Arena::CreateMaybeMessage<::object_detection::protos::XYPairs_XYPair>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace object_detection {
namespace protos {

enum TrainingDataType : int {
  DATA_TYPE_UNKNOWN = 0,
  ALL_CLASSES = 1,
  CLASS_SPECIFIC = 2
};
bool TrainingDataType_IsValid(int value);
constexpr TrainingDataType TrainingDataType_MIN = DATA_TYPE_UNKNOWN;
constexpr TrainingDataType TrainingDataType_MAX = CLASS_SPECIFIC;
constexpr int TrainingDataType_ARRAYSIZE = TrainingDataType_MAX + 1;

const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor* TrainingDataType_descriptor();
template<typename T>
inline const std::string& TrainingDataType_Name(T enum_t_value) {
  static_assert(::std::is_same<T, TrainingDataType>::value ||
    ::std::is_integral<T>::value,
    "Incorrect type passed to function TrainingDataType_Name.");
  return ::PROTOBUF_NAMESPACE_ID::internal::NameOfEnum(
    TrainingDataType_descriptor(), enum_t_value);
}
inline bool TrainingDataType_Parse(
    ::PROTOBUF_NAMESPACE_ID::ConstStringParam name, TrainingDataType* value) {
  return ::PROTOBUF_NAMESPACE_ID::internal::ParseNamedEnum<TrainingDataType>(
    TrainingDataType_descriptor(), name, value);
}
// ===================================================================

class CalibrationConfig final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.CalibrationConfig) */ {
 public:
  inline CalibrationConfig() : CalibrationConfig(nullptr) {}
  ~CalibrationConfig() override;
  explicit PROTOBUF_CONSTEXPR CalibrationConfig(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  CalibrationConfig(const CalibrationConfig& from);
  CalibrationConfig(CalibrationConfig&& from) noexcept
    : CalibrationConfig() {
    *this = ::std::move(from);
  }

  inline CalibrationConfig& operator=(const CalibrationConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline CalibrationConfig& operator=(CalibrationConfig&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const CalibrationConfig& default_instance() {
    return *internal_default_instance();
  }
  enum CalibratorCase {
    kFunctionApproximation = 1,
    kClassIdFunctionApproximations = 2,
    kSigmoidCalibration = 3,
    kClassIdSigmoidCalibrations = 4,
    CALIBRATOR_NOT_SET = 0,
  };

  static inline const CalibrationConfig* internal_default_instance() {
    return reinterpret_cast<const CalibrationConfig*>(
               &_CalibrationConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(CalibrationConfig& a, CalibrationConfig& b) {
    a.Swap(&b);
  }
  inline void Swap(CalibrationConfig* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(CalibrationConfig* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  CalibrationConfig* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<CalibrationConfig>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const CalibrationConfig& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const CalibrationConfig& from) {
    CalibrationConfig::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(CalibrationConfig* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.CalibrationConfig";
  }
  protected:
  explicit CalibrationConfig(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kFunctionApproximationFieldNumber = 1,
    kClassIdFunctionApproximationsFieldNumber = 2,
    kSigmoidCalibrationFieldNumber = 3,
    kClassIdSigmoidCalibrationsFieldNumber = 4,
  };
  // .object_detection.protos.FunctionApproximation function_approximation = 1;
  bool has_function_approximation() const;
  private:
  bool _internal_has_function_approximation() const;
  public:
  void clear_function_approximation();
  const ::object_detection::protos::FunctionApproximation& function_approximation() const;
  PROTOBUF_NODISCARD ::object_detection::protos::FunctionApproximation* release_function_approximation();
  ::object_detection::protos::FunctionApproximation* mutable_function_approximation();
  void set_allocated_function_approximation(::object_detection::protos::FunctionApproximation* function_approximation);
  private:
  const ::object_detection::protos::FunctionApproximation& _internal_function_approximation() const;
  ::object_detection::protos::FunctionApproximation* _internal_mutable_function_approximation();
  public:
  void unsafe_arena_set_allocated_function_approximation(
      ::object_detection::protos::FunctionApproximation* function_approximation);
  ::object_detection::protos::FunctionApproximation* unsafe_arena_release_function_approximation();

  // .object_detection.protos.ClassIdFunctionApproximations class_id_function_approximations = 2;
  bool has_class_id_function_approximations() const;
  private:
  bool _internal_has_class_id_function_approximations() const;
  public:
  void clear_class_id_function_approximations();
  const ::object_detection::protos::ClassIdFunctionApproximations& class_id_function_approximations() const;
  PROTOBUF_NODISCARD ::object_detection::protos::ClassIdFunctionApproximations* release_class_id_function_approximations();
  ::object_detection::protos::ClassIdFunctionApproximations* mutable_class_id_function_approximations();
  void set_allocated_class_id_function_approximations(::object_detection::protos::ClassIdFunctionApproximations* class_id_function_approximations);
  private:
  const ::object_detection::protos::ClassIdFunctionApproximations& _internal_class_id_function_approximations() const;
  ::object_detection::protos::ClassIdFunctionApproximations* _internal_mutable_class_id_function_approximations();
  public:
  void unsafe_arena_set_allocated_class_id_function_approximations(
      ::object_detection::protos::ClassIdFunctionApproximations* class_id_function_approximations);
  ::object_detection::protos::ClassIdFunctionApproximations* unsafe_arena_release_class_id_function_approximations();

  // .object_detection.protos.SigmoidCalibration sigmoid_calibration = 3;
  bool has_sigmoid_calibration() const;
  private:
  bool _internal_has_sigmoid_calibration() const;
  public:
  void clear_sigmoid_calibration();
  const ::object_detection::protos::SigmoidCalibration& sigmoid_calibration() const;
  PROTOBUF_NODISCARD ::object_detection::protos::SigmoidCalibration* release_sigmoid_calibration();
  ::object_detection::protos::SigmoidCalibration* mutable_sigmoid_calibration();
  void set_allocated_sigmoid_calibration(::object_detection::protos::SigmoidCalibration* sigmoid_calibration);
  private:
  const ::object_detection::protos::SigmoidCalibration& _internal_sigmoid_calibration() const;
  ::object_detection::protos::SigmoidCalibration* _internal_mutable_sigmoid_calibration();
  public:
  void unsafe_arena_set_allocated_sigmoid_calibration(
      ::object_detection::protos::SigmoidCalibration* sigmoid_calibration);
  ::object_detection::protos::SigmoidCalibration* unsafe_arena_release_sigmoid_calibration();

  // .object_detection.protos.ClassIdSigmoidCalibrations class_id_sigmoid_calibrations = 4;
  bool has_class_id_sigmoid_calibrations() const;
  private:
  bool _internal_has_class_id_sigmoid_calibrations() const;
  public:
  void clear_class_id_sigmoid_calibrations();
  const ::object_detection::protos::ClassIdSigmoidCalibrations& class_id_sigmoid_calibrations() const;
  PROTOBUF_NODISCARD ::object_detection::protos::ClassIdSigmoidCalibrations* release_class_id_sigmoid_calibrations();
  ::object_detection::protos::ClassIdSigmoidCalibrations* mutable_class_id_sigmoid_calibrations();
  void set_allocated_class_id_sigmoid_calibrations(::object_detection::protos::ClassIdSigmoidCalibrations* class_id_sigmoid_calibrations);
  private:
  const ::object_detection::protos::ClassIdSigmoidCalibrations& _internal_class_id_sigmoid_calibrations() const;
  ::object_detection::protos::ClassIdSigmoidCalibrations* _internal_mutable_class_id_sigmoid_calibrations();
  public:
  void unsafe_arena_set_allocated_class_id_sigmoid_calibrations(
      ::object_detection::protos::ClassIdSigmoidCalibrations* class_id_sigmoid_calibrations);
  ::object_detection::protos::ClassIdSigmoidCalibrations* unsafe_arena_release_class_id_sigmoid_calibrations();

  void clear_calibrator();
  CalibratorCase calibrator_case() const;
  // @@protoc_insertion_point(class_scope:object_detection.protos.CalibrationConfig)
 private:
  class _Internal;
  void set_has_function_approximation();
  void set_has_class_id_function_approximations();
  void set_has_sigmoid_calibration();
  void set_has_class_id_sigmoid_calibrations();

  inline bool has_calibrator() const;
  inline void clear_has_calibrator();

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    union CalibratorUnion {
      constexpr CalibratorUnion() : _constinit_{} {}
        ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized _constinit_;
      ::object_detection::protos::FunctionApproximation* function_approximation_;
      ::object_detection::protos::ClassIdFunctionApproximations* class_id_function_approximations_;
      ::object_detection::protos::SigmoidCalibration* sigmoid_calibration_;
      ::object_detection::protos::ClassIdSigmoidCalibrations* class_id_sigmoid_calibrations_;
    } calibrator_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    uint32_t _oneof_case_[1];

  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto;
};
// -------------------------------------------------------------------

class FunctionApproximation final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.FunctionApproximation) */ {
 public:
  inline FunctionApproximation() : FunctionApproximation(nullptr) {}
  ~FunctionApproximation() override;
  explicit PROTOBUF_CONSTEXPR FunctionApproximation(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  FunctionApproximation(const FunctionApproximation& from);
  FunctionApproximation(FunctionApproximation&& from) noexcept
    : FunctionApproximation() {
    *this = ::std::move(from);
  }

  inline FunctionApproximation& operator=(const FunctionApproximation& from) {
    CopyFrom(from);
    return *this;
  }
  inline FunctionApproximation& operator=(FunctionApproximation&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const FunctionApproximation& default_instance() {
    return *internal_default_instance();
  }
  static inline const FunctionApproximation* internal_default_instance() {
    return reinterpret_cast<const FunctionApproximation*>(
               &_FunctionApproximation_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(FunctionApproximation& a, FunctionApproximation& b) {
    a.Swap(&b);
  }
  inline void Swap(FunctionApproximation* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(FunctionApproximation* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  FunctionApproximation* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<FunctionApproximation>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const FunctionApproximation& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const FunctionApproximation& from) {
    FunctionApproximation::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(FunctionApproximation* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.FunctionApproximation";
  }
  protected:
  explicit FunctionApproximation(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kXYPairsFieldNumber = 1,
  };
  // optional .object_detection.protos.XYPairs x_y_pairs = 1;
  bool has_x_y_pairs() const;
  private:
  bool _internal_has_x_y_pairs() const;
  public:
  void clear_x_y_pairs();
  const ::object_detection::protos::XYPairs& x_y_pairs() const;
  PROTOBUF_NODISCARD ::object_detection::protos::XYPairs* release_x_y_pairs();
  ::object_detection::protos::XYPairs* mutable_x_y_pairs();
  void set_allocated_x_y_pairs(::object_detection::protos::XYPairs* x_y_pairs);
  private:
  const ::object_detection::protos::XYPairs& _internal_x_y_pairs() const;
  ::object_detection::protos::XYPairs* _internal_mutable_x_y_pairs();
  public:
  void unsafe_arena_set_allocated_x_y_pairs(
      ::object_detection::protos::XYPairs* x_y_pairs);
  ::object_detection::protos::XYPairs* unsafe_arena_release_x_y_pairs();

  // @@protoc_insertion_point(class_scope:object_detection.protos.FunctionApproximation)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    ::object_detection::protos::XYPairs* x_y_pairs_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto;
};
// -------------------------------------------------------------------

class ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse : public ::PROTOBUF_NAMESPACE_ID::internal::MapEntry<ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse, 
    int32_t, ::object_detection::protos::XYPairs,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_INT32,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_MESSAGE> {
public:
  typedef ::PROTOBUF_NAMESPACE_ID::internal::MapEntry<ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse, 
    int32_t, ::object_detection::protos::XYPairs,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_INT32,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_MESSAGE> SuperType;
  ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse();
  explicit PROTOBUF_CONSTEXPR ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse(
      ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);
  explicit ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  void MergeFrom(const ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse& other);
  static const ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse* internal_default_instance() { return reinterpret_cast<const ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse*>(&_ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse_default_instance_); }
  static bool ValidateKey(void*) { return true; }
  static bool ValidateValue(void*) { return true; }
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto;
};

// -------------------------------------------------------------------

class ClassIdFunctionApproximations final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.ClassIdFunctionApproximations) */ {
 public:
  inline ClassIdFunctionApproximations() : ClassIdFunctionApproximations(nullptr) {}
  ~ClassIdFunctionApproximations() override;
  explicit PROTOBUF_CONSTEXPR ClassIdFunctionApproximations(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  ClassIdFunctionApproximations(const ClassIdFunctionApproximations& from);
  ClassIdFunctionApproximations(ClassIdFunctionApproximations&& from) noexcept
    : ClassIdFunctionApproximations() {
    *this = ::std::move(from);
  }

  inline ClassIdFunctionApproximations& operator=(const ClassIdFunctionApproximations& from) {
    CopyFrom(from);
    return *this;
  }
  inline ClassIdFunctionApproximations& operator=(ClassIdFunctionApproximations&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ClassIdFunctionApproximations& default_instance() {
    return *internal_default_instance();
  }
  static inline const ClassIdFunctionApproximations* internal_default_instance() {
    return reinterpret_cast<const ClassIdFunctionApproximations*>(
               &_ClassIdFunctionApproximations_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    3;

  friend void swap(ClassIdFunctionApproximations& a, ClassIdFunctionApproximations& b) {
    a.Swap(&b);
  }
  inline void Swap(ClassIdFunctionApproximations* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ClassIdFunctionApproximations* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ClassIdFunctionApproximations* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<ClassIdFunctionApproximations>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const ClassIdFunctionApproximations& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const ClassIdFunctionApproximations& from) {
    ClassIdFunctionApproximations::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(ClassIdFunctionApproximations* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.ClassIdFunctionApproximations";
  }
  protected:
  explicit ClassIdFunctionApproximations(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  private:
  static void ArenaDtor(void* object);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------


  // accessors -------------------------------------------------------

  enum : int {
    kClassIdXyPairsMapFieldNumber = 1,
  };
  // map<int32, .object_detection.protos.XYPairs> class_id_xy_pairs_map = 1;
  int class_id_xy_pairs_map_size() const;
  private:
  int _internal_class_id_xy_pairs_map_size() const;
  public:
  void clear_class_id_xy_pairs_map();
  private:
  const ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::XYPairs >&
      _internal_class_id_xy_pairs_map() const;
  ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::XYPairs >*
      _internal_mutable_class_id_xy_pairs_map();
  public:
  const ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::XYPairs >&
      class_id_xy_pairs_map() const;
  ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::XYPairs >*
      mutable_class_id_xy_pairs_map();

  // @@protoc_insertion_point(class_scope:object_detection.protos.ClassIdFunctionApproximations)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::MapField<
        ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse,
        int32_t, ::object_detection::protos::XYPairs,
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_INT32,
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_MESSAGE> class_id_xy_pairs_map_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto;
};
// -------------------------------------------------------------------

class SigmoidCalibration final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.SigmoidCalibration) */ {
 public:
  inline SigmoidCalibration() : SigmoidCalibration(nullptr) {}
  ~SigmoidCalibration() override;
  explicit PROTOBUF_CONSTEXPR SigmoidCalibration(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  SigmoidCalibration(const SigmoidCalibration& from);
  SigmoidCalibration(SigmoidCalibration&& from) noexcept
    : SigmoidCalibration() {
    *this = ::std::move(from);
  }

  inline SigmoidCalibration& operator=(const SigmoidCalibration& from) {
    CopyFrom(from);
    return *this;
  }
  inline SigmoidCalibration& operator=(SigmoidCalibration&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SigmoidCalibration& default_instance() {
    return *internal_default_instance();
  }
  static inline const SigmoidCalibration* internal_default_instance() {
    return reinterpret_cast<const SigmoidCalibration*>(
               &_SigmoidCalibration_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    4;

  friend void swap(SigmoidCalibration& a, SigmoidCalibration& b) {
    a.Swap(&b);
  }
  inline void Swap(SigmoidCalibration* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SigmoidCalibration* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SigmoidCalibration* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<SigmoidCalibration>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const SigmoidCalibration& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const SigmoidCalibration& from) {
    SigmoidCalibration::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(SigmoidCalibration* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.SigmoidCalibration";
  }
  protected:
  explicit SigmoidCalibration(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kSigmoidParametersFieldNumber = 1,
  };
  // optional .object_detection.protos.SigmoidParameters sigmoid_parameters = 1;
  bool has_sigmoid_parameters() const;
  private:
  bool _internal_has_sigmoid_parameters() const;
  public:
  void clear_sigmoid_parameters();
  const ::object_detection::protos::SigmoidParameters& sigmoid_parameters() const;
  PROTOBUF_NODISCARD ::object_detection::protos::SigmoidParameters* release_sigmoid_parameters();
  ::object_detection::protos::SigmoidParameters* mutable_sigmoid_parameters();
  void set_allocated_sigmoid_parameters(::object_detection::protos::SigmoidParameters* sigmoid_parameters);
  private:
  const ::object_detection::protos::SigmoidParameters& _internal_sigmoid_parameters() const;
  ::object_detection::protos::SigmoidParameters* _internal_mutable_sigmoid_parameters();
  public:
  void unsafe_arena_set_allocated_sigmoid_parameters(
      ::object_detection::protos::SigmoidParameters* sigmoid_parameters);
  ::object_detection::protos::SigmoidParameters* unsafe_arena_release_sigmoid_parameters();

  // @@protoc_insertion_point(class_scope:object_detection.protos.SigmoidCalibration)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    ::object_detection::protos::SigmoidParameters* sigmoid_parameters_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto;
};
// -------------------------------------------------------------------

class ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse : public ::PROTOBUF_NAMESPACE_ID::internal::MapEntry<ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse, 
    int32_t, ::object_detection::protos::SigmoidParameters,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_INT32,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_MESSAGE> {
public:
  typedef ::PROTOBUF_NAMESPACE_ID::internal::MapEntry<ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse, 
    int32_t, ::object_detection::protos::SigmoidParameters,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_INT32,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_MESSAGE> SuperType;
  ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse();
  explicit PROTOBUF_CONSTEXPR ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse(
      ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);
  explicit ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  void MergeFrom(const ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse& other);
  static const ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse* internal_default_instance() { return reinterpret_cast<const ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse*>(&_ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse_default_instance_); }
  static bool ValidateKey(void*) { return true; }
  static bool ValidateValue(void*) { return true; }
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto;
};

// -------------------------------------------------------------------

class ClassIdSigmoidCalibrations final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.ClassIdSigmoidCalibrations) */ {
 public:
  inline ClassIdSigmoidCalibrations() : ClassIdSigmoidCalibrations(nullptr) {}
  ~ClassIdSigmoidCalibrations() override;
  explicit PROTOBUF_CONSTEXPR ClassIdSigmoidCalibrations(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  ClassIdSigmoidCalibrations(const ClassIdSigmoidCalibrations& from);
  ClassIdSigmoidCalibrations(ClassIdSigmoidCalibrations&& from) noexcept
    : ClassIdSigmoidCalibrations() {
    *this = ::std::move(from);
  }

  inline ClassIdSigmoidCalibrations& operator=(const ClassIdSigmoidCalibrations& from) {
    CopyFrom(from);
    return *this;
  }
  inline ClassIdSigmoidCalibrations& operator=(ClassIdSigmoidCalibrations&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ClassIdSigmoidCalibrations& default_instance() {
    return *internal_default_instance();
  }
  static inline const ClassIdSigmoidCalibrations* internal_default_instance() {
    return reinterpret_cast<const ClassIdSigmoidCalibrations*>(
               &_ClassIdSigmoidCalibrations_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    6;

  friend void swap(ClassIdSigmoidCalibrations& a, ClassIdSigmoidCalibrations& b) {
    a.Swap(&b);
  }
  inline void Swap(ClassIdSigmoidCalibrations* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ClassIdSigmoidCalibrations* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ClassIdSigmoidCalibrations* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<ClassIdSigmoidCalibrations>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const ClassIdSigmoidCalibrations& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const ClassIdSigmoidCalibrations& from) {
    ClassIdSigmoidCalibrations::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(ClassIdSigmoidCalibrations* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.ClassIdSigmoidCalibrations";
  }
  protected:
  explicit ClassIdSigmoidCalibrations(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  private:
  static void ArenaDtor(void* object);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------


  // accessors -------------------------------------------------------

  enum : int {
    kClassIdSigmoidParametersMapFieldNumber = 1,
  };
  // map<int32, .object_detection.protos.SigmoidParameters> class_id_sigmoid_parameters_map = 1;
  int class_id_sigmoid_parameters_map_size() const;
  private:
  int _internal_class_id_sigmoid_parameters_map_size() const;
  public:
  void clear_class_id_sigmoid_parameters_map();
  private:
  const ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::SigmoidParameters >&
      _internal_class_id_sigmoid_parameters_map() const;
  ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::SigmoidParameters >*
      _internal_mutable_class_id_sigmoid_parameters_map();
  public:
  const ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::SigmoidParameters >&
      class_id_sigmoid_parameters_map() const;
  ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::SigmoidParameters >*
      mutable_class_id_sigmoid_parameters_map();

  // @@protoc_insertion_point(class_scope:object_detection.protos.ClassIdSigmoidCalibrations)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::MapField<
        ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse,
        int32_t, ::object_detection::protos::SigmoidParameters,
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_INT32,
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_MESSAGE> class_id_sigmoid_parameters_map_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto;
};
// -------------------------------------------------------------------

class XYPairs_XYPair final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.XYPairs.XYPair) */ {
 public:
  inline XYPairs_XYPair() : XYPairs_XYPair(nullptr) {}
  ~XYPairs_XYPair() override;
  explicit PROTOBUF_CONSTEXPR XYPairs_XYPair(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  XYPairs_XYPair(const XYPairs_XYPair& from);
  XYPairs_XYPair(XYPairs_XYPair&& from) noexcept
    : XYPairs_XYPair() {
    *this = ::std::move(from);
  }

  inline XYPairs_XYPair& operator=(const XYPairs_XYPair& from) {
    CopyFrom(from);
    return *this;
  }
  inline XYPairs_XYPair& operator=(XYPairs_XYPair&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const XYPairs_XYPair& default_instance() {
    return *internal_default_instance();
  }
  static inline const XYPairs_XYPair* internal_default_instance() {
    return reinterpret_cast<const XYPairs_XYPair*>(
               &_XYPairs_XYPair_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    7;

  friend void swap(XYPairs_XYPair& a, XYPairs_XYPair& b) {
    a.Swap(&b);
  }
  inline void Swap(XYPairs_XYPair* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(XYPairs_XYPair* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  XYPairs_XYPair* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<XYPairs_XYPair>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const XYPairs_XYPair& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const XYPairs_XYPair& from) {
    XYPairs_XYPair::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(XYPairs_XYPair* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.XYPairs.XYPair";
  }
  protected:
  explicit XYPairs_XYPair(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kXFieldNumber = 1,
    kYFieldNumber = 2,
  };
  // optional float x = 1;
  bool has_x() const;
  private:
  bool _internal_has_x() const;
  public:
  void clear_x();
  float x() const;
  void set_x(float value);
  private:
  float _internal_x() const;
  void _internal_set_x(float value);
  public:

  // optional float y = 2;
  bool has_y() const;
  private:
  bool _internal_has_y() const;
  public:
  void clear_y();
  float y() const;
  void set_y(float value);
  private:
  float _internal_y() const;
  void _internal_set_y(float value);
  public:

  // @@protoc_insertion_point(class_scope:object_detection.protos.XYPairs.XYPair)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    float x_;
    float y_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto;
};
// -------------------------------------------------------------------

class XYPairs final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.XYPairs) */ {
 public:
  inline XYPairs() : XYPairs(nullptr) {}
  ~XYPairs() override;
  explicit PROTOBUF_CONSTEXPR XYPairs(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  XYPairs(const XYPairs& from);
  XYPairs(XYPairs&& from) noexcept
    : XYPairs() {
    *this = ::std::move(from);
  }

  inline XYPairs& operator=(const XYPairs& from) {
    CopyFrom(from);
    return *this;
  }
  inline XYPairs& operator=(XYPairs&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const XYPairs& default_instance() {
    return *internal_default_instance();
  }
  static inline const XYPairs* internal_default_instance() {
    return reinterpret_cast<const XYPairs*>(
               &_XYPairs_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    8;

  friend void swap(XYPairs& a, XYPairs& b) {
    a.Swap(&b);
  }
  inline void Swap(XYPairs* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(XYPairs* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  XYPairs* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<XYPairs>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const XYPairs& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const XYPairs& from) {
    XYPairs::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(XYPairs* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.XYPairs";
  }
  protected:
  explicit XYPairs(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  typedef XYPairs_XYPair XYPair;

  // accessors -------------------------------------------------------

  enum : int {
    kXYPairFieldNumber = 1,
    kTrainingDataTypeFieldNumber = 2,
  };
  // repeated .object_detection.protos.XYPairs.XYPair x_y_pair = 1;
  int x_y_pair_size() const;
  private:
  int _internal_x_y_pair_size() const;
  public:
  void clear_x_y_pair();
  ::object_detection::protos::XYPairs_XYPair* mutable_x_y_pair(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::XYPairs_XYPair >*
      mutable_x_y_pair();
  private:
  const ::object_detection::protos::XYPairs_XYPair& _internal_x_y_pair(int index) const;
  ::object_detection::protos::XYPairs_XYPair* _internal_add_x_y_pair();
  public:
  const ::object_detection::protos::XYPairs_XYPair& x_y_pair(int index) const;
  ::object_detection::protos::XYPairs_XYPair* add_x_y_pair();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::XYPairs_XYPair >&
      x_y_pair() const;

  // optional .object_detection.protos.TrainingDataType training_data_type = 2;
  bool has_training_data_type() const;
  private:
  bool _internal_has_training_data_type() const;
  public:
  void clear_training_data_type();
  ::object_detection::protos::TrainingDataType training_data_type() const;
  void set_training_data_type(::object_detection::protos::TrainingDataType value);
  private:
  ::object_detection::protos::TrainingDataType _internal_training_data_type() const;
  void _internal_set_training_data_type(::object_detection::protos::TrainingDataType value);
  public:

  // @@protoc_insertion_point(class_scope:object_detection.protos.XYPairs)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::XYPairs_XYPair > x_y_pair_;
    int training_data_type_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto;
};
// -------------------------------------------------------------------

class SigmoidParameters final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.SigmoidParameters) */ {
 public:
  inline SigmoidParameters() : SigmoidParameters(nullptr) {}
  ~SigmoidParameters() override;
  explicit PROTOBUF_CONSTEXPR SigmoidParameters(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  SigmoidParameters(const SigmoidParameters& from);
  SigmoidParameters(SigmoidParameters&& from) noexcept
    : SigmoidParameters() {
    *this = ::std::move(from);
  }

  inline SigmoidParameters& operator=(const SigmoidParameters& from) {
    CopyFrom(from);
    return *this;
  }
  inline SigmoidParameters& operator=(SigmoidParameters&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SigmoidParameters& default_instance() {
    return *internal_default_instance();
  }
  static inline const SigmoidParameters* internal_default_instance() {
    return reinterpret_cast<const SigmoidParameters*>(
               &_SigmoidParameters_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    9;

  friend void swap(SigmoidParameters& a, SigmoidParameters& b) {
    a.Swap(&b);
  }
  inline void Swap(SigmoidParameters* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SigmoidParameters* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SigmoidParameters* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<SigmoidParameters>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const SigmoidParameters& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const SigmoidParameters& from) {
    SigmoidParameters::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(SigmoidParameters* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.SigmoidParameters";
  }
  protected:
  explicit SigmoidParameters(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kBFieldNumber = 2,
    kAFieldNumber = 1,
  };
  // optional float b = 2 [default = 0];
  bool has_b() const;
  private:
  bool _internal_has_b() const;
  public:
  void clear_b();
  float b() const;
  void set_b(float value);
  private:
  float _internal_b() const;
  void _internal_set_b(float value);
  public:

  // optional float a = 1 [default = -1];
  bool has_a() const;
  private:
  bool _internal_has_a() const;
  public:
  void clear_a();
  float a() const;
  void set_a(float value);
  private:
  float _internal_a() const;
  void _internal_set_a(float value);
  public:

  // @@protoc_insertion_point(class_scope:object_detection.protos.SigmoidParameters)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    float b_;
    float a_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// CalibrationConfig

// .object_detection.protos.FunctionApproximation function_approximation = 1;
inline bool CalibrationConfig::_internal_has_function_approximation() const {
  return calibrator_case() == kFunctionApproximation;
}
inline bool CalibrationConfig::has_function_approximation() const {
  return _internal_has_function_approximation();
}
inline void CalibrationConfig::set_has_function_approximation() {
  _impl_._oneof_case_[0] = kFunctionApproximation;
}
inline void CalibrationConfig::clear_function_approximation() {
  if (_internal_has_function_approximation()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.calibrator_.function_approximation_;
    }
    clear_has_calibrator();
  }
}
inline ::object_detection::protos::FunctionApproximation* CalibrationConfig::release_function_approximation() {
  // @@protoc_insertion_point(field_release:object_detection.protos.CalibrationConfig.function_approximation)
  if (_internal_has_function_approximation()) {
    clear_has_calibrator();
    ::object_detection::protos::FunctionApproximation* temp = _impl_.calibrator_.function_approximation_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.calibrator_.function_approximation_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::object_detection::protos::FunctionApproximation& CalibrationConfig::_internal_function_approximation() const {
  return _internal_has_function_approximation()
      ? *_impl_.calibrator_.function_approximation_
      : reinterpret_cast< ::object_detection::protos::FunctionApproximation&>(::object_detection::protos::_FunctionApproximation_default_instance_);
}
inline const ::object_detection::protos::FunctionApproximation& CalibrationConfig::function_approximation() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.CalibrationConfig.function_approximation)
  return _internal_function_approximation();
}
inline ::object_detection::protos::FunctionApproximation* CalibrationConfig::unsafe_arena_release_function_approximation() {
  // @@protoc_insertion_point(field_unsafe_arena_release:object_detection.protos.CalibrationConfig.function_approximation)
  if (_internal_has_function_approximation()) {
    clear_has_calibrator();
    ::object_detection::protos::FunctionApproximation* temp = _impl_.calibrator_.function_approximation_;
    _impl_.calibrator_.function_approximation_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void CalibrationConfig::unsafe_arena_set_allocated_function_approximation(::object_detection::protos::FunctionApproximation* function_approximation) {
  clear_calibrator();
  if (function_approximation) {
    set_has_function_approximation();
    _impl_.calibrator_.function_approximation_ = function_approximation;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.CalibrationConfig.function_approximation)
}
inline ::object_detection::protos::FunctionApproximation* CalibrationConfig::_internal_mutable_function_approximation() {
  if (!_internal_has_function_approximation()) {
    clear_calibrator();
    set_has_function_approximation();
    _impl_.calibrator_.function_approximation_ = CreateMaybeMessage< ::object_detection::protos::FunctionApproximation >(GetArenaForAllocation());
  }
  return _impl_.calibrator_.function_approximation_;
}
inline ::object_detection::protos::FunctionApproximation* CalibrationConfig::mutable_function_approximation() {
  ::object_detection::protos::FunctionApproximation* _msg = _internal_mutable_function_approximation();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.CalibrationConfig.function_approximation)
  return _msg;
}

// .object_detection.protos.ClassIdFunctionApproximations class_id_function_approximations = 2;
inline bool CalibrationConfig::_internal_has_class_id_function_approximations() const {
  return calibrator_case() == kClassIdFunctionApproximations;
}
inline bool CalibrationConfig::has_class_id_function_approximations() const {
  return _internal_has_class_id_function_approximations();
}
inline void CalibrationConfig::set_has_class_id_function_approximations() {
  _impl_._oneof_case_[0] = kClassIdFunctionApproximations;
}
inline void CalibrationConfig::clear_class_id_function_approximations() {
  if (_internal_has_class_id_function_approximations()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.calibrator_.class_id_function_approximations_;
    }
    clear_has_calibrator();
  }
}
inline ::object_detection::protos::ClassIdFunctionApproximations* CalibrationConfig::release_class_id_function_approximations() {
  // @@protoc_insertion_point(field_release:object_detection.protos.CalibrationConfig.class_id_function_approximations)
  if (_internal_has_class_id_function_approximations()) {
    clear_has_calibrator();
    ::object_detection::protos::ClassIdFunctionApproximations* temp = _impl_.calibrator_.class_id_function_approximations_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.calibrator_.class_id_function_approximations_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::object_detection::protos::ClassIdFunctionApproximations& CalibrationConfig::_internal_class_id_function_approximations() const {
  return _internal_has_class_id_function_approximations()
      ? *_impl_.calibrator_.class_id_function_approximations_
      : reinterpret_cast< ::object_detection::protos::ClassIdFunctionApproximations&>(::object_detection::protos::_ClassIdFunctionApproximations_default_instance_);
}
inline const ::object_detection::protos::ClassIdFunctionApproximations& CalibrationConfig::class_id_function_approximations() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.CalibrationConfig.class_id_function_approximations)
  return _internal_class_id_function_approximations();
}
inline ::object_detection::protos::ClassIdFunctionApproximations* CalibrationConfig::unsafe_arena_release_class_id_function_approximations() {
  // @@protoc_insertion_point(field_unsafe_arena_release:object_detection.protos.CalibrationConfig.class_id_function_approximations)
  if (_internal_has_class_id_function_approximations()) {
    clear_has_calibrator();
    ::object_detection::protos::ClassIdFunctionApproximations* temp = _impl_.calibrator_.class_id_function_approximations_;
    _impl_.calibrator_.class_id_function_approximations_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void CalibrationConfig::unsafe_arena_set_allocated_class_id_function_approximations(::object_detection::protos::ClassIdFunctionApproximations* class_id_function_approximations) {
  clear_calibrator();
  if (class_id_function_approximations) {
    set_has_class_id_function_approximations();
    _impl_.calibrator_.class_id_function_approximations_ = class_id_function_approximations;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.CalibrationConfig.class_id_function_approximations)
}
inline ::object_detection::protos::ClassIdFunctionApproximations* CalibrationConfig::_internal_mutable_class_id_function_approximations() {
  if (!_internal_has_class_id_function_approximations()) {
    clear_calibrator();
    set_has_class_id_function_approximations();
    _impl_.calibrator_.class_id_function_approximations_ = CreateMaybeMessage< ::object_detection::protos::ClassIdFunctionApproximations >(GetArenaForAllocation());
  }
  return _impl_.calibrator_.class_id_function_approximations_;
}
inline ::object_detection::protos::ClassIdFunctionApproximations* CalibrationConfig::mutable_class_id_function_approximations() {
  ::object_detection::protos::ClassIdFunctionApproximations* _msg = _internal_mutable_class_id_function_approximations();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.CalibrationConfig.class_id_function_approximations)
  return _msg;
}

// .object_detection.protos.SigmoidCalibration sigmoid_calibration = 3;
inline bool CalibrationConfig::_internal_has_sigmoid_calibration() const {
  return calibrator_case() == kSigmoidCalibration;
}
inline bool CalibrationConfig::has_sigmoid_calibration() const {
  return _internal_has_sigmoid_calibration();
}
inline void CalibrationConfig::set_has_sigmoid_calibration() {
  _impl_._oneof_case_[0] = kSigmoidCalibration;
}
inline void CalibrationConfig::clear_sigmoid_calibration() {
  if (_internal_has_sigmoid_calibration()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.calibrator_.sigmoid_calibration_;
    }
    clear_has_calibrator();
  }
}
inline ::object_detection::protos::SigmoidCalibration* CalibrationConfig::release_sigmoid_calibration() {
  // @@protoc_insertion_point(field_release:object_detection.protos.CalibrationConfig.sigmoid_calibration)
  if (_internal_has_sigmoid_calibration()) {
    clear_has_calibrator();
    ::object_detection::protos::SigmoidCalibration* temp = _impl_.calibrator_.sigmoid_calibration_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.calibrator_.sigmoid_calibration_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::object_detection::protos::SigmoidCalibration& CalibrationConfig::_internal_sigmoid_calibration() const {
  return _internal_has_sigmoid_calibration()
      ? *_impl_.calibrator_.sigmoid_calibration_
      : reinterpret_cast< ::object_detection::protos::SigmoidCalibration&>(::object_detection::protos::_SigmoidCalibration_default_instance_);
}
inline const ::object_detection::protos::SigmoidCalibration& CalibrationConfig::sigmoid_calibration() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.CalibrationConfig.sigmoid_calibration)
  return _internal_sigmoid_calibration();
}
inline ::object_detection::protos::SigmoidCalibration* CalibrationConfig::unsafe_arena_release_sigmoid_calibration() {
  // @@protoc_insertion_point(field_unsafe_arena_release:object_detection.protos.CalibrationConfig.sigmoid_calibration)
  if (_internal_has_sigmoid_calibration()) {
    clear_has_calibrator();
    ::object_detection::protos::SigmoidCalibration* temp = _impl_.calibrator_.sigmoid_calibration_;
    _impl_.calibrator_.sigmoid_calibration_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void CalibrationConfig::unsafe_arena_set_allocated_sigmoid_calibration(::object_detection::protos::SigmoidCalibration* sigmoid_calibration) {
  clear_calibrator();
  if (sigmoid_calibration) {
    set_has_sigmoid_calibration();
    _impl_.calibrator_.sigmoid_calibration_ = sigmoid_calibration;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.CalibrationConfig.sigmoid_calibration)
}
inline ::object_detection::protos::SigmoidCalibration* CalibrationConfig::_internal_mutable_sigmoid_calibration() {
  if (!_internal_has_sigmoid_calibration()) {
    clear_calibrator();
    set_has_sigmoid_calibration();
    _impl_.calibrator_.sigmoid_calibration_ = CreateMaybeMessage< ::object_detection::protos::SigmoidCalibration >(GetArenaForAllocation());
  }
  return _impl_.calibrator_.sigmoid_calibration_;
}
inline ::object_detection::protos::SigmoidCalibration* CalibrationConfig::mutable_sigmoid_calibration() {
  ::object_detection::protos::SigmoidCalibration* _msg = _internal_mutable_sigmoid_calibration();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.CalibrationConfig.sigmoid_calibration)
  return _msg;
}

// .object_detection.protos.ClassIdSigmoidCalibrations class_id_sigmoid_calibrations = 4;
inline bool CalibrationConfig::_internal_has_class_id_sigmoid_calibrations() const {
  return calibrator_case() == kClassIdSigmoidCalibrations;
}
inline bool CalibrationConfig::has_class_id_sigmoid_calibrations() const {
  return _internal_has_class_id_sigmoid_calibrations();
}
inline void CalibrationConfig::set_has_class_id_sigmoid_calibrations() {
  _impl_._oneof_case_[0] = kClassIdSigmoidCalibrations;
}
inline void CalibrationConfig::clear_class_id_sigmoid_calibrations() {
  if (_internal_has_class_id_sigmoid_calibrations()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.calibrator_.class_id_sigmoid_calibrations_;
    }
    clear_has_calibrator();
  }
}
inline ::object_detection::protos::ClassIdSigmoidCalibrations* CalibrationConfig::release_class_id_sigmoid_calibrations() {
  // @@protoc_insertion_point(field_release:object_detection.protos.CalibrationConfig.class_id_sigmoid_calibrations)
  if (_internal_has_class_id_sigmoid_calibrations()) {
    clear_has_calibrator();
    ::object_detection::protos::ClassIdSigmoidCalibrations* temp = _impl_.calibrator_.class_id_sigmoid_calibrations_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.calibrator_.class_id_sigmoid_calibrations_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::object_detection::protos::ClassIdSigmoidCalibrations& CalibrationConfig::_internal_class_id_sigmoid_calibrations() const {
  return _internal_has_class_id_sigmoid_calibrations()
      ? *_impl_.calibrator_.class_id_sigmoid_calibrations_
      : reinterpret_cast< ::object_detection::protos::ClassIdSigmoidCalibrations&>(::object_detection::protos::_ClassIdSigmoidCalibrations_default_instance_);
}
inline const ::object_detection::protos::ClassIdSigmoidCalibrations& CalibrationConfig::class_id_sigmoid_calibrations() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.CalibrationConfig.class_id_sigmoid_calibrations)
  return _internal_class_id_sigmoid_calibrations();
}
inline ::object_detection::protos::ClassIdSigmoidCalibrations* CalibrationConfig::unsafe_arena_release_class_id_sigmoid_calibrations() {
  // @@protoc_insertion_point(field_unsafe_arena_release:object_detection.protos.CalibrationConfig.class_id_sigmoid_calibrations)
  if (_internal_has_class_id_sigmoid_calibrations()) {
    clear_has_calibrator();
    ::object_detection::protos::ClassIdSigmoidCalibrations* temp = _impl_.calibrator_.class_id_sigmoid_calibrations_;
    _impl_.calibrator_.class_id_sigmoid_calibrations_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void CalibrationConfig::unsafe_arena_set_allocated_class_id_sigmoid_calibrations(::object_detection::protos::ClassIdSigmoidCalibrations* class_id_sigmoid_calibrations) {
  clear_calibrator();
  if (class_id_sigmoid_calibrations) {
    set_has_class_id_sigmoid_calibrations();
    _impl_.calibrator_.class_id_sigmoid_calibrations_ = class_id_sigmoid_calibrations;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.CalibrationConfig.class_id_sigmoid_calibrations)
}
inline ::object_detection::protos::ClassIdSigmoidCalibrations* CalibrationConfig::_internal_mutable_class_id_sigmoid_calibrations() {
  if (!_internal_has_class_id_sigmoid_calibrations()) {
    clear_calibrator();
    set_has_class_id_sigmoid_calibrations();
    _impl_.calibrator_.class_id_sigmoid_calibrations_ = CreateMaybeMessage< ::object_detection::protos::ClassIdSigmoidCalibrations >(GetArenaForAllocation());
  }
  return _impl_.calibrator_.class_id_sigmoid_calibrations_;
}
inline ::object_detection::protos::ClassIdSigmoidCalibrations* CalibrationConfig::mutable_class_id_sigmoid_calibrations() {
  ::object_detection::protos::ClassIdSigmoidCalibrations* _msg = _internal_mutable_class_id_sigmoid_calibrations();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.CalibrationConfig.class_id_sigmoid_calibrations)
  return _msg;
}

inline bool CalibrationConfig::has_calibrator() const {
  return calibrator_case() != CALIBRATOR_NOT_SET;
}
inline void CalibrationConfig::clear_has_calibrator() {
  _impl_._oneof_case_[0] = CALIBRATOR_NOT_SET;
}
inline CalibrationConfig::CalibratorCase CalibrationConfig::calibrator_case() const {
  return CalibrationConfig::CalibratorCase(_impl_._oneof_case_[0]);
}
// -------------------------------------------------------------------

// FunctionApproximation

// optional .object_detection.protos.XYPairs x_y_pairs = 1;
inline bool FunctionApproximation::_internal_has_x_y_pairs() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.x_y_pairs_ != nullptr);
  return value;
}
inline bool FunctionApproximation::has_x_y_pairs() const {
  return _internal_has_x_y_pairs();
}
inline void FunctionApproximation::clear_x_y_pairs() {
  if (_impl_.x_y_pairs_ != nullptr) _impl_.x_y_pairs_->Clear();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const ::object_detection::protos::XYPairs& FunctionApproximation::_internal_x_y_pairs() const {
  const ::object_detection::protos::XYPairs* p = _impl_.x_y_pairs_;
  return p != nullptr ? *p : reinterpret_cast<const ::object_detection::protos::XYPairs&>(
      ::object_detection::protos::_XYPairs_default_instance_);
}
inline const ::object_detection::protos::XYPairs& FunctionApproximation::x_y_pairs() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.FunctionApproximation.x_y_pairs)
  return _internal_x_y_pairs();
}
inline void FunctionApproximation::unsafe_arena_set_allocated_x_y_pairs(
    ::object_detection::protos::XYPairs* x_y_pairs) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.x_y_pairs_);
  }
  _impl_.x_y_pairs_ = x_y_pairs;
  if (x_y_pairs) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.FunctionApproximation.x_y_pairs)
}
inline ::object_detection::protos::XYPairs* FunctionApproximation::release_x_y_pairs() {
  _impl_._has_bits_[0] &= ~0x00000001u;
  ::object_detection::protos::XYPairs* temp = _impl_.x_y_pairs_;
  _impl_.x_y_pairs_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::object_detection::protos::XYPairs* FunctionApproximation::unsafe_arena_release_x_y_pairs() {
  // @@protoc_insertion_point(field_release:object_detection.protos.FunctionApproximation.x_y_pairs)
  _impl_._has_bits_[0] &= ~0x00000001u;
  ::object_detection::protos::XYPairs* temp = _impl_.x_y_pairs_;
  _impl_.x_y_pairs_ = nullptr;
  return temp;
}
inline ::object_detection::protos::XYPairs* FunctionApproximation::_internal_mutable_x_y_pairs() {
  _impl_._has_bits_[0] |= 0x00000001u;
  if (_impl_.x_y_pairs_ == nullptr) {
    auto* p = CreateMaybeMessage<::object_detection::protos::XYPairs>(GetArenaForAllocation());
    _impl_.x_y_pairs_ = p;
  }
  return _impl_.x_y_pairs_;
}
inline ::object_detection::protos::XYPairs* FunctionApproximation::mutable_x_y_pairs() {
  ::object_detection::protos::XYPairs* _msg = _internal_mutable_x_y_pairs();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.FunctionApproximation.x_y_pairs)
  return _msg;
}
inline void FunctionApproximation::set_allocated_x_y_pairs(::object_detection::protos::XYPairs* x_y_pairs) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete _impl_.x_y_pairs_;
  }
  if (x_y_pairs) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(x_y_pairs);
    if (message_arena != submessage_arena) {
      x_y_pairs = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, x_y_pairs, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.x_y_pairs_ = x_y_pairs;
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.FunctionApproximation.x_y_pairs)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ClassIdFunctionApproximations

// map<int32, .object_detection.protos.XYPairs> class_id_xy_pairs_map = 1;
inline int ClassIdFunctionApproximations::_internal_class_id_xy_pairs_map_size() const {
  return _impl_.class_id_xy_pairs_map_.size();
}
inline int ClassIdFunctionApproximations::class_id_xy_pairs_map_size() const {
  return _internal_class_id_xy_pairs_map_size();
}
inline void ClassIdFunctionApproximations::clear_class_id_xy_pairs_map() {
  _impl_.class_id_xy_pairs_map_.Clear();
}
inline const ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::XYPairs >&
ClassIdFunctionApproximations::_internal_class_id_xy_pairs_map() const {
  return _impl_.class_id_xy_pairs_map_.GetMap();
}
inline const ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::XYPairs >&
ClassIdFunctionApproximations::class_id_xy_pairs_map() const {
  // @@protoc_insertion_point(field_map:object_detection.protos.ClassIdFunctionApproximations.class_id_xy_pairs_map)
  return _internal_class_id_xy_pairs_map();
}
inline ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::XYPairs >*
ClassIdFunctionApproximations::_internal_mutable_class_id_xy_pairs_map() {
  return _impl_.class_id_xy_pairs_map_.MutableMap();
}
inline ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::XYPairs >*
ClassIdFunctionApproximations::mutable_class_id_xy_pairs_map() {
  // @@protoc_insertion_point(field_mutable_map:object_detection.protos.ClassIdFunctionApproximations.class_id_xy_pairs_map)
  return _internal_mutable_class_id_xy_pairs_map();
}

// -------------------------------------------------------------------

// SigmoidCalibration

// optional .object_detection.protos.SigmoidParameters sigmoid_parameters = 1;
inline bool SigmoidCalibration::_internal_has_sigmoid_parameters() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.sigmoid_parameters_ != nullptr);
  return value;
}
inline bool SigmoidCalibration::has_sigmoid_parameters() const {
  return _internal_has_sigmoid_parameters();
}
inline void SigmoidCalibration::clear_sigmoid_parameters() {
  if (_impl_.sigmoid_parameters_ != nullptr) _impl_.sigmoid_parameters_->Clear();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const ::object_detection::protos::SigmoidParameters& SigmoidCalibration::_internal_sigmoid_parameters() const {
  const ::object_detection::protos::SigmoidParameters* p = _impl_.sigmoid_parameters_;
  return p != nullptr ? *p : reinterpret_cast<const ::object_detection::protos::SigmoidParameters&>(
      ::object_detection::protos::_SigmoidParameters_default_instance_);
}
inline const ::object_detection::protos::SigmoidParameters& SigmoidCalibration::sigmoid_parameters() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.SigmoidCalibration.sigmoid_parameters)
  return _internal_sigmoid_parameters();
}
inline void SigmoidCalibration::unsafe_arena_set_allocated_sigmoid_parameters(
    ::object_detection::protos::SigmoidParameters* sigmoid_parameters) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.sigmoid_parameters_);
  }
  _impl_.sigmoid_parameters_ = sigmoid_parameters;
  if (sigmoid_parameters) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.SigmoidCalibration.sigmoid_parameters)
}
inline ::object_detection::protos::SigmoidParameters* SigmoidCalibration::release_sigmoid_parameters() {
  _impl_._has_bits_[0] &= ~0x00000001u;
  ::object_detection::protos::SigmoidParameters* temp = _impl_.sigmoid_parameters_;
  _impl_.sigmoid_parameters_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::object_detection::protos::SigmoidParameters* SigmoidCalibration::unsafe_arena_release_sigmoid_parameters() {
  // @@protoc_insertion_point(field_release:object_detection.protos.SigmoidCalibration.sigmoid_parameters)
  _impl_._has_bits_[0] &= ~0x00000001u;
  ::object_detection::protos::SigmoidParameters* temp = _impl_.sigmoid_parameters_;
  _impl_.sigmoid_parameters_ = nullptr;
  return temp;
}
inline ::object_detection::protos::SigmoidParameters* SigmoidCalibration::_internal_mutable_sigmoid_parameters() {
  _impl_._has_bits_[0] |= 0x00000001u;
  if (_impl_.sigmoid_parameters_ == nullptr) {
    auto* p = CreateMaybeMessage<::object_detection::protos::SigmoidParameters>(GetArenaForAllocation());
    _impl_.sigmoid_parameters_ = p;
  }
  return _impl_.sigmoid_parameters_;
}
inline ::object_detection::protos::SigmoidParameters* SigmoidCalibration::mutable_sigmoid_parameters() {
  ::object_detection::protos::SigmoidParameters* _msg = _internal_mutable_sigmoid_parameters();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.SigmoidCalibration.sigmoid_parameters)
  return _msg;
}
inline void SigmoidCalibration::set_allocated_sigmoid_parameters(::object_detection::protos::SigmoidParameters* sigmoid_parameters) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete _impl_.sigmoid_parameters_;
  }
  if (sigmoid_parameters) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(sigmoid_parameters);
    if (message_arena != submessage_arena) {
      sigmoid_parameters = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, sigmoid_parameters, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.sigmoid_parameters_ = sigmoid_parameters;
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.SigmoidCalibration.sigmoid_parameters)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ClassIdSigmoidCalibrations

// map<int32, .object_detection.protos.SigmoidParameters> class_id_sigmoid_parameters_map = 1;
inline int ClassIdSigmoidCalibrations::_internal_class_id_sigmoid_parameters_map_size() const {
  return _impl_.class_id_sigmoid_parameters_map_.size();
}
inline int ClassIdSigmoidCalibrations::class_id_sigmoid_parameters_map_size() const {
  return _internal_class_id_sigmoid_parameters_map_size();
}
inline void ClassIdSigmoidCalibrations::clear_class_id_sigmoid_parameters_map() {
  _impl_.class_id_sigmoid_parameters_map_.Clear();
}
inline const ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::SigmoidParameters >&
ClassIdSigmoidCalibrations::_internal_class_id_sigmoid_parameters_map() const {
  return _impl_.class_id_sigmoid_parameters_map_.GetMap();
}
inline const ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::SigmoidParameters >&
ClassIdSigmoidCalibrations::class_id_sigmoid_parameters_map() const {
  // @@protoc_insertion_point(field_map:object_detection.protos.ClassIdSigmoidCalibrations.class_id_sigmoid_parameters_map)
  return _internal_class_id_sigmoid_parameters_map();
}
inline ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::SigmoidParameters >*
ClassIdSigmoidCalibrations::_internal_mutable_class_id_sigmoid_parameters_map() {
  return _impl_.class_id_sigmoid_parameters_map_.MutableMap();
}
inline ::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::SigmoidParameters >*
ClassIdSigmoidCalibrations::mutable_class_id_sigmoid_parameters_map() {
  // @@protoc_insertion_point(field_mutable_map:object_detection.protos.ClassIdSigmoidCalibrations.class_id_sigmoid_parameters_map)
  return _internal_mutable_class_id_sigmoid_parameters_map();
}

// -------------------------------------------------------------------

// XYPairs_XYPair

// optional float x = 1;
inline bool XYPairs_XYPair::_internal_has_x() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool XYPairs_XYPair::has_x() const {
  return _internal_has_x();
}
inline void XYPairs_XYPair::clear_x() {
  _impl_.x_ = 0;
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline float XYPairs_XYPair::_internal_x() const {
  return _impl_.x_;
}
inline float XYPairs_XYPair::x() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.XYPairs.XYPair.x)
  return _internal_x();
}
inline void XYPairs_XYPair::_internal_set_x(float value) {
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.x_ = value;
}
inline void XYPairs_XYPair::set_x(float value) {
  _internal_set_x(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.XYPairs.XYPair.x)
}

// optional float y = 2;
inline bool XYPairs_XYPair::_internal_has_y() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool XYPairs_XYPair::has_y() const {
  return _internal_has_y();
}
inline void XYPairs_XYPair::clear_y() {
  _impl_.y_ = 0;
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline float XYPairs_XYPair::_internal_y() const {
  return _impl_.y_;
}
inline float XYPairs_XYPair::y() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.XYPairs.XYPair.y)
  return _internal_y();
}
inline void XYPairs_XYPair::_internal_set_y(float value) {
  _impl_._has_bits_[0] |= 0x00000002u;
  _impl_.y_ = value;
}
inline void XYPairs_XYPair::set_y(float value) {
  _internal_set_y(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.XYPairs.XYPair.y)
}

// -------------------------------------------------------------------

// XYPairs

// repeated .object_detection.protos.XYPairs.XYPair x_y_pair = 1;
inline int XYPairs::_internal_x_y_pair_size() const {
  return _impl_.x_y_pair_.size();
}
inline int XYPairs::x_y_pair_size() const {
  return _internal_x_y_pair_size();
}
inline void XYPairs::clear_x_y_pair() {
  _impl_.x_y_pair_.Clear();
}
inline ::object_detection::protos::XYPairs_XYPair* XYPairs::mutable_x_y_pair(int index) {
  // @@protoc_insertion_point(field_mutable:object_detection.protos.XYPairs.x_y_pair)
  return _impl_.x_y_pair_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::XYPairs_XYPair >*
XYPairs::mutable_x_y_pair() {
  // @@protoc_insertion_point(field_mutable_list:object_detection.protos.XYPairs.x_y_pair)
  return &_impl_.x_y_pair_;
}
inline const ::object_detection::protos::XYPairs_XYPair& XYPairs::_internal_x_y_pair(int index) const {
  return _impl_.x_y_pair_.Get(index);
}
inline const ::object_detection::protos::XYPairs_XYPair& XYPairs::x_y_pair(int index) const {
  // @@protoc_insertion_point(field_get:object_detection.protos.XYPairs.x_y_pair)
  return _internal_x_y_pair(index);
}
inline ::object_detection::protos::XYPairs_XYPair* XYPairs::_internal_add_x_y_pair() {
  return _impl_.x_y_pair_.Add();
}
inline ::object_detection::protos::XYPairs_XYPair* XYPairs::add_x_y_pair() {
  ::object_detection::protos::XYPairs_XYPair* _add = _internal_add_x_y_pair();
  // @@protoc_insertion_point(field_add:object_detection.protos.XYPairs.x_y_pair)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::XYPairs_XYPair >&
XYPairs::x_y_pair() const {
  // @@protoc_insertion_point(field_list:object_detection.protos.XYPairs.x_y_pair)
  return _impl_.x_y_pair_;
}

// optional .object_detection.protos.TrainingDataType training_data_type = 2;
inline bool XYPairs::_internal_has_training_data_type() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool XYPairs::has_training_data_type() const {
  return _internal_has_training_data_type();
}
inline void XYPairs::clear_training_data_type() {
  _impl_.training_data_type_ = 0;
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline ::object_detection::protos::TrainingDataType XYPairs::_internal_training_data_type() const {
  return static_cast< ::object_detection::protos::TrainingDataType >(_impl_.training_data_type_);
}
inline ::object_detection::protos::TrainingDataType XYPairs::training_data_type() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.XYPairs.training_data_type)
  return _internal_training_data_type();
}
inline void XYPairs::_internal_set_training_data_type(::object_detection::protos::TrainingDataType value) {
  assert(::object_detection::protos::TrainingDataType_IsValid(value));
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.training_data_type_ = value;
}
inline void XYPairs::set_training_data_type(::object_detection::protos::TrainingDataType value) {
  _internal_set_training_data_type(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.XYPairs.training_data_type)
}

// -------------------------------------------------------------------

// SigmoidParameters

// optional float a = 1 [default = -1];
inline bool SigmoidParameters::_internal_has_a() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool SigmoidParameters::has_a() const {
  return _internal_has_a();
}
inline void SigmoidParameters::clear_a() {
  _impl_.a_ = -1;
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline float SigmoidParameters::_internal_a() const {
  return _impl_.a_;
}
inline float SigmoidParameters::a() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.SigmoidParameters.a)
  return _internal_a();
}
inline void SigmoidParameters::_internal_set_a(float value) {
  _impl_._has_bits_[0] |= 0x00000002u;
  _impl_.a_ = value;
}
inline void SigmoidParameters::set_a(float value) {
  _internal_set_a(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.SigmoidParameters.a)
}

// optional float b = 2 [default = 0];
inline bool SigmoidParameters::_internal_has_b() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool SigmoidParameters::has_b() const {
  return _internal_has_b();
}
inline void SigmoidParameters::clear_b() {
  _impl_.b_ = 0;
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline float SigmoidParameters::_internal_b() const {
  return _impl_.b_;
}
inline float SigmoidParameters::b() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.SigmoidParameters.b)
  return _internal_b();
}
inline void SigmoidParameters::_internal_set_b(float value) {
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.b_ = value;
}
inline void SigmoidParameters::set_b(float value) {
  _internal_set_b(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.SigmoidParameters.b)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------


// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace object_detection

PROTOBUF_NAMESPACE_OPEN

template <> struct is_proto_enum< ::object_detection::protos::TrainingDataType> : ::std::true_type {};
template <>
inline const EnumDescriptor* GetEnumDescriptor< ::object_detection::protos::TrainingDataType>() {
  return ::object_detection::protos::TrainingDataType_descriptor();
}

PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fcalibration_2eproto
