// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: object_detection/protos/pipeline.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fpipeline_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fpipeline_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "object_detection/protos/eval.pb.h"
#include "object_detection/protos/graph_rewriter.pb.h"
#include "object_detection/protos/input_reader.pb.h"
#include "object_detection/protos/model.pb.h"
#include "object_detection/protos/train.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_object_5fdetection_2fprotos_2fpipeline_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_object_5fdetection_2fprotos_2fpipeline_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto;
namespace object_detection {
namespace protos {
class TrainEvalPipelineConfig;
struct TrainEvalPipelineConfigDefaultTypeInternal;
extern TrainEvalPipelineConfigDefaultTypeInternal _TrainEvalPipelineConfig_default_instance_;
}  // namespace protos
}  // namespace object_detection
PROTOBUF_NAMESPACE_OPEN
template<> ::object_detection::protos::TrainEvalPipelineConfig* Arena::CreateMaybeMessage<::object_detection::protos::TrainEvalPipelineConfig>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace object_detection {
namespace protos {

// ===================================================================

class TrainEvalPipelineConfig final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.TrainEvalPipelineConfig) */ {
 public:
  inline TrainEvalPipelineConfig() : TrainEvalPipelineConfig(nullptr) {}
  ~TrainEvalPipelineConfig() override;
  explicit PROTOBUF_CONSTEXPR TrainEvalPipelineConfig(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  TrainEvalPipelineConfig(const TrainEvalPipelineConfig& from);
  TrainEvalPipelineConfig(TrainEvalPipelineConfig&& from) noexcept
    : TrainEvalPipelineConfig() {
    *this = ::std::move(from);
  }

  inline TrainEvalPipelineConfig& operator=(const TrainEvalPipelineConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline TrainEvalPipelineConfig& operator=(TrainEvalPipelineConfig&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const TrainEvalPipelineConfig& default_instance() {
    return *internal_default_instance();
  }
  static inline const TrainEvalPipelineConfig* internal_default_instance() {
    return reinterpret_cast<const TrainEvalPipelineConfig*>(
               &_TrainEvalPipelineConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(TrainEvalPipelineConfig& a, TrainEvalPipelineConfig& b) {
    a.Swap(&b);
  }
  inline void Swap(TrainEvalPipelineConfig* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TrainEvalPipelineConfig* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  TrainEvalPipelineConfig* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<TrainEvalPipelineConfig>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const TrainEvalPipelineConfig& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const TrainEvalPipelineConfig& from) {
    TrainEvalPipelineConfig::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(TrainEvalPipelineConfig* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.TrainEvalPipelineConfig";
  }
  protected:
  explicit TrainEvalPipelineConfig(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kEvalInputReaderFieldNumber = 5,
    kModelFieldNumber = 1,
    kTrainConfigFieldNumber = 2,
    kTrainInputReaderFieldNumber = 3,
    kEvalConfigFieldNumber = 4,
    kGraphRewriterFieldNumber = 6,
  };
  // repeated .object_detection.protos.InputReader eval_input_reader = 5;
  int eval_input_reader_size() const;
  private:
  int _internal_eval_input_reader_size() const;
  public:
  void clear_eval_input_reader();
  ::object_detection::protos::InputReader* mutable_eval_input_reader(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::InputReader >*
      mutable_eval_input_reader();
  private:
  const ::object_detection::protos::InputReader& _internal_eval_input_reader(int index) const;
  ::object_detection::protos::InputReader* _internal_add_eval_input_reader();
  public:
  const ::object_detection::protos::InputReader& eval_input_reader(int index) const;
  ::object_detection::protos::InputReader* add_eval_input_reader();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::InputReader >&
      eval_input_reader() const;

  // optional .object_detection.protos.DetectionModel model = 1;
  bool has_model() const;
  private:
  bool _internal_has_model() const;
  public:
  void clear_model();
  const ::object_detection::protos::DetectionModel& model() const;
  PROTOBUF_NODISCARD ::object_detection::protos::DetectionModel* release_model();
  ::object_detection::protos::DetectionModel* mutable_model();
  void set_allocated_model(::object_detection::protos::DetectionModel* model);
  private:
  const ::object_detection::protos::DetectionModel& _internal_model() const;
  ::object_detection::protos::DetectionModel* _internal_mutable_model();
  public:
  void unsafe_arena_set_allocated_model(
      ::object_detection::protos::DetectionModel* model);
  ::object_detection::protos::DetectionModel* unsafe_arena_release_model();

  // optional .object_detection.protos.TrainConfig train_config = 2;
  bool has_train_config() const;
  private:
  bool _internal_has_train_config() const;
  public:
  void clear_train_config();
  const ::object_detection::protos::TrainConfig& train_config() const;
  PROTOBUF_NODISCARD ::object_detection::protos::TrainConfig* release_train_config();
  ::object_detection::protos::TrainConfig* mutable_train_config();
  void set_allocated_train_config(::object_detection::protos::TrainConfig* train_config);
  private:
  const ::object_detection::protos::TrainConfig& _internal_train_config() const;
  ::object_detection::protos::TrainConfig* _internal_mutable_train_config();
  public:
  void unsafe_arena_set_allocated_train_config(
      ::object_detection::protos::TrainConfig* train_config);
  ::object_detection::protos::TrainConfig* unsafe_arena_release_train_config();

  // optional .object_detection.protos.InputReader train_input_reader = 3;
  bool has_train_input_reader() const;
  private:
  bool _internal_has_train_input_reader() const;
  public:
  void clear_train_input_reader();
  const ::object_detection::protos::InputReader& train_input_reader() const;
  PROTOBUF_NODISCARD ::object_detection::protos::InputReader* release_train_input_reader();
  ::object_detection::protos::InputReader* mutable_train_input_reader();
  void set_allocated_train_input_reader(::object_detection::protos::InputReader* train_input_reader);
  private:
  const ::object_detection::protos::InputReader& _internal_train_input_reader() const;
  ::object_detection::protos::InputReader* _internal_mutable_train_input_reader();
  public:
  void unsafe_arena_set_allocated_train_input_reader(
      ::object_detection::protos::InputReader* train_input_reader);
  ::object_detection::protos::InputReader* unsafe_arena_release_train_input_reader();

  // optional .object_detection.protos.EvalConfig eval_config = 4;
  bool has_eval_config() const;
  private:
  bool _internal_has_eval_config() const;
  public:
  void clear_eval_config();
  const ::object_detection::protos::EvalConfig& eval_config() const;
  PROTOBUF_NODISCARD ::object_detection::protos::EvalConfig* release_eval_config();
  ::object_detection::protos::EvalConfig* mutable_eval_config();
  void set_allocated_eval_config(::object_detection::protos::EvalConfig* eval_config);
  private:
  const ::object_detection::protos::EvalConfig& _internal_eval_config() const;
  ::object_detection::protos::EvalConfig* _internal_mutable_eval_config();
  public:
  void unsafe_arena_set_allocated_eval_config(
      ::object_detection::protos::EvalConfig* eval_config);
  ::object_detection::protos::EvalConfig* unsafe_arena_release_eval_config();

  // optional .object_detection.protos.GraphRewriter graph_rewriter = 6;
  bool has_graph_rewriter() const;
  private:
  bool _internal_has_graph_rewriter() const;
  public:
  void clear_graph_rewriter();
  const ::object_detection::protos::GraphRewriter& graph_rewriter() const;
  PROTOBUF_NODISCARD ::object_detection::protos::GraphRewriter* release_graph_rewriter();
  ::object_detection::protos::GraphRewriter* mutable_graph_rewriter();
  void set_allocated_graph_rewriter(::object_detection::protos::GraphRewriter* graph_rewriter);
  private:
  const ::object_detection::protos::GraphRewriter& _internal_graph_rewriter() const;
  ::object_detection::protos::GraphRewriter* _internal_mutable_graph_rewriter();
  public:
  void unsafe_arena_set_allocated_graph_rewriter(
      ::object_detection::protos::GraphRewriter* graph_rewriter);
  ::object_detection::protos::GraphRewriter* unsafe_arena_release_graph_rewriter();


  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline bool HasExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id) const {

    return _impl_._extensions_.Has(id.number());
  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline void ClearExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id) {
    _impl_._extensions_.ClearExtension(id.number());

  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline int ExtensionSize(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id) const {

    return _impl_._extensions_.ExtensionSize(id.number());
  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Singular::ConstType GetExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id) const {

    return _proto_TypeTraits::Get(id.number(), _impl_._extensions_,
                                  id.default_value());
  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Singular::MutableType MutableExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id) {

    return _proto_TypeTraits::Mutable(id.number(), _field_type,
                                      &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline void SetExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Singular::ConstType value) {
    _proto_TypeTraits::Set(id.number(), _field_type, value, &_impl_._extensions_);

  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline void SetAllocatedExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Singular::MutableType value) {
    _proto_TypeTraits::SetAllocated(id.number(), _field_type, value,
                                    &_impl_._extensions_);

  }
  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline void UnsafeArenaSetAllocatedExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Singular::MutableType value) {
    _proto_TypeTraits::UnsafeArenaSetAllocated(id.number(), _field_type,
                                               value, &_impl_._extensions_);

  }
  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  PROTOBUF_NODISCARD inline
      typename _proto_TypeTraits::Singular::MutableType
      ReleaseExtension(
          const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
              TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id) {

    return _proto_TypeTraits::Release(id.number(), _field_type,
                                      &_impl_._extensions_);
  }
  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Singular::MutableType
  UnsafeArenaReleaseExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id) {

    return _proto_TypeTraits::UnsafeArenaRelease(id.number(), _field_type,
                                                 &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Repeated::ConstType GetExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id,
      int index) const {

    return _proto_TypeTraits::Get(id.number(), _impl_._extensions_, index);
  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Repeated::MutableType MutableExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id,
      int index) {

    return _proto_TypeTraits::Mutable(id.number(), index, &_impl_._extensions_);
  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline void SetExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id,
      int index, typename _proto_TypeTraits::Repeated::ConstType value) {
    _proto_TypeTraits::Set(id.number(), index, value, &_impl_._extensions_);

  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Repeated::MutableType AddExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id) {
    typename _proto_TypeTraits::Repeated::MutableType to_add =
        _proto_TypeTraits::Add(id.number(), _field_type, &_impl_._extensions_);

    return to_add;
  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline void AddExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id,
      typename _proto_TypeTraits::Repeated::ConstType value) {
    _proto_TypeTraits::Add(id.number(), _field_type, _is_packed, value,
                           &_impl_._extensions_);

  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline const typename _proto_TypeTraits::Repeated::RepeatedFieldType&
  GetRepeatedExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id) const {

    return _proto_TypeTraits::GetRepeated(id.number(), _impl_._extensions_);
  }

  template <typename _proto_TypeTraits,
            ::PROTOBUF_NAMESPACE_ID::internal::FieldType _field_type,
            bool _is_packed>
  inline typename _proto_TypeTraits::Repeated::RepeatedFieldType*
  MutableRepeatedExtension(
      const ::PROTOBUF_NAMESPACE_ID::internal::ExtensionIdentifier<
          TrainEvalPipelineConfig, _proto_TypeTraits, _field_type, _is_packed>& id) {

    return _proto_TypeTraits::MutableRepeated(id.number(), _field_type,
                                              _is_packed, &_impl_._extensions_);
  }

  // @@protoc_insertion_point(class_scope:object_detection.protos.TrainEvalPipelineConfig)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::ExtensionSet _extensions_;

    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::InputReader > eval_input_reader_;
    ::object_detection::protos::DetectionModel* model_;
    ::object_detection::protos::TrainConfig* train_config_;
    ::object_detection::protos::InputReader* train_input_reader_;
    ::object_detection::protos::EvalConfig* eval_config_;
    ::object_detection::protos::GraphRewriter* graph_rewriter_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fpipeline_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// TrainEvalPipelineConfig

// optional .object_detection.protos.DetectionModel model = 1;
inline bool TrainEvalPipelineConfig::_internal_has_model() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.model_ != nullptr);
  return value;
}
inline bool TrainEvalPipelineConfig::has_model() const {
  return _internal_has_model();
}
inline const ::object_detection::protos::DetectionModel& TrainEvalPipelineConfig::_internal_model() const {
  const ::object_detection::protos::DetectionModel* p = _impl_.model_;
  return p != nullptr ? *p : reinterpret_cast<const ::object_detection::protos::DetectionModel&>(
      ::object_detection::protos::_DetectionModel_default_instance_);
}
inline const ::object_detection::protos::DetectionModel& TrainEvalPipelineConfig::model() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.TrainEvalPipelineConfig.model)
  return _internal_model();
}
inline void TrainEvalPipelineConfig::unsafe_arena_set_allocated_model(
    ::object_detection::protos::DetectionModel* model) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.model_);
  }
  _impl_.model_ = model;
  if (model) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.TrainEvalPipelineConfig.model)
}
inline ::object_detection::protos::DetectionModel* TrainEvalPipelineConfig::release_model() {
  _impl_._has_bits_[0] &= ~0x00000001u;
  ::object_detection::protos::DetectionModel* temp = _impl_.model_;
  _impl_.model_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::object_detection::protos::DetectionModel* TrainEvalPipelineConfig::unsafe_arena_release_model() {
  // @@protoc_insertion_point(field_release:object_detection.protos.TrainEvalPipelineConfig.model)
  _impl_._has_bits_[0] &= ~0x00000001u;
  ::object_detection::protos::DetectionModel* temp = _impl_.model_;
  _impl_.model_ = nullptr;
  return temp;
}
inline ::object_detection::protos::DetectionModel* TrainEvalPipelineConfig::_internal_mutable_model() {
  _impl_._has_bits_[0] |= 0x00000001u;
  if (_impl_.model_ == nullptr) {
    auto* p = CreateMaybeMessage<::object_detection::protos::DetectionModel>(GetArenaForAllocation());
    _impl_.model_ = p;
  }
  return _impl_.model_;
}
inline ::object_detection::protos::DetectionModel* TrainEvalPipelineConfig::mutable_model() {
  ::object_detection::protos::DetectionModel* _msg = _internal_mutable_model();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.TrainEvalPipelineConfig.model)
  return _msg;
}
inline void TrainEvalPipelineConfig::set_allocated_model(::object_detection::protos::DetectionModel* model) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.model_);
  }
  if (model) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(model));
    if (message_arena != submessage_arena) {
      model = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, model, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.model_ = model;
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.TrainEvalPipelineConfig.model)
}

// optional .object_detection.protos.TrainConfig train_config = 2;
inline bool TrainEvalPipelineConfig::_internal_has_train_config() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.train_config_ != nullptr);
  return value;
}
inline bool TrainEvalPipelineConfig::has_train_config() const {
  return _internal_has_train_config();
}
inline const ::object_detection::protos::TrainConfig& TrainEvalPipelineConfig::_internal_train_config() const {
  const ::object_detection::protos::TrainConfig* p = _impl_.train_config_;
  return p != nullptr ? *p : reinterpret_cast<const ::object_detection::protos::TrainConfig&>(
      ::object_detection::protos::_TrainConfig_default_instance_);
}
inline const ::object_detection::protos::TrainConfig& TrainEvalPipelineConfig::train_config() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.TrainEvalPipelineConfig.train_config)
  return _internal_train_config();
}
inline void TrainEvalPipelineConfig::unsafe_arena_set_allocated_train_config(
    ::object_detection::protos::TrainConfig* train_config) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.train_config_);
  }
  _impl_.train_config_ = train_config;
  if (train_config) {
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.TrainEvalPipelineConfig.train_config)
}
inline ::object_detection::protos::TrainConfig* TrainEvalPipelineConfig::release_train_config() {
  _impl_._has_bits_[0] &= ~0x00000002u;
  ::object_detection::protos::TrainConfig* temp = _impl_.train_config_;
  _impl_.train_config_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::object_detection::protos::TrainConfig* TrainEvalPipelineConfig::unsafe_arena_release_train_config() {
  // @@protoc_insertion_point(field_release:object_detection.protos.TrainEvalPipelineConfig.train_config)
  _impl_._has_bits_[0] &= ~0x00000002u;
  ::object_detection::protos::TrainConfig* temp = _impl_.train_config_;
  _impl_.train_config_ = nullptr;
  return temp;
}
inline ::object_detection::protos::TrainConfig* TrainEvalPipelineConfig::_internal_mutable_train_config() {
  _impl_._has_bits_[0] |= 0x00000002u;
  if (_impl_.train_config_ == nullptr) {
    auto* p = CreateMaybeMessage<::object_detection::protos::TrainConfig>(GetArenaForAllocation());
    _impl_.train_config_ = p;
  }
  return _impl_.train_config_;
}
inline ::object_detection::protos::TrainConfig* TrainEvalPipelineConfig::mutable_train_config() {
  ::object_detection::protos::TrainConfig* _msg = _internal_mutable_train_config();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.TrainEvalPipelineConfig.train_config)
  return _msg;
}
inline void TrainEvalPipelineConfig::set_allocated_train_config(::object_detection::protos::TrainConfig* train_config) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.train_config_);
  }
  if (train_config) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(train_config));
    if (message_arena != submessage_arena) {
      train_config = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, train_config, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  _impl_.train_config_ = train_config;
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.TrainEvalPipelineConfig.train_config)
}

// optional .object_detection.protos.InputReader train_input_reader = 3;
inline bool TrainEvalPipelineConfig::_internal_has_train_input_reader() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.train_input_reader_ != nullptr);
  return value;
}
inline bool TrainEvalPipelineConfig::has_train_input_reader() const {
  return _internal_has_train_input_reader();
}
inline const ::object_detection::protos::InputReader& TrainEvalPipelineConfig::_internal_train_input_reader() const {
  const ::object_detection::protos::InputReader* p = _impl_.train_input_reader_;
  return p != nullptr ? *p : reinterpret_cast<const ::object_detection::protos::InputReader&>(
      ::object_detection::protos::_InputReader_default_instance_);
}
inline const ::object_detection::protos::InputReader& TrainEvalPipelineConfig::train_input_reader() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.TrainEvalPipelineConfig.train_input_reader)
  return _internal_train_input_reader();
}
inline void TrainEvalPipelineConfig::unsafe_arena_set_allocated_train_input_reader(
    ::object_detection::protos::InputReader* train_input_reader) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.train_input_reader_);
  }
  _impl_.train_input_reader_ = train_input_reader;
  if (train_input_reader) {
    _impl_._has_bits_[0] |= 0x00000004u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000004u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.TrainEvalPipelineConfig.train_input_reader)
}
inline ::object_detection::protos::InputReader* TrainEvalPipelineConfig::release_train_input_reader() {
  _impl_._has_bits_[0] &= ~0x00000004u;
  ::object_detection::protos::InputReader* temp = _impl_.train_input_reader_;
  _impl_.train_input_reader_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::object_detection::protos::InputReader* TrainEvalPipelineConfig::unsafe_arena_release_train_input_reader() {
  // @@protoc_insertion_point(field_release:object_detection.protos.TrainEvalPipelineConfig.train_input_reader)
  _impl_._has_bits_[0] &= ~0x00000004u;
  ::object_detection::protos::InputReader* temp = _impl_.train_input_reader_;
  _impl_.train_input_reader_ = nullptr;
  return temp;
}
inline ::object_detection::protos::InputReader* TrainEvalPipelineConfig::_internal_mutable_train_input_reader() {
  _impl_._has_bits_[0] |= 0x00000004u;
  if (_impl_.train_input_reader_ == nullptr) {
    auto* p = CreateMaybeMessage<::object_detection::protos::InputReader>(GetArenaForAllocation());
    _impl_.train_input_reader_ = p;
  }
  return _impl_.train_input_reader_;
}
inline ::object_detection::protos::InputReader* TrainEvalPipelineConfig::mutable_train_input_reader() {
  ::object_detection::protos::InputReader* _msg = _internal_mutable_train_input_reader();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.TrainEvalPipelineConfig.train_input_reader)
  return _msg;
}
inline void TrainEvalPipelineConfig::set_allocated_train_input_reader(::object_detection::protos::InputReader* train_input_reader) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.train_input_reader_);
  }
  if (train_input_reader) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(train_input_reader));
    if (message_arena != submessage_arena) {
      train_input_reader = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, train_input_reader, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000004u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000004u;
  }
  _impl_.train_input_reader_ = train_input_reader;
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.TrainEvalPipelineConfig.train_input_reader)
}

// optional .object_detection.protos.EvalConfig eval_config = 4;
inline bool TrainEvalPipelineConfig::_internal_has_eval_config() const {
  bool value = (_impl_._has_bits_[0] & 0x00000008u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.eval_config_ != nullptr);
  return value;
}
inline bool TrainEvalPipelineConfig::has_eval_config() const {
  return _internal_has_eval_config();
}
inline const ::object_detection::protos::EvalConfig& TrainEvalPipelineConfig::_internal_eval_config() const {
  const ::object_detection::protos::EvalConfig* p = _impl_.eval_config_;
  return p != nullptr ? *p : reinterpret_cast<const ::object_detection::protos::EvalConfig&>(
      ::object_detection::protos::_EvalConfig_default_instance_);
}
inline const ::object_detection::protos::EvalConfig& TrainEvalPipelineConfig::eval_config() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.TrainEvalPipelineConfig.eval_config)
  return _internal_eval_config();
}
inline void TrainEvalPipelineConfig::unsafe_arena_set_allocated_eval_config(
    ::object_detection::protos::EvalConfig* eval_config) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.eval_config_);
  }
  _impl_.eval_config_ = eval_config;
  if (eval_config) {
    _impl_._has_bits_[0] |= 0x00000008u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000008u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.TrainEvalPipelineConfig.eval_config)
}
inline ::object_detection::protos::EvalConfig* TrainEvalPipelineConfig::release_eval_config() {
  _impl_._has_bits_[0] &= ~0x00000008u;
  ::object_detection::protos::EvalConfig* temp = _impl_.eval_config_;
  _impl_.eval_config_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::object_detection::protos::EvalConfig* TrainEvalPipelineConfig::unsafe_arena_release_eval_config() {
  // @@protoc_insertion_point(field_release:object_detection.protos.TrainEvalPipelineConfig.eval_config)
  _impl_._has_bits_[0] &= ~0x00000008u;
  ::object_detection::protos::EvalConfig* temp = _impl_.eval_config_;
  _impl_.eval_config_ = nullptr;
  return temp;
}
inline ::object_detection::protos::EvalConfig* TrainEvalPipelineConfig::_internal_mutable_eval_config() {
  _impl_._has_bits_[0] |= 0x00000008u;
  if (_impl_.eval_config_ == nullptr) {
    auto* p = CreateMaybeMessage<::object_detection::protos::EvalConfig>(GetArenaForAllocation());
    _impl_.eval_config_ = p;
  }
  return _impl_.eval_config_;
}
inline ::object_detection::protos::EvalConfig* TrainEvalPipelineConfig::mutable_eval_config() {
  ::object_detection::protos::EvalConfig* _msg = _internal_mutable_eval_config();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.TrainEvalPipelineConfig.eval_config)
  return _msg;
}
inline void TrainEvalPipelineConfig::set_allocated_eval_config(::object_detection::protos::EvalConfig* eval_config) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.eval_config_);
  }
  if (eval_config) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(eval_config));
    if (message_arena != submessage_arena) {
      eval_config = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, eval_config, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000008u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000008u;
  }
  _impl_.eval_config_ = eval_config;
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.TrainEvalPipelineConfig.eval_config)
}

// repeated .object_detection.protos.InputReader eval_input_reader = 5;
inline int TrainEvalPipelineConfig::_internal_eval_input_reader_size() const {
  return _impl_.eval_input_reader_.size();
}
inline int TrainEvalPipelineConfig::eval_input_reader_size() const {
  return _internal_eval_input_reader_size();
}
inline ::object_detection::protos::InputReader* TrainEvalPipelineConfig::mutable_eval_input_reader(int index) {
  // @@protoc_insertion_point(field_mutable:object_detection.protos.TrainEvalPipelineConfig.eval_input_reader)
  return _impl_.eval_input_reader_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::InputReader >*
TrainEvalPipelineConfig::mutable_eval_input_reader() {
  // @@protoc_insertion_point(field_mutable_list:object_detection.protos.TrainEvalPipelineConfig.eval_input_reader)
  return &_impl_.eval_input_reader_;
}
inline const ::object_detection::protos::InputReader& TrainEvalPipelineConfig::_internal_eval_input_reader(int index) const {
  return _impl_.eval_input_reader_.Get(index);
}
inline const ::object_detection::protos::InputReader& TrainEvalPipelineConfig::eval_input_reader(int index) const {
  // @@protoc_insertion_point(field_get:object_detection.protos.TrainEvalPipelineConfig.eval_input_reader)
  return _internal_eval_input_reader(index);
}
inline ::object_detection::protos::InputReader* TrainEvalPipelineConfig::_internal_add_eval_input_reader() {
  return _impl_.eval_input_reader_.Add();
}
inline ::object_detection::protos::InputReader* TrainEvalPipelineConfig::add_eval_input_reader() {
  ::object_detection::protos::InputReader* _add = _internal_add_eval_input_reader();
  // @@protoc_insertion_point(field_add:object_detection.protos.TrainEvalPipelineConfig.eval_input_reader)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::InputReader >&
TrainEvalPipelineConfig::eval_input_reader() const {
  // @@protoc_insertion_point(field_list:object_detection.protos.TrainEvalPipelineConfig.eval_input_reader)
  return _impl_.eval_input_reader_;
}

// optional .object_detection.protos.GraphRewriter graph_rewriter = 6;
inline bool TrainEvalPipelineConfig::_internal_has_graph_rewriter() const {
  bool value = (_impl_._has_bits_[0] & 0x00000010u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.graph_rewriter_ != nullptr);
  return value;
}
inline bool TrainEvalPipelineConfig::has_graph_rewriter() const {
  return _internal_has_graph_rewriter();
}
inline const ::object_detection::protos::GraphRewriter& TrainEvalPipelineConfig::_internal_graph_rewriter() const {
  const ::object_detection::protos::GraphRewriter* p = _impl_.graph_rewriter_;
  return p != nullptr ? *p : reinterpret_cast<const ::object_detection::protos::GraphRewriter&>(
      ::object_detection::protos::_GraphRewriter_default_instance_);
}
inline const ::object_detection::protos::GraphRewriter& TrainEvalPipelineConfig::graph_rewriter() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.TrainEvalPipelineConfig.graph_rewriter)
  return _internal_graph_rewriter();
}
inline void TrainEvalPipelineConfig::unsafe_arena_set_allocated_graph_rewriter(
    ::object_detection::protos::GraphRewriter* graph_rewriter) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.graph_rewriter_);
  }
  _impl_.graph_rewriter_ = graph_rewriter;
  if (graph_rewriter) {
    _impl_._has_bits_[0] |= 0x00000010u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000010u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.TrainEvalPipelineConfig.graph_rewriter)
}
inline ::object_detection::protos::GraphRewriter* TrainEvalPipelineConfig::release_graph_rewriter() {
  _impl_._has_bits_[0] &= ~0x00000010u;
  ::object_detection::protos::GraphRewriter* temp = _impl_.graph_rewriter_;
  _impl_.graph_rewriter_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::object_detection::protos::GraphRewriter* TrainEvalPipelineConfig::unsafe_arena_release_graph_rewriter() {
  // @@protoc_insertion_point(field_release:object_detection.protos.TrainEvalPipelineConfig.graph_rewriter)
  _impl_._has_bits_[0] &= ~0x00000010u;
  ::object_detection::protos::GraphRewriter* temp = _impl_.graph_rewriter_;
  _impl_.graph_rewriter_ = nullptr;
  return temp;
}
inline ::object_detection::protos::GraphRewriter* TrainEvalPipelineConfig::_internal_mutable_graph_rewriter() {
  _impl_._has_bits_[0] |= 0x00000010u;
  if (_impl_.graph_rewriter_ == nullptr) {
    auto* p = CreateMaybeMessage<::object_detection::protos::GraphRewriter>(GetArenaForAllocation());
    _impl_.graph_rewriter_ = p;
  }
  return _impl_.graph_rewriter_;
}
inline ::object_detection::protos::GraphRewriter* TrainEvalPipelineConfig::mutable_graph_rewriter() {
  ::object_detection::protos::GraphRewriter* _msg = _internal_mutable_graph_rewriter();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.TrainEvalPipelineConfig.graph_rewriter)
  return _msg;
}
inline void TrainEvalPipelineConfig::set_allocated_graph_rewriter(::object_detection::protos::GraphRewriter* graph_rewriter) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.graph_rewriter_);
  }
  if (graph_rewriter) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(graph_rewriter));
    if (message_arena != submessage_arena) {
      graph_rewriter = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, graph_rewriter, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000010u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000010u;
  }
  _impl_.graph_rewriter_ = graph_rewriter;
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.TrainEvalPipelineConfig.graph_rewriter)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace object_detection

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fpipeline_2eproto
