/*
# Copyright 2022-2023 Advanced Micro Devices Inc.
#
# Distributed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
*/
/* const char* xilinx_ @PROJECT_NAME @_version() { */
/*   return "vitis_ai_library-v3.0-25-gb7953a2a9"; */
/* } */

static char version_info_short[] =
    "Advanced Micro Devices vitis_ai_library facedetect Version: 3.5.0-b7953a2a9f60e23efdfced5c186328dd1449665c  2023-06-29 03:20:28 [UTC] "
    "";

static char version_info_long[] =
    "Advanced Micro Devices vitis_ai_library "
    "v3.5.0"
    "\n\tcomponent: facedetect"
    "\n\tbuild_id: "
    "\n\tbuild_user: "
    "\n\tbuilt_time: 2023-06-29 03:20:28 [UTC]"
    "\n\tcurrent_binary_dir: /scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/build/facedetect"
    "\n\tcurrent_source_dir: /scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/git/src/vai_library/facedetect"
    "\n\tgit_version: v3.0-25-gb7953a2a9"
    "\n\ttarget_file: "
    "\n\tlink_flags: "
    "\n\tcxx: /scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/recipe-sysroot-native/usr/bin/aarch64-xilinx-linux/aarch64-xilinx-linux-g++  GNU "
    "12.2.0"
    "\n\thost: 8bd9f1d93f95 Linux-5.15.0-41-generic-x86_64"
    // "\n\tcmake_cxx_flags: -g -ggdb -O0 -fno-inline -U_FORTIFY_SOURCE -D_FORTIFY_SOURCE=0  -mcpu=cortex-a72.cortex-a53 -march=armv8-a+crc -fstack-protector-strong  -O2 -D_FORTIFY_SOURCE=2 -Wformat -Wformat-security -Werror=format-security  --sysroot=/scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/recipe-sysroot  -O2 -pipe -g -feliminate-unused-debug-types -fmacro-prefix-map=/scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/git/src/vai_library=/usr/src/debug/vitis-ai-library/3.5-r0  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/git/src/vai_library=/usr/src/debug/vitis-ai-library/3.5-r0  -fmacro-prefix-map=/scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/build=/usr/src/debug/vitis-ai-library/3.5-r0  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/build=/usr/src/debug/vitis-ai-library/3.5-r0  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/recipe-sysroot=  -fmacro-prefix-map=/scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/recipe-sysroot=  -fdebug-prefix-map=/scratch/jenkins-BUILDS-eSDK-2023.2_stable-pipeline-build_async-62_k26_smk_kv_full/build/tmp/work/cortexa72-cortexa53-xilinx-linux/vitis-ai-library/3.5-r0/recipe-sysroot-native=  -fvisibility-inlines-hidden -std=gnu++17 -Wall -Werror -Wno-pessimizing-move -Wno-deprecated-declarations"
    //
    ;
const char* xilinx_version() { return version_info_short; }
const char* xilinx_version_verbose() { return version_info_long; }
