// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: object_detection/protos/pipeline.proto

#include "object_detection/protos/pipeline.pb.h"

#include <algorithm>

#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/extension_set.h>
#include <google/protobuf/wire_format_lite.h>
#include <google/protobuf/descriptor.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/reflection_ops.h>
#include <google/protobuf/wire_format.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>

PROTOBUF_PRAGMA_INIT_SEG

namespace _pb = ::PROTOBUF_NAMESPACE_ID;
namespace _pbi = _pb::internal;

namespace object_detection {
namespace protos {
PROTOBUF_CONSTEXPR TrainEvalPipelineConfig::TrainEvalPipelineConfig(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_._extensions_)*/{}
  , /*decltype(_impl_._has_bits_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_.eval_input_reader_)*/{}
  , /*decltype(_impl_.model_)*/nullptr
  , /*decltype(_impl_.train_config_)*/nullptr
  , /*decltype(_impl_.train_input_reader_)*/nullptr
  , /*decltype(_impl_.eval_config_)*/nullptr
  , /*decltype(_impl_.graph_rewriter_)*/nullptr} {}
struct TrainEvalPipelineConfigDefaultTypeInternal {
  PROTOBUF_CONSTEXPR TrainEvalPipelineConfigDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~TrainEvalPipelineConfigDefaultTypeInternal() {}
  union {
    TrainEvalPipelineConfig _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 TrainEvalPipelineConfigDefaultTypeInternal _TrainEvalPipelineConfig_default_instance_;
}  // namespace protos
}  // namespace object_detection
static ::_pb::Metadata file_level_metadata_object_5fdetection_2fprotos_2fpipeline_2eproto[1];
static constexpr ::_pb::EnumDescriptor const** file_level_enum_descriptors_object_5fdetection_2fprotos_2fpipeline_2eproto = nullptr;
static constexpr ::_pb::ServiceDescriptor const** file_level_service_descriptors_object_5fdetection_2fprotos_2fpipeline_2eproto = nullptr;

const uint32_t TableStruct_object_5fdetection_2fprotos_2fpipeline_2eproto::offsets[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::TrainEvalPipelineConfig, _impl_._has_bits_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::TrainEvalPipelineConfig, _internal_metadata_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::TrainEvalPipelineConfig, _impl_._extensions_),
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::TrainEvalPipelineConfig, _impl_.model_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::TrainEvalPipelineConfig, _impl_.train_config_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::TrainEvalPipelineConfig, _impl_.train_input_reader_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::TrainEvalPipelineConfig, _impl_.eval_config_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::TrainEvalPipelineConfig, _impl_.eval_input_reader_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::TrainEvalPipelineConfig, _impl_.graph_rewriter_),
  0,
  1,
  2,
  3,
  ~0u,
  4,
};
static const ::_pbi::MigrationSchema schemas[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  { 0, 12, -1, sizeof(::object_detection::protos::TrainEvalPipelineConfig)},
};

static const ::_pb::Message* const file_default_instances[] = {
  &::object_detection::protos::_TrainEvalPipelineConfig_default_instance_._instance,
};

const char descriptor_table_protodef_object_5fdetection_2fprotos_2fpipeline_2eproto[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) =
  "\n&object_detection/protos/pipeline.proto"
  "\022\027object_detection.protos\032\"object_detect"
  "ion/protos/eval.proto\032,object_detection/"
  "protos/graph_rewriter.proto\032*object_dete"
  "ction/protos/input_reader.proto\032#object_"
  "detection/protos/model.proto\032#object_det"
  "ection/protos/train.proto\"\225\003\n\027TrainEvalP"
  "ipelineConfig\0226\n\005model\030\001 \001(\0132\'.object_de"
  "tection.protos.DetectionModel\022:\n\014train_c"
  "onfig\030\002 \001(\0132$.object_detection.protos.Tr"
  "ainConfig\022@\n\022train_input_reader\030\003 \001(\0132$."
  "object_detection.protos.InputReader\0228\n\013e"
  "val_config\030\004 \001(\0132#.object_detection.prot"
  "os.EvalConfig\022\?\n\021eval_input_reader\030\005 \003(\013"
  "2$.object_detection.protos.InputReader\022>"
  "\n\016graph_rewriter\030\006 \001(\0132&.object_detectio"
  "n.protos.GraphRewriter*\t\010\350\007\020\200\200\200\200\002"
  ;
static const ::_pbi::DescriptorTable* const descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto_deps[5] = {
  &::descriptor_table_object_5fdetection_2fprotos_2feval_2eproto,
  &::descriptor_table_object_5fdetection_2fprotos_2fgraph_5frewriter_2eproto,
  &::descriptor_table_object_5fdetection_2fprotos_2finput_5freader_2eproto,
  &::descriptor_table_object_5fdetection_2fprotos_2fmodel_2eproto,
  &::descriptor_table_object_5fdetection_2fprotos_2ftrain_2eproto,
};
static ::_pbi::once_flag descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto_once;
const ::_pbi::DescriptorTable descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto = {
    false, false, 673, descriptor_table_protodef_object_5fdetection_2fprotos_2fpipeline_2eproto,
    "object_detection/protos/pipeline.proto",
    &descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto_once, descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto_deps, 5, 1,
    schemas, file_default_instances, TableStruct_object_5fdetection_2fprotos_2fpipeline_2eproto::offsets,
    file_level_metadata_object_5fdetection_2fprotos_2fpipeline_2eproto, file_level_enum_descriptors_object_5fdetection_2fprotos_2fpipeline_2eproto,
    file_level_service_descriptors_object_5fdetection_2fprotos_2fpipeline_2eproto,
};
PROTOBUF_ATTRIBUTE_WEAK const ::_pbi::DescriptorTable* descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto_getter() {
  return &descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto;
}

// Force running AddDescriptors() at dynamic initialization time.
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::_pbi::AddDescriptorsRunner dynamic_init_dummy_object_5fdetection_2fprotos_2fpipeline_2eproto(&descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto);
namespace object_detection {
namespace protos {

// ===================================================================

class TrainEvalPipelineConfig::_Internal {
 public:
  using HasBits = decltype(std::declval<TrainEvalPipelineConfig>()._impl_._has_bits_);
  static const ::object_detection::protos::DetectionModel& model(const TrainEvalPipelineConfig* msg);
  static void set_has_model(HasBits* has_bits) {
    (*has_bits)[0] |= 1u;
  }
  static const ::object_detection::protos::TrainConfig& train_config(const TrainEvalPipelineConfig* msg);
  static void set_has_train_config(HasBits* has_bits) {
    (*has_bits)[0] |= 2u;
  }
  static const ::object_detection::protos::InputReader& train_input_reader(const TrainEvalPipelineConfig* msg);
  static void set_has_train_input_reader(HasBits* has_bits) {
    (*has_bits)[0] |= 4u;
  }
  static const ::object_detection::protos::EvalConfig& eval_config(const TrainEvalPipelineConfig* msg);
  static void set_has_eval_config(HasBits* has_bits) {
    (*has_bits)[0] |= 8u;
  }
  static const ::object_detection::protos::GraphRewriter& graph_rewriter(const TrainEvalPipelineConfig* msg);
  static void set_has_graph_rewriter(HasBits* has_bits) {
    (*has_bits)[0] |= 16u;
  }
};

const ::object_detection::protos::DetectionModel&
TrainEvalPipelineConfig::_Internal::model(const TrainEvalPipelineConfig* msg) {
  return *msg->_impl_.model_;
}
const ::object_detection::protos::TrainConfig&
TrainEvalPipelineConfig::_Internal::train_config(const TrainEvalPipelineConfig* msg) {
  return *msg->_impl_.train_config_;
}
const ::object_detection::protos::InputReader&
TrainEvalPipelineConfig::_Internal::train_input_reader(const TrainEvalPipelineConfig* msg) {
  return *msg->_impl_.train_input_reader_;
}
const ::object_detection::protos::EvalConfig&
TrainEvalPipelineConfig::_Internal::eval_config(const TrainEvalPipelineConfig* msg) {
  return *msg->_impl_.eval_config_;
}
const ::object_detection::protos::GraphRewriter&
TrainEvalPipelineConfig::_Internal::graph_rewriter(const TrainEvalPipelineConfig* msg) {
  return *msg->_impl_.graph_rewriter_;
}
void TrainEvalPipelineConfig::clear_model() {
  if (_impl_.model_ != nullptr) _impl_.model_->Clear();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
void TrainEvalPipelineConfig::clear_train_config() {
  if (_impl_.train_config_ != nullptr) _impl_.train_config_->Clear();
  _impl_._has_bits_[0] &= ~0x00000002u;
}
void TrainEvalPipelineConfig::clear_train_input_reader() {
  if (_impl_.train_input_reader_ != nullptr) _impl_.train_input_reader_->Clear();
  _impl_._has_bits_[0] &= ~0x00000004u;
}
void TrainEvalPipelineConfig::clear_eval_config() {
  if (_impl_.eval_config_ != nullptr) _impl_.eval_config_->Clear();
  _impl_._has_bits_[0] &= ~0x00000008u;
}
void TrainEvalPipelineConfig::clear_eval_input_reader() {
  _impl_.eval_input_reader_.Clear();
}
void TrainEvalPipelineConfig::clear_graph_rewriter() {
  if (_impl_.graph_rewriter_ != nullptr) _impl_.graph_rewriter_->Clear();
  _impl_._has_bits_[0] &= ~0x00000010u;
}
TrainEvalPipelineConfig::TrainEvalPipelineConfig(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:object_detection.protos.TrainEvalPipelineConfig)
}
TrainEvalPipelineConfig::TrainEvalPipelineConfig(const TrainEvalPipelineConfig& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  TrainEvalPipelineConfig* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      /*decltype(_impl_._extensions_)*/{}
    , decltype(_impl_._has_bits_){from._impl_._has_bits_}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.eval_input_reader_){from._impl_.eval_input_reader_}
    , decltype(_impl_.model_){nullptr}
    , decltype(_impl_.train_config_){nullptr}
    , decltype(_impl_.train_input_reader_){nullptr}
    , decltype(_impl_.eval_config_){nullptr}
    , decltype(_impl_.graph_rewriter_){nullptr}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _impl_._extensions_.MergeFrom(internal_default_instance(), from._impl_._extensions_);
  if (from._internal_has_model()) {
    _this->_impl_.model_ = new ::object_detection::protos::DetectionModel(*from._impl_.model_);
  }
  if (from._internal_has_train_config()) {
    _this->_impl_.train_config_ = new ::object_detection::protos::TrainConfig(*from._impl_.train_config_);
  }
  if (from._internal_has_train_input_reader()) {
    _this->_impl_.train_input_reader_ = new ::object_detection::protos::InputReader(*from._impl_.train_input_reader_);
  }
  if (from._internal_has_eval_config()) {
    _this->_impl_.eval_config_ = new ::object_detection::protos::EvalConfig(*from._impl_.eval_config_);
  }
  if (from._internal_has_graph_rewriter()) {
    _this->_impl_.graph_rewriter_ = new ::object_detection::protos::GraphRewriter(*from._impl_.graph_rewriter_);
  }
  // @@protoc_insertion_point(copy_constructor:object_detection.protos.TrainEvalPipelineConfig)
}

inline void TrainEvalPipelineConfig::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      /*decltype(_impl_._extensions_)*/{::_pbi::ArenaInitialized(), arena}
    , decltype(_impl_._has_bits_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.eval_input_reader_){arena}
    , decltype(_impl_.model_){nullptr}
    , decltype(_impl_.train_config_){nullptr}
    , decltype(_impl_.train_input_reader_){nullptr}
    , decltype(_impl_.eval_config_){nullptr}
    , decltype(_impl_.graph_rewriter_){nullptr}
  };
}

TrainEvalPipelineConfig::~TrainEvalPipelineConfig() {
  // @@protoc_insertion_point(destructor:object_detection.protos.TrainEvalPipelineConfig)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void TrainEvalPipelineConfig::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_._extensions_.~ExtensionSet();
  _impl_.eval_input_reader_.~RepeatedPtrField();
  if (this != internal_default_instance()) delete _impl_.model_;
  if (this != internal_default_instance()) delete _impl_.train_config_;
  if (this != internal_default_instance()) delete _impl_.train_input_reader_;
  if (this != internal_default_instance()) delete _impl_.eval_config_;
  if (this != internal_default_instance()) delete _impl_.graph_rewriter_;
}

void TrainEvalPipelineConfig::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void TrainEvalPipelineConfig::Clear() {
// @@protoc_insertion_point(message_clear_start:object_detection.protos.TrainEvalPipelineConfig)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_._extensions_.Clear();
  _impl_.eval_input_reader_.Clear();
  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x0000001fu) {
    if (cached_has_bits & 0x00000001u) {
      GOOGLE_DCHECK(_impl_.model_ != nullptr);
      _impl_.model_->Clear();
    }
    if (cached_has_bits & 0x00000002u) {
      GOOGLE_DCHECK(_impl_.train_config_ != nullptr);
      _impl_.train_config_->Clear();
    }
    if (cached_has_bits & 0x00000004u) {
      GOOGLE_DCHECK(_impl_.train_input_reader_ != nullptr);
      _impl_.train_input_reader_->Clear();
    }
    if (cached_has_bits & 0x00000008u) {
      GOOGLE_DCHECK(_impl_.eval_config_ != nullptr);
      _impl_.eval_config_->Clear();
    }
    if (cached_has_bits & 0x00000010u) {
      GOOGLE_DCHECK(_impl_.graph_rewriter_ != nullptr);
      _impl_.graph_rewriter_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* TrainEvalPipelineConfig::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  _Internal::HasBits has_bits{};
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // optional .object_detection.protos.DetectionModel model = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr = ctx->ParseMessage(_internal_mutable_model(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional .object_detection.protos.TrainConfig train_config = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 18)) {
          ptr = ctx->ParseMessage(_internal_mutable_train_config(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional .object_detection.protos.InputReader train_input_reader = 3;
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 26)) {
          ptr = ctx->ParseMessage(_internal_mutable_train_input_reader(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional .object_detection.protos.EvalConfig eval_config = 4;
      case 4:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 34)) {
          ptr = ctx->ParseMessage(_internal_mutable_eval_config(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // repeated .object_detection.protos.InputReader eval_input_reader = 5;
      case 5:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 42)) {
          ptr -= 1;
          do {
            ptr += 1;
            ptr = ctx->ParseMessage(_internal_add_eval_input_reader(), ptr);
            CHK_(ptr);
            if (!ctx->DataAvailable(ptr)) break;
          } while (::PROTOBUF_NAMESPACE_ID::internal::ExpectTag<42>(ptr));
        } else
          goto handle_unusual;
        continue;
      // optional .object_detection.protos.GraphRewriter graph_rewriter = 6;
      case 6:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 50)) {
          ptr = ctx->ParseMessage(_internal_mutable_graph_rewriter(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    if ((8000u <= tag)) {
      ptr = _impl_._extensions_.ParseField(tag, ptr, internal_default_instance(), &_internal_metadata_, ctx);
      CHK_(ptr != nullptr);
      continue;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  _impl_._has_bits_.Or(has_bits);
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* TrainEvalPipelineConfig::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:object_detection.protos.TrainEvalPipelineConfig)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  // optional .object_detection.protos.DetectionModel model = 1;
  if (cached_has_bits & 0x00000001u) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(1, _Internal::model(this),
        _Internal::model(this).GetCachedSize(), target, stream);
  }

  // optional .object_detection.protos.TrainConfig train_config = 2;
  if (cached_has_bits & 0x00000002u) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(2, _Internal::train_config(this),
        _Internal::train_config(this).GetCachedSize(), target, stream);
  }

  // optional .object_detection.protos.InputReader train_input_reader = 3;
  if (cached_has_bits & 0x00000004u) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(3, _Internal::train_input_reader(this),
        _Internal::train_input_reader(this).GetCachedSize(), target, stream);
  }

  // optional .object_detection.protos.EvalConfig eval_config = 4;
  if (cached_has_bits & 0x00000008u) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(4, _Internal::eval_config(this),
        _Internal::eval_config(this).GetCachedSize(), target, stream);
  }

  // repeated .object_detection.protos.InputReader eval_input_reader = 5;
  for (unsigned i = 0,
      n = static_cast<unsigned>(this->_internal_eval_input_reader_size()); i < n; i++) {
    const auto& repfield = this->_internal_eval_input_reader(i);
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
        InternalWriteMessage(5, repfield, repfield.GetCachedSize(), target, stream);
  }

  // optional .object_detection.protos.GraphRewriter graph_rewriter = 6;
  if (cached_has_bits & 0x00000010u) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(6, _Internal::graph_rewriter(this),
        _Internal::graph_rewriter(this).GetCachedSize(), target, stream);
  }

  // Extension range [1000, 536870912)
  target = _impl_._extensions_._InternalSerialize(
  internal_default_instance(), 1000, 536870912, target, stream);

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:object_detection.protos.TrainEvalPipelineConfig)
  return target;
}

size_t TrainEvalPipelineConfig::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:object_detection.protos.TrainEvalPipelineConfig)
  size_t total_size = 0;

  total_size += _impl_._extensions_.ByteSize();

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // repeated .object_detection.protos.InputReader eval_input_reader = 5;
  total_size += 1UL * this->_internal_eval_input_reader_size();
  for (const auto& msg : this->_impl_.eval_input_reader_) {
    total_size +=
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(msg);
  }

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x0000001fu) {
    // optional .object_detection.protos.DetectionModel model = 1;
    if (cached_has_bits & 0x00000001u) {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.model_);
    }

    // optional .object_detection.protos.TrainConfig train_config = 2;
    if (cached_has_bits & 0x00000002u) {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.train_config_);
    }

    // optional .object_detection.protos.InputReader train_input_reader = 3;
    if (cached_has_bits & 0x00000004u) {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.train_input_reader_);
    }

    // optional .object_detection.protos.EvalConfig eval_config = 4;
    if (cached_has_bits & 0x00000008u) {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.eval_config_);
    }

    // optional .object_detection.protos.GraphRewriter graph_rewriter = 6;
    if (cached_has_bits & 0x00000010u) {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.graph_rewriter_);
    }

  }
  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData TrainEvalPipelineConfig::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    TrainEvalPipelineConfig::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*TrainEvalPipelineConfig::GetClassData() const { return &_class_data_; }


void TrainEvalPipelineConfig::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<TrainEvalPipelineConfig*>(&to_msg);
  auto& from = static_cast<const TrainEvalPipelineConfig&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:object_detection.protos.TrainEvalPipelineConfig)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.eval_input_reader_.MergeFrom(from._impl_.eval_input_reader_);
  cached_has_bits = from._impl_._has_bits_[0];
  if (cached_has_bits & 0x0000001fu) {
    if (cached_has_bits & 0x00000001u) {
      _this->_internal_mutable_model()->::object_detection::protos::DetectionModel::MergeFrom(
          from._internal_model());
    }
    if (cached_has_bits & 0x00000002u) {
      _this->_internal_mutable_train_config()->::object_detection::protos::TrainConfig::MergeFrom(
          from._internal_train_config());
    }
    if (cached_has_bits & 0x00000004u) {
      _this->_internal_mutable_train_input_reader()->::object_detection::protos::InputReader::MergeFrom(
          from._internal_train_input_reader());
    }
    if (cached_has_bits & 0x00000008u) {
      _this->_internal_mutable_eval_config()->::object_detection::protos::EvalConfig::MergeFrom(
          from._internal_eval_config());
    }
    if (cached_has_bits & 0x00000010u) {
      _this->_internal_mutable_graph_rewriter()->::object_detection::protos::GraphRewriter::MergeFrom(
          from._internal_graph_rewriter());
    }
  }
  _this->_impl_._extensions_.MergeFrom(internal_default_instance(), from._impl_._extensions_);
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void TrainEvalPipelineConfig::CopyFrom(const TrainEvalPipelineConfig& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:object_detection.protos.TrainEvalPipelineConfig)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool TrainEvalPipelineConfig::IsInitialized() const {
  if (!_impl_._extensions_.IsInitialized()) {
    return false;
  }

  if (!::PROTOBUF_NAMESPACE_ID::internal::AllAreInitialized(_impl_.eval_input_reader_))
    return false;
  if (_internal_has_train_input_reader()) {
    if (!_impl_.train_input_reader_->IsInitialized()) return false;
  }
  if (_internal_has_graph_rewriter()) {
    if (!_impl_.graph_rewriter_->IsInitialized()) return false;
  }
  return true;
}

void TrainEvalPipelineConfig::InternalSwap(TrainEvalPipelineConfig* other) {
  using std::swap;
  _impl_._extensions_.InternalSwap(&other->_impl_._extensions_);
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.eval_input_reader_.InternalSwap(&other->_impl_.eval_input_reader_);
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(TrainEvalPipelineConfig, _impl_.graph_rewriter_)
      + sizeof(TrainEvalPipelineConfig::_impl_.graph_rewriter_)
      - PROTOBUF_FIELD_OFFSET(TrainEvalPipelineConfig, _impl_.model_)>(
          reinterpret_cast<char*>(&_impl_.model_),
          reinterpret_cast<char*>(&other->_impl_.model_));
}

::PROTOBUF_NAMESPACE_ID::Metadata TrainEvalPipelineConfig::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fpipeline_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fpipeline_2eproto[0]);
}

// @@protoc_insertion_point(namespace_scope)
}  // namespace protos
}  // namespace object_detection
PROTOBUF_NAMESPACE_OPEN
template<> PROTOBUF_NOINLINE ::object_detection::protos::TrainEvalPipelineConfig*
Arena::CreateMaybeMessage< ::object_detection::protos::TrainEvalPipelineConfig >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::TrainEvalPipelineConfig >(arena);
}
PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)
#include <google/protobuf/port_undef.inc>
