// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: object_detection/protos/eval.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2feval_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2feval_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_object_5fdetection_2fprotos_2feval_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_object_5fdetection_2fprotos_2feval_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_object_5fdetection_2fprotos_2feval_2eproto;
namespace object_detection {
namespace protos {
class EvalConfig;
struct EvalConfigDefaultTypeInternal;
extern EvalConfigDefaultTypeInternal _EvalConfig_default_instance_;
}  // namespace protos
}  // namespace object_detection
PROTOBUF_NAMESPACE_OPEN
template<> ::object_detection::protos::EvalConfig* Arena::CreateMaybeMessage<::object_detection::protos::EvalConfig>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace object_detection {
namespace protos {

// ===================================================================

class EvalConfig final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.EvalConfig) */ {
 public:
  inline EvalConfig() : EvalConfig(nullptr) {}
  ~EvalConfig() override;
  explicit PROTOBUF_CONSTEXPR EvalConfig(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  EvalConfig(const EvalConfig& from);
  EvalConfig(EvalConfig&& from) noexcept
    : EvalConfig() {
    *this = ::std::move(from);
  }

  inline EvalConfig& operator=(const EvalConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline EvalConfig& operator=(EvalConfig&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const EvalConfig& default_instance() {
    return *internal_default_instance();
  }
  static inline const EvalConfig* internal_default_instance() {
    return reinterpret_cast<const EvalConfig*>(
               &_EvalConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(EvalConfig& a, EvalConfig& b) {
    a.Swap(&b);
  }
  inline void Swap(EvalConfig* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(EvalConfig* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  EvalConfig* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<EvalConfig>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const EvalConfig& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const EvalConfig& from) {
    EvalConfig::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(EvalConfig* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.EvalConfig";
  }
  protected:
  explicit EvalConfig(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kMetricsSetFieldNumber = 8,
    kVisualizationExportDirFieldNumber = 6,
    kEvalMasterFieldNumber = 7,
    kExportPathFieldNumber = 9,
    kGroundtruthBoxVisualizationColorFieldNumber = 18,
    kMaxEvalsFieldNumber = 4,
    kSaveGraphFieldNumber = 5,
    kIgnoreGroundtruthFieldNumber = 10,
    kUseMovingAveragesFieldNumber = 11,
    kEvalInstanceMasksFieldNumber = 12,
    kSkipScoresFieldNumber = 15,
    kSkipLabelsFieldNumber = 16,
    kVisualizeGroundtruthBoxesFieldNumber = 17,
    kKeepImageIdForVisualizationExportFieldNumber = 19,
    kIncludeMetricsPerCategoryFieldNumber = 24,
    kRecallLowerBoundFieldNumber = 26,
    kNumVisualizationsFieldNumber = 1,
    kNumExamplesFieldNumber = 2,
    kEvalIntervalSecsFieldNumber = 3,
    kMinScoreThresholdFieldNumber = 13,
    kMaxNumBoxesToVisualizeFieldNumber = 14,
    kRetainOriginalImagesFieldNumber = 23,
    kBatchSizeFieldNumber = 25,
    kRecallUpperBoundFieldNumber = 27,
  };
  // repeated string metrics_set = 8;
  int metrics_set_size() const;
  private:
  int _internal_metrics_set_size() const;
  public:
  void clear_metrics_set();
  const std::string& metrics_set(int index) const;
  std::string* mutable_metrics_set(int index);
  void set_metrics_set(int index, const std::string& value);
  void set_metrics_set(int index, std::string&& value);
  void set_metrics_set(int index, const char* value);
  void set_metrics_set(int index, const char* value, size_t size);
  std::string* add_metrics_set();
  void add_metrics_set(const std::string& value);
  void add_metrics_set(std::string&& value);
  void add_metrics_set(const char* value);
  void add_metrics_set(const char* value, size_t size);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>& metrics_set() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>* mutable_metrics_set();
  private:
  const std::string& _internal_metrics_set(int index) const;
  std::string* _internal_add_metrics_set();
  public:

  // optional string visualization_export_dir = 6 [default = ""];
  bool has_visualization_export_dir() const;
  private:
  bool _internal_has_visualization_export_dir() const;
  public:
  void clear_visualization_export_dir();
  const std::string& visualization_export_dir() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_visualization_export_dir(ArgT0&& arg0, ArgT... args);
  std::string* mutable_visualization_export_dir();
  PROTOBUF_NODISCARD std::string* release_visualization_export_dir();
  void set_allocated_visualization_export_dir(std::string* visualization_export_dir);
  private:
  const std::string& _internal_visualization_export_dir() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_visualization_export_dir(const std::string& value);
  std::string* _internal_mutable_visualization_export_dir();
  public:

  // optional string eval_master = 7 [default = ""];
  bool has_eval_master() const;
  private:
  bool _internal_has_eval_master() const;
  public:
  void clear_eval_master();
  const std::string& eval_master() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_eval_master(ArgT0&& arg0, ArgT... args);
  std::string* mutable_eval_master();
  PROTOBUF_NODISCARD std::string* release_eval_master();
  void set_allocated_eval_master(std::string* eval_master);
  private:
  const std::string& _internal_eval_master() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_eval_master(const std::string& value);
  std::string* _internal_mutable_eval_master();
  public:

  // optional string export_path = 9 [default = ""];
  bool has_export_path() const;
  private:
  bool _internal_has_export_path() const;
  public:
  void clear_export_path();
  const std::string& export_path() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_export_path(ArgT0&& arg0, ArgT... args);
  std::string* mutable_export_path();
  PROTOBUF_NODISCARD std::string* release_export_path();
  void set_allocated_export_path(std::string* export_path);
  private:
  const std::string& _internal_export_path() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_export_path(const std::string& value);
  std::string* _internal_mutable_export_path();
  public:

  // optional string groundtruth_box_visualization_color = 18 [default = "black"];
  bool has_groundtruth_box_visualization_color() const;
  private:
  bool _internal_has_groundtruth_box_visualization_color() const;
  public:
  void clear_groundtruth_box_visualization_color();
  const std::string& groundtruth_box_visualization_color() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_groundtruth_box_visualization_color(ArgT0&& arg0, ArgT... args);
  std::string* mutable_groundtruth_box_visualization_color();
  PROTOBUF_NODISCARD std::string* release_groundtruth_box_visualization_color();
  void set_allocated_groundtruth_box_visualization_color(std::string* groundtruth_box_visualization_color);
  private:
  const std::string& _internal_groundtruth_box_visualization_color() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_groundtruth_box_visualization_color(const std::string& value);
  std::string* _internal_mutable_groundtruth_box_visualization_color();
  public:

  // optional uint32 max_evals = 4 [default = 0, deprecated = true];
  PROTOBUF_DEPRECATED bool has_max_evals() const;
  private:
  bool _internal_has_max_evals() const;
  public:
  PROTOBUF_DEPRECATED void clear_max_evals();
  PROTOBUF_DEPRECATED uint32_t max_evals() const;
  PROTOBUF_DEPRECATED void set_max_evals(uint32_t value);
  private:
  uint32_t _internal_max_evals() const;
  void _internal_set_max_evals(uint32_t value);
  public:

  // optional bool save_graph = 5 [default = false];
  bool has_save_graph() const;
  private:
  bool _internal_has_save_graph() const;
  public:
  void clear_save_graph();
  bool save_graph() const;
  void set_save_graph(bool value);
  private:
  bool _internal_save_graph() const;
  void _internal_set_save_graph(bool value);
  public:

  // optional bool ignore_groundtruth = 10 [default = false];
  bool has_ignore_groundtruth() const;
  private:
  bool _internal_has_ignore_groundtruth() const;
  public:
  void clear_ignore_groundtruth();
  bool ignore_groundtruth() const;
  void set_ignore_groundtruth(bool value);
  private:
  bool _internal_ignore_groundtruth() const;
  void _internal_set_ignore_groundtruth(bool value);
  public:

  // optional bool use_moving_averages = 11 [default = false];
  bool has_use_moving_averages() const;
  private:
  bool _internal_has_use_moving_averages() const;
  public:
  void clear_use_moving_averages();
  bool use_moving_averages() const;
  void set_use_moving_averages(bool value);
  private:
  bool _internal_use_moving_averages() const;
  void _internal_set_use_moving_averages(bool value);
  public:

  // optional bool eval_instance_masks = 12 [default = false];
  bool has_eval_instance_masks() const;
  private:
  bool _internal_has_eval_instance_masks() const;
  public:
  void clear_eval_instance_masks();
  bool eval_instance_masks() const;
  void set_eval_instance_masks(bool value);
  private:
  bool _internal_eval_instance_masks() const;
  void _internal_set_eval_instance_masks(bool value);
  public:

  // optional bool skip_scores = 15 [default = false];
  bool has_skip_scores() const;
  private:
  bool _internal_has_skip_scores() const;
  public:
  void clear_skip_scores();
  bool skip_scores() const;
  void set_skip_scores(bool value);
  private:
  bool _internal_skip_scores() const;
  void _internal_set_skip_scores(bool value);
  public:

  // optional bool skip_labels = 16 [default = false];
  bool has_skip_labels() const;
  private:
  bool _internal_has_skip_labels() const;
  public:
  void clear_skip_labels();
  bool skip_labels() const;
  void set_skip_labels(bool value);
  private:
  bool _internal_skip_labels() const;
  void _internal_set_skip_labels(bool value);
  public:

  // optional bool visualize_groundtruth_boxes = 17 [default = false];
  bool has_visualize_groundtruth_boxes() const;
  private:
  bool _internal_has_visualize_groundtruth_boxes() const;
  public:
  void clear_visualize_groundtruth_boxes();
  bool visualize_groundtruth_boxes() const;
  void set_visualize_groundtruth_boxes(bool value);
  private:
  bool _internal_visualize_groundtruth_boxes() const;
  void _internal_set_visualize_groundtruth_boxes(bool value);
  public:

  // optional bool keep_image_id_for_visualization_export = 19 [default = false];
  bool has_keep_image_id_for_visualization_export() const;
  private:
  bool _internal_has_keep_image_id_for_visualization_export() const;
  public:
  void clear_keep_image_id_for_visualization_export();
  bool keep_image_id_for_visualization_export() const;
  void set_keep_image_id_for_visualization_export(bool value);
  private:
  bool _internal_keep_image_id_for_visualization_export() const;
  void _internal_set_keep_image_id_for_visualization_export(bool value);
  public:

  // optional bool include_metrics_per_category = 24 [default = false];
  bool has_include_metrics_per_category() const;
  private:
  bool _internal_has_include_metrics_per_category() const;
  public:
  void clear_include_metrics_per_category();
  bool include_metrics_per_category() const;
  void set_include_metrics_per_category(bool value);
  private:
  bool _internal_include_metrics_per_category() const;
  void _internal_set_include_metrics_per_category(bool value);
  public:

  // optional float recall_lower_bound = 26 [default = 0];
  bool has_recall_lower_bound() const;
  private:
  bool _internal_has_recall_lower_bound() const;
  public:
  void clear_recall_lower_bound();
  float recall_lower_bound() const;
  void set_recall_lower_bound(float value);
  private:
  float _internal_recall_lower_bound() const;
  void _internal_set_recall_lower_bound(float value);
  public:

  // optional uint32 num_visualizations = 1 [default = 10];
  bool has_num_visualizations() const;
  private:
  bool _internal_has_num_visualizations() const;
  public:
  void clear_num_visualizations();
  uint32_t num_visualizations() const;
  void set_num_visualizations(uint32_t value);
  private:
  uint32_t _internal_num_visualizations() const;
  void _internal_set_num_visualizations(uint32_t value);
  public:

  // optional uint32 num_examples = 2 [default = 5000, deprecated = true];
  PROTOBUF_DEPRECATED bool has_num_examples() const;
  private:
  bool _internal_has_num_examples() const;
  public:
  PROTOBUF_DEPRECATED void clear_num_examples();
  PROTOBUF_DEPRECATED uint32_t num_examples() const;
  PROTOBUF_DEPRECATED void set_num_examples(uint32_t value);
  private:
  uint32_t _internal_num_examples() const;
  void _internal_set_num_examples(uint32_t value);
  public:

  // optional uint32 eval_interval_secs = 3 [default = 300];
  bool has_eval_interval_secs() const;
  private:
  bool _internal_has_eval_interval_secs() const;
  public:
  void clear_eval_interval_secs();
  uint32_t eval_interval_secs() const;
  void set_eval_interval_secs(uint32_t value);
  private:
  uint32_t _internal_eval_interval_secs() const;
  void _internal_set_eval_interval_secs(uint32_t value);
  public:

  // optional float min_score_threshold = 13 [default = 0.5];
  bool has_min_score_threshold() const;
  private:
  bool _internal_has_min_score_threshold() const;
  public:
  void clear_min_score_threshold();
  float min_score_threshold() const;
  void set_min_score_threshold(float value);
  private:
  float _internal_min_score_threshold() const;
  void _internal_set_min_score_threshold(float value);
  public:

  // optional int32 max_num_boxes_to_visualize = 14 [default = 20];
  bool has_max_num_boxes_to_visualize() const;
  private:
  bool _internal_has_max_num_boxes_to_visualize() const;
  public:
  void clear_max_num_boxes_to_visualize();
  int32_t max_num_boxes_to_visualize() const;
  void set_max_num_boxes_to_visualize(int32_t value);
  private:
  int32_t _internal_max_num_boxes_to_visualize() const;
  void _internal_set_max_num_boxes_to_visualize(int32_t value);
  public:

  // optional bool retain_original_images = 23 [default = true];
  bool has_retain_original_images() const;
  private:
  bool _internal_has_retain_original_images() const;
  public:
  void clear_retain_original_images();
  bool retain_original_images() const;
  void set_retain_original_images(bool value);
  private:
  bool _internal_retain_original_images() const;
  void _internal_set_retain_original_images(bool value);
  public:

  // optional uint32 batch_size = 25 [default = 1];
  bool has_batch_size() const;
  private:
  bool _internal_has_batch_size() const;
  public:
  void clear_batch_size();
  uint32_t batch_size() const;
  void set_batch_size(uint32_t value);
  private:
  uint32_t _internal_batch_size() const;
  void _internal_set_batch_size(uint32_t value);
  public:

  // optional float recall_upper_bound = 27 [default = 1];
  bool has_recall_upper_bound() const;
  private:
  bool _internal_has_recall_upper_bound() const;
  public:
  void clear_recall_upper_bound();
  float recall_upper_bound() const;
  void set_recall_upper_bound(float value);
  private:
  float _internal_recall_upper_bound() const;
  void _internal_set_recall_upper_bound(float value);
  public:

  // @@protoc_insertion_point(class_scope:object_detection.protos.EvalConfig)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string> metrics_set_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr visualization_export_dir_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr eval_master_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr export_path_;
    static const ::PROTOBUF_NAMESPACE_ID::internal::LazyString _i_give_permission_to_break_this_code_default_groundtruth_box_visualization_color_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr groundtruth_box_visualization_color_;
    uint32_t max_evals_;
    bool save_graph_;
    bool ignore_groundtruth_;
    bool use_moving_averages_;
    bool eval_instance_masks_;
    bool skip_scores_;
    bool skip_labels_;
    bool visualize_groundtruth_boxes_;
    bool keep_image_id_for_visualization_export_;
    bool include_metrics_per_category_;
    float recall_lower_bound_;
    uint32_t num_visualizations_;
    uint32_t num_examples_;
    uint32_t eval_interval_secs_;
    float min_score_threshold_;
    int32_t max_num_boxes_to_visualize_;
    bool retain_original_images_;
    uint32_t batch_size_;
    float recall_upper_bound_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2feval_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// EvalConfig

// optional uint32 batch_size = 25 [default = 1];
inline bool EvalConfig::_internal_has_batch_size() const {
  bool value = (_impl_._has_bits_[0] & 0x00200000u) != 0;
  return value;
}
inline bool EvalConfig::has_batch_size() const {
  return _internal_has_batch_size();
}
inline void EvalConfig::clear_batch_size() {
  _impl_.batch_size_ = 1u;
  _impl_._has_bits_[0] &= ~0x00200000u;
}
inline uint32_t EvalConfig::_internal_batch_size() const {
  return _impl_.batch_size_;
}
inline uint32_t EvalConfig::batch_size() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.batch_size)
  return _internal_batch_size();
}
inline void EvalConfig::_internal_set_batch_size(uint32_t value) {
  _impl_._has_bits_[0] |= 0x00200000u;
  _impl_.batch_size_ = value;
}
inline void EvalConfig::set_batch_size(uint32_t value) {
  _internal_set_batch_size(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.batch_size)
}

// optional uint32 num_visualizations = 1 [default = 10];
inline bool EvalConfig::_internal_has_num_visualizations() const {
  bool value = (_impl_._has_bits_[0] & 0x00008000u) != 0;
  return value;
}
inline bool EvalConfig::has_num_visualizations() const {
  return _internal_has_num_visualizations();
}
inline void EvalConfig::clear_num_visualizations() {
  _impl_.num_visualizations_ = 10u;
  _impl_._has_bits_[0] &= ~0x00008000u;
}
inline uint32_t EvalConfig::_internal_num_visualizations() const {
  return _impl_.num_visualizations_;
}
inline uint32_t EvalConfig::num_visualizations() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.num_visualizations)
  return _internal_num_visualizations();
}
inline void EvalConfig::_internal_set_num_visualizations(uint32_t value) {
  _impl_._has_bits_[0] |= 0x00008000u;
  _impl_.num_visualizations_ = value;
}
inline void EvalConfig::set_num_visualizations(uint32_t value) {
  _internal_set_num_visualizations(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.num_visualizations)
}

// optional uint32 num_examples = 2 [default = 5000, deprecated = true];
inline bool EvalConfig::_internal_has_num_examples() const {
  bool value = (_impl_._has_bits_[0] & 0x00010000u) != 0;
  return value;
}
inline bool EvalConfig::has_num_examples() const {
  return _internal_has_num_examples();
}
inline void EvalConfig::clear_num_examples() {
  _impl_.num_examples_ = 5000u;
  _impl_._has_bits_[0] &= ~0x00010000u;
}
inline uint32_t EvalConfig::_internal_num_examples() const {
  return _impl_.num_examples_;
}
inline uint32_t EvalConfig::num_examples() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.num_examples)
  return _internal_num_examples();
}
inline void EvalConfig::_internal_set_num_examples(uint32_t value) {
  _impl_._has_bits_[0] |= 0x00010000u;
  _impl_.num_examples_ = value;
}
inline void EvalConfig::set_num_examples(uint32_t value) {
  _internal_set_num_examples(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.num_examples)
}

// optional uint32 eval_interval_secs = 3 [default = 300];
inline bool EvalConfig::_internal_has_eval_interval_secs() const {
  bool value = (_impl_._has_bits_[0] & 0x00020000u) != 0;
  return value;
}
inline bool EvalConfig::has_eval_interval_secs() const {
  return _internal_has_eval_interval_secs();
}
inline void EvalConfig::clear_eval_interval_secs() {
  _impl_.eval_interval_secs_ = 300u;
  _impl_._has_bits_[0] &= ~0x00020000u;
}
inline uint32_t EvalConfig::_internal_eval_interval_secs() const {
  return _impl_.eval_interval_secs_;
}
inline uint32_t EvalConfig::eval_interval_secs() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.eval_interval_secs)
  return _internal_eval_interval_secs();
}
inline void EvalConfig::_internal_set_eval_interval_secs(uint32_t value) {
  _impl_._has_bits_[0] |= 0x00020000u;
  _impl_.eval_interval_secs_ = value;
}
inline void EvalConfig::set_eval_interval_secs(uint32_t value) {
  _internal_set_eval_interval_secs(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.eval_interval_secs)
}

// optional uint32 max_evals = 4 [default = 0, deprecated = true];
inline bool EvalConfig::_internal_has_max_evals() const {
  bool value = (_impl_._has_bits_[0] & 0x00000010u) != 0;
  return value;
}
inline bool EvalConfig::has_max_evals() const {
  return _internal_has_max_evals();
}
inline void EvalConfig::clear_max_evals() {
  _impl_.max_evals_ = 0u;
  _impl_._has_bits_[0] &= ~0x00000010u;
}
inline uint32_t EvalConfig::_internal_max_evals() const {
  return _impl_.max_evals_;
}
inline uint32_t EvalConfig::max_evals() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.max_evals)
  return _internal_max_evals();
}
inline void EvalConfig::_internal_set_max_evals(uint32_t value) {
  _impl_._has_bits_[0] |= 0x00000010u;
  _impl_.max_evals_ = value;
}
inline void EvalConfig::set_max_evals(uint32_t value) {
  _internal_set_max_evals(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.max_evals)
}

// optional bool save_graph = 5 [default = false];
inline bool EvalConfig::_internal_has_save_graph() const {
  bool value = (_impl_._has_bits_[0] & 0x00000020u) != 0;
  return value;
}
inline bool EvalConfig::has_save_graph() const {
  return _internal_has_save_graph();
}
inline void EvalConfig::clear_save_graph() {
  _impl_.save_graph_ = false;
  _impl_._has_bits_[0] &= ~0x00000020u;
}
inline bool EvalConfig::_internal_save_graph() const {
  return _impl_.save_graph_;
}
inline bool EvalConfig::save_graph() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.save_graph)
  return _internal_save_graph();
}
inline void EvalConfig::_internal_set_save_graph(bool value) {
  _impl_._has_bits_[0] |= 0x00000020u;
  _impl_.save_graph_ = value;
}
inline void EvalConfig::set_save_graph(bool value) {
  _internal_set_save_graph(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.save_graph)
}

// optional string visualization_export_dir = 6 [default = ""];
inline bool EvalConfig::_internal_has_visualization_export_dir() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool EvalConfig::has_visualization_export_dir() const {
  return _internal_has_visualization_export_dir();
}
inline void EvalConfig::clear_visualization_export_dir() {
  _impl_.visualization_export_dir_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const std::string& EvalConfig::visualization_export_dir() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.visualization_export_dir)
  return _internal_visualization_export_dir();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void EvalConfig::set_visualization_export_dir(ArgT0&& arg0, ArgT... args) {
 _impl_._has_bits_[0] |= 0x00000001u;
 _impl_.visualization_export_dir_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.visualization_export_dir)
}
inline std::string* EvalConfig::mutable_visualization_export_dir() {
  std::string* _s = _internal_mutable_visualization_export_dir();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.EvalConfig.visualization_export_dir)
  return _s;
}
inline const std::string& EvalConfig::_internal_visualization_export_dir() const {
  return _impl_.visualization_export_dir_.Get();
}
inline void EvalConfig::_internal_set_visualization_export_dir(const std::string& value) {
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.visualization_export_dir_.Set(value, GetArenaForAllocation());
}
inline std::string* EvalConfig::_internal_mutable_visualization_export_dir() {
  _impl_._has_bits_[0] |= 0x00000001u;
  return _impl_.visualization_export_dir_.Mutable(GetArenaForAllocation());
}
inline std::string* EvalConfig::release_visualization_export_dir() {
  // @@protoc_insertion_point(field_release:object_detection.protos.EvalConfig.visualization_export_dir)
  if (!_internal_has_visualization_export_dir()) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001u;
  auto* p = _impl_.visualization_export_dir_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.visualization_export_dir_.IsDefault()) {
    _impl_.visualization_export_dir_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void EvalConfig::set_allocated_visualization_export_dir(std::string* visualization_export_dir) {
  if (visualization_export_dir != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.visualization_export_dir_.SetAllocated(visualization_export_dir, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.visualization_export_dir_.IsDefault()) {
    _impl_.visualization_export_dir_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.EvalConfig.visualization_export_dir)
}

// optional string eval_master = 7 [default = ""];
inline bool EvalConfig::_internal_has_eval_master() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool EvalConfig::has_eval_master() const {
  return _internal_has_eval_master();
}
inline void EvalConfig::clear_eval_master() {
  _impl_.eval_master_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline const std::string& EvalConfig::eval_master() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.eval_master)
  return _internal_eval_master();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void EvalConfig::set_eval_master(ArgT0&& arg0, ArgT... args) {
 _impl_._has_bits_[0] |= 0x00000002u;
 _impl_.eval_master_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.eval_master)
}
inline std::string* EvalConfig::mutable_eval_master() {
  std::string* _s = _internal_mutable_eval_master();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.EvalConfig.eval_master)
  return _s;
}
inline const std::string& EvalConfig::_internal_eval_master() const {
  return _impl_.eval_master_.Get();
}
inline void EvalConfig::_internal_set_eval_master(const std::string& value) {
  _impl_._has_bits_[0] |= 0x00000002u;
  _impl_.eval_master_.Set(value, GetArenaForAllocation());
}
inline std::string* EvalConfig::_internal_mutable_eval_master() {
  _impl_._has_bits_[0] |= 0x00000002u;
  return _impl_.eval_master_.Mutable(GetArenaForAllocation());
}
inline std::string* EvalConfig::release_eval_master() {
  // @@protoc_insertion_point(field_release:object_detection.protos.EvalConfig.eval_master)
  if (!_internal_has_eval_master()) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000002u;
  auto* p = _impl_.eval_master_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.eval_master_.IsDefault()) {
    _impl_.eval_master_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void EvalConfig::set_allocated_eval_master(std::string* eval_master) {
  if (eval_master != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  _impl_.eval_master_.SetAllocated(eval_master, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.eval_master_.IsDefault()) {
    _impl_.eval_master_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.EvalConfig.eval_master)
}

// repeated string metrics_set = 8;
inline int EvalConfig::_internal_metrics_set_size() const {
  return _impl_.metrics_set_.size();
}
inline int EvalConfig::metrics_set_size() const {
  return _internal_metrics_set_size();
}
inline void EvalConfig::clear_metrics_set() {
  _impl_.metrics_set_.Clear();
}
inline std::string* EvalConfig::add_metrics_set() {
  std::string* _s = _internal_add_metrics_set();
  // @@protoc_insertion_point(field_add_mutable:object_detection.protos.EvalConfig.metrics_set)
  return _s;
}
inline const std::string& EvalConfig::_internal_metrics_set(int index) const {
  return _impl_.metrics_set_.Get(index);
}
inline const std::string& EvalConfig::metrics_set(int index) const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.metrics_set)
  return _internal_metrics_set(index);
}
inline std::string* EvalConfig::mutable_metrics_set(int index) {
  // @@protoc_insertion_point(field_mutable:object_detection.protos.EvalConfig.metrics_set)
  return _impl_.metrics_set_.Mutable(index);
}
inline void EvalConfig::set_metrics_set(int index, const std::string& value) {
  _impl_.metrics_set_.Mutable(index)->assign(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.metrics_set)
}
inline void EvalConfig::set_metrics_set(int index, std::string&& value) {
  _impl_.metrics_set_.Mutable(index)->assign(std::move(value));
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.metrics_set)
}
inline void EvalConfig::set_metrics_set(int index, const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  _impl_.metrics_set_.Mutable(index)->assign(value);
  // @@protoc_insertion_point(field_set_char:object_detection.protos.EvalConfig.metrics_set)
}
inline void EvalConfig::set_metrics_set(int index, const char* value, size_t size) {
  _impl_.metrics_set_.Mutable(index)->assign(
    reinterpret_cast<const char*>(value), size);
  // @@protoc_insertion_point(field_set_pointer:object_detection.protos.EvalConfig.metrics_set)
}
inline std::string* EvalConfig::_internal_add_metrics_set() {
  return _impl_.metrics_set_.Add();
}
inline void EvalConfig::add_metrics_set(const std::string& value) {
  _impl_.metrics_set_.Add()->assign(value);
  // @@protoc_insertion_point(field_add:object_detection.protos.EvalConfig.metrics_set)
}
inline void EvalConfig::add_metrics_set(std::string&& value) {
  _impl_.metrics_set_.Add(std::move(value));
  // @@protoc_insertion_point(field_add:object_detection.protos.EvalConfig.metrics_set)
}
inline void EvalConfig::add_metrics_set(const char* value) {
  GOOGLE_DCHECK(value != nullptr);
  _impl_.metrics_set_.Add()->assign(value);
  // @@protoc_insertion_point(field_add_char:object_detection.protos.EvalConfig.metrics_set)
}
inline void EvalConfig::add_metrics_set(const char* value, size_t size) {
  _impl_.metrics_set_.Add()->assign(reinterpret_cast<const char*>(value), size);
  // @@protoc_insertion_point(field_add_pointer:object_detection.protos.EvalConfig.metrics_set)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>&
EvalConfig::metrics_set() const {
  // @@protoc_insertion_point(field_list:object_detection.protos.EvalConfig.metrics_set)
  return _impl_.metrics_set_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField<std::string>*
EvalConfig::mutable_metrics_set() {
  // @@protoc_insertion_point(field_mutable_list:object_detection.protos.EvalConfig.metrics_set)
  return &_impl_.metrics_set_;
}

// optional string export_path = 9 [default = ""];
inline bool EvalConfig::_internal_has_export_path() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool EvalConfig::has_export_path() const {
  return _internal_has_export_path();
}
inline void EvalConfig::clear_export_path() {
  _impl_.export_path_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline const std::string& EvalConfig::export_path() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.export_path)
  return _internal_export_path();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void EvalConfig::set_export_path(ArgT0&& arg0, ArgT... args) {
 _impl_._has_bits_[0] |= 0x00000004u;
 _impl_.export_path_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.export_path)
}
inline std::string* EvalConfig::mutable_export_path() {
  std::string* _s = _internal_mutable_export_path();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.EvalConfig.export_path)
  return _s;
}
inline const std::string& EvalConfig::_internal_export_path() const {
  return _impl_.export_path_.Get();
}
inline void EvalConfig::_internal_set_export_path(const std::string& value) {
  _impl_._has_bits_[0] |= 0x00000004u;
  _impl_.export_path_.Set(value, GetArenaForAllocation());
}
inline std::string* EvalConfig::_internal_mutable_export_path() {
  _impl_._has_bits_[0] |= 0x00000004u;
  return _impl_.export_path_.Mutable(GetArenaForAllocation());
}
inline std::string* EvalConfig::release_export_path() {
  // @@protoc_insertion_point(field_release:object_detection.protos.EvalConfig.export_path)
  if (!_internal_has_export_path()) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000004u;
  auto* p = _impl_.export_path_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.export_path_.IsDefault()) {
    _impl_.export_path_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void EvalConfig::set_allocated_export_path(std::string* export_path) {
  if (export_path != nullptr) {
    _impl_._has_bits_[0] |= 0x00000004u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000004u;
  }
  _impl_.export_path_.SetAllocated(export_path, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.export_path_.IsDefault()) {
    _impl_.export_path_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.EvalConfig.export_path)
}

// optional bool ignore_groundtruth = 10 [default = false];
inline bool EvalConfig::_internal_has_ignore_groundtruth() const {
  bool value = (_impl_._has_bits_[0] & 0x00000040u) != 0;
  return value;
}
inline bool EvalConfig::has_ignore_groundtruth() const {
  return _internal_has_ignore_groundtruth();
}
inline void EvalConfig::clear_ignore_groundtruth() {
  _impl_.ignore_groundtruth_ = false;
  _impl_._has_bits_[0] &= ~0x00000040u;
}
inline bool EvalConfig::_internal_ignore_groundtruth() const {
  return _impl_.ignore_groundtruth_;
}
inline bool EvalConfig::ignore_groundtruth() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.ignore_groundtruth)
  return _internal_ignore_groundtruth();
}
inline void EvalConfig::_internal_set_ignore_groundtruth(bool value) {
  _impl_._has_bits_[0] |= 0x00000040u;
  _impl_.ignore_groundtruth_ = value;
}
inline void EvalConfig::set_ignore_groundtruth(bool value) {
  _internal_set_ignore_groundtruth(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.ignore_groundtruth)
}

// optional bool use_moving_averages = 11 [default = false];
inline bool EvalConfig::_internal_has_use_moving_averages() const {
  bool value = (_impl_._has_bits_[0] & 0x00000080u) != 0;
  return value;
}
inline bool EvalConfig::has_use_moving_averages() const {
  return _internal_has_use_moving_averages();
}
inline void EvalConfig::clear_use_moving_averages() {
  _impl_.use_moving_averages_ = false;
  _impl_._has_bits_[0] &= ~0x00000080u;
}
inline bool EvalConfig::_internal_use_moving_averages() const {
  return _impl_.use_moving_averages_;
}
inline bool EvalConfig::use_moving_averages() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.use_moving_averages)
  return _internal_use_moving_averages();
}
inline void EvalConfig::_internal_set_use_moving_averages(bool value) {
  _impl_._has_bits_[0] |= 0x00000080u;
  _impl_.use_moving_averages_ = value;
}
inline void EvalConfig::set_use_moving_averages(bool value) {
  _internal_set_use_moving_averages(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.use_moving_averages)
}

// optional bool eval_instance_masks = 12 [default = false];
inline bool EvalConfig::_internal_has_eval_instance_masks() const {
  bool value = (_impl_._has_bits_[0] & 0x00000100u) != 0;
  return value;
}
inline bool EvalConfig::has_eval_instance_masks() const {
  return _internal_has_eval_instance_masks();
}
inline void EvalConfig::clear_eval_instance_masks() {
  _impl_.eval_instance_masks_ = false;
  _impl_._has_bits_[0] &= ~0x00000100u;
}
inline bool EvalConfig::_internal_eval_instance_masks() const {
  return _impl_.eval_instance_masks_;
}
inline bool EvalConfig::eval_instance_masks() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.eval_instance_masks)
  return _internal_eval_instance_masks();
}
inline void EvalConfig::_internal_set_eval_instance_masks(bool value) {
  _impl_._has_bits_[0] |= 0x00000100u;
  _impl_.eval_instance_masks_ = value;
}
inline void EvalConfig::set_eval_instance_masks(bool value) {
  _internal_set_eval_instance_masks(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.eval_instance_masks)
}

// optional float min_score_threshold = 13 [default = 0.5];
inline bool EvalConfig::_internal_has_min_score_threshold() const {
  bool value = (_impl_._has_bits_[0] & 0x00040000u) != 0;
  return value;
}
inline bool EvalConfig::has_min_score_threshold() const {
  return _internal_has_min_score_threshold();
}
inline void EvalConfig::clear_min_score_threshold() {
  _impl_.min_score_threshold_ = 0.5f;
  _impl_._has_bits_[0] &= ~0x00040000u;
}
inline float EvalConfig::_internal_min_score_threshold() const {
  return _impl_.min_score_threshold_;
}
inline float EvalConfig::min_score_threshold() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.min_score_threshold)
  return _internal_min_score_threshold();
}
inline void EvalConfig::_internal_set_min_score_threshold(float value) {
  _impl_._has_bits_[0] |= 0x00040000u;
  _impl_.min_score_threshold_ = value;
}
inline void EvalConfig::set_min_score_threshold(float value) {
  _internal_set_min_score_threshold(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.min_score_threshold)
}

// optional int32 max_num_boxes_to_visualize = 14 [default = 20];
inline bool EvalConfig::_internal_has_max_num_boxes_to_visualize() const {
  bool value = (_impl_._has_bits_[0] & 0x00080000u) != 0;
  return value;
}
inline bool EvalConfig::has_max_num_boxes_to_visualize() const {
  return _internal_has_max_num_boxes_to_visualize();
}
inline void EvalConfig::clear_max_num_boxes_to_visualize() {
  _impl_.max_num_boxes_to_visualize_ = 20;
  _impl_._has_bits_[0] &= ~0x00080000u;
}
inline int32_t EvalConfig::_internal_max_num_boxes_to_visualize() const {
  return _impl_.max_num_boxes_to_visualize_;
}
inline int32_t EvalConfig::max_num_boxes_to_visualize() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.max_num_boxes_to_visualize)
  return _internal_max_num_boxes_to_visualize();
}
inline void EvalConfig::_internal_set_max_num_boxes_to_visualize(int32_t value) {
  _impl_._has_bits_[0] |= 0x00080000u;
  _impl_.max_num_boxes_to_visualize_ = value;
}
inline void EvalConfig::set_max_num_boxes_to_visualize(int32_t value) {
  _internal_set_max_num_boxes_to_visualize(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.max_num_boxes_to_visualize)
}

// optional bool skip_scores = 15 [default = false];
inline bool EvalConfig::_internal_has_skip_scores() const {
  bool value = (_impl_._has_bits_[0] & 0x00000200u) != 0;
  return value;
}
inline bool EvalConfig::has_skip_scores() const {
  return _internal_has_skip_scores();
}
inline void EvalConfig::clear_skip_scores() {
  _impl_.skip_scores_ = false;
  _impl_._has_bits_[0] &= ~0x00000200u;
}
inline bool EvalConfig::_internal_skip_scores() const {
  return _impl_.skip_scores_;
}
inline bool EvalConfig::skip_scores() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.skip_scores)
  return _internal_skip_scores();
}
inline void EvalConfig::_internal_set_skip_scores(bool value) {
  _impl_._has_bits_[0] |= 0x00000200u;
  _impl_.skip_scores_ = value;
}
inline void EvalConfig::set_skip_scores(bool value) {
  _internal_set_skip_scores(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.skip_scores)
}

// optional bool skip_labels = 16 [default = false];
inline bool EvalConfig::_internal_has_skip_labels() const {
  bool value = (_impl_._has_bits_[0] & 0x00000400u) != 0;
  return value;
}
inline bool EvalConfig::has_skip_labels() const {
  return _internal_has_skip_labels();
}
inline void EvalConfig::clear_skip_labels() {
  _impl_.skip_labels_ = false;
  _impl_._has_bits_[0] &= ~0x00000400u;
}
inline bool EvalConfig::_internal_skip_labels() const {
  return _impl_.skip_labels_;
}
inline bool EvalConfig::skip_labels() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.skip_labels)
  return _internal_skip_labels();
}
inline void EvalConfig::_internal_set_skip_labels(bool value) {
  _impl_._has_bits_[0] |= 0x00000400u;
  _impl_.skip_labels_ = value;
}
inline void EvalConfig::set_skip_labels(bool value) {
  _internal_set_skip_labels(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.skip_labels)
}

// optional bool visualize_groundtruth_boxes = 17 [default = false];
inline bool EvalConfig::_internal_has_visualize_groundtruth_boxes() const {
  bool value = (_impl_._has_bits_[0] & 0x00000800u) != 0;
  return value;
}
inline bool EvalConfig::has_visualize_groundtruth_boxes() const {
  return _internal_has_visualize_groundtruth_boxes();
}
inline void EvalConfig::clear_visualize_groundtruth_boxes() {
  _impl_.visualize_groundtruth_boxes_ = false;
  _impl_._has_bits_[0] &= ~0x00000800u;
}
inline bool EvalConfig::_internal_visualize_groundtruth_boxes() const {
  return _impl_.visualize_groundtruth_boxes_;
}
inline bool EvalConfig::visualize_groundtruth_boxes() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.visualize_groundtruth_boxes)
  return _internal_visualize_groundtruth_boxes();
}
inline void EvalConfig::_internal_set_visualize_groundtruth_boxes(bool value) {
  _impl_._has_bits_[0] |= 0x00000800u;
  _impl_.visualize_groundtruth_boxes_ = value;
}
inline void EvalConfig::set_visualize_groundtruth_boxes(bool value) {
  _internal_set_visualize_groundtruth_boxes(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.visualize_groundtruth_boxes)
}

// optional string groundtruth_box_visualization_color = 18 [default = "black"];
inline bool EvalConfig::_internal_has_groundtruth_box_visualization_color() const {
  bool value = (_impl_._has_bits_[0] & 0x00000008u) != 0;
  return value;
}
inline bool EvalConfig::has_groundtruth_box_visualization_color() const {
  return _internal_has_groundtruth_box_visualization_color();
}
inline void EvalConfig::clear_groundtruth_box_visualization_color() {
  _impl_.groundtruth_box_visualization_color_.ClearToDefault(::object_detection::protos::EvalConfig::Impl_::_i_give_permission_to_break_this_code_default_groundtruth_box_visualization_color_, GetArenaForAllocation());
  _impl_._has_bits_[0] &= ~0x00000008u;
}
inline const std::string& EvalConfig::groundtruth_box_visualization_color() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.groundtruth_box_visualization_color)
  if (_impl_.groundtruth_box_visualization_color_.IsDefault()) return Impl_::_i_give_permission_to_break_this_code_default_groundtruth_box_visualization_color_.get();
  return _internal_groundtruth_box_visualization_color();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void EvalConfig::set_groundtruth_box_visualization_color(ArgT0&& arg0, ArgT... args) {
 _impl_._has_bits_[0] |= 0x00000008u;
 _impl_.groundtruth_box_visualization_color_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.groundtruth_box_visualization_color)
}
inline std::string* EvalConfig::mutable_groundtruth_box_visualization_color() {
  std::string* _s = _internal_mutable_groundtruth_box_visualization_color();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.EvalConfig.groundtruth_box_visualization_color)
  return _s;
}
inline const std::string& EvalConfig::_internal_groundtruth_box_visualization_color() const {
  return _impl_.groundtruth_box_visualization_color_.Get();
}
inline void EvalConfig::_internal_set_groundtruth_box_visualization_color(const std::string& value) {
  _impl_._has_bits_[0] |= 0x00000008u;
  _impl_.groundtruth_box_visualization_color_.Set(value, GetArenaForAllocation());
}
inline std::string* EvalConfig::_internal_mutable_groundtruth_box_visualization_color() {
  _impl_._has_bits_[0] |= 0x00000008u;
  return _impl_.groundtruth_box_visualization_color_.Mutable(::object_detection::protos::EvalConfig::Impl_::_i_give_permission_to_break_this_code_default_groundtruth_box_visualization_color_, GetArenaForAllocation());
}
inline std::string* EvalConfig::release_groundtruth_box_visualization_color() {
  // @@protoc_insertion_point(field_release:object_detection.protos.EvalConfig.groundtruth_box_visualization_color)
  if (!_internal_has_groundtruth_box_visualization_color()) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000008u;
  auto* p = _impl_.groundtruth_box_visualization_color_.Release();
  return p;
}
inline void EvalConfig::set_allocated_groundtruth_box_visualization_color(std::string* groundtruth_box_visualization_color) {
  if (groundtruth_box_visualization_color != nullptr) {
    _impl_._has_bits_[0] |= 0x00000008u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000008u;
  }
  _impl_.groundtruth_box_visualization_color_.SetAllocated(groundtruth_box_visualization_color, GetArenaForAllocation());
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.EvalConfig.groundtruth_box_visualization_color)
}

// optional bool keep_image_id_for_visualization_export = 19 [default = false];
inline bool EvalConfig::_internal_has_keep_image_id_for_visualization_export() const {
  bool value = (_impl_._has_bits_[0] & 0x00001000u) != 0;
  return value;
}
inline bool EvalConfig::has_keep_image_id_for_visualization_export() const {
  return _internal_has_keep_image_id_for_visualization_export();
}
inline void EvalConfig::clear_keep_image_id_for_visualization_export() {
  _impl_.keep_image_id_for_visualization_export_ = false;
  _impl_._has_bits_[0] &= ~0x00001000u;
}
inline bool EvalConfig::_internal_keep_image_id_for_visualization_export() const {
  return _impl_.keep_image_id_for_visualization_export_;
}
inline bool EvalConfig::keep_image_id_for_visualization_export() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.keep_image_id_for_visualization_export)
  return _internal_keep_image_id_for_visualization_export();
}
inline void EvalConfig::_internal_set_keep_image_id_for_visualization_export(bool value) {
  _impl_._has_bits_[0] |= 0x00001000u;
  _impl_.keep_image_id_for_visualization_export_ = value;
}
inline void EvalConfig::set_keep_image_id_for_visualization_export(bool value) {
  _internal_set_keep_image_id_for_visualization_export(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.keep_image_id_for_visualization_export)
}

// optional bool retain_original_images = 23 [default = true];
inline bool EvalConfig::_internal_has_retain_original_images() const {
  bool value = (_impl_._has_bits_[0] & 0x00100000u) != 0;
  return value;
}
inline bool EvalConfig::has_retain_original_images() const {
  return _internal_has_retain_original_images();
}
inline void EvalConfig::clear_retain_original_images() {
  _impl_.retain_original_images_ = true;
  _impl_._has_bits_[0] &= ~0x00100000u;
}
inline bool EvalConfig::_internal_retain_original_images() const {
  return _impl_.retain_original_images_;
}
inline bool EvalConfig::retain_original_images() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.retain_original_images)
  return _internal_retain_original_images();
}
inline void EvalConfig::_internal_set_retain_original_images(bool value) {
  _impl_._has_bits_[0] |= 0x00100000u;
  _impl_.retain_original_images_ = value;
}
inline void EvalConfig::set_retain_original_images(bool value) {
  _internal_set_retain_original_images(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.retain_original_images)
}

// optional bool include_metrics_per_category = 24 [default = false];
inline bool EvalConfig::_internal_has_include_metrics_per_category() const {
  bool value = (_impl_._has_bits_[0] & 0x00002000u) != 0;
  return value;
}
inline bool EvalConfig::has_include_metrics_per_category() const {
  return _internal_has_include_metrics_per_category();
}
inline void EvalConfig::clear_include_metrics_per_category() {
  _impl_.include_metrics_per_category_ = false;
  _impl_._has_bits_[0] &= ~0x00002000u;
}
inline bool EvalConfig::_internal_include_metrics_per_category() const {
  return _impl_.include_metrics_per_category_;
}
inline bool EvalConfig::include_metrics_per_category() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.include_metrics_per_category)
  return _internal_include_metrics_per_category();
}
inline void EvalConfig::_internal_set_include_metrics_per_category(bool value) {
  _impl_._has_bits_[0] |= 0x00002000u;
  _impl_.include_metrics_per_category_ = value;
}
inline void EvalConfig::set_include_metrics_per_category(bool value) {
  _internal_set_include_metrics_per_category(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.include_metrics_per_category)
}

// optional float recall_lower_bound = 26 [default = 0];
inline bool EvalConfig::_internal_has_recall_lower_bound() const {
  bool value = (_impl_._has_bits_[0] & 0x00004000u) != 0;
  return value;
}
inline bool EvalConfig::has_recall_lower_bound() const {
  return _internal_has_recall_lower_bound();
}
inline void EvalConfig::clear_recall_lower_bound() {
  _impl_.recall_lower_bound_ = 0;
  _impl_._has_bits_[0] &= ~0x00004000u;
}
inline float EvalConfig::_internal_recall_lower_bound() const {
  return _impl_.recall_lower_bound_;
}
inline float EvalConfig::recall_lower_bound() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.recall_lower_bound)
  return _internal_recall_lower_bound();
}
inline void EvalConfig::_internal_set_recall_lower_bound(float value) {
  _impl_._has_bits_[0] |= 0x00004000u;
  _impl_.recall_lower_bound_ = value;
}
inline void EvalConfig::set_recall_lower_bound(float value) {
  _internal_set_recall_lower_bound(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.recall_lower_bound)
}

// optional float recall_upper_bound = 27 [default = 1];
inline bool EvalConfig::_internal_has_recall_upper_bound() const {
  bool value = (_impl_._has_bits_[0] & 0x00400000u) != 0;
  return value;
}
inline bool EvalConfig::has_recall_upper_bound() const {
  return _internal_has_recall_upper_bound();
}
inline void EvalConfig::clear_recall_upper_bound() {
  _impl_.recall_upper_bound_ = 1;
  _impl_._has_bits_[0] &= ~0x00400000u;
}
inline float EvalConfig::_internal_recall_upper_bound() const {
  return _impl_.recall_upper_bound_;
}
inline float EvalConfig::recall_upper_bound() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.EvalConfig.recall_upper_bound)
  return _internal_recall_upper_bound();
}
inline void EvalConfig::_internal_set_recall_upper_bound(float value) {
  _impl_._has_bits_[0] |= 0x00400000u;
  _impl_.recall_upper_bound_ = value;
}
inline void EvalConfig::set_recall_upper_bound(float value) {
  _internal_set_recall_upper_bound(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.EvalConfig.recall_upper_bound)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace object_detection

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2feval_2eproto
