// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/sampler.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fsampler_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fsampler_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/map.h>  // IWYU pragma: export
#include <google/protobuf/map_entry.h>
#include <google/protobuf/map_field_inl.h>
#include <google/protobuf/unknown_field_set.h>
#include "second/protos/preprocess.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_second_2fprotos_2fsampler_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_second_2fprotos_2fsampler_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_second_2fprotos_2fsampler_2eproto;
namespace second {
namespace protos {
class Group;
struct GroupDefaultTypeInternal;
extern GroupDefaultTypeInternal _Group_default_instance_;
class Group_NameToMaxNumEntry_DoNotUse;
struct Group_NameToMaxNumEntry_DoNotUseDefaultTypeInternal;
extern Group_NameToMaxNumEntry_DoNotUseDefaultTypeInternal _Group_NameToMaxNumEntry_DoNotUse_default_instance_;
class Sampler;
struct SamplerDefaultTypeInternal;
extern SamplerDefaultTypeInternal _Sampler_default_instance_;
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> ::second::protos::Group* Arena::CreateMaybeMessage<::second::protos::Group>(Arena*);
template<> ::second::protos::Group_NameToMaxNumEntry_DoNotUse* Arena::CreateMaybeMessage<::second::protos::Group_NameToMaxNumEntry_DoNotUse>(Arena*);
template<> ::second::protos::Sampler* Arena::CreateMaybeMessage<::second::protos::Sampler>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace second {
namespace protos {

// ===================================================================

class Group_NameToMaxNumEntry_DoNotUse : public ::PROTOBUF_NAMESPACE_ID::internal::MapEntry<Group_NameToMaxNumEntry_DoNotUse, 
    std::string, uint32_t,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_STRING,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_UINT32> {
public:
  typedef ::PROTOBUF_NAMESPACE_ID::internal::MapEntry<Group_NameToMaxNumEntry_DoNotUse, 
    std::string, uint32_t,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_STRING,
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_UINT32> SuperType;
  Group_NameToMaxNumEntry_DoNotUse();
  explicit PROTOBUF_CONSTEXPR Group_NameToMaxNumEntry_DoNotUse(
      ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);
  explicit Group_NameToMaxNumEntry_DoNotUse(::PROTOBUF_NAMESPACE_ID::Arena* arena);
  void MergeFrom(const Group_NameToMaxNumEntry_DoNotUse& other);
  static const Group_NameToMaxNumEntry_DoNotUse* internal_default_instance() { return reinterpret_cast<const Group_NameToMaxNumEntry_DoNotUse*>(&_Group_NameToMaxNumEntry_DoNotUse_default_instance_); }
  static bool ValidateKey(std::string* s) {
    return ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(s->data(), static_cast<int>(s->size()), ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::PARSE, "second.protos.Group.NameToMaxNumEntry.key");
 }
  static bool ValidateValue(void*) { return true; }
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;
  friend struct ::TableStruct_second_2fprotos_2fsampler_2eproto;
};

// -------------------------------------------------------------------

class Group final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.Group) */ {
 public:
  inline Group() : Group(nullptr) {}
  ~Group() override;
  explicit PROTOBUF_CONSTEXPR Group(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Group(const Group& from);
  Group(Group&& from) noexcept
    : Group() {
    *this = ::std::move(from);
  }

  inline Group& operator=(const Group& from) {
    CopyFrom(from);
    return *this;
  }
  inline Group& operator=(Group&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Group& default_instance() {
    return *internal_default_instance();
  }
  static inline const Group* internal_default_instance() {
    return reinterpret_cast<const Group*>(
               &_Group_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(Group& a, Group& b) {
    a.Swap(&b);
  }
  inline void Swap(Group* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Group* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Group* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<Group>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const Group& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const Group& from) {
    Group::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Group* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.Group";
  }
  protected:
  explicit Group(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  private:
  static void ArenaDtor(void* object);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------


  // accessors -------------------------------------------------------

  enum : int {
    kNameToMaxNumFieldNumber = 1,
  };
  // map<string, uint32> name_to_max_num = 1;
  int name_to_max_num_size() const;
  private:
  int _internal_name_to_max_num_size() const;
  public:
  void clear_name_to_max_num();
  private:
  const ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >&
      _internal_name_to_max_num() const;
  ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >*
      _internal_mutable_name_to_max_num();
  public:
  const ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >&
      name_to_max_num() const;
  ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >*
      mutable_name_to_max_num();

  // @@protoc_insertion_point(class_scope:second.protos.Group)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::MapField<
        Group_NameToMaxNumEntry_DoNotUse,
        std::string, uint32_t,
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_STRING,
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::TYPE_UINT32> name_to_max_num_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fsampler_2eproto;
};
// -------------------------------------------------------------------

class Sampler final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.Sampler) */ {
 public:
  inline Sampler() : Sampler(nullptr) {}
  ~Sampler() override;
  explicit PROTOBUF_CONSTEXPR Sampler(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Sampler(const Sampler& from);
  Sampler(Sampler&& from) noexcept
    : Sampler() {
    *this = ::std::move(from);
  }

  inline Sampler& operator=(const Sampler& from) {
    CopyFrom(from);
    return *this;
  }
  inline Sampler& operator=(Sampler&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Sampler& default_instance() {
    return *internal_default_instance();
  }
  static inline const Sampler* internal_default_instance() {
    return reinterpret_cast<const Sampler*>(
               &_Sampler_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    2;

  friend void swap(Sampler& a, Sampler& b) {
    a.Swap(&b);
  }
  inline void Swap(Sampler* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Sampler* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Sampler* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<Sampler>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const Sampler& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const Sampler& from) {
    Sampler::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Sampler* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.Sampler";
  }
  protected:
  explicit Sampler(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kSampleGroupsFieldNumber = 2,
    kDatabasePrepStepsFieldNumber = 3,
    kGlobalRandomRotationRangePerObjectFieldNumber = 4,
    kDatabaseInfoPathFieldNumber = 1,
    kRateFieldNumber = 5,
  };
  // repeated .second.protos.Group sample_groups = 2;
  int sample_groups_size() const;
  private:
  int _internal_sample_groups_size() const;
  public:
  void clear_sample_groups();
  ::second::protos::Group* mutable_sample_groups(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::Group >*
      mutable_sample_groups();
  private:
  const ::second::protos::Group& _internal_sample_groups(int index) const;
  ::second::protos::Group* _internal_add_sample_groups();
  public:
  const ::second::protos::Group& sample_groups(int index) const;
  ::second::protos::Group* add_sample_groups();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::Group >&
      sample_groups() const;

  // repeated .second.protos.DatabasePreprocessingStep database_prep_steps = 3;
  int database_prep_steps_size() const;
  private:
  int _internal_database_prep_steps_size() const;
  public:
  void clear_database_prep_steps();
  ::second::protos::DatabasePreprocessingStep* mutable_database_prep_steps(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::DatabasePreprocessingStep >*
      mutable_database_prep_steps();
  private:
  const ::second::protos::DatabasePreprocessingStep& _internal_database_prep_steps(int index) const;
  ::second::protos::DatabasePreprocessingStep* _internal_add_database_prep_steps();
  public:
  const ::second::protos::DatabasePreprocessingStep& database_prep_steps(int index) const;
  ::second::protos::DatabasePreprocessingStep* add_database_prep_steps();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::DatabasePreprocessingStep >&
      database_prep_steps() const;

  // repeated float global_random_rotation_range_per_object = 4;
  int global_random_rotation_range_per_object_size() const;
  private:
  int _internal_global_random_rotation_range_per_object_size() const;
  public:
  void clear_global_random_rotation_range_per_object();
  private:
  float _internal_global_random_rotation_range_per_object(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      _internal_global_random_rotation_range_per_object() const;
  void _internal_add_global_random_rotation_range_per_object(float value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      _internal_mutable_global_random_rotation_range_per_object();
  public:
  float global_random_rotation_range_per_object(int index) const;
  void set_global_random_rotation_range_per_object(int index, float value);
  void add_global_random_rotation_range_per_object(float value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
      global_random_rotation_range_per_object() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
      mutable_global_random_rotation_range_per_object();

  // string database_info_path = 1;
  void clear_database_info_path();
  const std::string& database_info_path() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_database_info_path(ArgT0&& arg0, ArgT... args);
  std::string* mutable_database_info_path();
  PROTOBUF_NODISCARD std::string* release_database_info_path();
  void set_allocated_database_info_path(std::string* database_info_path);
  private:
  const std::string& _internal_database_info_path() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_database_info_path(const std::string& value);
  std::string* _internal_mutable_database_info_path();
  public:

  // float rate = 5;
  void clear_rate();
  float rate() const;
  void set_rate(float value);
  private:
  float _internal_rate() const;
  void _internal_set_rate(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.Sampler)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::Group > sample_groups_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::DatabasePreprocessingStep > database_prep_steps_;
    ::PROTOBUF_NAMESPACE_ID::RepeatedField< float > global_random_rotation_range_per_object_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr database_info_path_;
    float rate_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2fsampler_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Group

// map<string, uint32> name_to_max_num = 1;
inline int Group::_internal_name_to_max_num_size() const {
  return _impl_.name_to_max_num_.size();
}
inline int Group::name_to_max_num_size() const {
  return _internal_name_to_max_num_size();
}
inline void Group::clear_name_to_max_num() {
  _impl_.name_to_max_num_.Clear();
}
inline const ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >&
Group::_internal_name_to_max_num() const {
  return _impl_.name_to_max_num_.GetMap();
}
inline const ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >&
Group::name_to_max_num() const {
  // @@protoc_insertion_point(field_map:second.protos.Group.name_to_max_num)
  return _internal_name_to_max_num();
}
inline ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >*
Group::_internal_mutable_name_to_max_num() {
  return _impl_.name_to_max_num_.MutableMap();
}
inline ::PROTOBUF_NAMESPACE_ID::Map< std::string, uint32_t >*
Group::mutable_name_to_max_num() {
  // @@protoc_insertion_point(field_mutable_map:second.protos.Group.name_to_max_num)
  return _internal_mutable_name_to_max_num();
}

// -------------------------------------------------------------------

// Sampler

// string database_info_path = 1;
inline void Sampler::clear_database_info_path() {
  _impl_.database_info_path_.ClearToEmpty();
}
inline const std::string& Sampler::database_info_path() const {
  // @@protoc_insertion_point(field_get:second.protos.Sampler.database_info_path)
  return _internal_database_info_path();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void Sampler::set_database_info_path(ArgT0&& arg0, ArgT... args) {
 
 _impl_.database_info_path_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:second.protos.Sampler.database_info_path)
}
inline std::string* Sampler::mutable_database_info_path() {
  std::string* _s = _internal_mutable_database_info_path();
  // @@protoc_insertion_point(field_mutable:second.protos.Sampler.database_info_path)
  return _s;
}
inline const std::string& Sampler::_internal_database_info_path() const {
  return _impl_.database_info_path_.Get();
}
inline void Sampler::_internal_set_database_info_path(const std::string& value) {
  
  _impl_.database_info_path_.Set(value, GetArenaForAllocation());
}
inline std::string* Sampler::_internal_mutable_database_info_path() {
  
  return _impl_.database_info_path_.Mutable(GetArenaForAllocation());
}
inline std::string* Sampler::release_database_info_path() {
  // @@protoc_insertion_point(field_release:second.protos.Sampler.database_info_path)
  return _impl_.database_info_path_.Release();
}
inline void Sampler::set_allocated_database_info_path(std::string* database_info_path) {
  if (database_info_path != nullptr) {
    
  } else {
    
  }
  _impl_.database_info_path_.SetAllocated(database_info_path, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.database_info_path_.IsDefault()) {
    _impl_.database_info_path_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:second.protos.Sampler.database_info_path)
}

// repeated .second.protos.Group sample_groups = 2;
inline int Sampler::_internal_sample_groups_size() const {
  return _impl_.sample_groups_.size();
}
inline int Sampler::sample_groups_size() const {
  return _internal_sample_groups_size();
}
inline void Sampler::clear_sample_groups() {
  _impl_.sample_groups_.Clear();
}
inline ::second::protos::Group* Sampler::mutable_sample_groups(int index) {
  // @@protoc_insertion_point(field_mutable:second.protos.Sampler.sample_groups)
  return _impl_.sample_groups_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::Group >*
Sampler::mutable_sample_groups() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.Sampler.sample_groups)
  return &_impl_.sample_groups_;
}
inline const ::second::protos::Group& Sampler::_internal_sample_groups(int index) const {
  return _impl_.sample_groups_.Get(index);
}
inline const ::second::protos::Group& Sampler::sample_groups(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.Sampler.sample_groups)
  return _internal_sample_groups(index);
}
inline ::second::protos::Group* Sampler::_internal_add_sample_groups() {
  return _impl_.sample_groups_.Add();
}
inline ::second::protos::Group* Sampler::add_sample_groups() {
  ::second::protos::Group* _add = _internal_add_sample_groups();
  // @@protoc_insertion_point(field_add:second.protos.Sampler.sample_groups)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::Group >&
Sampler::sample_groups() const {
  // @@protoc_insertion_point(field_list:second.protos.Sampler.sample_groups)
  return _impl_.sample_groups_;
}

// repeated .second.protos.DatabasePreprocessingStep database_prep_steps = 3;
inline int Sampler::_internal_database_prep_steps_size() const {
  return _impl_.database_prep_steps_.size();
}
inline int Sampler::database_prep_steps_size() const {
  return _internal_database_prep_steps_size();
}
inline ::second::protos::DatabasePreprocessingStep* Sampler::mutable_database_prep_steps(int index) {
  // @@protoc_insertion_point(field_mutable:second.protos.Sampler.database_prep_steps)
  return _impl_.database_prep_steps_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::DatabasePreprocessingStep >*
Sampler::mutable_database_prep_steps() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.Sampler.database_prep_steps)
  return &_impl_.database_prep_steps_;
}
inline const ::second::protos::DatabasePreprocessingStep& Sampler::_internal_database_prep_steps(int index) const {
  return _impl_.database_prep_steps_.Get(index);
}
inline const ::second::protos::DatabasePreprocessingStep& Sampler::database_prep_steps(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.Sampler.database_prep_steps)
  return _internal_database_prep_steps(index);
}
inline ::second::protos::DatabasePreprocessingStep* Sampler::_internal_add_database_prep_steps() {
  return _impl_.database_prep_steps_.Add();
}
inline ::second::protos::DatabasePreprocessingStep* Sampler::add_database_prep_steps() {
  ::second::protos::DatabasePreprocessingStep* _add = _internal_add_database_prep_steps();
  // @@protoc_insertion_point(field_add:second.protos.Sampler.database_prep_steps)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::DatabasePreprocessingStep >&
Sampler::database_prep_steps() const {
  // @@protoc_insertion_point(field_list:second.protos.Sampler.database_prep_steps)
  return _impl_.database_prep_steps_;
}

// repeated float global_random_rotation_range_per_object = 4;
inline int Sampler::_internal_global_random_rotation_range_per_object_size() const {
  return _impl_.global_random_rotation_range_per_object_.size();
}
inline int Sampler::global_random_rotation_range_per_object_size() const {
  return _internal_global_random_rotation_range_per_object_size();
}
inline void Sampler::clear_global_random_rotation_range_per_object() {
  _impl_.global_random_rotation_range_per_object_.Clear();
}
inline float Sampler::_internal_global_random_rotation_range_per_object(int index) const {
  return _impl_.global_random_rotation_range_per_object_.Get(index);
}
inline float Sampler::global_random_rotation_range_per_object(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.Sampler.global_random_rotation_range_per_object)
  return _internal_global_random_rotation_range_per_object(index);
}
inline void Sampler::set_global_random_rotation_range_per_object(int index, float value) {
  _impl_.global_random_rotation_range_per_object_.Set(index, value);
  // @@protoc_insertion_point(field_set:second.protos.Sampler.global_random_rotation_range_per_object)
}
inline void Sampler::_internal_add_global_random_rotation_range_per_object(float value) {
  _impl_.global_random_rotation_range_per_object_.Add(value);
}
inline void Sampler::add_global_random_rotation_range_per_object(float value) {
  _internal_add_global_random_rotation_range_per_object(value);
  // @@protoc_insertion_point(field_add:second.protos.Sampler.global_random_rotation_range_per_object)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
Sampler::_internal_global_random_rotation_range_per_object() const {
  return _impl_.global_random_rotation_range_per_object_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >&
Sampler::global_random_rotation_range_per_object() const {
  // @@protoc_insertion_point(field_list:second.protos.Sampler.global_random_rotation_range_per_object)
  return _internal_global_random_rotation_range_per_object();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
Sampler::_internal_mutable_global_random_rotation_range_per_object() {
  return &_impl_.global_random_rotation_range_per_object_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< float >*
Sampler::mutable_global_random_rotation_range_per_object() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.Sampler.global_random_rotation_range_per_object)
  return _internal_mutable_global_random_rotation_range_per_object();
}

// float rate = 5;
inline void Sampler::clear_rate() {
  _impl_.rate_ = 0;
}
inline float Sampler::_internal_rate() const {
  return _impl_.rate_;
}
inline float Sampler::rate() const {
  // @@protoc_insertion_point(field_get:second.protos.Sampler.rate)
  return _internal_rate();
}
inline void Sampler::_internal_set_rate(float value) {
  
  _impl_.rate_ = value;
}
inline void Sampler::set_rate(float value) {
  _internal_set_rate(value);
  // @@protoc_insertion_point(field_set:second.protos.Sampler.rate)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------

// -------------------------------------------------------------------


// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace second

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2fsampler_2eproto
