// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/target.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2ftarget_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2ftarget_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "second/protos/anchors.pb.h"
#include "second/protos/similarity.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_second_2fprotos_2ftarget_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_second_2fprotos_2ftarget_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_second_2fprotos_2ftarget_2eproto;
namespace second {
namespace protos {
class TargetAssigner;
struct TargetAssignerDefaultTypeInternal;
extern TargetAssignerDefaultTypeInternal _TargetAssigner_default_instance_;
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> ::second::protos::TargetAssigner* Arena::CreateMaybeMessage<::second::protos::TargetAssigner>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace second {
namespace protos {

// ===================================================================

class TargetAssigner final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.TargetAssigner) */ {
 public:
  inline TargetAssigner() : TargetAssigner(nullptr) {}
  ~TargetAssigner() override;
  explicit PROTOBUF_CONSTEXPR TargetAssigner(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  TargetAssigner(const TargetAssigner& from);
  TargetAssigner(TargetAssigner&& from) noexcept
    : TargetAssigner() {
    *this = ::std::move(from);
  }

  inline TargetAssigner& operator=(const TargetAssigner& from) {
    CopyFrom(from);
    return *this;
  }
  inline TargetAssigner& operator=(TargetAssigner&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const TargetAssigner& default_instance() {
    return *internal_default_instance();
  }
  static inline const TargetAssigner* internal_default_instance() {
    return reinterpret_cast<const TargetAssigner*>(
               &_TargetAssigner_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(TargetAssigner& a, TargetAssigner& b) {
    a.Swap(&b);
  }
  inline void Swap(TargetAssigner* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TargetAssigner* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  TargetAssigner* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<TargetAssigner>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const TargetAssigner& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const TargetAssigner& from) {
    TargetAssigner::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(TargetAssigner* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.TargetAssigner";
  }
  protected:
  explicit TargetAssigner(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kAnchorGeneratorsFieldNumber = 1,
    kClassNameFieldNumber = 5,
    kRegionSimilarityCalculatorFieldNumber = 6,
    kSamplePositiveFractionFieldNumber = 2,
    kSampleSizeFieldNumber = 3,
    kUseRotateIouFieldNumber = 4,
  };
  // repeated .second.protos.AnchorGeneratorCollection anchor_generators = 1;
  int anchor_generators_size() const;
  private:
  int _internal_anchor_generators_size() const;
  public:
  void clear_anchor_generators();
  ::second::protos::AnchorGeneratorCollection* mutable_anchor_generators(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::AnchorGeneratorCollection >*
      mutable_anchor_generators();
  private:
  const ::second::protos::AnchorGeneratorCollection& _internal_anchor_generators(int index) const;
  ::second::protos::AnchorGeneratorCollection* _internal_add_anchor_generators();
  public:
  const ::second::protos::AnchorGeneratorCollection& anchor_generators(int index) const;
  ::second::protos::AnchorGeneratorCollection* add_anchor_generators();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::AnchorGeneratorCollection >&
      anchor_generators() const;

  // string class_name = 5;
  void clear_class_name();
  const std::string& class_name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_class_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_class_name();
  PROTOBUF_NODISCARD std::string* release_class_name();
  void set_allocated_class_name(std::string* class_name);
  private:
  const std::string& _internal_class_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_class_name(const std::string& value);
  std::string* _internal_mutable_class_name();
  public:

  // .second.protos.RegionSimilarityCalculator region_similarity_calculator = 6;
  bool has_region_similarity_calculator() const;
  private:
  bool _internal_has_region_similarity_calculator() const;
  public:
  void clear_region_similarity_calculator();
  const ::second::protos::RegionSimilarityCalculator& region_similarity_calculator() const;
  PROTOBUF_NODISCARD ::second::protos::RegionSimilarityCalculator* release_region_similarity_calculator();
  ::second::protos::RegionSimilarityCalculator* mutable_region_similarity_calculator();
  void set_allocated_region_similarity_calculator(::second::protos::RegionSimilarityCalculator* region_similarity_calculator);
  private:
  const ::second::protos::RegionSimilarityCalculator& _internal_region_similarity_calculator() const;
  ::second::protos::RegionSimilarityCalculator* _internal_mutable_region_similarity_calculator();
  public:
  void unsafe_arena_set_allocated_region_similarity_calculator(
      ::second::protos::RegionSimilarityCalculator* region_similarity_calculator);
  ::second::protos::RegionSimilarityCalculator* unsafe_arena_release_region_similarity_calculator();

  // float sample_positive_fraction = 2;
  void clear_sample_positive_fraction();
  float sample_positive_fraction() const;
  void set_sample_positive_fraction(float value);
  private:
  float _internal_sample_positive_fraction() const;
  void _internal_set_sample_positive_fraction(float value);
  public:

  // uint32 sample_size = 3;
  void clear_sample_size();
  uint32_t sample_size() const;
  void set_sample_size(uint32_t value);
  private:
  uint32_t _internal_sample_size() const;
  void _internal_set_sample_size(uint32_t value);
  public:

  // bool use_rotate_iou = 4;
  void clear_use_rotate_iou();
  bool use_rotate_iou() const;
  void set_use_rotate_iou(bool value);
  private:
  bool _internal_use_rotate_iou() const;
  void _internal_set_use_rotate_iou(bool value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.TargetAssigner)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::AnchorGeneratorCollection > anchor_generators_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr class_name_;
    ::second::protos::RegionSimilarityCalculator* region_similarity_calculator_;
    float sample_positive_fraction_;
    uint32_t sample_size_;
    bool use_rotate_iou_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2ftarget_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// TargetAssigner

// repeated .second.protos.AnchorGeneratorCollection anchor_generators = 1;
inline int TargetAssigner::_internal_anchor_generators_size() const {
  return _impl_.anchor_generators_.size();
}
inline int TargetAssigner::anchor_generators_size() const {
  return _internal_anchor_generators_size();
}
inline ::second::protos::AnchorGeneratorCollection* TargetAssigner::mutable_anchor_generators(int index) {
  // @@protoc_insertion_point(field_mutable:second.protos.TargetAssigner.anchor_generators)
  return _impl_.anchor_generators_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::AnchorGeneratorCollection >*
TargetAssigner::mutable_anchor_generators() {
  // @@protoc_insertion_point(field_mutable_list:second.protos.TargetAssigner.anchor_generators)
  return &_impl_.anchor_generators_;
}
inline const ::second::protos::AnchorGeneratorCollection& TargetAssigner::_internal_anchor_generators(int index) const {
  return _impl_.anchor_generators_.Get(index);
}
inline const ::second::protos::AnchorGeneratorCollection& TargetAssigner::anchor_generators(int index) const {
  // @@protoc_insertion_point(field_get:second.protos.TargetAssigner.anchor_generators)
  return _internal_anchor_generators(index);
}
inline ::second::protos::AnchorGeneratorCollection* TargetAssigner::_internal_add_anchor_generators() {
  return _impl_.anchor_generators_.Add();
}
inline ::second::protos::AnchorGeneratorCollection* TargetAssigner::add_anchor_generators() {
  ::second::protos::AnchorGeneratorCollection* _add = _internal_add_anchor_generators();
  // @@protoc_insertion_point(field_add:second.protos.TargetAssigner.anchor_generators)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::second::protos::AnchorGeneratorCollection >&
TargetAssigner::anchor_generators() const {
  // @@protoc_insertion_point(field_list:second.protos.TargetAssigner.anchor_generators)
  return _impl_.anchor_generators_;
}

// float sample_positive_fraction = 2;
inline void TargetAssigner::clear_sample_positive_fraction() {
  _impl_.sample_positive_fraction_ = 0;
}
inline float TargetAssigner::_internal_sample_positive_fraction() const {
  return _impl_.sample_positive_fraction_;
}
inline float TargetAssigner::sample_positive_fraction() const {
  // @@protoc_insertion_point(field_get:second.protos.TargetAssigner.sample_positive_fraction)
  return _internal_sample_positive_fraction();
}
inline void TargetAssigner::_internal_set_sample_positive_fraction(float value) {
  
  _impl_.sample_positive_fraction_ = value;
}
inline void TargetAssigner::set_sample_positive_fraction(float value) {
  _internal_set_sample_positive_fraction(value);
  // @@protoc_insertion_point(field_set:second.protos.TargetAssigner.sample_positive_fraction)
}

// uint32 sample_size = 3;
inline void TargetAssigner::clear_sample_size() {
  _impl_.sample_size_ = 0u;
}
inline uint32_t TargetAssigner::_internal_sample_size() const {
  return _impl_.sample_size_;
}
inline uint32_t TargetAssigner::sample_size() const {
  // @@protoc_insertion_point(field_get:second.protos.TargetAssigner.sample_size)
  return _internal_sample_size();
}
inline void TargetAssigner::_internal_set_sample_size(uint32_t value) {
  
  _impl_.sample_size_ = value;
}
inline void TargetAssigner::set_sample_size(uint32_t value) {
  _internal_set_sample_size(value);
  // @@protoc_insertion_point(field_set:second.protos.TargetAssigner.sample_size)
}

// bool use_rotate_iou = 4;
inline void TargetAssigner::clear_use_rotate_iou() {
  _impl_.use_rotate_iou_ = false;
}
inline bool TargetAssigner::_internal_use_rotate_iou() const {
  return _impl_.use_rotate_iou_;
}
inline bool TargetAssigner::use_rotate_iou() const {
  // @@protoc_insertion_point(field_get:second.protos.TargetAssigner.use_rotate_iou)
  return _internal_use_rotate_iou();
}
inline void TargetAssigner::_internal_set_use_rotate_iou(bool value) {
  
  _impl_.use_rotate_iou_ = value;
}
inline void TargetAssigner::set_use_rotate_iou(bool value) {
  _internal_set_use_rotate_iou(value);
  // @@protoc_insertion_point(field_set:second.protos.TargetAssigner.use_rotate_iou)
}

// string class_name = 5;
inline void TargetAssigner::clear_class_name() {
  _impl_.class_name_.ClearToEmpty();
}
inline const std::string& TargetAssigner::class_name() const {
  // @@protoc_insertion_point(field_get:second.protos.TargetAssigner.class_name)
  return _internal_class_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void TargetAssigner::set_class_name(ArgT0&& arg0, ArgT... args) {
 
 _impl_.class_name_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:second.protos.TargetAssigner.class_name)
}
inline std::string* TargetAssigner::mutable_class_name() {
  std::string* _s = _internal_mutable_class_name();
  // @@protoc_insertion_point(field_mutable:second.protos.TargetAssigner.class_name)
  return _s;
}
inline const std::string& TargetAssigner::_internal_class_name() const {
  return _impl_.class_name_.Get();
}
inline void TargetAssigner::_internal_set_class_name(const std::string& value) {
  
  _impl_.class_name_.Set(value, GetArenaForAllocation());
}
inline std::string* TargetAssigner::_internal_mutable_class_name() {
  
  return _impl_.class_name_.Mutable(GetArenaForAllocation());
}
inline std::string* TargetAssigner::release_class_name() {
  // @@protoc_insertion_point(field_release:second.protos.TargetAssigner.class_name)
  return _impl_.class_name_.Release();
}
inline void TargetAssigner::set_allocated_class_name(std::string* class_name) {
  if (class_name != nullptr) {
    
  } else {
    
  }
  _impl_.class_name_.SetAllocated(class_name, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.class_name_.IsDefault()) {
    _impl_.class_name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:second.protos.TargetAssigner.class_name)
}

// .second.protos.RegionSimilarityCalculator region_similarity_calculator = 6;
inline bool TargetAssigner::_internal_has_region_similarity_calculator() const {
  return this != internal_default_instance() && _impl_.region_similarity_calculator_ != nullptr;
}
inline bool TargetAssigner::has_region_similarity_calculator() const {
  return _internal_has_region_similarity_calculator();
}
inline const ::second::protos::RegionSimilarityCalculator& TargetAssigner::_internal_region_similarity_calculator() const {
  const ::second::protos::RegionSimilarityCalculator* p = _impl_.region_similarity_calculator_;
  return p != nullptr ? *p : reinterpret_cast<const ::second::protos::RegionSimilarityCalculator&>(
      ::second::protos::_RegionSimilarityCalculator_default_instance_);
}
inline const ::second::protos::RegionSimilarityCalculator& TargetAssigner::region_similarity_calculator() const {
  // @@protoc_insertion_point(field_get:second.protos.TargetAssigner.region_similarity_calculator)
  return _internal_region_similarity_calculator();
}
inline void TargetAssigner::unsafe_arena_set_allocated_region_similarity_calculator(
    ::second::protos::RegionSimilarityCalculator* region_similarity_calculator) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.region_similarity_calculator_);
  }
  _impl_.region_similarity_calculator_ = region_similarity_calculator;
  if (region_similarity_calculator) {
    
  } else {
    
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.TargetAssigner.region_similarity_calculator)
}
inline ::second::protos::RegionSimilarityCalculator* TargetAssigner::release_region_similarity_calculator() {
  
  ::second::protos::RegionSimilarityCalculator* temp = _impl_.region_similarity_calculator_;
  _impl_.region_similarity_calculator_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::second::protos::RegionSimilarityCalculator* TargetAssigner::unsafe_arena_release_region_similarity_calculator() {
  // @@protoc_insertion_point(field_release:second.protos.TargetAssigner.region_similarity_calculator)
  
  ::second::protos::RegionSimilarityCalculator* temp = _impl_.region_similarity_calculator_;
  _impl_.region_similarity_calculator_ = nullptr;
  return temp;
}
inline ::second::protos::RegionSimilarityCalculator* TargetAssigner::_internal_mutable_region_similarity_calculator() {
  
  if (_impl_.region_similarity_calculator_ == nullptr) {
    auto* p = CreateMaybeMessage<::second::protos::RegionSimilarityCalculator>(GetArenaForAllocation());
    _impl_.region_similarity_calculator_ = p;
  }
  return _impl_.region_similarity_calculator_;
}
inline ::second::protos::RegionSimilarityCalculator* TargetAssigner::mutable_region_similarity_calculator() {
  ::second::protos::RegionSimilarityCalculator* _msg = _internal_mutable_region_similarity_calculator();
  // @@protoc_insertion_point(field_mutable:second.protos.TargetAssigner.region_similarity_calculator)
  return _msg;
}
inline void TargetAssigner::set_allocated_region_similarity_calculator(::second::protos::RegionSimilarityCalculator* region_similarity_calculator) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.region_similarity_calculator_);
  }
  if (region_similarity_calculator) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(region_similarity_calculator));
    if (message_arena != submessage_arena) {
      region_similarity_calculator = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, region_similarity_calculator, submessage_arena);
    }
    
  } else {
    
  }
  _impl_.region_similarity_calculator_ = region_similarity_calculator;
  // @@protoc_insertion_point(field_set_allocated:second.protos.TargetAssigner.region_similarity_calculator)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace second

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2ftarget_2eproto
