// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/second.proto

#include "second/protos/second.pb.h"

#include <algorithm>

#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/extension_set.h>
#include <google/protobuf/wire_format_lite.h>
#include <google/protobuf/descriptor.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/reflection_ops.h>
#include <google/protobuf/wire_format.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>

PROTOBUF_PRAGMA_INIT_SEG

namespace _pb = ::PROTOBUF_NAMESPACE_ID;
namespace _pbi = _pb::internal;

namespace second {
namespace protos {
PROTOBUF_CONSTEXPR VoxelNet_VoxelFeatureExtractor::VoxelNet_VoxelFeatureExtractor(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.num_filters_)*/{}
  , /*decltype(_impl_._num_filters_cached_byte_size_)*/{0}
  , /*decltype(_impl_.module_class_name_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.with_distance_)*/false
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct VoxelNet_VoxelFeatureExtractorDefaultTypeInternal {
  PROTOBUF_CONSTEXPR VoxelNet_VoxelFeatureExtractorDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~VoxelNet_VoxelFeatureExtractorDefaultTypeInternal() {}
  union {
    VoxelNet_VoxelFeatureExtractor _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 VoxelNet_VoxelFeatureExtractorDefaultTypeInternal _VoxelNet_VoxelFeatureExtractor_default_instance_;
PROTOBUF_CONSTEXPR VoxelNet_MiddleFeatureExtractor::VoxelNet_MiddleFeatureExtractor(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.num_filters_down1_)*/{}
  , /*decltype(_impl_._num_filters_down1_cached_byte_size_)*/{0}
  , /*decltype(_impl_.num_filters_down2_)*/{}
  , /*decltype(_impl_._num_filters_down2_cached_byte_size_)*/{0}
  , /*decltype(_impl_.module_class_name_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct VoxelNet_MiddleFeatureExtractorDefaultTypeInternal {
  PROTOBUF_CONSTEXPR VoxelNet_MiddleFeatureExtractorDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~VoxelNet_MiddleFeatureExtractorDefaultTypeInternal() {}
  union {
    VoxelNet_MiddleFeatureExtractor _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 VoxelNet_MiddleFeatureExtractorDefaultTypeInternal _VoxelNet_MiddleFeatureExtractor_default_instance_;
PROTOBUF_CONSTEXPR VoxelNet_RPN::VoxelNet_RPN(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.layer_nums_)*/{}
  , /*decltype(_impl_._layer_nums_cached_byte_size_)*/{0}
  , /*decltype(_impl_.layer_strides_)*/{}
  , /*decltype(_impl_._layer_strides_cached_byte_size_)*/{0}
  , /*decltype(_impl_.num_filters_)*/{}
  , /*decltype(_impl_._num_filters_cached_byte_size_)*/{0}
  , /*decltype(_impl_.upsample_strides_)*/{}
  , /*decltype(_impl_._upsample_strides_cached_byte_size_)*/{0}
  , /*decltype(_impl_.num_upsample_filters_)*/{}
  , /*decltype(_impl_._num_upsample_filters_cached_byte_size_)*/{0}
  , /*decltype(_impl_.module_class_name_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.use_groupnorm_)*/false
  , /*decltype(_impl_.num_groups_)*/0u
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct VoxelNet_RPNDefaultTypeInternal {
  PROTOBUF_CONSTEXPR VoxelNet_RPNDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~VoxelNet_RPNDefaultTypeInternal() {}
  union {
    VoxelNet_RPN _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 VoxelNet_RPNDefaultTypeInternal _VoxelNet_RPN_default_instance_;
PROTOBUF_CONSTEXPR VoxelNet::VoxelNet(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.post_center_limit_range_)*/{}
  , /*decltype(_impl_.voxel_feature_extractor_)*/nullptr
  , /*decltype(_impl_.middle_feature_extractor_)*/nullptr
  , /*decltype(_impl_.rpn_)*/nullptr
  , /*decltype(_impl_.loss_)*/nullptr
  , /*decltype(_impl_.box_coder_)*/nullptr
  , /*decltype(_impl_.target_assigner_)*/nullptr
  , /*decltype(_impl_.voxel_generator_)*/nullptr
  , /*decltype(_impl_.num_class_)*/0u
  , /*decltype(_impl_.use_sigmoid_score_)*/false
  , /*decltype(_impl_.encode_rad_error_by_sin_)*/false
  , /*decltype(_impl_.encode_background_as_zeros_)*/false
  , /*decltype(_impl_.use_aux_classifier_)*/false
  , /*decltype(_impl_.nms_pre_max_size_)*/0u
  , /*decltype(_impl_.nms_post_max_size_)*/0u
  , /*decltype(_impl_.nms_score_threshold_)*/0
  , /*decltype(_impl_.nms_iou_threshold_)*/0
  , /*decltype(_impl_.use_rotate_nms_)*/false
  , /*decltype(_impl_.use_multi_class_nms_)*/false
  , /*decltype(_impl_.use_direction_classifier_)*/false
  , /*decltype(_impl_.use_bev_)*/false
  , /*decltype(_impl_.direction_loss_weight_)*/0
  , /*decltype(_impl_.pos_class_weight_)*/0
  , /*decltype(_impl_.neg_class_weight_)*/0
  , /*decltype(_impl_.loss_norm_type_)*/0
  , /*decltype(_impl_.without_reflectivity_)*/false
  , /*decltype(_impl_.encode_angle_to_vector_)*/false
  , /*decltype(_impl_.lidar_input_)*/false
  , /*decltype(_impl_.num_point_features_)*/0u
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct VoxelNetDefaultTypeInternal {
  PROTOBUF_CONSTEXPR VoxelNetDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~VoxelNetDefaultTypeInternal() {}
  union {
    VoxelNet _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 VoxelNetDefaultTypeInternal _VoxelNet_default_instance_;
}  // namespace protos
}  // namespace second
static ::_pb::Metadata file_level_metadata_second_2fprotos_2fsecond_2eproto[4];
static const ::_pb::EnumDescriptor* file_level_enum_descriptors_second_2fprotos_2fsecond_2eproto[1];
static constexpr ::_pb::ServiceDescriptor const** file_level_service_descriptors_second_2fprotos_2fsecond_2eproto = nullptr;

const uint32_t TableStruct_second_2fprotos_2fsecond_2eproto::offsets[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_VoxelFeatureExtractor, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_VoxelFeatureExtractor, _impl_.module_class_name_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_VoxelFeatureExtractor, _impl_.num_filters_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_VoxelFeatureExtractor, _impl_.with_distance_),
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_MiddleFeatureExtractor, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_MiddleFeatureExtractor, _impl_.module_class_name_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_MiddleFeatureExtractor, _impl_.num_filters_down1_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_MiddleFeatureExtractor, _impl_.num_filters_down2_),
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_RPN, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_RPN, _impl_.module_class_name_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_RPN, _impl_.layer_nums_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_RPN, _impl_.layer_strides_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_RPN, _impl_.num_filters_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_RPN, _impl_.upsample_strides_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_RPN, _impl_.num_upsample_filters_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_RPN, _impl_.use_groupnorm_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet_RPN, _impl_.num_groups_),
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.num_class_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.voxel_feature_extractor_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.middle_feature_extractor_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.rpn_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.use_sigmoid_score_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.loss_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.encode_rad_error_by_sin_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.encode_background_as_zeros_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.use_aux_classifier_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.use_rotate_nms_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.use_multi_class_nms_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.nms_pre_max_size_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.nms_post_max_size_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.nms_score_threshold_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.nms_iou_threshold_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.post_center_limit_range_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.use_direction_classifier_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.direction_loss_weight_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.pos_class_weight_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.neg_class_weight_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.loss_norm_type_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.use_bev_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.without_reflectivity_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.encode_angle_to_vector_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.box_coder_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.target_assigner_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.lidar_input_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.num_point_features_),
  PROTOBUF_FIELD_OFFSET(::second::protos::VoxelNet, _impl_.voxel_generator_),
};
static const ::_pbi::MigrationSchema schemas[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  { 0, -1, -1, sizeof(::second::protos::VoxelNet_VoxelFeatureExtractor)},
  { 9, -1, -1, sizeof(::second::protos::VoxelNet_MiddleFeatureExtractor)},
  { 18, -1, -1, sizeof(::second::protos::VoxelNet_RPN)},
  { 32, -1, -1, sizeof(::second::protos::VoxelNet)},
};

static const ::_pb::Message* const file_default_instances[] = {
  &::second::protos::_VoxelNet_VoxelFeatureExtractor_default_instance_._instance,
  &::second::protos::_VoxelNet_MiddleFeatureExtractor_default_instance_._instance,
  &::second::protos::_VoxelNet_RPN_default_instance_._instance,
  &::second::protos::_VoxelNet_default_instance_._instance,
};

const char descriptor_table_protodef_second_2fprotos_2fsecond_2eproto[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) =
  "\n\032second/protos/second.proto\022\rsecond.pro"
  "tos\032\032second/protos/losses.proto\032\035second/"
  "protos/box_coder.proto\032\032second/protos/ta"
  "rget.proto\032#second/protos/voxel_generato"
  "r.proto\"\377\013\n\010VoxelNet\022\021\n\tnum_class\030\001 \001(\r\022"
  "N\n\027voxel_feature_extractor\030\002 \001(\0132-.secon"
  "d.protos.VoxelNet.VoxelFeatureExtractor\022"
  "P\n\030middle_feature_extractor\030\003 \001(\0132..seco"
  "nd.protos.VoxelNet.MiddleFeatureExtracto"
  "r\022(\n\003rpn\030\004 \001(\0132\033.second.protos.VoxelNet."
  "RPN\022\031\n\021use_sigmoid_score\030\005 \001(\010\022!\n\004loss\030\006"
  " \001(\0132\023.second.protos.Loss\022\037\n\027encode_rad_"
  "error_by_sin\030\007 \001(\010\022\"\n\032encode_background_"
  "as_zeros\030\010 \001(\010\022\032\n\022use_aux_classifier\030\t \001"
  "(\010\022\026\n\016use_rotate_nms\030\n \001(\010\022\033\n\023use_multi_"
  "class_nms\030\013 \001(\010\022\030\n\020nms_pre_max_size\030\014 \001("
  "\r\022\031\n\021nms_post_max_size\030\r \001(\r\022\033\n\023nms_scor"
  "e_threshold\030\016 \001(\002\022\031\n\021nms_iou_threshold\030\017"
  " \001(\002\022\037\n\027post_center_limit_range\030\020 \003(\002\022 \n"
  "\030use_direction_classifier\030\021 \001(\010\022\035\n\025direc"
  "tion_loss_weight\030\022 \001(\002\022\030\n\020pos_class_weig"
  "ht\030\023 \001(\002\022\030\n\020neg_class_weight\030\024 \001(\002\022<\n\016lo"
  "ss_norm_type\030\025 \001(\0162$.second.protos.Voxel"
  "Net.LossNormType\022\017\n\007use_bev\030\026 \001(\010\022\034\n\024wit"
  "hout_reflectivity\030\027 \001(\010\022\036\n\026encode_angle_"
  "to_vector\030\030 \001(\010\022*\n\tbox_coder\030\031 \001(\0132\027.sec"
  "ond.protos.BoxCoder\0226\n\017target_assigner\030\032"
  " \001(\0132\035.second.protos.TargetAssigner\022\023\n\013l"
  "idar_input\030\033 \001(\010\022\032\n\022num_point_features\030\034"
  " \001(\r\0226\n\017voxel_generator\030\035 \001(\0132\035.second.p"
  "rotos.VoxelGenerator\032^\n\025VoxelFeatureExtr"
  "actor\022\031\n\021module_class_name\030\001 \001(\t\022\023\n\013num_"
  "filters\030\002 \003(\r\022\025\n\rwith_distance\030\003 \001(\010\032i\n\026"
  "MiddleFeatureExtractor\022\031\n\021module_class_n"
  "ame\030\001 \001(\t\022\031\n\021num_filters_down1\030\002 \003(\r\022\031\n\021"
  "num_filters_down2\030\003 \003(\r\032\303\001\n\003RPN\022\031\n\021modul"
  "e_class_name\030\001 \001(\t\022\022\n\nlayer_nums\030\002 \003(\r\022\025"
  "\n\rlayer_strides\030\003 \003(\r\022\023\n\013num_filters\030\004 \003"
  "(\r\022\030\n\020upsample_strides\030\005 \003(\r\022\034\n\024num_upsa"
  "mple_filters\030\006 \003(\r\022\025\n\ruse_groupnorm\030\007 \001("
  "\010\022\022\n\nnum_groups\030\010 \001(\r\"R\n\014LossNormType\022\025\n"
  "\021NormByNumExamples\020\000\022\026\n\022NormByNumPositiv"
  "es\020\001\022\023\n\017NormByNumPosNeg\020\002b\006proto3"
  ;
static const ::_pbi::DescriptorTable* const descriptor_table_second_2fprotos_2fsecond_2eproto_deps[4] = {
  &::descriptor_table_second_2fprotos_2fbox_5fcoder_2eproto,
  &::descriptor_table_second_2fprotos_2flosses_2eproto,
  &::descriptor_table_second_2fprotos_2ftarget_2eproto,
  &::descriptor_table_second_2fprotos_2fvoxel_5fgenerator_2eproto,
};
static ::_pbi::once_flag descriptor_table_second_2fprotos_2fsecond_2eproto_once;
const ::_pbi::DescriptorTable descriptor_table_second_2fprotos_2fsecond_2eproto = {
    false, false, 1713, descriptor_table_protodef_second_2fprotos_2fsecond_2eproto,
    "second/protos/second.proto",
    &descriptor_table_second_2fprotos_2fsecond_2eproto_once, descriptor_table_second_2fprotos_2fsecond_2eproto_deps, 4, 4,
    schemas, file_default_instances, TableStruct_second_2fprotos_2fsecond_2eproto::offsets,
    file_level_metadata_second_2fprotos_2fsecond_2eproto, file_level_enum_descriptors_second_2fprotos_2fsecond_2eproto,
    file_level_service_descriptors_second_2fprotos_2fsecond_2eproto,
};
PROTOBUF_ATTRIBUTE_WEAK const ::_pbi::DescriptorTable* descriptor_table_second_2fprotos_2fsecond_2eproto_getter() {
  return &descriptor_table_second_2fprotos_2fsecond_2eproto;
}

// Force running AddDescriptors() at dynamic initialization time.
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::_pbi::AddDescriptorsRunner dynamic_init_dummy_second_2fprotos_2fsecond_2eproto(&descriptor_table_second_2fprotos_2fsecond_2eproto);
namespace second {
namespace protos {
const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor* VoxelNet_LossNormType_descriptor() {
  ::PROTOBUF_NAMESPACE_ID::internal::AssignDescriptors(&descriptor_table_second_2fprotos_2fsecond_2eproto);
  return file_level_enum_descriptors_second_2fprotos_2fsecond_2eproto[0];
}
bool VoxelNet_LossNormType_IsValid(int value) {
  switch (value) {
    case 0:
    case 1:
    case 2:
      return true;
    default:
      return false;
  }
}

#if (__cplusplus < 201703) && (!defined(_MSC_VER) || (_MSC_VER >= 1900 && _MSC_VER < 1912))
constexpr VoxelNet_LossNormType VoxelNet::NormByNumExamples;
constexpr VoxelNet_LossNormType VoxelNet::NormByNumPositives;
constexpr VoxelNet_LossNormType VoxelNet::NormByNumPosNeg;
constexpr VoxelNet_LossNormType VoxelNet::LossNormType_MIN;
constexpr VoxelNet_LossNormType VoxelNet::LossNormType_MAX;
constexpr int VoxelNet::LossNormType_ARRAYSIZE;
#endif  // (__cplusplus < 201703) && (!defined(_MSC_VER) || (_MSC_VER >= 1900 && _MSC_VER < 1912))

// ===================================================================

class VoxelNet_VoxelFeatureExtractor::_Internal {
 public:
};

VoxelNet_VoxelFeatureExtractor::VoxelNet_VoxelFeatureExtractor(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.VoxelNet.VoxelFeatureExtractor)
}
VoxelNet_VoxelFeatureExtractor::VoxelNet_VoxelFeatureExtractor(const VoxelNet_VoxelFeatureExtractor& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  VoxelNet_VoxelFeatureExtractor* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.num_filters_){from._impl_.num_filters_}
    , /*decltype(_impl_._num_filters_cached_byte_size_)*/{0}
    , decltype(_impl_.module_class_name_){}
    , decltype(_impl_.with_distance_){}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _impl_.module_class_name_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.module_class_name_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (!from._internal_module_class_name().empty()) {
    _this->_impl_.module_class_name_.Set(from._internal_module_class_name(), 
      _this->GetArenaForAllocation());
  }
  _this->_impl_.with_distance_ = from._impl_.with_distance_;
  // @@protoc_insertion_point(copy_constructor:second.protos.VoxelNet.VoxelFeatureExtractor)
}

inline void VoxelNet_VoxelFeatureExtractor::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.num_filters_){arena}
    , /*decltype(_impl_._num_filters_cached_byte_size_)*/{0}
    , decltype(_impl_.module_class_name_){}
    , decltype(_impl_.with_distance_){false}
    , /*decltype(_impl_._cached_size_)*/{}
  };
  _impl_.module_class_name_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.module_class_name_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
}

VoxelNet_VoxelFeatureExtractor::~VoxelNet_VoxelFeatureExtractor() {
  // @@protoc_insertion_point(destructor:second.protos.VoxelNet.VoxelFeatureExtractor)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void VoxelNet_VoxelFeatureExtractor::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.num_filters_.~RepeatedField();
  _impl_.module_class_name_.Destroy();
}

void VoxelNet_VoxelFeatureExtractor::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void VoxelNet_VoxelFeatureExtractor::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.VoxelNet.VoxelFeatureExtractor)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.num_filters_.Clear();
  _impl_.module_class_name_.ClearToEmpty();
  _impl_.with_distance_ = false;
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* VoxelNet_VoxelFeatureExtractor::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // string module_class_name = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          auto str = _internal_mutable_module_class_name();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          CHK_(::_pbi::VerifyUTF8(str, "second.protos.VoxelNet.VoxelFeatureExtractor.module_class_name"));
        } else
          goto handle_unusual;
        continue;
      // repeated uint32 num_filters = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 18)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedUInt32Parser(_internal_mutable_num_filters(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 16) {
          _internal_add_num_filters(::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr));
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool with_distance = 3;
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 24)) {
          _impl_.with_distance_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* VoxelNet_VoxelFeatureExtractor::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.VoxelNet.VoxelFeatureExtractor)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // string module_class_name = 1;
  if (!this->_internal_module_class_name().empty()) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(
      this->_internal_module_class_name().data(), static_cast<int>(this->_internal_module_class_name().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::SERIALIZE,
      "second.protos.VoxelNet.VoxelFeatureExtractor.module_class_name");
    target = stream->WriteStringMaybeAliased(
        1, this->_internal_module_class_name(), target);
  }

  // repeated uint32 num_filters = 2;
  {
    int byte_size = _impl_._num_filters_cached_byte_size_.load(std::memory_order_relaxed);
    if (byte_size > 0) {
      target = stream->WriteUInt32Packed(
          2, _internal_num_filters(), byte_size, target);
    }
  }

  // bool with_distance = 3;
  if (this->_internal_with_distance() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(3, this->_internal_with_distance(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.VoxelNet.VoxelFeatureExtractor)
  return target;
}

size_t VoxelNet_VoxelFeatureExtractor::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.VoxelNet.VoxelFeatureExtractor)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // repeated uint32 num_filters = 2;
  {
    size_t data_size = ::_pbi::WireFormatLite::
      UInt32Size(this->_impl_.num_filters_);
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    int cached_size = ::_pbi::ToCachedSize(data_size);
    _impl_._num_filters_cached_byte_size_.store(cached_size,
                                    std::memory_order_relaxed);
    total_size += data_size;
  }

  // string module_class_name = 1;
  if (!this->_internal_module_class_name().empty()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
        this->_internal_module_class_name());
  }

  // bool with_distance = 3;
  if (this->_internal_with_distance() != 0) {
    total_size += 1 + 1;
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData VoxelNet_VoxelFeatureExtractor::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    VoxelNet_VoxelFeatureExtractor::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*VoxelNet_VoxelFeatureExtractor::GetClassData() const { return &_class_data_; }


void VoxelNet_VoxelFeatureExtractor::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<VoxelNet_VoxelFeatureExtractor*>(&to_msg);
  auto& from = static_cast<const VoxelNet_VoxelFeatureExtractor&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.VoxelNet.VoxelFeatureExtractor)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.num_filters_.MergeFrom(from._impl_.num_filters_);
  if (!from._internal_module_class_name().empty()) {
    _this->_internal_set_module_class_name(from._internal_module_class_name());
  }
  if (from._internal_with_distance() != 0) {
    _this->_internal_set_with_distance(from._internal_with_distance());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void VoxelNet_VoxelFeatureExtractor::CopyFrom(const VoxelNet_VoxelFeatureExtractor& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.VoxelNet.VoxelFeatureExtractor)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool VoxelNet_VoxelFeatureExtractor::IsInitialized() const {
  return true;
}

void VoxelNet_VoxelFeatureExtractor::InternalSwap(VoxelNet_VoxelFeatureExtractor* other) {
  using std::swap;
  auto* lhs_arena = GetArenaForAllocation();
  auto* rhs_arena = other->GetArenaForAllocation();
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  _impl_.num_filters_.InternalSwap(&other->_impl_.num_filters_);
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.module_class_name_, lhs_arena,
      &other->_impl_.module_class_name_, rhs_arena
  );
  swap(_impl_.with_distance_, other->_impl_.with_distance_);
}

::PROTOBUF_NAMESPACE_ID::Metadata VoxelNet_VoxelFeatureExtractor::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fsecond_2eproto_getter, &descriptor_table_second_2fprotos_2fsecond_2eproto_once,
      file_level_metadata_second_2fprotos_2fsecond_2eproto[0]);
}

// ===================================================================

class VoxelNet_MiddleFeatureExtractor::_Internal {
 public:
};

VoxelNet_MiddleFeatureExtractor::VoxelNet_MiddleFeatureExtractor(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.VoxelNet.MiddleFeatureExtractor)
}
VoxelNet_MiddleFeatureExtractor::VoxelNet_MiddleFeatureExtractor(const VoxelNet_MiddleFeatureExtractor& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  VoxelNet_MiddleFeatureExtractor* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.num_filters_down1_){from._impl_.num_filters_down1_}
    , /*decltype(_impl_._num_filters_down1_cached_byte_size_)*/{0}
    , decltype(_impl_.num_filters_down2_){from._impl_.num_filters_down2_}
    , /*decltype(_impl_._num_filters_down2_cached_byte_size_)*/{0}
    , decltype(_impl_.module_class_name_){}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _impl_.module_class_name_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.module_class_name_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (!from._internal_module_class_name().empty()) {
    _this->_impl_.module_class_name_.Set(from._internal_module_class_name(), 
      _this->GetArenaForAllocation());
  }
  // @@protoc_insertion_point(copy_constructor:second.protos.VoxelNet.MiddleFeatureExtractor)
}

inline void VoxelNet_MiddleFeatureExtractor::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.num_filters_down1_){arena}
    , /*decltype(_impl_._num_filters_down1_cached_byte_size_)*/{0}
    , decltype(_impl_.num_filters_down2_){arena}
    , /*decltype(_impl_._num_filters_down2_cached_byte_size_)*/{0}
    , decltype(_impl_.module_class_name_){}
    , /*decltype(_impl_._cached_size_)*/{}
  };
  _impl_.module_class_name_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.module_class_name_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
}

VoxelNet_MiddleFeatureExtractor::~VoxelNet_MiddleFeatureExtractor() {
  // @@protoc_insertion_point(destructor:second.protos.VoxelNet.MiddleFeatureExtractor)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void VoxelNet_MiddleFeatureExtractor::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.num_filters_down1_.~RepeatedField();
  _impl_.num_filters_down2_.~RepeatedField();
  _impl_.module_class_name_.Destroy();
}

void VoxelNet_MiddleFeatureExtractor::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void VoxelNet_MiddleFeatureExtractor::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.VoxelNet.MiddleFeatureExtractor)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.num_filters_down1_.Clear();
  _impl_.num_filters_down2_.Clear();
  _impl_.module_class_name_.ClearToEmpty();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* VoxelNet_MiddleFeatureExtractor::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // string module_class_name = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          auto str = _internal_mutable_module_class_name();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          CHK_(::_pbi::VerifyUTF8(str, "second.protos.VoxelNet.MiddleFeatureExtractor.module_class_name"));
        } else
          goto handle_unusual;
        continue;
      // repeated uint32 num_filters_down1 = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 18)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedUInt32Parser(_internal_mutable_num_filters_down1(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 16) {
          _internal_add_num_filters_down1(::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr));
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // repeated uint32 num_filters_down2 = 3;
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 26)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedUInt32Parser(_internal_mutable_num_filters_down2(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 24) {
          _internal_add_num_filters_down2(::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr));
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* VoxelNet_MiddleFeatureExtractor::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.VoxelNet.MiddleFeatureExtractor)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // string module_class_name = 1;
  if (!this->_internal_module_class_name().empty()) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(
      this->_internal_module_class_name().data(), static_cast<int>(this->_internal_module_class_name().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::SERIALIZE,
      "second.protos.VoxelNet.MiddleFeatureExtractor.module_class_name");
    target = stream->WriteStringMaybeAliased(
        1, this->_internal_module_class_name(), target);
  }

  // repeated uint32 num_filters_down1 = 2;
  {
    int byte_size = _impl_._num_filters_down1_cached_byte_size_.load(std::memory_order_relaxed);
    if (byte_size > 0) {
      target = stream->WriteUInt32Packed(
          2, _internal_num_filters_down1(), byte_size, target);
    }
  }

  // repeated uint32 num_filters_down2 = 3;
  {
    int byte_size = _impl_._num_filters_down2_cached_byte_size_.load(std::memory_order_relaxed);
    if (byte_size > 0) {
      target = stream->WriteUInt32Packed(
          3, _internal_num_filters_down2(), byte_size, target);
    }
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.VoxelNet.MiddleFeatureExtractor)
  return target;
}

size_t VoxelNet_MiddleFeatureExtractor::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.VoxelNet.MiddleFeatureExtractor)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // repeated uint32 num_filters_down1 = 2;
  {
    size_t data_size = ::_pbi::WireFormatLite::
      UInt32Size(this->_impl_.num_filters_down1_);
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    int cached_size = ::_pbi::ToCachedSize(data_size);
    _impl_._num_filters_down1_cached_byte_size_.store(cached_size,
                                    std::memory_order_relaxed);
    total_size += data_size;
  }

  // repeated uint32 num_filters_down2 = 3;
  {
    size_t data_size = ::_pbi::WireFormatLite::
      UInt32Size(this->_impl_.num_filters_down2_);
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    int cached_size = ::_pbi::ToCachedSize(data_size);
    _impl_._num_filters_down2_cached_byte_size_.store(cached_size,
                                    std::memory_order_relaxed);
    total_size += data_size;
  }

  // string module_class_name = 1;
  if (!this->_internal_module_class_name().empty()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
        this->_internal_module_class_name());
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData VoxelNet_MiddleFeatureExtractor::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    VoxelNet_MiddleFeatureExtractor::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*VoxelNet_MiddleFeatureExtractor::GetClassData() const { return &_class_data_; }


void VoxelNet_MiddleFeatureExtractor::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<VoxelNet_MiddleFeatureExtractor*>(&to_msg);
  auto& from = static_cast<const VoxelNet_MiddleFeatureExtractor&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.VoxelNet.MiddleFeatureExtractor)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.num_filters_down1_.MergeFrom(from._impl_.num_filters_down1_);
  _this->_impl_.num_filters_down2_.MergeFrom(from._impl_.num_filters_down2_);
  if (!from._internal_module_class_name().empty()) {
    _this->_internal_set_module_class_name(from._internal_module_class_name());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void VoxelNet_MiddleFeatureExtractor::CopyFrom(const VoxelNet_MiddleFeatureExtractor& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.VoxelNet.MiddleFeatureExtractor)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool VoxelNet_MiddleFeatureExtractor::IsInitialized() const {
  return true;
}

void VoxelNet_MiddleFeatureExtractor::InternalSwap(VoxelNet_MiddleFeatureExtractor* other) {
  using std::swap;
  auto* lhs_arena = GetArenaForAllocation();
  auto* rhs_arena = other->GetArenaForAllocation();
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  _impl_.num_filters_down1_.InternalSwap(&other->_impl_.num_filters_down1_);
  _impl_.num_filters_down2_.InternalSwap(&other->_impl_.num_filters_down2_);
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.module_class_name_, lhs_arena,
      &other->_impl_.module_class_name_, rhs_arena
  );
}

::PROTOBUF_NAMESPACE_ID::Metadata VoxelNet_MiddleFeatureExtractor::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fsecond_2eproto_getter, &descriptor_table_second_2fprotos_2fsecond_2eproto_once,
      file_level_metadata_second_2fprotos_2fsecond_2eproto[1]);
}

// ===================================================================

class VoxelNet_RPN::_Internal {
 public:
};

VoxelNet_RPN::VoxelNet_RPN(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.VoxelNet.RPN)
}
VoxelNet_RPN::VoxelNet_RPN(const VoxelNet_RPN& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  VoxelNet_RPN* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.layer_nums_){from._impl_.layer_nums_}
    , /*decltype(_impl_._layer_nums_cached_byte_size_)*/{0}
    , decltype(_impl_.layer_strides_){from._impl_.layer_strides_}
    , /*decltype(_impl_._layer_strides_cached_byte_size_)*/{0}
    , decltype(_impl_.num_filters_){from._impl_.num_filters_}
    , /*decltype(_impl_._num_filters_cached_byte_size_)*/{0}
    , decltype(_impl_.upsample_strides_){from._impl_.upsample_strides_}
    , /*decltype(_impl_._upsample_strides_cached_byte_size_)*/{0}
    , decltype(_impl_.num_upsample_filters_){from._impl_.num_upsample_filters_}
    , /*decltype(_impl_._num_upsample_filters_cached_byte_size_)*/{0}
    , decltype(_impl_.module_class_name_){}
    , decltype(_impl_.use_groupnorm_){}
    , decltype(_impl_.num_groups_){}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _impl_.module_class_name_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.module_class_name_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (!from._internal_module_class_name().empty()) {
    _this->_impl_.module_class_name_.Set(from._internal_module_class_name(), 
      _this->GetArenaForAllocation());
  }
  ::memcpy(&_impl_.use_groupnorm_, &from._impl_.use_groupnorm_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.num_groups_) -
    reinterpret_cast<char*>(&_impl_.use_groupnorm_)) + sizeof(_impl_.num_groups_));
  // @@protoc_insertion_point(copy_constructor:second.protos.VoxelNet.RPN)
}

inline void VoxelNet_RPN::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.layer_nums_){arena}
    , /*decltype(_impl_._layer_nums_cached_byte_size_)*/{0}
    , decltype(_impl_.layer_strides_){arena}
    , /*decltype(_impl_._layer_strides_cached_byte_size_)*/{0}
    , decltype(_impl_.num_filters_){arena}
    , /*decltype(_impl_._num_filters_cached_byte_size_)*/{0}
    , decltype(_impl_.upsample_strides_){arena}
    , /*decltype(_impl_._upsample_strides_cached_byte_size_)*/{0}
    , decltype(_impl_.num_upsample_filters_){arena}
    , /*decltype(_impl_._num_upsample_filters_cached_byte_size_)*/{0}
    , decltype(_impl_.module_class_name_){}
    , decltype(_impl_.use_groupnorm_){false}
    , decltype(_impl_.num_groups_){0u}
    , /*decltype(_impl_._cached_size_)*/{}
  };
  _impl_.module_class_name_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.module_class_name_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
}

VoxelNet_RPN::~VoxelNet_RPN() {
  // @@protoc_insertion_point(destructor:second.protos.VoxelNet.RPN)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void VoxelNet_RPN::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.layer_nums_.~RepeatedField();
  _impl_.layer_strides_.~RepeatedField();
  _impl_.num_filters_.~RepeatedField();
  _impl_.upsample_strides_.~RepeatedField();
  _impl_.num_upsample_filters_.~RepeatedField();
  _impl_.module_class_name_.Destroy();
}

void VoxelNet_RPN::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void VoxelNet_RPN::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.VoxelNet.RPN)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.layer_nums_.Clear();
  _impl_.layer_strides_.Clear();
  _impl_.num_filters_.Clear();
  _impl_.upsample_strides_.Clear();
  _impl_.num_upsample_filters_.Clear();
  _impl_.module_class_name_.ClearToEmpty();
  ::memset(&_impl_.use_groupnorm_, 0, static_cast<size_t>(
      reinterpret_cast<char*>(&_impl_.num_groups_) -
      reinterpret_cast<char*>(&_impl_.use_groupnorm_)) + sizeof(_impl_.num_groups_));
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* VoxelNet_RPN::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // string module_class_name = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          auto str = _internal_mutable_module_class_name();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          CHK_(::_pbi::VerifyUTF8(str, "second.protos.VoxelNet.RPN.module_class_name"));
        } else
          goto handle_unusual;
        continue;
      // repeated uint32 layer_nums = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 18)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedUInt32Parser(_internal_mutable_layer_nums(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 16) {
          _internal_add_layer_nums(::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr));
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // repeated uint32 layer_strides = 3;
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 26)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedUInt32Parser(_internal_mutable_layer_strides(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 24) {
          _internal_add_layer_strides(::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr));
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // repeated uint32 num_filters = 4;
      case 4:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 34)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedUInt32Parser(_internal_mutable_num_filters(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 32) {
          _internal_add_num_filters(::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr));
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // repeated uint32 upsample_strides = 5;
      case 5:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 42)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedUInt32Parser(_internal_mutable_upsample_strides(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 40) {
          _internal_add_upsample_strides(::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr));
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // repeated uint32 num_upsample_filters = 6;
      case 6:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 50)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedUInt32Parser(_internal_mutable_num_upsample_filters(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 48) {
          _internal_add_num_upsample_filters(::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr));
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool use_groupnorm = 7;
      case 7:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 56)) {
          _impl_.use_groupnorm_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // uint32 num_groups = 8;
      case 8:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 64)) {
          _impl_.num_groups_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* VoxelNet_RPN::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.VoxelNet.RPN)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // string module_class_name = 1;
  if (!this->_internal_module_class_name().empty()) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(
      this->_internal_module_class_name().data(), static_cast<int>(this->_internal_module_class_name().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::SERIALIZE,
      "second.protos.VoxelNet.RPN.module_class_name");
    target = stream->WriteStringMaybeAliased(
        1, this->_internal_module_class_name(), target);
  }

  // repeated uint32 layer_nums = 2;
  {
    int byte_size = _impl_._layer_nums_cached_byte_size_.load(std::memory_order_relaxed);
    if (byte_size > 0) {
      target = stream->WriteUInt32Packed(
          2, _internal_layer_nums(), byte_size, target);
    }
  }

  // repeated uint32 layer_strides = 3;
  {
    int byte_size = _impl_._layer_strides_cached_byte_size_.load(std::memory_order_relaxed);
    if (byte_size > 0) {
      target = stream->WriteUInt32Packed(
          3, _internal_layer_strides(), byte_size, target);
    }
  }

  // repeated uint32 num_filters = 4;
  {
    int byte_size = _impl_._num_filters_cached_byte_size_.load(std::memory_order_relaxed);
    if (byte_size > 0) {
      target = stream->WriteUInt32Packed(
          4, _internal_num_filters(), byte_size, target);
    }
  }

  // repeated uint32 upsample_strides = 5;
  {
    int byte_size = _impl_._upsample_strides_cached_byte_size_.load(std::memory_order_relaxed);
    if (byte_size > 0) {
      target = stream->WriteUInt32Packed(
          5, _internal_upsample_strides(), byte_size, target);
    }
  }

  // repeated uint32 num_upsample_filters = 6;
  {
    int byte_size = _impl_._num_upsample_filters_cached_byte_size_.load(std::memory_order_relaxed);
    if (byte_size > 0) {
      target = stream->WriteUInt32Packed(
          6, _internal_num_upsample_filters(), byte_size, target);
    }
  }

  // bool use_groupnorm = 7;
  if (this->_internal_use_groupnorm() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(7, this->_internal_use_groupnorm(), target);
  }

  // uint32 num_groups = 8;
  if (this->_internal_num_groups() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(8, this->_internal_num_groups(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.VoxelNet.RPN)
  return target;
}

size_t VoxelNet_RPN::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.VoxelNet.RPN)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // repeated uint32 layer_nums = 2;
  {
    size_t data_size = ::_pbi::WireFormatLite::
      UInt32Size(this->_impl_.layer_nums_);
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    int cached_size = ::_pbi::ToCachedSize(data_size);
    _impl_._layer_nums_cached_byte_size_.store(cached_size,
                                    std::memory_order_relaxed);
    total_size += data_size;
  }

  // repeated uint32 layer_strides = 3;
  {
    size_t data_size = ::_pbi::WireFormatLite::
      UInt32Size(this->_impl_.layer_strides_);
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    int cached_size = ::_pbi::ToCachedSize(data_size);
    _impl_._layer_strides_cached_byte_size_.store(cached_size,
                                    std::memory_order_relaxed);
    total_size += data_size;
  }

  // repeated uint32 num_filters = 4;
  {
    size_t data_size = ::_pbi::WireFormatLite::
      UInt32Size(this->_impl_.num_filters_);
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    int cached_size = ::_pbi::ToCachedSize(data_size);
    _impl_._num_filters_cached_byte_size_.store(cached_size,
                                    std::memory_order_relaxed);
    total_size += data_size;
  }

  // repeated uint32 upsample_strides = 5;
  {
    size_t data_size = ::_pbi::WireFormatLite::
      UInt32Size(this->_impl_.upsample_strides_);
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    int cached_size = ::_pbi::ToCachedSize(data_size);
    _impl_._upsample_strides_cached_byte_size_.store(cached_size,
                                    std::memory_order_relaxed);
    total_size += data_size;
  }

  // repeated uint32 num_upsample_filters = 6;
  {
    size_t data_size = ::_pbi::WireFormatLite::
      UInt32Size(this->_impl_.num_upsample_filters_);
    if (data_size > 0) {
      total_size += 1 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    int cached_size = ::_pbi::ToCachedSize(data_size);
    _impl_._num_upsample_filters_cached_byte_size_.store(cached_size,
                                    std::memory_order_relaxed);
    total_size += data_size;
  }

  // string module_class_name = 1;
  if (!this->_internal_module_class_name().empty()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
        this->_internal_module_class_name());
  }

  // bool use_groupnorm = 7;
  if (this->_internal_use_groupnorm() != 0) {
    total_size += 1 + 1;
  }

  // uint32 num_groups = 8;
  if (this->_internal_num_groups() != 0) {
    total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_num_groups());
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData VoxelNet_RPN::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    VoxelNet_RPN::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*VoxelNet_RPN::GetClassData() const { return &_class_data_; }


void VoxelNet_RPN::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<VoxelNet_RPN*>(&to_msg);
  auto& from = static_cast<const VoxelNet_RPN&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.VoxelNet.RPN)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.layer_nums_.MergeFrom(from._impl_.layer_nums_);
  _this->_impl_.layer_strides_.MergeFrom(from._impl_.layer_strides_);
  _this->_impl_.num_filters_.MergeFrom(from._impl_.num_filters_);
  _this->_impl_.upsample_strides_.MergeFrom(from._impl_.upsample_strides_);
  _this->_impl_.num_upsample_filters_.MergeFrom(from._impl_.num_upsample_filters_);
  if (!from._internal_module_class_name().empty()) {
    _this->_internal_set_module_class_name(from._internal_module_class_name());
  }
  if (from._internal_use_groupnorm() != 0) {
    _this->_internal_set_use_groupnorm(from._internal_use_groupnorm());
  }
  if (from._internal_num_groups() != 0) {
    _this->_internal_set_num_groups(from._internal_num_groups());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void VoxelNet_RPN::CopyFrom(const VoxelNet_RPN& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.VoxelNet.RPN)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool VoxelNet_RPN::IsInitialized() const {
  return true;
}

void VoxelNet_RPN::InternalSwap(VoxelNet_RPN* other) {
  using std::swap;
  auto* lhs_arena = GetArenaForAllocation();
  auto* rhs_arena = other->GetArenaForAllocation();
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  _impl_.layer_nums_.InternalSwap(&other->_impl_.layer_nums_);
  _impl_.layer_strides_.InternalSwap(&other->_impl_.layer_strides_);
  _impl_.num_filters_.InternalSwap(&other->_impl_.num_filters_);
  _impl_.upsample_strides_.InternalSwap(&other->_impl_.upsample_strides_);
  _impl_.num_upsample_filters_.InternalSwap(&other->_impl_.num_upsample_filters_);
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.module_class_name_, lhs_arena,
      &other->_impl_.module_class_name_, rhs_arena
  );
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(VoxelNet_RPN, _impl_.num_groups_)
      + sizeof(VoxelNet_RPN::_impl_.num_groups_)
      - PROTOBUF_FIELD_OFFSET(VoxelNet_RPN, _impl_.use_groupnorm_)>(
          reinterpret_cast<char*>(&_impl_.use_groupnorm_),
          reinterpret_cast<char*>(&other->_impl_.use_groupnorm_));
}

::PROTOBUF_NAMESPACE_ID::Metadata VoxelNet_RPN::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fsecond_2eproto_getter, &descriptor_table_second_2fprotos_2fsecond_2eproto_once,
      file_level_metadata_second_2fprotos_2fsecond_2eproto[2]);
}

// ===================================================================

class VoxelNet::_Internal {
 public:
  static const ::second::protos::VoxelNet_VoxelFeatureExtractor& voxel_feature_extractor(const VoxelNet* msg);
  static const ::second::protos::VoxelNet_MiddleFeatureExtractor& middle_feature_extractor(const VoxelNet* msg);
  static const ::second::protos::VoxelNet_RPN& rpn(const VoxelNet* msg);
  static const ::second::protos::Loss& loss(const VoxelNet* msg);
  static const ::second::protos::BoxCoder& box_coder(const VoxelNet* msg);
  static const ::second::protos::TargetAssigner& target_assigner(const VoxelNet* msg);
  static const ::second::protos::VoxelGenerator& voxel_generator(const VoxelNet* msg);
};

const ::second::protos::VoxelNet_VoxelFeatureExtractor&
VoxelNet::_Internal::voxel_feature_extractor(const VoxelNet* msg) {
  return *msg->_impl_.voxel_feature_extractor_;
}
const ::second::protos::VoxelNet_MiddleFeatureExtractor&
VoxelNet::_Internal::middle_feature_extractor(const VoxelNet* msg) {
  return *msg->_impl_.middle_feature_extractor_;
}
const ::second::protos::VoxelNet_RPN&
VoxelNet::_Internal::rpn(const VoxelNet* msg) {
  return *msg->_impl_.rpn_;
}
const ::second::protos::Loss&
VoxelNet::_Internal::loss(const VoxelNet* msg) {
  return *msg->_impl_.loss_;
}
const ::second::protos::BoxCoder&
VoxelNet::_Internal::box_coder(const VoxelNet* msg) {
  return *msg->_impl_.box_coder_;
}
const ::second::protos::TargetAssigner&
VoxelNet::_Internal::target_assigner(const VoxelNet* msg) {
  return *msg->_impl_.target_assigner_;
}
const ::second::protos::VoxelGenerator&
VoxelNet::_Internal::voxel_generator(const VoxelNet* msg) {
  return *msg->_impl_.voxel_generator_;
}
void VoxelNet::clear_loss() {
  if (GetArenaForAllocation() == nullptr && _impl_.loss_ != nullptr) {
    delete _impl_.loss_;
  }
  _impl_.loss_ = nullptr;
}
void VoxelNet::clear_box_coder() {
  if (GetArenaForAllocation() == nullptr && _impl_.box_coder_ != nullptr) {
    delete _impl_.box_coder_;
  }
  _impl_.box_coder_ = nullptr;
}
void VoxelNet::clear_target_assigner() {
  if (GetArenaForAllocation() == nullptr && _impl_.target_assigner_ != nullptr) {
    delete _impl_.target_assigner_;
  }
  _impl_.target_assigner_ = nullptr;
}
void VoxelNet::clear_voxel_generator() {
  if (GetArenaForAllocation() == nullptr && _impl_.voxel_generator_ != nullptr) {
    delete _impl_.voxel_generator_;
  }
  _impl_.voxel_generator_ = nullptr;
}
VoxelNet::VoxelNet(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.VoxelNet)
}
VoxelNet::VoxelNet(const VoxelNet& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  VoxelNet* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.post_center_limit_range_){from._impl_.post_center_limit_range_}
    , decltype(_impl_.voxel_feature_extractor_){nullptr}
    , decltype(_impl_.middle_feature_extractor_){nullptr}
    , decltype(_impl_.rpn_){nullptr}
    , decltype(_impl_.loss_){nullptr}
    , decltype(_impl_.box_coder_){nullptr}
    , decltype(_impl_.target_assigner_){nullptr}
    , decltype(_impl_.voxel_generator_){nullptr}
    , decltype(_impl_.num_class_){}
    , decltype(_impl_.use_sigmoid_score_){}
    , decltype(_impl_.encode_rad_error_by_sin_){}
    , decltype(_impl_.encode_background_as_zeros_){}
    , decltype(_impl_.use_aux_classifier_){}
    , decltype(_impl_.nms_pre_max_size_){}
    , decltype(_impl_.nms_post_max_size_){}
    , decltype(_impl_.nms_score_threshold_){}
    , decltype(_impl_.nms_iou_threshold_){}
    , decltype(_impl_.use_rotate_nms_){}
    , decltype(_impl_.use_multi_class_nms_){}
    , decltype(_impl_.use_direction_classifier_){}
    , decltype(_impl_.use_bev_){}
    , decltype(_impl_.direction_loss_weight_){}
    , decltype(_impl_.pos_class_weight_){}
    , decltype(_impl_.neg_class_weight_){}
    , decltype(_impl_.loss_norm_type_){}
    , decltype(_impl_.without_reflectivity_){}
    , decltype(_impl_.encode_angle_to_vector_){}
    , decltype(_impl_.lidar_input_){}
    , decltype(_impl_.num_point_features_){}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  if (from._internal_has_voxel_feature_extractor()) {
    _this->_impl_.voxel_feature_extractor_ = new ::second::protos::VoxelNet_VoxelFeatureExtractor(*from._impl_.voxel_feature_extractor_);
  }
  if (from._internal_has_middle_feature_extractor()) {
    _this->_impl_.middle_feature_extractor_ = new ::second::protos::VoxelNet_MiddleFeatureExtractor(*from._impl_.middle_feature_extractor_);
  }
  if (from._internal_has_rpn()) {
    _this->_impl_.rpn_ = new ::second::protos::VoxelNet_RPN(*from._impl_.rpn_);
  }
  if (from._internal_has_loss()) {
    _this->_impl_.loss_ = new ::second::protos::Loss(*from._impl_.loss_);
  }
  if (from._internal_has_box_coder()) {
    _this->_impl_.box_coder_ = new ::second::protos::BoxCoder(*from._impl_.box_coder_);
  }
  if (from._internal_has_target_assigner()) {
    _this->_impl_.target_assigner_ = new ::second::protos::TargetAssigner(*from._impl_.target_assigner_);
  }
  if (from._internal_has_voxel_generator()) {
    _this->_impl_.voxel_generator_ = new ::second::protos::VoxelGenerator(*from._impl_.voxel_generator_);
  }
  ::memcpy(&_impl_.num_class_, &from._impl_.num_class_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.num_point_features_) -
    reinterpret_cast<char*>(&_impl_.num_class_)) + sizeof(_impl_.num_point_features_));
  // @@protoc_insertion_point(copy_constructor:second.protos.VoxelNet)
}

inline void VoxelNet::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.post_center_limit_range_){arena}
    , decltype(_impl_.voxel_feature_extractor_){nullptr}
    , decltype(_impl_.middle_feature_extractor_){nullptr}
    , decltype(_impl_.rpn_){nullptr}
    , decltype(_impl_.loss_){nullptr}
    , decltype(_impl_.box_coder_){nullptr}
    , decltype(_impl_.target_assigner_){nullptr}
    , decltype(_impl_.voxel_generator_){nullptr}
    , decltype(_impl_.num_class_){0u}
    , decltype(_impl_.use_sigmoid_score_){false}
    , decltype(_impl_.encode_rad_error_by_sin_){false}
    , decltype(_impl_.encode_background_as_zeros_){false}
    , decltype(_impl_.use_aux_classifier_){false}
    , decltype(_impl_.nms_pre_max_size_){0u}
    , decltype(_impl_.nms_post_max_size_){0u}
    , decltype(_impl_.nms_score_threshold_){0}
    , decltype(_impl_.nms_iou_threshold_){0}
    , decltype(_impl_.use_rotate_nms_){false}
    , decltype(_impl_.use_multi_class_nms_){false}
    , decltype(_impl_.use_direction_classifier_){false}
    , decltype(_impl_.use_bev_){false}
    , decltype(_impl_.direction_loss_weight_){0}
    , decltype(_impl_.pos_class_weight_){0}
    , decltype(_impl_.neg_class_weight_){0}
    , decltype(_impl_.loss_norm_type_){0}
    , decltype(_impl_.without_reflectivity_){false}
    , decltype(_impl_.encode_angle_to_vector_){false}
    , decltype(_impl_.lidar_input_){false}
    , decltype(_impl_.num_point_features_){0u}
    , /*decltype(_impl_._cached_size_)*/{}
  };
}

VoxelNet::~VoxelNet() {
  // @@protoc_insertion_point(destructor:second.protos.VoxelNet)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void VoxelNet::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.post_center_limit_range_.~RepeatedField();
  if (this != internal_default_instance()) delete _impl_.voxel_feature_extractor_;
  if (this != internal_default_instance()) delete _impl_.middle_feature_extractor_;
  if (this != internal_default_instance()) delete _impl_.rpn_;
  if (this != internal_default_instance()) delete _impl_.loss_;
  if (this != internal_default_instance()) delete _impl_.box_coder_;
  if (this != internal_default_instance()) delete _impl_.target_assigner_;
  if (this != internal_default_instance()) delete _impl_.voxel_generator_;
}

void VoxelNet::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void VoxelNet::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.VoxelNet)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.post_center_limit_range_.Clear();
  if (GetArenaForAllocation() == nullptr && _impl_.voxel_feature_extractor_ != nullptr) {
    delete _impl_.voxel_feature_extractor_;
  }
  _impl_.voxel_feature_extractor_ = nullptr;
  if (GetArenaForAllocation() == nullptr && _impl_.middle_feature_extractor_ != nullptr) {
    delete _impl_.middle_feature_extractor_;
  }
  _impl_.middle_feature_extractor_ = nullptr;
  if (GetArenaForAllocation() == nullptr && _impl_.rpn_ != nullptr) {
    delete _impl_.rpn_;
  }
  _impl_.rpn_ = nullptr;
  if (GetArenaForAllocation() == nullptr && _impl_.loss_ != nullptr) {
    delete _impl_.loss_;
  }
  _impl_.loss_ = nullptr;
  if (GetArenaForAllocation() == nullptr && _impl_.box_coder_ != nullptr) {
    delete _impl_.box_coder_;
  }
  _impl_.box_coder_ = nullptr;
  if (GetArenaForAllocation() == nullptr && _impl_.target_assigner_ != nullptr) {
    delete _impl_.target_assigner_;
  }
  _impl_.target_assigner_ = nullptr;
  if (GetArenaForAllocation() == nullptr && _impl_.voxel_generator_ != nullptr) {
    delete _impl_.voxel_generator_;
  }
  _impl_.voxel_generator_ = nullptr;
  ::memset(&_impl_.num_class_, 0, static_cast<size_t>(
      reinterpret_cast<char*>(&_impl_.num_point_features_) -
      reinterpret_cast<char*>(&_impl_.num_class_)) + sizeof(_impl_.num_point_features_));
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* VoxelNet::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // uint32 num_class = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 8)) {
          _impl_.num_class_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.VoxelNet.VoxelFeatureExtractor voxel_feature_extractor = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 18)) {
          ptr = ctx->ParseMessage(_internal_mutable_voxel_feature_extractor(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.VoxelNet.MiddleFeatureExtractor middle_feature_extractor = 3;
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 26)) {
          ptr = ctx->ParseMessage(_internal_mutable_middle_feature_extractor(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.VoxelNet.RPN rpn = 4;
      case 4:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 34)) {
          ptr = ctx->ParseMessage(_internal_mutable_rpn(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool use_sigmoid_score = 5;
      case 5:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 40)) {
          _impl_.use_sigmoid_score_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.Loss loss = 6;
      case 6:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 50)) {
          ptr = ctx->ParseMessage(_internal_mutable_loss(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool encode_rad_error_by_sin = 7;
      case 7:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 56)) {
          _impl_.encode_rad_error_by_sin_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool encode_background_as_zeros = 8;
      case 8:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 64)) {
          _impl_.encode_background_as_zeros_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool use_aux_classifier = 9;
      case 9:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 72)) {
          _impl_.use_aux_classifier_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool use_rotate_nms = 10;
      case 10:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 80)) {
          _impl_.use_rotate_nms_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool use_multi_class_nms = 11;
      case 11:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 88)) {
          _impl_.use_multi_class_nms_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // uint32 nms_pre_max_size = 12;
      case 12:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 96)) {
          _impl_.nms_pre_max_size_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // uint32 nms_post_max_size = 13;
      case 13:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 104)) {
          _impl_.nms_post_max_size_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // float nms_score_threshold = 14;
      case 14:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 117)) {
          _impl_.nms_score_threshold_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float nms_iou_threshold = 15;
      case 15:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 125)) {
          _impl_.nms_iou_threshold_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // repeated float post_center_limit_range = 16;
      case 16:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 130)) {
          ptr = ::PROTOBUF_NAMESPACE_ID::internal::PackedFloatParser(_internal_mutable_post_center_limit_range(), ptr, ctx);
          CHK_(ptr);
        } else if (static_cast<uint8_t>(tag) == 133) {
          _internal_add_post_center_limit_range(::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr));
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // bool use_direction_classifier = 17;
      case 17:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 136)) {
          _impl_.use_direction_classifier_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // float direction_loss_weight = 18;
      case 18:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 149)) {
          _impl_.direction_loss_weight_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float pos_class_weight = 19;
      case 19:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 157)) {
          _impl_.pos_class_weight_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // float neg_class_weight = 20;
      case 20:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 165)) {
          _impl_.neg_class_weight_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.VoxelNet.LossNormType loss_norm_type = 21;
      case 21:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 168)) {
          uint64_t val = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
          _internal_set_loss_norm_type(static_cast<::second::protos::VoxelNet_LossNormType>(val));
        } else
          goto handle_unusual;
        continue;
      // bool use_bev = 22;
      case 22:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 176)) {
          _impl_.use_bev_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool without_reflectivity = 23;
      case 23:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 184)) {
          _impl_.without_reflectivity_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool encode_angle_to_vector = 24;
      case 24:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 192)) {
          _impl_.encode_angle_to_vector_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.BoxCoder box_coder = 25;
      case 25:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 202)) {
          ptr = ctx->ParseMessage(_internal_mutable_box_coder(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.TargetAssigner target_assigner = 26;
      case 26:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 210)) {
          ptr = ctx->ParseMessage(_internal_mutable_target_assigner(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool lidar_input = 27;
      case 27:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 216)) {
          _impl_.lidar_input_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // uint32 num_point_features = 28;
      case 28:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 224)) {
          _impl_.num_point_features_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .second.protos.VoxelGenerator voxel_generator = 29;
      case 29:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 234)) {
          ptr = ctx->ParseMessage(_internal_mutable_voxel_generator(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* VoxelNet::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.VoxelNet)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // uint32 num_class = 1;
  if (this->_internal_num_class() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(1, this->_internal_num_class(), target);
  }

  // .second.protos.VoxelNet.VoxelFeatureExtractor voxel_feature_extractor = 2;
  if (this->_internal_has_voxel_feature_extractor()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(2, _Internal::voxel_feature_extractor(this),
        _Internal::voxel_feature_extractor(this).GetCachedSize(), target, stream);
  }

  // .second.protos.VoxelNet.MiddleFeatureExtractor middle_feature_extractor = 3;
  if (this->_internal_has_middle_feature_extractor()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(3, _Internal::middle_feature_extractor(this),
        _Internal::middle_feature_extractor(this).GetCachedSize(), target, stream);
  }

  // .second.protos.VoxelNet.RPN rpn = 4;
  if (this->_internal_has_rpn()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(4, _Internal::rpn(this),
        _Internal::rpn(this).GetCachedSize(), target, stream);
  }

  // bool use_sigmoid_score = 5;
  if (this->_internal_use_sigmoid_score() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(5, this->_internal_use_sigmoid_score(), target);
  }

  // .second.protos.Loss loss = 6;
  if (this->_internal_has_loss()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(6, _Internal::loss(this),
        _Internal::loss(this).GetCachedSize(), target, stream);
  }

  // bool encode_rad_error_by_sin = 7;
  if (this->_internal_encode_rad_error_by_sin() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(7, this->_internal_encode_rad_error_by_sin(), target);
  }

  // bool encode_background_as_zeros = 8;
  if (this->_internal_encode_background_as_zeros() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(8, this->_internal_encode_background_as_zeros(), target);
  }

  // bool use_aux_classifier = 9;
  if (this->_internal_use_aux_classifier() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(9, this->_internal_use_aux_classifier(), target);
  }

  // bool use_rotate_nms = 10;
  if (this->_internal_use_rotate_nms() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(10, this->_internal_use_rotate_nms(), target);
  }

  // bool use_multi_class_nms = 11;
  if (this->_internal_use_multi_class_nms() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(11, this->_internal_use_multi_class_nms(), target);
  }

  // uint32 nms_pre_max_size = 12;
  if (this->_internal_nms_pre_max_size() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(12, this->_internal_nms_pre_max_size(), target);
  }

  // uint32 nms_post_max_size = 13;
  if (this->_internal_nms_post_max_size() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(13, this->_internal_nms_post_max_size(), target);
  }

  // float nms_score_threshold = 14;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_nms_score_threshold = this->_internal_nms_score_threshold();
  uint32_t raw_nms_score_threshold;
  memcpy(&raw_nms_score_threshold, &tmp_nms_score_threshold, sizeof(tmp_nms_score_threshold));
  if (raw_nms_score_threshold != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(14, this->_internal_nms_score_threshold(), target);
  }

  // float nms_iou_threshold = 15;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_nms_iou_threshold = this->_internal_nms_iou_threshold();
  uint32_t raw_nms_iou_threshold;
  memcpy(&raw_nms_iou_threshold, &tmp_nms_iou_threshold, sizeof(tmp_nms_iou_threshold));
  if (raw_nms_iou_threshold != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(15, this->_internal_nms_iou_threshold(), target);
  }

  // repeated float post_center_limit_range = 16;
  if (this->_internal_post_center_limit_range_size() > 0) {
    target = stream->WriteFixedPacked(16, _internal_post_center_limit_range(), target);
  }

  // bool use_direction_classifier = 17;
  if (this->_internal_use_direction_classifier() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(17, this->_internal_use_direction_classifier(), target);
  }

  // float direction_loss_weight = 18;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_direction_loss_weight = this->_internal_direction_loss_weight();
  uint32_t raw_direction_loss_weight;
  memcpy(&raw_direction_loss_weight, &tmp_direction_loss_weight, sizeof(tmp_direction_loss_weight));
  if (raw_direction_loss_weight != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(18, this->_internal_direction_loss_weight(), target);
  }

  // float pos_class_weight = 19;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_pos_class_weight = this->_internal_pos_class_weight();
  uint32_t raw_pos_class_weight;
  memcpy(&raw_pos_class_weight, &tmp_pos_class_weight, sizeof(tmp_pos_class_weight));
  if (raw_pos_class_weight != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(19, this->_internal_pos_class_weight(), target);
  }

  // float neg_class_weight = 20;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_neg_class_weight = this->_internal_neg_class_weight();
  uint32_t raw_neg_class_weight;
  memcpy(&raw_neg_class_weight, &tmp_neg_class_weight, sizeof(tmp_neg_class_weight));
  if (raw_neg_class_weight != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(20, this->_internal_neg_class_weight(), target);
  }

  // .second.protos.VoxelNet.LossNormType loss_norm_type = 21;
  if (this->_internal_loss_norm_type() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteEnumToArray(
      21, this->_internal_loss_norm_type(), target);
  }

  // bool use_bev = 22;
  if (this->_internal_use_bev() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(22, this->_internal_use_bev(), target);
  }

  // bool without_reflectivity = 23;
  if (this->_internal_without_reflectivity() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(23, this->_internal_without_reflectivity(), target);
  }

  // bool encode_angle_to_vector = 24;
  if (this->_internal_encode_angle_to_vector() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(24, this->_internal_encode_angle_to_vector(), target);
  }

  // .second.protos.BoxCoder box_coder = 25;
  if (this->_internal_has_box_coder()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(25, _Internal::box_coder(this),
        _Internal::box_coder(this).GetCachedSize(), target, stream);
  }

  // .second.protos.TargetAssigner target_assigner = 26;
  if (this->_internal_has_target_assigner()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(26, _Internal::target_assigner(this),
        _Internal::target_assigner(this).GetCachedSize(), target, stream);
  }

  // bool lidar_input = 27;
  if (this->_internal_lidar_input() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(27, this->_internal_lidar_input(), target);
  }

  // uint32 num_point_features = 28;
  if (this->_internal_num_point_features() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(28, this->_internal_num_point_features(), target);
  }

  // .second.protos.VoxelGenerator voxel_generator = 29;
  if (this->_internal_has_voxel_generator()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(29, _Internal::voxel_generator(this),
        _Internal::voxel_generator(this).GetCachedSize(), target, stream);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.VoxelNet)
  return target;
}

size_t VoxelNet::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.VoxelNet)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // repeated float post_center_limit_range = 16;
  {
    unsigned int count = static_cast<unsigned int>(this->_internal_post_center_limit_range_size());
    size_t data_size = 4UL * count;
    if (data_size > 0) {
      total_size += 2 +
        ::_pbi::WireFormatLite::Int32Size(static_cast<int32_t>(data_size));
    }
    total_size += data_size;
  }

  // .second.protos.VoxelNet.VoxelFeatureExtractor voxel_feature_extractor = 2;
  if (this->_internal_has_voxel_feature_extractor()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.voxel_feature_extractor_);
  }

  // .second.protos.VoxelNet.MiddleFeatureExtractor middle_feature_extractor = 3;
  if (this->_internal_has_middle_feature_extractor()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.middle_feature_extractor_);
  }

  // .second.protos.VoxelNet.RPN rpn = 4;
  if (this->_internal_has_rpn()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.rpn_);
  }

  // .second.protos.Loss loss = 6;
  if (this->_internal_has_loss()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.loss_);
  }

  // .second.protos.BoxCoder box_coder = 25;
  if (this->_internal_has_box_coder()) {
    total_size += 2 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.box_coder_);
  }

  // .second.protos.TargetAssigner target_assigner = 26;
  if (this->_internal_has_target_assigner()) {
    total_size += 2 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.target_assigner_);
  }

  // .second.protos.VoxelGenerator voxel_generator = 29;
  if (this->_internal_has_voxel_generator()) {
    total_size += 2 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.voxel_generator_);
  }

  // uint32 num_class = 1;
  if (this->_internal_num_class() != 0) {
    total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_num_class());
  }

  // bool use_sigmoid_score = 5;
  if (this->_internal_use_sigmoid_score() != 0) {
    total_size += 1 + 1;
  }

  // bool encode_rad_error_by_sin = 7;
  if (this->_internal_encode_rad_error_by_sin() != 0) {
    total_size += 1 + 1;
  }

  // bool encode_background_as_zeros = 8;
  if (this->_internal_encode_background_as_zeros() != 0) {
    total_size += 1 + 1;
  }

  // bool use_aux_classifier = 9;
  if (this->_internal_use_aux_classifier() != 0) {
    total_size += 1 + 1;
  }

  // uint32 nms_pre_max_size = 12;
  if (this->_internal_nms_pre_max_size() != 0) {
    total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_nms_pre_max_size());
  }

  // uint32 nms_post_max_size = 13;
  if (this->_internal_nms_post_max_size() != 0) {
    total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_nms_post_max_size());
  }

  // float nms_score_threshold = 14;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_nms_score_threshold = this->_internal_nms_score_threshold();
  uint32_t raw_nms_score_threshold;
  memcpy(&raw_nms_score_threshold, &tmp_nms_score_threshold, sizeof(tmp_nms_score_threshold));
  if (raw_nms_score_threshold != 0) {
    total_size += 1 + 4;
  }

  // float nms_iou_threshold = 15;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_nms_iou_threshold = this->_internal_nms_iou_threshold();
  uint32_t raw_nms_iou_threshold;
  memcpy(&raw_nms_iou_threshold, &tmp_nms_iou_threshold, sizeof(tmp_nms_iou_threshold));
  if (raw_nms_iou_threshold != 0) {
    total_size += 1 + 4;
  }

  // bool use_rotate_nms = 10;
  if (this->_internal_use_rotate_nms() != 0) {
    total_size += 1 + 1;
  }

  // bool use_multi_class_nms = 11;
  if (this->_internal_use_multi_class_nms() != 0) {
    total_size += 1 + 1;
  }

  // bool use_direction_classifier = 17;
  if (this->_internal_use_direction_classifier() != 0) {
    total_size += 2 + 1;
  }

  // bool use_bev = 22;
  if (this->_internal_use_bev() != 0) {
    total_size += 2 + 1;
  }

  // float direction_loss_weight = 18;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_direction_loss_weight = this->_internal_direction_loss_weight();
  uint32_t raw_direction_loss_weight;
  memcpy(&raw_direction_loss_weight, &tmp_direction_loss_weight, sizeof(tmp_direction_loss_weight));
  if (raw_direction_loss_weight != 0) {
    total_size += 2 + 4;
  }

  // float pos_class_weight = 19;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_pos_class_weight = this->_internal_pos_class_weight();
  uint32_t raw_pos_class_weight;
  memcpy(&raw_pos_class_weight, &tmp_pos_class_weight, sizeof(tmp_pos_class_weight));
  if (raw_pos_class_weight != 0) {
    total_size += 2 + 4;
  }

  // float neg_class_weight = 20;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_neg_class_weight = this->_internal_neg_class_weight();
  uint32_t raw_neg_class_weight;
  memcpy(&raw_neg_class_weight, &tmp_neg_class_weight, sizeof(tmp_neg_class_weight));
  if (raw_neg_class_weight != 0) {
    total_size += 2 + 4;
  }

  // .second.protos.VoxelNet.LossNormType loss_norm_type = 21;
  if (this->_internal_loss_norm_type() != 0) {
    total_size += 2 +
      ::_pbi::WireFormatLite::EnumSize(this->_internal_loss_norm_type());
  }

  // bool without_reflectivity = 23;
  if (this->_internal_without_reflectivity() != 0) {
    total_size += 2 + 1;
  }

  // bool encode_angle_to_vector = 24;
  if (this->_internal_encode_angle_to_vector() != 0) {
    total_size += 2 + 1;
  }

  // bool lidar_input = 27;
  if (this->_internal_lidar_input() != 0) {
    total_size += 2 + 1;
  }

  // uint32 num_point_features = 28;
  if (this->_internal_num_point_features() != 0) {
    total_size += 2 +
      ::_pbi::WireFormatLite::UInt32Size(
        this->_internal_num_point_features());
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData VoxelNet::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    VoxelNet::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*VoxelNet::GetClassData() const { return &_class_data_; }


void VoxelNet::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<VoxelNet*>(&to_msg);
  auto& from = static_cast<const VoxelNet&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.VoxelNet)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.post_center_limit_range_.MergeFrom(from._impl_.post_center_limit_range_);
  if (from._internal_has_voxel_feature_extractor()) {
    _this->_internal_mutable_voxel_feature_extractor()->::second::protos::VoxelNet_VoxelFeatureExtractor::MergeFrom(
        from._internal_voxel_feature_extractor());
  }
  if (from._internal_has_middle_feature_extractor()) {
    _this->_internal_mutable_middle_feature_extractor()->::second::protos::VoxelNet_MiddleFeatureExtractor::MergeFrom(
        from._internal_middle_feature_extractor());
  }
  if (from._internal_has_rpn()) {
    _this->_internal_mutable_rpn()->::second::protos::VoxelNet_RPN::MergeFrom(
        from._internal_rpn());
  }
  if (from._internal_has_loss()) {
    _this->_internal_mutable_loss()->::second::protos::Loss::MergeFrom(
        from._internal_loss());
  }
  if (from._internal_has_box_coder()) {
    _this->_internal_mutable_box_coder()->::second::protos::BoxCoder::MergeFrom(
        from._internal_box_coder());
  }
  if (from._internal_has_target_assigner()) {
    _this->_internal_mutable_target_assigner()->::second::protos::TargetAssigner::MergeFrom(
        from._internal_target_assigner());
  }
  if (from._internal_has_voxel_generator()) {
    _this->_internal_mutable_voxel_generator()->::second::protos::VoxelGenerator::MergeFrom(
        from._internal_voxel_generator());
  }
  if (from._internal_num_class() != 0) {
    _this->_internal_set_num_class(from._internal_num_class());
  }
  if (from._internal_use_sigmoid_score() != 0) {
    _this->_internal_set_use_sigmoid_score(from._internal_use_sigmoid_score());
  }
  if (from._internal_encode_rad_error_by_sin() != 0) {
    _this->_internal_set_encode_rad_error_by_sin(from._internal_encode_rad_error_by_sin());
  }
  if (from._internal_encode_background_as_zeros() != 0) {
    _this->_internal_set_encode_background_as_zeros(from._internal_encode_background_as_zeros());
  }
  if (from._internal_use_aux_classifier() != 0) {
    _this->_internal_set_use_aux_classifier(from._internal_use_aux_classifier());
  }
  if (from._internal_nms_pre_max_size() != 0) {
    _this->_internal_set_nms_pre_max_size(from._internal_nms_pre_max_size());
  }
  if (from._internal_nms_post_max_size() != 0) {
    _this->_internal_set_nms_post_max_size(from._internal_nms_post_max_size());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_nms_score_threshold = from._internal_nms_score_threshold();
  uint32_t raw_nms_score_threshold;
  memcpy(&raw_nms_score_threshold, &tmp_nms_score_threshold, sizeof(tmp_nms_score_threshold));
  if (raw_nms_score_threshold != 0) {
    _this->_internal_set_nms_score_threshold(from._internal_nms_score_threshold());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_nms_iou_threshold = from._internal_nms_iou_threshold();
  uint32_t raw_nms_iou_threshold;
  memcpy(&raw_nms_iou_threshold, &tmp_nms_iou_threshold, sizeof(tmp_nms_iou_threshold));
  if (raw_nms_iou_threshold != 0) {
    _this->_internal_set_nms_iou_threshold(from._internal_nms_iou_threshold());
  }
  if (from._internal_use_rotate_nms() != 0) {
    _this->_internal_set_use_rotate_nms(from._internal_use_rotate_nms());
  }
  if (from._internal_use_multi_class_nms() != 0) {
    _this->_internal_set_use_multi_class_nms(from._internal_use_multi_class_nms());
  }
  if (from._internal_use_direction_classifier() != 0) {
    _this->_internal_set_use_direction_classifier(from._internal_use_direction_classifier());
  }
  if (from._internal_use_bev() != 0) {
    _this->_internal_set_use_bev(from._internal_use_bev());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_direction_loss_weight = from._internal_direction_loss_weight();
  uint32_t raw_direction_loss_weight;
  memcpy(&raw_direction_loss_weight, &tmp_direction_loss_weight, sizeof(tmp_direction_loss_weight));
  if (raw_direction_loss_weight != 0) {
    _this->_internal_set_direction_loss_weight(from._internal_direction_loss_weight());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_pos_class_weight = from._internal_pos_class_weight();
  uint32_t raw_pos_class_weight;
  memcpy(&raw_pos_class_weight, &tmp_pos_class_weight, sizeof(tmp_pos_class_weight));
  if (raw_pos_class_weight != 0) {
    _this->_internal_set_pos_class_weight(from._internal_pos_class_weight());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_neg_class_weight = from._internal_neg_class_weight();
  uint32_t raw_neg_class_weight;
  memcpy(&raw_neg_class_weight, &tmp_neg_class_weight, sizeof(tmp_neg_class_weight));
  if (raw_neg_class_weight != 0) {
    _this->_internal_set_neg_class_weight(from._internal_neg_class_weight());
  }
  if (from._internal_loss_norm_type() != 0) {
    _this->_internal_set_loss_norm_type(from._internal_loss_norm_type());
  }
  if (from._internal_without_reflectivity() != 0) {
    _this->_internal_set_without_reflectivity(from._internal_without_reflectivity());
  }
  if (from._internal_encode_angle_to_vector() != 0) {
    _this->_internal_set_encode_angle_to_vector(from._internal_encode_angle_to_vector());
  }
  if (from._internal_lidar_input() != 0) {
    _this->_internal_set_lidar_input(from._internal_lidar_input());
  }
  if (from._internal_num_point_features() != 0) {
    _this->_internal_set_num_point_features(from._internal_num_point_features());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void VoxelNet::CopyFrom(const VoxelNet& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.VoxelNet)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool VoxelNet::IsInitialized() const {
  return true;
}

void VoxelNet::InternalSwap(VoxelNet* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  _impl_.post_center_limit_range_.InternalSwap(&other->_impl_.post_center_limit_range_);
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(VoxelNet, _impl_.num_point_features_)
      + sizeof(VoxelNet::_impl_.num_point_features_)
      - PROTOBUF_FIELD_OFFSET(VoxelNet, _impl_.voxel_feature_extractor_)>(
          reinterpret_cast<char*>(&_impl_.voxel_feature_extractor_),
          reinterpret_cast<char*>(&other->_impl_.voxel_feature_extractor_));
}

::PROTOBUF_NAMESPACE_ID::Metadata VoxelNet::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2fsecond_2eproto_getter, &descriptor_table_second_2fprotos_2fsecond_2eproto_once,
      file_level_metadata_second_2fprotos_2fsecond_2eproto[3]);
}

// @@protoc_insertion_point(namespace_scope)
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> PROTOBUF_NOINLINE ::second::protos::VoxelNet_VoxelFeatureExtractor*
Arena::CreateMaybeMessage< ::second::protos::VoxelNet_VoxelFeatureExtractor >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::VoxelNet_VoxelFeatureExtractor >(arena);
}
template<> PROTOBUF_NOINLINE ::second::protos::VoxelNet_MiddleFeatureExtractor*
Arena::CreateMaybeMessage< ::second::protos::VoxelNet_MiddleFeatureExtractor >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::VoxelNet_MiddleFeatureExtractor >(arena);
}
template<> PROTOBUF_NOINLINE ::second::protos::VoxelNet_RPN*
Arena::CreateMaybeMessage< ::second::protos::VoxelNet_RPN >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::VoxelNet_RPN >(arena);
}
template<> PROTOBUF_NOINLINE ::second::protos::VoxelNet*
Arena::CreateMaybeMessage< ::second::protos::VoxelNet >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::VoxelNet >(arena);
}
PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)
#include <google/protobuf/port_undef.inc>
