// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: object_detection/protos/string_int_label_map.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fstring_5fint_5flabel_5fmap_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fstring_5fint_5flabel_5fmap_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_object_5fdetection_2fprotos_2fstring_5fint_5flabel_5fmap_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_object_5fdetection_2fprotos_2fstring_5fint_5flabel_5fmap_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_object_5fdetection_2fprotos_2fstring_5fint_5flabel_5fmap_2eproto;
namespace object_detection {
namespace protos {
class StringIntLabelMap;
struct StringIntLabelMapDefaultTypeInternal;
extern StringIntLabelMapDefaultTypeInternal _StringIntLabelMap_default_instance_;
class StringIntLabelMapItem;
struct StringIntLabelMapItemDefaultTypeInternal;
extern StringIntLabelMapItemDefaultTypeInternal _StringIntLabelMapItem_default_instance_;
}  // namespace protos
}  // namespace object_detection
PROTOBUF_NAMESPACE_OPEN
template<> ::object_detection::protos::StringIntLabelMap* Arena::CreateMaybeMessage<::object_detection::protos::StringIntLabelMap>(Arena*);
template<> ::object_detection::protos::StringIntLabelMapItem* Arena::CreateMaybeMessage<::object_detection::protos::StringIntLabelMapItem>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace object_detection {
namespace protos {

// ===================================================================

class StringIntLabelMapItem final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.StringIntLabelMapItem) */ {
 public:
  inline StringIntLabelMapItem() : StringIntLabelMapItem(nullptr) {}
  ~StringIntLabelMapItem() override;
  explicit PROTOBUF_CONSTEXPR StringIntLabelMapItem(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  StringIntLabelMapItem(const StringIntLabelMapItem& from);
  StringIntLabelMapItem(StringIntLabelMapItem&& from) noexcept
    : StringIntLabelMapItem() {
    *this = ::std::move(from);
  }

  inline StringIntLabelMapItem& operator=(const StringIntLabelMapItem& from) {
    CopyFrom(from);
    return *this;
  }
  inline StringIntLabelMapItem& operator=(StringIntLabelMapItem&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const StringIntLabelMapItem& default_instance() {
    return *internal_default_instance();
  }
  static inline const StringIntLabelMapItem* internal_default_instance() {
    return reinterpret_cast<const StringIntLabelMapItem*>(
               &_StringIntLabelMapItem_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(StringIntLabelMapItem& a, StringIntLabelMapItem& b) {
    a.Swap(&b);
  }
  inline void Swap(StringIntLabelMapItem* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StringIntLabelMapItem* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  StringIntLabelMapItem* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<StringIntLabelMapItem>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const StringIntLabelMapItem& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const StringIntLabelMapItem& from) {
    StringIntLabelMapItem::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(StringIntLabelMapItem* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.StringIntLabelMapItem";
  }
  protected:
  explicit StringIntLabelMapItem(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kNameFieldNumber = 1,
    kDisplayNameFieldNumber = 3,
    kIdFieldNumber = 2,
  };
  // optional string name = 1;
  bool has_name() const;
  private:
  bool _internal_has_name() const;
  public:
  void clear_name();
  const std::string& name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_name();
  PROTOBUF_NODISCARD std::string* release_name();
  void set_allocated_name(std::string* name);
  private:
  const std::string& _internal_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_name(const std::string& value);
  std::string* _internal_mutable_name();
  public:

  // optional string display_name = 3;
  bool has_display_name() const;
  private:
  bool _internal_has_display_name() const;
  public:
  void clear_display_name();
  const std::string& display_name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_display_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_display_name();
  PROTOBUF_NODISCARD std::string* release_display_name();
  void set_allocated_display_name(std::string* display_name);
  private:
  const std::string& _internal_display_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_display_name(const std::string& value);
  std::string* _internal_mutable_display_name();
  public:

  // optional int32 id = 2;
  bool has_id() const;
  private:
  bool _internal_has_id() const;
  public:
  void clear_id();
  int32_t id() const;
  void set_id(int32_t value);
  private:
  int32_t _internal_id() const;
  void _internal_set_id(int32_t value);
  public:

  // @@protoc_insertion_point(class_scope:object_detection.protos.StringIntLabelMapItem)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr name_;
    ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr display_name_;
    int32_t id_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fstring_5fint_5flabel_5fmap_2eproto;
};
// -------------------------------------------------------------------

class StringIntLabelMap final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.StringIntLabelMap) */ {
 public:
  inline StringIntLabelMap() : StringIntLabelMap(nullptr) {}
  ~StringIntLabelMap() override;
  explicit PROTOBUF_CONSTEXPR StringIntLabelMap(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  StringIntLabelMap(const StringIntLabelMap& from);
  StringIntLabelMap(StringIntLabelMap&& from) noexcept
    : StringIntLabelMap() {
    *this = ::std::move(from);
  }

  inline StringIntLabelMap& operator=(const StringIntLabelMap& from) {
    CopyFrom(from);
    return *this;
  }
  inline StringIntLabelMap& operator=(StringIntLabelMap&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const StringIntLabelMap& default_instance() {
    return *internal_default_instance();
  }
  static inline const StringIntLabelMap* internal_default_instance() {
    return reinterpret_cast<const StringIntLabelMap*>(
               &_StringIntLabelMap_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(StringIntLabelMap& a, StringIntLabelMap& b) {
    a.Swap(&b);
  }
  inline void Swap(StringIntLabelMap* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StringIntLabelMap* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  StringIntLabelMap* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<StringIntLabelMap>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const StringIntLabelMap& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const StringIntLabelMap& from) {
    StringIntLabelMap::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(StringIntLabelMap* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.StringIntLabelMap";
  }
  protected:
  explicit StringIntLabelMap(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kItemFieldNumber = 1,
  };
  // repeated .object_detection.protos.StringIntLabelMapItem item = 1;
  int item_size() const;
  private:
  int _internal_item_size() const;
  public:
  void clear_item();
  ::object_detection::protos::StringIntLabelMapItem* mutable_item(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::StringIntLabelMapItem >*
      mutable_item();
  private:
  const ::object_detection::protos::StringIntLabelMapItem& _internal_item(int index) const;
  ::object_detection::protos::StringIntLabelMapItem* _internal_add_item();
  public:
  const ::object_detection::protos::StringIntLabelMapItem& item(int index) const;
  ::object_detection::protos::StringIntLabelMapItem* add_item();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::StringIntLabelMapItem >&
      item() const;

  // @@protoc_insertion_point(class_scope:object_detection.protos.StringIntLabelMap)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::StringIntLabelMapItem > item_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fstring_5fint_5flabel_5fmap_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// StringIntLabelMapItem

// optional string name = 1;
inline bool StringIntLabelMapItem::_internal_has_name() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool StringIntLabelMapItem::has_name() const {
  return _internal_has_name();
}
inline void StringIntLabelMapItem::clear_name() {
  _impl_.name_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const std::string& StringIntLabelMapItem::name() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.StringIntLabelMapItem.name)
  return _internal_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void StringIntLabelMapItem::set_name(ArgT0&& arg0, ArgT... args) {
 _impl_._has_bits_[0] |= 0x00000001u;
 _impl_.name_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:object_detection.protos.StringIntLabelMapItem.name)
}
inline std::string* StringIntLabelMapItem::mutable_name() {
  std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.StringIntLabelMapItem.name)
  return _s;
}
inline const std::string& StringIntLabelMapItem::_internal_name() const {
  return _impl_.name_.Get();
}
inline void StringIntLabelMapItem::_internal_set_name(const std::string& value) {
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.name_.Set(value, GetArenaForAllocation());
}
inline std::string* StringIntLabelMapItem::_internal_mutable_name() {
  _impl_._has_bits_[0] |= 0x00000001u;
  return _impl_.name_.Mutable(GetArenaForAllocation());
}
inline std::string* StringIntLabelMapItem::release_name() {
  // @@protoc_insertion_point(field_release:object_detection.protos.StringIntLabelMapItem.name)
  if (!_internal_has_name()) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000001u;
  auto* p = _impl_.name_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void StringIntLabelMapItem::set_allocated_name(std::string* name) {
  if (name != nullptr) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.name_.SetAllocated(name, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.StringIntLabelMapItem.name)
}

// optional int32 id = 2;
inline bool StringIntLabelMapItem::_internal_has_id() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool StringIntLabelMapItem::has_id() const {
  return _internal_has_id();
}
inline void StringIntLabelMapItem::clear_id() {
  _impl_.id_ = 0;
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline int32_t StringIntLabelMapItem::_internal_id() const {
  return _impl_.id_;
}
inline int32_t StringIntLabelMapItem::id() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.StringIntLabelMapItem.id)
  return _internal_id();
}
inline void StringIntLabelMapItem::_internal_set_id(int32_t value) {
  _impl_._has_bits_[0] |= 0x00000004u;
  _impl_.id_ = value;
}
inline void StringIntLabelMapItem::set_id(int32_t value) {
  _internal_set_id(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.StringIntLabelMapItem.id)
}

// optional string display_name = 3;
inline bool StringIntLabelMapItem::_internal_has_display_name() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool StringIntLabelMapItem::has_display_name() const {
  return _internal_has_display_name();
}
inline void StringIntLabelMapItem::clear_display_name() {
  _impl_.display_name_.ClearToEmpty();
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline const std::string& StringIntLabelMapItem::display_name() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.StringIntLabelMapItem.display_name)
  return _internal_display_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void StringIntLabelMapItem::set_display_name(ArgT0&& arg0, ArgT... args) {
 _impl_._has_bits_[0] |= 0x00000002u;
 _impl_.display_name_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:object_detection.protos.StringIntLabelMapItem.display_name)
}
inline std::string* StringIntLabelMapItem::mutable_display_name() {
  std::string* _s = _internal_mutable_display_name();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.StringIntLabelMapItem.display_name)
  return _s;
}
inline const std::string& StringIntLabelMapItem::_internal_display_name() const {
  return _impl_.display_name_.Get();
}
inline void StringIntLabelMapItem::_internal_set_display_name(const std::string& value) {
  _impl_._has_bits_[0] |= 0x00000002u;
  _impl_.display_name_.Set(value, GetArenaForAllocation());
}
inline std::string* StringIntLabelMapItem::_internal_mutable_display_name() {
  _impl_._has_bits_[0] |= 0x00000002u;
  return _impl_.display_name_.Mutable(GetArenaForAllocation());
}
inline std::string* StringIntLabelMapItem::release_display_name() {
  // @@protoc_insertion_point(field_release:object_detection.protos.StringIntLabelMapItem.display_name)
  if (!_internal_has_display_name()) {
    return nullptr;
  }
  _impl_._has_bits_[0] &= ~0x00000002u;
  auto* p = _impl_.display_name_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.display_name_.IsDefault()) {
    _impl_.display_name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void StringIntLabelMapItem::set_allocated_display_name(std::string* display_name) {
  if (display_name != nullptr) {
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  _impl_.display_name_.SetAllocated(display_name, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (_impl_.display_name_.IsDefault()) {
    _impl_.display_name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.StringIntLabelMapItem.display_name)
}

// -------------------------------------------------------------------

// StringIntLabelMap

// repeated .object_detection.protos.StringIntLabelMapItem item = 1;
inline int StringIntLabelMap::_internal_item_size() const {
  return _impl_.item_.size();
}
inline int StringIntLabelMap::item_size() const {
  return _internal_item_size();
}
inline void StringIntLabelMap::clear_item() {
  _impl_.item_.Clear();
}
inline ::object_detection::protos::StringIntLabelMapItem* StringIntLabelMap::mutable_item(int index) {
  // @@protoc_insertion_point(field_mutable:object_detection.protos.StringIntLabelMap.item)
  return _impl_.item_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::StringIntLabelMapItem >*
StringIntLabelMap::mutable_item() {
  // @@protoc_insertion_point(field_mutable_list:object_detection.protos.StringIntLabelMap.item)
  return &_impl_.item_;
}
inline const ::object_detection::protos::StringIntLabelMapItem& StringIntLabelMap::_internal_item(int index) const {
  return _impl_.item_.Get(index);
}
inline const ::object_detection::protos::StringIntLabelMapItem& StringIntLabelMap::item(int index) const {
  // @@protoc_insertion_point(field_get:object_detection.protos.StringIntLabelMap.item)
  return _internal_item(index);
}
inline ::object_detection::protos::StringIntLabelMapItem* StringIntLabelMap::_internal_add_item() {
  return _impl_.item_.Add();
}
inline ::object_detection::protos::StringIntLabelMapItem* StringIntLabelMap::add_item() {
  ::object_detection::protos::StringIntLabelMapItem* _add = _internal_add_item();
  // @@protoc_insertion_point(field_add:object_detection.protos.StringIntLabelMap.item)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::object_detection::protos::StringIntLabelMapItem >&
StringIntLabelMap::item() const {
  // @@protoc_insertion_point(field_list:object_detection.protos.StringIntLabelMap.item)
  return _impl_.item_;
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------


// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace object_detection

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fstring_5fint_5flabel_5fmap_2eproto
