syntax = "proto2";

package object_detection.protos;

// Configuration proto for MeanStddevBoxCoder. See
// box_coders/mean_stddev_box_coder.py for details.
message MeanStddevBoxCoder {
  // The standard deviation used to encode and decode boxes.
  optional float stddev = 1 [default=0.01];
}
