// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: object_detection/protos/post_processing.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fpost_5fprocessing_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fpost_5fprocessing_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/generated_enum_reflection.h>
#include <google/protobuf/unknown_field_set.h>
#include "object_detection/protos/calibration.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_object_5fdetection_2fprotos_2fpost_5fprocessing_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_object_5fdetection_2fprotos_2fpost_5fprocessing_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_object_5fdetection_2fprotos_2fpost_5fprocessing_2eproto;
namespace object_detection {
namespace protos {
class BatchNonMaxSuppression;
struct BatchNonMaxSuppressionDefaultTypeInternal;
extern BatchNonMaxSuppressionDefaultTypeInternal _BatchNonMaxSuppression_default_instance_;
class PostProcessing;
struct PostProcessingDefaultTypeInternal;
extern PostProcessingDefaultTypeInternal _PostProcessing_default_instance_;
}  // namespace protos
}  // namespace object_detection
PROTOBUF_NAMESPACE_OPEN
template<> ::object_detection::protos::BatchNonMaxSuppression* Arena::CreateMaybeMessage<::object_detection::protos::BatchNonMaxSuppression>(Arena*);
template<> ::object_detection::protos::PostProcessing* Arena::CreateMaybeMessage<::object_detection::protos::PostProcessing>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace object_detection {
namespace protos {

enum PostProcessing_ScoreConverter : int {
  PostProcessing_ScoreConverter_IDENTITY = 0,
  PostProcessing_ScoreConverter_SIGMOID = 1,
  PostProcessing_ScoreConverter_SOFTMAX = 2
};
bool PostProcessing_ScoreConverter_IsValid(int value);
constexpr PostProcessing_ScoreConverter PostProcessing_ScoreConverter_ScoreConverter_MIN = PostProcessing_ScoreConverter_IDENTITY;
constexpr PostProcessing_ScoreConverter PostProcessing_ScoreConverter_ScoreConverter_MAX = PostProcessing_ScoreConverter_SOFTMAX;
constexpr int PostProcessing_ScoreConverter_ScoreConverter_ARRAYSIZE = PostProcessing_ScoreConverter_ScoreConverter_MAX + 1;

const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor* PostProcessing_ScoreConverter_descriptor();
template<typename T>
inline const std::string& PostProcessing_ScoreConverter_Name(T enum_t_value) {
  static_assert(::std::is_same<T, PostProcessing_ScoreConverter>::value ||
    ::std::is_integral<T>::value,
    "Incorrect type passed to function PostProcessing_ScoreConverter_Name.");
  return ::PROTOBUF_NAMESPACE_ID::internal::NameOfEnum(
    PostProcessing_ScoreConverter_descriptor(), enum_t_value);
}
inline bool PostProcessing_ScoreConverter_Parse(
    ::PROTOBUF_NAMESPACE_ID::ConstStringParam name, PostProcessing_ScoreConverter* value) {
  return ::PROTOBUF_NAMESPACE_ID::internal::ParseNamedEnum<PostProcessing_ScoreConverter>(
    PostProcessing_ScoreConverter_descriptor(), name, value);
}
// ===================================================================

class BatchNonMaxSuppression final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.BatchNonMaxSuppression) */ {
 public:
  inline BatchNonMaxSuppression() : BatchNonMaxSuppression(nullptr) {}
  ~BatchNonMaxSuppression() override;
  explicit PROTOBUF_CONSTEXPR BatchNonMaxSuppression(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  BatchNonMaxSuppression(const BatchNonMaxSuppression& from);
  BatchNonMaxSuppression(BatchNonMaxSuppression&& from) noexcept
    : BatchNonMaxSuppression() {
    *this = ::std::move(from);
  }

  inline BatchNonMaxSuppression& operator=(const BatchNonMaxSuppression& from) {
    CopyFrom(from);
    return *this;
  }
  inline BatchNonMaxSuppression& operator=(BatchNonMaxSuppression&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const BatchNonMaxSuppression& default_instance() {
    return *internal_default_instance();
  }
  static inline const BatchNonMaxSuppression* internal_default_instance() {
    return reinterpret_cast<const BatchNonMaxSuppression*>(
               &_BatchNonMaxSuppression_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(BatchNonMaxSuppression& a, BatchNonMaxSuppression& b) {
    a.Swap(&b);
  }
  inline void Swap(BatchNonMaxSuppression* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(BatchNonMaxSuppression* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  BatchNonMaxSuppression* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<BatchNonMaxSuppression>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const BatchNonMaxSuppression& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const BatchNonMaxSuppression& from) {
    BatchNonMaxSuppression::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(BatchNonMaxSuppression* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.BatchNonMaxSuppression";
  }
  protected:
  explicit BatchNonMaxSuppression(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kScoreThresholdFieldNumber = 1,
    kUseStaticShapesFieldNumber = 6,
    kUseClassAgnosticNmsFieldNumber = 7,
    kSoftNmsSigmaFieldNumber = 9,
    kIouThresholdFieldNumber = 2,
    kMaxDetectionsPerClassFieldNumber = 3,
    kMaxTotalDetectionsFieldNumber = 5,
    kMaxClassesPerDetectionFieldNumber = 8,
  };
  // optional float score_threshold = 1 [default = 0];
  bool has_score_threshold() const;
  private:
  bool _internal_has_score_threshold() const;
  public:
  void clear_score_threshold();
  float score_threshold() const;
  void set_score_threshold(float value);
  private:
  float _internal_score_threshold() const;
  void _internal_set_score_threshold(float value);
  public:

  // optional bool use_static_shapes = 6 [default = false];
  bool has_use_static_shapes() const;
  private:
  bool _internal_has_use_static_shapes() const;
  public:
  void clear_use_static_shapes();
  bool use_static_shapes() const;
  void set_use_static_shapes(bool value);
  private:
  bool _internal_use_static_shapes() const;
  void _internal_set_use_static_shapes(bool value);
  public:

  // optional bool use_class_agnostic_nms = 7 [default = false];
  bool has_use_class_agnostic_nms() const;
  private:
  bool _internal_has_use_class_agnostic_nms() const;
  public:
  void clear_use_class_agnostic_nms();
  bool use_class_agnostic_nms() const;
  void set_use_class_agnostic_nms(bool value);
  private:
  bool _internal_use_class_agnostic_nms() const;
  void _internal_set_use_class_agnostic_nms(bool value);
  public:

  // optional float soft_nms_sigma = 9 [default = 0];
  bool has_soft_nms_sigma() const;
  private:
  bool _internal_has_soft_nms_sigma() const;
  public:
  void clear_soft_nms_sigma();
  float soft_nms_sigma() const;
  void set_soft_nms_sigma(float value);
  private:
  float _internal_soft_nms_sigma() const;
  void _internal_set_soft_nms_sigma(float value);
  public:

  // optional float iou_threshold = 2 [default = 0.6];
  bool has_iou_threshold() const;
  private:
  bool _internal_has_iou_threshold() const;
  public:
  void clear_iou_threshold();
  float iou_threshold() const;
  void set_iou_threshold(float value);
  private:
  float _internal_iou_threshold() const;
  void _internal_set_iou_threshold(float value);
  public:

  // optional int32 max_detections_per_class = 3 [default = 100];
  bool has_max_detections_per_class() const;
  private:
  bool _internal_has_max_detections_per_class() const;
  public:
  void clear_max_detections_per_class();
  int32_t max_detections_per_class() const;
  void set_max_detections_per_class(int32_t value);
  private:
  int32_t _internal_max_detections_per_class() const;
  void _internal_set_max_detections_per_class(int32_t value);
  public:

  // optional int32 max_total_detections = 5 [default = 100];
  bool has_max_total_detections() const;
  private:
  bool _internal_has_max_total_detections() const;
  public:
  void clear_max_total_detections();
  int32_t max_total_detections() const;
  void set_max_total_detections(int32_t value);
  private:
  int32_t _internal_max_total_detections() const;
  void _internal_set_max_total_detections(int32_t value);
  public:

  // optional int32 max_classes_per_detection = 8 [default = 1];
  bool has_max_classes_per_detection() const;
  private:
  bool _internal_has_max_classes_per_detection() const;
  public:
  void clear_max_classes_per_detection();
  int32_t max_classes_per_detection() const;
  void set_max_classes_per_detection(int32_t value);
  private:
  int32_t _internal_max_classes_per_detection() const;
  void _internal_set_max_classes_per_detection(int32_t value);
  public:

  // @@protoc_insertion_point(class_scope:object_detection.protos.BatchNonMaxSuppression)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    float score_threshold_;
    bool use_static_shapes_;
    bool use_class_agnostic_nms_;
    float soft_nms_sigma_;
    float iou_threshold_;
    int32_t max_detections_per_class_;
    int32_t max_total_detections_;
    int32_t max_classes_per_detection_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fpost_5fprocessing_2eproto;
};
// -------------------------------------------------------------------

class PostProcessing final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:object_detection.protos.PostProcessing) */ {
 public:
  inline PostProcessing() : PostProcessing(nullptr) {}
  ~PostProcessing() override;
  explicit PROTOBUF_CONSTEXPR PostProcessing(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  PostProcessing(const PostProcessing& from);
  PostProcessing(PostProcessing&& from) noexcept
    : PostProcessing() {
    *this = ::std::move(from);
  }

  inline PostProcessing& operator=(const PostProcessing& from) {
    CopyFrom(from);
    return *this;
  }
  inline PostProcessing& operator=(PostProcessing&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PostProcessing& default_instance() {
    return *internal_default_instance();
  }
  static inline const PostProcessing* internal_default_instance() {
    return reinterpret_cast<const PostProcessing*>(
               &_PostProcessing_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(PostProcessing& a, PostProcessing& b) {
    a.Swap(&b);
  }
  inline void Swap(PostProcessing* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PostProcessing* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PostProcessing* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<PostProcessing>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const PostProcessing& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const PostProcessing& from) {
    PostProcessing::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(PostProcessing* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "object_detection.protos.PostProcessing";
  }
  protected:
  explicit PostProcessing(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  typedef PostProcessing_ScoreConverter ScoreConverter;
  static constexpr ScoreConverter IDENTITY =
    PostProcessing_ScoreConverter_IDENTITY;
  static constexpr ScoreConverter SIGMOID =
    PostProcessing_ScoreConverter_SIGMOID;
  static constexpr ScoreConverter SOFTMAX =
    PostProcessing_ScoreConverter_SOFTMAX;
  static inline bool ScoreConverter_IsValid(int value) {
    return PostProcessing_ScoreConverter_IsValid(value);
  }
  static constexpr ScoreConverter ScoreConverter_MIN =
    PostProcessing_ScoreConverter_ScoreConverter_MIN;
  static constexpr ScoreConverter ScoreConverter_MAX =
    PostProcessing_ScoreConverter_ScoreConverter_MAX;
  static constexpr int ScoreConverter_ARRAYSIZE =
    PostProcessing_ScoreConverter_ScoreConverter_ARRAYSIZE;
  static inline const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor*
  ScoreConverter_descriptor() {
    return PostProcessing_ScoreConverter_descriptor();
  }
  template<typename T>
  static inline const std::string& ScoreConverter_Name(T enum_t_value) {
    static_assert(::std::is_same<T, ScoreConverter>::value ||
      ::std::is_integral<T>::value,
      "Incorrect type passed to function ScoreConverter_Name.");
    return PostProcessing_ScoreConverter_Name(enum_t_value);
  }
  static inline bool ScoreConverter_Parse(::PROTOBUF_NAMESPACE_ID::ConstStringParam name,
      ScoreConverter* value) {
    return PostProcessing_ScoreConverter_Parse(name, value);
  }

  // accessors -------------------------------------------------------

  enum : int {
    kBatchNonMaxSuppressionFieldNumber = 1,
    kCalibrationConfigFieldNumber = 4,
    kScoreConverterFieldNumber = 2,
    kLogitScaleFieldNumber = 3,
  };
  // optional .object_detection.protos.BatchNonMaxSuppression batch_non_max_suppression = 1;
  bool has_batch_non_max_suppression() const;
  private:
  bool _internal_has_batch_non_max_suppression() const;
  public:
  void clear_batch_non_max_suppression();
  const ::object_detection::protos::BatchNonMaxSuppression& batch_non_max_suppression() const;
  PROTOBUF_NODISCARD ::object_detection::protos::BatchNonMaxSuppression* release_batch_non_max_suppression();
  ::object_detection::protos::BatchNonMaxSuppression* mutable_batch_non_max_suppression();
  void set_allocated_batch_non_max_suppression(::object_detection::protos::BatchNonMaxSuppression* batch_non_max_suppression);
  private:
  const ::object_detection::protos::BatchNonMaxSuppression& _internal_batch_non_max_suppression() const;
  ::object_detection::protos::BatchNonMaxSuppression* _internal_mutable_batch_non_max_suppression();
  public:
  void unsafe_arena_set_allocated_batch_non_max_suppression(
      ::object_detection::protos::BatchNonMaxSuppression* batch_non_max_suppression);
  ::object_detection::protos::BatchNonMaxSuppression* unsafe_arena_release_batch_non_max_suppression();

  // optional .object_detection.protos.CalibrationConfig calibration_config = 4;
  bool has_calibration_config() const;
  private:
  bool _internal_has_calibration_config() const;
  public:
  void clear_calibration_config();
  const ::object_detection::protos::CalibrationConfig& calibration_config() const;
  PROTOBUF_NODISCARD ::object_detection::protos::CalibrationConfig* release_calibration_config();
  ::object_detection::protos::CalibrationConfig* mutable_calibration_config();
  void set_allocated_calibration_config(::object_detection::protos::CalibrationConfig* calibration_config);
  private:
  const ::object_detection::protos::CalibrationConfig& _internal_calibration_config() const;
  ::object_detection::protos::CalibrationConfig* _internal_mutable_calibration_config();
  public:
  void unsafe_arena_set_allocated_calibration_config(
      ::object_detection::protos::CalibrationConfig* calibration_config);
  ::object_detection::protos::CalibrationConfig* unsafe_arena_release_calibration_config();

  // optional .object_detection.protos.PostProcessing.ScoreConverter score_converter = 2 [default = IDENTITY];
  bool has_score_converter() const;
  private:
  bool _internal_has_score_converter() const;
  public:
  void clear_score_converter();
  ::object_detection::protos::PostProcessing_ScoreConverter score_converter() const;
  void set_score_converter(::object_detection::protos::PostProcessing_ScoreConverter value);
  private:
  ::object_detection::protos::PostProcessing_ScoreConverter _internal_score_converter() const;
  void _internal_set_score_converter(::object_detection::protos::PostProcessing_ScoreConverter value);
  public:

  // optional float logit_scale = 3 [default = 1];
  bool has_logit_scale() const;
  private:
  bool _internal_has_logit_scale() const;
  public:
  void clear_logit_scale();
  float logit_scale() const;
  void set_logit_scale(float value);
  private:
  float _internal_logit_scale() const;
  void _internal_set_logit_scale(float value);
  public:

  // @@protoc_insertion_point(class_scope:object_detection.protos.PostProcessing)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    ::object_detection::protos::BatchNonMaxSuppression* batch_non_max_suppression_;
    ::object_detection::protos::CalibrationConfig* calibration_config_;
    int score_converter_;
    float logit_scale_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_object_5fdetection_2fprotos_2fpost_5fprocessing_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// BatchNonMaxSuppression

// optional float score_threshold = 1 [default = 0];
inline bool BatchNonMaxSuppression::_internal_has_score_threshold() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool BatchNonMaxSuppression::has_score_threshold() const {
  return _internal_has_score_threshold();
}
inline void BatchNonMaxSuppression::clear_score_threshold() {
  _impl_.score_threshold_ = 0;
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline float BatchNonMaxSuppression::_internal_score_threshold() const {
  return _impl_.score_threshold_;
}
inline float BatchNonMaxSuppression::score_threshold() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.BatchNonMaxSuppression.score_threshold)
  return _internal_score_threshold();
}
inline void BatchNonMaxSuppression::_internal_set_score_threshold(float value) {
  _impl_._has_bits_[0] |= 0x00000001u;
  _impl_.score_threshold_ = value;
}
inline void BatchNonMaxSuppression::set_score_threshold(float value) {
  _internal_set_score_threshold(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.BatchNonMaxSuppression.score_threshold)
}

// optional float iou_threshold = 2 [default = 0.6];
inline bool BatchNonMaxSuppression::_internal_has_iou_threshold() const {
  bool value = (_impl_._has_bits_[0] & 0x00000010u) != 0;
  return value;
}
inline bool BatchNonMaxSuppression::has_iou_threshold() const {
  return _internal_has_iou_threshold();
}
inline void BatchNonMaxSuppression::clear_iou_threshold() {
  _impl_.iou_threshold_ = 0.6f;
  _impl_._has_bits_[0] &= ~0x00000010u;
}
inline float BatchNonMaxSuppression::_internal_iou_threshold() const {
  return _impl_.iou_threshold_;
}
inline float BatchNonMaxSuppression::iou_threshold() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.BatchNonMaxSuppression.iou_threshold)
  return _internal_iou_threshold();
}
inline void BatchNonMaxSuppression::_internal_set_iou_threshold(float value) {
  _impl_._has_bits_[0] |= 0x00000010u;
  _impl_.iou_threshold_ = value;
}
inline void BatchNonMaxSuppression::set_iou_threshold(float value) {
  _internal_set_iou_threshold(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.BatchNonMaxSuppression.iou_threshold)
}

// optional int32 max_detections_per_class = 3 [default = 100];
inline bool BatchNonMaxSuppression::_internal_has_max_detections_per_class() const {
  bool value = (_impl_._has_bits_[0] & 0x00000020u) != 0;
  return value;
}
inline bool BatchNonMaxSuppression::has_max_detections_per_class() const {
  return _internal_has_max_detections_per_class();
}
inline void BatchNonMaxSuppression::clear_max_detections_per_class() {
  _impl_.max_detections_per_class_ = 100;
  _impl_._has_bits_[0] &= ~0x00000020u;
}
inline int32_t BatchNonMaxSuppression::_internal_max_detections_per_class() const {
  return _impl_.max_detections_per_class_;
}
inline int32_t BatchNonMaxSuppression::max_detections_per_class() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.BatchNonMaxSuppression.max_detections_per_class)
  return _internal_max_detections_per_class();
}
inline void BatchNonMaxSuppression::_internal_set_max_detections_per_class(int32_t value) {
  _impl_._has_bits_[0] |= 0x00000020u;
  _impl_.max_detections_per_class_ = value;
}
inline void BatchNonMaxSuppression::set_max_detections_per_class(int32_t value) {
  _internal_set_max_detections_per_class(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.BatchNonMaxSuppression.max_detections_per_class)
}

// optional int32 max_total_detections = 5 [default = 100];
inline bool BatchNonMaxSuppression::_internal_has_max_total_detections() const {
  bool value = (_impl_._has_bits_[0] & 0x00000040u) != 0;
  return value;
}
inline bool BatchNonMaxSuppression::has_max_total_detections() const {
  return _internal_has_max_total_detections();
}
inline void BatchNonMaxSuppression::clear_max_total_detections() {
  _impl_.max_total_detections_ = 100;
  _impl_._has_bits_[0] &= ~0x00000040u;
}
inline int32_t BatchNonMaxSuppression::_internal_max_total_detections() const {
  return _impl_.max_total_detections_;
}
inline int32_t BatchNonMaxSuppression::max_total_detections() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.BatchNonMaxSuppression.max_total_detections)
  return _internal_max_total_detections();
}
inline void BatchNonMaxSuppression::_internal_set_max_total_detections(int32_t value) {
  _impl_._has_bits_[0] |= 0x00000040u;
  _impl_.max_total_detections_ = value;
}
inline void BatchNonMaxSuppression::set_max_total_detections(int32_t value) {
  _internal_set_max_total_detections(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.BatchNonMaxSuppression.max_total_detections)
}

// optional bool use_static_shapes = 6 [default = false];
inline bool BatchNonMaxSuppression::_internal_has_use_static_shapes() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool BatchNonMaxSuppression::has_use_static_shapes() const {
  return _internal_has_use_static_shapes();
}
inline void BatchNonMaxSuppression::clear_use_static_shapes() {
  _impl_.use_static_shapes_ = false;
  _impl_._has_bits_[0] &= ~0x00000002u;
}
inline bool BatchNonMaxSuppression::_internal_use_static_shapes() const {
  return _impl_.use_static_shapes_;
}
inline bool BatchNonMaxSuppression::use_static_shapes() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.BatchNonMaxSuppression.use_static_shapes)
  return _internal_use_static_shapes();
}
inline void BatchNonMaxSuppression::_internal_set_use_static_shapes(bool value) {
  _impl_._has_bits_[0] |= 0x00000002u;
  _impl_.use_static_shapes_ = value;
}
inline void BatchNonMaxSuppression::set_use_static_shapes(bool value) {
  _internal_set_use_static_shapes(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.BatchNonMaxSuppression.use_static_shapes)
}

// optional bool use_class_agnostic_nms = 7 [default = false];
inline bool BatchNonMaxSuppression::_internal_has_use_class_agnostic_nms() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool BatchNonMaxSuppression::has_use_class_agnostic_nms() const {
  return _internal_has_use_class_agnostic_nms();
}
inline void BatchNonMaxSuppression::clear_use_class_agnostic_nms() {
  _impl_.use_class_agnostic_nms_ = false;
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline bool BatchNonMaxSuppression::_internal_use_class_agnostic_nms() const {
  return _impl_.use_class_agnostic_nms_;
}
inline bool BatchNonMaxSuppression::use_class_agnostic_nms() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.BatchNonMaxSuppression.use_class_agnostic_nms)
  return _internal_use_class_agnostic_nms();
}
inline void BatchNonMaxSuppression::_internal_set_use_class_agnostic_nms(bool value) {
  _impl_._has_bits_[0] |= 0x00000004u;
  _impl_.use_class_agnostic_nms_ = value;
}
inline void BatchNonMaxSuppression::set_use_class_agnostic_nms(bool value) {
  _internal_set_use_class_agnostic_nms(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.BatchNonMaxSuppression.use_class_agnostic_nms)
}

// optional int32 max_classes_per_detection = 8 [default = 1];
inline bool BatchNonMaxSuppression::_internal_has_max_classes_per_detection() const {
  bool value = (_impl_._has_bits_[0] & 0x00000080u) != 0;
  return value;
}
inline bool BatchNonMaxSuppression::has_max_classes_per_detection() const {
  return _internal_has_max_classes_per_detection();
}
inline void BatchNonMaxSuppression::clear_max_classes_per_detection() {
  _impl_.max_classes_per_detection_ = 1;
  _impl_._has_bits_[0] &= ~0x00000080u;
}
inline int32_t BatchNonMaxSuppression::_internal_max_classes_per_detection() const {
  return _impl_.max_classes_per_detection_;
}
inline int32_t BatchNonMaxSuppression::max_classes_per_detection() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.BatchNonMaxSuppression.max_classes_per_detection)
  return _internal_max_classes_per_detection();
}
inline void BatchNonMaxSuppression::_internal_set_max_classes_per_detection(int32_t value) {
  _impl_._has_bits_[0] |= 0x00000080u;
  _impl_.max_classes_per_detection_ = value;
}
inline void BatchNonMaxSuppression::set_max_classes_per_detection(int32_t value) {
  _internal_set_max_classes_per_detection(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.BatchNonMaxSuppression.max_classes_per_detection)
}

// optional float soft_nms_sigma = 9 [default = 0];
inline bool BatchNonMaxSuppression::_internal_has_soft_nms_sigma() const {
  bool value = (_impl_._has_bits_[0] & 0x00000008u) != 0;
  return value;
}
inline bool BatchNonMaxSuppression::has_soft_nms_sigma() const {
  return _internal_has_soft_nms_sigma();
}
inline void BatchNonMaxSuppression::clear_soft_nms_sigma() {
  _impl_.soft_nms_sigma_ = 0;
  _impl_._has_bits_[0] &= ~0x00000008u;
}
inline float BatchNonMaxSuppression::_internal_soft_nms_sigma() const {
  return _impl_.soft_nms_sigma_;
}
inline float BatchNonMaxSuppression::soft_nms_sigma() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.BatchNonMaxSuppression.soft_nms_sigma)
  return _internal_soft_nms_sigma();
}
inline void BatchNonMaxSuppression::_internal_set_soft_nms_sigma(float value) {
  _impl_._has_bits_[0] |= 0x00000008u;
  _impl_.soft_nms_sigma_ = value;
}
inline void BatchNonMaxSuppression::set_soft_nms_sigma(float value) {
  _internal_set_soft_nms_sigma(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.BatchNonMaxSuppression.soft_nms_sigma)
}

// -------------------------------------------------------------------

// PostProcessing

// optional .object_detection.protos.BatchNonMaxSuppression batch_non_max_suppression = 1;
inline bool PostProcessing::_internal_has_batch_non_max_suppression() const {
  bool value = (_impl_._has_bits_[0] & 0x00000001u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.batch_non_max_suppression_ != nullptr);
  return value;
}
inline bool PostProcessing::has_batch_non_max_suppression() const {
  return _internal_has_batch_non_max_suppression();
}
inline void PostProcessing::clear_batch_non_max_suppression() {
  if (_impl_.batch_non_max_suppression_ != nullptr) _impl_.batch_non_max_suppression_->Clear();
  _impl_._has_bits_[0] &= ~0x00000001u;
}
inline const ::object_detection::protos::BatchNonMaxSuppression& PostProcessing::_internal_batch_non_max_suppression() const {
  const ::object_detection::protos::BatchNonMaxSuppression* p = _impl_.batch_non_max_suppression_;
  return p != nullptr ? *p : reinterpret_cast<const ::object_detection::protos::BatchNonMaxSuppression&>(
      ::object_detection::protos::_BatchNonMaxSuppression_default_instance_);
}
inline const ::object_detection::protos::BatchNonMaxSuppression& PostProcessing::batch_non_max_suppression() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.PostProcessing.batch_non_max_suppression)
  return _internal_batch_non_max_suppression();
}
inline void PostProcessing::unsafe_arena_set_allocated_batch_non_max_suppression(
    ::object_detection::protos::BatchNonMaxSuppression* batch_non_max_suppression) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.batch_non_max_suppression_);
  }
  _impl_.batch_non_max_suppression_ = batch_non_max_suppression;
  if (batch_non_max_suppression) {
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.PostProcessing.batch_non_max_suppression)
}
inline ::object_detection::protos::BatchNonMaxSuppression* PostProcessing::release_batch_non_max_suppression() {
  _impl_._has_bits_[0] &= ~0x00000001u;
  ::object_detection::protos::BatchNonMaxSuppression* temp = _impl_.batch_non_max_suppression_;
  _impl_.batch_non_max_suppression_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::object_detection::protos::BatchNonMaxSuppression* PostProcessing::unsafe_arena_release_batch_non_max_suppression() {
  // @@protoc_insertion_point(field_release:object_detection.protos.PostProcessing.batch_non_max_suppression)
  _impl_._has_bits_[0] &= ~0x00000001u;
  ::object_detection::protos::BatchNonMaxSuppression* temp = _impl_.batch_non_max_suppression_;
  _impl_.batch_non_max_suppression_ = nullptr;
  return temp;
}
inline ::object_detection::protos::BatchNonMaxSuppression* PostProcessing::_internal_mutable_batch_non_max_suppression() {
  _impl_._has_bits_[0] |= 0x00000001u;
  if (_impl_.batch_non_max_suppression_ == nullptr) {
    auto* p = CreateMaybeMessage<::object_detection::protos::BatchNonMaxSuppression>(GetArenaForAllocation());
    _impl_.batch_non_max_suppression_ = p;
  }
  return _impl_.batch_non_max_suppression_;
}
inline ::object_detection::protos::BatchNonMaxSuppression* PostProcessing::mutable_batch_non_max_suppression() {
  ::object_detection::protos::BatchNonMaxSuppression* _msg = _internal_mutable_batch_non_max_suppression();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.PostProcessing.batch_non_max_suppression)
  return _msg;
}
inline void PostProcessing::set_allocated_batch_non_max_suppression(::object_detection::protos::BatchNonMaxSuppression* batch_non_max_suppression) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete _impl_.batch_non_max_suppression_;
  }
  if (batch_non_max_suppression) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(batch_non_max_suppression);
    if (message_arena != submessage_arena) {
      batch_non_max_suppression = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, batch_non_max_suppression, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000001u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000001u;
  }
  _impl_.batch_non_max_suppression_ = batch_non_max_suppression;
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.PostProcessing.batch_non_max_suppression)
}

// optional .object_detection.protos.PostProcessing.ScoreConverter score_converter = 2 [default = IDENTITY];
inline bool PostProcessing::_internal_has_score_converter() const {
  bool value = (_impl_._has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool PostProcessing::has_score_converter() const {
  return _internal_has_score_converter();
}
inline void PostProcessing::clear_score_converter() {
  _impl_.score_converter_ = 0;
  _impl_._has_bits_[0] &= ~0x00000004u;
}
inline ::object_detection::protos::PostProcessing_ScoreConverter PostProcessing::_internal_score_converter() const {
  return static_cast< ::object_detection::protos::PostProcessing_ScoreConverter >(_impl_.score_converter_);
}
inline ::object_detection::protos::PostProcessing_ScoreConverter PostProcessing::score_converter() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.PostProcessing.score_converter)
  return _internal_score_converter();
}
inline void PostProcessing::_internal_set_score_converter(::object_detection::protos::PostProcessing_ScoreConverter value) {
  assert(::object_detection::protos::PostProcessing_ScoreConverter_IsValid(value));
  _impl_._has_bits_[0] |= 0x00000004u;
  _impl_.score_converter_ = value;
}
inline void PostProcessing::set_score_converter(::object_detection::protos::PostProcessing_ScoreConverter value) {
  _internal_set_score_converter(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.PostProcessing.score_converter)
}

// optional float logit_scale = 3 [default = 1];
inline bool PostProcessing::_internal_has_logit_scale() const {
  bool value = (_impl_._has_bits_[0] & 0x00000008u) != 0;
  return value;
}
inline bool PostProcessing::has_logit_scale() const {
  return _internal_has_logit_scale();
}
inline void PostProcessing::clear_logit_scale() {
  _impl_.logit_scale_ = 1;
  _impl_._has_bits_[0] &= ~0x00000008u;
}
inline float PostProcessing::_internal_logit_scale() const {
  return _impl_.logit_scale_;
}
inline float PostProcessing::logit_scale() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.PostProcessing.logit_scale)
  return _internal_logit_scale();
}
inline void PostProcessing::_internal_set_logit_scale(float value) {
  _impl_._has_bits_[0] |= 0x00000008u;
  _impl_.logit_scale_ = value;
}
inline void PostProcessing::set_logit_scale(float value) {
  _internal_set_logit_scale(value);
  // @@protoc_insertion_point(field_set:object_detection.protos.PostProcessing.logit_scale)
}

// optional .object_detection.protos.CalibrationConfig calibration_config = 4;
inline bool PostProcessing::_internal_has_calibration_config() const {
  bool value = (_impl_._has_bits_[0] & 0x00000002u) != 0;
  PROTOBUF_ASSUME(!value || _impl_.calibration_config_ != nullptr);
  return value;
}
inline bool PostProcessing::has_calibration_config() const {
  return _internal_has_calibration_config();
}
inline const ::object_detection::protos::CalibrationConfig& PostProcessing::_internal_calibration_config() const {
  const ::object_detection::protos::CalibrationConfig* p = _impl_.calibration_config_;
  return p != nullptr ? *p : reinterpret_cast<const ::object_detection::protos::CalibrationConfig&>(
      ::object_detection::protos::_CalibrationConfig_default_instance_);
}
inline const ::object_detection::protos::CalibrationConfig& PostProcessing::calibration_config() const {
  // @@protoc_insertion_point(field_get:object_detection.protos.PostProcessing.calibration_config)
  return _internal_calibration_config();
}
inline void PostProcessing::unsafe_arena_set_allocated_calibration_config(
    ::object_detection::protos::CalibrationConfig* calibration_config) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.calibration_config_);
  }
  _impl_.calibration_config_ = calibration_config;
  if (calibration_config) {
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:object_detection.protos.PostProcessing.calibration_config)
}
inline ::object_detection::protos::CalibrationConfig* PostProcessing::release_calibration_config() {
  _impl_._has_bits_[0] &= ~0x00000002u;
  ::object_detection::protos::CalibrationConfig* temp = _impl_.calibration_config_;
  _impl_.calibration_config_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::object_detection::protos::CalibrationConfig* PostProcessing::unsafe_arena_release_calibration_config() {
  // @@protoc_insertion_point(field_release:object_detection.protos.PostProcessing.calibration_config)
  _impl_._has_bits_[0] &= ~0x00000002u;
  ::object_detection::protos::CalibrationConfig* temp = _impl_.calibration_config_;
  _impl_.calibration_config_ = nullptr;
  return temp;
}
inline ::object_detection::protos::CalibrationConfig* PostProcessing::_internal_mutable_calibration_config() {
  _impl_._has_bits_[0] |= 0x00000002u;
  if (_impl_.calibration_config_ == nullptr) {
    auto* p = CreateMaybeMessage<::object_detection::protos::CalibrationConfig>(GetArenaForAllocation());
    _impl_.calibration_config_ = p;
  }
  return _impl_.calibration_config_;
}
inline ::object_detection::protos::CalibrationConfig* PostProcessing::mutable_calibration_config() {
  ::object_detection::protos::CalibrationConfig* _msg = _internal_mutable_calibration_config();
  // @@protoc_insertion_point(field_mutable:object_detection.protos.PostProcessing.calibration_config)
  return _msg;
}
inline void PostProcessing::set_allocated_calibration_config(::object_detection::protos::CalibrationConfig* calibration_config) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.calibration_config_);
  }
  if (calibration_config) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(calibration_config));
    if (message_arena != submessage_arena) {
      calibration_config = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, calibration_config, submessage_arena);
    }
    _impl_._has_bits_[0] |= 0x00000002u;
  } else {
    _impl_._has_bits_[0] &= ~0x00000002u;
  }
  _impl_.calibration_config_ = calibration_config;
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.PostProcessing.calibration_config)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------


// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace object_detection

PROTOBUF_NAMESPACE_OPEN

template <> struct is_proto_enum< ::object_detection::protos::PostProcessing_ScoreConverter> : ::std::true_type {};
template <>
inline const EnumDescriptor* GetEnumDescriptor< ::object_detection::protos::PostProcessing_ScoreConverter>() {
  return ::object_detection::protos::PostProcessing_ScoreConverter_descriptor();
}

PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_object_5fdetection_2fprotos_2fpost_5fprocessing_2eproto
