// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/train.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2ftrain_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2ftrain_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3021000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3021005 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/unknown_field_set.h>
#include "second/protos/optimizer.pb.h"
#include "second/protos/preprocess.pb.h"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_second_2fprotos_2ftrain_2eproto
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct TableStruct_second_2fprotos_2ftrain_2eproto {
  static const uint32_t offsets[];
};
extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_second_2fprotos_2ftrain_2eproto;
namespace second {
namespace protos {
class L1Regularizer;
struct L1RegularizerDefaultTypeInternal;
extern L1RegularizerDefaultTypeInternal _L1Regularizer_default_instance_;
class L2Regularizer;
struct L2RegularizerDefaultTypeInternal;
extern L2RegularizerDefaultTypeInternal _L2Regularizer_default_instance_;
class Regularizer;
struct RegularizerDefaultTypeInternal;
extern RegularizerDefaultTypeInternal _Regularizer_default_instance_;
class TrainConfig;
struct TrainConfigDefaultTypeInternal;
extern TrainConfigDefaultTypeInternal _TrainConfig_default_instance_;
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> ::second::protos::L1Regularizer* Arena::CreateMaybeMessage<::second::protos::L1Regularizer>(Arena*);
template<> ::second::protos::L2Regularizer* Arena::CreateMaybeMessage<::second::protos::L2Regularizer>(Arena*);
template<> ::second::protos::Regularizer* Arena::CreateMaybeMessage<::second::protos::Regularizer>(Arena*);
template<> ::second::protos::TrainConfig* Arena::CreateMaybeMessage<::second::protos::TrainConfig>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace second {
namespace protos {

// ===================================================================

class Regularizer final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.Regularizer) */ {
 public:
  inline Regularizer() : Regularizer(nullptr) {}
  ~Regularizer() override;
  explicit PROTOBUF_CONSTEXPR Regularizer(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Regularizer(const Regularizer& from);
  Regularizer(Regularizer&& from) noexcept
    : Regularizer() {
    *this = ::std::move(from);
  }

  inline Regularizer& operator=(const Regularizer& from) {
    CopyFrom(from);
    return *this;
  }
  inline Regularizer& operator=(Regularizer&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Regularizer& default_instance() {
    return *internal_default_instance();
  }
  enum RegularizerOneofCase {
    kL1Regularizer = 1,
    kL2Regularizer = 2,
    REGULARIZER_ONEOF_NOT_SET = 0,
  };

  static inline const Regularizer* internal_default_instance() {
    return reinterpret_cast<const Regularizer*>(
               &_Regularizer_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(Regularizer& a, Regularizer& b) {
    a.Swap(&b);
  }
  inline void Swap(Regularizer* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Regularizer* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Regularizer* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<Regularizer>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const Regularizer& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const Regularizer& from) {
    Regularizer::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Regularizer* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.Regularizer";
  }
  protected:
  explicit Regularizer(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kL1RegularizerFieldNumber = 1,
    kL2RegularizerFieldNumber = 2,
  };
  // .second.protos.L1Regularizer l1_regularizer = 1;
  bool has_l1_regularizer() const;
  private:
  bool _internal_has_l1_regularizer() const;
  public:
  void clear_l1_regularizer();
  const ::second::protos::L1Regularizer& l1_regularizer() const;
  PROTOBUF_NODISCARD ::second::protos::L1Regularizer* release_l1_regularizer();
  ::second::protos::L1Regularizer* mutable_l1_regularizer();
  void set_allocated_l1_regularizer(::second::protos::L1Regularizer* l1_regularizer);
  private:
  const ::second::protos::L1Regularizer& _internal_l1_regularizer() const;
  ::second::protos::L1Regularizer* _internal_mutable_l1_regularizer();
  public:
  void unsafe_arena_set_allocated_l1_regularizer(
      ::second::protos::L1Regularizer* l1_regularizer);
  ::second::protos::L1Regularizer* unsafe_arena_release_l1_regularizer();

  // .second.protos.L2Regularizer l2_regularizer = 2;
  bool has_l2_regularizer() const;
  private:
  bool _internal_has_l2_regularizer() const;
  public:
  void clear_l2_regularizer();
  const ::second::protos::L2Regularizer& l2_regularizer() const;
  PROTOBUF_NODISCARD ::second::protos::L2Regularizer* release_l2_regularizer();
  ::second::protos::L2Regularizer* mutable_l2_regularizer();
  void set_allocated_l2_regularizer(::second::protos::L2Regularizer* l2_regularizer);
  private:
  const ::second::protos::L2Regularizer& _internal_l2_regularizer() const;
  ::second::protos::L2Regularizer* _internal_mutable_l2_regularizer();
  public:
  void unsafe_arena_set_allocated_l2_regularizer(
      ::second::protos::L2Regularizer* l2_regularizer);
  ::second::protos::L2Regularizer* unsafe_arena_release_l2_regularizer();

  void clear_regularizer_oneof();
  RegularizerOneofCase regularizer_oneof_case() const;
  // @@protoc_insertion_point(class_scope:second.protos.Regularizer)
 private:
  class _Internal;
  void set_has_l1_regularizer();
  void set_has_l2_regularizer();

  inline bool has_regularizer_oneof() const;
  inline void clear_has_regularizer_oneof();

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    union RegularizerOneofUnion {
      constexpr RegularizerOneofUnion() : _constinit_{} {}
        ::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized _constinit_;
      ::second::protos::L1Regularizer* l1_regularizer_;
      ::second::protos::L2Regularizer* l2_regularizer_;
    } regularizer_oneof_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
    uint32_t _oneof_case_[1];

  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2ftrain_2eproto;
};
// -------------------------------------------------------------------

class L1Regularizer final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.L1Regularizer) */ {
 public:
  inline L1Regularizer() : L1Regularizer(nullptr) {}
  ~L1Regularizer() override;
  explicit PROTOBUF_CONSTEXPR L1Regularizer(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  L1Regularizer(const L1Regularizer& from);
  L1Regularizer(L1Regularizer&& from) noexcept
    : L1Regularizer() {
    *this = ::std::move(from);
  }

  inline L1Regularizer& operator=(const L1Regularizer& from) {
    CopyFrom(from);
    return *this;
  }
  inline L1Regularizer& operator=(L1Regularizer&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const L1Regularizer& default_instance() {
    return *internal_default_instance();
  }
  static inline const L1Regularizer* internal_default_instance() {
    return reinterpret_cast<const L1Regularizer*>(
               &_L1Regularizer_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(L1Regularizer& a, L1Regularizer& b) {
    a.Swap(&b);
  }
  inline void Swap(L1Regularizer* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(L1Regularizer* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  L1Regularizer* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<L1Regularizer>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const L1Regularizer& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const L1Regularizer& from) {
    L1Regularizer::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(L1Regularizer* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.L1Regularizer";
  }
  protected:
  explicit L1Regularizer(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kWeightFieldNumber = 1,
  };
  // float weight = 1;
  void clear_weight();
  float weight() const;
  void set_weight(float value);
  private:
  float _internal_weight() const;
  void _internal_set_weight(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.L1Regularizer)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    float weight_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2ftrain_2eproto;
};
// -------------------------------------------------------------------

class L2Regularizer final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.L2Regularizer) */ {
 public:
  inline L2Regularizer() : L2Regularizer(nullptr) {}
  ~L2Regularizer() override;
  explicit PROTOBUF_CONSTEXPR L2Regularizer(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  L2Regularizer(const L2Regularizer& from);
  L2Regularizer(L2Regularizer&& from) noexcept
    : L2Regularizer() {
    *this = ::std::move(from);
  }

  inline L2Regularizer& operator=(const L2Regularizer& from) {
    CopyFrom(from);
    return *this;
  }
  inline L2Regularizer& operator=(L2Regularizer&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const L2Regularizer& default_instance() {
    return *internal_default_instance();
  }
  static inline const L2Regularizer* internal_default_instance() {
    return reinterpret_cast<const L2Regularizer*>(
               &_L2Regularizer_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    2;

  friend void swap(L2Regularizer& a, L2Regularizer& b) {
    a.Swap(&b);
  }
  inline void Swap(L2Regularizer* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(L2Regularizer* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  L2Regularizer* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<L2Regularizer>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const L2Regularizer& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const L2Regularizer& from) {
    L2Regularizer::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(L2Regularizer* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.L2Regularizer";
  }
  protected:
  explicit L2Regularizer(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kWeightFieldNumber = 1,
  };
  // float weight = 1;
  void clear_weight();
  float weight() const;
  void set_weight(float value);
  private:
  float _internal_weight() const;
  void _internal_set_weight(float value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.L2Regularizer)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    float weight_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2ftrain_2eproto;
};
// -------------------------------------------------------------------

class TrainConfig final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:second.protos.TrainConfig) */ {
 public:
  inline TrainConfig() : TrainConfig(nullptr) {}
  ~TrainConfig() override;
  explicit PROTOBUF_CONSTEXPR TrainConfig(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  TrainConfig(const TrainConfig& from);
  TrainConfig(TrainConfig&& from) noexcept
    : TrainConfig() {
    *this = ::std::move(from);
  }

  inline TrainConfig& operator=(const TrainConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline TrainConfig& operator=(TrainConfig&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const TrainConfig& default_instance() {
    return *internal_default_instance();
  }
  static inline const TrainConfig* internal_default_instance() {
    return reinterpret_cast<const TrainConfig*>(
               &_TrainConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    3;

  friend void swap(TrainConfig& a, TrainConfig& b) {
    a.Swap(&b);
  }
  inline void Swap(TrainConfig* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TrainConfig* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  TrainConfig* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<TrainConfig>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const TrainConfig& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom( const TrainConfig& from) {
    TrainConfig::MergeImpl(*this, from);
  }
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::PROTOBUF_NAMESPACE_ID::Arena* arena, bool is_message_owned);
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(TrainConfig* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "second.protos.TrainConfig";
  }
  protected:
  explicit TrainConfig(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kOptimizerFieldNumber = 1,
    kInterOpParallelismThreadsFieldNumber = 3,
    kIntraOpParallelismThreadsFieldNumber = 4,
    kStepsFieldNumber = 5,
    kStepsPerEvalFieldNumber = 6,
    kSaveCheckpointsSecsFieldNumber = 7,
    kSaveSummaryStepsFieldNumber = 8,
    kLossScaleFactorFieldNumber = 10,
    kEnableMixedPrecisionFieldNumber = 9,
    kClearMetricsEveryEpochFieldNumber = 11,
  };
  // .second.protos.Optimizer optimizer = 1;
  bool has_optimizer() const;
  private:
  bool _internal_has_optimizer() const;
  public:
  void clear_optimizer();
  const ::second::protos::Optimizer& optimizer() const;
  PROTOBUF_NODISCARD ::second::protos::Optimizer* release_optimizer();
  ::second::protos::Optimizer* mutable_optimizer();
  void set_allocated_optimizer(::second::protos::Optimizer* optimizer);
  private:
  const ::second::protos::Optimizer& _internal_optimizer() const;
  ::second::protos::Optimizer* _internal_mutable_optimizer();
  public:
  void unsafe_arena_set_allocated_optimizer(
      ::second::protos::Optimizer* optimizer);
  ::second::protos::Optimizer* unsafe_arena_release_optimizer();

  // uint32 inter_op_parallelism_threads = 3;
  void clear_inter_op_parallelism_threads();
  uint32_t inter_op_parallelism_threads() const;
  void set_inter_op_parallelism_threads(uint32_t value);
  private:
  uint32_t _internal_inter_op_parallelism_threads() const;
  void _internal_set_inter_op_parallelism_threads(uint32_t value);
  public:

  // uint32 intra_op_parallelism_threads = 4;
  void clear_intra_op_parallelism_threads();
  uint32_t intra_op_parallelism_threads() const;
  void set_intra_op_parallelism_threads(uint32_t value);
  private:
  uint32_t _internal_intra_op_parallelism_threads() const;
  void _internal_set_intra_op_parallelism_threads(uint32_t value);
  public:

  // uint32 steps = 5;
  void clear_steps();
  uint32_t steps() const;
  void set_steps(uint32_t value);
  private:
  uint32_t _internal_steps() const;
  void _internal_set_steps(uint32_t value);
  public:

  // uint32 steps_per_eval = 6;
  void clear_steps_per_eval();
  uint32_t steps_per_eval() const;
  void set_steps_per_eval(uint32_t value);
  private:
  uint32_t _internal_steps_per_eval() const;
  void _internal_set_steps_per_eval(uint32_t value);
  public:

  // uint32 save_checkpoints_secs = 7;
  void clear_save_checkpoints_secs();
  uint32_t save_checkpoints_secs() const;
  void set_save_checkpoints_secs(uint32_t value);
  private:
  uint32_t _internal_save_checkpoints_secs() const;
  void _internal_set_save_checkpoints_secs(uint32_t value);
  public:

  // uint32 save_summary_steps = 8;
  void clear_save_summary_steps();
  uint32_t save_summary_steps() const;
  void set_save_summary_steps(uint32_t value);
  private:
  uint32_t _internal_save_summary_steps() const;
  void _internal_set_save_summary_steps(uint32_t value);
  public:

  // float loss_scale_factor = 10;
  void clear_loss_scale_factor();
  float loss_scale_factor() const;
  void set_loss_scale_factor(float value);
  private:
  float _internal_loss_scale_factor() const;
  void _internal_set_loss_scale_factor(float value);
  public:

  // bool enable_mixed_precision = 9;
  void clear_enable_mixed_precision();
  bool enable_mixed_precision() const;
  void set_enable_mixed_precision(bool value);
  private:
  bool _internal_enable_mixed_precision() const;
  void _internal_set_enable_mixed_precision(bool value);
  public:

  // bool clear_metrics_every_epoch = 11;
  void clear_clear_metrics_every_epoch();
  bool clear_metrics_every_epoch() const;
  void set_clear_metrics_every_epoch(bool value);
  private:
  bool _internal_clear_metrics_every_epoch() const;
  void _internal_set_clear_metrics_every_epoch(bool value);
  public:

  // @@protoc_insertion_point(class_scope:second.protos.TrainConfig)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  struct Impl_ {
    ::second::protos::Optimizer* optimizer_;
    uint32_t inter_op_parallelism_threads_;
    uint32_t intra_op_parallelism_threads_;
    uint32_t steps_;
    uint32_t steps_per_eval_;
    uint32_t save_checkpoints_secs_;
    uint32_t save_summary_steps_;
    float loss_scale_factor_;
    bool enable_mixed_precision_;
    bool clear_metrics_every_epoch_;
    mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_second_2fprotos_2ftrain_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// Regularizer

// .second.protos.L1Regularizer l1_regularizer = 1;
inline bool Regularizer::_internal_has_l1_regularizer() const {
  return regularizer_oneof_case() == kL1Regularizer;
}
inline bool Regularizer::has_l1_regularizer() const {
  return _internal_has_l1_regularizer();
}
inline void Regularizer::set_has_l1_regularizer() {
  _impl_._oneof_case_[0] = kL1Regularizer;
}
inline void Regularizer::clear_l1_regularizer() {
  if (_internal_has_l1_regularizer()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.regularizer_oneof_.l1_regularizer_;
    }
    clear_has_regularizer_oneof();
  }
}
inline ::second::protos::L1Regularizer* Regularizer::release_l1_regularizer() {
  // @@protoc_insertion_point(field_release:second.protos.Regularizer.l1_regularizer)
  if (_internal_has_l1_regularizer()) {
    clear_has_regularizer_oneof();
    ::second::protos::L1Regularizer* temp = _impl_.regularizer_oneof_.l1_regularizer_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.regularizer_oneof_.l1_regularizer_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::second::protos::L1Regularizer& Regularizer::_internal_l1_regularizer() const {
  return _internal_has_l1_regularizer()
      ? *_impl_.regularizer_oneof_.l1_regularizer_
      : reinterpret_cast< ::second::protos::L1Regularizer&>(::second::protos::_L1Regularizer_default_instance_);
}
inline const ::second::protos::L1Regularizer& Regularizer::l1_regularizer() const {
  // @@protoc_insertion_point(field_get:second.protos.Regularizer.l1_regularizer)
  return _internal_l1_regularizer();
}
inline ::second::protos::L1Regularizer* Regularizer::unsafe_arena_release_l1_regularizer() {
  // @@protoc_insertion_point(field_unsafe_arena_release:second.protos.Regularizer.l1_regularizer)
  if (_internal_has_l1_regularizer()) {
    clear_has_regularizer_oneof();
    ::second::protos::L1Regularizer* temp = _impl_.regularizer_oneof_.l1_regularizer_;
    _impl_.regularizer_oneof_.l1_regularizer_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void Regularizer::unsafe_arena_set_allocated_l1_regularizer(::second::protos::L1Regularizer* l1_regularizer) {
  clear_regularizer_oneof();
  if (l1_regularizer) {
    set_has_l1_regularizer();
    _impl_.regularizer_oneof_.l1_regularizer_ = l1_regularizer;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.Regularizer.l1_regularizer)
}
inline ::second::protos::L1Regularizer* Regularizer::_internal_mutable_l1_regularizer() {
  if (!_internal_has_l1_regularizer()) {
    clear_regularizer_oneof();
    set_has_l1_regularizer();
    _impl_.regularizer_oneof_.l1_regularizer_ = CreateMaybeMessage< ::second::protos::L1Regularizer >(GetArenaForAllocation());
  }
  return _impl_.regularizer_oneof_.l1_regularizer_;
}
inline ::second::protos::L1Regularizer* Regularizer::mutable_l1_regularizer() {
  ::second::protos::L1Regularizer* _msg = _internal_mutable_l1_regularizer();
  // @@protoc_insertion_point(field_mutable:second.protos.Regularizer.l1_regularizer)
  return _msg;
}

// .second.protos.L2Regularizer l2_regularizer = 2;
inline bool Regularizer::_internal_has_l2_regularizer() const {
  return regularizer_oneof_case() == kL2Regularizer;
}
inline bool Regularizer::has_l2_regularizer() const {
  return _internal_has_l2_regularizer();
}
inline void Regularizer::set_has_l2_regularizer() {
  _impl_._oneof_case_[0] = kL2Regularizer;
}
inline void Regularizer::clear_l2_regularizer() {
  if (_internal_has_l2_regularizer()) {
    if (GetArenaForAllocation() == nullptr) {
      delete _impl_.regularizer_oneof_.l2_regularizer_;
    }
    clear_has_regularizer_oneof();
  }
}
inline ::second::protos::L2Regularizer* Regularizer::release_l2_regularizer() {
  // @@protoc_insertion_point(field_release:second.protos.Regularizer.l2_regularizer)
  if (_internal_has_l2_regularizer()) {
    clear_has_regularizer_oneof();
    ::second::protos::L2Regularizer* temp = _impl_.regularizer_oneof_.l2_regularizer_;
    if (GetArenaForAllocation() != nullptr) {
      temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
    }
    _impl_.regularizer_oneof_.l2_regularizer_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline const ::second::protos::L2Regularizer& Regularizer::_internal_l2_regularizer() const {
  return _internal_has_l2_regularizer()
      ? *_impl_.regularizer_oneof_.l2_regularizer_
      : reinterpret_cast< ::second::protos::L2Regularizer&>(::second::protos::_L2Regularizer_default_instance_);
}
inline const ::second::protos::L2Regularizer& Regularizer::l2_regularizer() const {
  // @@protoc_insertion_point(field_get:second.protos.Regularizer.l2_regularizer)
  return _internal_l2_regularizer();
}
inline ::second::protos::L2Regularizer* Regularizer::unsafe_arena_release_l2_regularizer() {
  // @@protoc_insertion_point(field_unsafe_arena_release:second.protos.Regularizer.l2_regularizer)
  if (_internal_has_l2_regularizer()) {
    clear_has_regularizer_oneof();
    ::second::protos::L2Regularizer* temp = _impl_.regularizer_oneof_.l2_regularizer_;
    _impl_.regularizer_oneof_.l2_regularizer_ = nullptr;
    return temp;
  } else {
    return nullptr;
  }
}
inline void Regularizer::unsafe_arena_set_allocated_l2_regularizer(::second::protos::L2Regularizer* l2_regularizer) {
  clear_regularizer_oneof();
  if (l2_regularizer) {
    set_has_l2_regularizer();
    _impl_.regularizer_oneof_.l2_regularizer_ = l2_regularizer;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.Regularizer.l2_regularizer)
}
inline ::second::protos::L2Regularizer* Regularizer::_internal_mutable_l2_regularizer() {
  if (!_internal_has_l2_regularizer()) {
    clear_regularizer_oneof();
    set_has_l2_regularizer();
    _impl_.regularizer_oneof_.l2_regularizer_ = CreateMaybeMessage< ::second::protos::L2Regularizer >(GetArenaForAllocation());
  }
  return _impl_.regularizer_oneof_.l2_regularizer_;
}
inline ::second::protos::L2Regularizer* Regularizer::mutable_l2_regularizer() {
  ::second::protos::L2Regularizer* _msg = _internal_mutable_l2_regularizer();
  // @@protoc_insertion_point(field_mutable:second.protos.Regularizer.l2_regularizer)
  return _msg;
}

inline bool Regularizer::has_regularizer_oneof() const {
  return regularizer_oneof_case() != REGULARIZER_ONEOF_NOT_SET;
}
inline void Regularizer::clear_has_regularizer_oneof() {
  _impl_._oneof_case_[0] = REGULARIZER_ONEOF_NOT_SET;
}
inline Regularizer::RegularizerOneofCase Regularizer::regularizer_oneof_case() const {
  return Regularizer::RegularizerOneofCase(_impl_._oneof_case_[0]);
}
// -------------------------------------------------------------------

// L1Regularizer

// float weight = 1;
inline void L1Regularizer::clear_weight() {
  _impl_.weight_ = 0;
}
inline float L1Regularizer::_internal_weight() const {
  return _impl_.weight_;
}
inline float L1Regularizer::weight() const {
  // @@protoc_insertion_point(field_get:second.protos.L1Regularizer.weight)
  return _internal_weight();
}
inline void L1Regularizer::_internal_set_weight(float value) {
  
  _impl_.weight_ = value;
}
inline void L1Regularizer::set_weight(float value) {
  _internal_set_weight(value);
  // @@protoc_insertion_point(field_set:second.protos.L1Regularizer.weight)
}

// -------------------------------------------------------------------

// L2Regularizer

// float weight = 1;
inline void L2Regularizer::clear_weight() {
  _impl_.weight_ = 0;
}
inline float L2Regularizer::_internal_weight() const {
  return _impl_.weight_;
}
inline float L2Regularizer::weight() const {
  // @@protoc_insertion_point(field_get:second.protos.L2Regularizer.weight)
  return _internal_weight();
}
inline void L2Regularizer::_internal_set_weight(float value) {
  
  _impl_.weight_ = value;
}
inline void L2Regularizer::set_weight(float value) {
  _internal_set_weight(value);
  // @@protoc_insertion_point(field_set:second.protos.L2Regularizer.weight)
}

// -------------------------------------------------------------------

// TrainConfig

// .second.protos.Optimizer optimizer = 1;
inline bool TrainConfig::_internal_has_optimizer() const {
  return this != internal_default_instance() && _impl_.optimizer_ != nullptr;
}
inline bool TrainConfig::has_optimizer() const {
  return _internal_has_optimizer();
}
inline const ::second::protos::Optimizer& TrainConfig::_internal_optimizer() const {
  const ::second::protos::Optimizer* p = _impl_.optimizer_;
  return p != nullptr ? *p : reinterpret_cast<const ::second::protos::Optimizer&>(
      ::second::protos::_Optimizer_default_instance_);
}
inline const ::second::protos::Optimizer& TrainConfig::optimizer() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainConfig.optimizer)
  return _internal_optimizer();
}
inline void TrainConfig::unsafe_arena_set_allocated_optimizer(
    ::second::protos::Optimizer* optimizer) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.optimizer_);
  }
  _impl_.optimizer_ = optimizer;
  if (optimizer) {
    
  } else {
    
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:second.protos.TrainConfig.optimizer)
}
inline ::second::protos::Optimizer* TrainConfig::release_optimizer() {
  
  ::second::protos::Optimizer* temp = _impl_.optimizer_;
  _impl_.optimizer_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::second::protos::Optimizer* TrainConfig::unsafe_arena_release_optimizer() {
  // @@protoc_insertion_point(field_release:second.protos.TrainConfig.optimizer)
  
  ::second::protos::Optimizer* temp = _impl_.optimizer_;
  _impl_.optimizer_ = nullptr;
  return temp;
}
inline ::second::protos::Optimizer* TrainConfig::_internal_mutable_optimizer() {
  
  if (_impl_.optimizer_ == nullptr) {
    auto* p = CreateMaybeMessage<::second::protos::Optimizer>(GetArenaForAllocation());
    _impl_.optimizer_ = p;
  }
  return _impl_.optimizer_;
}
inline ::second::protos::Optimizer* TrainConfig::mutable_optimizer() {
  ::second::protos::Optimizer* _msg = _internal_mutable_optimizer();
  // @@protoc_insertion_point(field_mutable:second.protos.TrainConfig.optimizer)
  return _msg;
}
inline void TrainConfig::set_allocated_optimizer(::second::protos::Optimizer* optimizer) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(_impl_.optimizer_);
  }
  if (optimizer) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(optimizer));
    if (message_arena != submessage_arena) {
      optimizer = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, optimizer, submessage_arena);
    }
    
  } else {
    
  }
  _impl_.optimizer_ = optimizer;
  // @@protoc_insertion_point(field_set_allocated:second.protos.TrainConfig.optimizer)
}

// uint32 inter_op_parallelism_threads = 3;
inline void TrainConfig::clear_inter_op_parallelism_threads() {
  _impl_.inter_op_parallelism_threads_ = 0u;
}
inline uint32_t TrainConfig::_internal_inter_op_parallelism_threads() const {
  return _impl_.inter_op_parallelism_threads_;
}
inline uint32_t TrainConfig::inter_op_parallelism_threads() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainConfig.inter_op_parallelism_threads)
  return _internal_inter_op_parallelism_threads();
}
inline void TrainConfig::_internal_set_inter_op_parallelism_threads(uint32_t value) {
  
  _impl_.inter_op_parallelism_threads_ = value;
}
inline void TrainConfig::set_inter_op_parallelism_threads(uint32_t value) {
  _internal_set_inter_op_parallelism_threads(value);
  // @@protoc_insertion_point(field_set:second.protos.TrainConfig.inter_op_parallelism_threads)
}

// uint32 intra_op_parallelism_threads = 4;
inline void TrainConfig::clear_intra_op_parallelism_threads() {
  _impl_.intra_op_parallelism_threads_ = 0u;
}
inline uint32_t TrainConfig::_internal_intra_op_parallelism_threads() const {
  return _impl_.intra_op_parallelism_threads_;
}
inline uint32_t TrainConfig::intra_op_parallelism_threads() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainConfig.intra_op_parallelism_threads)
  return _internal_intra_op_parallelism_threads();
}
inline void TrainConfig::_internal_set_intra_op_parallelism_threads(uint32_t value) {
  
  _impl_.intra_op_parallelism_threads_ = value;
}
inline void TrainConfig::set_intra_op_parallelism_threads(uint32_t value) {
  _internal_set_intra_op_parallelism_threads(value);
  // @@protoc_insertion_point(field_set:second.protos.TrainConfig.intra_op_parallelism_threads)
}

// uint32 steps = 5;
inline void TrainConfig::clear_steps() {
  _impl_.steps_ = 0u;
}
inline uint32_t TrainConfig::_internal_steps() const {
  return _impl_.steps_;
}
inline uint32_t TrainConfig::steps() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainConfig.steps)
  return _internal_steps();
}
inline void TrainConfig::_internal_set_steps(uint32_t value) {
  
  _impl_.steps_ = value;
}
inline void TrainConfig::set_steps(uint32_t value) {
  _internal_set_steps(value);
  // @@protoc_insertion_point(field_set:second.protos.TrainConfig.steps)
}

// uint32 steps_per_eval = 6;
inline void TrainConfig::clear_steps_per_eval() {
  _impl_.steps_per_eval_ = 0u;
}
inline uint32_t TrainConfig::_internal_steps_per_eval() const {
  return _impl_.steps_per_eval_;
}
inline uint32_t TrainConfig::steps_per_eval() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainConfig.steps_per_eval)
  return _internal_steps_per_eval();
}
inline void TrainConfig::_internal_set_steps_per_eval(uint32_t value) {
  
  _impl_.steps_per_eval_ = value;
}
inline void TrainConfig::set_steps_per_eval(uint32_t value) {
  _internal_set_steps_per_eval(value);
  // @@protoc_insertion_point(field_set:second.protos.TrainConfig.steps_per_eval)
}

// uint32 save_checkpoints_secs = 7;
inline void TrainConfig::clear_save_checkpoints_secs() {
  _impl_.save_checkpoints_secs_ = 0u;
}
inline uint32_t TrainConfig::_internal_save_checkpoints_secs() const {
  return _impl_.save_checkpoints_secs_;
}
inline uint32_t TrainConfig::save_checkpoints_secs() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainConfig.save_checkpoints_secs)
  return _internal_save_checkpoints_secs();
}
inline void TrainConfig::_internal_set_save_checkpoints_secs(uint32_t value) {
  
  _impl_.save_checkpoints_secs_ = value;
}
inline void TrainConfig::set_save_checkpoints_secs(uint32_t value) {
  _internal_set_save_checkpoints_secs(value);
  // @@protoc_insertion_point(field_set:second.protos.TrainConfig.save_checkpoints_secs)
}

// uint32 save_summary_steps = 8;
inline void TrainConfig::clear_save_summary_steps() {
  _impl_.save_summary_steps_ = 0u;
}
inline uint32_t TrainConfig::_internal_save_summary_steps() const {
  return _impl_.save_summary_steps_;
}
inline uint32_t TrainConfig::save_summary_steps() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainConfig.save_summary_steps)
  return _internal_save_summary_steps();
}
inline void TrainConfig::_internal_set_save_summary_steps(uint32_t value) {
  
  _impl_.save_summary_steps_ = value;
}
inline void TrainConfig::set_save_summary_steps(uint32_t value) {
  _internal_set_save_summary_steps(value);
  // @@protoc_insertion_point(field_set:second.protos.TrainConfig.save_summary_steps)
}

// bool enable_mixed_precision = 9;
inline void TrainConfig::clear_enable_mixed_precision() {
  _impl_.enable_mixed_precision_ = false;
}
inline bool TrainConfig::_internal_enable_mixed_precision() const {
  return _impl_.enable_mixed_precision_;
}
inline bool TrainConfig::enable_mixed_precision() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainConfig.enable_mixed_precision)
  return _internal_enable_mixed_precision();
}
inline void TrainConfig::_internal_set_enable_mixed_precision(bool value) {
  
  _impl_.enable_mixed_precision_ = value;
}
inline void TrainConfig::set_enable_mixed_precision(bool value) {
  _internal_set_enable_mixed_precision(value);
  // @@protoc_insertion_point(field_set:second.protos.TrainConfig.enable_mixed_precision)
}

// float loss_scale_factor = 10;
inline void TrainConfig::clear_loss_scale_factor() {
  _impl_.loss_scale_factor_ = 0;
}
inline float TrainConfig::_internal_loss_scale_factor() const {
  return _impl_.loss_scale_factor_;
}
inline float TrainConfig::loss_scale_factor() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainConfig.loss_scale_factor)
  return _internal_loss_scale_factor();
}
inline void TrainConfig::_internal_set_loss_scale_factor(float value) {
  
  _impl_.loss_scale_factor_ = value;
}
inline void TrainConfig::set_loss_scale_factor(float value) {
  _internal_set_loss_scale_factor(value);
  // @@protoc_insertion_point(field_set:second.protos.TrainConfig.loss_scale_factor)
}

// bool clear_metrics_every_epoch = 11;
inline void TrainConfig::clear_clear_metrics_every_epoch() {
  _impl_.clear_metrics_every_epoch_ = false;
}
inline bool TrainConfig::_internal_clear_metrics_every_epoch() const {
  return _impl_.clear_metrics_every_epoch_;
}
inline bool TrainConfig::clear_metrics_every_epoch() const {
  // @@protoc_insertion_point(field_get:second.protos.TrainConfig.clear_metrics_every_epoch)
  return _internal_clear_metrics_every_epoch();
}
inline void TrainConfig::_internal_set_clear_metrics_every_epoch(bool value) {
  
  _impl_.clear_metrics_every_epoch_ = value;
}
inline void TrainConfig::set_clear_metrics_every_epoch(bool value) {
  _internal_set_clear_metrics_every_epoch(value);
  // @@protoc_insertion_point(field_set:second.protos.TrainConfig.clear_metrics_every_epoch)
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------


// @@protoc_insertion_point(namespace_scope)

}  // namespace protos
}  // namespace second

// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_second_2fprotos_2ftrain_2eproto
