// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: object_detection/protos/calibration.proto

#include "object_detection/protos/calibration.pb.h"

#include <algorithm>

#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/extension_set.h>
#include <google/protobuf/wire_format_lite.h>
#include <google/protobuf/descriptor.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/reflection_ops.h>
#include <google/protobuf/wire_format.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>

PROTOBUF_PRAGMA_INIT_SEG

namespace _pb = ::PROTOBUF_NAMESPACE_ID;
namespace _pbi = _pb::internal;

namespace object_detection {
namespace protos {
PROTOBUF_CONSTEXPR CalibrationConfig::CalibrationConfig(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.calibrator_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_._oneof_case_)*/{}} {}
struct CalibrationConfigDefaultTypeInternal {
  PROTOBUF_CONSTEXPR CalibrationConfigDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~CalibrationConfigDefaultTypeInternal() {}
  union {
    CalibrationConfig _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 CalibrationConfigDefaultTypeInternal _CalibrationConfig_default_instance_;
PROTOBUF_CONSTEXPR FunctionApproximation::FunctionApproximation(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_._has_bits_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_.x_y_pairs_)*/nullptr} {}
struct FunctionApproximationDefaultTypeInternal {
  PROTOBUF_CONSTEXPR FunctionApproximationDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~FunctionApproximationDefaultTypeInternal() {}
  union {
    FunctionApproximation _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 FunctionApproximationDefaultTypeInternal _FunctionApproximation_default_instance_;
PROTOBUF_CONSTEXPR ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse(
    ::_pbi::ConstantInitialized) {}
struct ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUseDefaultTypeInternal {
  PROTOBUF_CONSTEXPR ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUseDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUseDefaultTypeInternal() {}
  union {
    ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUseDefaultTypeInternal _ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse_default_instance_;
PROTOBUF_CONSTEXPR ClassIdFunctionApproximations::ClassIdFunctionApproximations(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.class_id_xy_pairs_map_)*/{::_pbi::ConstantInitialized()}
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct ClassIdFunctionApproximationsDefaultTypeInternal {
  PROTOBUF_CONSTEXPR ClassIdFunctionApproximationsDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~ClassIdFunctionApproximationsDefaultTypeInternal() {}
  union {
    ClassIdFunctionApproximations _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 ClassIdFunctionApproximationsDefaultTypeInternal _ClassIdFunctionApproximations_default_instance_;
PROTOBUF_CONSTEXPR SigmoidCalibration::SigmoidCalibration(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_._has_bits_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_.sigmoid_parameters_)*/nullptr} {}
struct SigmoidCalibrationDefaultTypeInternal {
  PROTOBUF_CONSTEXPR SigmoidCalibrationDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~SigmoidCalibrationDefaultTypeInternal() {}
  union {
    SigmoidCalibration _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 SigmoidCalibrationDefaultTypeInternal _SigmoidCalibration_default_instance_;
PROTOBUF_CONSTEXPR ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse(
    ::_pbi::ConstantInitialized) {}
struct ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUseDefaultTypeInternal {
  PROTOBUF_CONSTEXPR ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUseDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUseDefaultTypeInternal() {}
  union {
    ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUseDefaultTypeInternal _ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse_default_instance_;
PROTOBUF_CONSTEXPR ClassIdSigmoidCalibrations::ClassIdSigmoidCalibrations(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.class_id_sigmoid_parameters_map_)*/{::_pbi::ConstantInitialized()}
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct ClassIdSigmoidCalibrationsDefaultTypeInternal {
  PROTOBUF_CONSTEXPR ClassIdSigmoidCalibrationsDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~ClassIdSigmoidCalibrationsDefaultTypeInternal() {}
  union {
    ClassIdSigmoidCalibrations _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 ClassIdSigmoidCalibrationsDefaultTypeInternal _ClassIdSigmoidCalibrations_default_instance_;
PROTOBUF_CONSTEXPR XYPairs_XYPair::XYPairs_XYPair(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_._has_bits_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_.x_)*/0
  , /*decltype(_impl_.y_)*/0} {}
struct XYPairs_XYPairDefaultTypeInternal {
  PROTOBUF_CONSTEXPR XYPairs_XYPairDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~XYPairs_XYPairDefaultTypeInternal() {}
  union {
    XYPairs_XYPair _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 XYPairs_XYPairDefaultTypeInternal _XYPairs_XYPair_default_instance_;
PROTOBUF_CONSTEXPR XYPairs::XYPairs(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_._has_bits_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_.x_y_pair_)*/{}
  , /*decltype(_impl_.training_data_type_)*/0} {}
struct XYPairsDefaultTypeInternal {
  PROTOBUF_CONSTEXPR XYPairsDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~XYPairsDefaultTypeInternal() {}
  union {
    XYPairs _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 XYPairsDefaultTypeInternal _XYPairs_default_instance_;
PROTOBUF_CONSTEXPR SigmoidParameters::SigmoidParameters(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_._has_bits_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_.b_)*/0
  , /*decltype(_impl_.a_)*/-1} {}
struct SigmoidParametersDefaultTypeInternal {
  PROTOBUF_CONSTEXPR SigmoidParametersDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~SigmoidParametersDefaultTypeInternal() {}
  union {
    SigmoidParameters _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 SigmoidParametersDefaultTypeInternal _SigmoidParameters_default_instance_;
}  // namespace protos
}  // namespace object_detection
static ::_pb::Metadata file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[10];
static const ::_pb::EnumDescriptor* file_level_enum_descriptors_object_5fdetection_2fprotos_2fcalibration_2eproto[1];
static constexpr ::_pb::ServiceDescriptor const** file_level_service_descriptors_object_5fdetection_2fprotos_2fcalibration_2eproto = nullptr;

const uint32_t TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto::offsets[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::CalibrationConfig, _internal_metadata_),
  ~0u,  // no _extensions_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::CalibrationConfig, _impl_._oneof_case_[0]),
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  ::_pbi::kInvalidFieldOffsetTag,
  ::_pbi::kInvalidFieldOffsetTag,
  ::_pbi::kInvalidFieldOffsetTag,
  ::_pbi::kInvalidFieldOffsetTag,
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::CalibrationConfig, _impl_.calibrator_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::FunctionApproximation, _impl_._has_bits_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::FunctionApproximation, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::FunctionApproximation, _impl_.x_y_pairs_),
  0,
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse, _has_bits_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse, key_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse, value_),
  0,
  1,
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdFunctionApproximations, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdFunctionApproximations, _impl_.class_id_xy_pairs_map_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::SigmoidCalibration, _impl_._has_bits_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::SigmoidCalibration, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::SigmoidCalibration, _impl_.sigmoid_parameters_),
  0,
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse, _has_bits_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse, key_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse, value_),
  0,
  1,
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdSigmoidCalibrations, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::ClassIdSigmoidCalibrations, _impl_.class_id_sigmoid_parameters_map_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::XYPairs_XYPair, _impl_._has_bits_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::XYPairs_XYPair, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::XYPairs_XYPair, _impl_.x_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::XYPairs_XYPair, _impl_.y_),
  0,
  1,
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::XYPairs, _impl_._has_bits_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::XYPairs, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::XYPairs, _impl_.x_y_pair_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::XYPairs, _impl_.training_data_type_),
  ~0u,
  0,
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::SigmoidParameters, _impl_._has_bits_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::SigmoidParameters, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::SigmoidParameters, _impl_.a_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::SigmoidParameters, _impl_.b_),
  1,
  0,
};
static const ::_pbi::MigrationSchema schemas[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  { 0, -1, -1, sizeof(::object_detection::protos::CalibrationConfig)},
  { 11, 18, -1, sizeof(::object_detection::protos::FunctionApproximation)},
  { 19, 27, -1, sizeof(::object_detection::protos::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse)},
  { 29, -1, -1, sizeof(::object_detection::protos::ClassIdFunctionApproximations)},
  { 36, 43, -1, sizeof(::object_detection::protos::SigmoidCalibration)},
  { 44, 52, -1, sizeof(::object_detection::protos::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse)},
  { 54, -1, -1, sizeof(::object_detection::protos::ClassIdSigmoidCalibrations)},
  { 61, 69, -1, sizeof(::object_detection::protos::XYPairs_XYPair)},
  { 71, 79, -1, sizeof(::object_detection::protos::XYPairs)},
  { 81, 89, -1, sizeof(::object_detection::protos::SigmoidParameters)},
};

static const ::_pb::Message* const file_default_instances[] = {
  &::object_detection::protos::_CalibrationConfig_default_instance_._instance,
  &::object_detection::protos::_FunctionApproximation_default_instance_._instance,
  &::object_detection::protos::_ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse_default_instance_._instance,
  &::object_detection::protos::_ClassIdFunctionApproximations_default_instance_._instance,
  &::object_detection::protos::_SigmoidCalibration_default_instance_._instance,
  &::object_detection::protos::_ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse_default_instance_._instance,
  &::object_detection::protos::_ClassIdSigmoidCalibrations_default_instance_._instance,
  &::object_detection::protos::_XYPairs_XYPair_default_instance_._instance,
  &::object_detection::protos::_XYPairs_default_instance_._instance,
  &::object_detection::protos::_SigmoidParameters_default_instance_._instance,
};

const char descriptor_table_protodef_object_5fdetection_2fprotos_2fcalibration_2eproto[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) =
  "\n)object_detection/protos/calibration.pr"
  "oto\022\027object_detection.protos\"\201\003\n\021Calibra"
  "tionConfig\022P\n\026function_approximation\030\001 \001"
  "(\0132..object_detection.protos.FunctionApp"
  "roximationH\000\022b\n class_id_function_approx"
  "imations\030\002 \001(\01326.object_detection.protos"
  ".ClassIdFunctionApproximationsH\000\022J\n\023sigm"
  "oid_calibration\030\003 \001(\0132+.object_detection"
  ".protos.SigmoidCalibrationH\000\022\\\n\035class_id"
  "_sigmoid_calibrations\030\004 \001(\01323.object_det"
  "ection.protos.ClassIdSigmoidCalibrations"
  "H\000B\014\n\ncalibrator\"L\n\025FunctionApproximatio"
  "n\0223\n\tx_y_pairs\030\001 \001(\0132 .object_detection."
  "protos.XYPairs\"\351\001\n\035ClassIdFunctionApprox"
  "imations\022l\n\025class_id_xy_pairs_map\030\001 \003(\0132"
  "M.object_detection.protos.ClassIdFunctio"
  "nApproximations.ClassIdXyPairsMapEntry\032Z"
  "\n\026ClassIdXyPairsMapEntry\022\013\n\003key\030\001 \001(\005\022/\n"
  "\005value\030\002 \001(\0132 .object_detection.protos.X"
  "YPairs:\0028\001\"\\\n\022SigmoidCalibration\022F\n\022sigm"
  "oid_parameters\030\001 \001(\0132*.object_detection."
  "protos.SigmoidParameters\"\213\002\n\032ClassIdSigm"
  "oidCalibrations\022}\n\037class_id_sigmoid_para"
  "meters_map\030\001 \003(\0132T.object_detection.prot"
  "os.ClassIdSigmoidCalibrations.ClassIdSig"
  "moidParametersMapEntry\032n\n ClassIdSigmoid"
  "ParametersMapEntry\022\013\n\003key\030\001 \001(\005\0229\n\005value"
  "\030\002 \001(\0132*.object_detection.protos.Sigmoid"
  "Parameters:\0028\001\"\253\001\n\007XYPairs\0229\n\010x_y_pair\030\001"
  " \003(\0132\'.object_detection.protos.XYPairs.X"
  "YPair\022E\n\022training_data_type\030\002 \001(\0162).obje"
  "ct_detection.protos.TrainingDataType\032\036\n\006"
  "XYPair\022\t\n\001x\030\001 \001(\002\022\t\n\001y\030\002 \001(\002\"0\n\021SigmoidP"
  "arameters\022\r\n\001a\030\001 \001(\002:\002-1\022\014\n\001b\030\002 \001(\002:\0010*N"
  "\n\020TrainingDataType\022\025\n\021DATA_TYPE_UNKNOWN\020"
  "\000\022\017\n\013ALL_CLASSES\020\001\022\022\n\016CLASS_SPECIFIC\020\002"
  ;
static ::_pbi::once_flag descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once;
const ::_pbi::DescriptorTable descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto = {
    false, false, 1438, descriptor_table_protodef_object_5fdetection_2fprotos_2fcalibration_2eproto,
    "object_detection/protos/calibration.proto",
    &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once, nullptr, 0, 10,
    schemas, file_default_instances, TableStruct_object_5fdetection_2fprotos_2fcalibration_2eproto::offsets,
    file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto, file_level_enum_descriptors_object_5fdetection_2fprotos_2fcalibration_2eproto,
    file_level_service_descriptors_object_5fdetection_2fprotos_2fcalibration_2eproto,
};
PROTOBUF_ATTRIBUTE_WEAK const ::_pbi::DescriptorTable* descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter() {
  return &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto;
}

// Force running AddDescriptors() at dynamic initialization time.
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::_pbi::AddDescriptorsRunner dynamic_init_dummy_object_5fdetection_2fprotos_2fcalibration_2eproto(&descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto);
namespace object_detection {
namespace protos {
const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor* TrainingDataType_descriptor() {
  ::PROTOBUF_NAMESPACE_ID::internal::AssignDescriptors(&descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto);
  return file_level_enum_descriptors_object_5fdetection_2fprotos_2fcalibration_2eproto[0];
}
bool TrainingDataType_IsValid(int value) {
  switch (value) {
    case 0:
    case 1:
    case 2:
      return true;
    default:
      return false;
  }
}


// ===================================================================

class CalibrationConfig::_Internal {
 public:
  static const ::object_detection::protos::FunctionApproximation& function_approximation(const CalibrationConfig* msg);
  static const ::object_detection::protos::ClassIdFunctionApproximations& class_id_function_approximations(const CalibrationConfig* msg);
  static const ::object_detection::protos::SigmoidCalibration& sigmoid_calibration(const CalibrationConfig* msg);
  static const ::object_detection::protos::ClassIdSigmoidCalibrations& class_id_sigmoid_calibrations(const CalibrationConfig* msg);
};

const ::object_detection::protos::FunctionApproximation&
CalibrationConfig::_Internal::function_approximation(const CalibrationConfig* msg) {
  return *msg->_impl_.calibrator_.function_approximation_;
}
const ::object_detection::protos::ClassIdFunctionApproximations&
CalibrationConfig::_Internal::class_id_function_approximations(const CalibrationConfig* msg) {
  return *msg->_impl_.calibrator_.class_id_function_approximations_;
}
const ::object_detection::protos::SigmoidCalibration&
CalibrationConfig::_Internal::sigmoid_calibration(const CalibrationConfig* msg) {
  return *msg->_impl_.calibrator_.sigmoid_calibration_;
}
const ::object_detection::protos::ClassIdSigmoidCalibrations&
CalibrationConfig::_Internal::class_id_sigmoid_calibrations(const CalibrationConfig* msg) {
  return *msg->_impl_.calibrator_.class_id_sigmoid_calibrations_;
}
void CalibrationConfig::set_allocated_function_approximation(::object_detection::protos::FunctionApproximation* function_approximation) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  clear_calibrator();
  if (function_approximation) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(function_approximation);
    if (message_arena != submessage_arena) {
      function_approximation = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, function_approximation, submessage_arena);
    }
    set_has_function_approximation();
    _impl_.calibrator_.function_approximation_ = function_approximation;
  }
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.CalibrationConfig.function_approximation)
}
void CalibrationConfig::set_allocated_class_id_function_approximations(::object_detection::protos::ClassIdFunctionApproximations* class_id_function_approximations) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  clear_calibrator();
  if (class_id_function_approximations) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(class_id_function_approximations);
    if (message_arena != submessage_arena) {
      class_id_function_approximations = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, class_id_function_approximations, submessage_arena);
    }
    set_has_class_id_function_approximations();
    _impl_.calibrator_.class_id_function_approximations_ = class_id_function_approximations;
  }
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.CalibrationConfig.class_id_function_approximations)
}
void CalibrationConfig::set_allocated_sigmoid_calibration(::object_detection::protos::SigmoidCalibration* sigmoid_calibration) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  clear_calibrator();
  if (sigmoid_calibration) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(sigmoid_calibration);
    if (message_arena != submessage_arena) {
      sigmoid_calibration = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, sigmoid_calibration, submessage_arena);
    }
    set_has_sigmoid_calibration();
    _impl_.calibrator_.sigmoid_calibration_ = sigmoid_calibration;
  }
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.CalibrationConfig.sigmoid_calibration)
}
void CalibrationConfig::set_allocated_class_id_sigmoid_calibrations(::object_detection::protos::ClassIdSigmoidCalibrations* class_id_sigmoid_calibrations) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  clear_calibrator();
  if (class_id_sigmoid_calibrations) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
      ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(class_id_sigmoid_calibrations);
    if (message_arena != submessage_arena) {
      class_id_sigmoid_calibrations = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, class_id_sigmoid_calibrations, submessage_arena);
    }
    set_has_class_id_sigmoid_calibrations();
    _impl_.calibrator_.class_id_sigmoid_calibrations_ = class_id_sigmoid_calibrations;
  }
  // @@protoc_insertion_point(field_set_allocated:object_detection.protos.CalibrationConfig.class_id_sigmoid_calibrations)
}
CalibrationConfig::CalibrationConfig(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:object_detection.protos.CalibrationConfig)
}
CalibrationConfig::CalibrationConfig(const CalibrationConfig& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  CalibrationConfig* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.calibrator_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , /*decltype(_impl_._oneof_case_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  clear_has_calibrator();
  switch (from.calibrator_case()) {
    case kFunctionApproximation: {
      _this->_internal_mutable_function_approximation()->::object_detection::protos::FunctionApproximation::MergeFrom(
          from._internal_function_approximation());
      break;
    }
    case kClassIdFunctionApproximations: {
      _this->_internal_mutable_class_id_function_approximations()->::object_detection::protos::ClassIdFunctionApproximations::MergeFrom(
          from._internal_class_id_function_approximations());
      break;
    }
    case kSigmoidCalibration: {
      _this->_internal_mutable_sigmoid_calibration()->::object_detection::protos::SigmoidCalibration::MergeFrom(
          from._internal_sigmoid_calibration());
      break;
    }
    case kClassIdSigmoidCalibrations: {
      _this->_internal_mutable_class_id_sigmoid_calibrations()->::object_detection::protos::ClassIdSigmoidCalibrations::MergeFrom(
          from._internal_class_id_sigmoid_calibrations());
      break;
    }
    case CALIBRATOR_NOT_SET: {
      break;
    }
  }
  // @@protoc_insertion_point(copy_constructor:object_detection.protos.CalibrationConfig)
}

inline void CalibrationConfig::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.calibrator_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , /*decltype(_impl_._oneof_case_)*/{}
  };
  clear_has_calibrator();
}

CalibrationConfig::~CalibrationConfig() {
  // @@protoc_insertion_point(destructor:object_detection.protos.CalibrationConfig)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void CalibrationConfig::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  if (has_calibrator()) {
    clear_calibrator();
  }
}

void CalibrationConfig::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void CalibrationConfig::clear_calibrator() {
// @@protoc_insertion_point(one_of_clear_start:object_detection.protos.CalibrationConfig)
  switch (calibrator_case()) {
    case kFunctionApproximation: {
      if (GetArenaForAllocation() == nullptr) {
        delete _impl_.calibrator_.function_approximation_;
      }
      break;
    }
    case kClassIdFunctionApproximations: {
      if (GetArenaForAllocation() == nullptr) {
        delete _impl_.calibrator_.class_id_function_approximations_;
      }
      break;
    }
    case kSigmoidCalibration: {
      if (GetArenaForAllocation() == nullptr) {
        delete _impl_.calibrator_.sigmoid_calibration_;
      }
      break;
    }
    case kClassIdSigmoidCalibrations: {
      if (GetArenaForAllocation() == nullptr) {
        delete _impl_.calibrator_.class_id_sigmoid_calibrations_;
      }
      break;
    }
    case CALIBRATOR_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = CALIBRATOR_NOT_SET;
}


void CalibrationConfig::Clear() {
// @@protoc_insertion_point(message_clear_start:object_detection.protos.CalibrationConfig)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  clear_calibrator();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* CalibrationConfig::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // .object_detection.protos.FunctionApproximation function_approximation = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr = ctx->ParseMessage(_internal_mutable_function_approximation(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .object_detection.protos.ClassIdFunctionApproximations class_id_function_approximations = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 18)) {
          ptr = ctx->ParseMessage(_internal_mutable_class_id_function_approximations(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .object_detection.protos.SigmoidCalibration sigmoid_calibration = 3;
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 26)) {
          ptr = ctx->ParseMessage(_internal_mutable_sigmoid_calibration(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // .object_detection.protos.ClassIdSigmoidCalibrations class_id_sigmoid_calibrations = 4;
      case 4:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 34)) {
          ptr = ctx->ParseMessage(_internal_mutable_class_id_sigmoid_calibrations(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* CalibrationConfig::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:object_detection.protos.CalibrationConfig)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  switch (calibrator_case()) {
    case kFunctionApproximation: {
      target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
        InternalWriteMessage(1, _Internal::function_approximation(this),
          _Internal::function_approximation(this).GetCachedSize(), target, stream);
      break;
    }
    case kClassIdFunctionApproximations: {
      target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
        InternalWriteMessage(2, _Internal::class_id_function_approximations(this),
          _Internal::class_id_function_approximations(this).GetCachedSize(), target, stream);
      break;
    }
    case kSigmoidCalibration: {
      target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
        InternalWriteMessage(3, _Internal::sigmoid_calibration(this),
          _Internal::sigmoid_calibration(this).GetCachedSize(), target, stream);
      break;
    }
    case kClassIdSigmoidCalibrations: {
      target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
        InternalWriteMessage(4, _Internal::class_id_sigmoid_calibrations(this),
          _Internal::class_id_sigmoid_calibrations(this).GetCachedSize(), target, stream);
      break;
    }
    default: ;
  }
  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:object_detection.protos.CalibrationConfig)
  return target;
}

size_t CalibrationConfig::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:object_detection.protos.CalibrationConfig)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  switch (calibrator_case()) {
    // .object_detection.protos.FunctionApproximation function_approximation = 1;
    case kFunctionApproximation: {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.calibrator_.function_approximation_);
      break;
    }
    // .object_detection.protos.ClassIdFunctionApproximations class_id_function_approximations = 2;
    case kClassIdFunctionApproximations: {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.calibrator_.class_id_function_approximations_);
      break;
    }
    // .object_detection.protos.SigmoidCalibration sigmoid_calibration = 3;
    case kSigmoidCalibration: {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.calibrator_.sigmoid_calibration_);
      break;
    }
    // .object_detection.protos.ClassIdSigmoidCalibrations class_id_sigmoid_calibrations = 4;
    case kClassIdSigmoidCalibrations: {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
          *_impl_.calibrator_.class_id_sigmoid_calibrations_);
      break;
    }
    case CALIBRATOR_NOT_SET: {
      break;
    }
  }
  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData CalibrationConfig::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    CalibrationConfig::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*CalibrationConfig::GetClassData() const { return &_class_data_; }


void CalibrationConfig::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<CalibrationConfig*>(&to_msg);
  auto& from = static_cast<const CalibrationConfig&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:object_detection.protos.CalibrationConfig)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  switch (from.calibrator_case()) {
    case kFunctionApproximation: {
      _this->_internal_mutable_function_approximation()->::object_detection::protos::FunctionApproximation::MergeFrom(
          from._internal_function_approximation());
      break;
    }
    case kClassIdFunctionApproximations: {
      _this->_internal_mutable_class_id_function_approximations()->::object_detection::protos::ClassIdFunctionApproximations::MergeFrom(
          from._internal_class_id_function_approximations());
      break;
    }
    case kSigmoidCalibration: {
      _this->_internal_mutable_sigmoid_calibration()->::object_detection::protos::SigmoidCalibration::MergeFrom(
          from._internal_sigmoid_calibration());
      break;
    }
    case kClassIdSigmoidCalibrations: {
      _this->_internal_mutable_class_id_sigmoid_calibrations()->::object_detection::protos::ClassIdSigmoidCalibrations::MergeFrom(
          from._internal_class_id_sigmoid_calibrations());
      break;
    }
    case CALIBRATOR_NOT_SET: {
      break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void CalibrationConfig::CopyFrom(const CalibrationConfig& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:object_detection.protos.CalibrationConfig)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool CalibrationConfig::IsInitialized() const {
  return true;
}

void CalibrationConfig::InternalSwap(CalibrationConfig* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_.calibrator_, other->_impl_.calibrator_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
}

::PROTOBUF_NAMESPACE_ID::Metadata CalibrationConfig::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[0]);
}

// ===================================================================

class FunctionApproximation::_Internal {
 public:
  using HasBits = decltype(std::declval<FunctionApproximation>()._impl_._has_bits_);
  static const ::object_detection::protos::XYPairs& x_y_pairs(const FunctionApproximation* msg);
  static void set_has_x_y_pairs(HasBits* has_bits) {
    (*has_bits)[0] |= 1u;
  }
};

const ::object_detection::protos::XYPairs&
FunctionApproximation::_Internal::x_y_pairs(const FunctionApproximation* msg) {
  return *msg->_impl_.x_y_pairs_;
}
FunctionApproximation::FunctionApproximation(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:object_detection.protos.FunctionApproximation)
}
FunctionApproximation::FunctionApproximation(const FunctionApproximation& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  FunctionApproximation* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){from._impl_._has_bits_}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.x_y_pairs_){nullptr}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  if (from._internal_has_x_y_pairs()) {
    _this->_impl_.x_y_pairs_ = new ::object_detection::protos::XYPairs(*from._impl_.x_y_pairs_);
  }
  // @@protoc_insertion_point(copy_constructor:object_detection.protos.FunctionApproximation)
}

inline void FunctionApproximation::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.x_y_pairs_){nullptr}
  };
}

FunctionApproximation::~FunctionApproximation() {
  // @@protoc_insertion_point(destructor:object_detection.protos.FunctionApproximation)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void FunctionApproximation::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  if (this != internal_default_instance()) delete _impl_.x_y_pairs_;
}

void FunctionApproximation::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void FunctionApproximation::Clear() {
// @@protoc_insertion_point(message_clear_start:object_detection.protos.FunctionApproximation)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000001u) {
    GOOGLE_DCHECK(_impl_.x_y_pairs_ != nullptr);
    _impl_.x_y_pairs_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* FunctionApproximation::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  _Internal::HasBits has_bits{};
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // optional .object_detection.protos.XYPairs x_y_pairs = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr = ctx->ParseMessage(_internal_mutable_x_y_pairs(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  _impl_._has_bits_.Or(has_bits);
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* FunctionApproximation::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:object_detection.protos.FunctionApproximation)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  // optional .object_detection.protos.XYPairs x_y_pairs = 1;
  if (cached_has_bits & 0x00000001u) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(1, _Internal::x_y_pairs(this),
        _Internal::x_y_pairs(this).GetCachedSize(), target, stream);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:object_detection.protos.FunctionApproximation)
  return target;
}

size_t FunctionApproximation::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:object_detection.protos.FunctionApproximation)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // optional .object_detection.protos.XYPairs x_y_pairs = 1;
  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000001u) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.x_y_pairs_);
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData FunctionApproximation::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    FunctionApproximation::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*FunctionApproximation::GetClassData() const { return &_class_data_; }


void FunctionApproximation::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<FunctionApproximation*>(&to_msg);
  auto& from = static_cast<const FunctionApproximation&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:object_detection.protos.FunctionApproximation)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  if (from._internal_has_x_y_pairs()) {
    _this->_internal_mutable_x_y_pairs()->::object_detection::protos::XYPairs::MergeFrom(
        from._internal_x_y_pairs());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void FunctionApproximation::CopyFrom(const FunctionApproximation& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:object_detection.protos.FunctionApproximation)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool FunctionApproximation::IsInitialized() const {
  return true;
}

void FunctionApproximation::InternalSwap(FunctionApproximation* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.x_y_pairs_, other->_impl_.x_y_pairs_);
}

::PROTOBUF_NAMESPACE_ID::Metadata FunctionApproximation::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[1]);
}

// ===================================================================

ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse() {}
ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse(::PROTOBUF_NAMESPACE_ID::Arena* arena)
    : SuperType(arena) {}
void ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse::MergeFrom(const ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse& other) {
  MergeFromInternal(other);
}
::PROTOBUF_NAMESPACE_ID::Metadata ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[2]);
}

// ===================================================================

class ClassIdFunctionApproximations::_Internal {
 public:
};

ClassIdFunctionApproximations::ClassIdFunctionApproximations(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  if (arena != nullptr && !is_message_owned) {
    arena->OwnCustomDestructor(this, &ClassIdFunctionApproximations::ArenaDtor);
  }
  // @@protoc_insertion_point(arena_constructor:object_detection.protos.ClassIdFunctionApproximations)
}
ClassIdFunctionApproximations::ClassIdFunctionApproximations(const ClassIdFunctionApproximations& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  ClassIdFunctionApproximations* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      /*decltype(_impl_.class_id_xy_pairs_map_)*/{}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _this->_impl_.class_id_xy_pairs_map_.MergeFrom(from._impl_.class_id_xy_pairs_map_);
  // @@protoc_insertion_point(copy_constructor:object_detection.protos.ClassIdFunctionApproximations)
}

inline void ClassIdFunctionApproximations::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      /*decltype(_impl_.class_id_xy_pairs_map_)*/{::_pbi::ArenaInitialized(), arena}
    , /*decltype(_impl_._cached_size_)*/{}
  };
}

ClassIdFunctionApproximations::~ClassIdFunctionApproximations() {
  // @@protoc_insertion_point(destructor:object_detection.protos.ClassIdFunctionApproximations)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    ArenaDtor(this);
    return;
  }
  SharedDtor();
}

inline void ClassIdFunctionApproximations::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.class_id_xy_pairs_map_.Destruct();
  _impl_.class_id_xy_pairs_map_.~MapField();
}

void ClassIdFunctionApproximations::ArenaDtor(void* object) {
  ClassIdFunctionApproximations* _this = reinterpret_cast< ClassIdFunctionApproximations* >(object);
  _this->_impl_.class_id_xy_pairs_map_.Destruct();
}
void ClassIdFunctionApproximations::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void ClassIdFunctionApproximations::Clear() {
// @@protoc_insertion_point(message_clear_start:object_detection.protos.ClassIdFunctionApproximations)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.class_id_xy_pairs_map_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* ClassIdFunctionApproximations::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // map<int32, .object_detection.protos.XYPairs> class_id_xy_pairs_map = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr -= 1;
          do {
            ptr += 1;
            ptr = ctx->ParseMessage(&_impl_.class_id_xy_pairs_map_, ptr);
            CHK_(ptr);
            if (!ctx->DataAvailable(ptr)) break;
          } while (::PROTOBUF_NAMESPACE_ID::internal::ExpectTag<10>(ptr));
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* ClassIdFunctionApproximations::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:object_detection.protos.ClassIdFunctionApproximations)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // map<int32, .object_detection.protos.XYPairs> class_id_xy_pairs_map = 1;
  if (!this->_internal_class_id_xy_pairs_map().empty()) {
    using MapType = ::_pb::Map<int32_t, ::object_detection::protos::XYPairs>;
    using WireHelper = ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse::Funcs;
    const auto& map_field = this->_internal_class_id_xy_pairs_map();

    if (stream->IsSerializationDeterministic() && map_field.size() > 1) {
      for (const auto& entry : ::_pbi::MapSorterFlat<MapType>(map_field)) {
        target = WireHelper::InternalSerialize(1, entry.first, entry.second, target, stream);
      }
    } else {
      for (const auto& entry : map_field) {
        target = WireHelper::InternalSerialize(1, entry.first, entry.second, target, stream);
      }
    }
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:object_detection.protos.ClassIdFunctionApproximations)
  return target;
}

size_t ClassIdFunctionApproximations::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:object_detection.protos.ClassIdFunctionApproximations)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // map<int32, .object_detection.protos.XYPairs> class_id_xy_pairs_map = 1;
  total_size += 1 *
      ::PROTOBUF_NAMESPACE_ID::internal::FromIntSize(this->_internal_class_id_xy_pairs_map_size());
  for (::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::XYPairs >::const_iterator
      it = this->_internal_class_id_xy_pairs_map().begin();
      it != this->_internal_class_id_xy_pairs_map().end(); ++it) {
    total_size += ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse::Funcs::ByteSizeLong(it->first, it->second);
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData ClassIdFunctionApproximations::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    ClassIdFunctionApproximations::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*ClassIdFunctionApproximations::GetClassData() const { return &_class_data_; }


void ClassIdFunctionApproximations::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<ClassIdFunctionApproximations*>(&to_msg);
  auto& from = static_cast<const ClassIdFunctionApproximations&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:object_detection.protos.ClassIdFunctionApproximations)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.class_id_xy_pairs_map_.MergeFrom(from._impl_.class_id_xy_pairs_map_);
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void ClassIdFunctionApproximations::CopyFrom(const ClassIdFunctionApproximations& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:object_detection.protos.ClassIdFunctionApproximations)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool ClassIdFunctionApproximations::IsInitialized() const {
  return true;
}

void ClassIdFunctionApproximations::InternalSwap(ClassIdFunctionApproximations* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  _impl_.class_id_xy_pairs_map_.InternalSwap(&other->_impl_.class_id_xy_pairs_map_);
}

::PROTOBUF_NAMESPACE_ID::Metadata ClassIdFunctionApproximations::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[3]);
}

// ===================================================================

class SigmoidCalibration::_Internal {
 public:
  using HasBits = decltype(std::declval<SigmoidCalibration>()._impl_._has_bits_);
  static const ::object_detection::protos::SigmoidParameters& sigmoid_parameters(const SigmoidCalibration* msg);
  static void set_has_sigmoid_parameters(HasBits* has_bits) {
    (*has_bits)[0] |= 1u;
  }
};

const ::object_detection::protos::SigmoidParameters&
SigmoidCalibration::_Internal::sigmoid_parameters(const SigmoidCalibration* msg) {
  return *msg->_impl_.sigmoid_parameters_;
}
SigmoidCalibration::SigmoidCalibration(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:object_detection.protos.SigmoidCalibration)
}
SigmoidCalibration::SigmoidCalibration(const SigmoidCalibration& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  SigmoidCalibration* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){from._impl_._has_bits_}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.sigmoid_parameters_){nullptr}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  if (from._internal_has_sigmoid_parameters()) {
    _this->_impl_.sigmoid_parameters_ = new ::object_detection::protos::SigmoidParameters(*from._impl_.sigmoid_parameters_);
  }
  // @@protoc_insertion_point(copy_constructor:object_detection.protos.SigmoidCalibration)
}

inline void SigmoidCalibration::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.sigmoid_parameters_){nullptr}
  };
}

SigmoidCalibration::~SigmoidCalibration() {
  // @@protoc_insertion_point(destructor:object_detection.protos.SigmoidCalibration)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void SigmoidCalibration::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  if (this != internal_default_instance()) delete _impl_.sigmoid_parameters_;
}

void SigmoidCalibration::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void SigmoidCalibration::Clear() {
// @@protoc_insertion_point(message_clear_start:object_detection.protos.SigmoidCalibration)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000001u) {
    GOOGLE_DCHECK(_impl_.sigmoid_parameters_ != nullptr);
    _impl_.sigmoid_parameters_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* SigmoidCalibration::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  _Internal::HasBits has_bits{};
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // optional .object_detection.protos.SigmoidParameters sigmoid_parameters = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr = ctx->ParseMessage(_internal_mutable_sigmoid_parameters(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  _impl_._has_bits_.Or(has_bits);
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* SigmoidCalibration::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:object_detection.protos.SigmoidCalibration)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  // optional .object_detection.protos.SigmoidParameters sigmoid_parameters = 1;
  if (cached_has_bits & 0x00000001u) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(1, _Internal::sigmoid_parameters(this),
        _Internal::sigmoid_parameters(this).GetCachedSize(), target, stream);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:object_detection.protos.SigmoidCalibration)
  return target;
}

size_t SigmoidCalibration::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:object_detection.protos.SigmoidCalibration)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // optional .object_detection.protos.SigmoidParameters sigmoid_parameters = 1;
  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000001u) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.sigmoid_parameters_);
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData SigmoidCalibration::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    SigmoidCalibration::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*SigmoidCalibration::GetClassData() const { return &_class_data_; }


void SigmoidCalibration::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<SigmoidCalibration*>(&to_msg);
  auto& from = static_cast<const SigmoidCalibration&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:object_detection.protos.SigmoidCalibration)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  if (from._internal_has_sigmoid_parameters()) {
    _this->_internal_mutable_sigmoid_parameters()->::object_detection::protos::SigmoidParameters::MergeFrom(
        from._internal_sigmoid_parameters());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void SigmoidCalibration::CopyFrom(const SigmoidCalibration& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:object_detection.protos.SigmoidCalibration)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool SigmoidCalibration::IsInitialized() const {
  return true;
}

void SigmoidCalibration::InternalSwap(SigmoidCalibration* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.sigmoid_parameters_, other->_impl_.sigmoid_parameters_);
}

::PROTOBUF_NAMESPACE_ID::Metadata SigmoidCalibration::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[4]);
}

// ===================================================================

ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse() {}
ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse(::PROTOBUF_NAMESPACE_ID::Arena* arena)
    : SuperType(arena) {}
void ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse::MergeFrom(const ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse& other) {
  MergeFromInternal(other);
}
::PROTOBUF_NAMESPACE_ID::Metadata ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[5]);
}

// ===================================================================

class ClassIdSigmoidCalibrations::_Internal {
 public:
};

ClassIdSigmoidCalibrations::ClassIdSigmoidCalibrations(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  if (arena != nullptr && !is_message_owned) {
    arena->OwnCustomDestructor(this, &ClassIdSigmoidCalibrations::ArenaDtor);
  }
  // @@protoc_insertion_point(arena_constructor:object_detection.protos.ClassIdSigmoidCalibrations)
}
ClassIdSigmoidCalibrations::ClassIdSigmoidCalibrations(const ClassIdSigmoidCalibrations& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  ClassIdSigmoidCalibrations* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      /*decltype(_impl_.class_id_sigmoid_parameters_map_)*/{}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _this->_impl_.class_id_sigmoid_parameters_map_.MergeFrom(from._impl_.class_id_sigmoid_parameters_map_);
  // @@protoc_insertion_point(copy_constructor:object_detection.protos.ClassIdSigmoidCalibrations)
}

inline void ClassIdSigmoidCalibrations::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      /*decltype(_impl_.class_id_sigmoid_parameters_map_)*/{::_pbi::ArenaInitialized(), arena}
    , /*decltype(_impl_._cached_size_)*/{}
  };
}

ClassIdSigmoidCalibrations::~ClassIdSigmoidCalibrations() {
  // @@protoc_insertion_point(destructor:object_detection.protos.ClassIdSigmoidCalibrations)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    ArenaDtor(this);
    return;
  }
  SharedDtor();
}

inline void ClassIdSigmoidCalibrations::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.class_id_sigmoid_parameters_map_.Destruct();
  _impl_.class_id_sigmoid_parameters_map_.~MapField();
}

void ClassIdSigmoidCalibrations::ArenaDtor(void* object) {
  ClassIdSigmoidCalibrations* _this = reinterpret_cast< ClassIdSigmoidCalibrations* >(object);
  _this->_impl_.class_id_sigmoid_parameters_map_.Destruct();
}
void ClassIdSigmoidCalibrations::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void ClassIdSigmoidCalibrations::Clear() {
// @@protoc_insertion_point(message_clear_start:object_detection.protos.ClassIdSigmoidCalibrations)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.class_id_sigmoid_parameters_map_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* ClassIdSigmoidCalibrations::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // map<int32, .object_detection.protos.SigmoidParameters> class_id_sigmoid_parameters_map = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr -= 1;
          do {
            ptr += 1;
            ptr = ctx->ParseMessage(&_impl_.class_id_sigmoid_parameters_map_, ptr);
            CHK_(ptr);
            if (!ctx->DataAvailable(ptr)) break;
          } while (::PROTOBUF_NAMESPACE_ID::internal::ExpectTag<10>(ptr));
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* ClassIdSigmoidCalibrations::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:object_detection.protos.ClassIdSigmoidCalibrations)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // map<int32, .object_detection.protos.SigmoidParameters> class_id_sigmoid_parameters_map = 1;
  if (!this->_internal_class_id_sigmoid_parameters_map().empty()) {
    using MapType = ::_pb::Map<int32_t, ::object_detection::protos::SigmoidParameters>;
    using WireHelper = ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse::Funcs;
    const auto& map_field = this->_internal_class_id_sigmoid_parameters_map();

    if (stream->IsSerializationDeterministic() && map_field.size() > 1) {
      for (const auto& entry : ::_pbi::MapSorterFlat<MapType>(map_field)) {
        target = WireHelper::InternalSerialize(1, entry.first, entry.second, target, stream);
      }
    } else {
      for (const auto& entry : map_field) {
        target = WireHelper::InternalSerialize(1, entry.first, entry.second, target, stream);
      }
    }
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:object_detection.protos.ClassIdSigmoidCalibrations)
  return target;
}

size_t ClassIdSigmoidCalibrations::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:object_detection.protos.ClassIdSigmoidCalibrations)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // map<int32, .object_detection.protos.SigmoidParameters> class_id_sigmoid_parameters_map = 1;
  total_size += 1 *
      ::PROTOBUF_NAMESPACE_ID::internal::FromIntSize(this->_internal_class_id_sigmoid_parameters_map_size());
  for (::PROTOBUF_NAMESPACE_ID::Map< int32_t, ::object_detection::protos::SigmoidParameters >::const_iterator
      it = this->_internal_class_id_sigmoid_parameters_map().begin();
      it != this->_internal_class_id_sigmoid_parameters_map().end(); ++it) {
    total_size += ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse::Funcs::ByteSizeLong(it->first, it->second);
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData ClassIdSigmoidCalibrations::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    ClassIdSigmoidCalibrations::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*ClassIdSigmoidCalibrations::GetClassData() const { return &_class_data_; }


void ClassIdSigmoidCalibrations::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<ClassIdSigmoidCalibrations*>(&to_msg);
  auto& from = static_cast<const ClassIdSigmoidCalibrations&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:object_detection.protos.ClassIdSigmoidCalibrations)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.class_id_sigmoid_parameters_map_.MergeFrom(from._impl_.class_id_sigmoid_parameters_map_);
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void ClassIdSigmoidCalibrations::CopyFrom(const ClassIdSigmoidCalibrations& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:object_detection.protos.ClassIdSigmoidCalibrations)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool ClassIdSigmoidCalibrations::IsInitialized() const {
  return true;
}

void ClassIdSigmoidCalibrations::InternalSwap(ClassIdSigmoidCalibrations* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  _impl_.class_id_sigmoid_parameters_map_.InternalSwap(&other->_impl_.class_id_sigmoid_parameters_map_);
}

::PROTOBUF_NAMESPACE_ID::Metadata ClassIdSigmoidCalibrations::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[6]);
}

// ===================================================================

class XYPairs_XYPair::_Internal {
 public:
  using HasBits = decltype(std::declval<XYPairs_XYPair>()._impl_._has_bits_);
  static void set_has_x(HasBits* has_bits) {
    (*has_bits)[0] |= 1u;
  }
  static void set_has_y(HasBits* has_bits) {
    (*has_bits)[0] |= 2u;
  }
};

XYPairs_XYPair::XYPairs_XYPair(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:object_detection.protos.XYPairs.XYPair)
}
XYPairs_XYPair::XYPairs_XYPair(const XYPairs_XYPair& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  XYPairs_XYPair* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){from._impl_._has_bits_}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.x_){}
    , decltype(_impl_.y_){}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  ::memcpy(&_impl_.x_, &from._impl_.x_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.y_) -
    reinterpret_cast<char*>(&_impl_.x_)) + sizeof(_impl_.y_));
  // @@protoc_insertion_point(copy_constructor:object_detection.protos.XYPairs.XYPair)
}

inline void XYPairs_XYPair::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.x_){0}
    , decltype(_impl_.y_){0}
  };
}

XYPairs_XYPair::~XYPairs_XYPair() {
  // @@protoc_insertion_point(destructor:object_detection.protos.XYPairs.XYPair)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void XYPairs_XYPair::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
}

void XYPairs_XYPair::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void XYPairs_XYPair::Clear() {
// @@protoc_insertion_point(message_clear_start:object_detection.protos.XYPairs.XYPair)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000003u) {
    ::memset(&_impl_.x_, 0, static_cast<size_t>(
        reinterpret_cast<char*>(&_impl_.y_) -
        reinterpret_cast<char*>(&_impl_.x_)) + sizeof(_impl_.y_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* XYPairs_XYPair::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  _Internal::HasBits has_bits{};
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // optional float x = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 13)) {
          _Internal::set_has_x(&has_bits);
          _impl_.x_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // optional float y = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 21)) {
          _Internal::set_has_y(&has_bits);
          _impl_.y_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  _impl_._has_bits_.Or(has_bits);
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* XYPairs_XYPair::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:object_detection.protos.XYPairs.XYPair)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  // optional float x = 1;
  if (cached_has_bits & 0x00000001u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(1, this->_internal_x(), target);
  }

  // optional float y = 2;
  if (cached_has_bits & 0x00000002u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(2, this->_internal_y(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:object_detection.protos.XYPairs.XYPair)
  return target;
}

size_t XYPairs_XYPair::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:object_detection.protos.XYPairs.XYPair)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000003u) {
    // optional float x = 1;
    if (cached_has_bits & 0x00000001u) {
      total_size += 1 + 4;
    }

    // optional float y = 2;
    if (cached_has_bits & 0x00000002u) {
      total_size += 1 + 4;
    }

  }
  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData XYPairs_XYPair::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    XYPairs_XYPair::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*XYPairs_XYPair::GetClassData() const { return &_class_data_; }


void XYPairs_XYPair::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<XYPairs_XYPair*>(&to_msg);
  auto& from = static_cast<const XYPairs_XYPair&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:object_detection.protos.XYPairs.XYPair)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (cached_has_bits & 0x00000003u) {
    if (cached_has_bits & 0x00000001u) {
      _this->_impl_.x_ = from._impl_.x_;
    }
    if (cached_has_bits & 0x00000002u) {
      _this->_impl_.y_ = from._impl_.y_;
    }
    _this->_impl_._has_bits_[0] |= cached_has_bits;
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void XYPairs_XYPair::CopyFrom(const XYPairs_XYPair& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:object_detection.protos.XYPairs.XYPair)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool XYPairs_XYPair::IsInitialized() const {
  return true;
}

void XYPairs_XYPair::InternalSwap(XYPairs_XYPair* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(XYPairs_XYPair, _impl_.y_)
      + sizeof(XYPairs_XYPair::_impl_.y_)
      - PROTOBUF_FIELD_OFFSET(XYPairs_XYPair, _impl_.x_)>(
          reinterpret_cast<char*>(&_impl_.x_),
          reinterpret_cast<char*>(&other->_impl_.x_));
}

::PROTOBUF_NAMESPACE_ID::Metadata XYPairs_XYPair::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[7]);
}

// ===================================================================

class XYPairs::_Internal {
 public:
  using HasBits = decltype(std::declval<XYPairs>()._impl_._has_bits_);
  static void set_has_training_data_type(HasBits* has_bits) {
    (*has_bits)[0] |= 1u;
  }
};

XYPairs::XYPairs(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:object_detection.protos.XYPairs)
}
XYPairs::XYPairs(const XYPairs& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  XYPairs* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){from._impl_._has_bits_}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.x_y_pair_){from._impl_.x_y_pair_}
    , decltype(_impl_.training_data_type_){}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _this->_impl_.training_data_type_ = from._impl_.training_data_type_;
  // @@protoc_insertion_point(copy_constructor:object_detection.protos.XYPairs)
}

inline void XYPairs::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.x_y_pair_){arena}
    , decltype(_impl_.training_data_type_){0}
  };
}

XYPairs::~XYPairs() {
  // @@protoc_insertion_point(destructor:object_detection.protos.XYPairs)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void XYPairs::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.x_y_pair_.~RepeatedPtrField();
}

void XYPairs::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void XYPairs::Clear() {
// @@protoc_insertion_point(message_clear_start:object_detection.protos.XYPairs)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.x_y_pair_.Clear();
  _impl_.training_data_type_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* XYPairs::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  _Internal::HasBits has_bits{};
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // repeated .object_detection.protos.XYPairs.XYPair x_y_pair = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr -= 1;
          do {
            ptr += 1;
            ptr = ctx->ParseMessage(_internal_add_x_y_pair(), ptr);
            CHK_(ptr);
            if (!ctx->DataAvailable(ptr)) break;
          } while (::PROTOBUF_NAMESPACE_ID::internal::ExpectTag<10>(ptr));
        } else
          goto handle_unusual;
        continue;
      // optional .object_detection.protos.TrainingDataType training_data_type = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 16)) {
          uint64_t val = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
          if (PROTOBUF_PREDICT_TRUE(::object_detection::protos::TrainingDataType_IsValid(val))) {
            _internal_set_training_data_type(static_cast<::object_detection::protos::TrainingDataType>(val));
          } else {
            ::PROTOBUF_NAMESPACE_ID::internal::WriteVarint(2, val, mutable_unknown_fields());
          }
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  _impl_._has_bits_.Or(has_bits);
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* XYPairs::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:object_detection.protos.XYPairs)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // repeated .object_detection.protos.XYPairs.XYPair x_y_pair = 1;
  for (unsigned i = 0,
      n = static_cast<unsigned>(this->_internal_x_y_pair_size()); i < n; i++) {
    const auto& repfield = this->_internal_x_y_pair(i);
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
        InternalWriteMessage(1, repfield, repfield.GetCachedSize(), target, stream);
  }

  cached_has_bits = _impl_._has_bits_[0];
  // optional .object_detection.protos.TrainingDataType training_data_type = 2;
  if (cached_has_bits & 0x00000001u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteEnumToArray(
      2, this->_internal_training_data_type(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:object_detection.protos.XYPairs)
  return target;
}

size_t XYPairs::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:object_detection.protos.XYPairs)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // repeated .object_detection.protos.XYPairs.XYPair x_y_pair = 1;
  total_size += 1UL * this->_internal_x_y_pair_size();
  for (const auto& msg : this->_impl_.x_y_pair_) {
    total_size +=
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(msg);
  }

  // optional .object_detection.protos.TrainingDataType training_data_type = 2;
  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000001u) {
    total_size += 1 +
      ::_pbi::WireFormatLite::EnumSize(this->_internal_training_data_type());
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData XYPairs::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    XYPairs::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*XYPairs::GetClassData() const { return &_class_data_; }


void XYPairs::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<XYPairs*>(&to_msg);
  auto& from = static_cast<const XYPairs&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:object_detection.protos.XYPairs)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.x_y_pair_.MergeFrom(from._impl_.x_y_pair_);
  if (from._internal_has_training_data_type()) {
    _this->_internal_set_training_data_type(from._internal_training_data_type());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void XYPairs::CopyFrom(const XYPairs& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:object_detection.protos.XYPairs)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool XYPairs::IsInitialized() const {
  return true;
}

void XYPairs::InternalSwap(XYPairs* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.x_y_pair_.InternalSwap(&other->_impl_.x_y_pair_);
  swap(_impl_.training_data_type_, other->_impl_.training_data_type_);
}

::PROTOBUF_NAMESPACE_ID::Metadata XYPairs::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[8]);
}

// ===================================================================

class SigmoidParameters::_Internal {
 public:
  using HasBits = decltype(std::declval<SigmoidParameters>()._impl_._has_bits_);
  static void set_has_a(HasBits* has_bits) {
    (*has_bits)[0] |= 2u;
  }
  static void set_has_b(HasBits* has_bits) {
    (*has_bits)[0] |= 1u;
  }
};

SigmoidParameters::SigmoidParameters(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:object_detection.protos.SigmoidParameters)
}
SigmoidParameters::SigmoidParameters(const SigmoidParameters& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  SigmoidParameters* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){from._impl_._has_bits_}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.b_){}
    , decltype(_impl_.a_){}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  ::memcpy(&_impl_.b_, &from._impl_.b_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.a_) -
    reinterpret_cast<char*>(&_impl_.b_)) + sizeof(_impl_.a_));
  // @@protoc_insertion_point(copy_constructor:object_detection.protos.SigmoidParameters)
}

inline void SigmoidParameters::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.b_){0}
    , decltype(_impl_.a_){-1}
  };
}

SigmoidParameters::~SigmoidParameters() {
  // @@protoc_insertion_point(destructor:object_detection.protos.SigmoidParameters)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void SigmoidParameters::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
}

void SigmoidParameters::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void SigmoidParameters::Clear() {
// @@protoc_insertion_point(message_clear_start:object_detection.protos.SigmoidParameters)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000003u) {
    _impl_.b_ = 0;
    _impl_.a_ = -1;
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* SigmoidParameters::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  _Internal::HasBits has_bits{};
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // optional float a = 1 [default = -1];
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 13)) {
          _Internal::set_has_a(&has_bits);
          _impl_.a_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // optional float b = 2 [default = 0];
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 21)) {
          _Internal::set_has_b(&has_bits);
          _impl_.b_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  _impl_._has_bits_.Or(has_bits);
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* SigmoidParameters::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:object_detection.protos.SigmoidParameters)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  // optional float a = 1 [default = -1];
  if (cached_has_bits & 0x00000002u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(1, this->_internal_a(), target);
  }

  // optional float b = 2 [default = 0];
  if (cached_has_bits & 0x00000001u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(2, this->_internal_b(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:object_detection.protos.SigmoidParameters)
  return target;
}

size_t SigmoidParameters::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:object_detection.protos.SigmoidParameters)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x00000003u) {
    // optional float b = 2 [default = 0];
    if (cached_has_bits & 0x00000001u) {
      total_size += 1 + 4;
    }

    // optional float a = 1 [default = -1];
    if (cached_has_bits & 0x00000002u) {
      total_size += 1 + 4;
    }

  }
  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData SigmoidParameters::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    SigmoidParameters::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*SigmoidParameters::GetClassData() const { return &_class_data_; }


void SigmoidParameters::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<SigmoidParameters*>(&to_msg);
  auto& from = static_cast<const SigmoidParameters&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:object_detection.protos.SigmoidParameters)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (cached_has_bits & 0x00000003u) {
    if (cached_has_bits & 0x00000001u) {
      _this->_impl_.b_ = from._impl_.b_;
    }
    if (cached_has_bits & 0x00000002u) {
      _this->_impl_.a_ = from._impl_.a_;
    }
    _this->_impl_._has_bits_[0] |= cached_has_bits;
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void SigmoidParameters::CopyFrom(const SigmoidParameters& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:object_detection.protos.SigmoidParameters)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool SigmoidParameters::IsInitialized() const {
  return true;
}

void SigmoidParameters::InternalSwap(SigmoidParameters* other) {
  using std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.b_, other->_impl_.b_);
  swap(_impl_.a_, other->_impl_.a_);
}

::PROTOBUF_NAMESPACE_ID::Metadata SigmoidParameters::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2fcalibration_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2fcalibration_2eproto[9]);
}

// @@protoc_insertion_point(namespace_scope)
}  // namespace protos
}  // namespace object_detection
PROTOBUF_NAMESPACE_OPEN
template<> PROTOBUF_NOINLINE ::object_detection::protos::CalibrationConfig*
Arena::CreateMaybeMessage< ::object_detection::protos::CalibrationConfig >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::CalibrationConfig >(arena);
}
template<> PROTOBUF_NOINLINE ::object_detection::protos::FunctionApproximation*
Arena::CreateMaybeMessage< ::object_detection::protos::FunctionApproximation >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::FunctionApproximation >(arena);
}
template<> PROTOBUF_NOINLINE ::object_detection::protos::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse*
Arena::CreateMaybeMessage< ::object_detection::protos::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::ClassIdFunctionApproximations_ClassIdXyPairsMapEntry_DoNotUse >(arena);
}
template<> PROTOBUF_NOINLINE ::object_detection::protos::ClassIdFunctionApproximations*
Arena::CreateMaybeMessage< ::object_detection::protos::ClassIdFunctionApproximations >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::ClassIdFunctionApproximations >(arena);
}
template<> PROTOBUF_NOINLINE ::object_detection::protos::SigmoidCalibration*
Arena::CreateMaybeMessage< ::object_detection::protos::SigmoidCalibration >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::SigmoidCalibration >(arena);
}
template<> PROTOBUF_NOINLINE ::object_detection::protos::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse*
Arena::CreateMaybeMessage< ::object_detection::protos::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::ClassIdSigmoidCalibrations_ClassIdSigmoidParametersMapEntry_DoNotUse >(arena);
}
template<> PROTOBUF_NOINLINE ::object_detection::protos::ClassIdSigmoidCalibrations*
Arena::CreateMaybeMessage< ::object_detection::protos::ClassIdSigmoidCalibrations >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::ClassIdSigmoidCalibrations >(arena);
}
template<> PROTOBUF_NOINLINE ::object_detection::protos::XYPairs_XYPair*
Arena::CreateMaybeMessage< ::object_detection::protos::XYPairs_XYPair >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::XYPairs_XYPair >(arena);
}
template<> PROTOBUF_NOINLINE ::object_detection::protos::XYPairs*
Arena::CreateMaybeMessage< ::object_detection::protos::XYPairs >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::XYPairs >(arena);
}
template<> PROTOBUF_NOINLINE ::object_detection::protos::SigmoidParameters*
Arena::CreateMaybeMessage< ::object_detection::protos::SigmoidParameters >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::SigmoidParameters >(arena);
}
PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)
#include <google/protobuf/port_undef.inc>
