// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: object_detection/protos/eval.proto

#include "object_detection/protos/eval.pb.h"

#include <algorithm>

#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/extension_set.h>
#include <google/protobuf/wire_format_lite.h>
#include <google/protobuf/descriptor.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/reflection_ops.h>
#include <google/protobuf/wire_format.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>

PROTOBUF_PRAGMA_INIT_SEG

namespace _pb = ::PROTOBUF_NAMESPACE_ID;
namespace _pbi = _pb::internal;

namespace object_detection {
namespace protos {
PROTOBUF_CONSTEXPR EvalConfig::EvalConfig(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_._has_bits_)*/{}
  , /*decltype(_impl_._cached_size_)*/{}
  , /*decltype(_impl_.metrics_set_)*/{}
  , /*decltype(_impl_.visualization_export_dir_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.eval_master_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.export_path_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.groundtruth_box_visualization_color_)*/{nullptr, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.max_evals_)*/0u
  , /*decltype(_impl_.save_graph_)*/false
  , /*decltype(_impl_.ignore_groundtruth_)*/false
  , /*decltype(_impl_.use_moving_averages_)*/false
  , /*decltype(_impl_.eval_instance_masks_)*/false
  , /*decltype(_impl_.skip_scores_)*/false
  , /*decltype(_impl_.skip_labels_)*/false
  , /*decltype(_impl_.visualize_groundtruth_boxes_)*/false
  , /*decltype(_impl_.keep_image_id_for_visualization_export_)*/false
  , /*decltype(_impl_.include_metrics_per_category_)*/false
  , /*decltype(_impl_.recall_lower_bound_)*/0
  , /*decltype(_impl_.num_visualizations_)*/10u
  , /*decltype(_impl_.num_examples_)*/5000u
  , /*decltype(_impl_.eval_interval_secs_)*/300u
  , /*decltype(_impl_.min_score_threshold_)*/0.5f
  , /*decltype(_impl_.max_num_boxes_to_visualize_)*/20
  , /*decltype(_impl_.retain_original_images_)*/true
  , /*decltype(_impl_.batch_size_)*/1u
  , /*decltype(_impl_.recall_upper_bound_)*/1} {}
struct EvalConfigDefaultTypeInternal {
  PROTOBUF_CONSTEXPR EvalConfigDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~EvalConfigDefaultTypeInternal() {}
  union {
    EvalConfig _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 EvalConfigDefaultTypeInternal _EvalConfig_default_instance_;
}  // namespace protos
}  // namespace object_detection
static ::_pb::Metadata file_level_metadata_object_5fdetection_2fprotos_2feval_2eproto[1];
static constexpr ::_pb::EnumDescriptor const** file_level_enum_descriptors_object_5fdetection_2fprotos_2feval_2eproto = nullptr;
static constexpr ::_pb::ServiceDescriptor const** file_level_service_descriptors_object_5fdetection_2fprotos_2feval_2eproto = nullptr;

const uint32_t TableStruct_object_5fdetection_2fprotos_2feval_2eproto::offsets[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_._has_bits_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.batch_size_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.num_visualizations_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.num_examples_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.eval_interval_secs_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.max_evals_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.save_graph_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.visualization_export_dir_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.eval_master_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.metrics_set_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.export_path_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.ignore_groundtruth_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.use_moving_averages_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.eval_instance_masks_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.min_score_threshold_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.max_num_boxes_to_visualize_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.skip_scores_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.skip_labels_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.visualize_groundtruth_boxes_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.groundtruth_box_visualization_color_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.keep_image_id_for_visualization_export_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.retain_original_images_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.include_metrics_per_category_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.recall_lower_bound_),
  PROTOBUF_FIELD_OFFSET(::object_detection::protos::EvalConfig, _impl_.recall_upper_bound_),
  21,
  15,
  16,
  17,
  4,
  5,
  0,
  1,
  ~0u,
  2,
  6,
  7,
  8,
  18,
  19,
  9,
  10,
  11,
  3,
  12,
  20,
  13,
  14,
  22,
};
static const ::_pbi::MigrationSchema schemas[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  { 0, 30, -1, sizeof(::object_detection::protos::EvalConfig)},
};

static const ::_pb::Message* const file_default_instances[] = {
  &::object_detection::protos::_EvalConfig_default_instance_._instance,
};

const char descriptor_table_protodef_object_5fdetection_2fprotos_2feval_2eproto[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) =
  "\n\"object_detection/protos/eval.proto\022\027ob"
  "ject_detection.protos\"\265\006\n\nEvalConfig\022\025\n\n"
  "batch_size\030\031 \001(\r:\0011\022\036\n\022num_visualization"
  "s\030\001 \001(\r:\00210\022\036\n\014num_examples\030\002 \001(\r:\0045000B"
  "\002\030\001\022\037\n\022eval_interval_secs\030\003 \001(\r:\003300\022\030\n\t"
  "max_evals\030\004 \001(\r:\0010B\002\030\001\022\031\n\nsave_graph\030\005 \001"
  "(\010:\005false\022\"\n\030visualization_export_dir\030\006 "
  "\001(\t:\000\022\025\n\013eval_master\030\007 \001(\t:\000\022\023\n\013metrics_"
  "set\030\010 \003(\t\022\025\n\013export_path\030\t \001(\t:\000\022!\n\022igno"
  "re_groundtruth\030\n \001(\010:\005false\022\"\n\023use_movin"
  "g_averages\030\013 \001(\010:\005false\022\"\n\023eval_instance"
  "_masks\030\014 \001(\010:\005false\022 \n\023min_score_thresho"
  "ld\030\r \001(\002:\0030.5\022&\n\032max_num_boxes_to_visual"
  "ize\030\016 \001(\005:\00220\022\032\n\013skip_scores\030\017 \001(\010:\005fals"
  "e\022\032\n\013skip_labels\030\020 \001(\010:\005false\022*\n\033visuali"
  "ze_groundtruth_boxes\030\021 \001(\010:\005false\0222\n#gro"
  "undtruth_box_visualization_color\030\022 \001(\t:\005"
  "black\0225\n&keep_image_id_for_visualization"
  "_export\030\023 \001(\010:\005false\022$\n\026retain_original_"
  "images\030\027 \001(\010:\004true\022+\n\034include_metrics_pe"
  "r_category\030\030 \001(\010:\005false\022\035\n\022recall_lower_"
  "bound\030\032 \001(\002:\0010\022\035\n\022recall_upper_bound\030\033 \001"
  "(\002:\0011"
  ;
static ::_pbi::once_flag descriptor_table_object_5fdetection_2fprotos_2feval_2eproto_once;
const ::_pbi::DescriptorTable descriptor_table_object_5fdetection_2fprotos_2feval_2eproto = {
    false, false, 885, descriptor_table_protodef_object_5fdetection_2fprotos_2feval_2eproto,
    "object_detection/protos/eval.proto",
    &descriptor_table_object_5fdetection_2fprotos_2feval_2eproto_once, nullptr, 0, 1,
    schemas, file_default_instances, TableStruct_object_5fdetection_2fprotos_2feval_2eproto::offsets,
    file_level_metadata_object_5fdetection_2fprotos_2feval_2eproto, file_level_enum_descriptors_object_5fdetection_2fprotos_2feval_2eproto,
    file_level_service_descriptors_object_5fdetection_2fprotos_2feval_2eproto,
};
PROTOBUF_ATTRIBUTE_WEAK const ::_pbi::DescriptorTable* descriptor_table_object_5fdetection_2fprotos_2feval_2eproto_getter() {
  return &descriptor_table_object_5fdetection_2fprotos_2feval_2eproto;
}

// Force running AddDescriptors() at dynamic initialization time.
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::_pbi::AddDescriptorsRunner dynamic_init_dummy_object_5fdetection_2fprotos_2feval_2eproto(&descriptor_table_object_5fdetection_2fprotos_2feval_2eproto);
namespace object_detection {
namespace protos {

// ===================================================================

class EvalConfig::_Internal {
 public:
  using HasBits = decltype(std::declval<EvalConfig>()._impl_._has_bits_);
  static void set_has_batch_size(HasBits* has_bits) {
    (*has_bits)[0] |= 2097152u;
  }
  static void set_has_num_visualizations(HasBits* has_bits) {
    (*has_bits)[0] |= 32768u;
  }
  static void set_has_num_examples(HasBits* has_bits) {
    (*has_bits)[0] |= 65536u;
  }
  static void set_has_eval_interval_secs(HasBits* has_bits) {
    (*has_bits)[0] |= 131072u;
  }
  static void set_has_max_evals(HasBits* has_bits) {
    (*has_bits)[0] |= 16u;
  }
  static void set_has_save_graph(HasBits* has_bits) {
    (*has_bits)[0] |= 32u;
  }
  static void set_has_visualization_export_dir(HasBits* has_bits) {
    (*has_bits)[0] |= 1u;
  }
  static void set_has_eval_master(HasBits* has_bits) {
    (*has_bits)[0] |= 2u;
  }
  static void set_has_export_path(HasBits* has_bits) {
    (*has_bits)[0] |= 4u;
  }
  static void set_has_ignore_groundtruth(HasBits* has_bits) {
    (*has_bits)[0] |= 64u;
  }
  static void set_has_use_moving_averages(HasBits* has_bits) {
    (*has_bits)[0] |= 128u;
  }
  static void set_has_eval_instance_masks(HasBits* has_bits) {
    (*has_bits)[0] |= 256u;
  }
  static void set_has_min_score_threshold(HasBits* has_bits) {
    (*has_bits)[0] |= 262144u;
  }
  static void set_has_max_num_boxes_to_visualize(HasBits* has_bits) {
    (*has_bits)[0] |= 524288u;
  }
  static void set_has_skip_scores(HasBits* has_bits) {
    (*has_bits)[0] |= 512u;
  }
  static void set_has_skip_labels(HasBits* has_bits) {
    (*has_bits)[0] |= 1024u;
  }
  static void set_has_visualize_groundtruth_boxes(HasBits* has_bits) {
    (*has_bits)[0] |= 2048u;
  }
  static void set_has_groundtruth_box_visualization_color(HasBits* has_bits) {
    (*has_bits)[0] |= 8u;
  }
  static void set_has_keep_image_id_for_visualization_export(HasBits* has_bits) {
    (*has_bits)[0] |= 4096u;
  }
  static void set_has_retain_original_images(HasBits* has_bits) {
    (*has_bits)[0] |= 1048576u;
  }
  static void set_has_include_metrics_per_category(HasBits* has_bits) {
    (*has_bits)[0] |= 8192u;
  }
  static void set_has_recall_lower_bound(HasBits* has_bits) {
    (*has_bits)[0] |= 16384u;
  }
  static void set_has_recall_upper_bound(HasBits* has_bits) {
    (*has_bits)[0] |= 4194304u;
  }
};

const ::PROTOBUF_NAMESPACE_ID::internal::LazyString EvalConfig::Impl_::_i_give_permission_to_break_this_code_default_groundtruth_box_visualization_color_{{{"black", 5}}, {nullptr}};
EvalConfig::EvalConfig(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:object_detection.protos.EvalConfig)
}
EvalConfig::EvalConfig(const EvalConfig& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  EvalConfig* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){from._impl_._has_bits_}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.metrics_set_){from._impl_.metrics_set_}
    , decltype(_impl_.visualization_export_dir_){}
    , decltype(_impl_.eval_master_){}
    , decltype(_impl_.export_path_){}
    , decltype(_impl_.groundtruth_box_visualization_color_){}
    , decltype(_impl_.max_evals_){}
    , decltype(_impl_.save_graph_){}
    , decltype(_impl_.ignore_groundtruth_){}
    , decltype(_impl_.use_moving_averages_){}
    , decltype(_impl_.eval_instance_masks_){}
    , decltype(_impl_.skip_scores_){}
    , decltype(_impl_.skip_labels_){}
    , decltype(_impl_.visualize_groundtruth_boxes_){}
    , decltype(_impl_.keep_image_id_for_visualization_export_){}
    , decltype(_impl_.include_metrics_per_category_){}
    , decltype(_impl_.recall_lower_bound_){}
    , decltype(_impl_.num_visualizations_){}
    , decltype(_impl_.num_examples_){}
    , decltype(_impl_.eval_interval_secs_){}
    , decltype(_impl_.min_score_threshold_){}
    , decltype(_impl_.max_num_boxes_to_visualize_){}
    , decltype(_impl_.retain_original_images_){}
    , decltype(_impl_.batch_size_){}
    , decltype(_impl_.recall_upper_bound_){}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _impl_.visualization_export_dir_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.visualization_export_dir_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (from._internal_has_visualization_export_dir()) {
    _this->_impl_.visualization_export_dir_.Set(from._internal_visualization_export_dir(), 
      _this->GetArenaForAllocation());
  }
  _impl_.eval_master_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.eval_master_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (from._internal_has_eval_master()) {
    _this->_impl_.eval_master_.Set(from._internal_eval_master(), 
      _this->GetArenaForAllocation());
  }
  _impl_.export_path_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.export_path_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (from._internal_has_export_path()) {
    _this->_impl_.export_path_.Set(from._internal_export_path(), 
      _this->GetArenaForAllocation());
  }
  _impl_.groundtruth_box_visualization_color_.InitDefault();
  if (from._internal_has_groundtruth_box_visualization_color()) {
    _this->_impl_.groundtruth_box_visualization_color_.Set(from._internal_groundtruth_box_visualization_color(), 
      _this->GetArenaForAllocation());
  }
  ::memcpy(&_impl_.max_evals_, &from._impl_.max_evals_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.recall_upper_bound_) -
    reinterpret_cast<char*>(&_impl_.max_evals_)) + sizeof(_impl_.recall_upper_bound_));
  // @@protoc_insertion_point(copy_constructor:object_detection.protos.EvalConfig)
}

inline void EvalConfig::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_._has_bits_){}
    , /*decltype(_impl_._cached_size_)*/{}
    , decltype(_impl_.metrics_set_){arena}
    , decltype(_impl_.visualization_export_dir_){}
    , decltype(_impl_.eval_master_){}
    , decltype(_impl_.export_path_){}
    , decltype(_impl_.groundtruth_box_visualization_color_){}
    , decltype(_impl_.max_evals_){0u}
    , decltype(_impl_.save_graph_){false}
    , decltype(_impl_.ignore_groundtruth_){false}
    , decltype(_impl_.use_moving_averages_){false}
    , decltype(_impl_.eval_instance_masks_){false}
    , decltype(_impl_.skip_scores_){false}
    , decltype(_impl_.skip_labels_){false}
    , decltype(_impl_.visualize_groundtruth_boxes_){false}
    , decltype(_impl_.keep_image_id_for_visualization_export_){false}
    , decltype(_impl_.include_metrics_per_category_){false}
    , decltype(_impl_.recall_lower_bound_){0}
    , decltype(_impl_.num_visualizations_){10u}
    , decltype(_impl_.num_examples_){5000u}
    , decltype(_impl_.eval_interval_secs_){300u}
    , decltype(_impl_.min_score_threshold_){0.5f}
    , decltype(_impl_.max_num_boxes_to_visualize_){20}
    , decltype(_impl_.retain_original_images_){true}
    , decltype(_impl_.batch_size_){1u}
    , decltype(_impl_.recall_upper_bound_){1}
  };
  _impl_.visualization_export_dir_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.visualization_export_dir_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  _impl_.eval_master_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.eval_master_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  _impl_.export_path_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.export_path_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  _impl_.groundtruth_box_visualization_color_.InitDefault();
}

EvalConfig::~EvalConfig() {
  // @@protoc_insertion_point(destructor:object_detection.protos.EvalConfig)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void EvalConfig::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.metrics_set_.~RepeatedPtrField();
  _impl_.visualization_export_dir_.Destroy();
  _impl_.eval_master_.Destroy();
  _impl_.export_path_.Destroy();
  _impl_.groundtruth_box_visualization_color_.Destroy();
}

void EvalConfig::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void EvalConfig::Clear() {
// @@protoc_insertion_point(message_clear_start:object_detection.protos.EvalConfig)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.metrics_set_.Clear();
  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x0000000fu) {
    if (cached_has_bits & 0x00000001u) {
      _impl_.visualization_export_dir_.ClearNonDefaultToEmpty();
    }
    if (cached_has_bits & 0x00000002u) {
      _impl_.eval_master_.ClearNonDefaultToEmpty();
    }
    if (cached_has_bits & 0x00000004u) {
      _impl_.export_path_.ClearNonDefaultToEmpty();
    }
    if (cached_has_bits & 0x00000008u) {
      _impl_.groundtruth_box_visualization_color_.ClearToDefault(::object_detection::protos::EvalConfig::Impl_::_i_give_permission_to_break_this_code_default_groundtruth_box_visualization_color_, GetArenaForAllocation());
       }
  }
  if (cached_has_bits & 0x000000f0u) {
    ::memset(&_impl_.max_evals_, 0, static_cast<size_t>(
        reinterpret_cast<char*>(&_impl_.use_moving_averages_) -
        reinterpret_cast<char*>(&_impl_.max_evals_)) + sizeof(_impl_.use_moving_averages_));
  }
  if (cached_has_bits & 0x0000ff00u) {
    ::memset(&_impl_.eval_instance_masks_, 0, static_cast<size_t>(
        reinterpret_cast<char*>(&_impl_.recall_lower_bound_) -
        reinterpret_cast<char*>(&_impl_.eval_instance_masks_)) + sizeof(_impl_.recall_lower_bound_));
    _impl_.num_visualizations_ = 10u;
  }
  if (cached_has_bits & 0x007f0000u) {
    _impl_.num_examples_ = 5000u;
    _impl_.eval_interval_secs_ = 300u;
    _impl_.min_score_threshold_ = 0.5f;
    _impl_.max_num_boxes_to_visualize_ = 20;
    _impl_.retain_original_images_ = true;
    _impl_.batch_size_ = 1u;
    _impl_.recall_upper_bound_ = 1;
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* EvalConfig::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  _Internal::HasBits has_bits{};
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // optional uint32 num_visualizations = 1 [default = 10];
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 8)) {
          _Internal::set_has_num_visualizations(&has_bits);
          _impl_.num_visualizations_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional uint32 num_examples = 2 [default = 5000, deprecated = true];
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 16)) {
          _Internal::set_has_num_examples(&has_bits);
          _impl_.num_examples_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional uint32 eval_interval_secs = 3 [default = 300];
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 24)) {
          _Internal::set_has_eval_interval_secs(&has_bits);
          _impl_.eval_interval_secs_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional uint32 max_evals = 4 [default = 0, deprecated = true];
      case 4:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 32)) {
          _Internal::set_has_max_evals(&has_bits);
          _impl_.max_evals_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool save_graph = 5 [default = false];
      case 5:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 40)) {
          _Internal::set_has_save_graph(&has_bits);
          _impl_.save_graph_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional string visualization_export_dir = 6 [default = ""];
      case 6:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 50)) {
          auto str = _internal_mutable_visualization_export_dir();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          #ifndef NDEBUG
          ::_pbi::VerifyUTF8(str, "object_detection.protos.EvalConfig.visualization_export_dir");
          #endif  // !NDEBUG
        } else
          goto handle_unusual;
        continue;
      // optional string eval_master = 7 [default = ""];
      case 7:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 58)) {
          auto str = _internal_mutable_eval_master();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          #ifndef NDEBUG
          ::_pbi::VerifyUTF8(str, "object_detection.protos.EvalConfig.eval_master");
          #endif  // !NDEBUG
        } else
          goto handle_unusual;
        continue;
      // repeated string metrics_set = 8;
      case 8:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 66)) {
          ptr -= 1;
          do {
            ptr += 1;
            auto str = _internal_add_metrics_set();
            ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
            CHK_(ptr);
            #ifndef NDEBUG
            ::_pbi::VerifyUTF8(str, "object_detection.protos.EvalConfig.metrics_set");
            #endif  // !NDEBUG
            if (!ctx->DataAvailable(ptr)) break;
          } while (::PROTOBUF_NAMESPACE_ID::internal::ExpectTag<66>(ptr));
        } else
          goto handle_unusual;
        continue;
      // optional string export_path = 9 [default = ""];
      case 9:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 74)) {
          auto str = _internal_mutable_export_path();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          #ifndef NDEBUG
          ::_pbi::VerifyUTF8(str, "object_detection.protos.EvalConfig.export_path");
          #endif  // !NDEBUG
        } else
          goto handle_unusual;
        continue;
      // optional bool ignore_groundtruth = 10 [default = false];
      case 10:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 80)) {
          _Internal::set_has_ignore_groundtruth(&has_bits);
          _impl_.ignore_groundtruth_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool use_moving_averages = 11 [default = false];
      case 11:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 88)) {
          _Internal::set_has_use_moving_averages(&has_bits);
          _impl_.use_moving_averages_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool eval_instance_masks = 12 [default = false];
      case 12:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 96)) {
          _Internal::set_has_eval_instance_masks(&has_bits);
          _impl_.eval_instance_masks_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional float min_score_threshold = 13 [default = 0.5];
      case 13:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 109)) {
          _Internal::set_has_min_score_threshold(&has_bits);
          _impl_.min_score_threshold_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // optional int32 max_num_boxes_to_visualize = 14 [default = 20];
      case 14:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 112)) {
          _Internal::set_has_max_num_boxes_to_visualize(&has_bits);
          _impl_.max_num_boxes_to_visualize_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool skip_scores = 15 [default = false];
      case 15:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 120)) {
          _Internal::set_has_skip_scores(&has_bits);
          _impl_.skip_scores_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool skip_labels = 16 [default = false];
      case 16:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 128)) {
          _Internal::set_has_skip_labels(&has_bits);
          _impl_.skip_labels_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool visualize_groundtruth_boxes = 17 [default = false];
      case 17:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 136)) {
          _Internal::set_has_visualize_groundtruth_boxes(&has_bits);
          _impl_.visualize_groundtruth_boxes_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional string groundtruth_box_visualization_color = 18 [default = "black"];
      case 18:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 146)) {
          auto str = _internal_mutable_groundtruth_box_visualization_color();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          #ifndef NDEBUG
          ::_pbi::VerifyUTF8(str, "object_detection.protos.EvalConfig.groundtruth_box_visualization_color");
          #endif  // !NDEBUG
        } else
          goto handle_unusual;
        continue;
      // optional bool keep_image_id_for_visualization_export = 19 [default = false];
      case 19:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 152)) {
          _Internal::set_has_keep_image_id_for_visualization_export(&has_bits);
          _impl_.keep_image_id_for_visualization_export_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool retain_original_images = 23 [default = true];
      case 23:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 184)) {
          _Internal::set_has_retain_original_images(&has_bits);
          _impl_.retain_original_images_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional bool include_metrics_per_category = 24 [default = false];
      case 24:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 192)) {
          _Internal::set_has_include_metrics_per_category(&has_bits);
          _impl_.include_metrics_per_category_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional uint32 batch_size = 25 [default = 1];
      case 25:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 200)) {
          _Internal::set_has_batch_size(&has_bits);
          _impl_.batch_size_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // optional float recall_lower_bound = 26 [default = 0];
      case 26:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 213)) {
          _Internal::set_has_recall_lower_bound(&has_bits);
          _impl_.recall_lower_bound_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // optional float recall_upper_bound = 27 [default = 1];
      case 27:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 221)) {
          _Internal::set_has_recall_upper_bound(&has_bits);
          _impl_.recall_upper_bound_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  _impl_._has_bits_.Or(has_bits);
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* EvalConfig::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:object_detection.protos.EvalConfig)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  // optional uint32 num_visualizations = 1 [default = 10];
  if (cached_has_bits & 0x00008000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(1, this->_internal_num_visualizations(), target);
  }

  // optional uint32 num_examples = 2 [default = 5000, deprecated = true];
  if (cached_has_bits & 0x00010000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(2, this->_internal_num_examples(), target);
  }

  // optional uint32 eval_interval_secs = 3 [default = 300];
  if (cached_has_bits & 0x00020000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(3, this->_internal_eval_interval_secs(), target);
  }

  // optional uint32 max_evals = 4 [default = 0, deprecated = true];
  if (cached_has_bits & 0x00000010u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(4, this->_internal_max_evals(), target);
  }

  // optional bool save_graph = 5 [default = false];
  if (cached_has_bits & 0x00000020u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(5, this->_internal_save_graph(), target);
  }

  // optional string visualization_export_dir = 6 [default = ""];
  if (cached_has_bits & 0x00000001u) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::VerifyUTF8StringNamedField(
      this->_internal_visualization_export_dir().data(), static_cast<int>(this->_internal_visualization_export_dir().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::SERIALIZE,
      "object_detection.protos.EvalConfig.visualization_export_dir");
    target = stream->WriteStringMaybeAliased(
        6, this->_internal_visualization_export_dir(), target);
  }

  // optional string eval_master = 7 [default = ""];
  if (cached_has_bits & 0x00000002u) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::VerifyUTF8StringNamedField(
      this->_internal_eval_master().data(), static_cast<int>(this->_internal_eval_master().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::SERIALIZE,
      "object_detection.protos.EvalConfig.eval_master");
    target = stream->WriteStringMaybeAliased(
        7, this->_internal_eval_master(), target);
  }

  // repeated string metrics_set = 8;
  for (int i = 0, n = this->_internal_metrics_set_size(); i < n; i++) {
    const auto& s = this->_internal_metrics_set(i);
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::VerifyUTF8StringNamedField(
      s.data(), static_cast<int>(s.length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::SERIALIZE,
      "object_detection.protos.EvalConfig.metrics_set");
    target = stream->WriteString(8, s, target);
  }

  // optional string export_path = 9 [default = ""];
  if (cached_has_bits & 0x00000004u) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::VerifyUTF8StringNamedField(
      this->_internal_export_path().data(), static_cast<int>(this->_internal_export_path().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::SERIALIZE,
      "object_detection.protos.EvalConfig.export_path");
    target = stream->WriteStringMaybeAliased(
        9, this->_internal_export_path(), target);
  }

  // optional bool ignore_groundtruth = 10 [default = false];
  if (cached_has_bits & 0x00000040u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(10, this->_internal_ignore_groundtruth(), target);
  }

  // optional bool use_moving_averages = 11 [default = false];
  if (cached_has_bits & 0x00000080u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(11, this->_internal_use_moving_averages(), target);
  }

  // optional bool eval_instance_masks = 12 [default = false];
  if (cached_has_bits & 0x00000100u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(12, this->_internal_eval_instance_masks(), target);
  }

  // optional float min_score_threshold = 13 [default = 0.5];
  if (cached_has_bits & 0x00040000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(13, this->_internal_min_score_threshold(), target);
  }

  // optional int32 max_num_boxes_to_visualize = 14 [default = 20];
  if (cached_has_bits & 0x00080000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteInt32ToArray(14, this->_internal_max_num_boxes_to_visualize(), target);
  }

  // optional bool skip_scores = 15 [default = false];
  if (cached_has_bits & 0x00000200u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(15, this->_internal_skip_scores(), target);
  }

  // optional bool skip_labels = 16 [default = false];
  if (cached_has_bits & 0x00000400u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(16, this->_internal_skip_labels(), target);
  }

  // optional bool visualize_groundtruth_boxes = 17 [default = false];
  if (cached_has_bits & 0x00000800u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(17, this->_internal_visualize_groundtruth_boxes(), target);
  }

  // optional string groundtruth_box_visualization_color = 18 [default = "black"];
  if (cached_has_bits & 0x00000008u) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::VerifyUTF8StringNamedField(
      this->_internal_groundtruth_box_visualization_color().data(), static_cast<int>(this->_internal_groundtruth_box_visualization_color().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormat::SERIALIZE,
      "object_detection.protos.EvalConfig.groundtruth_box_visualization_color");
    target = stream->WriteStringMaybeAliased(
        18, this->_internal_groundtruth_box_visualization_color(), target);
  }

  // optional bool keep_image_id_for_visualization_export = 19 [default = false];
  if (cached_has_bits & 0x00001000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(19, this->_internal_keep_image_id_for_visualization_export(), target);
  }

  // optional bool retain_original_images = 23 [default = true];
  if (cached_has_bits & 0x00100000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(23, this->_internal_retain_original_images(), target);
  }

  // optional bool include_metrics_per_category = 24 [default = false];
  if (cached_has_bits & 0x00002000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(24, this->_internal_include_metrics_per_category(), target);
  }

  // optional uint32 batch_size = 25 [default = 1];
  if (cached_has_bits & 0x00200000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(25, this->_internal_batch_size(), target);
  }

  // optional float recall_lower_bound = 26 [default = 0];
  if (cached_has_bits & 0x00004000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(26, this->_internal_recall_lower_bound(), target);
  }

  // optional float recall_upper_bound = 27 [default = 1];
  if (cached_has_bits & 0x00400000u) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(27, this->_internal_recall_upper_bound(), target);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:object_detection.protos.EvalConfig)
  return target;
}

size_t EvalConfig::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:object_detection.protos.EvalConfig)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // repeated string metrics_set = 8;
  total_size += 1 *
      ::PROTOBUF_NAMESPACE_ID::internal::FromIntSize(_impl_.metrics_set_.size());
  for (int i = 0, n = _impl_.metrics_set_.size(); i < n; i++) {
    total_size += ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
      _impl_.metrics_set_.Get(i));
  }

  cached_has_bits = _impl_._has_bits_[0];
  if (cached_has_bits & 0x000000ffu) {
    // optional string visualization_export_dir = 6 [default = ""];
    if (cached_has_bits & 0x00000001u) {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
          this->_internal_visualization_export_dir());
    }

    // optional string eval_master = 7 [default = ""];
    if (cached_has_bits & 0x00000002u) {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
          this->_internal_eval_master());
    }

    // optional string export_path = 9 [default = ""];
    if (cached_has_bits & 0x00000004u) {
      total_size += 1 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
          this->_internal_export_path());
    }

    // optional string groundtruth_box_visualization_color = 18 [default = "black"];
    if (cached_has_bits & 0x00000008u) {
      total_size += 2 +
        ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
          this->_internal_groundtruth_box_visualization_color());
    }

    // optional uint32 max_evals = 4 [default = 0, deprecated = true];
    if (cached_has_bits & 0x00000010u) {
      total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_max_evals());
    }

    // optional bool save_graph = 5 [default = false];
    if (cached_has_bits & 0x00000020u) {
      total_size += 1 + 1;
    }

    // optional bool ignore_groundtruth = 10 [default = false];
    if (cached_has_bits & 0x00000040u) {
      total_size += 1 + 1;
    }

    // optional bool use_moving_averages = 11 [default = false];
    if (cached_has_bits & 0x00000080u) {
      total_size += 1 + 1;
    }

  }
  if (cached_has_bits & 0x0000ff00u) {
    // optional bool eval_instance_masks = 12 [default = false];
    if (cached_has_bits & 0x00000100u) {
      total_size += 1 + 1;
    }

    // optional bool skip_scores = 15 [default = false];
    if (cached_has_bits & 0x00000200u) {
      total_size += 1 + 1;
    }

    // optional bool skip_labels = 16 [default = false];
    if (cached_has_bits & 0x00000400u) {
      total_size += 2 + 1;
    }

    // optional bool visualize_groundtruth_boxes = 17 [default = false];
    if (cached_has_bits & 0x00000800u) {
      total_size += 2 + 1;
    }

    // optional bool keep_image_id_for_visualization_export = 19 [default = false];
    if (cached_has_bits & 0x00001000u) {
      total_size += 2 + 1;
    }

    // optional bool include_metrics_per_category = 24 [default = false];
    if (cached_has_bits & 0x00002000u) {
      total_size += 2 + 1;
    }

    // optional float recall_lower_bound = 26 [default = 0];
    if (cached_has_bits & 0x00004000u) {
      total_size += 2 + 4;
    }

    // optional uint32 num_visualizations = 1 [default = 10];
    if (cached_has_bits & 0x00008000u) {
      total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_num_visualizations());
    }

  }
  if (cached_has_bits & 0x007f0000u) {
    // optional uint32 num_examples = 2 [default = 5000, deprecated = true];
    if (cached_has_bits & 0x00010000u) {
      total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_num_examples());
    }

    // optional uint32 eval_interval_secs = 3 [default = 300];
    if (cached_has_bits & 0x00020000u) {
      total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_eval_interval_secs());
    }

    // optional float min_score_threshold = 13 [default = 0.5];
    if (cached_has_bits & 0x00040000u) {
      total_size += 1 + 4;
    }

    // optional int32 max_num_boxes_to_visualize = 14 [default = 20];
    if (cached_has_bits & 0x00080000u) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(this->_internal_max_num_boxes_to_visualize());
    }

    // optional bool retain_original_images = 23 [default = true];
    if (cached_has_bits & 0x00100000u) {
      total_size += 2 + 1;
    }

    // optional uint32 batch_size = 25 [default = 1];
    if (cached_has_bits & 0x00200000u) {
      total_size += 2 +
        ::_pbi::WireFormatLite::UInt32Size(
          this->_internal_batch_size());
    }

    // optional float recall_upper_bound = 27 [default = 1];
    if (cached_has_bits & 0x00400000u) {
      total_size += 2 + 4;
    }

  }
  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData EvalConfig::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    EvalConfig::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*EvalConfig::GetClassData() const { return &_class_data_; }


void EvalConfig::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<EvalConfig*>(&to_msg);
  auto& from = static_cast<const EvalConfig&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:object_detection.protos.EvalConfig)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.metrics_set_.MergeFrom(from._impl_.metrics_set_);
  cached_has_bits = from._impl_._has_bits_[0];
  if (cached_has_bits & 0x000000ffu) {
    if (cached_has_bits & 0x00000001u) {
      _this->_internal_set_visualization_export_dir(from._internal_visualization_export_dir());
    }
    if (cached_has_bits & 0x00000002u) {
      _this->_internal_set_eval_master(from._internal_eval_master());
    }
    if (cached_has_bits & 0x00000004u) {
      _this->_internal_set_export_path(from._internal_export_path());
    }
    if (cached_has_bits & 0x00000008u) {
      _this->_internal_set_groundtruth_box_visualization_color(from._internal_groundtruth_box_visualization_color());
    }
    if (cached_has_bits & 0x00000010u) {
      _this->_impl_.max_evals_ = from._impl_.max_evals_;
    }
    if (cached_has_bits & 0x00000020u) {
      _this->_impl_.save_graph_ = from._impl_.save_graph_;
    }
    if (cached_has_bits & 0x00000040u) {
      _this->_impl_.ignore_groundtruth_ = from._impl_.ignore_groundtruth_;
    }
    if (cached_has_bits & 0x00000080u) {
      _this->_impl_.use_moving_averages_ = from._impl_.use_moving_averages_;
    }
    _this->_impl_._has_bits_[0] |= cached_has_bits;
  }
  if (cached_has_bits & 0x0000ff00u) {
    if (cached_has_bits & 0x00000100u) {
      _this->_impl_.eval_instance_masks_ = from._impl_.eval_instance_masks_;
    }
    if (cached_has_bits & 0x00000200u) {
      _this->_impl_.skip_scores_ = from._impl_.skip_scores_;
    }
    if (cached_has_bits & 0x00000400u) {
      _this->_impl_.skip_labels_ = from._impl_.skip_labels_;
    }
    if (cached_has_bits & 0x00000800u) {
      _this->_impl_.visualize_groundtruth_boxes_ = from._impl_.visualize_groundtruth_boxes_;
    }
    if (cached_has_bits & 0x00001000u) {
      _this->_impl_.keep_image_id_for_visualization_export_ = from._impl_.keep_image_id_for_visualization_export_;
    }
    if (cached_has_bits & 0x00002000u) {
      _this->_impl_.include_metrics_per_category_ = from._impl_.include_metrics_per_category_;
    }
    if (cached_has_bits & 0x00004000u) {
      _this->_impl_.recall_lower_bound_ = from._impl_.recall_lower_bound_;
    }
    if (cached_has_bits & 0x00008000u) {
      _this->_impl_.num_visualizations_ = from._impl_.num_visualizations_;
    }
    _this->_impl_._has_bits_[0] |= cached_has_bits;
  }
  if (cached_has_bits & 0x007f0000u) {
    if (cached_has_bits & 0x00010000u) {
      _this->_impl_.num_examples_ = from._impl_.num_examples_;
    }
    if (cached_has_bits & 0x00020000u) {
      _this->_impl_.eval_interval_secs_ = from._impl_.eval_interval_secs_;
    }
    if (cached_has_bits & 0x00040000u) {
      _this->_impl_.min_score_threshold_ = from._impl_.min_score_threshold_;
    }
    if (cached_has_bits & 0x00080000u) {
      _this->_impl_.max_num_boxes_to_visualize_ = from._impl_.max_num_boxes_to_visualize_;
    }
    if (cached_has_bits & 0x00100000u) {
      _this->_impl_.retain_original_images_ = from._impl_.retain_original_images_;
    }
    if (cached_has_bits & 0x00200000u) {
      _this->_impl_.batch_size_ = from._impl_.batch_size_;
    }
    if (cached_has_bits & 0x00400000u) {
      _this->_impl_.recall_upper_bound_ = from._impl_.recall_upper_bound_;
    }
    _this->_impl_._has_bits_[0] |= cached_has_bits;
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void EvalConfig::CopyFrom(const EvalConfig& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:object_detection.protos.EvalConfig)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool EvalConfig::IsInitialized() const {
  return true;
}

void EvalConfig::InternalSwap(EvalConfig* other) {
  using std::swap;
  auto* lhs_arena = GetArenaForAllocation();
  auto* rhs_arena = other->GetArenaForAllocation();
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.metrics_set_.InternalSwap(&other->_impl_.metrics_set_);
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.visualization_export_dir_, lhs_arena,
      &other->_impl_.visualization_export_dir_, rhs_arena
  );
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.eval_master_, lhs_arena,
      &other->_impl_.eval_master_, rhs_arena
  );
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.export_path_, lhs_arena,
      &other->_impl_.export_path_, rhs_arena
  );
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.groundtruth_box_visualization_color_, lhs_arena,
      &other->_impl_.groundtruth_box_visualization_color_, rhs_arena
  );
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(EvalConfig, _impl_.recall_lower_bound_)
      + sizeof(EvalConfig::_impl_.recall_lower_bound_)
      - PROTOBUF_FIELD_OFFSET(EvalConfig, _impl_.max_evals_)>(
          reinterpret_cast<char*>(&_impl_.max_evals_),
          reinterpret_cast<char*>(&other->_impl_.max_evals_));
  swap(_impl_.num_visualizations_, other->_impl_.num_visualizations_);
  swap(_impl_.num_examples_, other->_impl_.num_examples_);
  swap(_impl_.eval_interval_secs_, other->_impl_.eval_interval_secs_);
  swap(_impl_.min_score_threshold_, other->_impl_.min_score_threshold_);
  swap(_impl_.max_num_boxes_to_visualize_, other->_impl_.max_num_boxes_to_visualize_);
  swap(_impl_.retain_original_images_, other->_impl_.retain_original_images_);
  swap(_impl_.batch_size_, other->_impl_.batch_size_);
  swap(_impl_.recall_upper_bound_, other->_impl_.recall_upper_bound_);
}

::PROTOBUF_NAMESPACE_ID::Metadata EvalConfig::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_object_5fdetection_2fprotos_2feval_2eproto_getter, &descriptor_table_object_5fdetection_2fprotos_2feval_2eproto_once,
      file_level_metadata_object_5fdetection_2fprotos_2feval_2eproto[0]);
}

// @@protoc_insertion_point(namespace_scope)
}  // namespace protos
}  // namespace object_detection
PROTOBUF_NAMESPACE_OPEN
template<> PROTOBUF_NOINLINE ::object_detection::protos::EvalConfig*
Arena::CreateMaybeMessage< ::object_detection::protos::EvalConfig >(Arena* arena) {
  return Arena::CreateMessageInternal< ::object_detection::protos::EvalConfig >(arena);
}
PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)
#include <google/protobuf/port_undef.inc>
