/*
 * Copyright 2022-2023 Advanced Micro Devices Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once
#if __has_include(<any>)  && __cplusplus > 201700
#include <any>
#if !__cpp_lib_any > 0
#error "must enable -std=c++17 to use std::any"
#endif
namespace xir {
using any = std::any;
namespace stdx = ::std;
}  // namespace xir
#else
#error "must enable c++17"
#endif
