// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: second/protos/target.proto

#include "second/protos/target.pb.h"

#include <algorithm>

#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/extension_set.h>
#include <google/protobuf/wire_format_lite.h>
#include <google/protobuf/descriptor.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/reflection_ops.h>
#include <google/protobuf/wire_format.h>
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>

PROTOBUF_PRAGMA_INIT_SEG

namespace _pb = ::PROTOBUF_NAMESPACE_ID;
namespace _pbi = _pb::internal;

namespace second {
namespace protos {
PROTOBUF_CONSTEXPR TargetAssigner::TargetAssigner(
    ::_pbi::ConstantInitialized): _impl_{
    /*decltype(_impl_.anchor_generators_)*/{}
  , /*decltype(_impl_.class_name_)*/{&::_pbi::fixed_address_empty_string, ::_pbi::ConstantInitialized{}}
  , /*decltype(_impl_.region_similarity_calculator_)*/nullptr
  , /*decltype(_impl_.sample_positive_fraction_)*/0
  , /*decltype(_impl_.sample_size_)*/0u
  , /*decltype(_impl_.use_rotate_iou_)*/false
  , /*decltype(_impl_._cached_size_)*/{}} {}
struct TargetAssignerDefaultTypeInternal {
  PROTOBUF_CONSTEXPR TargetAssignerDefaultTypeInternal()
      : _instance(::_pbi::ConstantInitialized{}) {}
  ~TargetAssignerDefaultTypeInternal() {}
  union {
    TargetAssigner _instance;
  };
};
PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 TargetAssignerDefaultTypeInternal _TargetAssigner_default_instance_;
}  // namespace protos
}  // namespace second
static ::_pb::Metadata file_level_metadata_second_2fprotos_2ftarget_2eproto[1];
static constexpr ::_pb::EnumDescriptor const** file_level_enum_descriptors_second_2fprotos_2ftarget_2eproto = nullptr;
static constexpr ::_pb::ServiceDescriptor const** file_level_service_descriptors_second_2fprotos_2ftarget_2eproto = nullptr;

const uint32_t TableStruct_second_2fprotos_2ftarget_2eproto::offsets[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  ~0u,  // no _has_bits_
  PROTOBUF_FIELD_OFFSET(::second::protos::TargetAssigner, _internal_metadata_),
  ~0u,  // no _extensions_
  ~0u,  // no _oneof_case_
  ~0u,  // no _weak_field_map_
  ~0u,  // no _inlined_string_donated_
  PROTOBUF_FIELD_OFFSET(::second::protos::TargetAssigner, _impl_.anchor_generators_),
  PROTOBUF_FIELD_OFFSET(::second::protos::TargetAssigner, _impl_.sample_positive_fraction_),
  PROTOBUF_FIELD_OFFSET(::second::protos::TargetAssigner, _impl_.sample_size_),
  PROTOBUF_FIELD_OFFSET(::second::protos::TargetAssigner, _impl_.use_rotate_iou_),
  PROTOBUF_FIELD_OFFSET(::second::protos::TargetAssigner, _impl_.class_name_),
  PROTOBUF_FIELD_OFFSET(::second::protos::TargetAssigner, _impl_.region_similarity_calculator_),
};
static const ::_pbi::MigrationSchema schemas[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) = {
  { 0, -1, -1, sizeof(::second::protos::TargetAssigner)},
};

static const ::_pb::Message* const file_default_instances[] = {
  &::second::protos::_TargetAssigner_default_instance_._instance,
};

const char descriptor_table_protodef_second_2fprotos_2ftarget_2eproto[] PROTOBUF_SECTION_VARIABLE(protodesc_cold) =
  "\n\032second/protos/target.proto\022\rsecond.pro"
  "tos\032\033second/protos/anchors.proto\032\036second"
  "/protos/similarity.proto\"\211\002\n\016TargetAssig"
  "ner\022C\n\021anchor_generators\030\001 \003(\0132(.second."
  "protos.AnchorGeneratorCollection\022 \n\030samp"
  "le_positive_fraction\030\002 \001(\002\022\023\n\013sample_siz"
  "e\030\003 \001(\r\022\026\n\016use_rotate_iou\030\004 \001(\010\022\022\n\nclass"
  "_name\030\005 \001(\t\022O\n\034region_similarity_calcula"
  "tor\030\006 \001(\0132).second.protos.RegionSimilari"
  "tyCalculatorb\006proto3"
  ;
static const ::_pbi::DescriptorTable* const descriptor_table_second_2fprotos_2ftarget_2eproto_deps[2] = {
  &::descriptor_table_second_2fprotos_2fanchors_2eproto,
  &::descriptor_table_second_2fprotos_2fsimilarity_2eproto,
};
static ::_pbi::once_flag descriptor_table_second_2fprotos_2ftarget_2eproto_once;
const ::_pbi::DescriptorTable descriptor_table_second_2fprotos_2ftarget_2eproto = {
    false, false, 380, descriptor_table_protodef_second_2fprotos_2ftarget_2eproto,
    "second/protos/target.proto",
    &descriptor_table_second_2fprotos_2ftarget_2eproto_once, descriptor_table_second_2fprotos_2ftarget_2eproto_deps, 2, 1,
    schemas, file_default_instances, TableStruct_second_2fprotos_2ftarget_2eproto::offsets,
    file_level_metadata_second_2fprotos_2ftarget_2eproto, file_level_enum_descriptors_second_2fprotos_2ftarget_2eproto,
    file_level_service_descriptors_second_2fprotos_2ftarget_2eproto,
};
PROTOBUF_ATTRIBUTE_WEAK const ::_pbi::DescriptorTable* descriptor_table_second_2fprotos_2ftarget_2eproto_getter() {
  return &descriptor_table_second_2fprotos_2ftarget_2eproto;
}

// Force running AddDescriptors() at dynamic initialization time.
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::_pbi::AddDescriptorsRunner dynamic_init_dummy_second_2fprotos_2ftarget_2eproto(&descriptor_table_second_2fprotos_2ftarget_2eproto);
namespace second {
namespace protos {

// ===================================================================

class TargetAssigner::_Internal {
 public:
  static const ::second::protos::RegionSimilarityCalculator& region_similarity_calculator(const TargetAssigner* msg);
};

const ::second::protos::RegionSimilarityCalculator&
TargetAssigner::_Internal::region_similarity_calculator(const TargetAssigner* msg) {
  return *msg->_impl_.region_similarity_calculator_;
}
void TargetAssigner::clear_anchor_generators() {
  _impl_.anchor_generators_.Clear();
}
void TargetAssigner::clear_region_similarity_calculator() {
  if (GetArenaForAllocation() == nullptr && _impl_.region_similarity_calculator_ != nullptr) {
    delete _impl_.region_similarity_calculator_;
  }
  _impl_.region_similarity_calculator_ = nullptr;
}
TargetAssigner::TargetAssigner(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                         bool is_message_owned)
  : ::PROTOBUF_NAMESPACE_ID::Message(arena, is_message_owned) {
  SharedCtor(arena, is_message_owned);
  // @@protoc_insertion_point(arena_constructor:second.protos.TargetAssigner)
}
TargetAssigner::TargetAssigner(const TargetAssigner& from)
  : ::PROTOBUF_NAMESPACE_ID::Message() {
  TargetAssigner* const _this = this; (void)_this;
  new (&_impl_) Impl_{
      decltype(_impl_.anchor_generators_){from._impl_.anchor_generators_}
    , decltype(_impl_.class_name_){}
    , decltype(_impl_.region_similarity_calculator_){nullptr}
    , decltype(_impl_.sample_positive_fraction_){}
    , decltype(_impl_.sample_size_){}
    , decltype(_impl_.use_rotate_iou_){}
    , /*decltype(_impl_._cached_size_)*/{}};

  _internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
  _impl_.class_name_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.class_name_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (!from._internal_class_name().empty()) {
    _this->_impl_.class_name_.Set(from._internal_class_name(), 
      _this->GetArenaForAllocation());
  }
  if (from._internal_has_region_similarity_calculator()) {
    _this->_impl_.region_similarity_calculator_ = new ::second::protos::RegionSimilarityCalculator(*from._impl_.region_similarity_calculator_);
  }
  ::memcpy(&_impl_.sample_positive_fraction_, &from._impl_.sample_positive_fraction_,
    static_cast<size_t>(reinterpret_cast<char*>(&_impl_.use_rotate_iou_) -
    reinterpret_cast<char*>(&_impl_.sample_positive_fraction_)) + sizeof(_impl_.use_rotate_iou_));
  // @@protoc_insertion_point(copy_constructor:second.protos.TargetAssigner)
}

inline void TargetAssigner::SharedCtor(
    ::_pb::Arena* arena, bool is_message_owned) {
  (void)arena;
  (void)is_message_owned;
  new (&_impl_) Impl_{
      decltype(_impl_.anchor_generators_){arena}
    , decltype(_impl_.class_name_){}
    , decltype(_impl_.region_similarity_calculator_){nullptr}
    , decltype(_impl_.sample_positive_fraction_){0}
    , decltype(_impl_.sample_size_){0u}
    , decltype(_impl_.use_rotate_iou_){false}
    , /*decltype(_impl_._cached_size_)*/{}
  };
  _impl_.class_name_.InitDefault();
  #ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
    _impl_.class_name_.Set("", GetArenaForAllocation());
  #endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
}

TargetAssigner::~TargetAssigner() {
  // @@protoc_insertion_point(destructor:second.protos.TargetAssigner)
  if (auto *arena = _internal_metadata_.DeleteReturnArena<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>()) {
  (void)arena;
    return;
  }
  SharedDtor();
}

inline void TargetAssigner::SharedDtor() {
  GOOGLE_DCHECK(GetArenaForAllocation() == nullptr);
  _impl_.anchor_generators_.~RepeatedPtrField();
  _impl_.class_name_.Destroy();
  if (this != internal_default_instance()) delete _impl_.region_similarity_calculator_;
}

void TargetAssigner::SetCachedSize(int size) const {
  _impl_._cached_size_.Set(size);
}

void TargetAssigner::Clear() {
// @@protoc_insertion_point(message_clear_start:second.protos.TargetAssigner)
  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  _impl_.anchor_generators_.Clear();
  _impl_.class_name_.ClearToEmpty();
  if (GetArenaForAllocation() == nullptr && _impl_.region_similarity_calculator_ != nullptr) {
    delete _impl_.region_similarity_calculator_;
  }
  _impl_.region_similarity_calculator_ = nullptr;
  ::memset(&_impl_.sample_positive_fraction_, 0, static_cast<size_t>(
      reinterpret_cast<char*>(&_impl_.use_rotate_iou_) -
      reinterpret_cast<char*>(&_impl_.sample_positive_fraction_)) + sizeof(_impl_.use_rotate_iou_));
  _internal_metadata_.Clear<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
}

const char* TargetAssigner::_InternalParse(const char* ptr, ::_pbi::ParseContext* ctx) {
#define CHK_(x) if (PROTOBUF_PREDICT_FALSE(!(x))) goto failure
  while (!ctx->Done(&ptr)) {
    uint32_t tag;
    ptr = ::_pbi::ReadTag(ptr, &tag);
    switch (tag >> 3) {
      // repeated .second.protos.AnchorGeneratorCollection anchor_generators = 1;
      case 1:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 10)) {
          ptr -= 1;
          do {
            ptr += 1;
            ptr = ctx->ParseMessage(_internal_add_anchor_generators(), ptr);
            CHK_(ptr);
            if (!ctx->DataAvailable(ptr)) break;
          } while (::PROTOBUF_NAMESPACE_ID::internal::ExpectTag<10>(ptr));
        } else
          goto handle_unusual;
        continue;
      // float sample_positive_fraction = 2;
      case 2:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 21)) {
          _impl_.sample_positive_fraction_ = ::PROTOBUF_NAMESPACE_ID::internal::UnalignedLoad<float>(ptr);
          ptr += sizeof(float);
        } else
          goto handle_unusual;
        continue;
      // uint32 sample_size = 3;
      case 3:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 24)) {
          _impl_.sample_size_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint32(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // bool use_rotate_iou = 4;
      case 4:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 32)) {
          _impl_.use_rotate_iou_ = ::PROTOBUF_NAMESPACE_ID::internal::ReadVarint64(&ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      // string class_name = 5;
      case 5:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 42)) {
          auto str = _internal_mutable_class_name();
          ptr = ::_pbi::InlineGreedyStringParser(str, ptr, ctx);
          CHK_(ptr);
          CHK_(::_pbi::VerifyUTF8(str, "second.protos.TargetAssigner.class_name"));
        } else
          goto handle_unusual;
        continue;
      // .second.protos.RegionSimilarityCalculator region_similarity_calculator = 6;
      case 6:
        if (PROTOBUF_PREDICT_TRUE(static_cast<uint8_t>(tag) == 50)) {
          ptr = ctx->ParseMessage(_internal_mutable_region_similarity_calculator(), ptr);
          CHK_(ptr);
        } else
          goto handle_unusual;
        continue;
      default:
        goto handle_unusual;
    }  // switch
  handle_unusual:
    if ((tag == 0) || ((tag & 7) == 4)) {
      CHK_(ptr);
      ctx->SetLastTag(tag);
      goto message_done;
    }
    ptr = UnknownFieldParse(
        tag,
        _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(),
        ptr, ctx);
    CHK_(ptr != nullptr);
  }  // while
message_done:
  return ptr;
failure:
  ptr = nullptr;
  goto message_done;
#undef CHK_
}

uint8_t* TargetAssigner::_InternalSerialize(
    uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const {
  // @@protoc_insertion_point(serialize_to_array_start:second.protos.TargetAssigner)
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  // repeated .second.protos.AnchorGeneratorCollection anchor_generators = 1;
  for (unsigned i = 0,
      n = static_cast<unsigned>(this->_internal_anchor_generators_size()); i < n; i++) {
    const auto& repfield = this->_internal_anchor_generators(i);
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
        InternalWriteMessage(1, repfield, repfield.GetCachedSize(), target, stream);
  }

  // float sample_positive_fraction = 2;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_sample_positive_fraction = this->_internal_sample_positive_fraction();
  uint32_t raw_sample_positive_fraction;
  memcpy(&raw_sample_positive_fraction, &tmp_sample_positive_fraction, sizeof(tmp_sample_positive_fraction));
  if (raw_sample_positive_fraction != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteFloatToArray(2, this->_internal_sample_positive_fraction(), target);
  }

  // uint32 sample_size = 3;
  if (this->_internal_sample_size() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteUInt32ToArray(3, this->_internal_sample_size(), target);
  }

  // bool use_rotate_iou = 4;
  if (this->_internal_use_rotate_iou() != 0) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(4, this->_internal_use_rotate_iou(), target);
  }

  // string class_name = 5;
  if (!this->_internal_class_name().empty()) {
    ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::VerifyUtf8String(
      this->_internal_class_name().data(), static_cast<int>(this->_internal_class_name().length()),
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::SERIALIZE,
      "second.protos.TargetAssigner.class_name");
    target = stream->WriteStringMaybeAliased(
        5, this->_internal_class_name(), target);
  }

  // .second.protos.RegionSimilarityCalculator region_similarity_calculator = 6;
  if (this->_internal_has_region_similarity_calculator()) {
    target = ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::
      InternalWriteMessage(6, _Internal::region_similarity_calculator(this),
        _Internal::region_similarity_calculator(this).GetCachedSize(), target, stream);
  }

  if (PROTOBUF_PREDICT_FALSE(_internal_metadata_.have_unknown_fields())) {
    target = ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
        _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:second.protos.TargetAssigner)
  return target;
}

size_t TargetAssigner::ByteSizeLong() const {
// @@protoc_insertion_point(message_byte_size_start:second.protos.TargetAssigner)
  size_t total_size = 0;

  uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  // repeated .second.protos.AnchorGeneratorCollection anchor_generators = 1;
  total_size += 1UL * this->_internal_anchor_generators_size();
  for (const auto& msg : this->_impl_.anchor_generators_) {
    total_size +=
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(msg);
  }

  // string class_name = 5;
  if (!this->_internal_class_name().empty()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::StringSize(
        this->_internal_class_name());
  }

  // .second.protos.RegionSimilarityCalculator region_similarity_calculator = 6;
  if (this->_internal_has_region_similarity_calculator()) {
    total_size += 1 +
      ::PROTOBUF_NAMESPACE_ID::internal::WireFormatLite::MessageSize(
        *_impl_.region_similarity_calculator_);
  }

  // float sample_positive_fraction = 2;
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_sample_positive_fraction = this->_internal_sample_positive_fraction();
  uint32_t raw_sample_positive_fraction;
  memcpy(&raw_sample_positive_fraction, &tmp_sample_positive_fraction, sizeof(tmp_sample_positive_fraction));
  if (raw_sample_positive_fraction != 0) {
    total_size += 1 + 4;
  }

  // uint32 sample_size = 3;
  if (this->_internal_sample_size() != 0) {
    total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(this->_internal_sample_size());
  }

  // bool use_rotate_iou = 4;
  if (this->_internal_use_rotate_iou() != 0) {
    total_size += 1 + 1;
  }

  return MaybeComputeUnknownFieldsSize(total_size, &_impl_._cached_size_);
}

const ::PROTOBUF_NAMESPACE_ID::Message::ClassData TargetAssigner::_class_data_ = {
    ::PROTOBUF_NAMESPACE_ID::Message::CopyWithSourceCheck,
    TargetAssigner::MergeImpl
};
const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*TargetAssigner::GetClassData() const { return &_class_data_; }


void TargetAssigner::MergeImpl(::PROTOBUF_NAMESPACE_ID::Message& to_msg, const ::PROTOBUF_NAMESPACE_ID::Message& from_msg) {
  auto* const _this = static_cast<TargetAssigner*>(&to_msg);
  auto& from = static_cast<const TargetAssigner&>(from_msg);
  // @@protoc_insertion_point(class_specific_merge_from_start:second.protos.TargetAssigner)
  GOOGLE_DCHECK_NE(&from, _this);
  uint32_t cached_has_bits = 0;
  (void) cached_has_bits;

  _this->_impl_.anchor_generators_.MergeFrom(from._impl_.anchor_generators_);
  if (!from._internal_class_name().empty()) {
    _this->_internal_set_class_name(from._internal_class_name());
  }
  if (from._internal_has_region_similarity_calculator()) {
    _this->_internal_mutable_region_similarity_calculator()->::second::protos::RegionSimilarityCalculator::MergeFrom(
        from._internal_region_similarity_calculator());
  }
  static_assert(sizeof(uint32_t) == sizeof(float), "Code assumes uint32_t and float are the same size.");
  float tmp_sample_positive_fraction = from._internal_sample_positive_fraction();
  uint32_t raw_sample_positive_fraction;
  memcpy(&raw_sample_positive_fraction, &tmp_sample_positive_fraction, sizeof(tmp_sample_positive_fraction));
  if (raw_sample_positive_fraction != 0) {
    _this->_internal_set_sample_positive_fraction(from._internal_sample_positive_fraction());
  }
  if (from._internal_sample_size() != 0) {
    _this->_internal_set_sample_size(from._internal_sample_size());
  }
  if (from._internal_use_rotate_iou() != 0) {
    _this->_internal_set_use_rotate_iou(from._internal_use_rotate_iou());
  }
  _this->_internal_metadata_.MergeFrom<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(from._internal_metadata_);
}

void TargetAssigner::CopyFrom(const TargetAssigner& from) {
// @@protoc_insertion_point(class_specific_copy_from_start:second.protos.TargetAssigner)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

bool TargetAssigner::IsInitialized() const {
  return true;
}

void TargetAssigner::InternalSwap(TargetAssigner* other) {
  using std::swap;
  auto* lhs_arena = GetArenaForAllocation();
  auto* rhs_arena = other->GetArenaForAllocation();
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  _impl_.anchor_generators_.InternalSwap(&other->_impl_.anchor_generators_);
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr::InternalSwap(
      &_impl_.class_name_, lhs_arena,
      &other->_impl_.class_name_, rhs_arena
  );
  ::PROTOBUF_NAMESPACE_ID::internal::memswap<
      PROTOBUF_FIELD_OFFSET(TargetAssigner, _impl_.use_rotate_iou_)
      + sizeof(TargetAssigner::_impl_.use_rotate_iou_)
      - PROTOBUF_FIELD_OFFSET(TargetAssigner, _impl_.region_similarity_calculator_)>(
          reinterpret_cast<char*>(&_impl_.region_similarity_calculator_),
          reinterpret_cast<char*>(&other->_impl_.region_similarity_calculator_));
}

::PROTOBUF_NAMESPACE_ID::Metadata TargetAssigner::GetMetadata() const {
  return ::_pbi::AssignDescriptors(
      &descriptor_table_second_2fprotos_2ftarget_2eproto_getter, &descriptor_table_second_2fprotos_2ftarget_2eproto_once,
      file_level_metadata_second_2fprotos_2ftarget_2eproto[0]);
}

// @@protoc_insertion_point(namespace_scope)
}  // namespace protos
}  // namespace second
PROTOBUF_NAMESPACE_OPEN
template<> PROTOBUF_NOINLINE ::second::protos::TargetAssigner*
Arena::CreateMaybeMessage< ::second::protos::TargetAssigner >(Arena* arena) {
  return Arena::CreateMessageInternal< ::second::protos::TargetAssigner >(arena);
}
PROTOBUF_NAMESPACE_CLOSE

// @@protoc_insertion_point(global_scope)
#include <google/protobuf/port_undef.inc>
